#!	/bin/csh -f

# Serpent Version 1.2
#=============================================================================
# Declaration Block
#-----------------------------------------------------------------------------
#  DESCRIPTION: Provides all of the appropriate paths and file names for
#               install. Paths and file names include hidden files for internal
#               use by install and serpent, paths for likely places to find
#               X / Mortif / Athena libraries. It also includes a list of
#               sensitive commands which will be checked automatically before
#               installation can begin.
#=============================================================================

#-------------------------------------
#              FILES
#-------------------------------------

set CDEBUGDEFAULT	= "-O -DNDEBUG"
set VERSION 		= "Serpent Version 1.2 csh"
set USR_FILE_NAME	= "usr.cf"
set USR_FILE		= "./config/usr.cf"
set REGISTER_FILE 	= "/tmp/serpent.reg"
set HIDDEN_TECHFILE	= "./.tech.install"
set HIDDEN_DATAFILE	= "./.data.install"
set ADDRESS_FILE	= "./config/.serpent.addr"
set CHECKP_SOURCE	= "./config/checkp.c"
set CHECKP_OBJECT	= "./config/checkp"
set CHECKP_RESULT	= "./config/.checkp.install"
set HEADER_FILE		= "./config/.header.install"
set HIDDEN_INSTALL	= "./.ins.done"

#-------------------------------------
#               PATHS
#-------------------------------------

set INCDEFAULTLIST	= ("/usr/include" "/usr/include/X11R4" "/include" "/usr/local/X11R4/include" "/usr/local/include" "/usr/local/X11R4" "/usr/X11R4" "/usr/local")
set LIBDEFAULTLIST	= ("/usr/lib" "/usr/lib/X11R4" "/lib" "/usr/local/X11R4/lib")
set MOTIFDEFLIST	= ("/usr/Motif" "/usr/lib/X11R4/Motif" "/usr/local/Motif" "/usr/local/X11R4/Motif" "/usr/lib/X11R4/Motif" "/usr" "/usr/local")

#-------------------------------------
#              COMMANDS
#-------------------------------------

set COMMANDLIST = ("date" "grep" "ln" "ctags" "printenv")

#=============================================================================
# Previous Installation check
#-----------------------------------------------------------------------------
#  DESCRIPTION: Checks for the existance of a file called ./.ins.done which
#               contains a list of "checkpoints" through which install has
#               passed. If in fact the file exists, it is a strong indication
#               that the user has previously run install and may have just
#               tweaked his system to get it to compile right. If this is the
#               case install will give the option of skipping over various
#               questions which have already by answered.
#=============================================================================

set RESTART = "y"

if (-e $HIDDEN_INSTALL) then
echo ""
echo "Welcome To Serpent Installation."
echo ""
echo "  It appears as though Install has been previously run. In order to save"
echo "time and effort, Install can try to pick up where you last left off. This"
echo "means that your previous answers will be used to build serpent. You can also"
echo "rebuild serpent from scratch and go through the entire installation process"
echo "again."
echo ""
echo -n "  Would you like to Install serpent from scratch?"
unset RESTART
while (! $?RESTART)
   echo -n " [y/n] :"
   switch ("$<")
 	case [Yy]:
    	   set RESTART = "y"
	   echo ""
 	   echo "  Starting from scratch..."
	   rm $HIDDEN_INSTALL
 	   breaksw
	case [Nn]:
           set RESTART = "n"
	   echo ""
	   echo -n "  Loading your previous answers..."
	   source $HIDDEN_INSTALL
	   echo "done."
	   breaksw
	default:
	   echo -n "  Please select y)es or n)o"
	   breaksw
   endsw
end #HIDDEN_INSTALL

endif
if ($RESTART == "y") then

#=============================================================================
# Shell Verification
#-----------------------------------------------------------------------------
#  DESCRIPTION: If the user is starting from scratch, Install will check the
#		shell commands it uses to verify two things. 1) that they
#		exist, and 2) that they are not aliased to anything which
#		could potentially damage the system. If such an alias is found
#		the user is informed of the problem and if it is deemed
#		harmful, install quits.		       
#=============================================================================

echo ""
echo ""
echo "Welcome to Serpent Installation."
echo ""
echo "  Before the installation may begin, Install will verify that all the commands"
echo "used during this process are present and will not damage your system."
echo "(eg clear aliased to rm)."
echo ""
echo "  Please wait while your shell and system commands are verified. This process"
echo "will take a few seconds."
echo ""
echo -n "Verifying Shell Commands..."

foreach COMMAND ($COMMANDLIST)
    ./config/verify $COMMAND
    set the_status = $status
    echo -n "."
    if ($the_status == "1") then
    	echo ""
     	echo ""
     	echo "**** ERROR ****"
     	echo "$COMMAND was not found as a valid command in your path. Please add it to"
	echo "your system or install it in your path if you are going to run Serpent."
	echo "Serpent will NOT execute without it."
        exit 1
    endif
    if ($the_status == "2") then
    	echo ""
    	echo ""
    	echo "**** WARNING ****"
    	echo "$COMMAND is aliased."
    	echo -n "  Will this command execute without harm to your system?"
    	unset NOTDONE
    	while (! $?NOTDONE)
 	    echo -n " [y/n] :"
 	    switch ("$<")
 	    case [Yy]:
 	    	set NOTDONE
	    	echo ""
	    	echo -n "Resuming Check."
 	    	breaksw
 	    case [Nn]:
 	  	echo "  Please unalias this command...Serpent requires it to execute properly."
	  	exit 1
	  	breaksw
	    default:
	   	echo -n "  Please select y)es or n)o"
	   	breaksw
	    endsw
        end # NOTDONE
    endif # the_status
end # foreach

if ($status != 0) then
  echo "Installation Failed."
  exit 1
else
  echo "done."
  echo "set VERIFY" > $HIDDEN_INSTALL
endif
endif # RESTART

#=============================================================================
# Misc Setup Details
#-----------------------------------------------------------------------------
#  DESCRIPTION: This section contains a check for the clear command, a
#		check for a mailing address and an editor selection option.
#		These are all small details which must be addressed before
#		the actual installation begins.
#=============================================================================

#------------------------------------
# Clear must be verified every time.
#------------------------------------

./config/verify clear
set the_status = $status
if ($the_status != 0) then
    ./config/verify cls
    set the_status = $status
    if ($the_status == 0) then
       unalias CLEAR
       alias CLEAR cls
    else
        set NOCLEAR
    endif
else
    alias CLEAR clear
endif

#----------------------------------
# Get the mailing address and path
#----------------------------------

source $ADDRESS_FILE

echo ""
echo -n "                          Press Return to Continue"
set bogotroid = $<

if (! $?NOCLEAR) then
  CLEAR
endif

#------------------
# Select an Editor
#------------------
if ($?editor) then
else
set editor = emacs

if ($?VISUAL) then
    if ("$?VISUAL" != "") set editor = "$VISUAL"
else if ($?EDITOR) then
    if ("$?EDITOR" != "") set editor = "$EDITOR"
else
    echo "You do not have a default editor set. Please specify an editor which you"
    echo "can run."
    echo ""
    echo -n "What editor would you like to use? [$editor]: "
    set editor = $<
    if ("$editor" == "") set editor = emacs
endif

set the_editor = $editor
echo "set editor = $the_editor" >> $HIDDEN_INSTALL
endif

#=============================================================================
# Install 
#-----------------------------------------------------------------------------
#  DESCRIPTION: This is the point at which the real installation of serpent
#               begins.
#=============================================================================

if (! $?NOCLEAR) then
  CLEAR
endif
echo "================================================================================"
echo ""
echo "     ####### #                                                       ##"
echo "   ####  ####                                                       ###"
echo "   ####          #####   ### ####  ### ####     #####   ### ####   #######"
echo "    #########   ###  ###  ####  ##  ####  ###  ###  ###  ####  ###  ###"
echo "          ####  ########  ###       ###   ###  ########  ###   ###  ###"
echo "    ####  ####  ###       ###       ####  ###  ###       ###   ###  ###  ##"
echo "   # #######     ######  #####      ### ###     ######  ##### #####  #####"
echo "                                    ###"
echo "                                   #####       INSTALLATION"
echo ""
echo "================================================================================"

echo ""

echo "  Welcome to Serpent. Serpent can only be installed if the serpent.env file"
echo "found in this directory has been added to your .cshrc file (or the"
echo "equivalent to csh environment variables are set for your shell)."
echo "If you have not done this already, please quit, make the changes and"
echo "re-source your .cshrc file."
echo ""
echo -n "  Do you wish to proceed with the installation at this time?"
unset okay
while (! $?okay)
    echo -n " [y/n] :"
    switch("$<")
    case [Yy]:
	set okay
	breaksw
    case [Nn]:
	echo ""
	echo "Installation Aborted"
	exit(1)
	breaksw
    default:
	echo -n "  Please select y)es or n)o"
	breaksw
    endsw
end # okay

#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================

#------------------------------
# Ask for Mailing Instructions
#------------------------------
#--------------------------------------
# Make sure the user isn't too uptight
#--------------------------------------

if ($?MAIL_OK) then
else
source $HEADER_FILE
echo "  The Serpent installation program sets up information which is used by"
echo "several serpent systems to communcate with us. This includes bug reports"
echo "and registration forms. This feature will help us provide support for your"
echo "particular architecture."
echo ""
echo "  We would greatly appreciate receiving this information from you, so we"
echo "tried to make the process as easy as possible. If you have email"
echo "capabilities, Serpent installation will try to make use of them unless you"
echo "specify otherwise."
echo ""
echo -n "  Would you like to use the automatic email feature?"
unset bummer
while (! $?bummer)
    echo -n " [y/n] :"
    switch ("$<")
    case [Nn]:
	echo ""
	echo "  Would you like Install to produce files which you could mail to us"
	echo -n "yourself?"
	unset doublebummer
	while (! $?doublebummer)
	    echo -n " [y/n] :"
	    switch ("$<")
	    case [Yy]:
		unset BS
		while (! $?BS)
		    echo ""
		    echo "  Please enter a directory to which to save these files:"
		    echo -n "  >"
		    set MAIL_PATH = $<
		    if (-e $MAIL_PATH) then
			set BS
		    else
		    	echo ""
		    	echo "  [$MAIL_PATH] was not a valid path. Please try again."
		    endif
		end
		set doublebummer
		set SNAILMAIL
		set NOEMAIL
		breaksw
	    case [Nn]:
		set NOEMAIL
		set doublebummer
		breaksw
	    default:
		echo -n "  Please select y)es or n)o"
		breaksw
	    endsw
	end # doublebummer
	set bummer
	breaksw
    case [Yy]: # You really have to love this person...
	set EMAIL
	set bummer
	breaksw
    default:
	echo -n "Please select y)es or n)o"
	breaksw
    endsw
end #bummer
echo "set MAIL_OK" >> $HIDDEN_INSTALL
endif

#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================

#----------------------
# Get PATH for SENDMAIL
#----------------------

if (! $?NOEMAIL) then

if ($?SENDMAIL_OK) then
else

source $HEADER_FILE

if (($MAILPATH == "NULL") || (!(-e $MAILPATH/sendmail))) then
    echo "  Install is unable to locate sendmail. Can you send mail from your particular"
    echo -n "location?"
    unset done
    while (! $?done)
        echo -n " [y/n] :"
        switch ("$<")
        case [Yy]:
	    unset found
	    while (! $?found)
	  	echo ""
	  	echo "  Please type the full path to the directory containing sendmail."
	  	echo ""
	  	echo -n "  >"
	  	set MAILPATH = $<
	  	if (-e $MAILPATH/sendmail) then
	    	    set found
          	endif
	    end # found
            breaksw
        case [Nn]:
            set done
	    echo "set MAILPATH = NULL" > $ADDRESS_FILE
                echo "unset LIST_ADDR" >> $ADDRESS_FILE
                echo "unset BUGS_ADDR" >> $ADDRESS_FILE
		echo "unset ADDRESS" >> $ADDRESS_FILE
	    source $ADDRESS_FILE
            breaksw
        default:
            echo -n "Please select y)es or n)o"
            breaksw
        endsw
    end # done
endif #MAILPATH stuff

#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================

#---------------------------
# Get Address
#---------------------------

unset nomail
source $ADDRESS_FILE

if ($MAILPATH != "NULL") then
    echo ""
    echo "  In case of future problems, Serpent must have an address at which you can"
    echo "reach its development team."
    echo "  Two different addresses must be specified for use by serpent and"
    echo "install."
    echo ""
    if (! $?ADDRESS) then
      echo "  No Addresses have been specified. This suggests that you cannot send us"
      echo -n "email. Is this correct?"
      unset okay
      while (! $?okay)
        echo -n " [y/n] :"
        switch ("$<")
        case [Yy]
            set okay
            set nomail
            echo "set MAILPATH = $MAILPATH" > $ADDRESS_FILE
            echo "unset LIST_ADDR" >> $ADDRESS_FILE
            echo "unset BUGS_ADDR" >> $ADDRESS_FILE
            echo "unset ADDRESS" >> $ADDRESS_FILE
	    breaksw
	case [Nn]
	    set okay
	    unset nomail
	    echo ""
	    set LIST_ADDR = "serpent-list-request@sei.cmu.edu"
	    set BUGS_ADDR = "serpent-bugs@sei.cmu.edu"
	    set ADDRESS
	    breaksw
	default:
	    echo -n "Please select y)es or n)o "
	    breaksw
	endsw
    endif

if (! $?nomail) then
      echo "  Can you reach serpent-list-request by sending mail to:"
      echo -n "  [$LIST_ADDR] ?"
    unset okay
    while (! $?okay)
	echo -n " [y/n] :"
	switch ("$<")
	case [Yy]
	    set okay
                echo "set MAILPATH = $MAILPATH" > $ADDRESS_FILE
                echo "set LIST_ADDR = $LIST_ADDR" >> $ADDRESS_FILE
            breaksw
	case [Nn]
            echo ""
	    echo "  Please enter the correct address below. Press Return if you cannot reach"
	    echo "us via email."
	    echo -n "  >"
            set LIST_ADDR = $<
	    if ($LIST_ADDR == "") then
	   	echo "set MAILPATH = $MAILPATH" > $ADDRESS_FILE
		echo "unset LIST_ADDR" >> $ADDRESS_FILE
		echo "unset BUGS_ADDR" >> $ADDRESS_FILE
		echo "unset ADDRESS" >> $ADDRESS_FILE
                set nomail
	    else
	   	echo "set MAILPATH = $MAILPATH" > $ADDRESS_FILE
		echo "set LIST_ADDR = $LIST_ADDR" >> $ADDRESS_FILE
	    endif # ADDRESS
            set okay
            breaksw
	default:
	    echo -n "  Please select y)es, n)o"
	    breaksw
   	endsw
    end #okay
   
  if (! $?nomail) then
    unset okay
    echo ""
    echo "  Can you reach serpent-bugs by sending mail to:"
    echo -n "  [$BUGS_ADDR] ?"
    while (! $?okay)
        echo -n " [y/n] :"
        switch ("$<")
        case [Yy]
            set okay
            echo "set BUGS_ADDR = $BUGS_ADDR" >> $ADDRESS_FILE
            echo "set ADDRESS" >> $ADDRESS_FILE
            breaksw
        case [Nn]
            echo ""
            echo "  Please enter the correct address below."
            echo -n "  >"
            set BUGS_ADDR = $<
            echo "set BUGS_ADDR = $BUGS_ADDR" >> $ADDRESS_FILE
            echo "set ADDRESS" >> $ADDRESS_FILE
            set okay
            breaksw
        default:
            echo -n "  Please select y)es, n)o"
            breaksw
        endsw
    end #okay
    source $ADDRESS_FILE
  endif #nomail - yes there are two here.
  endif #nomail
endif #MAILPATH
echo "set SENDMAIL_OK" >> $HIDDEN_INSTALL
endif
endif #NOEMAIL


#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================

#------------------------
# Registration Procedure
#------------------------

#----------------------
# REGISTER USER
#----------------------
if ($?REGISTERED) then
else

if ((($?LIST_ADDR) && (! $?NOEMAIL)) || ($?SNAILMAIL)) then

    if (! $?skip) then

	source $HEADER_FILE
	echo "  As Serpent is still experimental, it is necessary for the Serpent"
	echo "development team to keep track of who has a copy of the system. This will"
	echo "allow us to send you further information about Serpent and keep you informed"
	echo "about future releases."
	echo ""
	echo -n "  Would you like to register your copy of serpent?"

	unset done
	while (! $?done)
            echo -n " [y/n] :"
            switch ("$<")
            case [Yy] :
            	unset register
             	set done
             	breaksw
            case [Nn] :
             	set register
             	set done
             	breaksw
	    default:
	    	echo -n "Please select Y)es or N)o"
	     	breaksw
            endsw
	end

	if (! $?register) then

	    #---------------
	    # REGISTER USER
	    #---------------

	    if ($?SNAILMAIL) then
		unset REGISTER_FILE
		set REGISTER_FILE = "${MAIL_PATH}/Register.SERPENT"
	    endif
	    echo ""
	    echo "  A registration template will now be loaded for you to edit. Please complete"
	    echo "the form, save it and exit the editor."
	    if ($?SNAILMAIL) then
		echo "" > $REGISTER_FILE
		echo "Affiliate Relations - Serpent" >> $REGISTER_FILE
		echo "Software Engineering Institute" >> $REGISTER_FILE
		echo "Carnegie Mellon University" >> $REGISTER_FILE
		echo "Pittsburgh, PA 15213-3890" >> $REGISTER_FILE
		echo "" >> $REGISTER_FILE
		echo "==============================================================================" >> $REGISTER_FILE
	    else
		echo "" > $REGISTER_FILE
		echo "==============================================================================" >> $REGISTER_FILE
	    endif
	    echo "                        Serpent Registration Template" >> $REGISTER_FILE
	    echo "==============================================================================" >> $REGISTER_FILE
	    echo "Name		:" >> $REGISTER_FILE
	    echo "Organization	:" >> $REGISTER_FILE
	    echo "Address		:" >> $REGISTER_FILE
	    echo "		:" >> $REGISTER_FILE
	    echo "Phone		:" >> $REGISTER_FILE
	    echo "Email 		:" >> $REGISTER_FILE
	    echo "------------------------------------------------------------------------------" >> $REGISTER_FILE
	    echo "How did you acquire Serpent?" >> $REGISTER_FILE
	    echo "" >> $REGISTER_FILE
	    echo "How did you find out about Serpent?" >> $REGISTER_FILE
	    echo "" >> $REGISTER_FILE
	    echo "What hardware/software platform are you running Serpent on?" >> $REGISTER_FILE
	    echo "" >> $REGISTER_FILE
	    echo "Do you wish to be added to the Serpent technical mailing list?" >> $REGISTER_FILE
	    echo "------------------------------------------------------------------------------" >> $REGISTER_FILE
	    echo "Comments:" >> $REGISTER_FILE
	    echo "" >> $REGISTER_FILE
	    echo "" >> $REGISTER_FILE
	    echo "==============================================================================" >> $REGISTER_FILE
	    echo "			     $VERSION" >> $REGISTER_FILE

	    $editor $REGISTER_FILE
	    while ($status != 0)
		echo "Something happened to the editor session; please try again"
		$editor $REGISTER_FILE
	    end
	    if ($?SNAILMAIL) then
	        echo -n "would you like to S)ave the registration, E)dit it again or A)bort?"
	    else
	        echo -n "would you like to S)end the registration, E)dit it again or A)bort?"
	    endif
	    unset done
	    while (! $?done)
		echo -n " [s/e/a] :"
		switch ("$<")
		case [Ss]:
		    if ($?SNAILMAIL) then
			echo "  $REGISTER_FILE saved, please send it as soon as possible."
			echo ""
			echo -n "                         Press Return to Continue"
		    else
	  	    	source $HEADER_FILE
	  	    	echo "           You are being registered as a serpent user...please wait."
          	    	(\
	  	    	echo "To: $LIST_ADDR" ;\
	  	    	echo "From: $user" ;\
	  	    	echo "Subject: Serpent 1.2 Registration" ;\
	  	    	echo "" ;\
	  	    	cat $REGISTER_FILE ;\
	  	    	) | /usr/lib/sendmail -t -om
	  	    	if ($status != 0) then
	     	   	    echo "An Error occured while mailing...please verify address."
			    echo "The registration file was saved to $REGISTER_FILE
	  	        else
	  	            rm $REGISTER_FILE*
			endif
		    endif
	  	    set done
		    breaksw
		case [Ee]:
	  	    $editor $REGISTER_FILE
	  	    breaksw
		case [Aa]
	  	    exit 1
	  	    breaksw
		default:
		    if ($?SNAILMAIL) then
			echo -n "  Please select s)ave, e)dit or a)bort"
		    else
	  	        echo -n "  Please select s)end, e)dit or a)bort"
		    endif
	  	    braksw
		endsw
	    end

	endif # register

    endif # skip

endif # ADDRESS

if ($status != 0) then
  exit 1
endif
echo "set REGISTERED" >> $HIDDEN_INSTALL
endif

#------------------------
# Configuration Procedure
#------------------------
#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================

#---------------
# CONFIGURATION
#---------------

if ($?CF_OK) then
else

source $HEADER_FILE
   echo "                   Please wait...Checking your system type..."

cc $CHECKP_SOURCE -o $CHECKP_OBJECT
set the_status = $status
if ($the_status != 0) then
  echo "The cc command blew up; are you sure you have cc?"
  exit(1)
endif
$CHECKP_OBJECT > $CHECKP_RESULT
source $CHECKP_RESULT

source $HEADER_FILE
echo "  Serpent must refer to a machine specific configuration file. Such files"
echo "are included with Serpent. It is suggested that the configuration file that"
echo "fits your machine be checked to ensure that it matches your particular"
echo "configuration."
echo ""
echo "  Your machine has been identified as a $MACHINETYPE."
echo "  The $CFFILE file has been selected as your configuration file."
if ($MACHINETYPE =~ "sun") then
  echo ""
  echo "  *** NOTE *** "
  echo "    Sun users should edit this file to verify which version of SunOS"
  echo "    is being used by their system."
endif
echo ""
echo "  Would you care to view/edit this file before continuing with the installation"
echo -n "process?"
unset okay
while (! $?okay)
    echo -n " [y/n] :"
    switch ("$<")
    case [Yy]:
	echo ""
	echo "Preparing $CFFILE for editing."
	echo "When you are finished, please save the file and exit the editor."
	$editor config/$CFFILE
	unset done
	while (! $?done)
	    source $HEADER_FILE
	    echo "  The configuration file $CFFILE has been edited."
     	    echo ""
            echo -n "Do you wish to C)ontinue, E)dit the file again, or A)bort?"
     	    echo -n " [c/e/a] :"
     	    switch ("$<")
     		case [Aa]:
		   exit 1
		   breaksw
     		case [Ee]:
        	   $editor ./config/$CFFILE
		   breaksw
     		case [Cc]:
		   set done
        	   breaksw
     	    endsw
   	end # while
   	set okay
   	breaksw
    case [Nn]:
   	set okay
      	breaksw
    endsw
end # okay

if ($status != 0) then
  exit 1
endif
echo "set CF_OK" >> $HIDDEN_INSTALL
endif

#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================
#------------------------
# Check Site.def
#------------------------

if ((-e $HIDDEN_TECHFILE) && (-e $USR_FILE)) then
   source $HEADER_FILE
   echo "  Serpent makes use of a file called $USR_FILE_NAME to store variables which"
   echo "are used to determin your particular configuration. These variables include"
   echo "information about such things as where X, Motif and Athena are stored,"
   echo "compilation parameters and compiler types."
   echo ""
   echo "  Install has detected that $USR_FILE_NAME has previously been installed."
   echo ""
   echo "   Would you like to E)dit $USR_FILE_NAME, R)ebuild $USR_FILE_NAME from scratch or"
   echo -n "  C)ontinue installation?"
   unset done
   while (! $?done)
     echo -n " [e/r/c] :"
     switch ("$<")
	case [Cc]:
	   set SKIPUSRDEFS
	   echo ""
	   echo -n "  Have you changed $USR_FILE_NAME since the last installation?"
	   unset cool
	   while (! $?cool)
		echo -n " [y/n] :"
		switch ("$<")
		case [Yy]:
		   set EDITUSER
		   if (-e $HIDDEN_TECHFILE) then
			rm $HIDDEN_TECHFILE
		   endif
		   set cool
		   breaksw
		case [Nn]:
		   unset EDITUSER
		   set cool
		   breaksw
		default:
		   echo "Please select y)es or n)o"
		   breaksw
		endsw
	   end #cool
	   set done
	   breaksw
	case [Ee]:
	   set SKIPUSRDEFS
	   echo ""
	   unset EditDone
	   while (! $?EditDone)
	     $editor $USR_FILE
	     echo "Would you like to E)dit $USR_FILE_NAME again, C)ontinue with the installation,"
	     echo -n "R)ebuild $USR_FILE_NAME from scratch or A)bort?"
	     unset ECA
	     while (! $?ECA)
		echo -n " [e/c/r/a] :"
		switch ("$<")
		  case [Ee]:
		    set ECA
		    breaksw
		  case [Cc]:
		    set ECA
		    set EditDone
		    set EDITUSER
		    if (-e $HIDDEN_TECHFILE) then
			rm $HIDDEN_TECHFILE
		    endif
		    breaksw
		  case [Aa]:
		    echo ""
		    echo "  Serpent Installation Aborted."
		    exit 1
		    breaksw
		  case [Rr]:
		   unset SKIPUSRDEFS
		   set EditDone
		   set ECA
		   breaksw
		  default:
		    echo -n "  Please select E)dit, C)ontinue, R)ebuild or A)bort"
		    breaksw
		endsw
	     end #while
	   end #while
	case [Rr]:
	   unset SKIPUSRDEFS
	   set done
	   breaksw
	default:
	   echo -n "  Please select E)dit, R)ebuild or C)ontinue"
	   breaksw
	endsw
     end #while
endif

#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================

if (! $?SKIPUSRDEFS) then
 
    unset HASATHENA
    unset HASMOTIF]
    unset NOTECH

    while ((! $?HASATHENA) && (! $?HASMOTIF) && (! $?NOTECH))

	#----------------------------
	# Get X11 Inc & Athena Dirs
	#----------------------------

#----------------------------------------------------------------
# DIRECORY PATHS EDIT ON A *VERY* LARGE SCREEN (over 120 cols.)
#----------------------------------------------------------------

if ($?HAS_R4) then
else

source $HEADER_FILE

echo "  The X Window System, Version 11 Release 4 is required for Serpent to"
echo "build and execute. X11R4 files must be present for Serpent to use."
echo ""
echo -n "  Do you have X11R4?"
unset yeah
while (! $?yeah)
    echo -n " [y/n] :"
    switch ("$<")
    case [Yy]:
	set HASR4
	echo "set HAS_R4" >> $HIDDEN_INSTALL
	set Version = "X11R4"
	set yeah
	breaksw
    case [Nn]:
	set yeah
	breaksw
    default:
	echo -n "  Please select y)es or n)o"
	breaksw
    endsw
end #yeah

if (! $?HASR4) then

#-----------------------------------
# Does not have X11R4...
#   who is this person?
#-----------------------------------

    source $HEADER_FILE
    echo "  Serpent can currently only be run under X11R4."
    echo "  After installing X11R4, please re-install Serpent."
    echo ""
    echo "  Serpent has Aborted."
    if (-e $USR_FILE) then
	rm $USR_FILE
    endif
    if (-e $HIDDEN_TECHFILE) then
	rm $HIDDEN_TECHFILE
    endif
    exit 1
endif
endif

#-----------------------------------
# Has R3 or R4... Check all the fun
# places it might be hiding...
#-----------------------------------

if ($?R4_INC) then
else

source $HEADER_FILE
echo "  Since X11 is not distributed with serpent, it is necessary to verify the"
echo "location of the $Version Include and Library Directories. Note that the"
echo "X11 Include files should be stored in a directory named /X11."
echo ""
echo "               Verifying /X11 Include Directory...Please Wait..."

echo "/*-------------------------------------------------------------" > $USR_FILE
echo "* User specified macro definitions for use by Imake." >> $USR_FILE
echo "* These definitions were set at installation time and may be  " >> $USR_FILE
echo "* changed whenever the User wishes to do so." >> $USR_FILE
echo "*-------------------------------------------------------------*/" >> $USR_FILE
echo "" >> $USR_FILE

unset FOUNDPATH
foreach  PATH ($INCDEFAULTLIST)
    if (-e $PATH/X11/X.h) then
	set FOUNDPATH = $PATH;
	set legaldir
	break;
    endif
end # foreach

if ($?FOUNDPATH) then
   echo ""
   echo "  The /X11 Include directory was found to be:"
   echo "  [$PATH/X11]"
   echo ""
   echo -n "  Is this correct?"
   unset done
   while (! $?done)
	echo -n " [y/n] :"
	switch("$<")
	case [Yy]:
       	    echo "SERPENTXINCDIR = $FOUNDPATH" >> $USR_FILE
	    set X11INCLUDE = $FOUNDPATH
            set done
            breaksw
     	case [Nn]:
            unset FOUNDPATH
            set done
            breaksw
     	default:
	    echo -n "  Please Select y)es or n)o."
	    breaksw
     	endsw
   end # while
endif # FOUNDPATH

#--------------------------
# Sheesh...I can't find it
#--------------------------

if (! $?FOUNDPATH) then

    source $HEADER_FILE
    echo "       **** INSTALL WAS UNABLE TO LOCATE YOUR X11 INCLUDE DIRECTORY ****"
    echo ""
    echo "  Remember that all X11 files must be found in a /X11 directory. This structure"
    echo "is required for Serpent to build and execute under X11."
    echo ""
    unset peachy
    while (! $?peachy)
	echo "  Please type the full path to your X11 Include files:"
	echo -n "  >"
	set FOUNDPATH = $<
	if (-e $FOUNDPATH/X.h) then
	    echo ""
	    echo "  The X11 Include path was verfied to be [$FOUNDPATH]"
	    echo ""
	    echo -n "                          Press Return to Continue"
	    set bogus = $<
	    echo "SERPENTXINCDIR = $FOUNDPATH:h" >> $USR_FILE
	    set X11INCLUDE = $FOUNDPATH:h
	    set peachy
	else
	    echo ""
	    echo "  Verification of [$FOUNDPATH] failed. Please try again."
	    echo ""
	endif
    end #peachy

endif # FOUNDPPATH


	if ($status != 0) then
	    exit 1
	endif
echo "set R4_INC" >> $HIDDEN_INSTALL
endif
	#----------------------------
	# Get X11 Library Directory
	#---------------------------

#-----------------------
# X LIBRARY DIRECTORY
#-----------------------

if ($?R4_LIB) then
else

source $HEADER_FILE

echo "                Verifying X11 Library Directory...Please Wait..."

set FOUNDPATH = $FOUNDPATH:h

unset FOUND
if (-e $FOUNDPATH/lib/libX11.a) then
    set FOUND
else
    unset FOUNDPATH
    foreach PATH ($LIBDEFAULTLIST)
	if (-e $PATH/libX11.a) then
	    set FOUNDPATH = $PATH:h
	    set FOUND
	    break
	endif
    end #foreach
endif #FOUNDPATH STUFF

if ($?FOUND) then
   echo ""
   echo "  The X11 library files were found to reside in:"
   echo "  [$FOUNDPATH/lib]"
   echo ""
   echo -n "  Is this correct?"
   unset done
   while (! $?done)
	echo -n " [y/n] :"
    	switch("$<")
    	case [Yy]:
            echo "SERPENTXLIBDIR = $FOUNDPATH/lib" >> $USR_FILE
            set done
            breaksw
    	case [Nn]:
            unset FOUND
            set done
            breaksw
    	default:
            echo -n "  Please Select y)es or n)o."
            breaksw
   	endsw
   end # while
endif

if (! $?FOUND) then
   source $HEADER_FILE
   echo ""
   echo "      **** INSTALL WAS UNABLE TO LOCATE YOUR X11R4 LIBRARY DIRECTORY ****"
   while (! $?FOUND)
   	echo ""
   	echo "  Please type in the full path for your X11 library directory:"
   	echo ""
   	echo -n "  >"
   	set LIBCHANGE = $<
   	if (-e $LIBCHANGE/libX11.a) then
     	    echo ""
     	    echo "  The X11 library directory has been confirmed to be:"
     	    echo "  [$LIBCHANGE]"
     	    echo ""
     	    echo -n "                            Press Return To Continue"
     	    set Bogus = $<
     	    echo "SERPENTXLIBDIR = $LIBCHANGE" >> $USR_FILE
     	    set FOUND
   	else
     	    echo ""
     	    echo "  Verification of [$LIBCHANGE] failed. Please try again."
   	endif
   end
endif
echo "set R4_LIB" >> $HIDDEN_INSTALL
endif
	#----------------------------
	# Get Athena
	#----------------------------

#--------
# ATHENA
#--------

if ($?ATHENA_INC) then
set HASATHENA
else

unset RETRYLOOP
echo "" >> $USR_FILE

while (! $?RETRYLOOP)

unset HASATHENA4
unset NOATHENA
unset NOINTRINSICS
unset HASATHENA

    source $HEADER_FILE
    echo "                    Verifying Athena Files...Please Wait..."
    if (-e $X11INCLUDE/X11/Intrinsic.h) then
    	if ((-e $X11INCLUDE/X11/Xaw) && (-e $X11INCLUDE/X11/Xaw/Cardinals.h)) then
	    if ($?HASR4) then
	    	set HASATHENA4
		set RETRYLOOP
	    endif
	else
	    set NOATHENA
	    set RETRYLOOP
        endif
    else
   	set NOINTRINSICS
    endif

    if ($?NOATHENA) then
	echo ""
	echo "  Athena was not found in the /X11 directory. It will not be installed."
	echo ""
	echo -n "                       Press Return to Continue"
	set bogus = $<
	echo "#undef HasAthena" >> $USR_FILE
	unset HASATHENA
    endif #NoAthena
    
    if ($?NOINTRINSICS) then
    	echo ""
    	echo "  The files necessary to implement Athena were not found in the /X11"
    	echo "directory. If you want to install Athena, please make the necessary"
    	echo "changes."
    	echo ""
    	echo -n "                       Press Return to Continue"
    	set bogus = $<
    	echo "#undef HasAthena" >> $USR_FILE
    	unset HASATHENA
	set RETRYLOOP
    endif

end #RETRYLOOP

if ($?HASATHENA4) then
    echo ""
    echo -n "  Athena has been detected. Would you like to install it?"
    unset peachy
    while (! $?peachy)
	echo -n " [y/n] :"
	switch ("$<")
	    case [Yy] :
		echo "#define HasAthena" >> $USR_FILE
		set DONE
		set HASATHENA
		set peachy
		breaksw
	    case [Nn] :
		echo "#undef HasAthena" >> $USR_FILE
		set DONE
		unset HASATHENA
		set peachy
		breaksw
	    default:
		echo -n "  Please select y)es or n)o"
	endsw
    end #peachy
endif # HASATHENA4

	if ($status != 0) then
	    exit 1
	endif
echo "set ATHENA_INC" >> $HIDDEN_INSTALL
endif

	#----------------------------
	# Get Motif Direcories
	#----------------------------

#------------------
# MOTIF INCLUDES
#------------------

if ($?MOTIF_INC) then
set HASMOTIF
else

source $HEADER_FILE
echo "               Verifying Motif Include Directory...Please Wait..."
unset FOUND
unset FOUNDPATH
unset SPATH
foreach SPATH ($MOTIFDEFLIST)
    if (-e $SPATH/include) then
	set FOUND
	set FOUNDPATH = $SPATH
	set DISCLAIMER
	break
    endif
end

if (! $?FOUND) then
    echo ""
    echo "  Install was unable to detect the presence of Motif."
    echo -n "  Do you have Motif?"
    unset okay
    while (! $?okay)
	echo -n " [y/n] :"
	switch("$<")
	case [Yy]:
	    set DISCLAIMER
	    set okay
	    breaksw
	case [Nn]:
	    set SKIPITALL
	    unset HASMOTIF
	    set okay
	    breaksw
	default:
	    echo -n "  Please select y)es or n)o"
	    breaksw
	endsw
    end #okay
endif

if ($?DISCLAIMER) then
    echo ""
    echo "  Serpent currently runs using Motif 1.1.1 under X11R4. Although"
    echo "other types of Motif may work, we cannot guarantee that they will,"
    echo "nor do we suggest you use them."
    echo ""
    echo -n "  Would you like to install Motif?"
    unset okay
    while (! $?okay)
	echo -n " [y/n] :"
	switch ("$<")
	case [Yy]:
	    set okay
	    unset SKIPITALL
	    breaksw
	case [Nn]:
	    set SKIPITALL
	    set okay
	    breaksw
	default:
	    echo -n "  Please select y)es or n)o"
	    breaksw
	endsw
    end # okay
endif # DISCLAIMER

if (! $?SKIPITALL) then #------------------------------------------------------------------

    if ($?FOUND) then
    	source $HEADER_FILE
    	echo ""
    	echo "  The Motif Include files were found to reside in:"
    	echo ""
    	echo "  [$FOUNDPATH/include]"
    	echo ""
    	echo -n "  Is this correct?"
    	unset YN
    	while (! $?YN)
  	    echo -n " [y/n] :"
	    switch ("$<")
	    case [Yy]:
	    	echo ""
	    	echo "#define HasMotif" >> $USR_FILE
		echo "" >>$USR_FILE
	    	echo "SERPENTMOTIFINCDIR = $FOUNDPATH/include" >> $USR_FILE
	    	set YN
	    	set HASMOTIF
	    	breaksw
	    case [Nn]
	    	set YN
	    	unset FOUND
	    	breaksw
	    default:
	    	echo -n "  Please select y)es or n)o"
	    	breaksw
	    endsw
    	end # YN
    endif # FOUND

    if (! $?FOUND) then
    	source $HEADER_FILE
    	echo "       *** INSTALL WAS UNABLE TO FIND YOUR MOTIF INCLUDE DIRECTORY ***"
    	echo "" >> $USR_FILE
    	echo "#define HasMotif" >> $USR_FILE
    	echo ""
    	while (! $?FOUND)
    	    echo "  Please type in the complete path for your Motif Include directory:"
    	    echo ""
    	    echo -n "  >"
    	    set FOUNDPATH = $<
    	    if (-e $FOUNDPATH) then
      	    	echo ""
      	    	echo "  The Motif Include directory has been verified."
      	    	set FOUND
		echo "#define HasMotif" >> $USR_FILE
      	    	echo "SERPENTMOTIFINCDIR = $FOUNDPATH" >> $USR_FILE
      	    	echo ""
      	    	echo -n "  Press Return to Continue"
      	    	set bogus = $<
      	    	set FOUNDPATH = $FOUNDPATH:h
	    	set HASMOTIF
      	    else
      	    	echo ""
      	    	echo "  The Motif include directory was not found. Please try again."
      	    	echo ""
    	    endif
    	end #FOUND
    endif #FOUND
echo "set MOTIF_INC" >> $HIDDEN_INSTALL

    #------------------
    # MOTIF LIBRARIES
    #------------------
if ($?MOTIF_LIB) then
set HASMOTIF
else

    source $HEADER_FILE
    echo "              Verifying Motif Library Directory...Please Wait..."

    unset FOUND
    if (-e $FOUNDPATH/lib) then                    # TEST LAST PATH /LIB
   	set FOUND
    endif

    if (! $?FOUND) then			       # TEST LIST O PATHS
	unset FOUNDPATH
	unset SPATH
	foreach SPATH ($MOTIFDEFLIST)
	    if (-e $SPATH/lib) then
     		set FOUND
     		set FOUNDPATH = $SPATH
     		break
  	    endif
	end
    endif

    unset SKIPHAS

    if ($?FOUND) then
    	echo ""
    	echo "  The Motif Library files were found to reside in:"
    	echo ""
	echo "  [$FOUNDPATH/lib]"
  	echo ""
  	echo -n "  Is this correct?"
  	unset YN
  	while (! $?YN)
  	    echo -n " [y/n] :"
	    switch ("$<")
	    case [Yy]:
	  	echo ""
	  	echo "SERPENTMOTIFLIBDIR = $FOUNDPATH/lib" >> $USR_FILE
	  	set YN
	  	set SKIPHAS
		set FOUND
	  	breaksw
	    case [Nn]
	  	set YN
	  	unset FOUND
	  	breaksw
	    default:
	  	echo -n "  Please select Y)es or N)o"
	  	breaksw
	    endsw
	end # YN
    endif # FOUND

    if (! $?FOUND) then
  	source $HEADER_FILE
  	echo "       *** INSTALL WAS UNABLE TO FIND YOUR MOTIF LIBRARY DIRECTORY ***"
  	echo ""
 	while (! $?FOUND)
    	    echo "  Please type in the complete path for your Motif Library directory:"
    	    echo ""
    	    echo -n "  >"
    	    set FOUNDPATH = $<
    	    if (-e $FOUNDPATH) then
      		echo ""
      		echo "  The Motif Library directory has been verified."
      		set FOUND
      		echo "SERPENTMOTIFLIBDIR = $FOUNDPATH" >> $USR_FILE
      		echo ""
      		echo -n "                             Press Return to Continue"
      		set bogus = $<
   	    else
      		echo ""
      		echo "  The Motif Library directory was not found. Please try again."
      		echo ""
    	    endif # FOUNDPATH
 	end # FOUND
    endif # FOUND
echo "set MOTIF_LIB" >> $HIDDEN_INSTALL
endif

endif # SKIPITALL
endif # MOTIF_INC

#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================


if ((! $?HASMOTIF) && (! $?HASATHENA)) then
    source $HEADER_FILE
    echo "  You have opted to install Serpent without Athena and Motif. If you continue"
    echo "installation, you will be unable to use any of our demos."
    echo ""
    echo "  Do you wish to C)ontinue installation, or return to I)nstall"
    echo -n "Athena and/or Motif?"
    unset okay
    while (! $?okay)
	echo -n " [c/i] :"
	switch ("$<")
	case [Cc]:
	    set NOTECH
	    set okay
	    breaksw
	case [Ii]:
	    set okay
	    breaksw
	default:
	    echo -n "  Please select c)ontinue or i)nstall"
            breaksw
	endsw
    end #okay
endif # HASMOTIF HASATHENA --- TECH CHECK


#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================

#--------------------------
# SET UP RECORD OF INSTALL
#--------------------------

if (-e $HIDDEN_TECHFILE) then
    rm $HIDDEN_TECHFILE
endif

if ($?NOTECH) then
   echo "set NOTECH" >> $HIDDEN_TECHFILE
endif
if ($?HASMOTIF) then
   echo "set HASMOTIF" >> $HIDDEN_TECHFILE
else
   echo "unset HASMOTIF" >> $HIDDEN_TECHFILE
endif
if ($?HASATHENA) then
   echo "set HASATHENA" >> $HIDDEN_TECHFILE
else
   echo "unset HASATHENA" >> $HIDDEN_TECHFILE
endif
unset EDITUSER

	if ($status != 0) then
	    exit 1
	endif

    end # HASATHENA HASMOTIF NOTECH

    #---------------------------
    # Get DEBUG Flags
    #----------------------------

#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================


#---------------
# Debug Flags
#---------------
if ($?DEBUG_SET) then
else

source $HEADER_FILE

echo "  It is possible to compile Serpent in Debug mode. Turning on the Serpent"
echo "debug mode will increase the size of objects, libraries and load modules by as"
echo "much as 90%. It will however provide you with an accurate trace of Serpent's"
echo "operations."
echo ""
echo -n "  Would you like to turn on Debug?"
unset done
unset TOTAL
while (! $?done)
    echo -n " [y/n] :"
    switch ("$<")
    case [Yy]:
	set done
	breaksw
    case [Nn]:
	set TOTAL = "-DNDEBUG"
	set done
	breaksw
    default:
	echo -n "  Please select y)es or n)o"
	breaksw
    endsw
end
echo "set DEBUG_SET" >> $HIDDEN_INSTALL
endif

if ($?OPT_SET) then
else

echo -n "  Would you like to Optimize the Serpent code?"
unset done
while (! $?done)
    echo -n " [y/n] :"
    switch("$<")
    case [Yy]:
	if (! $?TOTAL) then
	    set TOTAL = "-O"
	else
	    set TOTAL = "$TOTAL -O"
	endif
	set done
	breaksw
    case [Nn]:
	set done
	breaksw
    default:
	echo -n "  Please select y)es or n)o"
	breaksw
    endsw
end # done
echo "set OPT_SET" >> $HIDDEN_INSTALL
endif

if ($?G_SET) then
else

echo "  Would you like to compile Serpent with additional symbol table information"
echo -n "for use with dbx? (-g option)"
unset done
while (! $?done)
    echo -n " [y/n] :"
    switch("$<")
    case [Yy]:
	if (! $?TOTAL) then
	    set TOTAL = "-g"
	else
            set TOTAL = "$TOTAL -g"
	endif
        set done
        breaksw
    case [Nn]:
        set done
        breaksw
    default:
        echo -n "  Please select y)es or n)o"
        breaksw
    endsw
end
echo "set G_SET" >> $HIDDEN_INSTALL
endif

if (! $?TOTAL) then
    echo ""
else
    echo "" >> $USR_FILE
    echo "CDEBUGFLAGS = $TOTAL" >> $USR_FILE
endif

    #----------------------------
    # HASSABERC?
    #----------------------------
  
#-------------------------
# SABER C?
#-------------------------

if ($?SABER_SET) then
else

source $HEADER_FILE
echo -n "  Do you have SaberC?"
unset done
while (! $?done)
        echo -n " [y/n] :"
        switch ("$<")
	case [Yy]:
	  echo "" >> $USR_FILE
	  echo "#undef HasSaberC" >> $USR_FILE
	  echo "#define HasSaberC YES" >> $USR_FILE
	  set done
	  breaksw
	case [Nn]:
	  echo "#undef HasSaberC" >> $USR_FILE
	  set done
	  breaksw
	default:
	  echo -n "Please select y)es or n)o"
	  breaksw
	endsw
end
echo "set SABER_SET" >> $HIDDEN_INSTALL
endif

endif #SKIPUSRDEFS

#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================


#--------------------------
# Install Imake
#--------------------------

#--------------------
# IMAKE INSTALLATION
#--------------------

source $HEADER_FILE
echo "                    Now installing Imake files...please wait."
cd ./config
echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
make
set the_status = $status
cd ..
echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
if ($the_status != 0) then
    echo "Imake installation blew up"
    echo "NOTE: the Serpent version of imake is required to build Serpent"
    exit 1
endif
echo "                              Imake installed."
echo ""
echo -n "                          Press Return to Continue."
set bogus = $<

#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================


#------------
# MAKEFILES
#------------

source $HEADER_FILE
echo "                     Please Wait....Now Building Makefiles."
echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
imake
set the_status = $status
if ($the_status != 0) then
  echo "Imake blew up; exiting"
  exit 1
endif
make Makefiles
set the_status = $status
if ($the_status != 0) then
  echo "Making the Makefiles blew up; exiting"
  exit 1
endif

echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
echo "                               Makefiles Complete"
echo ""
echo -n "                           Press Return to Continue"
set bogus = $<

#------------------------------
# BUILD SERPENT
#------------------------------

#=============================================================================
# 
#-----------------------------------------------------------------------------
#  DESCRIPTION:
#=============================================================================


#-------------------------------
# BUILD SERPENT
#-------------------------------

set BUILDTIME = "3"
set BUILDEXEC = "1 1/2"
set BUILDDEP  = "1 1/2"
unset READY


while (! $?READY)
    source $HEADER_FILE

    if (($?EDITUSER) || (!(-e $HIDDEN_TECHFILE))) then
	echo "                             **** WARNING ****"
	echo "  You have made changes to usr.cf which Install has been unable to keep"
	echo "records of. It is impossible for install to tell if you have installed Athena"
	echo "or Motif. While this will not effect the compilation of Serpent, please"
	echo "remember that if you have not installed either Athena or Motif, the demos"
	echo "supplied with Serpent will not work. If this is the case, please only build"
	echo "the Serpent executables."
	echo ""
	echo -n "                         Press Return to Continue"
	set bogus = $<
	source $HEADER_FILE
    endif

    if ((! $?NOTECH) && (! $?HASATHENA) && (! $?HASMOTIF)) then
	if (-e $HIDDEN_TECHFILE) then
	    source $HIDDEN_TECHFILE
	endif
    endif

    if ($?NOTECH) then
	echo "  Only the executable portions of serpent may be built at this time. Motif and"
	echo "Athena have not been installed, thus preventing the compilation of the demos."
	echo ""
        echo "OPTION  DESCRIPTION                                                 TIME (Hours)"
        echo "--------------------------------------------------------------------------------"
	echo "  A     Only the Serpent executables are compiled.	               1 3/4"
	echo "        All intermediate files are removed after compilation."
	echo ""
	echo "  B     Only the Serpent executables are compiled.                       1 1/2"
	echo "--------------------------------------------------------------------------------"
	echo ""
        echo -n "  Please select a configuration [A-B] or [Q] to quit. :"
	unset done
        while (! $?done)
	    echo -n " [A-B,Q]:"
	    switch("$<")
	    case [Aa]:
	    	set done
		set READY
		set BUILD_EandC
		set TOTAL = "1 3/4"
		breaksw
	    case [Bb]:
		set done
		set READY
		set BUILD_EXECS
		set TOTAL = "1 1/2"
		breaksw
	    case [Qq]:
		set done
		breaksw
	    default:
		echo -n "Please select [A-B] or Q)uit "
		breaksw
	    endsw
	end
	unset ALLDEMOS
    else
        echo "  Serpent can be installed in several configurations."
        echo ""
        echo "OPTION  DESCRIPTION                                                 TIME (Hours)"
        echo "--------------------------------------------------------------------------------"
        echo "  A     All demos and Serpent executables are compiled.                 5 1/4"
        echo "        All intermediate files are removed after compilation."
        echo "        A dependency list is generated for Serpent."
        echo ""
        echo "  B     All demos and Serpent executables are compiled.                 3"
        echo "        A dependency list is generated for Serpent."
        echo ""
        echo "  C     All demos and Serpent executables are compiled.                 2 1/4"
        echo "        All intermediate files are removed after compilation."
        echo ""
        echo "  D     All demos and Serpent executables are compiled.                 2"
        echo ""
        echo "  E     Only the Serpent executables are compiled.                      1 3/4"
        echo "        All intermediate files are removed after compilation."
        echo ""
        echo "  F     Only the Serpent executables are compiled.                      1 1/2"
        echo "---------------------------------------------------------------------------------"
        echo "Note: Depending on configurations, Serpent may take up to 50MB."
	echo "All times represent time required on a fairly slow machine."
	echo "You will probably see better performance than that predicted."
        echo ""
        echo -n "  Please select a configuration [A-F] or [Q] to quit. :"
        unset done
        while (! $?done)
            echo -n " [A-F,Q]:"
            switch ("$<")
            case [Aa]:
                set done
		set READY
                set BUILD_EVERYTHING
                set TOTAL = "3"
                breaksw
	    case [Bb]:
		set done
		set READY
		set BUILD_DandD
		set TOTAL = "3"
	        breaksw
            case [Cc]:
		set done
		set READY
		set BUILD_CLEAN
		set TOTAL = "2 1/4"
		breaksw
	    case [Dd]:
		set done
		set READY
		set BUILD_DEMOS
		set TOTAL = "2"
	    case [Ee]:
		set done
		set READY
		set BUILD_EandC
		set TOTAL = "1 3/4"
		breaksw
	    case [Ff]:
		set done
		set READY
		set BUILD_EXECS
		set TOTAL = "1 1/2"
		breaksw
	    case [qQ]:
		set done
		breaksw
	    default:
		echo -n "Please select [A-F] or Q)uit"
		breaksw
	    endsw
	end
    endif

    if (! $?READY) then
        source $HEADER_FILE
    	echo "  You have opted not to build Serpent at this time."
    	echo ""
    	echo -n "Is this correct?"
    	unset done
    	while (! $?done)
	    echo -n " [y/n] :"
            switch ("$<")
            case [Yy]:
            	exit 1
            	breaksw
            case [Nn]:
            	set done
            	breaksw
            default:
            	echo -n "Please select y)es or n)o"
            	breaksw
            endsw
    	end # done
    endif # READY

end # READY

source $HEADER_FILE

echo "               NOW BUILDING SERPENT. PROCESS WILL TAKE $TOTAL HOURS."
echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
echo "  Lots of lines will be printed on your screen. Occasionally, there may"
echo "be a warning message or an indicator that an error code was ignored."
echo "These are no cause for alarm. The only thing you should worry about is"
echo "if the whole process fails; if that happens, see the file named"
echo "$SERPENT/notes/BUILD for further instructions"

set the_status = 0

if (($?BUILD_EVERYTHING) || ($?BUILD_DandD)) then
    echo "-------------------------------BUILDING SERPENT-------------------------------"
    make build
    set the_status = $status
    echo "------------------------------MAKE BUILD COMPLETE-----------------------------"
endif # build all demos

if ($the_status == 0) then
if (($?BUILD_CLEAN) || ($?BUILD_DEMOS) || ($?BUILD_EandC) || ($?BUILD_EXECS)) then
  echo "------------------------------BUILDING EXECUTABLES----------------------------"
    make serpent
    set the_status = $status
  echo "-----------------------------MAKE SERPENT COMPLETE----------------------------"
endif
endif

if ($the_status == 0) then
if (($?BUILD_CLEAN) || ($?BUILD_DEMOS)) then
  echo "------------------------------BUILDING DEMOS----------------------------------"
    make demo
    set the_status = $status
  echo "-----------------------------MAKE DEMO COMPLETE-------------------------------"
endif

if ($the_status == 0) then
if (($?BUILD_EVERYTHING) || ($?BUILD_CLEAN) || ($?BUILD_EandC)) then
  echo "-------------------------CLEANING OUT TEMPORARY FILES-------------------------"
    make clean
    set the_status = $status
  echo "-----------------------------MAKE CLEAN COMPLETE------------------------------"
endif

if ($the_status != 0) then
  echo make failed
  exit(1)
else
echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
echo "                               Building Complete"
echo ""
echo -n "                           Press Return to Continue"

set bogus = $<

source $HEADER_FILE
echo "                         Serpent Installation Complete."
echo ""
endif







