#!	/bin/sh

SERPENT=`pwd`
SERPENT_BASE_PATH=$SERPENT
SERPENT_DATA_PATH=.:$SERPENT_BASE_PATH/data:$SERPENT_BASE_PATH/include
SERPENT_EXE_PATH=.:$SERPENT_BASE_PATH/bin
IMAKEINCLUDE=-I$SERPENT_BASE_PATH/config
PATH=$PATH:$SERPENT_EXE_PATH
export SERPENT SERPENT_BASE_PATH SERPENT_DATA_PATH SERPENT_EXE_PATH PATH
export IMAKEINCLUDE

echo "export SERPENT=$SERPENT" > serpent.env.sh
echo "export SERPENT_BASE_PATH=$SERPENT" >> serpent.env.sh
echo "export SERPENT_DATA_PATH=$SERPENT_DATA_PATH" >> serpent.env.sh
echo "export SERPENT_EXE_PATH=$SERPENT_EXE_PATH" >> serpent.env.sh
echo "export PATH=$PATH" >> serpent.env.sh
echo "export IMAKEINCLUDE=-I$SERPENT_BASE_PATH/config" >> serpent.env.sh

# Serpent Version 1.2
#=============================================================================
# Declaration Block
#-----------------------------------------------------------------------------
#  DESCRIPTION: Provides all of the appropriate paths and file names for
#               install. Paths and file names include hidden files for internal
#               use by install and serpent, paths for likely places to find
#               X / Mortif / Athena libraries. It also includes a list of
#               sensitive commands which will be checked automatically before
#               installation can begin.
#=============================================================================

#-------------------------------------
#              FILES
#-------------------------------------

   CDEBUGDEFAULT="-O -DNDEBUG"
         VERSION="Serpent Version 1.2 sh"
   USR_FILE_NAME="usr.cf"
        USR_FILE="./config/usr.cf"
   REGISTER_FILE="/tmp/serpent.reg"
 HIDDEN_TECHFILE="./.tech.install"
 HIDDEN_DATAFILE="./.data.install"
    ADDRESS_FILE="./config/.serpent.addr.sh"
   CHECKP_SOURCE="./config/shcheckp.c"
   CHECKP_OBJECT="./config/shcheckp"
   CHECKP_RESULT="./config/.shcheckp.install"
     HEADER_FILE="./config/.header.install.sh"
  HIDDEN_INSTALL="./.ins.done"

#-------------------------------------
#               PATHS
#-------------------------------------

  INCDEFAULTLIST="/usr/include /usr/include/X11R4 /include /usr/local/X11R4/include /usr/local/include /usr/local/X11R4 /usr/X11R4 /usr/local"
  LIBDEFAULTLIST="/usr /usr/lib/X11R4 /usr/local/X11R4"
    MOTIFDEFLIST="/usr/Motif /usr/lib/X11R4/Motif /usr/local/Motif /usr/local/X11R4/Motif /usr/lib/X11R4/Motif /usr /usr/local"

#-------------------------------------
#              COMMANDS
#-------------------------------------

 COMMANDLIST="date grep ln ctags printenv"

 NONE="n"
 SOME="y"

#=============================================================================
# Previous Installation check
#-----------------------------------------------------------------------------
#  DESCRIPTION: Checks for the existance of a file called ./.ins.done which
#               contains a list of "checkpoints" through which install has
#               passed. If in fact the file exists, it is a strong indication
#               that the user has previously run install and may have just
#               tweaked his system to get it to compile right. If this is the
#               case install will give the option of skipping over various
#               questions which have already by answered.
#=============================================================================

RESTART="y"

[ -f "$HIDDEN_INSTALL" ] && {
   echo ""
   echo "Welcome To Serpent Installation."
   echo ""
   echo "  It appears as though Install has been previously run. In order to save"
   echo "time and effort, Install can try to pick up where you last left off. This"
   echo "means that your previous answers will be used to build serpent. You can also"
   echo "rebuild serpent from scratch and go through the entire installation process"
   echo "again."
   echo ""
   echo -n "  Would you like to Install serpent from scratch?"
   
   RESTART=" "
   while [ "$RESTART" != n ] && [ "$RESTART" != y ]; do
	echo -n " [y/n] :"
   	read ANSWER
 	if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
    	   RESTART="y"
	   echo ""
 	   echo "  Starting from scratch..."
	   rm $HIDDEN_INSTALL
	elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
           RESTART="n"
	   echo ""
	   echo -n "  Loading your previous answers..."
	   . $HIDDEN_INSTALL
           echo "done."
	else
	   echo -n "  Please select y)es or n)o"
	   
        fi
   done
} #HIDDEN_INSTALL

#=============================================================================
# Shell Verification
#-----------------------------------------------------------------------------
#  DESCRIPTION: If the user is starting from scratch, Install will check the
#		shell commands it uses to verify two things. 1) that they
#		exist, and 2) that they are not aliased to anything which
#		could potentially damage the system. If such an alias is found
#		the user is informed of the problem and if it is deemed
#		harmful, install quits.		       
#=============================================================================

if [ "$RESTART" = y ]; then

   echo ""
   echo ""
   echo "Welcome to Serpent Installation."
   echo ""
   echo "  Before the installation may begin, Install will verify that all the commands"
   echo "used during this process are present and will not damage your system."
   echo "(eg clear aliased to rm)."
   echo ""
   echo "  Please wait while your shell and system commands are verified. This process"
   echo "will take a few seconds."
   echo ""
   echo -n "Verifying Shell Commands..."

   the_status=0
   for COMMAND in $COMMANDLIST; do
    	echo -n "."
   	if [ `./config/shverify $COMMAND` != "ok" ]; then
     	   the_status="aliased"
           if [ `./config/shverify $COMMAND` = "absent" ]; then
              the_status="absent";
           fi
 	fi
    	if [ "$the_status" = absent ]; then
    	   echo ""
     	   echo ""
     	   echo "**** ERROR ****"
     	   echo "$COMMAND was not found as a valid command in your path. Please add it to"
	   echo "your system or install it in your path if you are going to run Serpent."
	   echo "Serpent will NOT execute without it."
           exit 1
    	fi
    	if [ "$the_status" = aliased ]; then
    	   echo ""
    	   echo ""
    	   echo "**** WARNING ****"
    	   echo "$COMMAND is aliased."
    	   echo -n "  Will this command execute without harm to your system?"
    	   okay="n"
    	   while [ "$okay" = n ]; do
 	   	echo -n " [y/n] :"
 	   	read ANSWER
 	    	if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
 	    	   ANSWER="y"
	    	   echo ""
	    	   echo -n "Resuming Check."
		   okay="y"
 	    	elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
 	  	   echo "  Please unalias this command...Serpent requires it to execute properly."
	  	   exit 1
	  	
	    	else
	   	   echo -n "  Please select y)es or n)o"
	   	fi
	   done # ANSWER
	fi # the_status
   done # for
   echo "done."
   echo " VERIFY=$SOME" > $HIDDEN_INSTALL
fi # RESTART


#=============================================================================
# Misc Setup Details
#-----------------------------------------------------------------------------
#  DESCRIPTION: This section contains a check for the clear command, a
#		check for a mailing address and an editor selection option.
#		These are all small details which must be addressed before
#		the actual installation begins.
#=============================================================================

#------------------------------
# Clear must ALWAYS be checked
#------------------------------

if [ `./config/shverify clear` != ok ]; then
  NOCLEAR="y";
  
  if [ `./config/shverify cls` != ok ]; then
    the_status="aliased"
       if [ `./config/shverify cls` = absent]; then
        the_status="absent"
     fi
  else
    the_status="ok"
  fi

  if [ "$the_status" = ok]; then
     unalias CLEAR
     CLEAR="cls"
    else
     	NOCLEAR="y"
  fi

else
    CLEAR="clear"
    NOCLEAR="n"
fi


#----------------------------------
# Get the mailing address and path
#----------------------------------

. $ADDRESS_FILE

echo ""
echo -n "                          Press Return to Continue"
read bogotroid

if [ "$NOCLEAR" = n ]; then
  ($CLEAR);
fi

NULL=""

#------------------
# Select an Editor
#------------------
if [ "$editor" != $NULL ]; then
    the_editor=$editor
else
    editor="emacs"
fi

if [ "$VISUAL" != $NULL ]; then
    the_editor=$VISUAL
fi
if [ "$EDITOR" != $NULL ]; then
    the_editor=$EDITOR
fi

if [ "$the_editor" = $NULL ]; then
    echo "You do not have a default editor . Please specify an editor which you"
    echo "can run."
    echo ""
    echo -n "What editor would you like to use? [$editor]: "
     read the_editor
fi

if [ "$the_editor" = $NULL ]; then
    the_editor="emacs"
echo " editor=$the_editor" >> $HIDDEN_INSTALL

#--------------------
# Begin Installation
#--------------------

if [ "$NOCLEAR" != y ]; then
  ($CLEAR)
fi
echo "================================================================================"
echo ""
echo "     ####### #                                                       ##"
echo "   ####  ####                                                       ###"
echo "   ####          #####   ### ####  ### ####     #####   ### ####   #######"
echo "    #########   ###  ###  ####  ##  ####  ###  ###  ###  ####  ###  ###"
echo "          ####  ########  ###       ###   ###  ########  ###   ###  ###"
echo "    ####  ####  ###       ###       ####  ###  ###       ###   ###  ###  ##"
echo "   # #######     ######  #####      ### ###     ######  ##### #####  #####"
echo "                                    ###"
echo "                                   #####       INSTALLATION"
echo ""
echo "================================================================================"

echo ""

echo "  Welcome to Serpent. For Serpent to run properly, it will be necessary for"
echo "you to add the serpent.env.sh file to your .profile or your .login file."
echo "  This script has automatically included these changes in the environment, so"
echo "that you will not have to quit and do this now. Please remember to do this"
echo "after you have finished the installation."
echo ""
echo "  There is also a file called serpent.env.csh in this directory, for those"
echo "running under csh. This should be added to your .cshrc file after completing"
echo "installation (just set the value of SERPENT)."
echo ""
echo -n "  Do you wish to proceed with the installation at this time?"
okay="n"
while [ "$okay" = n ]; do  
    echo -n " [y/n] :"
    read ANSWER
    if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
	 okay="y"
    elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
	echo ""
	echo "Installation Aborted"
        exit 1
    else
	echo -n "  Please select y)es or n)o"
	
    fi
done # okay

#------------------------------
# Ask for Mailing Instructions
#------------------------------
#--------------------------------------
# Make sure the user isn`t too uptight
#--------------------------------------

if [ "$MAIL_OK" != y ]; then
. $HEADER_FILE
echo "  The Serpent installation program saves information which is used by"
echo "several Serpent systems to communicate with us. This includes bug reports"
echo "and registration forms. This feature will help us provide support for your"
echo "particular architecture."
echo ""
echo "  We would greatly appreciate receiving this information from you, so we"
echo "tried to make the process as easy as possible. If you have email"
echo "capabilities, Serpent installation will try to make use of them unless you"
echo "specify otherwise."
echo ""
echo -n "  Would you like to use the automatic email feature?"
okay="n"
while [ "$okay" = n ]; do 
    echo -n " [y/n] :"
    read ANSWER
    if [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
	echo ""
	echo "  Would you like Install to produce files which you could mail to us"
echo -n "yourself?"
	okay2="n";
	while [ "$okay2" = n ]; do
	    echo -n " [y/n] :"
	    read ANSWER
	    if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
		BS="n"
		while [ "$BS" = n ]; do
		    echo ""
		    echo "  Please enter a directory to which to save these files:"
		    echo -n "  >"
		    read MAIL_PATH
		    if [ -s ${MAIL_PATH} ]; then
			 BS="y"
		    else
		    	echo ""
		    	echo "  [$MAIL_PATH] was not a valid path. Please try again."
		    fi
		done
		okay2="y"
		SNAILMAIL="y"
		NOEMAIL="y"
	    elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
		NOEMAIL="y"
		okay2="y"
	    else
		echo -n "  Please select y)es or n)o"
	    fi
	done #okay2
	okay="y"
    	
    elif [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then # You really have to love this person...
	 EMAIL="y"
	 NOEMAIL="n"
	 okay="y"
    else
	echo -n "Please select y)es or n)o"
    fi
done #okay
echo " MAIL_OK=$SOME" >> $HIDDEN_INSTALL
echo " NOEMAIL=$NOEMAIL" >> $HIDDEN_INSTALL
fi #MAIL-OK

#----------------------
# Get PATH for SENDMAIL
#----------------------

if [ "$NOEMAIL" != y ]; then

if [ "$SENDMAIL_OK" != y ]; then

. $HEADER_FILE

if [ -f $MAILPATH/sendmail ]; then echo
else
    echo "  Install is unable to locate sendmail. Can you send mail from your particular"
    echo -n "location?"
    fin="n"
    while [ "$fin" = n ]; do
        echo -n " [y/n] :"
        read ANSWER
        if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
	    found="n"
	    while [ "$found" = n ]; do
	  	echo ""
	  	echo "  Please type the full path to the directory containing sendmail."
	  	echo ""
	  	echo -n "  >"
	  	read MAILPATH
	  	if [ -f $MAILPATH/sendmail ]; then
	    	     found="y"
                else
 		     echo ""
                     echo "  sendmail was not found in [$MAILPATH]."
                fi
	     done
       	    fin="y" # found
        elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
            fin="y"
	    echo " MAILPATH=$NONE" > $ADDRESS_FILE
                echo "LIST_ADDR=$NONE" >> $ADDRESS_FILE
                echo "BUGS_ADDR=$NONE" >> $ADDRESS_FILE
		echo "ADDRESS=$NONE" >> $ADDRESS_FILE
	    . $ADDRESS_FILE            
        else
            echo -n "Please select y)es or n)o"
        fi
    done # fin
fi #MAILPATH stuff

#---------------------------
# Get Address
#---------------------------

nomail="n"
. $ADDRESS_FILE

if [ "$MAILPATH" != n ]; then
    echo ""
    echo "  In case of future problems, Serpent must have an address at which you can"
    echo "reach its development team."
    echo "  Two different addresses must be specified for use by serpent and"
    echo "install."
    echo ""
    if [ "$ADDRESS" = n ]; then
      echo "  No Addresses have been specified. This suggests that you cannot send us"
      echo -n "email. Is this correct?"
      okay="n"
      while [ "$okay" = n ]; do
        echo -n " [y/n] :"
        read ANSWER
        if [ "$ANSWER" = Y ] || [ "$ANSWER" = y ]; then
             okay="y"
             nomail="y"
            echo "MAILPATH=$MAILPATH" > $ADDRESS_FILE
            echo "LIST_ADDR=$NONE" >> $ADDRESS_FILE
            echo "BUGS_ADDR=$NONE" >> $ADDRESS_FILE
            echo "ADDRESS=$NONE" >> $ADDRESS_FILE
	elif [ "$ANSWER" = N ] || [ "$ANSWER" = n ]; then
	    okay="y"
	    nomail="n"
	    echo ""
	     LIST_ADDR="serpent-list-request@sei.cmu.edu"
	     BUGS_ADDR="serpent-bugs@sei.cmu.edu"
	     ADDRESS="y"
	else
	    echo -n "Please select y)es or n)o "
	fi
      done
    fi

if [ "$nomail" != y ]; then
      echo "  Can you reach serpent-list-request by sending mail to:"
      echo -n "  [$LIST_ADDR] ?"
    okay="n"
    while [ "$okay" = n ]; do
	echo -n " [y/n] :"
	read ANSWER
	if [ "$ANSWER" = Y ] || [ "$ANSWER" = y ]; then
	     okay="y"
                echo " MAILPATH=$MAILPATH" > $ADDRESS_FILE
                echo " LIST_ADDR=$LIST_ADDR" >> $ADDRESS_FILE
        elif [ "$ANSWER" = N ] || [ "$ANSWER" = n ]; then
            echo ""
	    echo "  Please enter the correct address below. Press Return if you cannot reach"
	    echo "us via email."
	    echo -n "  >"
            read LIST_ADDR
	    if [ "$LIST_ADDR" = $NULL ]; then
	   	echo " MAILPATH=$MAILPATH" > $ADDRESS_FILE
		echo " LIST_ADDR=$NONE" >> $ADDRESS_FILE
		echo " BUGS_ADDR=$NONE" >> $ADDRESS_FILE
		echo " ADDRESS=$NONE" >> $ADDRESS_FILE
                nomail="y"
	    else
	   	echo " MAILPATH=$MAILPATH" > $ADDRESS_FILE
		echo " LIST_ADDR=$LIST_ADDR" >> $ADDRESS_FILE
	    fi # ADDRESS
            okay="y"
	else
	    echo -n "  Please select y)es, n)o"
	    
   	fi
    done #okay
   
  if [ "$nomail" != y ]; then
    okay="n"
    echo ""
    echo "  Can you reach serpent-bugs by sending mail to:"
    echo -n "  [$BUGS_ADDR] ?"
    while [ "$okay" = "n" ]; do
        echo -n " [y/n] :"
        read ANSWER
        if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
            okay="y"
            echo " BUGS_ADDR=$BUGS_ADDR" >> $ADDRESS_FILE
            echo " ADDRESS=$SOME" >> $ADDRESS_FILE
        elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
            echo ""
            echo "  Please enter the correct address below."
            echo -n "  >"
            read BUGS_ADDR
            echo " BUGS_ADDR=$BUGS_ADDR" >> $ADDRESS_FILE
            echo " ADDRESS=$SOME" >> $ADDRESS_FILE
            okay="y"
        else
            echo -n "  Please select y)es, n)o"
        fi
    done #okay
    . $ADDRESS_FILE
  fi #nomail - yes there are two here.
  fi #nomail
fi #MAILPATH
echo "SENDMAIL_OK=$SOME" >> $HIDDEN_INSTALL
fi
fi #NOEMAIL


#------------------------
# Registration Procedure
#------------------------

#----------------------
# REGISTER USER
#----------------------
if [ "$REGISTERED" = y ]; then doit="n"
elif [ "$SNAILMAIL" = y ]; then doit="y"
elif [ "$LIST_ADDR" != n ] && [ "$NOEMAIL" = n ]; then doit="y"
fi

if [ "$doit" = y ]; then

    if [ "$skip" != y ]; then

	. $HEADER_FILE
	echo "  As Serpent is still experimental, it is necessary for the Serpent"
	echo "development team to keep track of who has a copy of the system. This will"
	echo "allow us to send you further information about Serpent and keep you informed"
	echo "about future releases."
	echo ""
	echo -n "  Would you like to register your copy of serpent?"

	okay="n"
	while [ "$okay" = n ]; do
            echo -n " [y/n] :"
            read ANSWER
            if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
            	register="y"
             	okay="y"
            elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
             	register="n"
             	okay="y"             	
	    else
	    	echo -n "Please select Y)es or N)o"	     	
            fi
        done

	if [ "$register" = y ]; then

	    #---------------
	    # REGISTER USER
	    #---------------

	    if [ "$SNAILMAIL" = y ]; then
		 REGISTER_FILE="${MAIL_PATH}/Register.SERPENT"
	    fi
	    echo ""
	    echo "  A registration template will now be loaded for you to edit. Please complete"
	    echo "the form, save it and exit the editor."
	    if [ "$SNAILMAIL" = y ]; then
		echo "" > $REGISTER_FILE
		echo "Affiliate relations - Serpent" >> $REGISTER_FILE
		echo "Software Engineering Institute" >> $REGISTER_FILE
		echo "Carnegie Mellon University" >> $REGISTER_FILE
		echo "Pittsburgh, PA 15213-3890" >> $REGISTER_FILE
		echo "" >> $REGISTER_FILE
		echo "==============================================================================" >> $REGISTER_FILE
	    else
		echo "" > $REGISTER_FILE
		echo "==============================================================================" >> $REGISTER_FILE
	    fi
	    echo "                        Serpent Registration Template" >> $REGISTER_FILE
	    echo "==============================================================================" >> $REGISTER_FILE
	    echo "Name		:" >> $REGISTER_FILE
	    echo "Organization	:" >> $REGISTER_FILE
	    echo "Address		:" >> $REGISTER_FILE
	    echo "		:" >> $REGISTER_FILE
	    echo "Phone		:" >> $REGISTER_FILE
	    echo "Email 		:" >> $REGISTER_FILE
	    echo "------------------------------------------------------------------------------" >> $REGISTER_FILE
	    echo "How did you acquire Serpent?" >> $REGISTER_FILE
	    echo "" >> $REGISTER_FILE
	    echo "How did you find out about Serpent?" >> $REGISTER_FILE
	    echo "" >> $REGISTER_FILE
	    echo "What hardware/software platform are you running Serpent on?" >> $REGISTER_FILE
	    echo "" >> $REGISTER_FILE
	    echo "Do you wish to be added to the Serpent technical mailing list?" >> $REGISTER_FILE
	    echo "------------------------------------------------------------------------------" >> $REGISTER_FILE
	    echo "Comments:" >> $REGISTER_FILE
	    echo "" >> $REGISTER_FILE
	    echo "" >> $REGISTER_FILE
	    echo "==============================================================================" >> $REGISTER_FILE
	    echo "			     $VERSION" >> $REGISTER_FILE

	    $editor $REGISTER_FILE
	    if [ "$SNAILMAIL" = y ]; then
	        echo -n "would you like to S)ave the registration, E)dit it again or A)bort?"
	    else
	        echo -n "would you like to S)end the registration, E)dit it again or A)bort?"
	    fi
	    okay="n"
	    while [ "$okay" = n ]; do
		echo -n " [s/e/a] :"
		read ANSWER
		if [ "$ANSWER" = s ] || [ "$ANSWER" = S ]; then
		    if [ "$SNAILMAIL" = y ]; then
			echo "  $REGISTER_FILE saved, please send it as soon as possible."
			echo ""
			echo -n "                         Press Return to Continue"
		    else
	  	    	. $HEADER_FILE
	  	    	echo "           You are being registered as a serpent user...please wait."
          	    	(\
	  	    	echo "To: $LIST_ADDR" ;\
	  	    	echo "From: $USER" ;\
	  	    	echo "Subject: Serpent 1.2 Registration" ;\
	  	    	echo "" ;\
	  	    	cat $REGISTER_FILE ;\
	  	    	) | /usr/lib/sendmail -t -om
                        rm $REGISTER_FILE*
		    fi
	  	     okay="y"
		    
		elif [ "$ANSWER" = e ] || [ "$ANSWER" = e ]; then
	  	    $editor $REGISTER_FILE
	  	    
		elif [ "$ANSWER" = a ] || [ "$ANSWER" = A ]; then
	  	    exit 1
	  	
		else
		    if [ "$SNAILMAIL" = "y" ]; then
			echo -n "  Please select s)ave, e)dit or a)bort"
		    else
	  	        echo -n "  Please select s)end, e)dit or a)bort"
		    fi
		fi
	    done

	fi # register

    fi # skip

fi # ADDRESS

echo " REGISTERED=$SOME" >> $HIDDEN_INSTALL


#------------------------
# Configuration Procedure
#------------------------


#---------------
# CONFIGURATION
#---------------

if [ "$CF_OK" != y ]; then

. $HEADER_FILE
   echo "                   Please wait...Checking your system type..."

cc $CHECKP_SOURCE -o $CHECKP_OBJECT

$CHECKP_OBJECT > $CHECKP_RESULT
. $CHECKP_RESULT

. $HEADER_FILE
echo "  Serpent must refer to a machine specific configuration file. Such files"
echo "are included with Serpent. It is suggested that the configuration file that"
echo "fits your machine be checked to ensure that it matches your particular"
echo "configuration."
echo ""
echo "  Your machine has been identified as a $MACHINETYPE."
echo "  The $CFFILE file has been selected as your configuration file."
if [ "$MACHINETYPE" = sun ]; then
  echo ""
  echo "  *** NOTE *** "
  echo "    Sun users should edit this file to verify which version of SunOS"
  echo "    is being used by their system."
fi
echo ""
echo "  Would you care to view/edit this file before continuing with the installation"
echo -n "process?"
okay="n"
while [ "$okay" = n ]; do
    echo -n " [y/n] :"
    read ANSWER
    if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
	echo ""
	echo "Preparing $CFFILE for editing."
	echo "When you are finished, please save the file and exit the editor."
	$editor config/$CFFILE
	okay2="n"
	while [ "$okay2" = "n" ]; do
	    . $HEADER_FILE
	    echo "  The configuration file $CFFILE has been edited."
     	    echo ""
            echo -n "Do you wish to C)ontinue, E)dit the file again, or A)bort?"
     	    echo -n " [c/e/a] :"
     	    read ANSWER
     		if [ "$ANSWER" = A ] || [ "$ANSWER" = a ]; then
		   exit 1
		   
     		elif [ "$ANSWER" = E ] || [ "$ANSWER" = e ]; then
        	   $editor ./config/$CFFILE
		   
     		elif [ "$ANSWER" = C ] || [ "$ANSWER" = c ]; then
		    okay2="y"
        	   
     	    fi
   	done # while
   	okay="y"

    elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
   	 okay="y"
      	
    fi
done # okay


echo " CF_OK=$SOME" >> $HIDDEN_INSTALL
fi #CF_OK

#------------------------
# Check Site.def
#------------------------

SKIPUSRDEFS="n"

if [ -f "$HIDDEN_TECHFILE" ] && [ -f "$USR_FILE" ]; then
   . $HEADER_FILE
   echo "  Serpent makes use of a file called $USR_FILE_NAME to store variables which"
   echo "are used to determin your particular configuration. These variables include"
   echo "information about such things as where X, Motif and Athena are stored,"
   echo "compilation parameters and compiler types."
   echo ""
   echo "  Install has detected that $USR_FILE_NAME has previously been installed."
   echo ""
   echo "   Would you like to E)dit $USR_FILE_NAME, R)ebuild $USR_FILE_NAME from scratch or"
   echo -n "  C)ontinue installation?"
   okay="n"
   while [ "$okay" = n ]; do
     echo -n " [e/r/c] :"
     read ANSWER
	if [ "$ANSWER" = C ] || [ "$ANSWER" = c ]; then
	   SKIPUSRDEFS="y"
	   echo ""
	   echo -n "  Have you changed $USR_FILE_NAME since the last installation?"
	   cool="n"
	   while [ "$cool" = n ]; do
		echo -n " [y/n] :"
		read ANSWER
		if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
		   EDITUSER="y"
		   [ -f "$HIDDEN_TECHFILE" ] && {
			rm $HIDDEN_TECHFILE
		   }
		   cool="y"
		   
		elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
		   EDITUSER="n"
		   cool="y"
		   
		else
		   echo "Please select y)es or n)o"
		   
		fi
	   done #cool
	   okay="y"
	   
	elif [ "$ANSWER" = E ] || [ "$ANSWER" = e ]; then
	   SKIPUSRDEFS="y"
	   echo ""
	   EditDone="n"
	   while [ "$EditDone" = n ]; do
	     $editor $USR_FILE
	     echo "Would you like to E)dit $USR_FILE_NAME again, C)ontinue with the installation,"
	     echo -n "R)ebuild $USR_FILE_NAME from scratch or A)bort?"
	     ECA="n"
	     while [ "$ECA" = n ]; do
		echo -n " [e/c/r/a] :"
		read ANSWER
		  if [ "$ANSWER" = E ] || [ "$ANSWER" = e ]; then
		     ECA="y"
		    
		  elif [ "$ANSWER" = C ] || [ "$ANSWER" = c ]; then
		     ECA="y"
		     EditDone="y"
		     EDITUSER="y"
		    [ -f "$HIDDEN_TECHFILE" ] && {
			rm $HIDDEN_TECHFILE
		    }
		    		  
		  elif [ "$ANSWER" = A ] || [ "$ANSWER" = a ]; then
		    echo ""
		    echo "  Serpent Installation Aborted."
		    exit 1
		    
		  elif [ "$ANSWER" = R ] || [ "$ANSWER" = r ]; then
		    SKIPUSRDEFS="n"
		    EditDone="y"
		    okay="y"
		    ECA="y"
		   
		  else
		    echo -n "  Please select E)dit, C)ontinue, R)ebuild or A)bort"
		    
		fi
	     done #while
	   done #while
	elif [ "$ANSWER" = R ] || [ "$ANSWER" = r ]; then
	   SKIPUSRDEFS="n"
	   okay="y"
	   
	else
	   echo -n "  Please select E)dit, R)ebuild or C)ontinue"
	   
	fi
     done #while
fi #Hidden techfile & stuff

if [ "$SKIPUSRDEFS" = n ]; then
 
    HASATHENA="n"
    HASMOTIF="n"
    NOTECH="n"

    while [ "$HASATHENA" = n ] && [ "$HASMOTIF" = n ] && [ "$NOTECH" = n ]; do

	#----------------------------
	# Get X11 Inc & Athena Dirs
	#----------------------------


#----------------------------------------------------------------
# DIRECORY PATHS EDIT ON A *VERY* LARGE SCREEN (over 120 cols.)
#----------------------------------------------------------------

	if [ "$HAS_R4" != y ] || [ "$SKIPUSRDEFS" != y ]; then

. $HEADER_FILE

echo "  The X Window System, Version 11 Release 4 is required for Serpent to"
echo "build and execute. X11R4 files must be present for Serpent to use."
echo ""
echo -n "  Do you have X11R4?"
yeah="n"
while [ "$yeah" = n ]; do
    echo -n " [y/n] :"
    read ANSWER
    if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
	HASR4="y"
	echo " HAS_R4=$SOME" >> $HIDDEN_INSTALL
	 Version="X11R4"
	 yeah="y"
	
    elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
	 yeah="y"
	 HASR4="n"
    else
	echo -n "  Please select y)es or n)o"
	
    fi
done #yeah

if [ "$HASR4" = n ]; then
echo ""
#-----------------------------------
# Does not have X11R4...
#   who is this person?
#-----------------------------------

    . $HEADER_FILE
    echo "  Serpent can currently only be run under X11R4."
    echo "  After installing X11R4, please re-install Serpent."
    echo ""
    echo "  Serpent has Aborted."
    if [ -f $USR_FILE ]; then 
	rm $USR_FILE
    fi
    if [ -f $HIDDEN_TECHFILE ]; then
	rm $HIDDEN_TECHFILE
    fi

    exit 1
fi
fi

#-----------------------------------
# Has R3 or R4... Check all the fun
# places it might be hiding...
#-----------------------------------

if [ "$R4_INC" != y ] || [ "$SKIPUSRDEFS" != y ]; then
echo ""
. $HEADER_FILE
echo "  Since X11 is not distributed with serpent, it is necessary to verify the"
echo "location of the $Version Include and Library Directories. Note that the"
echo "X11 Include files should be stored in a directory named /X11."
echo ""
echo "               Verifying /X11 Include Directory...Please Wait..."

echo "/*-------------------------------------------------------------" > $USR_FILE
echo "* User specified macro definitions for use by Imake." >> $USR_FILE
echo "* These definitions were  at installation time and may be  " >> $USR_FILE
echo "* changed whenever the User wishes to do so." >> $USR_FILE
echo "*-------------------------------------------------------------*/" >> $USR_FILE
echo "" >> $USR_FILE

FOUNDPATH="n"
for INCPATH in $INCDEFAULTLIST; do
    if [ -f $INCPATH/X11/X.h ]; then
	 FOUNDPATH=$INCPATH;
	 legaldir="y"
	 break
    fi
done # foreach

if [ "$FOUNDPATH" != n ]; then
   echo ""
   echo "  The /X11 Include directory was found to be:"
   echo "  [$INCPATH/X11]"
   echo ""
   echo -n "  Is this correct?"
   okay="n"
   while [ "$okay" = n ]; do
	echo -n " [y/n] :"
	read ANSWER
	if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
       	    echo "SERPENTXINCDIR=$FOUNDPATH" >> $USR_FILE
	     X11INCLUDE=$FOUNDPATH
            okay="y"
            
     	elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
            FOUNDPATH="n"
            okay="y"
            
     	else
	    echo -n "  Please Select y)es or n)o."
	    
     	fi
   done # while
fi # FOUNDPATH

#--------------------------
# Sheesh...I can`t find it
#--------------------------

if [ "$FOUNDPATH" = n ]; then

    . $HEADER_FILE
    echo "       **** INSTALL WAS UNABLE TO LOCATE YOUR X11 INCLUDE DIRECTORY ****"
    echo ""
    echo "  Remember that all X11 files must be found in a /X11 directory. This structure"
    echo "is required for Serpent to build and execute under X11."
    echo ""
    peachy="n"
    while [ "$peachy" = n ]; do
	echo "  Please type the full path to your X11 Include directory:"
        echo "  (The /X11 extension will be added automatically)"
	echo -n "  >"
	 read FOUNDPATH
	if [ -f $FOUNDPATH/X11/X.h ]; then
	    echo ""
	    echo "  The X11 Include path was verfied to be [$FOUNDPATH/X11]"
	    echo ""
	    echo -n "                          Press Return to Continue"
	    read  bogus
	    echo "SERPENTXINCDIR = $FOUNDPATH" >> $USR_FILE
	    X11INCLUDE=$FOUNDPATH
	    peachy="y"
	else
	    echo ""
	    echo "  Verification of [$FOUNDPATH/X11] failed. Please try again."
	    echo ""
	fi
    done #peachy

fi # FOUNDPPATH


echo "R4_INC=$SOME" >> $HIDDEN_INSTALL
fi
	#----------------------------
	# Get X11 Library Directory
	#----------------------------

#-----------------------
# X LIBRARY DIRECTORY
#-----------------------

if [ "$R4_LIB" != y ] || [ "$SKIPUSRDEFS" != y ]; then
echo ""
. $HEADER_FILE

echo "                Verifying X11 Library Directory...Please Wait..."

FOUNDPATH=$FOUNDPATH:h

FOUND="n"
if [ -f "$FOUNDPATH/lib/libX11.a" ]; then
     FOUND="y"
else
    FOUNDPATH="y"
    for LIBPATH in $LIBDEFAULTLIST; do
	if [ -f "$LIBPATH/lib/libX11.a" ]; then
	     FOUNDPATH=$LIBPATH
	     FOUND="y"
	     break
	fi
    done #foreach
fi #FOUNDPATH STUFF

if [ "$FOUND" = y ]; then
   echo ""
   echo "  The X11 library files were found to reside in:"
   echo "  [$FOUNDPATH/lib]"
   echo ""
   echo -n "  Is this correct?"
   okay="n"
   while [ "$okay" = n ]; do
	echo -n " [y/n] :"
    	read ANSWER
    	if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
            echo "SERPENTXLIBDIR = $FOUNDPATH/lib" >> $USR_FILE
            okay="y"
            
    	elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
            FOUND="n"
            okay="y"
            
    	else
            echo -n "  Please Select y)es or n)o."
            
   	fi
   done # while
fi

if [ "$FOUND" = n ]; then
   . $HEADER_FILE
   echo ""
   echo "      **** INSTALL WAS UNABLE TO LOCATE YOUR X11R4 LIBRARY DIRECTORY ****"
   while [ "$FOUND" = n ]; do
   	echo ""
   	echo "  Please type in the full path for your X11 library directory:"
   	echo ""
   	echo -n "  >"
   	read LIBCHANGE
   	if [ -f "$LIBCHANGE/libX11.a" ]; then
     	    echo ""
     	    echo "  The X11 library directory has been confirmed to be:"
     	    echo "  [$LIBCHANGE]"
     	    echo ""
     	    echo -n "                            Press Return To Continue"
     	    read Bogus
     	    echo "SERPENTXLIBDIR = $LIBCHANGE" >> $USR_FILE
     	    FOUND="y"
   	else
     	    echo ""
     	    echo "  Verification of [$LIBCHANGE] failed. Please try again."
   	fi
   done
fi
echo " R4_LIB=$SOME" >> $HIDDEN_INSTALL
fi
	#----------------------------
	# Get Athena
	#----------------------------

#--------
# ATHENA
#--------

if [ "$ATHENA_INC" = y ] && [ "$SKIPUSRDEFS" != y ]; then
 HASATHENA="y"
else

RETRYLOOP="n"

echo "" >> $USR_FILE

RETRYLOOP="n"
while [ "$RETRYLOOP" = n ]; do

HASATHENA4="n"
NOATHENA="n"
NOINTRINSICS="n"
HASATHENA="n"

. $HEADER_FILE
    echo "                    Verifying Athena Files...Please Wait..."
    if [ -f $X11INCLUDE/X11/Intrinsic.h ]; then
    	if [ -s $X11INCLUDE/X11/Xaw ] && [ -f $X11INCLUDE/X11/Xaw/Cardinals.h ]; then
	    if [ "$HASR4" = y ]; then
	    	 HASATHENA4="y"
		 RETRYLOOP="y"
	    fi
	else
	     NOATHENA="y"
	     RETRYLOOP="y"
        fi
    else
   	 NOINTRINSICS="y"
    fi

    if [ "$NOATHENA" = y ]; then
	echo ""
	echo "  Athena was not found in the /X11 directory. It will not be installed."
	echo ""
	echo -n "                       Press Return to Continue"
	read bogus
	echo "#undef HasAthena" >> $USR_FILE
	HASATHENA="n"
    fi #NoAthena
    
    if [ "$NOINTRINSICS" = y ]; then
    	echo ""
    	echo "  The files necessary to implement Athena were not found in the /X11"
    	echo "directory. If you want to install Athena, please make the necessary"
    	echo "changes."
    	echo ""
    	echo -n "                       Press Return to Continue"
    	read bogus
    	echo "#undef HasAthena" >> $USR_FILE
    	HASATHENA="n"
	RETRYLOOP="y"
    fi

done #RETRYLOOP

if [ "$HASATHENA4" = y ]; then
    echo ""
    echo -n "  Athena has been detected. Would you like to install it?"
    peachy="n"
    while [ "$peachy" = n ]; do
	echo -n " [y/n] :"
	read ANSWER
	   if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
		echo "#define HasAthena" >> $USR_FILE
		 DONE="y"
		 HASATHENA="y"
		 peachy="y"
		
	   elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
		echo "#undef HasAthena" >> $USR_FILE
		DONE="y"
		HASATHENA="n"
		peachy="y"
		
	    else
		echo -n "  Please select y)es or n)o"
	fi
    done #peachy
fi # HASATHENA4

echo " ATHENA_INC=$SOME" >> $HIDDEN_INSTALL
fi

	#----------------------------
	# Get Motif Direcories
	#----------------------------

#------------------
# MOTIF INCLUDES
#------------------

if [ "$MOTIF_INC" != y ] || [ "$SKIPUSRDEFS" != y ]; then

. $HEADER_FILE
echo "               Verifying Motif Include Directory...Please Wait..."
FOUND="n"
FOUNDPATH="n"
SPATH="n"
for SPATH in $MOTIFDEFLIST; do
    if [ -s $SPATH/include ]; then
	 FOUND="y"
	 FOUNDPATH=$SPATH
	 DISCLAIMER="y"
	 break
    fi
done

if [ "$FOUND" = n ]; then
    echo ""
    echo "  Install was unable to detect the presence of Motif."
    echo -n "  Do you have Motif?"
    okay="n"
    while [ "$okay" = n ]; do
	echo -n " [y/n] :"
	read ANSWER
	if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
	     DISCLAIMER="y"
	     okay="y"
	    
	elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
	     SKIPITALL="y"
	     HASMOTIF="n"
	     okay="y"
	    
	else
	    echo -n "  Please select y)es or n)o"
	    
	fi
    done #okay
fi

if [ "$DISCLAIMER" = y ]; then
    echo ""
    echo "  Serpent currently runs using Motif 1.1.1 under X11R4. Although"
    echo "other types of Motif may work, we cannot guarantee that they will,"
    echo "nor do we suggest you use them."
    echo ""
    echo -n "  Would you like to install Motif?"
    okay="n"
    while [ "$okay" = n ]; do
	echo -n " [y/n] :"
	read ANSWER
	if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
	    okay="y"
	    SKIPITALL="n"
	    
	elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
	     SKIPITALL="y"
	     okay="y"
	    
	else
	    echo -n "  Please select y)es or n)o"
	    
	fi
    done # okay
fi # DISCLAIMER

if [ "$SKIPITALL" = n ]; then #------------------------------------------------------------------

    if [ "$FOUND" = y ]; then
    	. $HEADER_FILE
    	echo ""
    	echo "  The Motif Include files were found to reside in:"
    	echo ""
    	echo "  [$FOUNDPATH/include]"
    	echo ""
    	echo -n "  Is this correct?"
    	YN="n"
    	while [ "$YN" = n ]; do
  	    echo -n " [y/n] :"
	    read ANSWER
	    if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
	    	echo ""
	    	echo "#define HasMotif" >> $USR_FILE
		echo "" >>$USR_FILE
	    	echo "SERPENTMOTIFINCDIR = $FOUNDPATH/include" >> $USR_FILE
	    	 YN="y"
	    	 HASMOTIF="y"
	    	
	    elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
	    	 YN="y"
	    	FOUND="n"
	    	
	    else
	    	echo -n "  Please select y)es or n)o"
	    	
	    fi
    	done # YN
    fi # FOUND

    if [ "$FOUND" = n ]; then
    	. $HEADER_FILE
    	echo "       *** INSTALL WAS UNABLE TO FIND YOUR MOTIF INCLUDE DIRECTORY ***"
    	echo "" >> $USR_FILE
    	echo "#define HasMotif" >> $USR_FILE
    	echo ""
    	while [ "$FOUND" = n ]; do
    	    echo "  Please type in the complete path for your Motif Include directory:"
    	    echo ""
    	    echo -n "  >"
    	    read FOUNDPATH
    	    if [ -f $FOUNDPATH ]; then
      	    	echo ""
      	    	echo "  The Motif Include directory has been verified."
      	    	 FOUND="y"
		echo "#define HasMotif" >> $USR_FILE
      	    	echo "SERPENTMOTIFINCDIR = $FOUNDPATH" >> $USR_FILE
      	    	echo ""
      	    	echo -n "  Press Return to Continue"
      	    	read bogus
      	    	 FOUNDPATH=$FOUNDPATH:h
	    	 HASMOTIF="y"
      	    else
      	    	echo ""
      	    	echo "  The Motif include directory was not found. Please try again."
      	    	echo ""
    	    fi
    	done #FOUND
    fi #FOUND
echo " MOTIF_INC=$SOME" >> $HIDDEN_INSTALL

    #------------------
    # MOTIF LIBRARIES
    #------------------
if [ "$MOTIF_LIB" = y ] && [ "$SKIPUSRDEFS" != y ]; then
 HASMOTIF="y"
else

    . $HEADER_FILE
    echo "              Verifying Motif Library Directory...Please Wait..."

    FOUND="n"
    if [ -s $FOUNDPATH/lib ]; then                    # TEST LAST PATH /LIB
   	 FOUND="y"
    fi

    if [ "$FOUND" = n ]; then			       # TEST LIST O PATHS
	FOUNDPATH="n"
	SPATH="n"
	for SPATH in $MOTIFDEFLIST; do
	    if [ -s $SPATH/lib ]; then
     		 FOUND="y"
     		 FOUNDPATH=$SPATH
		 break
       	    fi
	done
    fi

    SKIPHAS="n"

    if [ "$FOUND" = y ]; then
    	echo ""
    	echo "  The Motif Library files were found to reside in:"
    	echo ""
	echo "  [$FOUNDPATH/lib]"
  	echo ""
  	echo -n "  Is this correct?"
  	YN="n"
  	while [ "$YN" = n ]; do
  	    echo -n " [y/n] :"
	    read ANSWER
	    if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
	  	echo ""
	  	echo "SERPENTMOTIFLIBDIR = $FOUNDPATH/lib" >> $USR_FILE
	  	 YN="y"
	  	 SKIPHAS="y"
		 FOUND="y"
	  	
	    elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
	  	 YN="y"
	  	 FOUND="n"
	  	
	    else
	  	echo -n "  Please select Y)es or N)o"
	  	
	    fi
	done # YN
    fi # FOUND

    if [ "$FOUND" = n ]; then
   	. $HEADER_FILE
  	echo "       *** INSTALL WAS UNABLE TO FIND YOUR MOTIF LIBRARY DIRECTORY ***"
  	echo ""
 	while [ "$FOUND" = n ]; do
    	    echo "  Please type in the complete path for your Motif Library directory:"
    	    echo ""
    	    echo -n "  >"
    	    read FOUNDPATH
    	    if [ -f $FOUNDPATH ]; then
      		echo ""
      		echo "  The Motif Library directory has been verified."
      		FOUND="y"
      		echo "SERPENTMOTIFLIBDIR = $FOUNDPATH" >> $USR_FILE
      		echo ""
      		echo -n "                             Press Return to Continue"
      		read  bogus
   	    else
      		echo ""
      		echo "  The Motif Library directory was not found. Please try again."
      		echo ""
    	    fi # FOUNDPATH
 	done # FOUND
    fi # FOUND
echo " MOTIF_LIB=$SOME" >> $HIDDEN_INSTALL
fi

fi # SKIPITALL
else
 HASMOTIF="y"
fi # MOTIF_INC

if [ "$HASMOTIF" = n ] && [ "$HASATHENA" = n ]; then
    . $HEADER_FILE
    echo "  You have opted to install Serpent without Athena and Motif. If you continue"
    echo "installation, you will be unable to use any of our demos."
    echo ""
    echo "  Do you wish to C)ontinue installation, or return to I)nstall"
    echo -n "Athena and/or Motif?"
    okay="n"
    while [ "$okay" = n ]; do
	echo -n " [c/i] :"
	read ANSWER
	if [ "$ANSWER" = C ] || [ "$ANSWER" = c ]; then
	     NOTECH="y"
	     okay="y"
	    
	elif [ "$ANSWER" = I ] || [ "$ANSWER" = i ]; then
	     okay="y"
	    
	else
	    echo -n "  Please select c)ontinue or i)nstall"
            
	fi
    done #okay
fi # HASMOTIF HASATHENA --- TECH CHECK

#--------------------------
#  UP RECORD OF INSTALL
#--------------------------

if [ -f "$HIDDEN_TECHFILE" ]; then
    rm $HIDDEN_TECHFILE
fi

if [ "$?NOTECH" = y ]; then
   echo " NOTECH=$SOME" >> $HIDDEN_TECHFILE
fi
if [ "$HASMOTIF" = y ]; then
   echo " HASMOTIF=$SOME" >> $HIDDEN_TECHFILE
else
   echo " HASMOTIF=$NONE" >> $HIDDEN_TECHFILE
fi
if [ "$HASATHENA" = y ]; then
   echo " HASATHENA=$SOME" >> $HIDDEN_TECHFILE
else
   echo "HASATHENA=$NONE" >> $HIDDEN_TECHFILE
fi
EDITUSER="n"

    done # HASATHENA HASMOTIF NOTECH

    #---------------------------
    # Get DEBUG Flags
    #----------------------------



#---------------
# Debug Flags
#---------------
if [ "$DEBUG_" != y ] || [ "$SKIPUSRDEFS" != y ]; then

. $HEADER_FILE

echo "  It is possible to compile Serpent in Debug mode. Turning on the Serpent"
echo "debug mode will increase the size of objects, libraries and load modules by as"
echo "much as 90%. It will however provide you with an accurate trace of Serpent's"
echo "operations."
echo ""
echo -n "  Would you like to turn on Debug?"
okay="n"
TOTAL="n"
while [ "$okay" = n ]; do
    echo -n " [y/n] :"
    read ANSWER
    if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
	 okay="y"
	
    elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
	 TOTAL="-DNDEBUG"
	 okay="y"
	
    else
	echo -n "  Please select y)es or n)o"
	
    fi
done
echo " DEBUG_=$SOME" >> $HIDDEN_INSTALL
fi

if [ "$OPT_" != y ] || [ "$SKIPUSRDEFS" != y ]; then

echo -n "  Would you like to Optimize the Serpent code?"
okay="n"
while [ "$okay" = n ]; do
    echo -n " [y/n] :"
    read ANSWER
    if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
	if [ "$TOTAL" = n ]; then
	     TOTAL="-O"
	else
	     TOTAL="$TOTAL -O"
	fi
	okay="y"
	
    elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
	okay="y"
	
    else
	echo -n "  Please select y)es or n)o"
	
    fi
done # done
echo " OPT_=$SOME" >> $HIDDEN_INSTALL
fi

if [ "$G_" != y ] || [ "$SKIPUSRDEFS" != y ]; then

echo "  Would you like to compile Serpent with additional symbol table information"
echo -n "for use with dbx? (-g option)"
okay="n"
while [ "$okay" = n ]; do
    echo -n " [y/n] :"
    read ANSWER
    if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
	if [ "$TOTAL" = n ]; then
	     TOTAL="-g"
	else
             TOTAL="$TOTAL -g"
	fi
        okay="y"
        
    elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
        okay="y"
        
    else
        echo -n "  Please select y)es or n)o"
        
    fi
done
echo " G_=$SOME" >> $HIDDEN_INSTALL
fi

if [ "$TOTAL" != n ]; then
    echo "" >> $USR_FILE
    echo "CDEBUGFLAGS = $TOTAL" >> $USR_FILE
fi

    #----------------------------
    # HASSABERC?
    #----------------------------
  
#-------------------------
# SABER C?
#-------------------------

if [ "$SABER_" != y ] || [ "$SKIPUSRDEFS" != y ]; then

. $HEADER_FILE
echo -n "  Do you have SaberC?"
okay="n"
while [ "$okay" = n ]; do
        echo -n " [y/n] :"
        read ANSWER
	if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
	  echo "" >> $USR_FILE
	  echo "#undef HasSaberC" >> $USR_FILE
	  echo "#define HasSaberC YES" >> $USR_FILE
	  okay="y"
	  
	elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
	  echo "#undef HasSaberC" >> $USR_FILE
	  okay="y"
	  
	else
	  echo -n "Please select y)es or n)o"
	  
	fi
done
echo " SABER_=$SOME" >> $HIDDEN_INSTALL
fi

fi #SKIPUSRDEFS

#--------------------------
# Install Imake
#--------------------------

#--------------------
# IMAKE INSTALLATION
#--------------------

. $HEADER_FILE
echo "                    Now installing Imake files...please wait."
cd ./config
echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
make
cd ..
    echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
echo "                              Imake installed."
echo ""
echo -n "                          Press Return to Continue."
read  bogus 

#------------
# MAKEFILES
#------------

. $HEADER_FILE
echo "                     Please Wait....Now Building Makefiles."
echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
imake
make Makefiles

echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
echo "                               Makefiles Complete"
echo ""
echo -n "                           Press Return to Continue"
read bogus

#------------------------------
# BUILD SERPENT
#------------------------------



#-------------------------------
# BUILD SERPENT
#-------------------------------

 BUILDTIME="3"
 BUILDEXEC="1 1/2"
 BUILDDEP="1 1/2"
 READY="n"


while [ "$READY" = n ]; do
    . $HEADER_FILE

    if [ -f $HIDDEN_TECHFILE ]; then
        echo ""
    else

    if [ "$EDITUSER" = y ]; then
	echo "                             **** WARNING ****"
	echo "  You have made changes to usr.cf which Install has been unable to keep"
	echo "records of. It is impossible for install to tell if you have installed Athena"
	echo "or Motif. While this will not effect the compilation of Serpent, please"
	echo "remember that if you have not installed either Athena or Motif, the demos"
	echo "supplied with Serpent will not work. If this is the case, please only build"
	echo "the Serpent executables."
	echo ""
	echo -n "                         Press Return to Continue"
	read bogus
	. $HEADER_FILE
    fi
    fi

    if [ "$NOTECH" = n ] && [ "$HASATHENA" = n ] && [ "$HASMOTIF" = n ]; then
	[ -f $HIDDEN_TECHFILE ] && {
	    . $HIDDEN_TECHFILE
	}
    fi

    if [ "$NOTECH" = y ]; then
	echo "  Only the executable portions of serpent may be built at this time. Motif and"
	echo "Athena have not been installed, thus preventing the compilation of the demos."
	echo ""
        echo "OPTION  DESCRIPTION                                                 TIME (Hours)"
        echo "--------------------------------------------------------------------------------"
	echo "  A     Only the Serpent executables are compiled.	               1 3/4"
	echo "        All intermediate files are removed after compilation."
	echo ""
	echo "  B     Only the Serpent executables are compiled.                       1 1/2"
	echo "--------------------------------------------------------------------------------"
	echo ""
        echo -n "  Please select a configuration [A-B] or [Q] to quit. :"
	okay="n"
        while [ "$okay" = n ]; do
	    echo -n " [A-B,Q]:"
	    read ANSWER
	    if [ "$ANSWER" = a ] || [ "$ANSWER" = A ]; then
	    	 okay="y"
		 READY="y"
		 BUILD_EandC="y"
		 TOTAL="1 3/4"
		
	     elif [ "$ANSWER" = b ] || [ "$ANSWER" = B ]; then
		 okay="y"
		 READY="y"
		 BUILD_EXECS="y"
		 TOTAL="1 1/2"
		
	     elif [ "$ANSWER" = q ] || [ "$ANSWER" = Q ]; then
		 okay="y"
		
	    else
		echo -n "Please select [A-B] or Q)uit "
		
	    fi
	done
	ALLDEMOS="n"
    else
        echo "  Serpent can be installed in several configurations."
        echo ""
        echo "OPTION  DESCRIPTION                                                 TIME (Hours)"
        echo "--------------------------------------------------------------------------------"
        echo "  A     All demos and Serpent executables are compiled.                 5 1/4"
        echo "        All intermediate files are removed after compilation."
        echo "        A dependency list is generated for Serpent."
        echo ""
        echo "  B     All demos and Serpent executables are compiled.                 3"
        echo "        A dependency list is generated for Serpent."
        echo ""
        echo "  C     All demos and Serpent executables are compiled.                 2 1/4"
        echo "        All intermediate files are removed after compilation."
        echo ""
        echo "  D     All demos and Serpent executables are compiled.                 2"
        echo ""
        echo "  E     Only the Serpent executables are compiled.                      1 3/4"
        echo "        All intermediate files are removed after compilation."
        echo ""
        echo "  F     Only the Serpent executables are compiled.                      1 1/2"
        echo "---------------------------------------------------------------------------------"
        echo "Note: Depending on configurations, Serpent may take up to 50MB."
	echo "All times represent time required on a fairly slow machine."
	echo "You will probably see better performance than that predicted."
        echo ""
        echo -n "  Please select a configuration [A-F] or [Q] to quit. :"
        okay="n"
        while [ "$okay" = n ]; do
            echo -n " [A-F,Q]:"
            read ANSWER
	    if [ "$ANSWER" = a ] || [ "$ANSWER" = A ]; then
                 okay="y"
		 READY="y"
                 BUILD_EVERYTHING="y"
                 TOTAL="5 1/4"
            elif [ "$ANSWER" = b ] || [ "$ANSWER" = B ]; then
                 okay="y"
		 READY="y"
		 BUILD_DandD="y"
		 TOTAL="3"
            elif [ "$ANSWER" = c ] || [ "$ANSWER" = C ]; then
                 okay="y"
		 READY="y"
		 BUILD_CLEAN="y"
		 TOTAL="2 1/4"
	    elif [ "$ANSWER" = d ] || [ "$ANSWER" = D ]; then
                 okay="y"
		 READY="y"
		 BUILD_DEMOS="y"
		 TOTAL="2"
	    elif [ "$ANSWER" = e ] || [ "$ANSWER" = E ]; then
                 okay="y"
		 READY="y"
		 BUILD_EandC="y"
		 TOTAL="1 3/4"
	    elif [ "$ANSWER" = F ] || [ "$ANSWER" = f ]; then
                 okay="y"
		 READY="y"
		 BUILD_EXECS="y"
		 TOTAL="1 1/2"
	    elif [ "$ANSWER" = Q ] || [ "$ANSWER" = q ]; then
		 okay="y"
	    else
		echo -n "Please select [A-F] or Q)uit"
		
	    fi
	done
    fi

    if [ "$READY" != y ]; then
        . $HEADER_FILE
    	echo "  You have opted not to build Serpent at this time."
    	echo ""
    	echo -n "Is this correct?"
    	okay="n"
    	while [ "$okay" = n ]; do
	    echo -n " [y/n] :"
            read ANSWER
            if [ "$ANSWER" = y ] || [ "$ANSWER" = Y ]; then
            	exit 1
            	
            elif [ "$ANSWER" = n ] || [ "$ANSWER" = N ]; then
		okay="y"
            	
            else
            	echo -n "Please select y)es or n)o"

            fi
    	done # done
    fi # READY

done # READY

. $HEADER_FILE

echo "               NOW BUILDING SERPENT. PROCESS WILL TAKE $TOTAL HOURS."
echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
echo "  Lots of lines will be printed on your screen. Occasionally, there may"
echo "be a warning message or an indicator that an error code was ignored."
echo "These are no cause for alarm. The only thing you should worry about is"
echo "if the whole process fails; if that happens, see the file named"
echo "$SERPENT/notes/BUILD for further instructions"

 the_status=0

if [ "$BUILD_EVERYTHING" = y ] || [ "$BUILD_DandD" = y ]; then
    echo "-------------------------------BUILDING SERPENT-------------------------------"
    make build
    echo "------------------------------MAKE BUILD COMPLETE-----------------------------"
fi # build all demos

if [ "$BUILD_CLEAN" = y ] || [ "$BUILD_DEMOS" = y ] || [ "$BUILD_EandC" = y ] || [ "$BUILD_EXECS" = y ]; then
  echo "------------------------------BUILDING EXECUTABLES----------------------------"
  make serpent
  echo "-----------------------------MAKE SERPENT COMPLETE----------------------------"
fi

if [ "$BUILD_CLEAN" = y ] || [ "$BUILD_DEMOS" = y ]; then
  echo "------------------------------BUILDING DEMOS----------------------------------"
  make demo
  echo "-----------------------------MAKE DEMO COMPLETE-------------------------------"
fi

if [ "$BUILD_EVERYTHING" = y ] || [ "$BUILD_CLEAN" = y ] || [ "$BUILD_EandC" = y ]; then
  echo "-------------------------CLEANING OUT TEMPORARY FILES-------------------------"
  make clean
  echo "-----------------------------MAKE CLEAN COMPLETE------------------------------"
fi

echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
echo "                               Building Complete"
echo ""
echo -n "                           Press Return to Continue"

 read bogus

. $HEADER_FILE
echo "                         Serpent Installation Complete."
echo ""
echo "  Remember to update your .profile or .login file with the environment variables"
echo "found in the serpent.env.sh file (or update your .cshrc with serpent.env.csh"
echo "if your default shell is csh)."
echo ""

