#!	/usr/cs/bin/csh -f

# 	DO NOT CHECKIN TO RCS

umask 000

source ~serpent/integ/source/cm/src/dirs

#	files which cannot be checked in
set bad_files = (Makefile Imakefile)

#	following are self-explanatory
set src_dir = src
set include_dir = include
set externs_dir = externs
set docs_dir = docs
set manual_pages_dir = man/man1
set rcs_dir = RCS
set rcs_ext = ,v
set journal_file = check_journal
set do_rcs

unset dir
unset have_an_externs
unset need_copyright
unset need_header
unset check_makefile
unset demo
unset have_demo

#	if we did not get a module name, barf
if ($1 == "") then
  echo "USAGE"
  echo "	serpent_source_checkin module_name file_name"
  echo " "
  echo "	where module_name is one of:"
  echo "		$mod_names"
  echo "	and file_name is the file in question (include the extension)"
  exit (1)
endif

#	if we did not get a file name, barf
if ($2 == "") then
  echo "	need a file_name"
  exit (1)
endif

#	if the module name is not correct, barf
while (1)
  foreach name ($mod_names)
    if ($name == $1) then
      break; break
    endif
  end
  echo "$1 not a valid module name, contact the libarian"
  exit (1)
end

#       if the file name is not valid, barf
foreach name ($bad_files)
  if ($name == $2) then
    echo "$2 cannot be checked in, contact the libarian"
    exit (1)
  endif
end

#	determine the directory in which to install according to either 
#	   the module name or the file's extension (or lack of one)
set ext = $2:e
if ($1 == "docs") then
  set dir = $docs_dir
  if ($ext == "ps") then
    unset do_rcs
  endif
else if ($1 == "man") then
  set dir = $manual_pages_dir
  set make_dir = $dir
  if ($ext != "1") then
    echo 'man pages must have a ".1" extension."
    exit (11)
  endif
  set check_makefile
else if ( "$1" =~ demos* ) then
  set dir = $1
  set make_dir = $dir
  set demo = $1:t
  set check_makefile
  set have_demo
else if ( "$1" =~ int/ada* ) then
  set dir = $1
  set make_dir = $dir
  set check_makefile
else if ($ext == "h") then
  set dir = $1/$include_dir
  set make_dir = $1/$src_dir
  if ($1 != c_toolkit) then
    set need_copyright
    set need_header
  endif
  set check_makefile
else if ($ext == "c") then
  set dir = $1/$src_dir
  set make_dir = $dir
  set need_copyright
  set need_header
  set check_makefile
else if ($ext == "sl") then
  set dir = $1/$src_dir
  set make_dir = $dir
  set check_makefile
else if ($ext == "sbm") then
  set dir = $1/$src_dir
  set make_dir = $dir
  set check_makefile
else if ($ext == "gl") then
  set dir = $1/$src_dir
  set make_dir = $dir
  set check_makefile
else if ($ext == "me") then
  set dir = $1/$src_dir
  set make_dir = $dir
  set check_makefile
else if ($ext == "at") then
  set dir = $1/$src_dir
  set make_dir = $dir
  set check_makefile
else if ($ext == "l") then
  set dir = $1/$src_dir
  set make_dir = $dir
  set need_copyright
  set need_header
  set check_makefile
else if ($ext == "y") then
  set dir = $1/$src_dir
  set make_dir = $dir
  set need_copyright
  set need_header
  set check_makefile
else if ($ext == "sdd") then
  set dir = $1/$src_dir
  set make_dir = $dir
  set check_makefile
else if ($ext == "ext") then
  set dir = $1/$include_dir
  set make_dir = $1/$src_dir
  set need_copyright
  set need_header
  set check_makefile
else if ($ext == "tx") then
  set dir = $1/$src_dir
  set need_header
  set comment = '# '
else if ($ext == "") then
  set dir = $1/$src_dir
else 
  echo "$ext is an unknown file extension, contact the libarian"
  exit (1)
endif

# check to see if the file exists in the directory it's supposed to be
# checked in from
set localdir=$SERPENT_ROOT/source
if (-e $localdir/$dir/$2) then
else
  echo $2 does not exist in $localdir/$dir
  exit(1)
endif

set numchars = `echo $2|wc -c`
if ($numchars > 13) then
  echo "file name $2 is too long; pick one that's shorter"
  exit (1)
endif

if ($1 == "tools" || $1 == "devtools") then
  unset need_copyright
endif

set integdir=$integsrc/$dir
set integfile=$integdir/$2
set integRdir=$integdir/$rcs_dir
set integRfile=$integRdir/$2$rcs_ext

pushd $localdir/$dir >&/dev/null

#	ensure file mets our high standards
if ($?need_header) then
  grep '$Header[$:]' $2 >/dev/null
  if ($status) then
    echo 'file does not have header marker'
    exit (1)
  endif
  grep '$Log[$:]' $2 >/dev/null
  if ($status) then
    echo 'file does not have Log marker'
    exit (1)
  endif
endif

#	some files need the copyright notice
if ($?need_copyright) then
  grep 'Copyright (c) 1989' $2 >/dev/null
  if ($status) then
    grep 'Copyright (c) 1990' $2 >/dev/null
    if ($status) then
      echo 'file does not have copyright notice'
      exit (1)
    endif
  endif
endif

#	see if the file is new to serpent
if ($?do_rcs) then
  if (-e $integRfile) then
  else
    if (-e $integfile) then
    else
      echo "$2 does not appear to exist in the $1 module."
      echo "is this a new file to be added to Serpent? (y or n)"
      set ans=$<
      if ($ans == "y") then
        echo "new file named $2 will be checked in to module $1"
        echo "$2	$dir	$user	in	`date`" >> $integ/new_files
        set new_file
      else
        echo "$2 will not be checked in"
        exit (1)
      endif
    endif
  endif
endif

#	if the file is not there, barf
if (-e $2) then
else
  echo "$2 does not exist."
  exit (1)
endif

#	if the file is a link, barf
ls -F $2 |grep @ >/dev/null
if ($status == 0) then
  echo "$2 is a link.  it cannot be checked in."
  exit (1)
endif

#	if the file is empty, barf
if (-z $2) then
  echo "$2 is empty.  it cannot be checked in."
  exit (1)
endif

#	checkin the file and make the appropriate links to it 
if ($?do_rcs) then
  ci -f $integRfile
  set cistat=$status
  chmod 555 $integRfile
else
  rm -f $integfile
  cp $2 $integfile
  set cistat=$status
  if ($cistat == 0) then
    rm -f $2
  endif
endif
if ($cistat == 0) then
  if ($?new_file && $?do_rcs) then
    rcs -L -aljb,rcs,ejh,little,dvk,serpent,rnk,rm,clc,mpottier \
		$integRfile
    if ($?comment) then
      rcs -c$comment $integRfile
    endif
  endif
  ln -s $integfile
  pushd $integdir >/dev/null
  if ($?do_rcs) then
    if (-e $2) then
      rm -f $2
    endif
    recheckout $2
  endif

  popd >&/dev/null

  echo "$2	$1	$user	in	`date`" >>$integ/$journal_file
  echo "everything worked ok"
else
  echo " check in failed.  contact the libarian"
endif
popd >&/dev/null
  if ($?check_makefile) then
    grep $2 $integsrc/$make_dir/Imakefile >&/dev/null
    if ($status) then
      grep $2 $integ/$ARCH/$make_dir/Makefile >&/dev/null
      if ($status) then
        echo "$2 is NOT in makefile, notify librarian"
      endif
    endif
  endif
echo " "
