%{

        static char rcsid [] = "$Header: /u/04c/mnt/integ/source/compiler/src/RCS/cal.l,v 1.23 91/01/06 23:45:25 dvk Exp $";

/*--------------------------------------------------------------------------*\
|
|  Name:	cal.l
|
|  Description:
|	This is the lexical analyzer for SLANG.
|
|  Component of:
|	SLANG processor
|
|  Related files:
|	many yacc, lex, and c files
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:  
|	Erik J. Hardy
|	ejh@sei.cmu.edu
|
|  Change History:
$Log:	cal.l,v $
 * Revision 1.23  91/01/06  23:45:25  dvk
 * For some reason, the # rule broke (on the sun but not the Vax).  Fixed.
 * 
 * Revision 1.22  91/01/04  15:44:28  dvk
 * Improved string parsing - now allows multi-line strings, special characters
 * in strings, improved error checking, etc.  Also eliminated special tokens
 * for single character tokens (runs a little faster now).
 * 
 * Revision 1.21  90/11/12  14:36:20  rcs
 * modified parser to allow tokens to start with an underscore
 * 
 * Revision 1.20  90/04/11  12:48:38  little
 * fix the lexer so line with a # in col 1 need not have a file name after
 * the new line number and when that happens the file name remains the same
 * 
 * Revision 1.19  90/02/14  22:11:27  ejh
 * fixed include structure
 * 
 * Revision 1.18  90/01/30  20:18:15  clc
 * Moved the keylookup function, and all of its structures, to before
 * the reference to keylookup()
 * 
 * Revision 1.17  90/01/30  17:15:38  clc
 * No change
 * 
 * Revision 1.16  90/01/19  16:30:08  dvk
 * Now uses u22_convert_string_inplace (no need to make copies of keywords)
 * 
 * Revision 1.15  90/01/09  11:13:59  dvk
 * Changed DEBUG1's back to DEBUG3's - I forgot to do it when I checked it
 * back in last time.
 * 
 * Revision 1.14  90/01/04  17:47:09  dvk
 * Altered in a number of ways: 1) All words (combinations of letters,
 * digits, and underscores) are recognized with the same lexer rule,
 * instead of using separate rules for each keyword. 2) The separation
 * of keywords and Slang names is done in the new routine keylookup
 * and it's associated data structures.  3) The seperable tokens (i.e.,
 * "on", "create", and "oncreate") are now *all* reserved words.  This
 * causes a change in the parser also.  4) The token "elsif" has been
 * added, and the two tokens "else" followed by "if" are now *always*
 * recognized as two distinct tokens.  5) Single character tokens
 * (i.e., parens, punctuation, etc.) are now recognized by a single
 * rule and a quick lookup table.
 * 
 * Revision 1.13  89/11/25  23:39:22  serpent
 * fixed include dropping expression: doesn't need \n any more
 * 
 * Revision 1.12  89/10/23  19:59:13  ejh
 * switched over to u22_copy_string and u22_convert_string
 * 
 * Revision 1.11  89/09/20  11:59:43  little
 * change YYLMAX to be dependent on a system variable so it will be more
 * portable.  change rule for # so we correctly get the file name field
 * 
 * Revision 1.10  89/07/31  14:32:32  ejh
 * added new reserved words: ADI, BOOLEAN, BUFFER, EXTERNALS, ID, INTEGER,
 * REAL, STRING, VOID
 * 
 * Revision 1.9  89/03/14  17:40:42  rcs
 * modified string constants to allow eols
 * 
 * Revision 1.8  89/02/22  16:23:41  rcs
 * fixed real literal problem
 * 
 * Revision 1.7  89/02/10  12:58:32  ejh
 * moved lex offset symbol to top of file to make it lex
 * 
 * Revision 1.6  89/02/09  16:24:54  ejh
 * added copyright notice
 * 
 * Revision 1.5  89/02/06  11:35:51  ejh
 * changed "y.tab.h" to "cay.h" so the new universal make works
 * 
 * Revision 1.4  89/01/04  14:42:50  ejh
 * fixed the extraction of file name and line number for include files
 * 
 * Revision 1.3  88/12/20  15:41:08  ejh
 * removed WHENMODIFIED; redefined real numbers as (double *); added '&' and
 * '|'; added ability to pick up line numbers and file names from included
 * files
 * 
 * Revision 1.2  88/12/08  14:16:03  ejh
 * on_create => on create, etc.; added log
 *
|	01 Mar 1988:	module created
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	memoryPack

#include	<ctype.h>
#include	"global.h"	/* global definitions */
#include	<stdio.h>	/* for I/O */
#include	"listPack.h"	/* to pick up make_node */
#include	"string.h"	/* string handling stuff */
#include	"idd.h"
#include	"u22.h"		/* Serpent universals */
#include	"c22.h"		/* SLANG globals */
#include	"ca2.h"		/* SLANG analysis globals */
#include	"cay.h"		/* yacc token declarations */

	/* make the max input buffer size something reasonable */

#undef YYLMAX
#define YYLMAX BUFSIZ

char	lextext [YYLMAX];	/* line buffer; YYLMAX is plenty big enough */
char	*lextp = lextext;	/* pointer to above */
double  atof(); 
int	i, j;
static int token;

#define	get_line() \
	(*lextp == '\0'? (fgets (lextext, YYLMAX - 1, yyin), \
		lextp = lextext, *lextp++): *lextp++)

/*	these are to redefine input and unput, so that we can read files,
	while still using lex's variables */

#undef	input
#define	input() \
	(((yytchar = yysptr > yysbuf?U (*--yysptr):get_line ()) == \
		'\n'?yylineno++:0), feof (yyin)?0:yytchar)

#undef	unput
#define	unput(c) \
	{yytchar = (c); if (yytchar == '\n') yylineno--; *yysptr++=yytchar;}


/*
 *	Keyword lookup table goodies
 *
 *	Why, you may ask, do we have this routine here when lex can do all
 * of this silly stuff faster, and with a simple description?  The answer
 * is that it makes for a smaller, and faster lexer.  Doing a binary tree
 * search is pretty fast stuff.
 *
 * The macro Key is used to define a simple keyword.
 *
 * The table "bounds" is used to further reduce the binary search time
 * of the keyword lookup by imposing limits of the search tree.  NOTE that
 * you MUST keep the bounds table and the keyword table in synch!
 */

#define Key(str,retval)         str, retval
 
struct keytable {
    char *keyword;
    int token;
    };

static struct keytable keytable[] = {
    /* Alphabetical order is a MUST!! */		/*low*/	/*high*/
    Key("adi", xADI),					/*0*/
    Key("and", xAND),
    Key("attributes", xATTRIBUTES),				/*2*/
    Key("boolean", xBOOLEAN),				/*3*/
    Key("buffer", xBUFFER),					/*4*/
    Key("condition", xCONDITION),			/*5*/
    Key("create", xCREATE),
    Key("creation", xCREATION),
    Key("creationcondition", xCREATIONCONDITION),		/*8*/
    Key("destroy", xDESTROY),				/*9*/
    Key("do", xDO),						/*10*/
    Key("else", xELSE),					/*11*/
    Key("elsif", xELSIF),
    Key("end", xEND),
    Key("endif", xENDIF),
    Key("endvc", xENDVC),
    Key("endwhile", xENDWHILE),
    Key("externals", xEXTERNALS),				/*17*/
    Key("false", xFALSE),				/*18*/	/*18*/
    Key("id", xID),					/*19*/
    Key("if", xIF),
    Key("integer", xINTEGER),					/*21*/
    Key("methods", xMETHODS),				/*22*/	/*22*/
    Key("not", xNOT),					/*23*/
    Key("null", xNULL),						/*24*/
    Key("objects", xOBJECTS),				/*25*/
    Key("on", xON),
    Key("oncreate", xONCREATE),
    Key("ondestroy", xONDESTROY),
    Key("or", xOR),						/*29*/
    Key("real", xREAL),					/*30*/	/*30*/
    Key("string", xSTRING),				/*31*/	/*31*/
    Key("then", xTHEN),					/*32*/
    Key("true", xTRUE),						/*33*/
    Key("undefined", xUNDEFINED),			/*34*/	/*34*/
    Key("variables", xVARIABLES),			/*35*/
    Key("vc", xVC),
    Key("void", xVOID),						/*37*/
    Key("while", xWHILE)				/*38*/	/*38*/
    };
 
#define END(v) 			(v-1 + sizeof v / sizeof v[0])
#define Bound(from,to)	&keytable[(from)],&keytable[(to)]
#define NoBound			(struct keytable *)1,(struct keytable *)0

struct bounds {
    struct keytable *lb, *ub;
    };

static struct bounds bounds[] = {
    Bound(0,2),						/* a */
    Bound(3,4),
    Bound(5,8),
    Bound(9,10),
    Bound(11,17),
    Bound(18,18),					/* f */
    NoBound, NoBound,					/* g - h */
    Bound(19,21),					/* i */
    NoBound, NoBound, NoBound,				/* j - l */
    Bound(22,22),					/* m */
    Bound(23,24),
    Bound(25,29),					/* o */
    NoBound, NoBound,					/* p - q */
    Bound(30,30),					/* r */
    Bound(31,31),
    Bound(32,33),
    Bound(34,34),
    Bound(35,37),
    Bound(38,38),					/* w */
    NoBound, NoBound, NoBound				/* x - z */
    };

static int keylookup(word)
char *word;
{
    struct keytable *low = bounds[*word-'a'].lb,
		    *high = bounds[*word-'a'].ub,
		    *mid;
   int c;

   if (*word != '_') 
      {while (low <= high)
         {mid = low + (high-low)/2;
	 if ((c=strcmp(mid->keyword,word)) == 0)
            {return mid->token;
	    }
	 else if (c < 0)
	    low = mid+1;
	 else
	    high = mid-1;
	}
      }
    /*
     *	If we got here, it is not a keyword, so we call it a name.  To
     * insulate against possible parser lookahead (which would trash the
     * contents of yytext), we save symbol names in a static buffer.  Since
     * we will never see two symbols in a row (legally), this is protection
     * enough against losing symbol values.
     */
    yylval.xstring = u22_copy_string(yytext);
    return(xNAME);
}

/*
 * This routine and variables check for a string overrunning past the
 * end of a file.
 */

static int in_string = 0;
static int too_long = 0;
static int str_start_line;

yywrap()
{
    char err[128];

    if (in_string) {
	sprintf (err, "EOF encountered in string begun on line %d", str_start_line);
	c22_error (PARSING, err, yylineno);
	in_string = 0;
	}
    return 1;
}

%}

%Start tok str special octal2 octal3

%%

<tok>\"			{
			BEGIN str;
			str_start_line = yylineno;
			in_string = 1;
			too_long = 0;
			}

<str>[^"\n\\]*["\n\\]	{
			if (yytext[yyleng-1] == '\\') {
			    yymore();			/* More to come */
			    if (yyleng >= YYLMAX-132 && too_long == 0) {
				too_long = 1;
				c22_error (PARSING, "Warning: String too long", yylineno);
				}
			    BEGIN special;		/* Special processing */
			    yytext[--yyleng] = '\0';	/* Erase the '\' */
			    }
			else {
			    BEGIN tok;
			    in_string = 0;
			    if (yytext[yyleng-1] == '\n')
				c22_error (PARSING, "Missing close quote", yylineno);
			    yytext[--yyleng] = '\0';	/* erase '"' or '\n' */
			    yylval.xstring = u22_copy_string (yytext);
			    return(xQUOTE);
			    }
			}

<octal2>[0-7]		{
			yymore();
			if (yyleng >= YYLMAX-132 && too_long == 0) {
			    too_long = 1;
			    c22_error (PARSING, "Warning: String too long", yylineno);
			    }
			yytext[yyleng-2] *= 8;
			yytext[yyleng-2] += yytext[yyleng-1] - '0';
			yytext[--yyleng] = '\0';
			BEGIN octal3;
			}

<octal3>[0-7]		{
			yymore();
			if (yyleng >= YYLMAX-132 && too_long == 0) {
			    too_long = 1;
			    c22_error (PARSING, "Warning: String too long", yylineno);
			    }
			yytext[yyleng-2] *= 8;
			yytext[yyleng-2] += yytext[yyleng-1] - '0';
			yytext[--yyleng] = '\0';
			BEGIN str;
			}

<octal2,octal3>.	|
<octal2,octal3>\n	{
			yymore();
			if (yyleng >= YYLMAX-132 && too_long == 0) {
			    too_long = 1;
			    c22_error (PARSING, "Warning: String too long", yylineno);
			    }
			if (yytext[yyleng-2] == '\0')
			    c22_error (PARSING, "Warning: String contains the null char", yylineno);
			unput(yytext[yyleng-1]);
			yytext[--yyleng] = '\0';
			BEGIN str;
			}

<special>[0-7]		{
			yymore();
			if (yyleng >= YYLMAX-132 && too_long == 0) {
			    too_long = 1;
			    c22_error (PARSING, "Warning: String too long", yylineno);
			    }
			yytext[yyleng-1] -= '0';
			BEGIN octal2;
			}

<special>.		|
<special>\n		{
			yymore();
			if (yyleng >= YYLMAX-132 && too_long == 0) {
			    too_long = 1;
			    c22_error (PARSING, "Warning: String too long", yylineno);
			    }
			/*
			 * Replace the digraph with the special character
			 */
			switch (yytext[yyleng-1]) {
			    case 'n':
				yytext[yyleng-1] = '\n';
				break;
			    case 'b':
				yytext[yyleng-1] = '\b';
				break;
			    case 'f':
				yytext[yyleng-1] = '\f';
				break;
			    case 'r':
				yytext[yyleng-1] = '\r';
				break;
			    case 't':
				yytext[yyleng-1] = '\t';
				break;
			    case 'v':
				yytext[yyleng-1] = '\v';
				break;
			    case '\\':
				yytext[yyleng-1] = '\\';
				break;
			    case '"':
				yytext[yyleng-1] = '"';
				break;
			    case '\'':
				yytext[yyleng-1] = '\'';
				break;
			    case '\n':	/* This one means "ignore newline" */
				yytext[--yyleng] = '\0';
				break;
			    default:	/* Ignore everything else */
				c22_error (PARSING, "Unknown escape character ignored", yylineno);
				break;
			    }
			BEGIN str;	/* Continue string processing */
			}

<tok>"!*"[^("*!")]*"*!"	{

/*	this says: if you find a comment starter, read everything you find
	until you find the comment ender; then return only the text of the
	comment */

	yytext [yyleng - 2] = '\0';
	yylval.xstring = u22_copy_string (yytext + 2);
	return (xCOMMENT);
	}

<tok>"/*"[.\n]*"*/"	{

/*	this is similar to the comment, but this is a throw away comment, i.e.
	it can show up anywhere in the file, 'cause we're not gonna save it */

	}


<tok>[a-zA-Z_]([a-zA-Z0-9_]*[a-zA-Z0-9]+)*	{

/*	now we look for names; any alphanumeric string (including underscores)
	which starts with a letter is valid text; if it's bigger than 32
	characters, truncate it; finally, convert it to lower case, as SLANG
	is case insensitive (as are SADDLE and ILL) */

	if (yyleng > 32)
		{
		yytext [32] = '\0';
		yyleng = 32;
		}
	return (keylookup(u22_convert_string_inplace (yytext, LOWERCASE)));
	}

<tok>[0-9]+"."[0-9]*	|
<tok>"."[0-9]+	{

/*	now we look for real numbers; then convert from character to real */

	yylval.xreal = (double *) make_node (sizeof (double));
	*(yylval.xreal) = atof (yytext);
	return (xREALNUMBER);
	}

<tok>[0-9]+	{

/*	now we look for integer numbers; then we convert to integer */

	yylval.xint = atoi (yytext);
	return (xINTEGERNUMBER);
	}

<tok>"<>"	{
	return (xNOTEQUAL);
	}

<tok>"<="	{
	return (xLESSEQUAL);
	}

<tok>">="	{
	return (xGREATEREQUAL);
	}

<tok>":="	{
	return (xGETS);
	}

<tok>"**"	{
	return (xPOWER);
	}

<tok>[-+(){}\[\].,;=/&|<>:*]	{

	/*
	 * These are all the single special characters.  They MUST go
	 * after the double character specials
	 */
	return (yytext[0]);
	}

^"#"[^\n]*	{

/*	this says: if you find a pound sign at the beginning of a line, read
	the entire line, then pick off the line number and file name */

	BEGIN tok;
	i = 0;
	while (!isdigit (yytext [i]))
	    i++;
	yylineno = 0;
	while (isdigit (yytext [i])) {
	    yylineno = 10 * yylineno + yytext [i] - '0';
	    i++;
	    }
	while (yytext [i] != '"' && yytext[i] != '\0')
	    i++;
			/* if we have a file name, get it and set the
			   gobal */
	if (yytext[i] != '\0') {
	   i++;
	   j = i;
	   while (yytext [j] != '"')
	       j++;
	   yytext [j] = '\0';
	   c22_pp_filename = u22_copy_string (yytext + i);
	   }
	}

[ \t\n\f]+	{

/*	skip spaces, tabs, formfeeds and eols (yeah, that's a real space) */

	BEGIN tok;
	}

<tok>.	{

/*	this says: if you find any illegal characters (or sequence thereof),
	flush the rest of the line up to, but not including, the EOL and
	return an error */

	c22_error (ILLEGAL_CHAR, yytext, yylineno);
	}

.	|
\n	{
	/*
	 * This rule is here because 1) Any Lex rule not preceded by a start
	 * condition (i.e., <tok>) will match for any start condition, and
	 * 2) Lex always tries to match the most it can for any start
	 * condition.  This means that, for example, if you are looking for
	 * special character (start condition <special>), the rule which
	 * matches tokens will succeed over the rule to match a single
	 * special character *unless* the former rule is in a start condition
	 * which is excluded.  Thus we have the start condition <tok> which
	 * covers everything which is not otherwise covered.  However, we
	 * have to be able to get into that start condition, so this rule
	 * matches only one thing, and immediately shifts start conditions.
	 *
	 * Because the two rules immediately preceding this one will always
	 * match anything which this rule would match, even though this rule
	 * remains active, it is ignored after the first character.
	 */
	BEGIN tok;
	unput (yytext[0]);
	}

%%
