/*--------------------------------------------------------------------------*\
|
|  Name:	cay.y
|
|  Description:
|	This is the grammar for SLANG.
|
|  Component of:
|	SLANG processor (interpreter and compiler)
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Change History:
$Log:	cay.y,v $
 * Revision 1.46  91/02/14  13:25:34  little
 * no changes.  can you ever forgive me.
 * 
 * Revision 1.45  91/01/04  15:43:12  dvk
 * Cleaned up parser a little - eliminated special tokens for single
 * character tokens (runs a little faster now).
 * 
 * Revision 1.44  91/01/02  17:47:01  dvk
 * Sweeping changes throughout compiler for new interpreted output
 * 
 * Revision 1.43  90/11/12  12:21:17  dvk
 * Reflects changes in cus - each generated type now has it's own counter
 * 
 * Revision 1.42  90/10/25  17:57:07  little
 * fix parameters in call to c22_error
 * 
 * Revision 1.41  90/09/20  13:22:40  dvk
 * Fixed warning message about attribute name overlaying variable name.
 * It seems that a forward reference to a component of a VC would cause
 * the message to be erroneously generated (same for a forward reference
 * to an attribute using dot notation).
 * 
 * Revision 1.40  90/09/17  19:15:47  dvk
 * Fixed enumeration type clash and use of cus_gensym -> cus_gensym_no_add
 * 
 * Revision 1.39  90/08/23  15:12:27  dvk
 * Okay, I screwed up.  It works now...
 * 
 * Revision 1.38  90/08/22  17:12:44  dvk
 * Added smarts to the parser so that when an unknown element is seen in
 * the LHS or RHS of an assignment, or in the RHS of a declaration, it
 * looks to see if it is an (implicitly declared) attribute of the current
 * widget, and if so, uses it.  Otherwise, the old strategy of creating
 * a forward reference to a variable is used.
 * 
 * Revision 1.37  90/07/19  16:58:46  ejh
 * no change
 * 
 * Revision 1.36  90/06/16  12:27:34  little
 * add the editor import switch to the parser
 * 
 * Revision 1.35  90/05/11  13:18:02  little
 * fix up to save and restore file information
 * 
 * Revision 1.34  90/04/17  14:59:55  little
 * take out stupid termination message
 * 
 * Revision 1.33  90/04/17  14:04:32  little
 * new way of handling elsifs that does not use nested c ifs, use c
 * gotos instead
 * 
 * Revision 1.32  90/04/12  20:13:13  little
 * enable the statistic print
 * 
 * Revision 1.31  90/03/15  11:21:51  dvk
 * Fixed a bug with nested conditionals.  An if statement inside the 2nd
 * (or later) elsif would cause no code to be generated after the 2nd
 * elsif until the endif.  The problem was due to using global variables
 * when YACC stack variables should have been used (i.e. $0 and $-1).
 * 
 * Revision 1.30  90/03/12  13:49:54  dvk
 * Fixed bug in constant folding where equality/inequality tests were
 * calculated incorrectly.
 * 
 * Revision 1.29  90/02/22  14:54:03  dvk
 * No changes.
 * 
 * Revision 1.28  90/02/14  22:12:22  ejh
 * fixed include structure
 * 
 * Revision 1.27  90/02/07  18:51:00  dvk
 * Added semantic analysis to expression parsing.  Expressions which
 * are composed entirely of constants reduce to a single constant
 * (when integers and booleans are involved).  Expressions which
 * have constant components are reduced if encountered in a manner
 * conducive to left-recursive parsing.
 * 
 * Revision 1.26  90/01/19  16:30:53  dvk
 * Now frees strings associated with xNAME tokens after it is done with them
 * (should keep size of compiler down while executing)
 * 
 * Revision 1.25  90/01/16  15:19:20  little
 * change to do c generation for snippets by vc instead after entire parse
 * 
 * Revision 1.24  90/01/05  14:55:29  dvk
 * Corrected an apparently harmful shift/reduce conflict in the modified
 * grammar w.r.t. on create and on destroy - funny, usually these
 * things are totally benign...
 * 
 * Revision 1.23  90/01/04  17:52:27  dvk
 * Multiple changes concurrent with changes in the lexer. 1) Added new
 * reserved tokens xON, xCREATE, xDESTROY, xCREATION, xCONDITION, xEND.
 * 2) Changed parsing rules so that (for example) an "on create" is
 * parsed by either xON xCREATE or xONCREATE (that is, by one or two
 * keywords).  3) Changed the token name xELSEIF to xELSIF to reflect
 * the changes in the lexer.
 * 
 * Revision 1.22  89/12/21  11:51:43  little
 * add support for slang file name output during dm run-time error print
 * 
 * Revision 1.21  89/11/08  13:16:06  little
 * save each parsed external on god's externs_prop
 * save the actual type of the external on the external's external_type_string
 * propo
 * 
 * Revision 1.20  89/10/23  20:00:16  ejh
 * switched over to u22_copy_string
 * 
 * Revision 1.19  89/09/20  12:01:03  little
 * change the rather strange use of idd_get_component_lop to check and
 * see if a component is in an element to use idd_validate_component instead
 * change the elseif processing to correctly catch the last else in
 * a conditional with several (more than 1) elseifs.
 * 
 * Revision 1.18  89/08/07  16:41:41  ljb
 * put _ after dob_make
 * 
 * Revision 1.17  89/08/03  12:13:37  ejh
 * fixed parameter list in external function declarations
 * 
 * Revision 1.16  89/07/31  14:33:50  ejh
 * put the handling of externals (formerly function defs) inside lex and yacc
 * 
 * Revision 1.15  89/07/24  13:48:51  ejh
 * added snippets to variables
 * 
 * Revision 1.14  89/04/07  16:13:59  little
 * add vc, object, method, attribute, & var counters
 * 
 * Revision 1.13  89/02/27  17:04:44  little
 * cls
 * add call to cam_fix_news in the creation condition recognizer.  this
 * routine converts string parameters to new function into actual data
 * items with the same name as the string value.
 * 
 * Revision 1.12  89/02/22  16:23:04  rcs
 * no change
 * 
 * Revision 1.11  89/02/09  16:28:59  ejh
 * added copyright notice
 * 
 * Revision 1.10  89/01/26  17:32:24  little
 * add capability for lhs of assignment to be a qualified name
 * 
 * Revision 1.9  89/01/04  14:41:51  ejh
 * added optional eos after ENDVC name
 * 
 * Revision 1.8  88/12/21  17:01:27  ejh
 * new syntax for expressions
 * 
 * Revision 1.7  88/12/20  15:35:04  ejh
 * removed extraneous comments; added '&' and '|'; redefined real numbers
 * returned by lex to (double *); removed all references to WHENMODIFIED;
 * made code snippets optional in aoc and aod; allowed for code snippets to
 * be empty ({}); allowed for semicolon after any snippet; redefined eos to
 * be any number > 0 of semicolons, followed by any number >= 0 of comments
 * 
 * Revision 1.6  88/12/19  14:21:42  little
 * add some checks to 
 * 
 * 
 * add some code to check for redeclaration of names within the same scope. this
 * code needed to bne smart enuf to "know" about forward references.
 * 
 * Revision 1.5  88/12/14  10:44:18  little
 * 
 * add xLEFTSQUARE xRIGHTSQUARE in anticipation of using them later on.
 * allow eve to have variables, objects, creation and destruction actions
 * start building dot notation trees in a way which will support implementation
 * in code generation.
 * 
 * Revision 1.4  88/10/31  15:05:26  little
 * change expression statement trees for attributes to have the
 * type of assignment_sym to differentiate between those and "real" snippet
 * trees.
 * 
 * Revision 1.3  88/10/25  10:31:42  little
 * move the id message to a place where yacc will accept it.
 * 
 * Revision 1.2  88/10/24  17:03:23  little
 * take when_modified production out of recognizer
 * 
|	01 Mar 1988:	module created
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

%{
static char rcsid [] =
	"$Header: /u/04c/mnt/integ/source/compiler/src/RCS/cay.y,v 1.46 91/02/14 13:25:34 little Exp $";

#define	memoryPack

#include	"global.h"	/* global definitions */
#include	<stdio.h>	/* for I/O */
#include	"listPack.h"	/* list handling stuff */
#include	"string.h"	/* string handling stuff */
#include	"idd.h"		/* to pick up id_types and data_definitions */

/*	SLANG specific includes */

#include	"u22.h"		/* Serpent universals */
#include	"cus.h"		/* SLANG symbol table stuff */
#include	"c22.h"		/* SLANG globals */
#include	"ca2.h"

#define process_lineno(lineno, info) \
   info = (c22_src_struct *) make_node (sizeof (c22_src_struct)); \
   info -> line = lineno; \
   info -> file_name = c22_pp_filename;

cus_sym	current_vc,
	current_method,
	current_object,
	current_type,
	current_attribute,
	current_variable;

cus_sym	the_sym,
	this,
	if_term;

int	new;

extern	int	yylineno;

c22_src_struct
	*variable_lineno,
	*av_lineno,
	*method_lineno,
	*snippet_lineno,
	*if_lineno,
	*loop_lineno,
	*temp_lineno;

LIST	vc_list,
	the_list,
	another_list,
	temp_cc_list,
	attribute_list;

STACK	the_stack,
	if_term_stack;

string	the_type,
	object_type_name;

int	the_length,
	the_offset,
	the_position;

int	num_vc, num_obj, num_meth, num_att, num_var, num_parms;

boolean	in_cc;

char *saved_file_name;
int saved_line_number;

/*
 * These are used exclusively by cus_gensym_no_add
 */
static int avcs_seed = 0;
static int aves_seed = 0;
static int avies_seed = 0;
static int c_seed = 0;
static int cas_seed = 0;
static int das_seed = 0;
static int i_seed = 0;
static int iivs_seed = 0;
static int it_seed = 0;
static int ivs_seed = 0;
static int mhs_seed = 0;
static int r_seed = 0;

#define LR_INT	0
#define LR_BOOL	1
#define LR_MIX	2

/*
 * If the operand of the unary operator is a constant and is an integer
 * or a boolean, then we can squash the operation back to a constant.
 */
static int can_squash_unary (lhs, ret_type)
cus_sym lhs;
int *ret_type;
{
    idd_data_types lhs_type;

    if (cus_is_sym(lhs) && cus_get_prop(lhs, is_constant_prop))
	{
        lhs_type = (idd_data_types)cus_get_prop(lhs, current_type_prop);
	/*
	 * The calculation of ret_type assumes that there are only two
	 * possible types legal for semantic constant compression!  Note
	 * that even though it may be set incorrectly, it is the return
	 * value of the procedure that ultimately decides.
	 */
	*ret_type = (lhs_type == idd_integer) ? LR_INT : LR_BOOL;
	return (lhs_type == idd_integer) || (lhs_type == idd_boolean);
	}
    else
	return false;
}

/*
 * If both operands of the binary operator are constants and are integers
 * or booleans, then we can squash the operation back to a constant.
 */
static int can_squash_binary (lhs, rhs, ret_types)
cus_sym lhs, rhs;
int *ret_types;
{
    idd_data_types lhs_type, rhs_type;

    if (cus_is_sym(lhs) && cus_get_prop(lhs, is_constant_prop) &&
        cus_is_sym(rhs) && cus_get_prop(rhs, is_constant_prop))
	{
        lhs_type = (idd_data_types)cus_get_prop(lhs, current_type_prop);
        rhs_type = (idd_data_types)cus_get_prop(rhs, current_type_prop);
	/*
	 * The calculation of ret_types assumes that there are only two
	 * possible types legal for semantic constant compression!  Note
	 * that even though it may be set incorrectly, it is the return
	 * value of the procedure that ultimately decides.
	 */
	if (lhs_type == rhs_type)
	    if (lhs_type == idd_integer)
		*ret_types = LR_INT;
	    else
		*ret_types = LR_BOOL;
	else
	    *ret_types = LR_MIX;
	return ((lhs_type == idd_integer) || (lhs_type == idd_boolean)) &&
	       ((rhs_type == idd_integer) || (rhs_type == idd_boolean));
	}
    else
	return false;
}

/*
 * Logical operators can be squashed if both operands are boolean constants.
 * Non-boolean constants generate a compile time error.
 */
static LIST squash_logical (the_list)
LIST the_list;
{
    cus_sym lhs = (cus_sym) get_list_pos (the_list, 2);
    cus_sym rhs = (cus_sym) get_list_pos (the_list, 3);
    cus_sym head;
    int lhs_val, rhs_val, lr_types;

    if ( can_squash_binary (lhs, rhs, &lr_types))
	{
	head = (cus_sym) get_list_head (the_list);
	lhs_val = (int) cus_get_prop (lhs, current_value_prop);
	rhs_val = (int) cus_get_prop (rhs, current_value_prop);
	if (lr_types != LR_BOOL)
	    c22_error (INCOMPATIBLE_OP, "logical", yylineno);
	else if (head == car_and_symbol)
	    lhs = (lhs_val && rhs_val ? car_true_symbol : car_false_symbol);
	else if (head == car_or_symbol)
	    lhs = (lhs_val || rhs_val ? car_true_symbol : car_false_symbol);
	else
	    c22_error(YACC_ERROR, "Logical reduction botch", yylineno);
	(void) free_list (the_list, NOT_NODES);
	return (LIST) lhs;
	}
    else
	return the_list;

}

/*
 * Equality operators can be squashed if both operands are boolean constants,
 * if both operands are integers, or if one operand is boolean and the other
 * operand is an integer whose value is 0 or 1.
 */
static LIST squash_equality (the_list)
LIST the_list;
{
    cus_sym lhs = (cus_sym) get_list_pos (the_list, 2);
    cus_sym rhs = (cus_sym) get_list_pos (the_list, 3);
    cus_sym head;
    int lhs_val, rhs_val, lr_types;
    idd_data_types lhs_type, rhs_type;

    if ( can_squash_binary (lhs, rhs, &lr_types))
	{
	head = (cus_sym) get_list_head (the_list);
	lhs_val = (int) cus_get_prop (lhs, current_value_prop);
	rhs_val = (int) cus_get_prop (rhs, current_value_prop);
	lhs_type = (idd_data_types)cus_get_prop(lhs, current_type_prop);
	rhs_type = (idd_data_types)cus_get_prop(rhs, current_type_prop);
	if (lr_types == LR_MIX) {
	    if ((lhs_type == idd_integer && lhs_val != 0 && lhs_val != 1) ||
	        (rhs_type == idd_integer && rhs_val != 0 && rhs_val != 1)) {
		c22_error (INCOMPATIBLE_OP, "(in)equality", yylineno);
		return the_list;
		}
	    }
	if (lhs_type != idd_boolean)
	    (void) free_node (lhs);
	if (rhs_type != idd_boolean)
	    (void) free_node (rhs);
	if (head == car_equal_symbol) {
	    lhs = (lhs_val == rhs_val ? car_true_symbol : car_false_symbol);
	    }
	else if (head == car_notequal_symbol) {
	    lhs = (lhs_val != rhs_val ? car_true_symbol : car_false_symbol);
	    }
	else
	    c22_error(YACC_ERROR, "Equality reduction botch", yylineno);
	(void) free_list (the_list, NOT_NODES);
	return (LIST) lhs;
	}
    else
	return the_list;

}

/*
 * Relational operations can be squashed if both operands are integer
 * constants.  Mixed integer/boolean comparisons are illegal.
 */
static LIST squash_relational (the_list)
LIST the_list;
{
    cus_sym lhs = (cus_sym) get_list_pos (the_list, 2);
    cus_sym rhs = (cus_sym) get_list_pos (the_list, 3);
    cus_sym head;
    int lhs_val, rhs_val, lr_types;

    if ( can_squash_binary (lhs, rhs, &lr_types))
	{
	head = (cus_sym) get_list_head (the_list);
	lhs_val = (int) cus_get_prop (lhs, current_value_prop);
	rhs_val = (int) cus_get_prop (rhs, current_value_prop);
	(void) free_node (lhs);
	(void) free_node (rhs);
	if (lr_types != LR_INT)
	    c22_error (INCOMPATIBLE_OP, "relational", yylineno);
	else if (head == car_lessthan_symbol)
	    lhs = (lhs_val < rhs_val ? car_true_symbol : car_false_symbol);
	else if (head == car_lessequal_symbol)
	    lhs = (lhs_val <= rhs_val ? car_true_symbol : car_false_symbol);
	else if (head == car_greaterthan_symbol)
	    lhs = (lhs_val > rhs_val ? car_true_symbol : car_false_symbol);
	else if (head == car_greaterequal_symbol)
	    lhs = (lhs_val >= rhs_val ? car_true_symbol : car_false_symbol);
	else
	    c22_error(YACC_ERROR, "Relational reduction botch", yylineno);
	(void) free_list (the_list, NOT_NODES);
	return (LIST) lhs;
	}
    else
	return the_list;

}

/*
 * Addition and subtraction can be squashed if both operands are integer
 * constants.  Mixed integer/boolean arithmetic is illegal.
 */
static LIST squash_addition (the_list)
LIST the_list;
{
    cus_sym lhs = (cus_sym) get_list_pos (the_list, 2);
    cus_sym rhs = (cus_sym) get_list_pos (the_list, 3);
    cus_sym head;
    int lhs_val, rhs_val, lr_types;

    if ( can_squash_binary (lhs, rhs, &lr_types))
	{
	head = (cus_sym) get_list_head (the_list);
	lhs_val = (int) cus_get_prop (lhs, current_value_prop);
	rhs_val = (int) cus_get_prop (rhs, current_value_prop);
	if (lr_types != LR_INT)
	    c22_error (INCOMPATIBLE_OP, "+ or -", yylineno);
	else if (head == car_plus_symbol)
	    cus_put_prop (lhs, current_value_prop, lhs_val + rhs_val);
	else if (head == car_binary_minus_symbol)
	    cus_put_prop (lhs, current_value_prop, lhs_val - rhs_val);
	else
	    c22_error(YACC_ERROR, "Addition reduction botch", yylineno);
	(void) free_list (the_list, NOT_NODES);
	(void) free_node (rhs);
	return (LIST) lhs;
	}
    else
	return the_list;
}

/*
 * Multiplication and division can be squashed if both operands are integer
 * constants.  Mixed integer/boolean arithmetic is illegal, as is division
 * by zero.
 */
static LIST squash_multiplication (the_list)
LIST the_list;
{
    cus_sym lhs = (cus_sym) get_list_pos (the_list, 2);
    cus_sym rhs = (cus_sym) get_list_pos (the_list, 3);
    cus_sym head;
    int lhs_val, rhs_val, lr_types;
    int i_result;
    float f_result;

    if ( can_squash_binary (lhs, rhs, &lr_types))
	{
	head = (cus_sym) get_list_head (the_list);
	lhs_val = (int) cus_get_prop (lhs, current_value_prop);
	rhs_val = (int) cus_get_prop (rhs, current_value_prop);
	if (lr_types != LR_INT)
	    c22_error (INCOMPATIBLE_OP, "* or /", yylineno);
	else if (head == car_star_symbol)
	    cus_put_prop (lhs, current_value_prop, lhs_val * rhs_val);
	else if (head == car_slash_symbol) {
	    if (rhs_val == 0)
		c22_error (DIV_BY_ZERO, "", yylineno);
	    else {
		f_result = (float) lhs_val / (float) rhs_val;
		i_result = (float) (lhs_val / rhs_val);
		if (i_result == f_result)
		    cus_put_prop (lhs, current_value_prop, lhs_val / rhs_val);
		else
		    return the_list;
		}
	    }
	else
	    c22_error(YACC_ERROR, "Multiplication reduction botch", yylineno);
	(void) free_list (the_list, NOT_NODES);
	(void) free_node (rhs);
	return (LIST) lhs;
	}
    else
	return the_list;
}

/*
 * The power function can be reduced if both operands are positive integers.
 * Booleans generate a compiler error.
 */
static LIST squash_power (the_list)
LIST the_list;
{
    cus_sym lhs = (cus_sym) get_list_pos (the_list, 2);
    cus_sym rhs = (cus_sym) get_list_pos (the_list, 3);
    cus_sym head;
    int lhs_val, rhs_val, lr_types, result;

    if ( can_squash_binary (lhs, rhs, &lr_types))
	{
	head = (cus_sym) get_list_head (the_list);
	lhs_val = (int) cus_get_prop (lhs, current_value_prop);
	rhs_val = (int) cus_get_prop (rhs, current_value_prop);
	if (lr_types != LR_INT)
	    c22_error (INCOMPATIBLE_OP, "**", yylineno);
	else if (head == car_power_symbol) {
	    if (rhs_val == 0) {
		if (lhs_val == 0) {
		    c22_error (INCOMPATIBLE_OP, "**", yylineno);
		    return the_list;
		    }
		else
		    cus_put_prop (lhs, current_value_prop, 1);
		}
	    else if (rhs_val < 0)
		return the_list;
	    else {
		result = lhs_val;
		while (--rhs_val)
		    result *= lhs_val;
		cus_put_prop (lhs, current_value_prop, result);
		}
	    }
	else
	    c22_error(YACC_ERROR, "Exponentiation reduction botch", yylineno);
	(void) free_list (the_list, NOT_NODES);
	(void) free_node (rhs);
	return (LIST) lhs;
	}
    else
	return the_list;
}

/*
 * Unary negation of constant integers can be squashed, as can unary
 * complementation of boolean constants.  Negation is illegal for booleans
 * and complementation is illegal for integers.
 */
static LIST squash_unary (the_list)
LIST the_list;
{
    cus_sym lhs = (cus_sym) get_list_pos (the_list, 2);
    cus_sym head;
    int lhs_val, l_type;

    if ( can_squash_unary (lhs, &l_type))
	{
	head = (cus_sym) get_list_head (the_list);
	lhs_val = (int) cus_get_prop (lhs, current_value_prop);
	if (head == car_not_symbol)
	    if (l_type == LR_BOOL)
		cus_put_prop (lhs, current_value_prop, !lhs_val);
	    else
		c22_error (INCOMPATIBLE_OP, "unary complementation", yylineno);
	else if (head == car_unary_minus_symbol)
	    if (l_type == LR_INT)
		cus_put_prop (lhs, current_value_prop, -lhs_val);
	    else
		c22_error (INCOMPATIBLE_OP, "unary negation", yylineno);
	else
	    c22_error(YACC_ERROR, "Unary reduction botch", yylineno);
	(void) free_list (the_list, NOT_NODES);
	return (LIST) lhs;
	}
    else
	return the_list;
}


/*--------------------------------------------------------------------------*\
|  Routine: save_file_stuff
|
|  Description:
|     flushes the input stream
\*--------------------------------------------------------------------------*/
static void save_file_stuff ()
{
   saved_file_name = c22_pp_filename;
   saved_line_number = yylineno;
   return;
}					/* end save_file_stuff */


/*--------------------------------------------------------------------------*\
|  Routine: restore_file_stuff
|
|  Description:
|     flushes the input stream
\*--------------------------------------------------------------------------*/
static void restore_file_stuff ()
{
   c22_pp_filename = saved_file_name;
   yylineno = saved_line_number;
   return;
}					/* end restore_file_stuff */


%}
/*--------------------------------------------------------------------------*\
|  Type: union
| 
|  Description:
|	union of types for yacc/lex
| 
\* Definition:                                                              */

	%union	{
		int		xint;
		string		xstring;
		double		*xreal;
		LIST		xLIST;
		cus_sym		xcus_sym;
		idd_data_types	xidd_data_types;
		}

/*                                                                          *\
|
|  Components:
|	xint		for returning an integer for a token
|	xstring		for returning a character string
|	xreal		for returning a pointer to a double (real)
|	xLIST		for returning a LIST
|	xcus_sym	for returning a cus_sym
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Constants: all of the tokens (and those rules that need to be declared)
| 
|  Description:
|	these are all of the tokens and rules used by yacc for the CLEM
|	processor
| 
\* Definition:                                                              */

	%token	<xint>	xNOTEQUAL xGETS xLESSEQUAL xGREATEREQUAL xPOWER

	%token	<xint>	xAND xATTRIBUTES xCONDITION xCREATE xCREATION
	%token	<xint>	xCREATIONCONDITION xDESTROY xDO xELSE xELSIF xEND
	%token	<xint>	xENDIF xENDVC xENDWHILE xEXTERNALS xFALSE xIF xMETHODS
	%token	<xint>	xNOT xNULL xOBJECTS xON xONCREATE xONDESTROY xOR xTHEN
	%token	<xint>	xTRUE xUNDEFINED xVARIABLES xVC xWHILE

	%token	<xint>	xADI xBOOLEAN xBUFFER xID xINTEGER xREAL xSTRING xVOID

	%token	<xint>	xINTEGERNUMBER

	%token	<xreal>	xREALNUMBER

	%token	<xstring>	xNAME xQUOTE xCOMMENT

/*	and now all the rules that need to be declared */

	%type	<xLIST>	expression boolean_expression term factor id
	%type	<xLIST>	arithmetic_expression signed_id expressions
	%type	<xLIST>	opt_expressions qualified_name
	%type	<xLIST>	relational_expression

	%type	<xcus_sym>	relational_operator logical_operator constant
	%type	<xcus_sym>	multiplication_operator unary_operator
	%type	<xcus_sym>	addition_operator equality_operator

	%type	<xcus_sym>	variable_declaration object_declaration
	%type	<xcus_sym>	boolean_constant attribute_value
	%type	<xcus_sym>	method_handler

	%type	<xidd_data_types>	parm
	%type	<xidd_data_types>	external_type

	%type	<xLIST>	variable_declarations opt_variable_declarations
	%type	<xLIST>	av_choice vv_choice opt_gets_exp object_declarations
	%type	<xLIST>	attribute_values opt_attribute_values
	%type	<xLIST>	method_handlers opt_method_handlers eos

	%type	<xLIST>	opt_code_snippet code_snippet opt_statements
	%type	<xLIST>	statements statement assignment_statement opt_else
	%type	<xLIST>	conditional_statement if_first opt_if_middle if_middle
	%type	<xLIST>	imperative_statement loop_statement boolean_condition
	%type	<xLIST> opt_first_expression opt_first_boolean_expression
	%type	<xLIST>	opt_first_arithmetic_expression opt_first_term
	%type	<xLIST> opt_first_factor opt_unary_operator
	%type	<xLIST>	opt_first_relational_expression
	%type	<xLIST>	opt_comments comments function_call

/*	%start isn't really necessary in this case, as yacc will take the
	first production it finds as the start, but it's a good habit to get
	into, especially as you can define multiple starting points */

	%start	slang_program

	%%

/*                                                                          *\
\*--------------------------------------------------------------------------*/

/*	unfortunately, the : and first choice must appear on the top line */

slang_program:	opt_comments
		{
		num_vc = 0;
		num_obj = 0;
		num_meth = 0;
		num_att = 0;
		num_var = 0;
		the_stack = make_stack ();
		if_term_stack = make_stack ();
		if_term = NULL;
		current_vc = car_god_vc;
		if ($1 != NULL)
		    cus_put_prop (current_vc, vc_comments_prop, $1);
		vc_list = make_list ();
		(void) add_to_head (vc_list, current_vc);
		}
		opt_externals component_list
		{
		save_file_stuff ();
		cs2_vc_gen (car_god_vc);
		restore_file_stuff ();
		}
		opt_vcs
		{
		get_from_head (vc_list);	/* pop god from vc_list */
		cus_put_prop (car_god_vc, sub_vcs_prop, vc_list);
		csd_vc_write (car_god_vc);
		cs2_vc_cleanup (car_god_vc);
		fprintf (stdout, "Number of VCs = %d\n", num_vc);
		fprintf (stdout, "Number of Objects = %d\n", num_obj);
		fprintf (stdout, "Number of Methods = %d\n", num_meth);
		fprintf (stdout, "Number of Attributes = %d\n", num_att);
		fprintf (stdout, "Number of Variables = %d\n", num_var);
		}

opt_vcs:	/* empty string */

	|	vcs
	;

vcs:		vc

	|	vcs vc
	;

vc:		xVC ':' xNAME opt_comments
		{
		current_vc = cus_add_sym ($3, vc_sym, &new, WITH_CONTEXT);
		cus_put_prop (current_vc, u_name_prop, cus_genname ("VC"));
		cus_put_prop (current_vc, parent_vc_prop,
		    get_list_head (vc_list));
		(void) add_to_tail (vc_list, current_vc);
		(void) push_stack (the_stack, vc_list);
		vc_list = make_list ();
		(void) add_to_head (vc_list, current_vc);
		if ((! new) && (cus_get_prop (current_vc, user_defined_prop)))
		    c22_error (VC_EXISTS, $3, yylineno);
		cus_put_prop (current_vc, user_defined_prop, 1);
		cus_put_prop (current_vc, type_prop, vc_sym);
		}
		creation_condition
		{
			/* this craziness is done, because the cc generation
			   part assumes that the vc has a valid 
			   owned_context_prop, but it also assumes that it
			   is in the context of the parent vA
			*/
		cus_put_prop (current_vc, owned_context_prop,
		    cus_create_nu_context ());
		cus_deactivate_context ();
		save_file_stuff ();
		cs2_cc_gen (current_vc);
		restore_file_stuff ();
			/* now we have to set the correct context for futher
			   processing
			*/
		cus_activate_context (cus_get_prop (current_vc,
						    owned_context_prop));
		}
		component_list
		{
		save_file_stuff ();
		cs2_vc_gen (current_vc);
		restore_file_stuff ();
		}
		opt_vcs endvc xNAME opt_eos
/*
??? need to check the end vc name is same as vc name
*/
		{
		current_vc = (cus_sym) get_from_head (vc_list);
		if ($4 != NULL)
		    cus_put_prop (current_vc, vc_comments_prop, $4);
		cus_put_prop (current_vc, sub_vcs_prop, vc_list);
		csd_vc_write (current_vc);
		cs2_vc_cleanup (current_vc);
		vc_list = (LIST) pop_stack (the_stack);
		cus_deactivate_context ();
		num_vc++;
		(void) free_node($3);
		(void) free_node($12);
		}

endvc:		xENDVC

	|	xEND xVC
	;

opt_externals:	/* empty string */

	|	externals
	;

externals:	external

	|	externals external
	;

external:	xEXTERNALS ':' opt_external_declarations

opt_external_declarations:	/* empty string */

	|	external_declarations
	;

external_declarations:	external_declaration

	|	external_declarations external_declaration
	;

external_declaration:	external_type
		{
		num_parms = 0;
		}
		xNAME '('
		{
		the_sym = cus_add_sym ($3, id_sym, &new, NO_CONTEXT);
		if (!new)
		    c22_error (FUNCTION_DEFINITION, $3, yylineno);
		add_to_tail (cus_get_prop (car_god_vc, externs_prop),the_sym);
		/*
		 * Prop built_in is defined to 2 here, because the function
		 * is not really built in, but rather simply "defined".
		 */
		cus_put_prop (the_sym, is_built_in_prop, 2);
		cus_put_prop (the_sym,
			returned_value_conversion_function_prop, $1);
		cus_put_prop (the_sym, c_function_name_prop, $3);
		the_list = make_list ();
		cus_put_prop (the_sym, parameter_conversion_functions_prop,
		    the_list);
		}
		opt_parm_list ')' eos
		{
		cus_put_prop (the_sym, num_operands_prop, num_parms);
		/*
		 * Don't free_node($3), because it is still used indirectly
		 * by the call to cus_put_prop
		 */
		}

external_type:	xBOOLEAN
		{
		$$ = idd_boolean;
		}

	|	xBUFFER
		{
		$$ = idd_buffer;
		}

	|	xID
		{
		$$ = idd_id;
		}

	|	xINTEGER
		{
		$$ = idd_integer;
		}

	|	xREAL
		{
		$$ = idd_real;
		}

	|	xSTRING
		{
		$$ = idd_string;
		}

	|	xVOID
		{
		$$ = idd_no_data_type;
		}
	;

opt_parm_list:	/* empty string */

	|	parm_list
	;

parm_list:	parm
		{
		(void) add_to_tail (the_list, $1);
		num_parms = num_parms + 1;
		}

	|	parm_list ',' parm
		{
		(void) add_to_tail (the_list, $3);
		num_parms = num_parms + 1;
		}
	;

parm:		xADI
		{
		$$ = idd_no_data_type;
		}

	|	xBOOLEAN
		{
		$$ = idd_boolean;
		}

	|	xBUFFER
		{
		$$ = idd_buffer;
		}

	|	xID
		{
		$$ = idd_id;
		}

	|	xINTEGER
		{
		$$ = idd_integer;
		}

	|	xREAL
		{
		$$ = idd_real;
		}

	|	xSTRING
		{
		$$ = idd_string;
		}
	;

component_list:	opt_variables opt_objects optional_actions

creation_condition:	creationcondition
		{
		cus_put_prop (current_vc, cc_used_vars_prop, make_list ());
		in_cc = TRUE;
		}
		':' boolean_condition
		{
		cus_put_prop (current_vc, creation_condition_prop, $4);
		temp_cc_list = make_list ();
		add_to_head (temp_cc_list, $4);
		cam_fix_news (current_vc, temp_cc_list);
		free_list (temp_cc_list, NOT_NODES);
		process_lineno (yylineno, temp_lineno);
		cus_put_prop (current_vc, cc_lineno_prop, temp_lineno);
		in_cc = FALSE;
		}

creationcondition:	xCREATIONCONDITION

	|		xCREATION xCONDITION
	;

opt_variables:	/* empty string */

	|	xVARIABLES ':' opt_variable_declarations
		{
		cus_put_prop (current_vc, variables_prop, $3);
		}
	;

opt_variable_declarations:	/* empty string */
		{
		$$ = NULL;
		}

	|	variable_declarations
	;

variable_declarations:	variable_declaration
		{
		LIST the_list = make_list ();
		(void) add_to_tail (the_list, $1);
		$$ = the_list;
		}

	|	variable_declarations variable_declaration
		{
		(void) add_to_tail ($1, $2);
		}
	;

variable_declaration:	xNAME
		{
		current_variable = cus_add_sym ($1, variable_sym, &new,
		    WITH_CONTEXT);
		cus_put_prop (current_variable, u_name_prop,
		    cus_genname ("VAR"));
		if ((! new) && (cus_get_prop (current_variable,
		    user_defined_prop)))
		    c22_error (VAR_EXISTS, $1, yylineno);
		cus_put_prop (current_variable, user_defined_prop, 1);
		cus_put_prop (current_variable, type_prop, variable_sym);
		process_lineno (yylineno, variable_lineno);
		variable_lineno -> file_name = c22_pp_filename;
		}
		vv_choice
		{
		cus_put_prop (current_variable, parent_vc_prop, current_vc);
		if ($3 != NULL)
		    cus_put_prop (current_variable,
			initial_value_snippet_prop, $3);
		$$ = current_variable;
		num_var++;
		(void) free_node($1);
		}

vv_choice:	':' code_snippet
		{
		if ($2 != NULL)
		    {
		    the_sym = cus_gensym_no_add ("initial_value_snippet",
			snippet_sym, &ivs_seed);
		    (void) add_to_head_val ($2, the_sym, variable_lineno);
		    }
		$$ = $2;
		}

	|	opt_gets_exp eos
		{
		if ($1 != NULL)
		    {
		    LIST the_list = make_list ();
			
		    if (c22_edit_exp_proc)
			/* we are using exp_only_sym here, so the editor
			   importer will know this in not a real snippet */
		       the_sym = cus_gensym_no_add ("initial_inport_value_snippet",
			 		     exp_only_sym, &iivs_seed);
                     else
		       the_sym = cus_gensym_no_add ("initial_value_snippet",
			 		     snippet_sym, &ivs_seed);
		    (void) add_to_tail_val (the_list, the_sym,
			variable_lineno);
		    (void) add_to_tail (the_list, $1);
		    $$ = the_list;
		    }
		if ($2 != NULL)
		    cus_put_prop (current_variable, variable_comments_prop,
			$2);
		}
	;

opt_gets_exp:	/* empty string */
		{
		$$ = NULL;
		}

	|	':' expression
		{
		if (c22_edit_exp_proc)
                   $$ = $2;
                else
		   {
		   LIST the_list = make_list ();
		   process_lineno (yylineno, temp_lineno);
		   add_to_tail_val (the_list, car_gets_symbol, temp_lineno);
		   (void) add_to_tail (the_list, current_variable);
		   (void) add_to_tail (the_list, $2);
		   $$ = the_list;
                   }

		}
	;

opt_objects:	/* empty string */

	|	objects
	;

objects:	xOBJECTS ':' object_declarations
		{
		cus_put_prop (current_vc, objects_prop, $3);
		}

object_declarations:	object_declaration
		{
		LIST the_list = make_list ();
		(void) add_to_tail (the_list, $1);
		$$ = the_list;
		}

	|	object_declarations object_declaration
		{
		(void) add_to_tail ($1, $2);
		}
	;

object_declaration:	xNAME
		{
		current_object = cus_add_sym ($1, object_sym, &new,
		    WITH_CONTEXT);
		cus_put_prop (current_object, u_name_prop,
		    cus_genname ("OBJ"));
		if ((! new) && (cus_get_prop (current_object,
		    user_defined_prop)))
		    c22_error (OBJECT_EXISTS, $1, yylineno);
		cus_put_prop (current_object, user_defined_prop, 1);
		cus_put_prop (current_object, type_prop, object_sym);
		cus_put_prop (current_object, owned_context_prop,
		    cus_create_nu_context ());
		cus_put_prop (current_object, parent_vc_prop, current_vc);
		}
		':' object_type
		{
		(void) cus_deactivate_context ();
		$$ = current_object;
		num_obj++;
		(void) free_node($1);
		}

object_type:	xNAME opt_comments '{'
		{
		if ($2 != NULL)
		    cus_put_prop (current_object, object_comments_prop, $2);
		object_type_name = u22_copy_string ($1);
		this = ca2_get_sd_sym ($1);
		if (this == NULL)
		    c22_error (UNKNOWN_OBJECT_TYPE, $1, yylineno);
		else
		    cus_put_prop (current_object, data_definition_prop, this);
/*
?? should the object_type use cus_add_sym or cus_look_ups and then cus_add_sym
*/
		current_type = cus_add_sym ($1, sd_sym, &new, NO_CONTEXT);
		cus_put_prop (current_object, object_type_prop, current_type);
		}
		opt_attributes opt_methods '}'
		{
		(void) free_node($1);
		this = NULL;
		}

opt_attributes:	/* empty string */

	|	xATTRIBUTES ':' opt_attribute_values
		{
		if ($3 != NULL)
		    cus_put_prop (current_object, attributes_prop, $3);
		}
	;

opt_attribute_values:	/* empty string */
		{
		$$ = NULL;
		}

	|	attribute_values
	;

attribute_values:	attribute_value
		{
		LIST the_list = make_list ();
		(void) add_to_tail (the_list, $1);
		$$ = the_list;
		}

	|	attribute_values attribute_value
		{
		(void) add_to_tail ($1, $2);
		}
	;

attribute_value:	xNAME ':'
		{
		process_lineno (yylineno, av_lineno);
		av_lineno -> file_name = c22_pp_filename;
		current_attribute = cus_add_sym ($1, attribute_sym, &new,
		    WITH_CONTEXT);
		if (!new)
		    cus_put_prop (current_attribute, type_prop,
			attribute_sym);
		cus_put_prop (current_attribute, u_name_prop,
		    cus_genname ("ATT"));
		}
		av_choice
		{
		cus_put_prop (current_attribute, value_snippet_prop, $4);
		cus_put_prop (current_attribute, parent_object_prop,
		    current_object);
		if (this == NULL ||
		    (!idd_validate_component ((idd_data_definition)
			    cus_get_prop (this, idd_handle_prop),
			    object_type_name, $1)))
		       c22_error (UNKNOWN_ATTRIBUTE, $1, av_lineno-> line);
		$$ = current_attribute;
		num_att++;
		(void) free_node($1);
		}

av_choice:	code_snippet
		{
		if ($1 != NULL)
		    {
		    the_sym = cus_gensym_no_add ("av_code_snippet",
						    snippet_sym, &avcs_seed);
		    (void) add_to_head_val ($1, the_sym, av_lineno);
		    }
		}

	|	expression eos
		{
		attribute_list = make_list ();
				/* use a symbol type of assignment_sym so
				   we can tell the difference between this
			 	   and the full blown code snippet version
				   later on in code generation */

		    /* we are using exp_only_sym here, so the editor
		       importer will know this in not a real snippet */
		if (c22_edit_exp_proc) {
		   the_sym = cus_gensym_no_add ("av_inport_exp_snippet",
					  exp_only_sym, &avies_seed);
		   (void) add_to_tail_val (attribute_list, the_sym, av_lineno);
		   add_to_tail (attribute_list, $1);
                   }
                 else
		   {
		   the_sym = cus_gensym_no_add ("av_expression_snippet",
					  assignment_sym, &aves_seed);
		   (void) add_to_tail_val (attribute_list, the_sym, av_lineno);
		   the_list = make_list ();
		   add_to_tail_val (the_list, car_gets_symbol, av_lineno);
		   (void) add_to_tail (the_list, current_attribute);
		   (void) add_to_tail (the_list, $1);
		   (void) add_to_tail (attribute_list, the_list);
                   }
		if ($2 != NULL)
		   (void) append_list (attribute_list, $2);
		$$ = attribute_list;
		}
	;

opt_methods:	/* empty string */

	|	xMETHODS ':' opt_method_handlers
		{
		if ($3 != NULL)
		    cus_put_prop (current_object, methods_prop, $3);
		}
	;

opt_method_handlers:	/* empty string */
		{
		$$ = NULL;
		}

	|	method_handlers
	;

method_handlers:	method_handler
		{
		LIST the_list = make_list ();
		(void) add_to_tail (the_list, $1);
		$$ = the_list;
		}

	|	method_handlers method_handler
		{
		(void) add_to_tail ($1, $2);
		}
	;

method_handler:	xNAME ':'
		{
		process_lineno (yylineno, method_lineno);
		method_lineno -> file_name = c22_pp_filename;
		}
		code_snippet
		{
		the_sym = cus_gensym_no_add ("method_handler_snippet",
						snippet_sym, &mhs_seed);
		if ($4 != NULL)
		    (void) add_to_head_val ($4, the_sym, method_lineno);
		current_method = cus_add_sym ($1, id_sym, &new, WITH_CONTEXT);
		cus_put_prop (current_method, u_name_prop,
		    cus_genname ("METH"));
		cus_put_prop (current_method, value_snippet_prop, $4);
		cus_put_prop (current_method, parent_object_prop,
		    current_object);
		$$ = current_method;
		num_meth++;
		(void) free_node($1);
		}

optional_actions:	/* empty string */

	|	opt_aoc

	|	opt_aoc opt_aod

	|	opt_aod
	;

opt_aoc:	oncreate ':' opt_code_snippet
		{
		if ($3 != NULL)
		    {
		    the_sym = cus_gensym_no_add ("create_actions_snippet",
			snippet_sym, &cas_seed);
		    (void) add_to_head_val ($3, the_sym, snippet_lineno);
		    cus_put_prop (current_vc, create_actions_snippet_prop,$3);
		    }
		}
	;

oncreate:	xONCREATE

	|	xON xCREATE
	;

opt_aod:	ondestroy ':' opt_code_snippet
		{
		if ($3 != NULL)
		    {
		    the_sym = cus_gensym_no_add ("destroy_actions_snippet",
			snippet_sym, &das_seed);
		    (void) add_to_head_val ($3, the_sym, snippet_lineno);
		    cus_put_prop (current_vc, destroy_actions_snippet_prop, $3);
		    }
		}
	;

ondestroy:	xONDESTROY

	|	xON xDESTROY
	;

opt_code_snippet:	/* empty string */
		{
		$$ = NULL;
		}

	|	code_snippet
	;

code_snippet:	'{'
		{
		process_lineno (yylineno, snippet_lineno);
                snippet_lineno -> file_name = c22_pp_filename;
		}
		opt_comments opt_statements '}' opt_eos
		{
/*
		if ($3 != NULL)
		    {
		    if ($4 != NULL)
			(void) add_to_tail ($3, $4);
		    $$ = $3;
		    }
		else
		    $$ = $4;
*/
		if ($3 != NULL && $4 != NULL)
			(void) add_to_head ($4, $3);
		$$ = $4;
		}

opt_statements:	/* empty string */
		{
		$$ = NULL;
		}

	|	statements
	;

statements:	statement eos
		{
		LIST the_list = make_list ();
		(void) add_to_tail (the_list, $1);
		if ($2 != NULL)
		    (void) add_to_tail (the_list, $2);
		$$ = the_list;
		}

	|	statements statement eos
		{
		(void) add_to_tail ($1, $2);
		if ($3 != NULL)
		    (void) add_to_tail ($1, $3);
		}
	;

statement:	conditional_statement

	|	assignment_statement

	|	loop_statement

	|	imperative_statement
	;

conditional_statement:	if_first opt_if_middle opt_else endif
		{
	        if ($2 != NULL)
		   append_list ($1, $2);
	        if ($3 != NULL)
		   add_to_tail ($1, $3);
		$$ = $1;
		}

if_first:	xIF
		{
		process_lineno (yylineno, if_lineno);
		}
		boolean_condition xTHEN statements
		{
		LIST if_list = make_list ();
		(void) add_to_tail_val (if_list, car_if_symbol, if_lineno);
		(void) add_to_tail (if_list, $3);
		(void) add_to_tail (if_list, $5);
		$$ = if_list;
		}

opt_if_middle:	/* empty string */
		{
		$$ = NULL;
		}

	|
		if_middle
	;


if_middle:	xELSIF
		{
		process_lineno (yylineno, if_lineno);
		push_stack (if_term_stack, if_term);
		if_term = cus_gensym_no_add ("if_term", id_sym, &it_seed);
		}
		boolean_condition xTHEN statements
		{
		LIST elsif_list = make_list ();
		LIST the_list = make_list ();
		(void)add_to_tail_val (elsif_list, car_elsif_symbol, if_lineno);
		(void)add_to_tail (elsif_list, $3);
		(void)add_to_tail (elsif_list, $5);
		(void)add_to_tail (the_list, elsif_list);
		$$ = the_list;
		}
	|	if_middle xELSIF
		{
		process_lineno (yylineno, if_lineno);
		}
		boolean_condition xTHEN statements
		{
		LIST elsif_list = make_list ();
		(void)add_to_tail_val (elsif_list, car_elsif_symbol, if_lineno);
	        (void)add_to_tail (elsif_list, $4);
		(void)add_to_tail (elsif_list, $6);
		add_to_tail ($1, elsif_list);
		$$ = $1;
		}
	;


opt_else:	/* empty string */
		{
		$$ = NULL;
		}
	|	xELSE statements
		{
		$$ = $2;
		}
	;

endif:		xENDIF

	|	xEND xIF
	;


assignment_statement:	qualified_name xGETS expression
		{
		LIST the_list = make_list ();
		process_lineno (yylineno, temp_lineno);
		(void)add_to_tail_val (the_list, car_gets_symbol, temp_lineno);
		(void) add_to_tail (the_list, $1);
		(void) add_to_tail (the_list, $3);
		$$ = the_list;
		}

imperative_statement:	function_call

loop_statement:	xWHILE
		{
		process_lineno (yylineno, loop_lineno);
		}
		boolean_condition opt_xDO statements endwhile
		{
		LIST the_list = make_list ();
		(void) add_to_tail_val (the_list, car_while_symbol,
		    loop_lineno);
		(void) add_to_tail (the_list, $3);
		(void) add_to_tail (the_list, $5);
		$$ = the_list;
		}

opt_xDO:	/* empty string */

	|	xDO
	;

endwhile:	xENDWHILE

	|	xEND xWHILE
	;

boolean_condition:	'(' expression ')'
		{
		$$ = $2;
		}

expression:	opt_first_expression boolean_expression
		{
		if ($1 != NULL)
		    {
		    LIST the_list = make_list ();
		    (void) append_list (the_list, $1);
		    (void) add_to_tail (the_list, $2);
		    $$ = squash_logical (the_list);
		    }
		else
		    $$ = $2;
		}

opt_first_expression:	/* empty string */
		{
		$$ = NULL;
		}

	|	expression logical_operator
		{
		LIST the_list = make_list ();
		process_lineno (yylineno, temp_lineno);
		(void) add_to_tail_val (the_list, $2, temp_lineno);
		(void) add_to_tail (the_list, $1);
		$$ = the_list;
		}
	;

boolean_expression:	opt_first_boolean_expression relational_expression
		{
		if ($1 != NULL)
		    {
		    LIST the_list = make_list ();
		    (void) append_list (the_list, $1);
		    (void) add_to_tail (the_list, $2);
		    $$ = squash_equality (the_list);
		    }
		else
		    $$ = $2;
		}

opt_first_boolean_expression:	/* empty string */
		{
		$$ = NULL;
		}

	|	boolean_expression equality_operator
		{
		LIST the_list = make_list ();
                process_lineno (yylineno, temp_lineno);
		(void) add_to_tail_val (the_list, $2, temp_lineno);
		(void) add_to_tail (the_list, $1);
		$$ = the_list;
		}
	;

relational_expression:	opt_first_relational_expression arithmetic_expression
		{
		if ($1 != NULL)
		    {
		    LIST the_list = make_list ();
		    (void) append_list (the_list, $1);
		    (void) add_to_tail (the_list, $2);
		    $$ = squash_relational (the_list);
		    }
		else
		    $$ = $2;
		}

opt_first_relational_expression:	/* empty string */
		{
		$$ = NULL;
		}

	|	relational_expression relational_operator
		{
		LIST the_list = make_list ();
		process_lineno (yylineno, temp_lineno);
		(void) add_to_tail_val (the_list, $2, temp_lineno);
		(void) add_to_tail (the_list, $1);
		$$ = the_list;
		}
	;

arithmetic_expression:	opt_first_arithmetic_expression term
		{
		if ($1 != NULL)
		    {
		    LIST the_list = make_list ();
		    (void) append_list (the_list, $1);
		    (void) add_to_tail (the_list, $2);
		    $$ = squash_addition (the_list);
		    }
		else
		    $$ = $2;
		}

opt_first_arithmetic_expression:	/* empty string */
		{
		$$ = NULL;
		}

	|	arithmetic_expression addition_operator
		{
		LIST the_list = make_list ();
                process_lineno (yylineno, temp_lineno);
		(void) add_to_tail_val (the_list, $2, temp_lineno);
		(void) add_to_tail (the_list, $1);
		$$ = the_list;
		}
	;

term:		opt_first_term factor
		{
		if ($1 != NULL)
		    {
		    LIST the_list = make_list ();
		    (void) append_list (the_list, $1);
		    (void) add_to_tail (the_list, $2);
		    $$ = squash_multiplication (the_list);
		    }
		else
		    $$ = $2;
		}

opt_first_term:	/* empty string */
		{
		$$ = NULL;
		}

	|	term multiplication_operator
		{
		LIST the_list = make_list ();
		process_lineno (yylineno, temp_lineno);
		(void) add_to_tail_val (the_list, $2, temp_lineno);
		(void) add_to_tail (the_list, $1);
		$$ = the_list;
		}
	;

factor:		opt_first_factor signed_id
		{
		if ($1 != NULL)
		    {
		    LIST the_list = make_list ();
		    (void) append_list (the_list, $1);
		    (void) add_to_tail (the_list, $2);
		    $$ = squash_power (the_list);
		    }
		else
		    $$ = $2;
		}

opt_first_factor:	/* empty string */
		{
		$$ = NULL;
		}

	|	factor xPOWER
		{
		LIST the_list = make_list ();
                process_lineno (yylineno, temp_lineno);
		(void)add_to_tail_val (the_list,car_power_symbol,temp_lineno);
		(void) add_to_tail (the_list, $1);
		$$ = the_list;
		}
	;

signed_id:	opt_unary_operator id
		{
		if ($1 != NULL)
		    {
		    LIST the_list = make_list ();
		    (void) append_list (the_list, $1);
		    (void) add_to_tail (the_list, $2);
		    $$ = squash_unary (the_list);
		    }
		else
		    $$ = $2;
		}

opt_unary_operator:	/* empty string */
		{
		$$ = NULL;
		}

	|	unary_operator
		{
		LIST the_list = make_list ();
		process_lineno (yylineno, temp_lineno);
		(void) add_to_tail_val (the_list, $1, temp_lineno);
		$$ = the_list;
		}
	;

id:		qualified_name
		{
		if (in_cc)
		    (void) add_to_tail (cus_get_prop (current_vc,
			cc_used_vars_prop), $1);
		}

	|	constant
		{
		$$ = (LIST) $1;
		}

	|	'(' expression ')'
		{
		$$ = $2;
		}

	|	function_call
	;

qualified_name:	xNAME
		{
		/*
		 * If the symbol is in the symbol table, and it is not
		 * the name of an attribute, see if there is an attribute
		 * with that name.  If so, use the attribute (because it
		 * is implicitly defined for the object), but issue a
		 * warning message.  If there is no attribute by that name,
		 * then use the symbol that was found.
		 */
		if (the_sym = cus_look_up ($1)) {
		    if (((cus_sym_type)cus_get_prop (the_sym, type_prop) != attribute_sym) &&
		     this != NULL &&
		     idd_validate_component ((idd_data_definition)
			cus_get_prop (this, idd_handle_prop),
			    object_type_name, $1))
			{
			if (c22_pp_filename)
			    fprintf (stderr, "WARNING: at or near line # %d, in file %s\n\t", yylineno , c22_pp_filename);
			else
			    fprintf (stderr, "WARNING: ");
			fprintf (stderr, "The name \"%s\" is an implicitly declared attribute\n", $1);
			fprintf (stderr, "\tin a \"%s\".  Using the implicit attribute.\n\n", object_type_name);
			the_sym = cus_add_sym ($1, attribute_sym, &new, WITH_CONTEXT);
			if (!new)
			    cus_put_prop (the_sym, type_prop, attribute_sym);
			cus_put_prop (the_sym, u_name_prop, cus_genname("ATT"));
			}
		    }
		else {
		    /*
		     * If there is a RHS reference to a name that has not been
		     * previously defined, see if it is in the attribute list
		     * defined in the ILL file for "this" object.  If yes, then
		     * declare it, but don't give it any value.
		     */
		    if (this != NULL &&
			idd_validate_component ((idd_data_definition)
			    cus_get_prop (this, idd_handle_prop),
				object_type_name, $1))
			{
			the_sym = cus_add_sym ($1, attribute_sym, &new, WITH_CONTEXT);
			if (!new)
			    cus_put_prop (the_sym, type_prop, attribute_sym);
			cus_put_prop (the_sym, u_name_prop, cus_genname("ATT"));
			}
		    /*
		     * If it isn't an attribute name, just create it and
		     * assume that it will be a variable that will be defined
		     * elsewhere (and let later error checking get it in case
		     * it isn't declared).
		     */
		    else
			the_sym = cus_add_sym ($1, id_sym, &new, WITH_CONTEXT);
		    }
		$$ = (LIST) the_sym;
		(void) free_node ($1);
		}

	|	qualified_name '.' xNAME
		{
		LIST the_list;

		if (cus_is_sym ((cus_sym) $1))
		    {
		    the_list = make_list ();
		    (void) add_to_tail (the_list, car_dot_symbol);
		    (void) add_to_tail (the_list, $1);
		    }
		else
		    the_list = (LIST) $1;
		if (!(the_sym = cus_look_up ($3))) {
/*
?? it mite be an error (semantic?) if not already in the symbol table 
*/
		    /*
		     * If the last component of a dot expression is not in
		     * the symbol table, enter it.  If we are looking into
		     * a vc object, then we enter an id_sym.  Otherwise,
		     * we are (probably) looking into a object, in which case
		     * we enter an attribute_sym.
		     */
		    if ((cus_sym_type)cus_get_prop (the_sym, type_prop) == vc_sym)
			the_sym = cus_add_sym ($3, id_sym, &new, WITH_CONTEXT);
		    else
			the_sym = cus_add_sym ($3, attribute_sym, &new, WITH_CONTEXT);
		    }
		(void) add_to_tail (the_list, the_sym);
		$$ = the_list;
		(void) free_node ($3);
		}
	;

constant:	xINTEGERNUMBER
		{
		the_sym = cus_gensym_no_add ("integer", id_sym, &i_seed);
		cus_put_prop (the_sym, is_constant_prop, 1);
		cus_put_prop (the_sym, current_type_prop, idd_integer);
		cus_put_prop (the_sym, current_value_prop, $1);
		$$ = the_sym;
		}

	|	xREALNUMBER
		{
		the_sym = cus_gensym_no_add ("real", id_sym, &r_seed);
		cus_put_prop (the_sym, is_constant_prop, 1);
		cus_put_prop (the_sym, current_type_prop, idd_real);
		cus_put_prop (the_sym, current_value_prop, $1);
		$$ = the_sym;
		}

	|	boolean_constant

	|	xQUOTE
		{
		$$ = cus_string_constant ($1);
		}

	|	xUNDEFINED
		{
		$$ = car_undefined_symbol;
		}

	|	xNULL
		{
		$$ = car_null_symbol;
		}
	;

boolean_constant:	xTRUE
		{
		$$ = car_true_symbol;
		}

	|	xFALSE
		{
		$$ = car_false_symbol;
		}
	;

function_call:	xNAME '(' opt_expressions ')'
		{
		LIST the_list = make_list ();
		process_lineno (yylineno, temp_lineno);
		(void) add_to_tail_val (the_list, car_function_call_symbol,
		    temp_lineno);
/*
?? might not want to an add , if already there (preloaded built-ins)
*/
		(void) add_to_tail (the_list, cus_add_sym ($1, id_sym, &new,
		    NO_CONTEXT));
		if ($3 != NULL)
		    (void) append_list (the_list, $3);
		$$ = the_list;
		(void) free_node($1);
		}

opt_expressions:	/* empty string */
		{
		$$ = NULL;
		}

	|	expressions
	;

expressions:	expression
		{
		LIST the_list = make_list ();
		(void) add_to_tail (the_list, $1);
		$$ = the_list;
		}

	|	expressions ',' expression
		{
		(void) add_to_tail ($1, $3);
		}
	;

logical_operator:	xAND
		{
		$$ = car_and_symbol;
		}

	|	'&'
		{
		$$ = car_and_symbol;
		}

	|	xOR
		{
		$$ = car_or_symbol;
		}

	|	'|'
		{
		$$ = car_or_symbol;
		}
	;

equality_operator:	'='
		{
		$$ = car_equal_symbol;
		}

	|	xNOTEQUAL
		{
		$$ = car_notequal_symbol;
		}
	;

relational_operator:	'<'
		{
		$$ = car_lessthan_symbol;
		}

	|	xLESSEQUAL
		{
		$$ = car_lessequal_symbol;
		}

	|	'>'
		{
		$$ = car_greaterthan_symbol;
		}

	|	xGREATEREQUAL
		{
		$$ = car_greaterequal_symbol;
		}
	;

addition_operator:	'+'
		{
		$$ = car_plus_symbol;
		}

	|	'-'
		{
		$$ = car_binary_minus_symbol;
		}
	;

multiplication_operator:	'*'
		{
		$$ = car_star_symbol;
		}

	|	'/'
		{
		$$ = car_slash_symbol;
		}
	;

unary_operator:	xNOT
		{
		$$ = car_not_symbol;
		}

	|	'-'
		{
		$$ = car_unary_minus_symbol;
		}
	;

opt_eos:	/* empty string */

	|	eos
	;

eos:		semi opt_comments
		{
		$$ = $2;
		}

semi:		';'

	|	semi ';'
	;

opt_comments:	/* empty string */
		{
		$$ = NULL;
		}

	|	comments
	;

comments:	xCOMMENT
		{
		LIST the_list = make_list ();
		(void) add_to_tail (the_list, car_comment_symbol);
		the_sym = cus_gensym_no_add ("comment", comment_sym, &c_seed);
		cus_put_prop (the_sym, is_constant_prop, 1);
		cus_put_prop (the_sym, current_type_prop, idd_string);
		cus_put_prop (the_sym, current_value_prop, $1);
		process_lineno (yylineno, temp_lineno);
		(void) add_to_tail_val (the_list, the_sym, temp_lineno);
		$$ = the_list;
		}

	|	comments xCOMMENT
		{
		LIST the_list = make_list ();
                process_lineno (yylineno, temp_lineno);
		(void) add_to_tail_val (the_list, car_comment_symbol,
		    temp_lineno);
		the_sym = cus_gensym_no_add ("comment", comment_sym, &c_seed);
		cus_put_prop (the_sym, is_constant_prop, 1);
		cus_put_prop (the_sym, current_type_prop, idd_string);
		cus_put_prop (the_sym, current_value_prop, $2);
		(void) add_to_tail (the_list, the_sym);
		(void) append_list ($1, the_list);
		}
	;

	%%
