static char rcsid[] = "$Header: /u/04c/mnt/integ/source/compiler/src/RCS/csc_const.c,v 1.6 91/01/09 14:05:49 dvk Exp $";

/*--------------------------------------------------------------------------*\
|
|  Name:  Compiler constant-table generation - csc_const
|
|  Description: 
|     Maintains a table of constant values used by Slang and the DM
|
|  Component of
|     Slang Compiler
|
|  Related files
|
|  Project
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author: 
|     Daniel V. Klein
|     dvk@sei.cmu.edu 
|
|  Change History
|     January 10, 1990 : Module created
$Log:	csc_const.c,v $
 * Revision 1.6  91/01/09  14:05:49  dvk
 * Completed implementation of real constants
 * 
 * Revision 1.5  91/01/02  17:48:22  dvk
 * Sweeping changes throughout compiler for new interpreted output
 * 
 * Revision 1.4  90/09/17  19:18:04  dvk
 * Changed some printf's to fputs's
 * 
 * Revision 1.3  90/02/14  22:13:48  ejh
 * fixed include structure
 * 
 * Revision 1.2  90/01/19  16:31:48  dvk
 * Fixed to use make_node, not malloc.  Also added dob_id_NULL in code
 * generator routine.
 * 
 * Revision 1.1  90/01/12  14:59:36  dvk
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	memoryPack

#include	<stdio.h>
#include	<memoryPack.h>
#include	<dri.h>

static int int_const_idx = 0;
static int real_const_idx = 0;

struct csc_int_const {
    struct csc_int_const *left, *right;
    long value;
    int idx;
    };

struct csc_real_const {
    struct csc_real_const *left, *right;
    double value;
    int idx;
    };

static struct csc_int_const *int_const_root;
static struct csc_real_const *real_const_root;

/*
 * make_int_const
 *
 * Internal routine used to create a integer constant node.  As a side effect,
 * increments int_const_idx.
 */

static struct csc_int_const *make_int_const(val)
long val;
{
    struct csc_int_const *node;

    node = (struct csc_int_const *) make_node (sizeof (struct csc_int_const));
    node->left = node->right = (struct csc_int_const *) 0;
    node->value = val;
    node->idx = int_const_idx++;
    return (node);
}

/*
 * csc_gen_int_const
 *
 * Search through the integer constant tree and see if the requested constant
 * value exists.  If so, return it's index.  If it does not exist, then
 * create a new constant entry, and return the new index number.
 */
int csc_gen_int_const(val)
int val;
{
    struct csc_int_const *node;

    if (int_const_root == (struct csc_int_const *)0) {
	int_const_root = make_int_const(val);
	return(int_const_root->idx);
	}
    else
	node = int_const_root;

    while (1) {
	if (val < node->value) {
	    if (node->left != (struct csc_int_const *)0)
		node = node->left;
	    else {
		node->left = make_int_const(val);
		return (node->left->idx);
		}
	    }
	else if (val > node->value) {
	    if (node->right != (struct csc_int_const *)0)
		node = node->right;
	    else {
		node->right = make_int_const(val);
		return (node->right->idx);
		}
	    }
	else  /* (val == node->value) */
	    return (node->idx);
	}
}

/*
 * csc_pic_recurse
 *
 * This is the recurseive descent routine used by csc_dump_int_consts
 */
static void csc_pic_recurse(stream, node)
FILE *stream;
struct csc_int_const *node;
{
    static dri_int iconst;

    if (node == (struct csc_int_const *)0)
	return;
    csc_pic_recurse(stream, node->left);
    iconst.idx = node->idx;
    iconst.val = node->value;
    fwrite (&iconst, sizeof (dri_int), 1, stream);
    csc_pic_recurse(stream, node->right);
}

/*
 * csc_dump_int_consts
 *
 * Print the integer constant tree using recursive descent to the named stream.
 */
int csc_dump_int_consts(stream)
FILE *stream;
{
    if (int_const_idx != 0)
	csc_pic_recurse(stream, int_const_root);
    return int_const_idx;
}

/*
 * make_real_const
 *
 * Internal routine used to create a real constant node.  As a side effect,
 * increments real_const_idx.
 */

static struct csc_real_const *make_real_const(val)
double val;
{
    struct csc_real_const *node;

    node = (struct csc_real_const *) make_node (sizeof (struct csc_real_const));
    node->left = node->right = (struct csc_real_const *) 0;
    node->value = val;
    node->idx = real_const_idx++;
    return (node);
}

/*
 * csc_gen_real_const
 *
 * Search through the real constant tree and see if the requested constant
 * value exists.  If so, return it's index.  If it does not exist, then
 * create a new constant entry, and return the new index number.
 */
int csc_gen_real_const(val)
double val;
{
    struct csc_real_const *node;

    if (real_const_root == (struct csc_real_const *)0) {
	real_const_root = make_real_const(val);
	return(real_const_root->idx);
	}
    else
	node = real_const_root;

    while (1) {
	if (val < node->value) {
	    if (node->left != (struct csc_real_const *)0)
		node = node->left;
	    else {
		node->left = make_real_const(val);
		return (node->left->idx);
		}
	    }
	else if (val > node->value) {
	    if (node->right != (struct csc_real_const *)0)
		node = node->right;
	    else {
		node->right = make_real_const(val);
		return (node->right->idx);
		}
	    }
	else  /* (val == node->value) */
	    return (node->idx);
	}
}

/*
 * csc_prc_recurse
 *
 * This is the recurseive descent routine used by csc_dump_real_consts
 */
static void csc_prc_recurse(stream, node)
FILE *stream;
struct csc_real_const *node;
{
    static dri_real rconst;

    if (node == (struct csc_real_const *)0)
	return;
    csc_prc_recurse(stream, node->left);
    rconst.idx = node->idx;
    rconst.val = node->value;
    fwrite (&rconst, sizeof (dri_real), 1, stream);
    csc_prc_recurse(stream, node->right);
}

/*
 * csc_dump_real_consts
 *
 * Print the real constant tree using recursive descent to the named stream.
 */
int csc_dump_real_consts(stream)
FILE *stream;
{
    if (real_const_idx != 0)
	csc_prc_recurse(stream, real_const_root);
    return real_const_idx;
}
