/*
$Header: /u/04c/mnt/integ/source/de/include/RCS/eat.h,v 1.9 90/10/23 13:53:09 little Exp $
*/

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor Attribute Abstraction (eat)
|
|  Description:  
|     provides operaions on the attribute abstract type
|
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|

$Log:	eat.h,v $
 * Revision 1.9  90/10/23  13:53:09  little
 * change in some comments
 * 
 * Revision 1.8  90/07/05  15:14:04  little
 * more structure and visualizer parts integration
 * 
 * Revision 1.7  90/06/14  13:57:46  little
 * more changes to integrate visulization and structure editors
 * 
 * Revision 1.6  90/06/06  18:07:10  little
 * parital update to support combination of visual and structural editor
 * 
 * Revision 1.5  90/05/21  13:42:46  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.4  89/12/15  20:26:53  bmc
 * Added binary read routine and associated definitions.
 * 
 * Revision 1.3  89/12/15  13:55:40  rcs
 * je ne se pas.
 * 
 * Revision 1.2  89/12/12  12:37:41  bmc
 * memoryPack now conditionally defined
 * 
 * Revision 1.1  89/11/29  10:01:23  bmc
 * Initial revision
 * 

|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/



#ifndef eat
#define eat

#include "serpent.h"


/*--------------------------------------------------------------------------*\
|  Global Variable: eat_debug
|
|  Description:
|     Debug attribute for the abstraction.
|
\* Definition:                                                              */
#ifdef eat_main
#define EXTERN
  int eat_debug = 1;
#else
  extern int eat_debug;
#define EXTERN extern
#endif


/*--------------------------------------------------------------------------*\
|  Abstract Type: eat_attribute
|
|  Description:
|    The eat_attribute abstract type provides a attribute data instance.
|
\* Definition:                                                              */

typedef private eat_attribute;

			/* the eob.h include must be after the definition of
			   eat_attribute
			*/
#include "eob.h"
#ifndef memoryPack
#define memoryPack
#endif
#include "hashPack.h"

EXTERN HASH eat_template_hash;

typedef struct {int old_template;
               eat_attribute new_template;
               }
    eat_template_hash_entry;


#define eat_no_attribute ((eat_attribute) NULL)
#define EAT_NO_ATTRIBUTE eat_no_attribute

#define eat_no_more_attributes ((eat_attribute) 1)
#define EAT_NO_MORE_ATTRIBUTES eat_no_more_attributes


/*--------------------------------------------------------------------------*\
|  Routine: eat_create_attribute
| 
|  Description:
|     Creates a new attribute
|
\* Syntax:                                                                  */

	eat_attribute eat_create_attribute(
	  /* att_name	: in string */
		);

/*
|  Parameters:
|     att_name - the name the attribute is to take on
|
|  Returns:
|      the newly created attribute.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_destroy_attribute
| 
|  Description:  
|     Destroys a specified attribute and any sub-components.
|
\* Syntax:                                                                  */

	void eat_destroy_attribute (
	  /* the_attribute		: in out eat_attribute */
	);

/*
|  Parameters:
|     the_attribute - attribute to destroy.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eat_duplicate_attribute
| 
|  Description:  
|     Creates a duplicate of an attribute
|
\* Syntax:                                                                  */

	eat_attribute eat_duplicate_attribute (
	  /* the_attribute	: in out eat_attribute */
	);

/*
|  Parameters:
|     the_attribute - attribute to duplicate
|
|  Returns:
|     a copy of the attribute, or EAT_NO_ATTRIBUTE if an error occurred
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_get_parent
| 
|  Description:  
|     fetches the squirreled away parent for a attribute
|
\* Syntax:                                                                  */

	eob_object eat_get_parent (
	  /* the_attribute : in eat_attribute */
	);

/*
|  Parameters:
|     the_attribute - attribute in question
|
|  Returns:
|      the parent associated with the attribute
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_set_snippet
| 
|  Description:  
|     squirrels away a snippet for a attribute
|
\* Syntax:                                                                  */

	void eat_set_snippet (
	  /* the_attribute : in out eat_attribute */
	  /* the_snip: in buffer */
	);

/*
|  Parameters:
|     the_attribute - attribute in question
|     the_snip - snippet to be saved
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_get_snippet
| 
|  Description:  
|     fetches the squirreled away snippet for a attribute
|
\* Syntax:                                                                  */

	buffer eat_get_snippet (
	  /* the_attribute : in eat_attribute */
	);

/*
|  Parameters:
|     the_attribute - attribute in question
|
|  Returns:
|      the snippet associated with the attribute
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_set_comment
| 
|  Description:  
|     squirrels away comment for a attribute
|
\* Syntax:                                                                  */

	void eat_set_comment (
	  /* the_attribute : in out eat_attribute */
	  /* the_comment: in buffer */
	);

/*
|  Parameters:
|     the_attribute - attribute in question
|     the_comment - comment to be saved
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_get_comment
| 
|  Description:  
|     fetches the squirreled away comment for a attribute
|
\* Syntax:                                                                  */

	buffer eat_get_comment (
	  /* the_attribute : in eat_attribute */
	);

/*
|  Parameters:
|     the_attribute - attribute in question
|
|  Returns:
|      the comment associated with the attribute
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
| Routine: eat_set_name
| 
|  Description:  
|     squirrels away name for a att
|
\* Syntax:                                                                  */

	void eat_set_name (
	  /* the_attribute : in out eat_attribute */
	  /* the_name: in string */
	);

/*
|  Parameters:
|     the_attribute - attribute in question
|     the_name - name to be saved
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_get_name
| 
|  Description:  
|     fetches the squirreled away name for a att
|
\* Syntax:                                                                  */

	string eat_get_name (
	  /* the_attribute : in eat_attribute */
	);

/*
|  Parameters:
|     the_attribute - attribute in question
|
|  Returns:
|      the name associated with the att
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_parse_attribute_2_ed
|
|  Description:
|     convert the parser version of the attribute to editor version
\*--------------------------------------------------------------------------*/
eat_attribute eat_parse_attribute_2_ed (
/* attribute */
	);
/*
cus_sym attribute;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_set_comment
|
|  Description:
|     squirrels away a comment for a attribute
\*--------------------------------------------------------------------------*/
void eat_set_comment (
	/* the_attribute, the_comment */
		);
/*
eat_attribute the_attribute;
buffer the_comment;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_get_comment
|
|  Description:
|      fetches the squirreled away comment for a attribute
\*--------------------------------------------------------------------------*/
buffer eat_get_comment (
	/* the_attribute */
		);
/*
eat_attribute the_attribute;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_src_write
| 
|  Description:  
|     write a attribute to a source stream
|
\* Syntax:                                                                  */

	void eat_src_write (
	  /* the_attribute : in eat_attribute */
	  /* stream: in out FILE * */
	);

/*
|  Parameters:
|     the_attribute - attribute in question
|     stream - the stream on which to write the att
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eat_bin_write
| 
|  Description:  
|     write a attribute to a binary stream
|
\* Syntax:                                                                  */

	void eat_bin_write (
	  /* the_attribute : in eat_attribute */
	  /* stream: in out FILE * */
	);

/*
|  Parameters:
|     the_attribute - attribute in question
|     stream - the stream on which to write the att
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eat_bin_read
| 
|  Description:  
|     read an attribute from a binary stream
|
\* Syntax:                                                                  */

	eat_attribute eat_bin_read (
	  /* stream: in out FILE * */
	);

/*
|  Parameters:
|     stream - the stream on which to read the att
|
|  Returns:
|     EAT_NO_MORE_ATTRIBUTES - encountered end of list (and pushed it back
|                              on the input token stream)
|     EAT_NO_ATTRIBUTE       - bad or nonexistent attribute (status set)
|     otherwise              - decoded attribute
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_inst_2_temp
|
|  Description:
|     moves an instance value to a template
\*--------------------------------------------------------------------------*/
void eat_inst_2_temp (
/* the_att, name, value */
	);

/*
eat_attribute the_att;
string name;
string value;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_set_instance
|
|  Description:
|     Sets the instance of an attribute
\*--------------------------------------------------------------------------*/
void eat_set_instance (
/* the_att, att_instance */
	);
/*
eat_attribute the_att;
object_instance_attribute_sdd *att_instance;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_get_instance
|
|  Description:
|     Gets the instance of an attribute, if there is one.
\*--------------------------------------------------------------------------*/
object_instance_attribute_sdd *eat_get_instance (
/* the_att */
	);
/*
eat_attribute the_att;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_init_template_hash
|
|  Description:
|     initializes the att template hash table used when reading in dialogue
|       binaries
\*--------------------------------------------------------------------------*/
void eat_init_template_hash ();


/*--------------------------------------------------------------------------*\
|  Routine: eat_free_template_hash
|
|  Description:
|    destroys the att template hash table used when reading in dialogue
|       binaries
\*--------------------------------------------------------------------------*/
void eat_free_template_hash ();


/*--------------------------------------------------------------------------*\
|  Routine: eat_set_sd_element
|
|  Description:
|     Sets the sd_element of an attribute
\*--------------------------------------------------------------------------*/
void eat_set_sd_element (
/* the_att, att_sd_element */
	);
/*
eat_attribute the_att;
id_type att_sd_element;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_get_sd_element
|
|  Description:
|     Gets the sd_element of an attribute, if there is one.
\*--------------------------------------------------------------------------*/
id_type eat_get_sd_element (
/* the_att */
	);
/*
eat_attribute the_att;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_clear_sd_element
|
|  Description:
|     Clears the sd_element of an attribute
\*--------------------------------------------------------------------------*/
void eat_clear_sd_element (
/* the_att */
	);
/*
eat_attribute the_att;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_set_ghost
|
|  Description:
|     Sets the ghost of an attribute
\*--------------------------------------------------------------------------*/
void eat_set_ghost (
/* the_att */
	);
/*
eat_attribute the_att;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_get_ghost
|
|  Description:
|     Gets the ghost of an attribute, if there is one.
\*--------------------------------------------------------------------------*/
boolean eat_get_ghost (
/* the_att */
	);
/*
eat_attribute the_att;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_clear_ghost
|
|  Description:
|     Clears the ghost of an attribute
\*--------------------------------------------------------------------------*/
void eat_clear_ghost (
/* the_att */
	);
/*
eat_attribute the_att;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_get_snip_constant_p
|
|  Description:
|     Gets the snippet_constant_p_prop of an attribute 
\*--------------------------------------------------------------------------*/
boolean eat_get_snip_constant_p (
/* the_att */
	);
/*
eat_attribute the_att;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_snippet_2_constant
|
|  Description:
|    moves an att's snippet value to a string, ensures it is a constant
\*--------------------------------------------------------------------------*/
string eat_snippet_2_constant (
/* the_att, the_string, the_length */
	);
/*
eat_attribute the_att;
string the_string;
int the_length;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_temp_from_inst
|
|  Description:
|    creates an att template based on an instance
\*--------------------------------------------------------------------------*/
eat_attribute eat_temp_from_inst (
/* att_inst, parent_obj */
	);
/*
object_instance_attribute_sdd *att_instance;
eob_object parent_obj;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eat_inst_from_temp
|
|  Description:
|    creates an att instance based on a template (possiblly createing the
|       template)
\*--------------------------------------------------------------------------*/
object_instance_attribute_sdd *eat_inst_from_temp (
/* att_name, the_obj */
	);
/*
string att_name;
eob_object the_obj;
*/

#endif


