/*
$Header: /u/04c/mnt/integ/source/de/include/RCS/ebi.h,v 1.12 90/08/29 18:31:35 little Exp $
*/

/*--------------------------------------------------------------------------*\
|   
|  Name: Dialogue Editor Binary Input/Output Routines
|
|  Description:  
|     The binary input/output utility provides dialogue binary input/output
|     functions.
|

$Log:	ebi.h,v $
 * Revision 1.12  90/08/29  18:31:35  little
 * add integer read/write
 * 
 * Revision 1.11  90/07/05  15:14:46  little
 * more structure and visualizer parts integration
 * 
 * Revision 1.10  90/05/21  13:49:03  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.9  90/02/08  21:55:31  bmc
 * Added EBI_OBJ_INSTANCE_SELECTOR, EBI_OBJ_ATTR_INSTANCE_SELECTOR,
 * ebi_write_boolean() and ebi_read_boolean().
 * 
 * Revision 1.8  89/12/20  11:46:10  bmc
 * Added ebi_is_buffer_selector() routine.
 * 
 * Revision 1.7  89/12/18  14:44:27  bmc
 * Added parameter checking to ebi_write_token_char().
 * 
 * Revision 1.6  89/12/15  16:04:07  bmc
 * added push_token and push_token_char.
 * 
 * Revision 1.5  89/12/15  14:17:43  bmc
 * made some enhancements
 * 
 * Revision 1.4  89/12/14  15:30:47  little
 * fixed ebi_write_token
 * 
 * Revision 1.3  89/12/14  13:32:23  bmc
 * Added a few routine defs, changed a couple more to macros, and added some
 * more constants.
 * 
 * Revision 1.2  89/12/13  16:59:29  bmc
 * Changed to handle new binary format.
 * 
 * Revision 1.1  89/11/29  10:02:31  bmc
 * Initial revision
 * 

|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/
#ifndef ebi
#define ebi

/*--------------------------------------------------------------------------*\
|  Includes: ebi
|
\*                                                                          */

#include <stdio.h>		/* Unix standard I/O defs		    */
#include "global.h"             /* Global type defs and constants           */
#include "serpent.h"		/* General Serpent types.		    */

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Constants: Selectors
|
|  Description:
|     The following characters identify the various pieces of a stored
|     binary dialogue.  WARNING: These values must be unique!
|
\* Definition:                                                              */

    /*
    ** The following sub-group of constants represent binary elements
    ** that are stored as aggregate 'records' and are processed by individual
    ** I/O routines.
    */
#define EBI_DIALOGUE_SELECTOR		'D'
#define EBI_VC_SELECTOR			'V'
#define EBI_OBJECT_SELECTOR		'O'
#define EBI_VARIABLE_SELECTOR		'X'
#define EBI_ATTRIBUTE_SELECTOR		'A'
#define EBI_METHOD_SELECTOR		'M'
#define EBI_OBJ_INSTANCE_SELECTOR	'o'
#define EBI_OBJ_ATTR_INSTANCE_SELECTOR	'a'

    /*
    ** The following sub-group of constants represent binary elements that
    ** are stored as conceptual buffers and are ultimately processed by the
    ** ebi buffer primitives.  IF YOU ADD ONE BE SURE TO UPDATE THE 
    ** ebi_buffer_selectors ARRAY BELOW!
    */
#define EBI_SNIPPET_SELECTOR		'S'
#define EBI_ON_DESTROY_SELECTOR		'I'
#define EBI_ON_CREATE_SELECTOR		'T'
#define EBI_CREATION_COND_SELECTOR	'C'
#define EBI_COMMENTS_SELECTOR		'K'
#define EBI_EXTERNS_SELECTOR		'E'
#define EBI_MACROS_SELECTOR		'm'
#define EBI_SHARED_DATA_SELECTOR	's'
#define EBI_OBJECT_INSTANCE_VALUE_SELECTOR 'v'

    /*
    ** EBI_INVALID_SELECTOR defines a general 'illegal' constant.
    */

#define EBI_INVALID_SELECTOR		'?'

#ifdef ebi_main

    /*
    ** The following string provides a searchable array of buffer selector
    ** characters.  It's defined here so it'll be close to the constants.
    ** It's only used in `ebi.c'.
    */

    static char ebi_buffer_selectors [] =
	{
	EBI_SNIPPET_SELECTOR,
	EBI_ON_DESTROY_SELECTOR,
	EBI_ON_CREATE_SELECTOR,
	EBI_CREATION_COND_SELECTOR,
	EBI_COMMENTS_SELECTOR,
	EBI_EXTERNS_SELECTOR,
	EBI_MACROS_SELECTOR,
	EBI_SHARED_DATA_SELECTOR,
	EBI_OBJECT_INSTANCE_VALUE_SELECTOR,

	'\0' /* to end the string properly */
	};

#endif ebi_main

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Constants: Special Tokens
|
|  Description:
|     The following characters represent special tokens.  Normally, they
|     are only used by routines and macros defined by this abstraction.
|
\* Definition:                                                              */

#define EBI_OPEN_RECORD			'('
#define EBI_CLOSE_RECORD		')'
#define EBI_BAD_CHAR			'\0'

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Type: ebi_buffer
|
|  Description:
|     Provides an envelope for the Serpent buffer type, mainly because
|     dialogue binary files must store information about a buffer, as well
|     as the buffer itself.
|
\* Definition:                                                              */

    typedef struct
	{
	char selector;
	buffer actual_buffer;
	}
    ebi_buffer;

/*
|  Components:
|     selector      - The type of information stored in the buffer.  This value
|                     is restricted to one of the buffer-related selector
|                     constants defined above.
|     actual_buffer - The actual buffer.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Type: ebi_open_type
|
|  Description:
|     Provides two values for opening a dialogue binary file.  Used with
|     the ebi_open_dialogue_file macro.
|
\* Definition:                                                              */

    typedef enum
	{
	ebi_open_read,
	ebi_open_write
	}
    ebi_open_type;

/*
|  Legal Values:
|     ebi_open_read  - Open binary dialogue file for reading.
|     ebi_open_write - Open binary dialogue file for writing.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Global Variable: ebi_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef ebi_main
  int ebi_debug = 1;
#else
  extern int ebi_debug;
#endif

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Macro: ebi_open_dialogue_file
|
|  Description:
|     Opens a binary dialogue file, hiding the actual details of how the
|     file is opened.
|
|     WARNING TO MAINTAINERS OF THIS MODULE: The "w+" on the write fopen
|     call is correct!  The file must be opened in write/update mode so that
|     ebi_write_checksum can seek backwards to calculate the checksum!
|
\* Definition:                                                              */

#define ebi_open_dialogue_file(file_name, open_type) \
    (open_type == ebi_open_read ? \
	fopen (file_name, "r") : fopen (file_name, "w+"))

/*
|  Parameters:
|     file_name - Stream to which to write token.
|     open_type - How to open the file.  This parameter must be one of the
|                 values in the ebi_open_type enumeration.
|
|  Returns:
|     An open stream of type `FILE *', or `(FILE *) NULL' on error.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Macro: ebi_write_token_char
|
|  Description:
|     Writes a single-character token to an open dialogue binary file.
|
\* Definition:                                                              */

#define ebi_write_token_char(stream, token) \
    ( \
    (token == NULL) || (stream == (FILE *) NULL) ? \
	false : \
	(fputc (token, stream) != EOF) && (fputc (' ', stream) != EOF) \
    )

/*
|  Parameters:
|     stream - Stream to which to write token.
|     token  - The character representing the token to be written.
|
|  Returns:
|     FALSE - An error occurred while attempting to write the token.
|     TRUE  - Write succeeded.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Macro: ebi_write_token
| 
|  Description:
|     Writes a token to an open dialogue binary file.
|
\* Definition:                                                              */

#define ebi_write_token(stream, token) \
    ( (fputs (token, stream) != EOF) && (fputc (' ', stream) != EOF) )

/*
|  Parameters:
|     stream - Stream to which to write token.
|     token  - The token to be written.
|
|  Returns:
|     FALSE - An error occurred while attempting to write the token.
|     TRUE  - Write succeeded.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Macros:
|     boolean ebi_write_open_record
|     boolean ebi_write_close_record
|
|  Description:
|     These macros write the open/close record tokens.
|
\* Definition:                                                              */

#define ebi_write_open_record(stream) \
    ebi_write_token_char (stream, EBI_OPEN_RECORD)

#define ebi_write_close_record(stream) \
    ((fputc (' ', stream) != EOF) && \
      ebi_write_token_char (stream, EBI_CLOSE_RECORD) && \
     (fputc ('\n', stream) != EOF))

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_token
| 
|  Description:
|     Reads a token from an open dialogue binary file.
|
\* Syntax:                                                                  */

    string ebi_read_token
	(
    /*	stream	: in FILE *	*/
	);

/*
|  Parameters:
|     stream - Stream from which to read token.
|
|  Returns:
|     A string representing the next valid token, or NULL in case of error.
|     Storage for the string is allocated via make_node().  Freeing the
|     storage is the responsibility of the calling routine.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_push_token
| 
|  Description:
|     Pushes a token back onto the input stream.  WARNING: You can only push
|     a single token back.  No matter what stream you read the next token
|     from, it'll check the internal token register before actually reading
|     the stream.  So if you push a token read from a particular stream,
|     you should be sure to read the next token from the same stream.  (This
|     should be thought of as a feature, rather than a bug.)
|
|     ebi_push_token_char also uses the same internal token register, so
|     calling this routine immediately after calling ebi_push_token_char is
|     an error.
|
\* Syntax:                                                                  */

    boolean ebi_push_token
	(
    /*	token : in string	*/
	);

/*
|  Parameters:
|     token - The token to be pushed back.  The token is assumed to be
|             stored in memory allocated via make_node().  The calling routine
|             should assume that it is giving up responsibility for the
|             memory.
|
|  Returns:
|     FALSE if a token has already been pushed back, TRUE otherwise.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_token_char
| 
|  Description:
|     Reads a single-character token from an open dialogue binary file.
|
\* Syntax:                                                                  */

    char ebi_read_token_char
	(
    /*	stream	: in FILE *	*/
	);

/*
|  Parameters:
|     stream - Stream from which to read token.
|
|  Returns:
|    
|     EBI_BAD_CHAR if (1) there were no remaining tokens, or (2) the next
|     token was not a single character.  Otherwise, the character representing
|     the token is returned.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_push_token_char
| 
|  Description:
|     Pushes a token character back onto the input stream.   See the WARNING
|     associated with the ebi_push_token routine.
|
|     ebi_push_token also uses the same internal token register, so calling
|     this routine immediately after calling ebi_push_token_char is an error.
|
\* Syntax:                                                                  */

    boolean ebi_push_token_char
	(
    /*	token : in char	*/
	);

/*
|  Parameters:
|     token - The character to be pushed back.
|
|  Returns:
|     FALSE on error, TRUE otherwise.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_boolean
| 
|  Description:
|     Reads a stored boolean quantity from an open dialogue binary file.
|
\* Syntax:                                                                  */

    boolean ebi_read_boolean
	(
    /*	stream		  : in FILE *	*/
    /*  value		  : in pointer to boolean */
	);

/*
|  Parameters:
|     stream            - Stream from which to read boolean.
|     value		- Address of variable to receive boolean value.
|
|  Returns:
|     FALSE - An I/O error occurred.
|     TRUE  - Everything worked, and *value contains the boolean read in.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_write_boolean
| 
|  Description:
|     Writes a boolean value to an open dialogue binary file.
|
\* Syntax:                                                                  */

    boolean ebi_write_boolean
	(
    /*	stream		: in FILE *	*/
    /*  value		: in boolean */
	);

/*
|  Parameters:
|     stream            - Stream from which to read buffer.
|     value		- The boolean value to be written.
|
|  Returns:
|     FALSE - An I/O error occurred.
|     TRUE  - Write was successful.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_buffer
| 
|  Description:
|     Reads an ebi_buffer from an open dialogue binary file.
|
\* Syntax:                                                                  */

    ebi_buffer ebi_read_buffer
	(
    /*	stream		  : in FILE *	*/
	);

/*
|  Parameters:
|     stream            - Stream from which to read buffer.
|
|  Returns:
|     The buffer that was read.  If an error occurred, the `selector'
|     component is set to EBI_INVALID_SELECTOR.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_write_buffer
| 
|  Description:
|     Writes an ebi_buffer to an open dialogue binary file.
|
\* Syntax:                                                                  */

    boolean ebi_write_buffer
	(
    /*	stream		: in FILE *	*/
    /*	the_buffer	: in ebi_buffer	*/
	);

/*
|  Parameters:
|     stream      - Stream to which to write buffer.
|     the_buffer  - The ebi_buffer to be written.
|
|  Returns:
|     FALSE - An error occurred while attempting to write the buffer.
|     TRUE  - Write succeeded.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_is_buffer_selector
| 
|  Description:
|     Determines whether a selector character represents a buffer selector.
|
\* Syntax:                                                                  */

    boolean ebi_is_buffer_selector
	(
    /*	selector	: in char	*/
	);

/*
|  Parameters:
|     selector  - The character to test.
|
|  Returns:
|     FALSE - The character is not a buffer selector.
|     TRUE  - The character is a buffer selector.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_magic_number
| 
|  Description:
|     Reads an ebi magic number from an open dialogue binary file.
|
\* Syntax:                                                                  */

    boolean ebi_read_magic_number
	(
    /*	stream	: in FILE *	*/
	);

/*
|  Parameters:
|     stream - Stream from which to read magic_number.
|
|  Returns:
|     FALSE - A bad magic number was found in the binary file, which means
|             the contents of the file are suspect.
|     TRUE  - Valid magic number decoded, which means the file probably
|             represents a dialogue binary file.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_write_magic_number
| 
|  Description:
|     Writes an ebi magic number to an open dialogue binary file.
|
\* Syntax:                                                                  */

    boolean ebi_write_magic_number
	(
    /*	stream		: in FILE *	*/
	);

/*
|  Parameters:
|     stream - Stream to which to write magic_number.
|
|  Returns:
|     FALSE - An error occurred while attempting to write the magic number.
|     TRUE  - Write succeeded.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_calc_checksum
| 
|  Description:
|     Calculates the checksum for an open dialogue binary file.  Assumes the
|     file is at the point where calculation is to begin.
|
\* Syntax:                                                                  */

    int ebi_calc_checksum
	(
    /*	stream	: in FILE *	*/
    /*	bytes	: in int	*/
	);

/*
|  Parameters:
|     stream - Stream for which to calculate checksum.
|     bytes  - The number of bytes to incorporate into the checksum.  A value
|              of 0 indicates that calculation should terminate on end of file.
|
|  Returns:
|     -1 if an error occurs; otherwise, returns the calculated checksum.
|     The file pointer is repositioned `bytes' bytes ahead, or at EOF if
|     a premature EOF is encountered.
|
|  Status:
|     end_of_file - set on premature eof.
|
|     This routine does NOT initialize the status value to `ok'.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_checksum
| 
|  Description:
|     Reads the checksum from an open dialogue binary file.  Assumes the
|     file is positioned at the beginning of the checksum record.
|
\* Syntax:                                                                  */

    boolean ebi_read_checksum
	(
    /*	stream	: in FILE *	*/
	);

/*
|  Parameters:
|     stream - Stream from which to read checksum.
|
|  Returns:
|     FALSE - A bad checksum was found in the binary file, which means
|             the contents of the file probably been corrupted.
|     TRUE  - Valid checksum decoded, which means the file probably
|             represents an uncorrupted dialogue binary file.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ebi_write_checksum
| 
|  Description:
|     Re-scans the specified stream, calculating a checksum, and writes the
|     checksum to the end of the stream.
|
|     WARNING: For this routine to work properly, the file must have been
|     opened for writing with the ebi_open_dialogue_file macro!
|
\* Syntax:                                                                  */

    boolean ebi_write_checksum
	(
    /*	stream		: in FILE *	*/
	);

/*
|  Parameters:
|     stream - Stream to which to write checksum.
|
|  Returns:
|     FALSE - An error occurred while attempting to write the checksum.
|     TRUE  - Write succeeded.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: ebi_write_string
|
|  Description:
|     writes a string a stream
\*--------------------------------------------------------------------------*/
boolean ebi_write_string (
/* the_stream, the_string, the_selector */
	);
/*
FILE *stream;
string the_string;
char the_selector;
*/


/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_string
|
|  Description:
|     reads a string from a stream
\*--------------------------------------------------------------------------*/
string ebi_read_string (
/* the_stream, the_selector */
	);
/*
FILE *stream;
char selector;
*/


/*--------------------------------------------------------------------------*\
|  Routine: ebi_write_integer
| 
|  Description:
|     writes a integer a stream
\*--------------------------------------------------------------------------*/
boolean ebi_write_integer (
/* the_stream, the_integer */
	) ;
/*
FILE *the_stream;
int the_integer;
*/


/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_integer
| 
|  Description:
|     reads a integer a stream
\*--------------------------------------------------------------------------*/
int ebi_read_integer (
/* the_stream */
	) ;
/*
FILE *the_stream;
*/

#endif

