/* $Header: /u/04c/mnt/integ/source/de/include/RCS/ecd.h,v 1.5 91/03/20 23:39:57 little Exp $ */
/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Command Definitions (ecd)
|
|  Description:  
|     The dialogue editor command definitions contains the definition
|     of the commands supported by the DEA.  This component accesses
|     retained data stored in the dialogue editor command interpreter
|     component.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|
|     $Log:	ecd.h,v $
 * Revision 1.5  91/03/20  23:39:57  little
 * add definition for ecd._new_dialogue function
 * 
 * Revision 1.4  90/05/21  13:49:22  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.3  89/12/20  14:47:03  rcs
 * integration testing
 * 
 * Revision 1.2  89/12/12  07:50:20  rcs
 * added headers for dialogue i/o routines
 * 
 * Revision 1.1  89/11/28  18:17:20  rcs
 * Initial revision
 * 
|
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef ecd
#define ecd
/*--------------------------------------------------------------------------*\
|  Includes: ecd
|
|  Description:
|     Necessary includes for the ecd abstraction.
|
\* Definition:                                                              */

#include "edg.h"                /* dialogue editor dialogue abstraction     */

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Global Variable: ecd_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef ecd_main
  int ecd_debug = 1;
#else
  extern int ecd_debug;
#endif

/*
\*--------------------------------------------------------------------------*/
#if 0
/*--------------------------------------------------------------------------*\
|  Type: ecd_commands
|
|  Description:
|    The ecd_commands enumerates the commands supported by the dialogue
|    editor application.
|
\* Definition:                                                              */

	typedef enum commands {
	  ecd_no_command_cmd,
	  ecd_load_dialogue_cmd,
	  ecd_save_dialogue_cmd,
	  ecd_unload_dialogue_cmd,
	  ecd_quit_dialogue_cmd,
	  ecd_insert_vc_cmd,
	  ecd_copy_vc_cmd,
	  ecd_move_vc_cmd,
	  ecd_delete_vc_cmd,
	  ecd_get_subvcs_cmd,
	  ecd_get_objects_cmd,
	  ecd_insert_object_cmd,
	  ecd_copy_object_cmd,
	  ecd_move_object_cmd,
	  ecd_delete_object_cmd,
	  ecd_get_attributes_cmd,
	  ecd_get_methods_cmd,
	  ecd_insert_attribute_cmd,
	  ecd_delete_attribute_cmd,
	  ecd_object_instance_insert_cmd,
	  ecd_object_instance_copy_cmd,
	  ecd_object_instance_move_cmd,
	  ecd_object_instance_delete_cmd,
	  ecd_attribute_instance_insert_cmd,
	  ecd_attribute_instance_delete_cmd,
	  ecd_get_topmost_vcs_cmd,
	  ecd_file_cmd
	} ecd_commands;

/*
\*--------------------------------------------------------------------------*/
#endif 
/*--------------------------------------------------------------------------*\
|  Type: ecd_parameters
|
|  Description:
|    The ecd_parameters enumerates the parameters required by the commands.
|
\* Definition:                                                              */

	typedef enum parameters {
	  ecd_no_parameter		=0,
	  ecd_parent_vc			=1,
	  ecd_dialogue			=2,
	  ecd_vc			=3,
	  ecd_to_vc			=4
	} ecd_parameters;

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine:  ecd_load_dialogue
|
|  Description:
|     The load command is used to load a dialogue into the application.
|
\* Syntax:                                                                  */

        edg_dialogue ecd_load_dialogue(
          /* command : in command_sdd *         */
          /* transaction: in transaction_type   */
        );

/*
|  Parameters:
|     command -  the actual command sdd.
|     transaction - outgoing transaction.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine:  ecd_save_dialogue
|
|  Description:
|     The save command is used to write a dialogue to permanent store.  The
|     dialogue name is saved in the dialogue shared data element.
|
\* Syntax:                                                                  */

        void ecd_save_dialogue(
          /* command : in command_sdd *         */
          /* transaction: in transaction_type   */
        );

/*
|  Parameters:
|     command -  the actual command sdd.
|     transaction - outgoing transaction.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine:  ecd_unload_dialogue
|
|  Description:
|     The unload command is used remove a specified dialogue from both shared
|     data and application memory.
|
\* Syntax:                                                                  */

        void ecd_unload_dialogue(
          /* command : in command_sdd *         */
          /* transaction: in transaction_type   */
        );

/*
|  Parameters:
|     command -  the actual command sdd.
|     transaction - outgoing transaction.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine:  ecd_import_dialogue
|
|  Description:
|     The ecd_import_dialogue routine is used to input a dialogue from
|     Slang source.
|
\* Syntax:                                                                  */

        edg_dialogue ecd_import_dialogue(
          /* command : in command_sdd *         */
          /* transaction: in transaction_type   */
        );

/*
|  Parameters:
|     command -  the actual command sdd.
|     transaction - outgoing transaction.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine:  ecd_export_dialogue
|
|  Description:
|     The ecd_export_dialogue routine is used to save a dialogue as Slang.
|
\* Syntax:                                                                  */

        void ecd_export_dialogue(
          /* command : in command_sdd *         */
          /* transaction: in transaction_type   */
        );

/*
|  Parameters:
|     command -  the actual command sdd.
|     transaction - outgoing transaction.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine:  ecd_get_dialogue_technologies
|
|  Description:
|     The unload command is used remove a specified dialogue from both shared
|     data and application memory.
|
\* Syntax:                                                                  */

        void ecd_get_dialogue_technologies(
          /* command : in command_sdd *         */
          /* transaction: in transaction_type   */
        );

/*
|  Parameters:
|     command -  the actual command sdd.
|     transaction - outgoing transaction.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine:  ecd_set_technology
|
|  Description:
|     The ecd_set_technology command is to select and deselect technologies
|     for incluscion in a dialogue.
|
\* Syntax:                                                                  */

        void ecd_set_technology(
          /* command : in command_sdd *         */
          /* transaction: in transaction_type   */
        );

/*
|  Parameters:
|     command -  the actual command sdd.
|     transaction - outgoing transaction.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine:  ecd_new_dialogue
|
|  Description:
|     creates a new, empty dialogue
\*--------------------------------------------------------------------------*/
edg_dialogue ecd_new_dialogue (
/* command, transaction */
	);
/*
command_sdd *command;
transaction_type transaction;
*/

#endif

