/*
$Header: /usr/users/little/serpent/mips/de/include/RCS/edg.h,v 1.8 1991/05/07 15:23:00 little Exp $
*/

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Dialogue Abstraction (edg)
|
|  Description:  
|     The dialogue editor dialogue abstraction provides operations on the
|     dialouge abstract type.  These operations include create, destroy, 
|     and various get and modify operations.
|
|  Component of:
|     Dialogue Editor Layer.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|

$Log: edg.h,v $
 * Revision 1.8  1991/05/07  15:23:00  little
 * change definition of returned value from edg_add_technology
 *
 * Revision 1.7  1990/08/29  18:32:07  little
 * add the counter to the dialogue sd record
 *
 * Revision 1.6  90/07/05  15:15:14  little
 * more structure and visualizer parts integration
 * 
 * Revision 1.5  90/05/21  13:50:36  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.4  90/02/14  13:04:38  bmc
 * Now handles object instances.
 * 
 * Revision 1.3  89/12/18  14:22:08  little
 * fixed some shit that was wrong
 * 
 * Revision 1.2  89/12/11  16:12:05  little
 * fix definittions for  bin read/write
 * 
 * add definitions for bin & src file exists routines
 * 
 * Revision 1.1  89/11/29  10:02:59  bmc
 * Initial revision
 * 

|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef edg
#define edg


#define EDG_CREATE_TEMPLATE TRUE
#define EDG_DO_NOT_CREATE_TEMPLATE FALSE


#include "global.h"             /* Global type defs and constants           */
#include "serpent.h"		/* General Serpent defs			    */
#include "se.h"			/* Dialogue editor shared data defs	    */

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Global Variable: edg_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef edg_main
  int edg_debug = 1;
#else
  extern int edg_debug;
#endif

/*
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Abstract Type: edg_dialogue
|
|  Description:
|    The edg_dialogue abstract type provides a handle to a dialogue data 
|    instance.
|
\* Definition:                                                              */

	typedef private edg_dialogue;

#define edg_no_dialogue		((edg_dialogue) NULL)
#define EDG_NO_DIALOGUE		edg_no_dialogue


/*--------------------------------------------------------------------------*\
|  Other Abstraction Includes: Definitions for other editor abstraction
|     packages.
|
|  WARNING: These include directives must appear after the type definition
|  for edg_dialogue!
\*                                                                          */

#include "evc.h"                /* editor vc abstraction		    */


/*--------------------------------------------------------------------------*\
|  Routine: edg_create_dialogue
| 
|  Description:  
|     creates an new empty dialogue and returns it
|
\* Syntax:                                                                  */

	edg_dialogue edg_create_dialogue ();

/*
|  Parameters:
|     None.
|
|  Returns:
|     The newly created dialogue.  NULL is returned if it could not be
|     created.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: edg_destroy_dialogue
| 
|  Description:  
|     Destroys all storage associated with a previously-created dialogue.
|
\* Syntax:                                                                  */

	void edg_destroy_dialogue
	    (
	/*  dialogue    : in out edg_dialogue  */
	/*  transaction : in transaction_type */
	    );

/*
|  Parameters:
|     dialogue    - The dialogue to be destroyed.
|     transaction - Outgoing transaction, used to inform dialogue layer
|                   of any removed shared data.
|
|  Returns:
|     Nothing.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_is_dialogue
| 
|  Description:  
|     Returns true if the specified handle is a valid dialogue handle and
!     false if it is not.
|
\* Syntax:                                                                  */

	boolean edg_is_dialogue
	    (
	/*  dialogue	: in edg_dialogue	*/
	    );

/*
|  Parameters:
|     dialogue - The dialogue handle to be tested.
|
|  Returns:
|     True if dialogue handle is valid, false if it is not.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_set_vc
| 
|  Description:  
|     Set the dialogue VC (i.e., Eve) to the specified VC.
|
\* Syntax:                                                                  */

	void edg_set_vc
	    (
	/*  dialogue	: in out edg_dialogue	*/
	/*  vc		: in evc_vc		*/
	    );

/*
|  Parameters:
|     dialogue - The dialogue to be changed.
|     vc       - The VC to become Eve.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_get_vc
| 
|  Description:  
|     Returns the current dialogue VC (i.e., Eve).
|
\* Syntax:                                                                  */

	evc_vc edg_get_vc
	    (
	/*  dialogue	: in edg_dialogue */
	    );

/*
|  Parameters:
|     dialogue - The dialogue to be queried.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_get_file_name
| 
|  Description:  
|     Gets the file name associated with a dialogue.
|
\* Syntax:                                                                  */

	string edg_get_file_name
	    (
	/*  dialogue	: in edg_dialogue	*/
	    );

/*
|  Parameters:
|     dialogue - The dialogue to be queried.
|
|  Returns:
|      The file name, or NULL if none was previously set.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_set_file_name
| 
|  Description:  
|     Sets the file name associated with a dialogue.
|
\* Syntax:                                                                  */

	void edg_set_file_name
	    (
	/*  dialogue	: in out edg_dialogue	*/
	/*  file_name 	: in string		*/
	    );

/*
|  Parameters:
|     dialogue  - The dialogue to be changed.
|     file_name - The new file name.
|
|  Returns:
|     Nothing.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_get_prologue
| 
|  Description:  
|     Gets the prologue associated with a dialogue.
|
\* Syntax:                                                                  */

	buffer edg_get_prologue
	    (
	/*  dialogue	: in edg_dialogue	*/
	    );

/*
|  Parameters:
|     dialogue - The dialogue to be queried.
|
|  Returns:
|      A buffer containing the prologue.  If no prologue was previously set,
|      the buffer's contents are UNDEFINED.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_set_prologue
| 
|  Description:  
|     Sets the prologue associated with a dialogue.
|
\* Syntax:                                                                  */

	void edg_set_prologue (
	  /* dialogue	: in out edg_dialogue	*/
	  /* prologue	: in buffer		*/
	);

/*
|  Parameters:
|     dialogue - The dialogue to be changed.
|     prologue - The new prologue.
|
|  Returns:
|     Nothing.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: edg_get_macros
| 
|  Description:  
|     Gets the macros section of a dialogue, if any.
|
\* Syntax:                                                                  */

	buffer edg_get_macros
	    (
	/*  dialogue	: in edg_dialogue	*/
	    );

/*
|  Parameters:
|     dialogue - The dialogue to be queried.
|
|  Returns:
|      Returns the macros section in a buffer.  If there is no such section,
|      the returned buffer is UNDEFINED.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_set_macros
| 
|  Description:  
|     Sets the macros section of a dialogue.
|
\* Syntax:                                                                  */

	void edg_set_macros
	    (
	/*  dialogue	: in out edg_dialogue	*/
	/*  macros 	: in buffer 		*/
	    );

/*
|  Parameters:
|     dialogue - The dialogue to be changed.
|     macros   - The new macros section.
|
|  Returns:
|     Nothing.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_get_externs
| 
|  Description:  
|     Gets the externs section of a dialogue, if any.
|
\* Syntax:                                                                  */

	buffer edg_get_externs
	    (
	/*  dialogue	: in edg_dialogue	*/
	    );

/*
|  Parameters:
|     dialogue - The dialogue to be queried.
|
|  Returns:
|      Returns the externs section in a buffer.  If there is no such section,
|      the returned buffer is UNDEFINED.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_set_externs
| 
|  Description:  
|     Sets the externs section of a dialogue.
|
\* Syntax:                                                                  */

	void edg_set_externs
	    (
	/*  dialogue	: in out edg_dialogue	*/
	/*  externs 	: in buffer		*/
	    );

/*
|  Parameters:
|     dialogue - The dialogue to be changed.
|     externs  - The new externs.
|
|  Returns:
|     Nothing.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_get_shared_data
| 
|  Description:  
|     Gets the shared_data associated with a dialogue.
|
\* Syntax:                                                                  */

	buffer edg_get_shared_data
	    (
	/*  dialogue	: in edg_dialogue	*/
	    );

/*
|  Parameters:
|     dialogue - The dialogue to be queried.
|
|  Returns:
|      Returns the shared data section in a buffer.  If there is no such
|      section the returned buffer is UNDEFINED.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_set_shared_data
| 
|  Description:  
|     Sets the shared_data associated with a dialogue.
|
\* Syntax:                                                                  */

	void edg_set_shared_data
	    (
	/*  dialogue	: in out edg_dialogue	*/
	/*  shared_data : in buffer		*/
	    );

/*
|  Parameters:
|     dialogue    - The dialogue to be changed.
|     shared_data - The new shared data.
|
|  Returns:
|     Nothing.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_add_technology
| 
|  Description:  
|     Adds a technology to the technology list associated with a dialogue.
|
\* Syntax:                                                                  */

	string edg_add_technology
	    (
	/*  dialogue	: in out edg_dialogue	*/
	/*  technology	: in string		*/
	    );

/*
|  Parameters:
|     dialogue   - The dialogue to be changed.
|     technology - The technology to be added.
|
|  Returns:
|     Nothing.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_remove_technology
| 
|  Description:  
|     Removes a technology from the technology list associated with a dialogue.
|
\* Syntax:                                                                  */

	void edg_remove_technology
	    (
	/*  dialogue	: in out edg_dialogue	*/
	/*  technology	: in string		*/
	    );

/*
|  Parameters:
|     dialogue   - The dialogue to be changed.
|     technology - The technology to be removed.
|
|  Returns:
|     Nothing.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_is_a_technology
| 
|  Description:  
|     Determines if a technology is on the technology list associated with
|     a dialogue.
|
\* Syntax:                                                                  */

	boolean edg_is_a_technology
	    (
	/* dialogue	: in out edg_dialogue	*/
	/* technology	: in string		*/
	    );

/*
|  Parameters:
|     dialogue   - Dialogue to be queried.
|     technology - The technology to check.
|
|  Returns:
|     TRUE if technology is on the list, FALSE otherwise.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_src_read
|
|  Description:
|     Reads a source dialogue from a file;
|
\* Syntax:                                                                  */

	edg_dialogue edg_src_read
	    (
	/* name	: in string 	 */
	    );

/*
|  Parameters:
|     name - name of dialogue to read in
|
|  Returns:
|      Dialogue read from file, or EDG_NO_DIALOGUE if an error occurred.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_src_write
|
|  Description:
|     Writes a source dialogue to a file;
|
\* Syntax:                                                                  */

	void edg_src_write
	    (
	/*  dialogue	: in edg_dialogue	*/
	/*  name	: in string  		*/
	    );

/*
|  Parameters:
|     dialogue - the dialogue to be written out
|     name     - partial name of output file
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_bin_read
| 
|  Description:  
|     Reads a binary dialogue from a stream and creates a dialogue record.
|
\* Syntax:                                                                  */

	edg_dialogue edg_bin_read
	    (
	/*  name	: in string */
	/*  transaction : in transaction_type */
	    );

/*
|  Parameters:
|     name        - name of dialogue to read, is part of file name
|     transaction - outgoing transaction, used to create any shared data
|                   that is required during the read operation
|
|  Returns:
|      Dialogue read from stream.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_bin_write
| 
|  Description:  
|     Writes a dialogue out to a stream.
|
\* Syntax:                                                                  */

	void edg_bin_write
	    (
	/*  dialogue	: in edg_dialogue */
	/*  name	: in string	  */
	    );

/*
|  Parameters:
|     dialogue - Dialogue to be written 
|     name     - partial name of output file
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_src_file_exists
|
|  Description:
|     Determines if a dialogue source file exists
|
\* Syntax:                                                                  */

	boolean edg_src_file_exists
	    (
	/*  name : in string 	*/
	    );
/*
|  Parameters:
|     name - Name of source file to check.  This name should not have an
|            extension.
|
|  Returns:
|     FALSE - File does not exist
|     TRUE  - File does exist
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_bin_file_exists
|
|  Description:
|     Determines if a dialogue source file exists
|
\* Syntax:                                                                  */

	boolean edg_bin_file_exists
	    (
	/*  name : in string 	*/
	    );
/*
|  Parameters:
|     name - Name of source file to check.  This name should not have an
|            extension.
|
|  Returns:
|     FALSE - File does not exist
|     TRUE  - File does exist
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_set_count
| 
|  Description:  
|     Sets the counter associated with a dialogue.
\*--------------------------------------------------------------------------*/
void edg_set_count (
/* dialogue, the_count */
	);
/*
edg_dialogue dialogue;
int the_count;
*/


/*--------------------------------------------------------------------------*\
|  Routine: edg_get_count
| 
|  Description:  
|     Gets the counter associated with a dialogue.
\*--------------------------------------------------------------------------*/
int edg_get_count (
/* dialogue */
	);
/*
edg_dialogue dialogue;
*/

#endif
