/* $Header: /u/04c/mnt/integ/source/de/include/RCS/ehf.h,v 1.3 90/05/21 13:51:21 little Exp $ */
/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor Hash Functions (ehf)
|
|  Description:  
|     Constains the routines to support the various hash tables used
|     in the dialogue editor.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|
|  Change History:
|
|     $Log:	ehf.h,v $
 * Revision 1.3  90/05/21  13:51:21  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.2  89/12/12  07:52:55  rcs
 * fixed bugs found during integration testing
 * 
 * Revision 1.1  89/11/28  18:24:10  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef ehf

#include "hashPack.h"

/*--------------------------------------------------------------------------*\
|  Global Variable: ehf_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef ehf_main
  int ehf_debug = 1;
#else
  extern int ehf_debug;
#endif

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Constants: parameter_table, id_table, elem_table
|
|  Description:
|    The following constants are used in creating the hashtables and
|    calculating indicies.
|
\* Definition:                                                              */

#define PARM_HASH_SIZE 11
#define MAX_HASH 257

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Externs: parameter_table, id_table, elem_table
|
|  Description:
|    The ehf_map data structure provices a mapping between shared data 
|    elehfnts and actual dialogue elehfnts.
|
\* Definition:                                                              */

	extern HASH parameter_table;
	extern HASH id_table;
	extern HASH elem_table;

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ehf_match_id
| 
|  Description:  
|     Routine to compare an identifier with the id in a shared data record.
|
\* Syntax:                                                                  */

	int ehf_match_id(
	  /* id : iid_id_type */
	  /* map: ehf_map */
	);

/*
|  Parameters:
|      id - The id of the shared data elehfnt
|      map - The record mapping the id to the elehfnt.
|
|  Returns:
|      True if match, false otherwise.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ehf_hash_id
| 
|  Description:  
|     Internal function which will convert an id into an index
|     in the array of hash lists.
|
\* Syntax:                                                                  */

	int ehf_hash_id(
	  /* id :iid_id_type */
        );

/*
|  Parameters:
|      id - The id to be converted.
|
|  Returns:
|      The index.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ehf_match_elem
| 
|  Description:  
|     Routine to compare a dialogue element with the dialogue element 
|     in the mapping record.
|
\* Syntax:                                                                  */

	int ehf_match_elem(
	  /* elem : char *  */
	  /* map  : eme_map */
	);

/*
|  Parameters:
|      elem - A dialogue element.
|      map - The record mapping the id to the element.
|
|  Returns:
|      True if match, false otherwise.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ehf_hash_elem
| 
|  Description:  
|     Internal function which will convert an elem into indicies
|     in the array of hash lists.
|
\* Syntax:                                                                  */

	int ehf_hash_elem(
	  /* elem : char *  */

	);

/*
|  Parameters:
|      elem - The element to be converted.
|
|  Returns:
|      The index.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ehf_match_parm
| 
|  Description:  
|     Routine to compare the parameter type with parameter type stored in 
|     the parameter shared data record.
|
\* Syntax:                                                                  */

	int ehf_match_parm(
	  /* id : id_type */
	  /* parameter : parameter_sdd *  */
	);

/*
|  Parameters:
|      elem - A dialogue element.
|      map - The record mapping the id to the element.
|
|  Returns:
|      True if match, false otherwise.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: ehf_hash_parm
| 
|  Description:  
|     Internal function which will convert an parameter_type into indicies
|     in the array of hash lists.
|
\* Syntax:                                                                  */

	int ehf_hash_parm(
	  /* id : iid_id_type */
	);

/*
|  Parameters:
|      id - The id to be converted.
|
|  Returns:
|      The index.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/
#define ehf
#endif

