/* $Header: /u/04c/mnt/integ/source/de/include/RCS/eit.h,v 1.9 1991/06/27 17:11:03 dvk Exp $ */
/*--------------------------------------------------------------------------*\
|   
|  Name:  ID Table (eit.h)
|
|  Description:  
|     The ID table is used by the dialogue editor to maintain a mapping
|     between the names of object instances and the ID of the realization
|     object used to represent it.
|
|  Component of:
|     Dialoge Editor Dialogue.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log: eit.h,v $
 * Revision 1.9  1991/06/27  17:11:03  dvk
 * No changes - put 'em in externs/arith.ext
 *
 * Revision 1.8  1991/04/11  18:10:06  rcs
 * *** empty log message ***
 *
 * Revision 1.7  90/11/29  10:05:09  rcs
 * no change
 * 
 * Revision 1.6  90/09/21  00:08:49  rcs
 * fixed parent instantiation order problem
 * 
 * Revision 1.5  90/08/22  18:29:24  rcs
 * *** empty log message ***
 * 
 * Revision 1.4  90/05/21  13:52:01  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.3  90/03/14  10:30:32  rcs
 * turned off debug print
 * 
 * Revision 1.2  90/03/01  11:00:09  rcs
 * changed names of routines and added get_type
 * 
 * Revision 1.1  89/11/29  09:57:04  trm
 * Initial revision
 * 
 * Revision 1.1  89/09/28  12:06:32  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/
#ifndef eit
#define eit

#include "iid.h"
#include "serpent.h"            /* serpent                                  */


/*--------------------------------------------------------------------------*\
|  Global Variable: eit_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef eit_main
  int eit_debug = 1;
#else
  extern int eit_debug;
#endif

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: create_id_table
| 
|  Description:
|     Creates an create_id_table to be used within the dialogue editor to
|     provide a mapping between object instance names and realization ids.
|
\* Syntax:                                                                  */

	void create_id_table();

/*
|  Parameters:
|     None.
|
|  Returns:
|      void.
|
|  Statuses:
|     ok, out_of_memory.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: add_object
|
|  Description:
|     The add_object routine adds a new name to the hash table.
|
\* Syntax:                                                                  */

	void add_object(
	  /* name: in string	*/
        );

/*
|  Parameters:
|     name - the name of the object instance to add to the table.
|
|  Returns:
|      void.
|
|  Statuses:
|     ok, out_of_memory.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: remove_object
|
|  Description:
|     The remove_object routine removes a name from the hash table.
|
\* Syntax:                                                                  */

	void remove_object(
	  /* name: in string	*/
        );

/*
|  Parameters:
|     name - the name of the object instance to remove from the table.
|
|  Returns:
|      void.
|
|  Statuses:
|     ok, out_of_memory.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: set_id
|
|  Description:
|     The set_id routine sets the id for the given name to the specified id.
|
\* Syntax:                                                                  */

	void set_id(
	  /* name: in string	*/
	  /* id: in idd_id_type */
	);

/*
|  Parameters:
|     name - the name of the object instance.
|     id - the new value of id.
|
|  Returns:
|      void.
|
|  Statuses:
|     ok, not_found, out_of_memory.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: get_id
|
|  Description:
|     The get_id routine returns the id associated with the given object
|     instance name.
|
\* Syntax:                                                                  */

	iid_id_type get_id(
          /* name: in string   */
	);

/*
|  Parameters:
|     name - name of the object instance we need the id for.
|
|  Returns:
|      the current id associated with the speicified name.
|
|  Statuses:
|     ok, not_found.
\*--------------------------------------------------------------------------*/

#if 0
/*--------------------------------------------------------------------------*\
|  Routine: get_type
|
|  Description:
|     The get_type routine returns the type associated with the given object
|     instance name.
|
\* Syntax:                                                                  */

	string get_type(
          /* name: in string   */
	);

/*
|  Parameters:
|     name - name of the object instance we need the type for.
|
|  Returns:
|      the current type associated with the speicified name.
|
|  Statuses:
|     ok, not_found.
\*--------------------------------------------------------------------------*/
#endif

/*--------------------------------------------------------------------------*\
|  Routine: object_exists
|
|  Description:
|     The object_exists routine returns true if the specified object instance
|     name is in the hash and false if it is not.
|
\* Syntax:                                                                  */

	boolean object_exists(
          /* name: in string   */
	);

/*
|  Parameters:
|     name - name of the object instance.
|
|  Returns:
|      true if the name is in the hash table;  false otherwise.
|
|  Statuses:
|     ok.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: object_exists_and_not_null
|
|  Description:
|     The object_exists routine returns true if the name is in the table
|     and the id is not null and false if it is not.
|
\* Syntax:                                                                  */

	boolean object_exists_and_not_null(
          /* name: in string   */
	);

/*
|  Parameters:
|     name - name of the object instance.
|
|  Returns:
|      true if the name is in the hash table;  false otherwise.
|
|  Statuses:
|     ok.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: destroy_id_table
|
|  Description:
|     The destroy_id_table routine is used to destroy the id hash
|     table and all the nodes.
|
\* Syntax:                                                                  */

	void destroy_id_table();

/*
|  Parameters:
|     None.
|
|  Returns:
|      void.
|
|  Statuses:
|     ok, out_of_memory.
\*--------------------------------------------------------------------------*/

#endif

