/*
$Header: /u/04c/mnt/integ/source/de/include/RCS/eme.h,v 1.7 90/09/11 08:37:25 little Exp $
*/

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor Attribute Abstraction (eme)
|
|  Description:  
|     provides operaions on the method abstract type
|
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|

$Log:	eme.h,v $
 * Revision 1.7  90/09/11  08:37:25  little
 * add definition of eme_duplicate_method
 * 
 * Revision 1.6  90/05/21  13:52:41  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.5  89/12/15  20:28:42  bmc
 * Added binary read routine and associated definitions.
 * 
 * Revision 1.4  89/12/15  18:49:44  rcs
 * added set and get comment headers
 * 
 * Revision 1.3  89/12/15  15:13:27  bmc
 * no changes
 * 
 * Revision 1.2  89/12/15  11:57:29  little
 * no changes
 * 
 * Revision 1.1  89/11/29  10:04:16  bmc
 * Initial revision
 * 

|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/



#ifndef eme
#define eme

#include "serpent.h"


/*--------------------------------------------------------------------------*\
|  Abstract Type: eme_method
|
|  Description:
|    The eme_method abstract type provides a method data instance.
|
\* Definition:                                                              */

typedef private eme_method;


			/* the eob.h include must be after the definition of
			   eme_method
			*/
#include "eob.h"


/*--------------------------------------------------------------------------*\
|  Global Variable: eme_debug
|
|  Description:
|     Debug method for the abstraction.
|
\* Definition:                                                              */
#ifdef eme_main
  int eme_debug = 1;
#else
  extern int eme_debug;
#endif


#define eme_no_method ((eme_method) NULL)
#define EME_NO_METHOD eme_no_method

#define eme_no_more_methods ((eme_method) 1)
#define EME_NO_MORE_METHODS eme_no_more_methods


/*--------------------------------------------------------------------------*\
|  Routine: eme_create_method
| 
|  Description:
|     Creates a new method
|
\* Syntax:                                                                  */

	eme_method eme_create_method(
	  /* name	: in string */
		);

/*
|  Parameters:
|     name - the name the method is to take on
|
|  Returns:
|      the newly created method.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eme_destroy_method
| 
|  Description:  
|     Destroys a specified method and any sub-components.
|
\* Syntax:                                                                  */

	void eme_destroy_method (
	  /* the_method		: in out eme_method */
	);

/*
|  Parameters:
|     the_method - method to destroy.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eme_set_parent
| 
|  Description:  
|     fetches the squirreled away parent for a method
|
\* Syntax:                                                                  */

	void eme_set_parent (
	  /* the_method : in out eme_method */
          /* the_parent: in eob_object */
	);

/*
|  Parameters:
|     the_method - method in question
|     the_parent - new parent object for the_method
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eme_get_parent
| 
|  Description:  
|     fetches the squirreled away parent for a method
|
\* Syntax:                                                                  */

	eob_object eme_get_parent (
	  /* the_method : in eme_method */
	);

/*
|  Parameters:
|     the_method - method in question
|
|  Returns:
|      the parent associated with the method
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eme_set_snippet
| 
|  Description:  
|     squirrels away a snippet for a method
|
\* Syntax:                                                                  */

	void eme_set_snippet (
	  /* the_method : in out eme_method */
	  /* the_snip: in buffer */
	);

/*
|  Parameters:
|     the_method - method in question
|     the_snip - snippet to be saved
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eme_get_snippet
| 
|  Description:  
|     fetches the squirreled away snippet for a method
|
\* Syntax:                                                                  */

	buffer eme_get_snippet (
	  /* the_method : in eme_method */
	);

/*
|  Parameters:
|     the_method - method in question
|
|  Returns:
|      the snippet associated with the method
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eme_set_comment
|
|  Description:
|     squirrels away comment for a method
|
\* Syntax:                                                                  */

        void eme_set_comment (
          /* the_method : in out eme_method */
          /* the_comment: in buffer */
        );

/*
|  Parameters:
|     the_method - method in question
|     the_comment - comment to be saved
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eme_get_comment
|
|  Description:
|     fetches the squirreled away comment for a method
|
\* Syntax:                                                                  */

        buffer eme_get_comment (
          /* the_method : in eme_method */
        );

/*
|  Parameters:
|     the_method - method in question
|
|  Returns:
|      the comment associated with the method
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
| Routine: eme_set_name
| 
|  Description:  
|     squirrels away name for a meth
|
\* Syntax:                                                                  */

	void eme_set_name (
	  /* the_method : in out eme_method */
	  /* the_name: in string */
	);

/*
|  Parameters:
|     the_method - method in question
|     the_name - name to be saved
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eme_get_name
| 
|  Description:  
|     fetches the squirreled away name for a meth
|
\* Syntax:                                                                  */

	string eme_get_name (
	  /* the_method : in eme_method */
	);

/*
|  Parameters:
|     the_method - method in question
|
|  Returns:
|      the name associated with the meth
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eme_parse_method_2_ed
|
|  Description:
|     convert the parser version of the method to editor version
\*--------------------------------------------------------------------------*/
eme_method eme_parse_method_2_ed (
/* method */
	);
/*
cus_sym method;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eme_src_write
| 
|  Description:  
|     write a method to a source stream
|
\* Syntax:                                                                  */

	void eme_src_write (
	  /* the_method : in eme_method */
	  /* stream: in out FILE * */
	);

/*
|  Parameters:
|     the_method - method in question
|     stream - the stream on which to write the meth
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eme_bin_write
| 
|  Description:  
|     write a method to a binary stream
|
\* Syntax:                                                                  */

	void eme_bin_write (
	  /* the_method : in eme_method */
	  /* stream: in out FILE * */
	);

/*
|  Parameters:
|     the_method - method in question
|     stream - the stream on which to write the method
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eme_bin_read
| 
|  Description:  
|     read a method to a binary stream
|
\* Syntax:                                                                  */

	eme_method eme_bin_read
	    (
	/*  stream: in out FILE * */
	    );

/*
|  Parameters:
|     stream - the stream on which to read the method
|
|  Returns: 
|  
|     EME_NO_METHOD       - No method or illegal method.
|     EME_NO_MORE_METHODS - End of method list.  (i.e., encountered
|                           EBI_CLOSE_RECORD character, which was pushed
|                           back onto the token stream).
|     Otherwise           - returns the decoded method.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eme_duplicate_method
|
|  Description:
|     duplicates the specified method
\*--------------------------------------------------------------------------*/
eme_method eme_duplicate_method (
/* old_method */
	);
/*
eme_method old_method;
*/

#endif


