/* $Header: /u/04c/mnt/integ/source/de/include/RCS/emp.h,v 1.3 90/05/21 13:53:00 little Exp $ */
/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor Mapping Abstraction (emp)
|
|  Description:  
|     The dialogue editor mapping abstraction creates and destroys structures
|     that provide a mapping between shared data elements and actual dialogue
|     elements.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log:	emp.h,v $
 * Revision 1.3  90/05/21  13:53:00  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.2  89/12/12  07:53:21  rcs
 * fixed bugs found during integration testing
 * 
 * Revision 1.1  89/11/28  18:37:03  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef emp

#include "iid.h"                /* status code                              */
#include "serpent.h"		/* serpent interface			    */
#include <hashPack.h>           /* C Toolbox - hashPack                     */


/*--------------------------------------------------------------------------*\
|  Externs: hash tables
|
|  Description:
|     Externs for the hash tables.
|
\* Definition:                                                              */

        extern HASH id_table;
        extern HASH elem_table;

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Global Variable: emp_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef emp_main
  int emp_debug = 1;
#else
  extern int emp_debug;
#endif

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Type: emp_map
|
|  Description:
|    The emp_map data structure provices a mapping between shared data 
|    elements and actual dialogue elements.
|
\* Definition:                                                              */

	typedef struct {	/* this structure provides a mapping        */
	  iid_id_type id;	/*   between the shared data id and the     */
	  char *dialogue_element;	/*   actual dialogue element.	    */
	} emp_map;

/*
|
|  Components:
|      id - The id of the shared data element
|      dialogue_element - A pointer to the corresponding dialogue element.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: emp_base_create_map
| 
|  Description:  
|     Creates a mapping between a shared data element to a dialogue element. 
|
\* Syntax:                                                                  */
  
	void emp_base_create_map(
	  /* id : id_type	*/
	  /* dialogue_element : char *  */
	);

/*
|  Parameters:
|      id - The id of the shared data element
|      dialogue_element - A pointer to the corresponding dialogue element.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: emp_create_map
| 
|  Description:  
|     Creates a mapping between a shared data element to a dialogue element. 
|
|     Note:  Since the dialogue element may be of any type the procedure
|            definition forces a type cast to char *.
|
\* Syntax:                                                                  */
  
#define	emp_create_map(id, dialogue_element)\
  emp_base_create_map(id, (char *) (dialogue_element))

/*
|  Parameters:
|      id - The id of the shared data element
|      dialogue_element - A pointer to the corresponding dialogue element.
|
|  Returns:
|      A handle to the newly created mapping.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: emp_get_de
|
|  Description:
|     Gets the dialogue element associated with a particular id.
| 
\* Syntax:                                                                  */

	char *emp_get_de(
	  /* id : in id_type */
	);

/*
|  Parameters:
|      id - Id to find dialogue element for.
|
|  Returns:
|      Associated ID.
|
|  Statuses:
|      ok.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: emp_get_id
|
|  Description:
|     Gets the id associated with a particular dialogue element.
| 
\* Syntax:                                                                  */

	id_type emp_get_id(
	  /* dialogue_element : in char * */
	);

/*
|  Parameters:
|      id - Id to find dialogue element for.
|
|  Returns:
|      Associated ID.
|
|  Statuses:
|      ok.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: emp_destroy_map_by_id
| 
|  Description:  
|     Destroys a specified map and frees associated storage.
| 
\* Syntax:                                                                  */

	void emp_destroy_map_by_id(
          /* id : in id_type */
	);

/*
|  Parameters:
|      id - Destroy the map corresponding to this id.
|
|  Returns:
|      Void.
|
|  Statuses:
|      ok.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: emp_destroy_map_by_de
| 
|  Description:  
|     Destroys a specified map and frees associated storage.
| 
\* Syntax:                                                                  */

	void emp_destroy_map_by_de(
          /* dialogue_element : in char * */
	);

/*
|  Parameters:
|      dialogue_element - Destroy the map corresponding to this 
|      		dialogue element.
|
|  Returns:
|      Void.
|
|  Statuses:
|      ok.
|
\*--------------------------------------------------------------------------*/
#define emp
#endif

