/*
$Header: /usr/users/little/serpent/mips/de/include/RCS/eob.h,v 1.11 1991/05/14 22:30:17 little Exp $
*/

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Object Abstraction (eob)
|
|  Description:  
|     The dialogue editor object abstraction provides operations on the
|     object abstract type.
|
|  Component of:
|     Dialogue Editor Layer.
|

$Log: eob.h,v $
 * Revision 1.11  1991/05/14  22:30:17  little
 * fix things so the parent att works when instinstiating
 *
 * Revision 1.10  1990/10/23  13:53:54  little
 * add definitions for eob_[sg]et_widget
 *
 * Revision 1.9  90/08/06  16:46:33  little
 * add parameter to eob_destroy_object
 * 
 * Revision 1.8  90/07/05  15:15:40  little
 * more structure and visualizer parts integration
 * 
 * Revision 1.7  90/06/06  18:07:25  little
 * parital update to support combination of visual and structural editor
 * 
 * Revision 1.6  90/05/21  13:53:21  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.5  89/12/18  14:22:30  little
 * fixed some shit that was wrong
 * 
 * Revision 1.4  89/12/12  11:56:04  bmc
 * memoryPack now conditionally defined.
 * 
 * Revision 1.3  89/12/12  10:53:30  bmc
 * Changed eob_get_technology to return the correct type.
 * 
 * Revision 1.2  89/12/04  17:24:01  bmc
 * objects now store technology name and type
 * 
 * Revision 1.1  89/11/29  10:04:57  bmc
 * Initial revision
 * 

|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/
#ifndef eob
#define eob

/*--------------------------------------------------------------------------*\
|  General Purpose Includes: General-purpose definitions 
|
\* Definition:                                                              */

#ifndef memoryPack
#define memoryPack              /* use memoryPack rather than mallocPack    */
#endif  memoryPack

#include "se.h"
#include "glue_intf.h"
#include <listPack.h>           /* C Toolbox - listPack                     */
#include "global.h"             /* Global type defs and constants           */

/*--------------------------------------------------------------------------*\
|  Global Variable: eob_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */
#ifdef EXTERN
#undef EXTERN
#endif
#ifdef eob_main
#define EXTERN
  int eob_debug = 1;
#else
  extern int eob_debug;
#define EXTERN extern
#endif

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Abstract Type: eob_object
|
|  Description:
|    The eob_object abstract type provides a handle to a object data instance.
|
\* Definition:                                                              */

	typedef private eob_object;

#define eob_no_object	((eob_object) NULL)
#define EOB_NO_OBJECT	eob_no_object

#define eob_no_more_objects ((eob_object) 1)
#define EOB_NO_MORE_OBJECTS eob_no_more_objects


/*--------------------------------------------------------------------------*\
|  Other Abstraction Includes: Definitions for other editor abstraction
|     packages.
|
|  WARNING: These include directives must appear after the type definition
|  for eob_object!
\*                                                                          */

#include "eat.h" 		/* dialogue editor attribute abstraction    */
#include "eme.h"
#include "evc.h"

#ifndef memoryPack
#define memoryPack
#endif
#include "hashPack.h"

EXTERN HASH eob_template_hash;


typedef struct {int old_template;
               eob_object new_template;
               }
    eob_template_hash_entry;


/*--------------------------------------------------------------------------*\
|  Routine: eob_create_object
| 
|  Description:
|     Creates a new object record.
|
\* Syntax:                                                                  */

	eob_object eob_create_object
	    (
	/*  object_type : in string,	*/
	/*  technology  : in string	*/
	    );

/*
|  Parameters:
|     object_type - The type of the object (e.g., "command_widget")
|     technology  - The technology the object is associated with (e.g., "saw")
|
|  Returns:
|      The newly created object
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_destroy_object
| 
|  Description:  
|     Destroys a specified object and all sub-components.  
|
|     WARNING: It is the responsibility of the calling routine to ensure
|     that the object has been removed from any parent VC.  This routine
|     destroys the storage for the object, but it does not ensure that no
|     one still refers to that storage.
|
\* Syntax:                                                                  */

	void eob_destroy_object
	    (
	/*  the_object	: in eob_object */
	/*  out_trans	: in transaction_type */
	    );

/*
|  Parameters:
|     the_object - object to destroy.
|     out_trans - transaction in which to put anything needing to be
|		  commincated to the dialogue
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_duplicate_object
|
|  Description:
|     Duplicates the contents of an object, returning the new object.
|
|
\* Syntax:                                                                  */

	eob_object eob_duplicate_object
	    (
	/*  the_object	: in eob_object		*/
	    );

/*
|  Parameters:
|     the_object     - The object to duplicate
|
|  Returns:
|      The new object.  If an error occurs, EOB_NO_OBJECT is returned.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_get_technology
| 
|  Description:
|     Returns the technology associated with an object.
|
\* Syntax:                                                                  */

	string eob_get_technology
	    (
        /*  object	: in eob_object   */
	    );

/*
|  Parameters:
|     object - object to be queried.
|
|  Returns:
|      NULL on error; name of technology otherwise
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eob_get_parent
| 
|  Description:
|     Returns the parent VC for an object.
|
\* Syntax:                                                                  */

	evc_vc eob_get_parent
	    (
        /*  object	: in eob_object   */
	    );

/*
|  Parameters:
|     object - object to be queried.
|
|  Returns:
|      EVC_NO_VC if there is no parent VC; parent VC otherwise.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_set_name
| 
|  Description:
|     Sets an object's name.
|
\* Syntax:                                                                  */

	void eob_set_name
	    (
	/*  the_object	: in out eob_object	*/
	/*  the_name	: in string	*/
	    );

/*
|  Parameters:
|     the_object   - object to be named
|     the_name     - the name to give the object
|
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eob_get_name
| 
|  Description:
|     Gets an object's name.
|
\* Syntax:                                                                  */

	string eob_get_name
	    (
	/*  the_object	: in out eob_object	*/
	    );

/*
|  Parameters:
|     the_object - object for which to get name
|
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eob_set_comments
| 
|  Description:
|     Sets an object's retained comments field.
|
\* Syntax:                                                                  */

	void eob_set_comments
	    (
	/*  the_object		: in out eob_object	*/
	/*  the_comments	: in buffer	*/
	    );

/*
|  Parameters:
|     the_object   - object in which to put the comments
|     the_comments - the comments
|
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eob_get_comments
| 
|  Description:
|     Gets an object's retained comments field.
|
\* Syntax:                                                                  */

	buffer eob_get_comments
	    (
	/*  the_object	: in out eob_object	*/
	    );

/*
|  Parameters:
|     the_object   - object from which to get comments
|     the_comments - the comments
|
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eob_add_attribute
|
|  Description:
|     Adds an attribute to the set of attributes associated with an object
|
\* Syntax:                                                                  */

        void eob_add_attribute
	    (
        /*  the_object	: in out eob_object */
        /*  attribute	: eat_attribute	*/
	    );

/*
|  Parameters:
|     the_object - object to which to add attribute
|     attribute  - The attribute to add.  Note:  The actual attribute, not a
|                  copy is added to the object.  If the calling routine modifies
|                  the original attribute, the attribute in the list will
|                  automatically be updated.
|
|  Returns:
|     Nothing.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eob_remove_attribute
|
|  Description:
|     Removes an attribute from the set of attributes associated with its 
|     parent object.
|
\* Syntax:                                                                  */

        eat_attribute eob_remove_attribute
	    (
        /*  attribute	: eat_attribute	*/
	    );

/*
|  Parameters:
|     attribute - The attribute to remove.  The parent is not specified since 
|                 the attribute can determine who owns it.  WARNING: This
|                 routine assumes that the attribute is parented to an object.
|                 If, for some bizarre reason, the attribute has been parented
|                 to something else, all bets are off.
|
|  Returns:
|     For convenience, this function returns the removed attribute.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_get_attributes
|
|  Description:
|     Returns the set of attributes associated with an object, in the form of a
|     Zimmermann list.
|
\* Syntax:                                                                  */

        LIST eob_get_attributes
	    (
        /*  the_object  : in eob_object     */
	    );

/*
|  Parameters:
|     the_object - object from which to get the set of attributes.
|
|  Returns:
|      A list containing all the attributes, in no particular order.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_add_method
|
|  Description:
|     Adds an method to the set of methods associated with an object
|
\* Syntax:                                                                  */

        void eob_add_method
	    (
        /*  the_object	: in out eob_object */
        /*  method	: eme_method	*/
	    );

/*
|  Parameters:
|     the_object - object to which to add method
|     method  - The method to add.  Note:  The actual method, not a
|                  copy is added to the object.  If the calling routine modifies
|                  the original method, the method in the list will
|                  automatically be updated.
|
|  Returns:
|     Nothing.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_remove_method
|
|  Description:
|     Removes an method from the set of methods associated with its 
|     parent object.
|
\* Syntax:                                                                  */

        eme_method eob_remove_method
	    (
        /*  method	: eme_method	*/
	    );

/*
|  Parameters:
|     method - The method to remove.  The parent is not specified since 
|                 the method can determine who owns it.  WARNING: This
|                 routine assumes that the method is parented to an object.
|                 If, for some bizarre reason, the method has been parented
|                 to something else, all bets are off.
|
|  Returns:
|     For convenience, this function returns the removed method.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_get_methods
|
|  Description:
|     Returns the set of methods associated with an object, in the form of a
|     Zimmermann list.
|
\* Syntax:                                                                  */

        LIST eob_get_methods
	    (
        /*  the_object  : in eob_object     */
	    );

/*
|  Parameters:
|     the_object - object from which to get the set of methods.
|
|  Returns:
|      A list containing all the methods, in no particular order.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_parse_object_2_ed
|
|  Description:
|     convert the parser version of the object to editor version
|
\* Syntax:                                                                  */

	eob_object eob_parse_object_2_ed
	    (
	/*  the_object : in cus_sym	*/
	    );

/*
|  Parameters:
|     the_object - the symbol for the object to convert
|
|  Returns:
|      The converted object.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_get_type
|
|  Description:
|     Sets the type of an object
|
\* Syntax:                                                                  */

	string eob_get_type
	    (
	/*  the_object : in eob_object, */
	    );

/*
|  Parameters:
|     the_object - object to get type for
|
|  Returns:
|      The type, in string form
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_src_write
|
|  Description:
|     write a vc to a source stream
\*--------------------------------------------------------------------------*/
void eob_src_write (
/*
the_obj, stream
*/
	);
/*
eob_object the_obj;
FILE *stream;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_bin_write
|
|  Description:
|     write a vc to a binary stream
\*--------------------------------------------------------------------------*/
void eob_bin_write (
/* the_obj, stream */
	);
/*
eob_object the_obj;
FILE *stream;
*/



/*--------------------------------------------------------------------------*\
|  Routine: eob_bin_read
|
|  Description:
|     read a vc to a binary stream
\*--------------------------------------------------------------------------*/
eob_object eob_bin_read (
/* stream */
	);
/*
FILE *stream;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_set_instance
|
|  Description:
|     Sets the instance of an object
\*--------------------------------------------------------------------------*/
void eob_set_instance (
/* the_object, obj_instance */
	);

/*
eob_object the_object;
object_instance_sdd *obj_instance;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_get_instance
|
|  Description:
|     Gets the instance of an object, if there is one.
\*--------------------------------------------------------------------------*/
object_instance_sdd * eob_get_instance (
/* the_object */
	);

/*
eob_object the_object;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_instantiate
|
|  Description:
|    instantiate an object if it has not already
\*--------------------------------------------------------------------------*/
void eob_instantiate (
/* the_obj, out_transaction */
	);
/*
eob_object the_obj;
transaction_type out_transaction;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_uninstantiate
|
|  Description:
|    uninstantiates an object - remove from screen and take out of the
|       application data base
\*--------------------------------------------------------------------------*/
void eob_uninstantiate (
/* the_obj, out_transaction */
	);
/*
eob_object the_obj;
transaction_type out_transaction;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_remove_instance
|
|  Description:
|    remove an instance - remove from screen but leave in the app data base
\*--------------------------------------------------------------------------*/
void eob_remove_instance (
/* the_obj, out_transaction */
	);
/*
eob_object the_obj;
transaction_type out_transaction;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_get_att_by_name
|
|  Description:
|    get the attribute with the given name
\*--------------------------------------------------------------------------*/
eat_attribute eob_get_att_by_name (
/* the_obj, att_name */
	);
/*
eob_object the_obj;
string att_name;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_set_widget
|
|  Description:
|     Sets the glue widget of an object
\*--------------------------------------------------------------------------*/
void eob_set_widget (
/* the_object, new_widget */
	);
/*
eob_object the_object;
WIDG *new_widget;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_get_widget
|
|  Description:
|     Gets the glue widget of an object, if there is one.
\*--------------------------------------------------------------------------*/
WIDG *eob_get_widget (
/* the_object */
	);
/*
eob_object the_object;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eob_find_parent_object_template
|
|  Description:
|     finds the parent instance for a particular object template in anticipation
|       of instantiating that template
\*--------------------------------------------------------------------------*/
eob_object eob_find_parent_object_template (
/* att_template, obj_template */
	);
/*
eat_attribute att_template;
eob_object obj_template;
*/

#endif

