/*
$Header: /u/04c/mnt/integ/source/de/include/RCS/eoi.h,v 1.4 90/07/05 15:16:05 little Exp $
*/

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Object Instance Abstraction (eoi)
|
|  Description:  
|     The dialogue editor object instance abstraction provides operations
|     on the object instance abstract type.
|
|  Component of:
|     Dialogue Editor Layer.
|

$Log:	eoi.h,v $
 * Revision 1.4  90/07/05  15:16:05  little
 * more structure and visualizer parts integration
 * 
 * Revision 1.3  90/05/21  13:54:51  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.2  90/02/14  13:05:13  bmc
 * Object instances now handled completely.
 * 
 * Revision 1.1  90/01/08  17:08:43  bmc
 * Initial revision
 * 

|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef eoi
#define eoi

/*--------------------------------------------------------------------------*\
|  General Purpose Includes: General-purpose definitions 
|
\* Definition:                                                              */

#ifndef memoryPack
#define memoryPack              /* use memoryPack rather than mallocPack    */
#endif  memoryPack

#include "global.h"             /* Global type defs and constants           */
#include "se.h"			/* editor structure defs		    */

#define EOI_NO_OBJ_INSTANCE	((object_instance_sdd *) NULL)

/*--------------------------------------------------------------------------*\
|  Global Variable: eoi_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef eoi_main
  int eoi_debug = 1;
#else
  extern int eoi_debug;
#endif

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eoi_destroy_object_instance
| 
|  Description:  
|     Destroys a specified object instance and all sub-components, unmapping
|     them and removing them from shared data.
|
\* Syntax:                                                                  */

	void eoi_destroy_object_instance
	    (
	/*  instance	: in pointer to object_instance_sdd */
	/*  transaction : in transaction type */
	    );

/*
|  Parameters:
|     instance    - object instance to destroy.
|     transaction - transaction into which to register the destruction
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eoi_bin_write
| 
|  Description:  
|     Writes an object instance to a binary stream.
|
\* Syntax:                                                                  */

	void eoi_bin_write
	    (
	/*  stream 	: in out FILE * 			*/
	/*  instance 	: in pointer to object_instance_sdd	*/
	    );

/*
|  Parameters:
|     stream   - The stream on which to write the instance.
|     instance - The object instance to be written.
|
|  Returns:
|     Nothing
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eoi_bin_read
| 
|  Description:  
|     Reads an object instance from an open dialogue binary file
|
\* Syntax:                                                                  */

	object_instance_sdd *eoi_bin_read
	    (
	/*  stream      : in out FILE * */
	/*  transaction : transaction_type */
	    );

/*
|  Parameters:
|     stream      - the stream from which to read the object instance
|     transaction - the transaction into which to put shared data.
|
|  Returns:
|     A pointer to the object instance that was read, or NULL if there
|     were no more objects or if an error occurred.  (If an error occurred,
|     get_status() will be return something other than "ok")
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eoi_src_write
| 
|  Description:  
|     Writes an object instance to a stream in Slang source format.
|
\* Syntax:                                                                  */

	void eoi_src_write
	    (
	/*  stream 	: in out FILE * 			*/
	/*  instance 	: in pointer to object_instance_sdd	*/
	    );

/*
|  Parameters:
|     stream   - The stream on which to write the instance.
|     instance - The object instance to be written.
|
|  Returns:
|     Nothing
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eoi_visu_obj_instance
|
|  Description:
|     send obj instance on to the dialogue
|     NOTE: the att_list parameter contains the attributes in reverse order
|           from what the dialogue expects.
\*--------------------------------------------------------------------------*/
boolean eoi_visu_obj_instance (
/* obj, att_list, trans */
	);
/*
object_instance_sdd *obj;
LIST att_list;
transaction_type trans;
*/

#endif

