/* $Header: /u/04c/mnt/integ/source/de/include/RCS/esd.h,v 1.4 90/12/05 19:42:12 little Exp $ */
/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor Dialogue Shared Data Abstraction (esd)
|
|  Description:  
|     The dialogue editor dialogue sahred data abstraction provides
|     conversions and mappings between dialogue elements and shared
|     data.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log:	esd.h,v $
 * Revision 1.4  90/12/05  19:42:12  little
 * add esd_incr_count
 * 
 * Revision 1.3  90/05/21  13:59:54  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * 
 * Revision 1.2  89/12/12  07:54:12  rcs
 * fixed bugs found during integration testing
 * 
 * Revision 1.1  89/11/28  18:37:25  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef esd
#define esd


#include "se.h"			/* dialogue editor shared data defintions   */


/*--------------------------------------------------------------------------*\
|  Global Variable: esd_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef esd_main
  int esd_debug = 1;
#else
  extern int esd_debug;
#endif

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Constants: esd
|
|  Description:
|    The following constants are defined to improve the useability of
|    the view controller abstraction.
|
\* Definition:                                                              */

#define AND_VCS true
#define NOT_VCS false

#define AND_SUBVCS true
#define NOT_SUBVCS false

#define AND_OBJECTS true
#define NOT_OBJECTS false

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esd_set_dialogue_sdd
| 
|  Description:
|     Sets the fields of a dialogue shared data record to a specific dialogue.
|
\* Syntax:                                                                  */

	dialogue_sdd esd_set_dialogue_sdd(
          /* dialogue         : in     edg_dialogue */
	);

/*
|  Parameters:
|     dialogue - VC for which to create shared data record.
|
|  Returns:
|      A handle to the newly created dialogue shared data record.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esd_add_to_shared_data
| 
|  Description:
|     Creates a shared data element for a specified dialogue and adds it to 
|     the specified transaction.  Shared data elements are also created
|     for subvcs and objects if the appropriate flags are true.  Change 
|     type for shared elements is isd_create.
|
\* Syntax:                                                                  */

	void esd_add_to_shared_data(
          /* transaction : in transaction_type */
	  /* and_subvcs	 : in boolean	*/
	  /* and_objects : in boolean	*/
          /* dialogue    : in edg_dialogue */
	);

/*
|  Parameters:
|     dialogue - VC for which to create shared data record.
|     and_subvcs - True if shared data elements for objects are also to be 
|                  created, false if not.
|     and_objects - True if shared data elements for objects are also to 
|                   be created, false if not.
|     dialogue - VC for which to create shared data record.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esd_put_into_shared_data
| 
|  Description:
|     Creates a shared data element for a specified dialogue and adds it to 
|     the specified transaction.  Shared data elements are also created
|     for subvcs and objects if the appropriate flags are true.  Change 
|     type for shared elements is isd_modify.
|
\* Syntax:                                                                  */

	void esd_put_into_shared_data(
          /* transaction : in transaction_type */
	  /* and_subvcs	 : in boolean	*/
	  /* and_objects : in boolean	*/
          /* dialogue    : in edg_dialogue */
	);

/*
|  Parameters:
|     dialogue - VC for which to create shared data record.
|     and_subvcs - True if shared data elements for objects are also to be 
|                  created, false if not.
|     and_objects - True if shared data elements for objects are also to 
|                   be created, false if not.
|     dialogue - VC for which to create shared data record.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esd_remove_from_shared_data
| 
|  Description:
|     Creates a shared data element for a specified dialogue and adds it to 
|     the specified transaction.  Shared data elements are also created
|     for subvcs and objects if the appropriate flags are true.  Change 
|     type for shared elements is isd_remove.
|
\* Syntax:                                                                  */

	void esd_remove_from_shared_data(
          /* transaction : in transaction_type */
	  /* and_subvcs	 : in boolean	*/
	  /* and_objects : in boolean	*/
          /* dialogue    : in edg_dialogue */
	);

/*
|  Parameters:
|     dialogue - VC for which to create shared data record.
|     and_subvcs - True if shared data elements for objects are also to be 
|                  created, false if not.
|     and_objects - True if shared data elements for objects are also to 
|                   be created, false if not.
|     dialogue - VC for which to create shared data record.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esd_modify_dialogue
| 
|  Description:
|     Modifies an existing dialogue based as specified by a shared data record.
|
\* Syntax:                                                                  */

	void esd_modify_dialogue(
          /* dialogue         : in out edg_dialogue   */
	  /* dialogue_sd	: in     dialogue_sdd * */
	);

/*
|  Parameters:
|     dialogue - VC to be modified.
|     dialogue_sdd - Pointer to a shared data record containing 
|	       modifications to be made.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esd_destroy_dialogue_sdd
| 
|  Description:
|     Destroys a specified dialogue shared data record.
|
\* Syntax:                                                                  */

	void esd_destroy_dialogue_sdd(
	  /* dialogue_sd	: in     dialogue_sdd * */
	);

/*
|  Parameters:
|     dialogue_sdd - Shared data record to be destroyed.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, invalid_dialogue_handle, out_of_memory.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: esd_incr_count
|
|  Description:
|       increments the app version of the dialogue's count attribute and
|       notifies the dialogue of the new value.
\*--------------------------------------------------------------------------*/
int esd_incr_count (
/* transaction, dialogue */
	);
/*
transaction_type transaction;
edg_dialogue dialogue;
*/

#endif

