/*
$Header: /u/04c/mnt/integ/source/de/include/RCS/eso_sd.h,v 1.6 90/12/05 19:42:52 little Exp $
*/

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Object Shared Data (eso_sd.h)
|
|  Description:  
|     Provides object shared data functions.
|
|  Component of:
|     Dialogue Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213

$Log:	eso_sd.h,v $
 * Revision 1.6  90/12/05  19:42:52  little
 * add out_trans parm to eso_make_inst_sd_struct
 * 
 * Revision 1.5  90/07/05  15:16:55  little
 * more structure and visualizer parts integration
 * 
 * Revision 1.4  90/06/14  13:58:29  little
 * more changes to integrate visulization and structure editors
 * 
 * Revision 1.3  90/05/21  15:03:19  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.2  89/12/13  09:27:23  bmc
 * eso_debug defaults to '3'.
 * 
 * Revision 1.1  89/12/12  12:57:04  bmc
 * Initial revision
 * 
 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|    Copyright (c) 1989  by  Carnegie Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef eso
#define eso

/*--------------------------------------------------------------------------*\
|  Includes
|
|  Description:
|     Includes required for this header file.
|
\* Definition:                                                              */

#include "se.h"			/* dialogue editor shared data definitions */
#include "serpent.h"


/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Global Variable: eso_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef eso_main
  int eso_debug = 1;
#else
  extern int eso_debug;
#endif

/*
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eso_create_sd
|
|  Description:
|     Creates a shared data element for a specified object and adds it to
|     the specified transaction.
|
\* Definition:                                                              */

	void eso_create_sd
	    (
	/*  transaction	: in transaction_type,	*/
	/*  object	: in eob_object		*/
	    );

/*
|  Parameters:
|     transaction - The transaction to which to add the shared data
|     object      - The object for which shared data is to be created
|
|  Returns:
|     Nothing
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eso_make_sd_struct
|
|  Description:
|     Creates a shared data structure for an object.
|
\* Definition:                                                              */

	object_sdd eso_make_sd_struct
	    (
	/*  object	: in eob_object	*/
	    );

/*
|  Parameters:
|     object  - The object for which a shared data structure is to be created
|
|  Returns:
|     The shared data structure containing elements from the object
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eso_modify_sd
|
|  Description:
|     Modifies a shared data element for a specified object.
|
\* Definition:                                                              */

	void eso_modify_sd
	    (
	/*  transaction	: in transaction_type,	*/
	/*  object	: in eob_object		*/
	    );

/*
|  Parameters:
|     transaction - The transaction in which to modify the shared data
|     object      - The object for which shared data is to be modified
|
|  Returns:
|     Nothing
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eso_remove_from_sd
|
|  Description:
|     Removes an object from the shared database.
|
\* Definition:                                                              */

	void eso_remove_from_sd
	    (
	/*  transaction	: in transaction_type,	*/
	/*  object	: in eob_object		*/
	    );

/*
|  Parameters:
|     transaction - The transaction associated with the removal
|     object      - The object to be removed
|
|  Returns:
|     Nothing
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eso_modify_object
|
|  Description:
|     Modifies the various parts of an existing object as specified by a
|     shared data record and then destroys the shared data record.
|
|     Note:  This routine can not be used to reparent an object.
|
\* Definition:                                                              */

	void eso_modify_object
	    (
	/*  transaction : in transaction_type	*/
	/*  object	: in out eob_object	*/
	/*  object_sd	: in object_sdd		*/
	    );

/*
|  Parameters:
|     transaction - The transaction in which to modify the instance shared data
|     object      - The object containing the data to be changed
|     object_sd   - The shared data record to be used to make the changes
|
|  Returns:
|     Nothing
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eso_make_inst_sd_struct
|
|  Description:
|     Creates a shared data structure for an object instance
\*--------------------------------------------------------------------------*/
object_instance_sdd *eso_make_inst_sd_struct (
/* object, out_trans */

	);
/*
eob_object object;
transaction_type out_trans;
*/


#endif

