/*
$Header: /u/04c/mnt/integ/source/de/include/RCS/est.h,v 1.2 90/05/21 15:35:25 little Exp $
*/

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Symbol Table Routines
|
|  Description:  
|     This package provides useful symbol table routines isolated in a
|     provided by "cus".  They are completely compatible with the "cus"
|     routines and are intended to be used in tandem with "cus", rather than
|     in lieu of it.
|
|     In all cases, these routines assume that a symbol table has already
|     been created.
|
|  Component of:
|     Dialogue Editor Layer.
|

$Log:	est.h,v $
 * Revision 1.2  90/05/21  15:35:25  little
 *     fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.1  89/11/29  10:07:15  bmc
 * Initial revision
 * 

|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef est
#define est

/*--------------------------------------------------------------------------*\
|  Constant: EST_DEFAULT_NAME or est_default_name
|
|  Description:
|     Defines a constant for an default name, to be placed in the
|     name_text_prop upon creation of an object, VC, variable, etc.
|
\* Syntax:                                                                  */

#define EST_DEFAULT_NAME	"Unnamed"
#define est_default_name	EST_DEFAULT_NAME

/*--------------------------------------------------------------------------*\
|  Routine: est_set_buffer_property
|
|  Description:
|     Provides a convenient way to store a buffer in the symbol table.
|     This routine makes a copy of a buffer and stores the copy in the
|     symbol table.  If a buffer is already in the symbol table, this
|     routine frees it.
|
\* Syntax:                                                                  */

	void est_set_buffer_property
	    (
	/*  ste			: in out cus_sym	*/
	/*  property_name	: in cus_property_type	*/
	/*  the_buffer		: in buffer		*/
	    );

/*
|  Parameters:
|     ste           - The symbol table entry to be affected.
|     property_name - The name of the symbol table property to be set.
|                     This routine does NOT verify that a buffer actually
|                     belongs there!
|     the_buffer    - The buffer to be stored.  Actually, a copy of the
|                     buffer is stored in the symbol table.
|
|  Returns:
|      Nothing.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Macro: est_copy_buffer_property
|
|  Description:
|     Provides a convenient way to copy a buffer property from one symbol
|     to another.
|
\* Syntax:                                                                  */

#define est_copy_buffer_property(to_ste, from_ste, property_name) \
    est_set_buffer_property (to_ste, \
			     property_name, \
			     est_get_buffer_property (from_ste, property_name))
/*
|  Parameters:
|     from_ste      - The symbol table entry from which to copy.
|     to_ste        - The symbol table entry to which to copy.
|     property_name - The name of the symbol table property to be copied.
|                     This routine does NOT verify that the property actually
|                     represents a buffer!  (It can't, really.)
|  Returns:
|      Nothing.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: est_get_buffer_property
|
|  Description:
|     Provides a convenient way to retrieve a buffer property from the
|     symbol table. 
|
\* Syntax:                                                                  */

	buffer est_get_buffer_property
	    (
	/*  ste			: in out cus_sym	*/
	/*  property_name	: in cus_property_type	*/
	    );

/*
|  Parameters:
|     ste           - The symbol table entry.
|     property_name - The name of the symbol table property to be retrieved..
|                     This routine does NOT verify that the property actually
|                     represents a buffer!  (It can't, really.)
|  Returns:
|      If the specified property contains a value, that value is returned.
|      Otherwise, a buffer with undefined contents is returned.  Note that
|      a buffer, not a pointer to a buffer, is returned in either case.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: est_clear_buffer_property
|
|  Description:
|     Clears a symbol table buffer property previously set via
|     est_set_buffer_property, freeing all necessary storage.
|
|
\* Syntax:                                                                  */

	void est_clear_buffer_property
	    (
	/*  ste			: in out cus_sym	*/
	/*  property_name	: in cus_property_type	*/
	    );

/*
|  Parameters:
|     ste           - The symbol table entry to be affected.
|     property_name - The name of the symbol table property to be set.
|                     This routine does NOT verify that the property actually
|                     represents a buffer!  (It can't, really.)
|
|  Returns:
|      Nothing
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: est_set_string_property
|
|  Description:
|     Provides a convenient way to store a string in the symbol table.
|     This routine makes a copy of a string and stores the copy in the
|     symbol table.
|
\* Syntax:                                                                  */

	void est_set_string_property
	    (
	/*  ste			: in out cus_sym	*/
	/*  property_name	: in cus_property_type	*/
	/*  the_string		: in string		*/
	    );

/*
|  Parameters:
|     ste           - The symbol table entry to be affected.
|     property_name - The name of the symbol table property to be set.
|                     This routine does NOT verify that a string actually
|                     belongs there!
|     the_string    - The string to be stored.  Actually, a copy of the
|                     string is stored in the symbol table.  If this value
|                     is NULL, the string property is cleared.  (See the
|                     est_clear_string_property() routine.)
|
|  Returns:
|      Nothing
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Macro: est_copy_string_property
|
|  Description:
|     Provides a convenient way to copy a string property from one symbol
|     to another.
|
\* Syntax:                                                                  */

#define est_copy_string_property(to_ste, from_ste, property_name) \
    est_set_string_property (to_ste, \
			     property_name, \
			     (string) cus_get_prop (from_ste, property_name))
/*
|  Parameters:
|     from_ste      - The symbol table entry from which to copy.
|     to_ste        - The symbol table entry to which to copy.
|     property_name - The name of the symbol table property to be copied.
|                     This routine does NOT verify that the property actually
|                     represents a string!  (It can't, really.)
|  Returns:
|      Nothing.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: est_clear_string_property
|
|  Description:
|     Clears a symbol table string property previously set via
|     est_set_string_property, freeing all necessary storage.
|
|
\* Syntax:                                                                  */


	void est_clear_string_property
	    (
	/*  ste			: in out cus_sym	*/
	/*  property_name	: in cus_property_type	*/
	    );

/*
|  Parameters:
|     ste           - The symbol table entry to be affected.
|     property_name - The name of the symbol table property to be set.
|                     This routine does NOT verify that the property actually
|                     represents a string!  (It can't, really.)
|
|  Returns:
|      Nothing
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: est_free_list_property
|
|  Description:
|     Frees the list stored in the specified list property.  The nodes in the
|     list are not freed.
|
|
\* Syntax:                                                                  */


	void est_free_list_property
	    (
	/*  ste			: in out cus_sym	*/
	/*  property_name	: in cus_property_type	*/
	    );

/*
|  Parameters:
|     ste           - The symbol table entry to be affected.
|     property_name - The name of the symbol table property to be set.
|                     This routine does NOT verify that the property actually
|                     represents a list!  (It can't, really.)
|
|  Returns:
|      Nothing
|
\*--------------------------------------------------------------------------*/
#endif
