/* $Header: /u/04c/mnt/integ/source/de/include/RCS/esv.h,v 1.3 90/05/21 15:36:48 little Exp $ */
/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor VC Abstraction (esv)
|
|  Description:  
|     The dialogue editor VC abstraction provides operaions on the VC
|     abstract type.  These operations include insert, copy, move, delete,
|     and write.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log:	esv.h,v $
 * Revision 1.3  90/05/21  15:36:48  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.2  89/12/12  07:53:52  rcs
 * fixed bugs found during integration testing
 * 
 * Revision 1.1  89/11/28  18:37:57  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef esv
#define esv

#include "se.h"			/* dialogue editor shared data defintions   */
#include "emp.h"                /* dialogue editor mapping abstraction	    */


/*--------------------------------------------------------------------------*\
|  Global Variable: esv_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef esv_main
  int esv_debug = 1;
#else
  extern int esv_debug;
#endif

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Constants: esv
|
|  Description:
|    The following constants are defined to improve the useability of
|    the view controller abstraction.
|
\* Definition:                                                              */

#define AND_VCS true
#define NOT_VCS false

#define AND_SUBVCS true
#define NOT_SUBVCS false

#define AND_OBJECTS true
#define NOT_OBJECTS false

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esv_set_vc_sdd
| 
|  Description:
|     Creates a vc shared data record for a specified vc.
|
\* Syntax:                                                                  */

	vc_sdd esv_set_vc_sdd(
          /* vc         : in     evc_vc */
	);

/*
|  Parameters:
|     vc - VC for which to create shared data record.
|
|  Returns:
|      A handle to the newly created vc shared data record.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esv_add_to_shared_data
| 
|  Description:
|     Creates a shared data element for a specified vc and adds it to 
|     the specified transaction.  Shared data elements are also created
|     for subvcs and objects if the appropriate flags are true.  Change 
|     type for shared elements is isd_create.
|
\* Syntax:                                                                  */

	void esv_add_to_shared_data(
          /* transaction : in transaction_type */
	  /* and_subvcs	 : in boolean	*/
	  /* and_objects : in boolean	*/
          /* vc          : in evc_vc */
	);

/*
|  Parameters:
|     vc - VC for which to create shared data record.
|     and_subvcs - True if shared data elements for objects are also to be 
|                  created, false if not.
|     and_objects - True if shared data elements for objects are also to 
|                   be created, false if not.
|     vc - VC for which to create shared data record.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esv_put_into_shared_data
| 
|  Description:
|     Creates a shared data element for a specified vc and adds it to 
|     the specified transaction.  Shared data elements are also created
|     for subvcs and objects if the appropriate flags are true.  Change 
|     type for shared elements is isd_modify.
|
\* Syntax:                                                                  */

	void esv_put_into_shared_data(
          /* transaction : in transaction_type */
	  /* and_subvcs	 : in boolean	*/
	  /* and_objects : in boolean	*/
          /* vc          : in evc_vc */
	);

/*
|  Parameters:
|     vc - VC for which to create shared data record.
|     and_subvcs - True if shared data elements for objects are also to be 
|                  created, false if not.
|     and_objects - True if shared data elements for objects are also to 
|                   be created, false if not.
|     vc - VC for which to create shared data record.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esv_remove_from_shared_data
| 
|  Description:
|     Creates a shared data element for a specified vc and adds it to 
|     the specified transaction.  Shared data elements are also created
|     for subvcs and objects if the appropriate flags are true.  Change 
|     type for shared elements is isd_remove.
|
\* Syntax:                                                                  */

	void esv_remove_from_shared_data(
          /* transaction : in transaction_type */
	  /* and_subvcs	 : in boolean	*/
	  /* and_objects : in boolean	*/
          /* vc          : in evc_vc */
	);

/*
|  Parameters:
|     vc - VC for which to create shared data record.
|     and_subvcs - True if shared data elements for objects are also to be 
|                  created, false if not.
|     and_objects - True if shared data elements for objects are also to 
|                   be created, false if not.
|     vc - VC for which to create shared data record.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esv_modify_vc
| 
|  Description:
|     Modifies an existing vc based as specified by a shared data record.
|
\* Syntax:                                                                  */

	void esv_modify_vc(
          /* vc : in out evc_vc   */
	  /* id	: in     id_type  */
	);

/*
|  Parameters:
|     vc - VC to be modified.
|     id - ID of the shared data element.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esv_set_vc_map
|
|  Description:
|     Sets the map field of a VC record.
|
\* Syntax:                                                                  */

	void esv_set_vc_map(
	  /* vc		: in out evc_vc    */
	  /* map	: in out emp_map * */
	);

/*
|  Parameters:
|     vc - VC for which to set mapping field.
|     map - Pointer to map record.
|
|  Returns:
|      Void.
|
|  Statuses:
|     ok, invalid_vc_handle.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esv_get_vc_map
|
|  Description:
|     Gets the map field of a VC record.
|
\* Syntax:                                                                  */

	emp_map *esv_get_vc_map(
	  /* vc		: in out evc_vc    */
	);

/*
|  Parameters:
|     vc - VC for which to get mapping field.
|
|  Returns:
|     Pointer to map record.
|
|  Statuses:
|     ok, invalid_vc_handle.
|
\*--------------------------------------------------------------------------*/
#endif

