/*
$Header: /u/04c/mnt/integ/source/de/include/RCS/eta.h,v 1.5 90/12/05 19:43:37 little Exp $
*/

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Available Technologies Abstraction (eta)
|
|  Description:  
|     This abstraction provides operations to determine what technologies
|     (i.e., toolkits) are available to the dialogue writer.
|
|  Component of:
|     Dialogue Editor Layer.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|

$Log:	eta.h,v $
 * Revision 1.5  90/12/05  19:43:37  little
 * add eta_get_mailbox
 * 
 * Revision 1.4  90/10/23  13:55:31  little
 * add definition for eta_get_glue_widget
 * change eta_get_objects, eta_get_attributes, & eta_get_methods to return
 * alists instead of lists
 * 
 * Revision 1.3  90/05/21  15:40:28  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.2  89/12/20  14:47:29  rcs
 * integration testing
 * 
 * Revision 1.1  89/11/29  10:07:43  bmc
 * Initial revision
 * 

|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


#ifndef eta
#define eta

#include "glue_intf.h"
#include "global.h"
#ifndef memoryPack
#define memoryPack
#endif  memoryPack
#include "listPack.h"		/* Zimmermann list package 		    */

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Global Variable: eta_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef eta_main
  int eta_debug = 1;
#else
  extern int eta_debug;
#endif

/*
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Type: eta_technology_record
|        eta_technology
|
|  Description:
|     Specifies the structure containing information for a single technology.
|     eta_technology is a convenient name for a pointer to such a structure.
|
\* Definition:                                                              */

	typedef struct
	    {string short_name;
	    string long_name;
	    string ill_file_name;
	    } eta_technology_record;

	typedef eta_technology_record *eta_technology;
/*
| Components:
|     tech_name     - The short name for the technology.  This name is used
|                     to specify the technology to various other routines in
|                     this package.
|     long_name     - A more description name for the technology.
|     ill_file_name - The name of the ILL file corresponding to the technology.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eta_init
| 
|  Description:  
|     Initializes the available technologies database from the specified file
|
\* Syntax:                                                                  */

	boolean eta_init
	    (
	/*  file_name : in string 	*/
	    );

/*
|  Parameters:
|     file_name - File from which to read technologies.
|
|  Returns:
|     FALSE - Initialization failed.
|     TRUE  - Initialization succeeded.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eta_cleanup
| 
|  Description:  
|     Cleans up the available technologies database, releasing any allocated
|     storage
|
\* Syntax:                                                                  */

	void eta_cleanup ();

/*
|  Parameters:
|     None.
|
|  Returns:
|     Nothing.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eta_get_technologies
| 
|  Description:
|     Returns a list of all available technologies
|
|
\* Syntax:                                                                  */

	LIST eta_get_technologies(
	  /* get_once : in boolean	*/
	);

/*
|  Parameters:
|     get_once - indicates if the routine should return the list if
|		the technologies have already been gotten.
|
|  Returns:
|     A list, each element of which is a pointer to an eta_technology_record
|     structure.  Each such structure contains the information for a single
|     available technology.  THE STORAGE FOR EACH STRUCTURE IS STATIC AND
|     MUST NOT BE MODIFIED.
|
|     Disposition of the list structure itself is the responsibility of the
|     calling routine.
|
|     A return value of NULLlist signifies that an error occurred.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eta_is_valid
| 
|  Description:
|     Determines whether a specified technology name represents one of the
|     available technologies in the database.
|
\* Syntax:                                                                  */

	boolean eta_is_valid
	    (
	/*  tech_name : in string	*/
	    );

/*
|  Parameters:
|     tech_name  - The short name of the technology for which objects are
|                  to be retrieved.  (See the definition of the eta_technology
|                  type, above.)
|
|  Returns:
|     FALSE - The name does not correspond to a valid technology.
|     TRUE  - The name corresponds to a valid technology
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eta_get_objects
| 
|  Description:
|     Returns a list of all objects for a given technology.
|
\* Syntax:                                                                  */

	ALIST eta_get_objects
	    (
	/*  tech_name	: in string	*/
          /* get_once : in boolean      */
	    );

/*
|  Parameters:
|     tech_name  - The short name of the technology for which objects are
|                  to be retrieved.  (See the definition of the eta_technology
|                  type, above.)
|     get_once - indicates if the routine should return the list if
|		the objects have already been gotten.
|
|  Returns:
|     A list, each element of which is the name of an object.  THE STORAGE FOR
|     EACH OBJECT NAME IS IS STATIC AND MUST NOT BE MODIFIED.
|
|     Disposition of the list structure itself is the responsibility of the
|     calling routine.
|
|     A return value of NULLlist signifies that an error occurred.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eta_get_attributes
| 
|  Description:
|     Returns a list of attributes for a specific object type from a
|     technology.
|
\* Syntax:                                                                  */

	ALIST eta_get_attributes
	    (
	/*  tech_name  : in string,	*/
	/*  object_name : in string	*/
          /* get_once : in boolean      */
	    );

/*
|  Parameters:
|     tech_name   - The short name of the technology for which objects are
|                   to be retrieved.  (See the definition of the eta_technology
|                   type, above.)
|     object_name - The name of the object for which attributes are to be
|                   returned.
|     get_once - indicates if the routine should return the list if
|		the attributes have already been gotten.
|
|  Returns:
|     A list, each element of which is the name of an attribute.  THE STORAGE
|     FOR EACH ATTRIBUTE NAME IS IS STATIC AND MUST NOT BE MODIFIED.
|
|     Disposition of the list structure itself is the responsibility of the
|     calling routine.
|
|     A return value of NULLlist signifies that an error occurred.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eta_get_methods
| 
|  Description:
|     Returns a list of methods for a specific object type from a
|     technology.
|
\* Syntax:                                                                  */

	ALIST eta_get_methods
	    (
	/*  tech_name  : in string,	*/
	/*  object_name : in string	*/
          /* get_once : in boolean      */
	    );

/*
|  Parameters:
|     tech_name   - The short name of the technology for which objects are
|                   to be retrieved.  (See the definition of the eta_technology
|                   type, above.)
|     object_name - The name of the object for which methods are to be
|                   returned.
|     get_once - indicates if the routine should return the list if
|		the methods have already been gotten.
|
|  Returns:
|     A list, each element of which is the name of an method.  THE STORAGE
|     FOR EACH METHOD NAME IS IS STATIC AND MUST NOT BE MODIFIED.
|
|     Disposition of the list structure itself is the responsibility of the
|     calling routine.
|
|     A return value of NULLlist signifies that an error occurred.
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eta_get_glue_widget
|
|  Description:
|     Returns the glue widget for a specific object type from a
|     technology.
\*--------------------------------------------------------------------------*/
WIDG *eta_get_glue_widget (
/* tech_short_name, object_name */
	);
/*
string tech_short_name;
string object_name;
*/


/*--------------------------------------------------------------------------*\
|  Routine: eta_get_mailbox
|
|  Description:
|     Returns the glue widget for a specific object type from a
|     technology.
\*--------------------------------------------------------------------------*/
string eta_get_mailbox (
/* tech_short_name */
	);
/*
string tech_short_name;
*/

#endif eta

