/*
$Header: /u/04c/mnt/integ/source/de/include/RCS/eva.h,v 1.4 90/05/21 15:40:47 little Exp $
*/

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Variable Abstraction (eva)
|
|  Description:  
|     provides operaions on the variable abstract type
|
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|

$Log:	eva.h,v $
 * Revision 1.4  90/05/21  15:40:47  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.3  89/12/15  20:28:16  bmc
 * Added binary read routine and associated definitions.
 * 
 * Revision 1.2  89/12/15  11:58:05  little
 * no changes
 * 
 * Revision 1.1  89/11/29  10:08:17  bmc
 * Initial revision
 * 

|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/



#ifndef eva
#define eva

#include "serpent.h"


/*--------------------------------------------------------------------------*\
|  Abstract Type: eva_variable
|
|  Description:
|    The eva_variable abstract type provides a variable data instance.
|
\* Definition:                                                              */

typedef private eva_variable;


#include "evc.h"


/*--------------------------------------------------------------------------*\
|  Global Variable: eva_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */
#ifdef eva_main
  int eva_debug = 1;
#else
  extern int eva_debug;
#endif


#define eva_no_variable		((eva_variable) NULL)
#define EVA_NO_VARIABLE		eva_no_variable

#define eva_no_more_variables	((eva_variable) 1)
#define EVA_NO_MORE_VARIABLES	eva_no_more_variables


/*--------------------------------------------------------------------------*\
|  Routine: eva_create_variable
| 
|  Description:
|     Creates a new variable
|
\* Syntax:                                                                  */

	eva_variable eva_create_variable(
	  /* name	: in string */
		);

/*
|  Parameters:
|     name - the name the variable is to take on
|
|  Returns:
|      the newly created variable.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eva_destroy_variable
| 
|  Description:  
|     Destroys a specified variable and any sub-components.
|
\* Syntax:                                                                  */

	void eva_destroy_variable (
	  /* the_variable	: in out eva_variable */
	);

/*
|  Parameters:
|     the_variable - variable to destroy.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eva_duplicate_variable
| 
|  Description:  
|     Makes a copy of a variable
|
\* Syntax:                                                                  */

	eva_variable eva_duplicate_variable (
	  /* the_variable : in eva_variable */
	);

/*
|  Parameters:
|     the_variable - variable to copy
|
|  Returns:
|     a copy of the variable, or EVA_NO_VARIABLE if an error occurred
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eva_set_snippet
| 
|  Description:  
|     squirrels away a snippet for a variable
|
\* Syntax:                                                                  */

	void eva_set_snippet (
	  /* the_variable : in out eva_variable */
	  /* the_snip: in buffer */
	);

/*
|  Parameters:
|     the_variable - variable in question
|     the_snip - snippet to be saved
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eva_get_parent
| 
|  Description:  
|     fetches the squirreled away parent for a variable
|
\* Syntax:                                                                  */

	evc_vc eva_get_parent (
	  /* the_variable : in eva_variable */
	);

/*
|  Parameters:
|     the_variable - variable in question
|
|  Returns:
|      the parent associated with the variable
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eva_get_snippet
| 
|  Description:  
|     fetches the squirreled away snippet for a variable
|
\* Syntax:                                                                  */

	buffer eva_get_snippet (
	  /* the_variable : in eva_variable */
	);

/*
|  Parameters:
|     the_variable - variable in question
|
|  Returns:
|      the snippet associated with the variable
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eva_set_comment
| 
|  Description:  
|     squirrels away comment for a variable
|
\* Syntax:                                                                  */

	void eva_set_comment (
	  /* the_variable : in out eva_variable */
	  /* the_comment: in buffer */
	);

/*
|  Parameters:
|     the_variable - variable in question
|     the_comment - comment to be saved
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eva_get_comment
| 
|  Description:  
|     fetches the squirreled away comment for a variable
|
\* Syntax:                                                                  */

	buffer eva_get_comment (
	  /* the_variable : in eva_variable */
	);

/*
|  Parameters:
|     the_variable - variable in question
|
|  Returns:
|      the comment associated with the variable
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
| Routine: eva_set_name
| 
|  Description:  
|     squirrels away name for a variable
|
\* Syntax:                                                                  */

	void eva_set_name (
	  /* the_variable : in out eva_variable */
	  /* the_name: in string */
	);

/*
|  Parameters:
|     the_variable - variable in question
|     the_name - name to be saved
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eva_get_name
| 
|  Description:  
|     fetches the squirreled away name for a variable
|
\* Syntax:                                                                  */

	string eva_get_name (
	  /* the_variable : in eva_variable */
	);

/*
|  Parameters:
|     the_variable - variable in question
|
|  Returns:
|      the name associated with the variable
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eva_parse_variable_2_ed
| 
|  Description:  
|     convert the var from parser format to editor format
|
\* Syntax:                                                                  */

	eva_variable eva_parse_variable_2_ed (
	  /* the_variable : in cus_sym */
	);

/*
|  Parameters:
|     the_variable - variable in question
|
|  Returns:
|      editor form of the var
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eva_src_write
| 
|  Description:  
|     write a variable to a source stream
|
\* Syntax:                                                                  */

	void eva_src_write (
	  /* the_variable : in eva_variable */
	  /* stream: in out FILE * */
	);

/*
|  Parameters:
|     the_variable - variable in question
|     stream - the stream on which to write the variable
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: eva_bin_write
| 
|  Description:  
|     write a variable to a binary stream
|
\* Syntax:                                                                  */

	void eva_bin_write (
	  /* the_variable 	: in eva_variable */
	  /* stream 		: in out FILE * */
	);

/*
|  Parameters:
|     the_variable - variable in question
|     stream - the stream on which to write the variable
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: eva_bin_read
| 
|  Description:  
|     Reads a variable from an open dialogue binary file
|
\* Syntax:                                                                  */

	eva_variable eva_bin_read
	    (
	  /* stream 		: in out FILE * */
	    );

/*
|  Parameters:
|     stream - the stream from which to read the variable
|
|  Returns:
|     EVA_NO_VARIABLE       - No variable or illegal variable.
|     EVA_NO_MORE_VARIABLES - End of variable list.  (i.e., encountered
|                             EBI_CLOSE_RECORD character, which was pushed
|                             back onto the token stream).
|     Otherwise             - returns the decoded variable.
\*--------------------------------------------------------------------------*/


#endif


