/*
$Header: /u/04c/mnt/integ/source/de/include/RCS/evc.h,v 1.6 90/08/06 16:43:32 little Exp $
*/

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor VC Abstraction (evc)
|
|  Description:  
|     The dialogue editor VC abstraction provides operations on the VC
|     abstract type.
|
$Log:	evc.h,v $
 * Revision 1.6  90/08/06  16:43:32  little
 * added definition for evc_imported_dialogue.
 * 
 * Revision 1.5  90/07/05  15:17:24  little
 * more structure and visualizer parts integration
 * 
 * Revision 1.4  90/05/21  15:42:47  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.3  89/12/18  14:23:00  little
 * fixed some shit that was wrong
 * 
 * Revision 1.2  89/12/12  11:56:28  bmc
 * memoryPack now conditionally defined.
 * 
 * Revision 1.1  89/11/29  10:09:24  bmc
 * Initial revision
 * 
|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef evc
#define evc

/*--------------------------------------------------------------------------*\
|  General Purpose Includes: General-purpose definitions 
|
\* Definition:                                                              */

#ifndef memoryPack
#define memoryPack              /* use memoryPack rather than mallocPack    */
#endif  memoryPack

#include <listPack.h>           /* C Toolbox - listPack                     */
#include "global.h"             /* Global type defs and constants           */

/*--------------------------------------------------------------------------*\
|  Global Variable: evc_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef evc_main
  int evc_debug = 1;
#else
  extern int evc_debug;
#endif

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Type: evc_node_selection
|
|  Description:
|    Defines an enumeration of selection types used to modify the operation
|    of various VC manipulation routines.
|
\* Definition:                                                              */

typedef enum
    {
    evc_entire_hierarchy,
    evc_vc_only
    }
evc_node_selection;

#define EVC_ENTIRE_HIERARCHY	evc_entire_hierarchy
#define EVC_VC_ONLY		evc_vc_only

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Abstract Type: evc_vc
|
|  Description:
|    The evc_vc abstract type provides a handle to a VC data instance.
|
\* Definition:                                                              */

	typedef private evc_vc;
/*--------------------------------------------------------------------------*/


#define evc_no_vc	((evc_vc) NULL)
#define EVC_NO_VC	evc_no_vc

#define evc_no_more_vcs ((evc_vc) 1)
#define EVC_NO_MORE_VCS evc_no_more_vcs


/* WARNING: These include directives must appear after the type definition
	for evc_vc */

#include "edg.h"                /* dialogue editor dialogue abstraction	    */
#include "eob.h"                /* dialogue editor object abstraction	    */
#include "eva.h" 		/* dialogue editor variable abstraction	    */


/*--------------------------------------------------------------------------*\
|  Global Variable: evc_imported_dialogue
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef evc_main
   edg_dialogue evc_imported_dialogue;
#else
   extern edg_dialogue evc_imported_dialogue;
#endif
/*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_node_print_value
| 
|  Description:
|     Provides a print string for a variable of type evc_node_selection.
|     Useful mainly for debugging.
|
\* Syntax:                                                                  */

	string evc_node_print_value
	    (
	/*  node_selection	: in evc_node_selection	*/
	    );

/*
|  Parameters:
|     node_selection - The variable for which a print string is to be
|                      returned.
|
|  Returns:
|      The string corresponding to the node selection value, or a
|      predefined default if node_selection represents an illegal value.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_print_vc_tree
| 
|  Description:
|     Recursively prints a VC subtree.  Currently only prints the VC names.
|     Useful mainly for debugging.
|
\* Syntax:                                                                  */

	void evc_print_vc_tree
	    (
	/*  the_vc		: in evc_vc	*/
	/*  indent_level	: in integer	*/
	    );

/*
|  Parameters:
|     the_vc       - The VC to print
|     indent_level - The level of indentation required.  This value should be
|                    an integer no less than zero.  This value is multiplied
|                    by some constant spacing factor to producing an indentation
|                    value that is used to produce a tree-like display.
|
|  Returns:
|      Nothing.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_create_vc
| 
|  Description:
|     Creates a new VC record.
|
\* Syntax:                                                                  */

	evc_vc evc_create_vc ();

/*
|  Parameters:
|     None.
|
|  Returns:
|      The newly created VC
|
|  Statuses:
|     ok, out_of_memory.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_destroy_vc
| 
|  Description:  
|     Destroys the specified VC.  Allows the caller to remove just the VC
|     node or the entire hierarchy contained by the VC.  The parent VC need
|     not be specified because it can be determined from the VC being killed.
|
\* Syntax:                                                                  */

	void evc_destroy_vc
	    (
	/*  the_vc	: in evc_vc */
	/*  how_to_kill	: in evc_node_selection */
	/*  out_trans	: in transaction_type */
	    );

/*
|  Parameters:
|     the_vc      - VC to destroy.
|     how_to_kill - Defines how the VC should be destroyed; can take two values:
|
|                   EVC_ENTIRE_HIERARCHY:
|                      Recursively destroy the entire hierarchy associated
|                      with the VC.  All sub-VCs, objects, etc. are destroyed
|                      along with the VC.
|                   EVC_VC_ONLY:
|                      Destroy only the VC; any objects, sub-VCs and variables
|                      are given to the VC's parent VC.  IT IS AN ERROR TO
|                      DESTROYTHE DIALOGUE VC IN THIS MANNER! (Note: The
|                      creation condition, retained comments, actions on
|                      creation, and actions on destruction are never
|                      re-parented.)
|	out_trans - transaction to use for anything needing to be communicated
|		    to the dialogue
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_duplicate_vc
|
|  Description:
|     Duplicates the contents of a VC, returning the new VC.  Allows the
|     caller to copy just the VC node or the entire hierarchy contained by
|     the VC.
|
|
\* Syntax:                                                                  */

	evc_vc evc_duplicate_vc
	    (
	/*  the_vc	: in evc_vc		*/
	/*  how_to_dup	: in evc_node_selection	*/
	    );

/*
|  Parameters:
|     the_vc     - The VC to duplicate
|     how_to_dup - Defines how the VC should be duplicated; can take two values:
|
|                  EVC_ENTIRE_HIERARCHY:
|                     Copy the entire hierarchy associated with the VC.  The
|                     new VC will contain separate copies of the objects,
|                     sub-VCs and variables, creation condition, comments,
|                     actions on destruction and actions on creation 
|                     that are contained with the original VC.
|                  EVC_VC_ONLY:
|                     Copy only the VC itself; the new VC will contain copies
|                     of the creation condition, comments, actions on
|                     creation and actions on destruction as the original
|                     VC, but it will have no objects, sub-VCs or variables.
|
|  Returns:
|      The new VC.  If an error occurs, EVC_NO_VC is returned.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_get_parent
| 
|  Description:
|     Gets the parent of a VC, if there is one.
|
\* Syntax:                                                                  */

	evc_vc evc_get_parent
	    (
	/*  the_vc : in evc_vc	*/
	    );

/*
|  Parameters:
|     the_vc - VC for which parent is to be returned.
|
|  Returns:
|     Parent VC if there is one, EVC_NO_VC otherwise.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_get_dialogue
| 
|  Description:
|     Gets the dialogue to which a VC is hooked.  (Only the top-level VC,
|     Eve, is hooked to a dialogue.)
|
\* Syntax:                                                                  */

	edg_dialogue evc_get_dialogue
	    (
	/*  the_vc : in evc_vc	*/
	    );

/*
|  Parameters:
|     the_vc - VC for which the dialogue is to be returned.
|
|  Returns:
|     EDG_NO_DIALOGUE if the VC isn't hooked to a dialogue (i.e., it isn't
|     Eve), descriptor of dialogue otherwise.  (See "edg.h" for dialogue-
|     related definitions.)
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_set_name
| 
|  Description:
|     Sets a VC's name.
|
\* Syntax:                                                                  */

	void evc_set_name
	    (
	/*  the_vc	: in out evc_vc	*/
	/*  the_name	: in string	*/
	    );

/*
|  Parameters:
|     the_vc   - VC to be named
|     the_name - the name to give the VC
|
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_get_name
| 
|  Description:
|     Gets a VC's name.
|
\* Syntax:                                                                  */

	string evc_get_name
	    (
	/*  the_vc	: in out evc_vc	*/
	    );

/*
|  Parameters:
|     the_vc - VC for which to get name
|
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_set_comments
| 
|  Description:
|     Sets a VC's retained comments field.
|
\* Syntax:                                                                  */

	void evc_set_comments
	    (
	/*  the_vc		: in out evc_vc	*/
	/*  the_comments	: in buffer	*/
	    );

/*
|  Parameters:
|     the_vc       - VC in which to put the comments
|     the_comments - the comments
|
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_get_comments
| 
|  Description:
|     Gets a VC's retained comments field.
|
\* Syntax:                                                                  */

	buffer evc_get_comments
	    (
	/*  the_vc	: in out evc_vc	*/
	    );

/*
|  Parameters:
|     the_vc       - VC from which to get comments
|     the_comments - the comments
|
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_set_creation_condition
| 
|  Description:
|     sets the creation condition attribute of a vc
|
\* Syntax:                                                                  */

	void evc_set_creation_condition
	    (
	/*  the_vc			: in out evc_vc	*/
	/*  the_creation_condition	: in buffer	*/
	    );

/*
|  Parameters:
|     the_vc                 - VC in which to put the creation condition
|     the_creation_condition - the creation condition
|
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_get_creation_condition
| 
|  Description:
|     Gets the creation condition attribute of a VC.
|
\* Syntax:                                                                  */

	buffer evc_get_creation_condition
	    (
	/*  the_vc	: in evc_vc	*/
	    );

/*
|  Parameters:
|     the_vc - VC from which to get the creation condition
|
|  Returns:
|      a buffer containing creation condition
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_set_create_actions
| 
|  Description:
|     sets the creation actions attribute of a VC.
|
\* Syntax:                                                                  */

	void evc_set_create_actions
	    (
	/*  the_vc		: in out evc_vc	*/
	/*  create_actions	: in buffer	*/
	    );

/*
|  Parameters:
|     the_vc - VC in which to put the creation actions
|     the_create_actions - the creation actions
|
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_get_create_actions
| 
|  Description:
|     Gets the creation actions attribute of a VC.
|
\* Syntax:                                                                  */

	buffer evc_get_create_actions
	    (
	/*  the_vc	: in evc_vc	*/
	    );

/*
|  Parameters:
|     the_vc - VC from which to get the creation actions
|
|  Returns:
|      a buffer containing creation actions
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_set_destroy_actions
| 
|  Description:
|     Sets the destroy actions attribute of a VC.
|
\* Syntax:                                                                  */

	void evc_set_destroy_actions
	    (
	/*  the_vc		: in out evc_vc	*/
	/*  destroy_actions	: in buffer	*/
	    );

/*
|  Parameters:
|     the_vc - VC in which to put the destroy actions
|     the_destroy_actions - destroy actions
|
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_get_destroy_actions
| 
|  Description:
|     Gets the destroy actions attribute of a VC.
|
\* Syntax:                                                                  */

	buffer evc_get_destroy_actions
	    (
	/*  the_vc	: in evc_vc	*/
	    );

/*
|  Parameters:
|     the_vc - VC from which to get the destroy actions
|
|  Returns:
|      a buffer containing destroy actions
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_add_object
|
|  Description:
|     Adds an object to the set of objects associated with a VC.
|
\* Syntax:                                                                  */

        void evc_add_object
	    (
        /*  the_vc	: in out evc_vc */
        /*  object	: eob_object	*/
	    );

/*
|  Parameters:
|     the_vc - VC to which to add the objects
|     object - The object to add.  Note:  The actual object, not a copy,
|              is added to the VC.  If the calling routine modifies the 
|              original object, the object in the set will automatically
|              be updated.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_remove_object
|
|  Description:
|     Removes an object from the set of objects associated with its parent.
|     VC.
|
\* Syntax:                                                                  */

        eob_object evc_remove_object
	    (
        /*  object	: eob_object	*/
	    );

/*
|  Parameters:
|     object - The object to remove.  The parent is not specified since the
|              object can determine who owns it.  WARNING: This routine
|              assumes that the object is parented to a VC.  If, for some
|              bizarre reason, the object has been parented to something
|              else, all bets are off.
|
|  Returns:
|     For convenience, this function returns the removed object.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_get_objects
|
|  Description:
|     Returns the set of objects associated with a VC, in the form of a
|     Zimmermann list.
|
\* Syntax:                                                                  */

        LIST evc_get_objects
	    (
        /*  the_vc  : in evc_vc     */
	    );

/*
|  Parameters:
|     the_vc - VC from which to get the set of objects.
|
|  Returns:
|      A list containing all the objects, in no particular order.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_add_variable
|
|  Description:
|     Adds an variable to the set of variables associated with a VC.
|
\* Syntax:                                                                  */

        void evc_add_variable
	    (
        /*  the_vc	: in out evc_vc */
        /*  variable	: eva_variable	*/
	    );

/*
|  Parameters:
|     the_vc   - VC to which to add the variables
|     variable - The variable to add.  Note:  The actual variable, not a copy,
|                is added to the VC.  If the calling routine modifies the 
|                original variable, the object in the set will automatically
|                be updated.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_remove_variable
|
|  Description:
|     Removes an variable from the set of variables associated with its
|     parent VC.
|
\* Syntax:                                                                  */

        eva_variable evc_remove_variable
	    (
        /*  variable	: eva_variable	*/
	    );

/*
|  Parameters:
|     variable - The object to remove.  The parent is not specified since the
|                variable can determine who owns it.  WARNING: This routine
|                assumes that the variable is parented to a VC.  If, for some
|                bizarre reason, the variable has been parented to something
|                else, all bets are off.
|
|  Returns:
|     For convenience, this function returns the removed variable.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_get_variables
|
|  Description:
|     Returns the set of variables associated with a VC, in the form of a
|     Zimmermann list.
|
\* Syntax:                                                                  */

        LIST evc_get_variables
	    (
        /*  the_vc  : in evc_vc     */
	    );

/*
|  Parameters:
|     the_vc - VC from which to get the set of variables.
|
|  Returns:
|      A list containing all the variables, in no particular order.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_add_vc
|
|  Description:
|     Adds a VC to the set of sub-VCs associated with a VC.
|
\* Syntax:                                                                  */

        void evc_add_vc
	    (
        /*  parent_vc	: in out evc_vc */
        /*  vc_to_add	: evc_vc	*/
	    );

/*
|  Parameters:
|     parent_vc - VC to which to add the new VC.  This VC becomes the parent.
|     vc_to_add - the VC to add
|     vc_to_add - The VC to add.  Note:  The actual VC, not a copy, is added
|                 is added to the parent VC.  If the calling routine modifies
|                 the original object, the VC in the set will automatically 
|                 be updated.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_get_vcs
|
|  Description:
|     Returns the set of sub-VCs associated with a VC, in the form of a
|     Zimmermann list.
|
\* Syntax:                                                                  */

        LIST evc_get_vcs
	    (
        /*  the_vc  : in evc_vc     */
	    );

/*
|  Parameters:
|     the_vc - VC from which to get the set of vcs.
|
|  Returns:
|      A list containing all the vcs, in no particular order.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_remove_vc
|
|  Description:
|     Removes the specified VC from the internal hierarchy.  Allows the
|     caller to remove just the VC node or the entire hierarchy contained by
|     the VC.  The parent VC need not be specified because it can be
|     determined from the VC being removed.
|
\* Syntax:                                                                  */

	evc_vc evc_remove_vc
	    (
	/*  the_vc		: in out evc_vc		*/
	/*  how_to_remove	: in evc_node_selection	*/
	    );

/*
|  Parameters:
|     the_vc     - The VC to remove
|     how_to_dup - Defines how the VC should be removed; can take two values:
|
|                  EVC_ENTIRE_HIERARCHY:
|                     Recursively remove the entire hierarchy associated
|                     with the VC.  All sub-VCs, objects, etc. are removed
|                     along with the VC.  They remain "attached" to the VC,
|                     so if the entire VC is placed somewhere else in the
|                     overall hierarchy, its objects go with it.
|                  EVC_VC_ONLY:
|                     Remove only the VC; any objects, sub-VCs and variables
|                     are given to the VC's parent VC.  IT IS AN ERROR TO
|                     REMOVE THE DIALOGUE VC IN THIS MANNER! (Note: The
|                     creation condition, retained comments, actions on
|                     creation, and actions on destruction are never
|                     re-parented.)
|
|  Returns:
|      For convenience, this function returns the removed VC.
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_parse_vc_2_ed
|
|  Description:
|     convert the parser version of the vc to editor version
|
\* Syntax:                                                                  */

	evc_vc evc_parse_vc_2_ed
	    (
	/*  the_vc		: in cus_sym		*/
	    );

/*
|  Parameters:
|     the_vc     - The symbol for the VC to convert
|
|  Returns:
|      The converted dialogue VC
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_src_write
|
|  Description:
|     write a vc to a source stream
|
\* Syntax:                                                                  */

	void evc_src_write
	    (
	/*  the_vc		: in evc_vc		*/
	/*  stream		: in FILE *		*/
	    );

/*
|  Parameters:
|     the_vc     - The VC to be written
|     stream     - The stream to which to write it
|
|  Returns:
|      Nothing
|
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_bin_write
|
|  Description:
|     write a vc to a source stream
\*--------------------------------------------------------------------------*/
void evc_bin_write (
	/* the_vc, stream */
		);
/*
FILE *stream;
evc_vc the_vc;
*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_bin_read
|
|  Description:
|     read a vc from a binary stream
\*--------------------------------------------------------------------------*/
evc_vc evc_bin_read (
/* stream, transaction */
		);
/*
FILE *stream;
transaction_type transaction;
*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_instantiate_guys
|
|  Description:
|     do the instantiate type shit for a vc template
\*--------------------------------------------------------------------------*/
void evc_instantiate_guys (
/* the_vc, extent, command, transaction */
	);
/*
evc_vc the_vc;
evc_node_selection extent;
int command;
transaction_type transaction;
*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_add_object_instance
|
|  Description:
|     Adds an object instance to a vc
\*--------------------------------------------------------------------------*/
boolean evc_add_object_instance (
/* the_vc, obj_instance, transaction */
	);
/*
evc_vc the_vc;
object_instance_sdd *obj_instance;
transaction_type transaction;
*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_remove_object_instance
|
|  Description:
|     Removes an object instance from a vc
\*--------------------------------------------------------------------------*/
boolean evc_remove_object_instance (
/* the_vc, obj_instance, out_trans */
	);
/*
evc_vc the_vc;
object_instance_sdd *obj_instance;
transaction_type out_trans;
*/


/*--------------------------------------------------------------------------*\
|  Routine: evc_set_parent
| 
|  Description:
|     Sets the parent of a VC.
|
\* Syntax:                                                                  */

	void evc_set_parent
	    (
	/*  the_vc	: in out evc_vc	*/
	/*  new_parent	: in evc_vc	*/
	    );

/*
|  Parameters:
|     the_vc     - VC for which to set the parent.
|     new_parent - The new parent VC.
|
|  Returns:
|     Nothing.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: evc_set_dialogue
| 
|  Description:
|     Sets the dialogue to which a VC is hooked (i.e., effectively makes the
|     VC think it's Eve.)
|
\* Syntax:                                                                  */

	void evc_set_dialogue
	    (
	/*  the_vc	: in out evc_vc		*/
	/*  dialogue  	: in edg_dialogue	*/
	    );

/*
|  Parameters:
|     the_vc     - The target VC.
|     dialogue   - The dialogue to which to hook the VC.  If this parameter
|                  is EDG_NO_DIALOGUE, the dialogue setting is cleared.
|
|  Returns:
|     Nothing.
|
\*--------------------------------------------------------------------------*/

#endif

