/* $Header: /u/04b/mnt/ljb/RCS/boolean.sl,v 1.4 1991/04/23 18:00:57 ljb Exp $ */
/*--------------------------------------------------------------------------*\
|  
|  Name:  Boolean Xtype (boolean.sl)
|
|  Description:  
|     This file contains the Slang code to implement the edit fields
|     for boolean attributes.  A separate view controller is instantiated
|     for each boolean attribute.  These view 
|     controllers instantiate off dialogue shared data that is
|     created in the generic widget (gw.sl) module.
|
|  Component of:
|     Dialogue editor dialogue.
|
|  Related files:
|     gw.slang - generic widget code.
|
|  Project:
|     Serpent
|     User Interface Project
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Robert C. Seacord
|     rcs@sei.cmu.edu  
|
|  Change History:
|     Nov 20, 1990 : Module created.
|     $Log: boolean.sl,v $
% Revision 1.4  1991/04/23  18:00:57  ljb
% change background to form rather than box
%
% Revision 1.3  1991/03/21  17:12:35  rcs
% *** empty log message ***
%
% Revision 1.2  91/03/19  19:54:28  rcs
% added on destroy to delete shared data instace
% 
% Revision 1.1  90/11/29  17:33:09  rcs
% Initial revision
% 
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*------------------- constants --------------------------------------------*/

#undef DEBUG1
#undef DEBUG2
#undef DEBUG3

#define DEBUG1(statement) SERPENT_DEBUG1(boolean_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(boolean_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(boolean_debug, statement)

/*--------------------------------------------------------------------------*\
|  Name:  gt_boolean_vc
|
|  Description:
|    The boolean x type vc supports the editing of boolean attributes.
\*--------------------------------------------------------------------------*/
VC: gt_boolean_vc

Creation Condition: (new("boolean_xtype_sdd"))

Variables:
  boolean_help_id;
  attr_name: get_sd_value(boolean_xtype_sdd.attribute,"name");
  attr_value: get_sd_value(boolean_xtype_sdd.attribute,"value");

Objects:

  attribute_box(boolean)

  attribute_label(boolean)

  boolean_attr_yes_button: XawCommand {
   Attributes: 
     parent: boolean_attr_box; 
     horizDistance: boolean_attr_label.width + 20;
     label: "Yes"; 
     width: 50; 
     height: 20; 
     background: {

/*
**  If value is undefined check default value.
*/
       if (attr_value = UNDEFINED) then
/*
**  If default value is true hilite button.
*/
         if (
           glue_get_attr_default(widget_handle,attr_name,SERPENT_DEFAULT)=TRUE
         ) then 
           background := selected_background_color; 
/*
**  Default is no, normal background color.
*/
         else
           background := normal_background_color; 
         end if; 
/*
**  Attribute value specified.
*/
       else 
/*
**  If value is true, hilite button.
*/
         if (attr_value = TRUE) then 
           background := selected_background_color; 
/*
**  Value is false, normal background color.
*/
         else
           background := normal_background_color; 
         end if; 
       end if; 
     }
     foreground: {
       if (attr_value = UNDEFINED) then 
         if (glue_get_attr_default( 
               widget_handle,attr_name,SERPENT_DEFAULT 
             ) = TRUE
         ) then 
           foreground := selected_foreground_color; 
         else
           foreground := normal_foreground_color; 
         end if; 
       else 
         if (attr_value = TRUE) then
           foreground := selected_foreground_color; 
         else
           foreground := normal_foreground_color; 
         end if; 
       end if; 
     }
   Methods: 
     notify: {
/*
**  Update the application value.
*/
       put_sd_value(boolean_xtype_sdd.attribute,"value",TRUE); 
/*
**  Update the display value. 
*/
       if (id_exists(object_id)) then
         put_sd_value(object_id,attr_name,TRUE);
       end if;
     }
  }

  boolean_attr_no_button: XawCommand {
    Attributes: 
      parent: boolean_attr_box; 
      horizDistance: boolean_attr_label.width + 90; 
      label: "No"; 
      width: 50; 
      height: 20; 
      background: {
/*
**  If value is undefined check default value.
*/
       if (attr_value = UNDEFINED) then
/*
**  If default value is false hilite button.
*/
         if (glue_get_attr_default(
               widget_handle,attr_name,SERPENT_DEFAULT
             ) = FALSE
         ) then
            background := selected_background_color; 
          else
            background := normal_background_color; 
          end if; 
/*
**  Attribute value specified.
*/
        else
          if (attr_value = FALSE) then
            background := selected_background_color; 
          else
            background := normal_background_color; 
          end if; 
	end if;
      }
      foreground: {
/*
**  If value is undefined check default value.
*/
        if (attr_value = UNDEFINED) then
/*
**  If default value is false hilite button.
*/
          if (glue_get_attr_default(
                widget_handle,attr_name,SERPENT_DEFAULT
             ) = FALSE
          ) then
            foreground := selected_foreground_color; 
          else
            foreground := normal_foreground_color; 
          end if;     
/*
**  Attribute value specified.
*/
        else
          if (attr_value = FALSE) then
            foreground := selected_foreground_color; 
          else
            foreground := normal_foreground_color; 
          end if;     
        end if;
      }
   Methods: 
     notify: {
/*
**  Update the application value.
*/
       put_sd_value(boolean_xtype_sdd.attribute,"value",FALSE); 
/*
**  Update the display value. 
*/
       if (id_exists(object_id)) then
         put_sd_value(object_id,attr_name,FALSE);
       end if;
      
     }
  }

/*
** Destroy the shared data instance that caused this view controller
** to be created.
*/
  On Destroy: {
    destroy_sd_instance(boolean_xtype_sdd);
  }

end VC gt_boolean_vc;
