static char rcsid [] =
"$Header: /u/04c/mnt/integ/source/de/src/RCS/eac_cmd.c,v 1.6 90/10/23 13:58:17 little Exp $";

/*--------------------------------------------------------------------------*\
|     
|  Name:  Dialogue Editor Att Command Definitions  (eac)
|
|  Description:  
|     definitions of the attribute associated commands supported by the DEA.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log:	eac_cmd.c,v $
 * Revision 1.6  90/10/23  13:58:17  little
 * modify eac_get_attribute to work better
 * 
 * Revision 1.5  89/12/20  14:41:39  rcs
 * integration testing
 * 
 * Revision 1.4  89/12/13  13:34:12  rcs
 * fixed bug
 * 
 * Revision 1.3  89/12/12  17:01:32  little
 * change the order of the parameters fromthe dialogue to eac_get_attribute
 * 
 * Revision 1.2  89/12/12  15:15:30  little
 * - add eac_delete_attribute
 * - clean up some extreanous comments
 * ./
 * 
 * Revision 1.1  89/12/07  15:10:20  little
 * Initial revision
 * 
|
|   created 6 dec 1989
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute   (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|   (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


                        /* need DEBUG1 for check_parm_type to work, but do not
                           use any other debug statements, saber is a lot
                           better and definition must be before include of
			   ea2.h */
#define DEBUG1(statement) SERPENT_DEBUG1(eac_debug, statement)


#define eac_main
#include "eac_cmd.h"
#include "esa_sd.h"
#include "eob.h"
#include "eat.h"
#include "emp.h"
#include "ehf.h"
#include "ea2.h"
#include "serpent.h"
#include "idd.h"
#include "isc.h"
#ifndef memoryPack
#define memoryPack
#endif
#include "hashPack.h"
#include "se.h"


/*--------------------------------------------------------------------------*\
|  Routine:  eac_get_attribute
| 
|  Description:  
|     gets the specified (by a command parameter from the dialogue) attribute
|     from the abstraction and places it into a transaction
\*--------------------------------------------------------------------------*/
void eac_get_attribute (command, transaction)
command_sdd *command;
transaction_type transaction;
{  eob_object the_object;		/* parameter 1 */
   string att_name;			/* parameter 2 */
   eat_attribute the_att;
   parameter_sdd *parm;
   id_type obj_id;
   LIST atts;

   set_status (ok);

				/* get the first parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
						command-> parameters);
   check_parm_type (parm, idd_id,
		    "eac_get_attribute: first parameter is not an id\n");
   obj_id = *((id_type *) ((parm-> value).body));
   the_object = emp_get_de (obj_id);

				/* now, get the 2nd parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
						parm-> next);
   check_parm_type (parm, idd_string,
		    "eac_get_attribute: second parameter is not a string\n");
   att_name = (string) ((parm-> value).body);

				/* now find the attr in the object, if not
				   there create one.  then place the info
				   associated with the attribute into a sd
				   record */
   atts = eob_get_attributes (the_object);
   check_status("eac_get_attribute:  bad status from eob_get_attributes.");

   if (the_att = (eat_attribute) ea2_in_list (att_name, atts, eat_get_name))
      {if (emp_get_id (the_att) == null_id)
         {esa_create_sd (transaction, the_att);
         check_status (
           "eac_get_attribute:  bad status from esa_create_sd.");
         }
      else
         {esa_modify_sd (transaction, the_att);
         check_status(
           "eac_get_attribute:  bad status from esa_modify_sd.");
	 }
      }
   else
      {the_att = eat_create_attribute (att_name);
      check_status("eac_get_attribute:  bad status from eat_create_attr.");

      eob_add_attribute (the_object, the_att);
      check_status("eac_get_attribute:  bad status from eob_add_attribute.");

      esa_create_sd (transaction, the_att);
      check_status("eac_get_attribute:  bad status from esa_create_sd.");
      }

   return;
} 					/* end eac_get_attribute */


/*--------------------------------------------------------------------------*\
|  Routine:  eac_delete_attribute
|
|  Description:
|     deletes the specified (by a command parameter from the dialogue) att and
|       places a remove sd request into the transaction
\*--------------------------------------------------------------------------*/
void eac_delete_attribute (command, transaction)
command_sdd *command;
transaction_type transaction;
{  id_type att_id;			/* parameter 1 */
   parameter_sdd *parm;
   eat_attribute the_att; 

   set_status(ok);

                                /* get the first parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
                                                command-> parameters);
   check_parm_type (parm, idd_id,
                    "eac_delete_attribute: first parameter is not an id\n");
   att_id = *((id_type *) ((parm-> value).body));
   the_att = emp_get_de (att_id);

                                /* delete the att from it's parent obj */
   eob_remove_attribute (the_att);
   check_status("eac_delete_attribute:  bad status from eob_remove_attr.");
                                /* kill the att itself */
   eat_destroy_attribute (the_att);
   check_status("eac_delete_attribute:  bad status from eat_destroy_attr.");
                                /* now place a delete request into the
                                   xaction */
   esa_remove_from_sd (transaction, the_att);
   check_status("eac_delete_attribute:  bad status from esa_remove_from_sd.");

   return;
}                                       /* end eac_delete_attribute */


