static char rcsid [] =
	"$Header: /u/04c/mnt/integ/source/de/src/RCS/ebi.c,v 1.15 90/08/29 18:27:11 little Exp $";

/*--------------------------------------------------------------------------*\
|   
|  Name: Dialogue Editor Binary Input/Output Routines
|
|  Description:  
|     Provides primitives to read and write a dialogue editor binary file.
|

$Log:	ebi.c,v $
 * Revision 1.15  90/08/29  18:27:11  little
 * add the counter to the dialogue sd record
 * 
 * Revision 1.14  90/08/14  13:15:14  rcs
 * got rid of ok variable
 * 
 * Revision 1.13  90/07/05  15:03:40  little
 *  more structure and visualizer parts integration
 * 
 * Revision 1.12  90/02/14  11:50:59  bmc
 * ebi_write_boolean() now writes a space after the boolean value.
 * 
 * Revision 1.11  90/02/08  21:56:13  bmc
 * Added ebi_write_boolean() and ebi_read_boolean().
 * 
 * Revision 1.10  90/01/03  16:28:49  bmc
 * No changes.
 * 
 * Revision 1.9  89/12/20  11:46:35  bmc
 * Converted pushed-token buffer to a stack.  Did some general cleanup.
 * 
 * Revision 1.8  89/12/18  18:01:07  bmc
 * Added checksum override feature.
 * 
 * Revision 1.7  89/12/18  16:53:27  bmc
 * ebi_write_buffer() now writes buffers out based on length, rather than
 * using '\0' to terminate writes.
 * 
 * Revision 1.6  89/12/18  15:10:20  bmc
 * ebi_read_checksum now skips white space properly.
 * 
 * Revision 1.5  89/12/18  14:43:36  bmc
 * changed the way ebi_read_checksum works.
 * 
 * Revision 1.4  89/12/15  20:11:08  bmc
 * no changes
 * 
 * Revision 1.3  89/12/15  16:03:13  bmc
 * added read/write checksum, fixed some stuff.
 * 
 * Revision 1.2  89/12/14  13:32:56  bmc
 * Changed to work with new binary format.
 * 
 * Revision 1.1  89/11/29  09:51:10  bmc
 * Initial revision
 * 

|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*-------------------- Includes --------------------------------------------*/

#include <string.h>
#include <ctype.h>

#define ebi_main
#include "ebi.h"
#include "isc.h"
#include "u22.h"
#include "ubo.h"
#include "global.h"


#ifndef memoryPack
#define memoryPack
#endif  memoryPack
#include "memoryPack.h"

/*-------------------- Macros  ---------------------------------------------*/

#undef DEBUG1
#undef DEBUG2
#undef DEBUG3
#define DEBUG1(statement) SERPENT_DEBUG1(ebi_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(ebi_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(ebi_debug, statement)


/*--------------------------------------------------------------------------*\
|  Constants: Miscellaneous constants
|
|  Description:
|     EMPTY_BUF_COUNT - Magic buffer count used to denote an empty buffer.
|     NULL_BUF_COUNT  - Magic buffer count used to denote a NULL buffer.
|     UNDEF_BUF_COUNT - Magic buffer count used to denote an undefined buffer.
|     BAD_NUMBER      - Return value from convert_number indicating a bad
|                       conversion.  This value can be any negative number
|                       less than the smallest of the three magic numbers
|                       listed above.
|     MAX_TOKEN_CHARS - Maximum size of an input token.
|     BIN_FALSE	      - What a boolean 'false' looks like.
|     BIN_TRUE	      - What a boolean 'true' looks like.
\* Definition:                                                              */

#define EMPTY_BUF_COUNT		(0)
#define NULL_BUF_COUNT		(-1)
#define UNDEF_BUF_COUNT		(-2)
#define BAD_NUMBER		(-255)
#define MAX_TOKEN_CHARS		(255)

#define BIN_FALSE		'F'
#define BIN_TRUE		'T'


/*--------------------------------------------------------------------------*\
|  Constants: File Integrity
|
|  Description:
|     MAGIC_NUMBER     - Defines the magic number found at the top of
|                        dialogue binary files.
|     CHECKSUM_MASK    - Used to mask the calculated checksum value as it is
|                        being calculated.  The constant is deliberately chosen
|                        to be the largest possible 16-bit unsigned integer,
|                        on the theory that most machines can handle a 16-bit
|                        unsigned integer.  By restricting this value to
|                        16 bits, rather than using machine-specific MAXINT
|                        values, we hope to allow a user to port a dialogue
|                        binary file between architectures with no modification.
|     CHECKSUM_OVERRIDE- Character string used to override checksum.  If this
|                        string is found in place of the checksum, checksum
|                        verification is bypassed.  (This is mainly used for
|                        debugging.)  The constant is defined as a character
|                        variable to save string space in the executable in
|                        case it is referenced more than once.
|
\* Definition:                                                              */

#define MAGIC_NUMBER			(38)
#define CHECKSUM_MASK			(0xffff)

static char *CHECKSUM_OVERRIDE =	"*****";


/*--------------------------------------------------------------------------*\
|  Constants: Token Parsing
|
|  Description:
|     The following strings are really constants and are used when parsing
|     tokens.  Arrays are used for string constants instead of #define
|     macros to conserve string space in case the constants are referenced
|     more than once.  These arrays are to be used as constants.  DON'T
|     CHANGE THEM BY CODE ASSIGNMENT!
|
\* Definition:                                                              */

    /*
    ** TOKEN_DELIMS - The delimiters separating tokens in the input stream.
    ** WHITE_SPACE  - White space delimiters.
    **
    ** Note that TOKEN_DELIMS includes white space, so that WHITE_SPACE is
    ** defined based on TOKEN_DELIMS.  Non-white space tokens must appear
    ** first in the TOKEN_DELIMS array, and the WHITE_SPACE string definition
    ** may need to be be adjusted if TOKEN_DELIMS is expanded.
    */
static char TOKEN_DELIMS [] =
    {
    EBI_OPEN_RECORD,
    EBI_CLOSE_RECORD,
    '\t',
    '\n',
    '\f',
    ' ',
    '\0' /* terminate the string */
    };
static char *WHITE_SPACE = & TOKEN_DELIMS [2];

    /*
    ** BUFFER_COUNT_DELIM - The character used to delimit the count from
    **                      the run of characters in a buffer representation.
    ** COUNT_DELIM_STRING - A string version of BUFFER_COUNT_DELIM, needed
    **                      in some places instead of a character.
    */
#define BUFFER_COUNT_DELIM 	':'
static char COUNT_DELIM_STRING [] = {BUFFER_COUNT_DELIM, '\0'};


/*--------------------------------------------------------------------------*\
|  Constants and Local Data: Token Stack
|
|
token_stack, stack_size
|
|  Description:
|     Defines a stack and a stack pointer for pushed tokens.
|     TOKEN_STACK_SIZE - maximum number of pushed tokens.  (Arbitrary, but
|                        ought to be greater than 1.)
|     token_stack      - the token stack.  ebi_push_token() places tokens
|                        in here, if possible, and ebi_read_token() exhausts
|                        the token stack before reading the input stream.
|     stack_size       - the current number of tokens on the stack.
|
\* Definition:                                                              */

#define TOKEN_STACK_SIZE 	(5)

static string token_stack [TOKEN_STACK_SIZE];
static unsigned stack_size = 0;

/*--------------------------------------------------------------------------*\
|  Macros:
|     boolean push_token_stack (token);
|        returns true if token was pushed, false otherwise
|
|     string pop_token_stack ();
|        returns pointer to top element on stack if stack isn't empty,
|        NULL otherwise
|
|     boolean token_stack_empty ();
|        returns true if token stack is empty, false otherwise
|
|     boolean token_stack_full ();
|        returns true if token stack is full, false otherwise
|
\* Definition:                                                              */

#define push_token_stack(the_string) \
    (stack_size < TOKEN_STACK_SIZE ? \
	token_stack [stack_size++] = the_string, true : \
	false)

#define pop_token_stack() \
    (stack_size > 0 ? token_stack [--stack_size] : NULL)

#define token_stack_empty() \
    (stack_size == 0)

#define token_stack_full() \
    (stack_size == TOKEN_STACK_SIZE)

/*--------------------------------------------------------------------------*\
|  Macro: is_null_buffer, set_null_buffer, is_empty_buffer
|
|  Description:
|     is_null_buffer determines whether is buffer is a null buffer. 
|     is_empty_buffer determines whether is buffer is a empty buffer. 
|     set_null_buffer initializes a null buffer.  A null buffer has a length
|     of zero and a body that's null.
|
\* Definition:                                                              */

#define is_empty_buffer(the_buffer) \
    (((the_buffer).length == 1) && (*(the_buffer).body == '\0'))

#define is_null_buffer(the_buffer) \
    (((the_buffer).length == 0) && ((the_buffer).body == NULL))

/*--------------------------------------------------------------------------*\
|  Macro: is_buffer_selector
|
|  Description:
|     Determines whether a selector character is one of the buffer selector
|     characters.  Uses the ebi_buffer_selectors array defined in "ebi.h".
|
\* Definition:                                                              */

#define is_buffer_selector(the_character) \
    (ebi_index (ebi_buffer_selectors, the_character) != NULL)

/*--------------------------------------------------------------------------*\
|  Macro: is_digit
|
|  Description:
|     Determines whether a character represents a legal base 10 digit using
|     the ebi_index function, below.
|
\* Definition:                                                              */

#define is_digit(the_char) \
    (ebi_index ("0123456789", the_char) != NULL)


/*--------------------------------------------------------------------------*\
|  Local Routine: ebi_index
| 
|  Description:
|     Essentially performs the same work as strchr(3), but doesn't count the
|     terminal '\0' as part of the string.  WARNING: Can't handle a NULL
|     string.
\*--------------------------------------------------------------------------*/

static string ebi_index (the_string, the_char)
string the_string;
char the_char;
{
    while ( (*the_string != '\0') && (*the_string != the_char) )
	the_string++;

    return (*the_string == '\0' ? NULL : the_string);
}

/*--------------------------------------------------------------------------*\
|  Local Routine: ebi_skip_white
| 
|  Description:
|     Skips leading white space in a file, returning FALSE if an error
|     occurred, TRUE otherwise.
\*--------------------------------------------------------------------------*/

boolean ebi_skip_white (stream)
FILE *stream;
{
    int c;
    boolean error = true;

    while ( ((c = getc (stream)) != EOF) &&
	    (ebi_index (WHITE_SPACE, c) != NULL) )
	;

    if (c == EOF)
	error = false;

    else if (ungetc (c, stream) == EOF)
	error = false;

    return (error);
}					/* end ebi_skip_white */

/*--------------------------------------------------------------------------*\
|  Local Routine: convert_number
| 
|  Description:
|     Converts a string into a number, if possible, returning the number.
|     Returns BAD_NUMBER on error, and sets status appropriately.
\*--------------------------------------------------------------------------*/

static int convert_number (the_string)
string the_string;
{
    char *dummy;
    int number;

    extern long strtol ();


    set_status (ok);
    number = (int) strtol (the_string, &dummy, 10);
    /*
    ** If dummy points to the end of the string, then the string
    ** contained only digits (and, possibly, a minus sign).  We want
    ** a non-negative number here.
    */
    if (*dummy != '\0')
	{
	number = BAD_NUMBER;
	set_status (invalid_integer);
	}

    return (number);
}					/* end ebi_convert_number */

/*--------------------------------------------------------------------------*\
|  Local Routine: convert_buffer_count
|
|  Description:
|     Converts a buffer counter, returning BAD_NUMBER if the buffer count
|     is illegal.
|
\*--------------------------------------------------------------------------*/

static int convert_buffer_count (the_string)
string the_string;
{
    int number;


    if ( (number = convert_number (the_string)) != BAD_NUMBER )
	{
	if ( (number != EMPTY_BUF_COUNT) &&
	     (number != NULL_BUF_COUNT) &&
	     (number != UNDEF_BUF_COUNT) &&
	     (number < 0) )
	    number = BAD_NUMBER;
	}

    return (number);
}

/*--------------------------------------------------------------------------*\
|  Local Routine: ebi_read_chars
| 
|  Description:
|     Reads a specified number of characters from a stream, returning the
|     characters in an allocated string.  Returns NULL on error.  An error
|     will result from:
|
|        1. inability to allocate memory
|        2. premature EOF or newline
|
|     A count of 0 is legal and results in an empty string being returned.
\*--------------------------------------------------------------------------*/

static string ebi_read_chars (stream, count)
FILE *stream;
int count;
{
    char *the_string;


    set_status (ok);
    if ( (the_string = make_node (count + 1)) == NULL )
	{
	set_status (out_of_memory);
	DEBUG1 (fprintf (stderr,
		"ebi_read_chars: can't allocate memory for buffer body\n"););
	}
    else
	{
	/*
	** Read count characters into the string.  An unexpected EOF is an
	** error.  Note that embedded newlines are perfectly legal.
	*/
	char *s = the_string;
	int i = count;
	int c;

	while ( (i > 0) && ((c = getc (stream)) != EOF))
	    {
	    *s = (char) c;
	    s++;
	    i--;
	    }

	if (c == EOF)
	    {
	    set_status (end_of_file);
	    DEBUG1 (fprintf (stderr, "ebi_read_chars: premature EOF\n"););
	    free_node (the_string);
	    the_string = NULL;
	    }

	else
	    *s = '\0';
	}

    return (the_string);
}					/* end ebi_read_chars */

/*--------------------------------------------------------------------------*\
|  Local Routine: ebi_get_nonwhite_char
| 
|  Description:
|     Returns the next non-white character from a stream.  Returns EOF on
|     end of file.
\*--------------------------------------------------------------------------*/

static int ebi_get_nonwhite_char (stream)
FILE *stream;
{
    return (ebi_skip_white (stream) ? getc (stream) : EOF);
}


/*--------------------------------------------------------------------------*\
|  Routine: ebi_write_string
| 
|  Description:
|     writes a string a stream
\*--------------------------------------------------------------------------*/
boolean ebi_write_string (the_stream, the_string, the_selector) 
FILE *the_stream;
string the_string;
char the_selector;
{  ebi_buffer a_ebi_buffer; 
   buffer a_buffer; 

			/* we use buffer write to actually implement this */
   set_status (ok);
			/* if we have the undefined string, set up an undefined
			   buffer */
   if (!strcmp (the_string, UNDEFINED_STRING))
      {ubo_init_buffer (a_buffer);
      }
    else
      {a_buffer.body = the_string; 
      a_buffer.type = serpent_string; 
      a_buffer.length = strlen (the_string) + 1;
      }
   a_ebi_buffer.actual_buffer = a_buffer; 
   a_ebi_buffer.selector = the_selector; 
   if (!ebi_write_buffer (the_stream, a_ebi_buffer)) 
      {set_status (io_failure); 
      return (FALSE);
      }
   return (TRUE);
}					/* end ebi_write_string */


/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_string
| 
|  Description:
|     reads a string from a stream 
\*--------------------------------------------------------------------------*/
string ebi_read_string (the_stream, the_selector) 
FILE *the_stream;
char the_selector;
{  ebi_buffer a_ebi_buffer;
   buffer a_buffer;

			/* we use buffer read to actually implement this */
   set_status (ok);
   a_ebi_buffer = ebi_read_buffer (the_stream);
   if (a_ebi_buffer.selector != the_selector)
      {set_status (io_failure);
      fprintf (stderr,
          "ebi_read_string: Expected `%c' string, found `%c' string\n",
         the_selector, a_ebi_buffer.selector);
     
      return (NULL);
      }

			/* if the read buffer is undefined, convert it to the
			   undefined string */
   a_buffer = a_ebi_buffer.actual_buffer;
   if (ubo_is_undefined (a_buffer))
      {a_buffer.body = make_node (strlen (UNDEFINED_STRING) + 1);
      strcpy (a_buffer.body, UNDEFINED_STRING);
      }
   return (a_buffer.body);
}					/* end ebi_read_string */


/*--------------------------------------------------------------------------*\
|  Local Routine: ebi_do_read_token
| 
|  Description:
|     Reads a token from a stream, using the specified delimiters.
|     Assumes the stream is positioned at the start of a legal token.
|
|     WARNING: Tokens are currently limited to MAX_TOKEN_CHARS characters in
|     length!
\*--------------------------------------------------------------------------*/
string ebi_do_read_token (stream, delims)
FILE *stream;
string delims;
{
    char token_buf [MAX_TOKEN_CHARS + 1];
    string token = NULL;
#define bufsize   (sizeof (token_buf) - 1) /* allow room for trailing '\0' */
    int current = 0;
    int c;
    boolean found_delim = false;


    set_status (ok);
    token_buf [0] = '\0';

    while ( (! found_delim) &&
	    (get_status () == ok) &&
	    /* can't use ebi_get_nonwhite_char here, since white_space may
	    ** be a delimiter.
	    */
	    ((c = getc (stream)) != EOF) &&
	    (c != '\n') )
    	{
	if (ebi_index (delims, c) != NULL)
	    {
	    /* This delimiter may have special meaning.  Push it back onto
	    ** the input stream and let someone else deal with it.
	    */
	    found_delim = true;
	    if (ungetc (c, stream) == EOF)
		{
		set_status (io_failure);
		DEBUG1 (fprintf (stderr,
		    "ebi_do_read_token: can't push back delimiter '%c'\n",
		    c););
		}
	    }
	else
	    {
	    token_buf [current++] = c;
	    if (current == bufsize)
		{
		DEBUG1 (fprintf (stderr,
		    "ebi_do_read_token: buffer overflow\n"););
		set_status (overflow);
		}
	    }
	}

    if ( (get_status () == ok) && (current > 0) )
	{
	token_buf [current] = '\0';
	if ( (token = u22_copy_string (token_buf)) == NULL )
	    set_status (out_of_memory);
	}

    return (token);
}					/* end ebi_do_read_token */

/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_token
| 
|  Description:
|     Reads a token from an open dialogue binary file.
\*--------------------------------------------------------------------------*/

string ebi_read_token (stream)
FILE *stream;
{
    string token = NULL;
    char one_char_token [2];
    int c;
    boolean eof = false;


    set_status (ok);

    one_char_token [0] = '\0';
    one_char_token [1] = '\0';

    if ( (token = pop_token_stack ()) == NULL )
	{
	/* No token on token stack.  Read one. */

	while ( (token == NULL) && (! eof) )
	    {
	    /*
	    ** Check for the special character tokens.
	    */
	    switch (c = ebi_get_nonwhite_char (stream))
		{
		case EOF:
		    eof = true;
		    break;

		case EBI_OPEN_RECORD:
		case EBI_CLOSE_RECORD:
		    /*
		    ** These are special single-character tokens.
		    */
		    one_char_token [0] = c;
		    if ( (token = u22_copy_string (one_char_token)) == NULL )
			{
			DEBUG1 (fprintf (stderr,
			    "ebi_read_token: memory allocation error\n"););
			set_status (out_of_memory);
			eof = true; /* terminate the loop */
			}
		    break;

		default:
		    /*
		    ** The character is something else entirely.  Since it's
		    ** probably part of a legal token, and since
		    ** ebi_do_read_token expects the file pointer to be
		    ** positioned at the beginning of a token, let's push
		    ** this character back onto the stream.
		    */
		    if (ungetc (c, stream) == EOF)
			{
			DEBUG1 (fprintf (stderr,
			    "ebi_read_token: error in ungetc\n"););
			eof = true;
			set_status (system_operation_failed);
			}
		    else if ( (token = ebi_do_read_token (stream,
							TOKEN_DELIMS)) == NULL )
			eof = true;
		    break;

		} /* endsw */
	    } /* end while */
	} /* end else */

    return (token);
}					/* end ebi_read_token */

/*--------------------------------------------------------------------------*\
|  Routine: ebi_push_token
| 
|  Description:
|     Pushes a token back onto the input stream.
\*--------------------------------------------------------------------------*/

boolean ebi_push_token (the_token)
string the_token;
{
    set_status (ok);
    if (the_token == NULL)
	{
	set_status (null_memory_ptr);
	DEBUG1 (fprintf (stderr, "ebi_push_token: NULL argument passed\n"););
	}

    else if (! push_token_stack (the_token) )
	{
	set_status (overflow);
	DEBUG1 (fprintf (stderr, "ebi_push_token: too many tokens pushed\n"););
	}

    return (get_status () == ok);
}

/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_token_char
| 
|  Description:
|     Reads a single-character token from an open dialogue binary file.
\*--------------------------------------------------------------------------*/

char ebi_read_token_char (stream)
FILE *stream;
{
    string token;
    char token_char = EBI_BAD_CHAR;


    if ( (token = ebi_read_token (stream)) != NULL )
	{
	if (strlen (token) == 1)
	    token_char = *token;
	free_node (token);
	}

    return (token_char);
}					/* end ebi_read_token_char */

/*--------------------------------------------------------------------------*\
|  Routine: ebi_push_token_char
| 
|  Description:
|     Pushes a token character back onto the input stream.
\*--------------------------------------------------------------------------*/

boolean ebi_push_token_char (the_char)
char the_char;
{
    string token;

    set_status (ok);
    if (! isprint (the_char))
	{
	set_status (bad_parameter);
	DEBUG1 (fprintf (stderr,
	    "ebi_push_token_char: bad character, ASCII code %d\n", the_char););
	}

    else if (token_stack_full ())
	{
	set_status (overflow);
	DEBUG1 (fprintf (stderr,
	    "ebi_push_token_char: too many tokens pushed\n"););
	}

    else if ( (token = make_node (2)) == NULL )
	{
	set_status (out_of_memory);
	DEBUG1 (fprintf (stderr,
	    "ebi_push_token_char: can't allocate memory\n"););
	}

    else
	{
	*token = the_char;
	*(token + 1) = '\0';
	(void) push_token_stack (token);
	}

    return (get_status () == ok);
}


/*--------------------------------------------------------------------------*\
|  Routine: ebi_is_buffer_selector
| 
|  Description:
|     Determines whether a selector character represents a buffer selector.
|
\* Syntax:                                                                  */

boolean ebi_is_buffer_selector (selector)
char selector;
{
    return (is_buffer_selector (selector));
}


/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_buffer
| 
|  Description:
|     Reads an ebi_buffer from an open dialogue binary file.
\*--------------------------------------------------------------------------*/
ebi_buffer ebi_read_buffer (stream)
FILE *stream;
{
    ebi_buffer buf;
    string token;
    int count;
    char selector;
    boolean bad_buffer = true; /* by default */

    /*
    ** Buffers are stored like this (line breaks are insignificant):
    **
    ** EBI_OPEN_RECORD
    **    buffer_selector <count><BUFFER_COUNT_DELIM><stuff>
    ** EBI_CLOSE_RECORD
    **
    ** buffer_selector is one of the buffer selector characters.
    ** count           tells how many characters follow BUFFER_COUNT_DELIM
    **
    ** EBI_OPEN_RECORD followed immediately by EBI_CLOSE_RECORD denotes
    ** an empty buffer.
    */

    set_status (ok);
    ubo_init_buffer (buf.actual_buffer);

    /*
    ** Try to read an open record and a character token.  If that works,
    ** check the character token.  If it's a close_record, we have an
    ** empty buffer (which is okay).  Mark the buffer as such, and
    ** stop.  Otherwise, make sure the character token represents a
    ** legal buffer selector character. Then, skip any intervening white
    ** space, and read and decode the count.  Since the count ends with
    ** a BUFFER_COUNT_DELIM, we can simply use ebi_do_read_token to
    ** parse it, using the BUFFER_COUNT_DELIM as the token delimiter.
    ** Then, try to read `count' characters.
    */
    if (ebi_read_token_char (stream) != EBI_OPEN_RECORD)
	{
	set_status (invalid_buffer);
	DEBUG1 (fprintf (stderr, "ebi_read_buffer: missing opening %c\n",
	    EBI_OPEN_RECORD););
	}

    else if (! is_buffer_selector (selector = ebi_read_token_char (stream)))
	{
	set_status (invalid_buffer);
	DEBUG1 (fprintf (stderr,
	    "ebi_read_buffer: bad buffer selector char -- %c\n",
	    buf.selector););
	}

    else if (! ebi_skip_white (stream))
	{
	set_status (io_failure);
	DEBUG1 (fprintf (stderr, "ebi_read_buffer: can't skip white space\n"););
	}

    else if ( (token = ebi_do_read_token (stream, COUNT_DELIM_STRING)) == NULL ) 
	{
	set_status (invalid_buffer);
	DEBUG1 (fprintf (stderr,
	    "ebi_read_buffer: can't read character count\n"););
	}

    else if (getc (stream) != BUFFER_COUNT_DELIM)
	{
	set_status (invalid_buffer);
	DEBUG1 (fprintf (stderr,
	    "ebi_read_buffer: Missing %c after buffer count\n",
	    BUFFER_COUNT_DELIM););
	}

    else if ( (count = convert_buffer_count (token)) == BAD_NUMBER )
	{
	set_status (invalid_buffer);
	DEBUG1 (fprintf (stderr,
	    "ebi_read_buffer: bad character count -- \"%s\"\n", token););
	}

    else
	{
	buf.selector = selector;
	buf.actual_buffer.type = idd_string;

	/*
	** Process the count.  There are a number of special counts we have to
	** handle.  We can assume the count is good at this point.
	*/
	bad_buffer = false;

	switch (count)
	    {
	    case EMPTY_BUF_COUNT:
		/*
		** An empty buffer is actually a buffer of length 1, containing
		** an empty string.
		*/
		buf.actual_buffer.body = make_node (1);
		if (buf.actual_buffer.body == NULL)
		    {
		    set_status (out_of_memory);
		    bad_buffer = true;
		    DEBUG1 (fprintf (stderr,
			"ebi_read_buffer: can't create empty buffer\n"););
		    }
		else
		    {
		    *(buf.actual_buffer.body) = '\0';
		    buf.actual_buffer.length  = 1;
		    }
		break;

	    case NULL_BUF_COUNT:
		/*
		** A null buffer is a buffer of length 0 with a NULL pointer
		** in place of the buffer body.
		*/
		buf.actual_buffer.length = 0;
		buf.actual_buffer.type   = idd_string;
		buf.actual_buffer.body   = NULL;
		break;

	    case UNDEF_BUF_COUNT:
		/*
		** An undefined buffer is, simply, undefined. That's what
		** ubo_init_buffer produces.
		*/
		ubo_init_buffer (buf.actual_buffer);
		break;

	    default:
		/*
		** Non-special count.  Read the number of characters specified
		** by the count.
		*/

		if ((buf.actual_buffer.body = ebi_read_chars (stream,
							      count)) == NULL)
		    {
		    bad_buffer = true;
		    set_status (invalid_buffer);
		    DEBUG1 (fprintf (stderr,
			"ebi_read_buffer: error reading buffer body\n"););
		    }

		else
		    buf.actual_buffer.length = count + 1;
		break;
	    } /* end switch */


	/*
	** Read a close-record character in all cases.
	*/
	if ((! bad_buffer) && 
	    (ebi_read_token_char (stream) != EBI_CLOSE_RECORD))
	    {
	    set_status (invalid_buffer);
	    bad_buffer = true;
	    DEBUG1 (fprintf (stderr, "ebi_read_buffer: missing closing %c\n",
		EBI_CLOSE_RECORD););
		}

	} /* end else */

    if (bad_buffer)
	{
	if (buf.actual_buffer.body != NULL)
	    free_node (buf.actual_buffer.body);
	ubo_init_buffer (buf.actual_buffer);
	buf.selector = EBI_INVALID_SELECTOR;
	}

    return (buf);
}					/* end ebi_read_buffer */

/*--------------------------------------------------------------------------*\
|  Routine: ebi_write_buffer
| 
|  Description:
|     Writes an ebi_buffer to an open dialogue binary file.
\*--------------------------------------------------------------------------*/

boolean ebi_write_buffer (stream, buf)
FILE *stream;
ebi_buffer buf;
{
    /*
    ** Buffers are stored like this (line breaks are insignificant):
    **
    ** EBI_OPEN_RECORD
    **    buffer_selector <count><BUFFER_COUNT_DELIM><stuff>
    ** EBI_CLOSE_RECORD
    **
    ** buffer_selector is one of the buffer selector characters.
    ** count           tells how many characters follow BUFFER_COUNT_DELIM
    **
    ** EBI_OPEN_RECORD followed immediately by EBI_CLOSE_RECORD denotes
    ** an empty buffer.
    */

    set_status (ok);
    if (! is_buffer_selector (buf.selector))
	{
	set_status (invalid_buffer);
	DEBUG1 (fprintf (stderr, "ebi_write_buffer: bad selector -- %c\n",
		    buf.selector););
	}

    else if (! ebi_write_open_record (stream))
	{
	set_status (io_failure);
	DEBUG1 (fprintf (stderr, "ebi_write_buffer: can't write opening %c\n",
		    EBI_OPEN_RECORD););
	}

    else if ( (fputc (buf.selector, stream) == EOF) ||
	      (fputc (' ', stream) == EOF) )
	{
	set_status (io_failure);
	DEBUG1 (fprintf (stderr, "ebi_write_buffer: can't write selector %c\n",
		    buf.selector););
	}

    else
	{
	if (is_null_buffer (buf.actual_buffer))
	    fprintf (stream, "%d%c", NULL_BUF_COUNT, BUFFER_COUNT_DELIM);

	else if (is_empty_buffer (buf.actual_buffer))
	    fprintf (stream, "%d%c", EMPTY_BUF_COUNT, BUFFER_COUNT_DELIM);

	else if (ubo_is_undefined (buf.actual_buffer))
	    fprintf (stream, "%d%c", UNDEF_BUF_COUNT, BUFFER_COUNT_DELIM);

	else
	    {
	    /* write the count and the chars for a normal buffer */

	    int i;
	    char *s;


	    fprintf (stream, "%d%c", buf.actual_buffer.length - 1,
		BUFFER_COUNT_DELIM);

	    /* We want to start at 1 so we don't write out the trailing '\0' */

	    for (i = 1, s = buf.actual_buffer.body;
		 (i < buf.actual_buffer.length) && (get_status () == ok);
		 i++, s++)
		{
		if (fputc (*s, stream) == EOF)
		    {
		    set_status (io_failure);
		    DEBUG1 (fprintf (stderr,
			"ebi_write_buffer: failed to write character %c\n",
			*s););
		    }
		} /* end for */
	    } /* end else */

	if (! ebi_write_close_record (stream))
	    {
	    set_status (io_failure);
	    DEBUG1 (fprintf (stderr,
		"ebi_write_buffer: can't write opening %c\n",
		EBI_OPEN_RECORD););
	    }
	} /* end else */

    return (get_status () == ok);
}					/* end ebi_write_buffer */

/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_boolean
| 
|  Description:
|     Reads an boolean value from an open dialogue binary file.
\*--------------------------------------------------------------------------*/

boolean ebi_read_boolean (stream, value)
FILE *stream;
boolean *value;
{
    char bool_char;
    boolean success = true;


    *value = FALSE;
    switch (bool_char = ebi_read_token_char (stream))
	{
	case EBI_BAD_CHAR:
	    DEBUG1 (fprintf (stderr, "ebi_read_boolean: empty input.\n"););
	    set_status (io_failure); /* nothing there. */
	    break;

	case BIN_FALSE:
	    *value = false;
	    break;

	case BIN_TRUE:
	    *value = true;
	    break;

	default:
	    DEBUG1 (fprintf (stderr,
		"ebi_read_boolean: bad boolean '%c' in dialogue binary file\n",
		bool_char););
	    set_status (invalid_symbol);
	    break;
	}

    return (success);
}

/*--------------------------------------------------------------------------*\
|  Routine: ebi_write_boolean
| 
|  Description:
|     Writes an boolean value to an open dialogue binary file.
\*--------------------------------------------------------------------------*/

boolean ebi_write_boolean (stream, value)
FILE *stream;
boolean value;
{
    set_status (ok);

    if ( (putc (value ? BIN_TRUE : BIN_FALSE, stream) == EOF) ||
	 (putc (' ', stream) == EOF) )
	{
	set_status (io_failure);
	DEBUG1 (fprintf (stderr,
	    "ebi_write_boolean: failed to write boolean %c\n",
	    value ? BIN_TRUE : BIN_FALSE););
	}

    return (get_status () == ok);
}


/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_magic_number
| 
|  Description:
|     Reads an ebi magic number from an open dialogue binary file.
\*--------------------------------------------------------------------------*/

boolean ebi_read_magic_number (stream)
FILE *stream;
{
    string line;
    int magic_number;


    set_status (ok);

    /*
    ** The magic number exists all by itself on a single line of input.
    */

    if ( (line = u22_read_line (stream)) == NULL )
	{
	set_status (io_failure);
	DEBUG1 (fprintf (stderr,
	    "ebi_read_magic_number: can't read magic number\n"););
	}

    else if ( (magic_number = convert_number (line)) == BAD_NUMBER )
	{
	/* status already set */

	DEBUG1 (fprintf (stderr,
	    "ebi_read_magic_number: bad magic number of \"%s\"\n", line););
	}

    else if (magic_number != MAGIC_NUMBER)
	{
	set_status (invalid_dialogue);
	DEBUG1 (fprintf (stderr,
	    "ebi_read_magic_number: bad magic number of \"%s\"\n", line););
	}

    return (get_status () == ok);
}					/* end ebi_read_magic_number */

/*--------------------------------------------------------------------------*\
|  Routine: ebi_write_magic_number
| 
|  Description:
|     Writes an ebi magic number to an open dialogue binary file.
\*--------------------------------------------------------------------------*/

boolean ebi_write_magic_number (stream)
FILE *stream;
{
    /*
    ** It's tough to check the result of fprintf in a portable manner.
    ** This routine, therefore, will always return TRUE for now.
    */
    set_status (ok);
    fprintf (stream, "%d\n", MAGIC_NUMBER);
    return (true);
}					/* end ebi_write_magic_number */


/*--------------------------------------------------------------------------*\
|  Routine: ebi_calc_checksum
| 
|  Description:
|     Calculates a checksum by adding up N bytes in a stream.  Repositions
|     the file pointer.  Returns the checksum, or -1 on error.  Sets status
|     on error, but does not initialize it to `ok'.
\*--------------------------------------------------------------------------*/

int ebi_calc_checksum (stream, bytes)
FILE *stream;
long bytes;
{
    int c;
    int i = 0;
    int checksum = 0;
    boolean done = false;


    while (! done)
	{
	if ( (c = getc (stream)) == EOF )
	    done = true;

	else if ( (bytes > 0) && (i == bytes) )
	    done = true;

	else
	    checksum = (checksum + c) & CHECKSUM_MASK;

	i++;
	}

    if ( (bytes > 0) && (i < bytes) )
	{
	DEBUG1 (fprintf (stderr, "ebi_calc_checksum: premature EOF\n"););
	set_status (end_of_file);
	checksum = -1;
	}

    return (checksum);
}

/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_checksum
| 
|  Description:
|     Reads the checksum from an open dialogue binary file.  Assumes the
|     file is positioned at the beginning of the checksum record.
\*--------------------------------------------------------------------------*/

boolean ebi_read_checksum (stream)
FILE *stream;
{
    string token;
    int checksum_from_file;
    int calculated_checksum;
    long bytes;


    set_status (ok);

    /*
    ** Save current position before reading checksum.  When we calculate
    ** the checksum, we don't want to include the checksum bytes.
    **
    ** The checksum should be the last token on the input stream.  We can't
    ** use ebi_read_token directly, though, since it implicitly skips white
    ** space, which we have to count as part of the checksum.  So we'll skip
    ** the white space ourselves before marking our position in the file.
    ** THEN we'll read the checksum token.
    */
    bytes = ftell (stream);

    if (! ebi_skip_white (stream) )
	{
	set_status (io_failure);
	DEBUG1 (fprintf (stderr,
	    "ebi_read_checksum: can't skip white space\n"););
	}

    else if ( (bytes = ftell (stream)) < 0 )
	{
	set_status (io_failure);
	DEBUG1 (fprintf (stderr, "ebi_read_checksum: ftell failure.\n"););
	}

    else if ( (token = ebi_read_token (stream)) == NULL )
	{
	set_status (io_failure);
	DEBUG1 (fprintf (stderr, "ebi_read_checksum: can't read checksum\n"););
	}

    else if (ebi_read_token (stream) != NULL)
	{
	set_status (io_failure);
	DEBUG1 (fprintf (stderr,
	    "ebi_read_checksum: extraneous information follows checksum.\n"););
	}

    else if ( (checksum_from_file = convert_number (token)) == BAD_NUMBER )
	{
	/* status already set */

	if (strcmp (token, CHECKSUM_OVERRIDE) == 0)
	    {
	    /* Not an error.  Reset the status. */

	    set_status (ok);
	    }

	else
	    DEBUG1 (fprintf (stderr,
		"ebi_read_checksum: bad checksum of \"%s\"\n", token););
	}

    else
	{
	DEBUG2 (fprintf (stderr, "ebi_read_checksum: checksum from file = %d\n",
	    checksum_from_file););

	/* Rewind the input file and calculate the checksum. */

	rewind (stream);
	if ( (calculated_checksum = ebi_calc_checksum (stream, bytes)) < 0 )
	    {
	    /* status already set */

	    DEBUG1 (fprintf (stderr,
		"ebi_read_checksum: can't calculate checksum\n"););
	    }

	else if (calculated_checksum != checksum_from_file)
	    {
	    set_status (invalid_dialogue);
	    DEBUG1 (
		fprintf (stderr,
		    "ebi_read_checksum: checksum mismatch\n");
		fprintf (stderr,
		    "ebi_read_checksum: calculated checksum = %d\n",
		    calculated_checksum););
	    }
	}

    return (get_status () == ok);
}					/* end ebi_read_checksum */

/*--------------------------------------------------------------------------*\
|  Routine: ebi_write_checksum
| 
|  Description:
|     Re-scans the specified stream, calculating a checksum, and writes the
|     checksum to the end of the stream.
|
|     WARNING: For this routine to work properly, the file must have been
|     opened for writing with the ebi_open_dialogue_file macro!
\*--------------------------------------------------------------------------*/

boolean ebi_write_checksum (stream)
FILE *stream;
{
    boolean success = true;
    int checksum = 0;
    long bytes;

    /*
    ** This routine accomplishes its job by seeking to the beginning of the
    ** stream and reading each byte from there to the end, calculating the
    ** checksum as it goes.  When it has finished, it writes the calculated
    ** checksum, in ASCII format, to the end of the file.
    */

    bytes = ftell (stream);
    rewind (stream);  /* seek to beginning of file */

    if ( (checksum = ebi_calc_checksum (stream, bytes)) < 0)
	{
	/* status already set */

	DEBUG1 (fprintf (stderr,
	    "ebi_write_checksum: can't calculate checksum\n"););
	success = false;
	}

    else
	{
	DEBUG2 (fprintf (stderr, "ebi_write_checksum: writing checksum of %d\n",
	    checksum););

	fprintf (stream, "%d\n", checksum);
	}

    return (success);
}					/* end ebi_write_checksum */


/*--------------------------------------------------------------------------*\
|  Routine: ebi_write_integer
| 
|  Description:
|     writes a integer a stream
\*--------------------------------------------------------------------------*/
boolean ebi_write_integer (the_stream, the_integer) 
FILE *the_stream;
int the_integer;
{ 
   set_status (ok);
   fprintf (the_stream, "%d ", the_integer);
   return (TRUE);
}					/* end ebi_write_integer */


/*--------------------------------------------------------------------------*\
|  Routine: ebi_read_integer
| 
|  Description:
|     reads a integer a stream
\*--------------------------------------------------------------------------*/
int ebi_read_integer (the_stream) 
FILE *the_stream;
{  string token;
   int count;

   set_status (ok);
   if (! ebi_skip_white (the_stream))
      set_status (io_failure);
   else if ((token = ebi_read_token (the_stream)) == NULL) 
	set_status (io_failure);
   else if ((count = convert_buffer_count (token)) == BAD_NUMBER)
	set_status (io_failure);
   else
      return (count);

   return (-1);
}					/* end ebi_read_integer */



