static char rcsid [] =
        "$Header: /u/04c/mnt/integ/source/de/src/RCS/ecd.c,v 1.15 1991/07/05 22:22:17 dvk Exp $";
/*--------------------------------------------------------------------------*\
|       
|  Name:  Dialogue Editor Command Definitions (ecd)
|
|  Description:  
|     The dialogue editor command definitions contains the definition
|     of the commands supported by the DEA.  This component accesses
|     retained data stored in the dialogue editor command interpreter
|     component.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log: ecd.c,v $
 * Revision 1.15  1991/07/05  22:22:17  dvk
 * Got technologieds list to disply properly (added idx shared data component)
 *
 * Revision 1.14  1991/05/07  17:41:04  little
 * move the dialogue extern to e22.h and remove the DEBUG statements so dbx
 * does not get confused
 *
 * Revision 1.13  1991/04/11  18:09:42  rcs
 * *** empty log message ***
 *
 * Revision 1.12  91/03/23  14:02:23  ejh
 * changed transaction to out_trans in ecd_new_dialogue
 * 
 * Revision 1.11  91/03/20  23:38:54  little
 * add the new_dialogue function
 * 
 * Revision 1.10  90/09/19  16:40:20  rcs
 * no change
 * 
 * Revision 1.9  90/09/07  18:34:23  little
 * fix up the technology stuff so it works
 * 
 * Revision 1.8  90/05/21  13:32:52  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.7  90/03/23  14:22:55  rcs
 * *** empty log message ***
 * 
 * Revision 1.6  90/02/14  13:02:11  bmc
 * Now passes outgoing transaction to edg_bin_read().
 * 
 * Revision 1.5  89/12/21  16:27:47  rcs
 * fixed a bug
 * 
 * Revision 1.4  89/12/20  14:42:09  rcs
 * integration testing
 * 
 * Revision 1.3  89/12/12  07:57:00  rcs
 * fixed bugs found during integration testing
 * 
 * Revision 1.2  89/11/30  12:55:10  rcs
 * fixed spelling error
 * 
 * Revision 1.1  89/11/28  18:00:16  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*-------------------- Includes --------------------------------------------*/

#define ecd_main
#include "ecd.h"			/* command definitions		    */
#include "ehf.h"
#include "emp.h"
#include "eta.h"
#include "esd.h"
#include "ea2.h"
#include "se.h"
#include <string.h>
#include <stdio.h>

/*-------------------- Constants -------------------------------------------*/

#define DEBUG1(statement) SERPENT_DEBUG1(ecd_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(ecd_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(ecd_debug, statement)

#define VC_ONLY evc_vc_only
#define SUBTREE evc_entire_hierarchy



/*--------------------------------------------------------------------------*\
|  Routine:  ecd_load_dialogue
| 
|  Description:  
|     The load command is used to load a dialogue into the application.
\*--------------------------------------------------------------------------*/
edg_dialogue ecd_load_dialogue(command, transaction)
command_sdd *command;
transaction_type transaction;
{ 					/* local type definitions	    */
  parameter_sdd *parm;

  edg_dialogue dialogue;
  string dialogue_name;

  set_status(ok);
/*
   Get the parameter records and values.
*/
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    command->parameters
  );

			/* check the parameter for correct type */
   if ((idd_data_types) ((parm-> value).type) != idd_string)
      {set_status (bad_parameter);
      return NULL;
      }

  dialogue_name = (string)((parm->value).body);

/*
   If the binary file exists set status and return.
*/
  if (!edg_bin_file_exists(dialogue_name)) {	/* bin file exists	*/
    set_status(file_not_found);
    return NULL;
  }					/* end if bin file exists	*/
/*
   Read in the binary file.
*/
  dialogue = edg_bin_read(dialogue_name, transaction);
  rcheck_status("ecd_load_dialogue:  bad status from edg_bin_read.", NULL);
/*
   Add copy of dialogue to shared data.
*/
  esd_add_to_shared_data(
    transaction,
    EA2_NODE,
    dialogue
  );
  rcheck_status("ecd_load_dialogue:  bad status from evc_add_to_sd.", NULL);

  return dialogue;
} 					/* end ecd_load_dialogue	    */


/*--------------------------------------------------------------------------*\
|  Routine:  ecd_new_dialogue
| 
|  Description:  
|     creates a new, empty dialogue
\*--------------------------------------------------------------------------*/
edg_dialogue ecd_new_dialogue (command, out_trans)
command_sdd *command;
transaction_type out_trans;
{  edg_dialogue new_dialogue;
   dialogue_sdd dialogue_sd;
   id_type id;

   new_dialogue = edg_create_dialogue ();
   check_status ("ecd_new_dialogue:  bad status from edg_create_dialogue.");
			/* Add the dialogue shared data element to shared
			   data.  */
   dialogue_sd = esd_set_dialogue_sdd (new_dialogue);
   check_status ("ecd_new_dialogue:  bad status from esd_set_dialogue_sdd");
   id = add_shared_data (out_trans, "dialogue_sdd", NULL, &dialogue_sd);
   check_status ("ecd_new_dialogue:  bad status from add_shared_data.");
			/* Create a mapping between the shared data element and
			   the dialogue element.  */
   emp_create_map (id, new_dialogue);
   check_status ("ecd_new_dialogue:  bad status from emp_create_map.");

   return (new_dialogue);
}					/* end ecd_new_dialogue */


/*--------------------------------------------------------------------------*\
|  Routine:  ecd_save_dialogue
| 
|  Description:  
|     The save command is used to write a dialogue to permanent store.  The
|     dialogue name is saved in the dialogue shared data element.
\*--------------------------------------------------------------------------*/
void ecd_save_dialogue(command, transaction)
command_sdd *command;
transaction_type transaction;
{ 					/* local type definitions	    */
  parameter_sdd *parm;

  edg_dialogue dialogue;
  id_type dialogue_id;
  boolean overwrite;
  string dialogue_name;
/*
   Initialize.
*/
  set_status(ok);			/* begin 			    */
/*
   The first parameter is the id of the dialogue.
*/
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    command->parameters
  );
  check_parm_type(
    parm, 
    idd_id,
    "ecd_save_dialogue: parm is not an id.\n"
  );

  dialogue_id = *((id_type *)((parm->value).body));

/*
  The second parameter is the name of the dialogue.
*/
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    parm->next
  );
  check_parm_type(
    parm, 
    idd_string,
    "ecd_save_dialogue: parm is not a string.\n"
  );

  dialogue_name = (string)((parm->value).body);

/*
   The third parameter is a boolean which indicates if we 
   should overwrite an existing file or not.
*/
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    parm->next
  );
  check_parm_type(
    parm, 
    idd_boolean,
    "ecd_save_dialogue: parm is not a boolean.\n"
  );

  overwrite = *((boolean *)((parm->value).body));

/*
   Get the actual dialogue element corresponding to the dialogue id.
*/
    dialogue = emp_get_de(dialogue_id);
    check_status("ecd_save_dialogue:  bad status from emp_get_de.");
/*
   If overwrite is true, write out the file.
*/
  if (overwrite) {

    edg_bin_write(dialogue, dialogue_name);
    check_status("ecd_save_dialogue:  bad status from edg_bin_write.");

  }					/* end if overwrite		*/
  else {				/* else NOT overwrite		*/
/*
   If the binary file exists set status and return.
*/
    if (edg_bin_file_exists(dialogue_name)) {	/* bin file exists	*/
      set_status(file_already_exists);
      return;
    }					/* end if bin file exists	*/
/*
  Else if the binary file does not exist, write it out.
*/
    else {				/* bin file does not exist	*/

      edg_bin_write(dialogue, dialogue_name);
      check_status("ecd_save_dialogue:  bad status from edg_bin_write.");

    }					/* end if bin file does not exist   */
    
  }					/* end else NOT overwrite	*/
/*
   Return.
*/
  return;
} 					/* end ecd_save_dialogue	    */

/*--------------------------------------------------------------------------*\
|  Routine:  ecd_unload_dialogue
| 
|  Description:  
|     The unload command is used remove a specified dialogue from both shared
|     data and application memory.
\*--------------------------------------------------------------------------*/
void ecd_unload_dialogue(command, transaction)
command_sdd *command;
transaction_type transaction;
{ 					/* local type definitions	    */
  parameter_sdd *parm;

  id_type dialogue_id;
  edg_dialogue dialogue;
/*
   Initialize.
*/
  set_status(ok);			/* begin 			    */
/*
   Get the parameter records and values.
*/
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    command->parameters
  );
  check_parm_type(
    parm, 
    idd_id,
    "ecd_unload_dialogue: parm is not an id.\n"
  );

  dialogue_id = *((id_type *)((parm->value).body));
/*
   Get the actual dialogue element corresponding to the dialogue id.
*/
  dialogue = emp_get_de(dialogue_id);
  check_status("ecd_unload_dialogue:  bad status from emp_get_de.");
/*
   Remove entire dialogue from shared data.
*/
  esd_remove_from_shared_data(
    transaction,
    EA2_SUBTREE,
    dialogue
  );
  check_status("ecd_load_dialogue:  bad status from esd_remove_from_sd.");
/*
   Destroy dialogue and free storage.
*/
  edg_destroy_dialogue(dialogue, transaction);
  check_status("ecd_unload_dialogue:  bad status from edg_destroy_dialogue.");
/*
   Return.
*/
  return;
} 					/* end ecd_unload_dialogue	    */


/*--------------------------------------------------------------------------*\
|  Routine:  ecd_import_dialogue
| 
|  Description:  
|     The ecd_import_dialogue routine is used to input a dialogue from
|     Slang source.
\*--------------------------------------------------------------------------*/
edg_dialogue ecd_import_dialogue(command, transaction)
command_sdd *command;
transaction_type transaction;
{ 					/* local type definitions	    */
  parameter_sdd *parm;

  edg_dialogue dialogue;
  string dialogue_name;
/*
   Initialize.
*/
  set_status(ok);			/* begin 			    */
/*
   Get the parameter records and values.
*/
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    command->parameters
  );





                        /* check the parameter for correct type */
   if ((idd_data_types) ((parm-> value).type) != idd_string)
      {set_status (bad_parameter);
      return NULL;
      }

  dialogue_name = (string)((parm->value).body);

/*
   If the binary file exists set status and return.
*/
  if (!edg_src_file_exists(dialogue_name)) {	/* bin file exists	*/
    set_status(file_not_found);
    return NULL;
  }					/* end if bin file exists	*/
/*
   Read in the binary file.
*/
  dialogue = edg_src_read(dialogue_name);
  rcheck_status("ecd_import_dialogue:  bad status from edg_src_write.", NULL);
/*
   Add copy of dialogue to shared data.
*/
  esd_add_to_shared_data(
    transaction,
    EA2_NODE,
    dialogue
  );
  rcheck_status("ecd_import_dialogue:  bad status from evc_add_to_sd.", NULL);
/*
   Return.
*/
  return dialogue;
} 					/* end ecd_import_dialogue	    */

/*--------------------------------------------------------------------------*\
|  Routine:  ecd_export_dialogue
| 
|  Description:  
|     The ecd_export_dialogue routine is used to save a dialogue as Slang.
\*--------------------------------------------------------------------------*/
void ecd_export_dialogue(command, transaction)
command_sdd *command;
transaction_type transaction;
{ 					/* local type definitions	    */
  parameter_sdd *parm;

  edg_dialogue dialogue;
  id_type dialogue_id;
  boolean overwrite;
  string dialogue_name;
/*
   Initialize.
*/
  set_status(ok);			/* begin 			    */
/*
   The first parameter is the id of the dialogue.
*/
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    command->parameters
  );
  check_parm_type(
    parm, 
    idd_id,
    "ecd_export_dialogue: parm is not an id.\n"
  );

  dialogue_id = *((id_type *)((parm->value).body));

/*
  The second parameter is the name of the dialogue.
*/
  dialogue_name = (string)((parm->value).body);
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    parm->next
  );
  check_parm_type(
    parm, 
    idd_string,
    "ecd_export_dialogue: parm is not a string.\n"
  );

  dialogue_name = (string)((parm->value).body);

/*
   The third parameter is a boolean which indicates if we 
   should overwrite an existing file or not.
*/
  dialogue_name = (string)((parm->value).body);
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    parm->next
  );
  check_parm_type(
    parm, 
    idd_boolean,
    "ecd_export_dialogue: parm is not a boolean.\n"
  );

  overwrite = *((boolean *)((parm->value).body));

/*
   Get the actual dialogue element corresponding to the dialogue id.
*/
    dialogue = emp_get_de(dialogue_id);
    check_status("ecd_unload_dialogue:  bad status from emp_get_de.");
/*
   If overwrite is true, write out the file.
*/
  if (overwrite) {

    edg_src_write(dialogue, dialogue_name);
    check_status("ecd_export_dialogue:  bad status from edg_src_write.");

  }					/* end if overwrite		*/
  else {				/* else NOT overwrite		*/
/*
   If the source file exists set status and return.
*/
    if (!edg_src_file_exists(dialogue_name)) {	/* bin file exists	*/
      set_status(file_already_exists);
      return;
    }					/* end if bin file exists	*/
/*
  Else if the source file does not exist, write it out.
*/
    else {				/* bin file does not exist	*/

      edg_src_write(dialogue, dialogue_name);
      check_status("ecd_export_dialogue:  bad status from edg_src_write.");

    }					/* end if bin file does not exist   */
    
  }					/* end else NOT overwrite	*/
/*
   Return.
*/
  return;
} 					/* end ecd_export_dialogue	    */


/*--------------------------------------------------------------------------*\
|  Routine:  ecd_get_dialogue_technologies
| 
|  Description:  
|     The ecd_get_dialogue_technologies is used to get the names
|     of the technologies which have and haven't been included in 
|     a specific dialogue.
\*--------------------------------------------------------------------------*/
void ecd_get_dialogue_technologies(command, transaction)
command_sdd *command;
transaction_type transaction;
{ 					/* local type definitions	    */
  parameter_sdd *parm;

  id_type dialogue_id;
  edg_dialogue dialogue;
  LIST technology_list;
  eta_technology technology;
  int idx = 1;
  technology_sdd technology_sd;
/*
   Initialize.
*/
  set_status(ok);			/* begin 			    */
/*
   Get the parameter records and values.
*/
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    command->parameters
  );
  check_parm_type(
    parm, 
    idd_id,
    "ecd_get_dialogue_technologies: parm is not an id.\n"
  );

  dialogue_id = *((id_type *)((parm->value).body));
/*
   Get the actual dialogue element corresponding to the dialogue id.
*/
  dialogue = emp_get_de(dialogue_id);
  check_status("ecd_get_dialogue_technologies:  bad status from emp_get_de.");
/*
   Destroy dialogue and free storage.
*/
/*
   Get technologies.
*/
  technology_list = eta_get_technologies(false);
  check_status("ect_get_technologies:  bad status from eta_get_tech.");
/*
   Add list of subvcs to shared memory.
*/
  if (technology_list != NULLlist) {

    loop_through_list(technology_list, technology, eta_technology) {

      strcpy(technology_sd.short_name, technology->short_name);
      strcpy(technology_sd.long_name, technology->long_name);
      technology_sd.included = edg_is_a_technology(dialogue, technology-> short_name);
      technology_sd.idx = idx++;
      technology_sd.parent = dialogue_id;

      (void)add_shared_data(
        transaction,
        "technology_sdd",
        NULL,
        &technology_sd
      );
      check_status("ect_get_technologies:  bad status from add_shared_data.");

    }                           /* end loop through technology_list     */

    free_list(technology_list, NOT_NODES);

  }                             /* end if technology list not NULL      */
/*
   Return.
*/
  return;
} 				/* end ecd_get_dialogue_technologies	*/

/*--------------------------------------------------------------------------*\
|  Routine:  ecd_set_technology
| 
|  Description:  
|     The ecd_set_technology command is to select and deselect technologies
|     for incluscion in a dialogue.
\*--------------------------------------------------------------------------*/
void ecd_set_technology(command, transaction)
command_sdd *command;
transaction_type transaction;
{ 					/* local type definitions	    */
  parameter_sdd *parm;

  edg_dialogue dialogue;
  id_type dialogue_id;
  boolean selected;
  string technology_name;
  eta_technology technology;
/*
   Initialize.
*/
  set_status(ok);			/* begin 			    */
/*
   The first parameter is the id of the dialogue.
*/
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    command->parameters
  );
  check_parm_type(
    parm, 
    idd_id,
    "ecd_set_technology: parm is not an id.\n"
  );

  dialogue_id = *((id_type *)((parm->value).body));

/*
  The second parameter is the name of the dialogue.
*/
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    parm->next
  );
  check_parm_type(
    parm, 
    idd_string,
    "ecd_set_technology: parm is not a string.\n"
  );

  technology_name = (string)((parm->value).body);

/*
   The third parameter is a boolean which indicates if we 
   should selected an existing file or not.
*/
  parm = (parameter_sdd *)get_from_hashtable(
    parameter_table,
    parm->next
  );
  check_parm_type(
    parm, 
    idd_boolean,
    "ecd_set_technology: parm is not a boolean.\n"
  );

  selected = *((boolean *)((parm->value).body));

/*
   Get the actual dialogue element corresponding to the dialogue id.
*/
  dialogue = emp_get_de(dialogue_id);
  check_status("ecd_set_technology:  bad status from emp_get_de.");
/*
   Add or remove the technology based on select.
*/
  if (selected) {				/* selected 		    */
    edg_add_technology (dialogue, technology_name);
    check_status("ecd_set_technology:  bad status from edg_add_technology.");
  }						/* end if selected	    */
  else {					/* else not selected	    */
    edg_remove_technology(dialogue, technology_name);
    check_status("ecd_set_technology:  bad status from edg_remove_tech.");
  }						/* end else not selected    */
/*
   Return.
*/
  return;
} 					/* end ecd_set_technology	    */

