static char rcsid [] =
        "$Header: /usr/users/little/serpent/mips/de/src/RCS/ect.c,v 1.8 1991/05/07 17:42:52 little Exp $";
/*--------------------------------------------------------------------------*\
|     
|  Name:  Editor Technology Commands (ect)
|
|  Description:  
|     The dialogue editor technology command definitions specify the
|     implementation of the technology commands supported by the
|     application.
|     
|     This particular abstraction contains both the implementation
|     of the commands and the code to write the information out to 
|     shared data.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log: ect.c,v $
 * Revision 1.8  1991/05/07  17:42:52  little
 * move the dialogue extern to e22.h and change name to e22_dialogue.
 * remove the DEBUG statements so dbx does not get confused
 *
 * Revision 1.7  1990/11/29  09:54:55  rcs
 * symbol table fix?
 *
 * Revision 1.6  90/10/23  14:03:39  little
 * 
 * change call to eta routines to expect alists instead of lists
 * 
 * Revision 1.5  90/05/21  13:33:27  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.4  89/12/20  14:43:08  rcs
 * integration testing
 * 
 * Revision 1.3  89/12/12  09:06:25  rcs
 * fixed bugs found during integration testing
 * 
 * Revision 1.2  89/11/30  13:49:15  rcs
 * fixed spelling error
 * 
 * Revision 1.1  89/11/28  18:38:38  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*-------------------- Includes --------------------------------------------*/

#define ect_main
#include "ect.h"			/* command definitions		    */
#include "edg.h"
#include "ehf.h"
#include "eta.h"
#include "se.h"
#ifndef memoryPack
#define memoryPack
#endif
#include "listPack.h"
#include <stdio.h>
#include <string.h>

/*-------------------- Constants -------------------------------------------*/

#define DEBUG1(statement) SERPENT_DEBUG1(ect_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(ect_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(ect_debug, statement)

#define VC_ONLY evc_vc_only
#define SUBTREE evc_entire_hierarchy



/*--------------------------------------------------------------------------*\
|  Routine:  ect_get_technologies
| 
|  Description:  
|     Command to get a list of available technologies.  The application 
|     does not maintain a mapping for these technologies since the data
|     is never changed.
\*--------------------------------------------------------------------------*/
void ect_get_technologies(transaction)
transaction_type transaction;
{ 					/* local type definitions	    */
  LIST technology_list;
  eta_technology technology;
  avail_tech_sdd avail_tech_sd;
/*
   Initialize.
*/
  set_status(ok);			/* begin 			    */
/*
   Get technologies.
*/
  technology_list = eta_get_technologies(true);
  check_status("ect_get_technologies:  bad status from eta_get_tech.");
/*
   Add list of subvcs to shared memory.
*/
  if (technology_list != NULLlist) {

    loop_through_list(technology_list, technology, eta_technology) {

      strcpy(avail_tech_sd.short_name, technology->short_name);
      strcpy(avail_tech_sd.long_name, technology->long_name);

      (void)add_shared_data(
        transaction, 
        "avail_tech_sdd", 
        NULL, 
        &avail_tech_sd
      );
      check_status("ect_get_technologies:  bad status from add_shared_data.");

    }				/* end loop through technology_list	*/

    free_list(technology_list, NOT_NODES);

  }				/* end if technology list not NULL	*/
/*
   Return.
*/
  return;
} 				/* end ect_get_technologies		    */

/*--------------------------------------------------------------------------*\
|  Routine:  ect_get_object_types
| 
|  Description:  
|     Gets the list of object_types associated with a particular technology
|     in a particular technology.
\*--------------------------------------------------------------------------*/
void ect_get_object_types (command, transaction)
command_sdd *command;
transaction_type transaction;
{  parameter_sdd *parm1;		/* technology name		    */
   string tech_name;
   WIDG *widget;
   ALIST object_name_alist;
   obj_type_sdd obj_type_sd;

   set_status(ok);
			/* Get the technology name */
   parm1 = (parameter_sdd *)get_from_hashtable (parameter_table, 
    						command->parameters);
   tech_name = (string)((parm1->value).body);

			/* Get list of objects */
   object_name_alist = eta_get_objects (tech_name, true);
   check_status("ect_get_object_types:  bad status from eta_get_objects.");

			/* Add list of object to shared memory */
   if (object_name_alist != (ALIST) NULLlist)
      {loop_through_alist (object_name_alist, widget, WIDG *) {
         strcpy (obj_type_sd.name, widget->name);
         strcpy (obj_type_sd.tech, tech_name);
         strcpy (obj_type_sd.abbr, widget->userdef);
         add_shared_data (transaction, "obj_type_sdd", NULL, &obj_type_sd);
         check_status (
                    "ect_get_object_types:  bad status from add_shared_data.");
         }
      }				/* end if object list not NULL	*/
   return;
} 					/* end ect_get_object_types	    */


/*--------------------------------------------------------------------------*\
|  Routine:  ect_get_attributes
| 
|  Description:  
|     Gets the list of attributes associated with a particular object type
|     in a particular technology.
\*--------------------------------------------------------------------------*/
void ect_get_attributes(command, transaction)
command_sdd *command;
transaction_type transaction;
{  parameter_sdd *parm2, *parm1;
   string tech_name, object_name, att_name;
   ALIST attribute_alist;
   attribute_type_sdd attribute_type_sd;

   set_status(ok);
			/* Get the technology name.  */
   parm1 = (parameter_sdd *)get_from_hashtable (parameter_table, 
    						command->parameters);
   tech_name = (string)((parm1->value).body);

			/* Get the object name.  */
   parm2 = (parameter_sdd *)get_from_hashtable (parameter_table,
    						parm1->next);
   object_name = (string)((parm2->value).body);

			/* Get list of attributes.  */
   attribute_alist = eta_get_attributes (tech_name, object_name, true);

			/* Add list of subvcs to shared memory */
   if (attribute_alist != (ALIST) NULLlist)
      {loop_through_alist (attribute_alist, att_name, string)
         {strcpy (attribute_type_sd.name, att_name);
         strcpy (attribute_type_sd.obj_type, object_name);
         strcpy (attribute_type_sd.tech, tech_name);
         add_shared_data (transaction, "attribute_type_sdd", NULL,
			  &attribute_type_sd);
         check_status("ect_get_attributes:  bad status from add_shared_data.");
         }
			/* remember, we should free lists returned by
			   eta_get_attributes */
      free_alist (attribute_alist, NOT_NODES);
      }

   return;
} 					/* end ect_get_attributes	    */


/*--------------------------------------------------------------------------*\
|  Routine:  ect_get_methods
| 
|  Description:  
|     Gets the list of methods associated with a particular object type
|     in a particular technology.
\*--------------------------------------------------------------------------*/
void ect_get_methods (command, transaction)
command_sdd *command;
transaction_type transaction;
{  parameter_sdd *parm1;		/* technology name		    */
   parameter_sdd *parm2;		/* attribute name		    */
   string meth_name, tech_name, object_name;
   ALIST method_name_alist;
   method_type_sdd method_type_sd;

   set_status (ok);
			/* Get the technology name.  */
   parm1 = (parameter_sdd *) get_from_hashtable (parameter_table, 
    						 command->parameters);
   tech_name = (string)((parm1->value).body);

			/* Get the object name.  */
   parm2 = (parameter_sdd *) get_from_hashtable (parameter_table,
    						 parm1->next);
   object_name = (string)((parm2->value).body);

			/* Get list of methods */
   method_name_alist = eta_get_methods (tech_name, object_name, true);
			/* Add list of methods to shared memory.  */
   if (method_name_alist != (ALIST) NULLlist)
      {loop_through_alist (method_name_alist, meth_name, string)
         {strcpy (method_type_sd.name, meth_name);
         strcpy (method_type_sd.obj_type, object_name);
         strcpy (method_type_sd.tech, tech_name);
         add_shared_data (transaction, "method_type_sdd", NULL,
			  &method_type_sd);
         check_status ("ect_get_methods:  bad status from add_shared_data.");
         }
      }				/* end if method list not NULL	*/
   return;
} 					/* end ect_get_methods		    */

