static char rcsid [] = 
	"$Header: /usr/users/little/serpent/mips/de/src/RCS/edg.c,v 1.26 1991/05/30 14:53:59 little Exp $";

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Dialogue Abstraction (edg)
|
|  Description:  
|     The dialogue editor dialogue abstraction provides operations on the
|     dialogue abstract type.  These operations include create, destroy, 
|     and various get and modify operations.
|
|     A dialogue is composed of the following elements:
|
|     WHAT		TYPE		  DESCRIPTION
|     _______________________________________________________________________
|
|     file_name		string            name of file containing dialogue
|     EVE VC		evc_vc		  top level VC for dialogue
|     prologue		buffer            the prologue comments
|     macros		buffer            macros preceding the top level VC
|     externs 		buffer		  external definitions
|     shared_data	buffer            application and dialogue s.d. info
|     technologies	list of strings   names of all the technologies
|     name		string		  the name of the dialogue
|     objects intances	list of instances list of object instances
|
|  Component of:
|     Dialogue Editor Layer.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|

$Log: edg.c,v $
 * Revision 1.26  1991/05/30  14:53:59  little
 * set e22_dialogue in internal_src_read
 *
 * Revision 1.25  1991/05/14  22:29:06  little
 * fix things so the parent att works when instinstiating
 *
 * Revision 1.24  1991/05/07  17:43:38  little
 * remove the DEBUG statements so dbx does not get confused
 * add object counting functionality
 *
 * Revision 1.23  1991/03/20  23:37:15  little
 * no changes
 *
 * Revision 1.22  90/12/21  14:18:39  ejh
 * strings.h => string.h
 * 
 * Revision 1.21  90/11/29  09:55:30  rcs
 * symbol table fix
 * 
 * Revision 1.20  90/09/11  08:33:01  little
 * add third parameter to esi_buffer_src_write calls
 * 
 * 
 * Revision 1.19  90/09/07  18:25:54  little
 * add the macros stuff to src out
 * 
 * Revision 1.18  90/08/30  17:37:09  little
 * make those who call get_prop & donot care about status, set it ok
 * 
 * Revision 1.17  90/08/29  18:30:10  little
 * add the counter to the dialogue sd record
 * 
 * Revision 1.16  90/08/06  16:35:23  little
 * change things so dialogues go through scpp instead of cpp before
 * importation.
 * 
 * add stuff to cleanup the symbol table after dialogue importation
 * 
 * Revision 1.15  90/07/05  15:05:12  little
 *  more structure and visualizer parts integration
 * 
 * Revision 1.14  90/06/16  12:33:01  little
 * new editor import parameter to the c22_compile routine
 * 
 * fix the hardwired saw.ill src printout to happen only if there are no
 * technologies in the dialogue
 * 
 * Revision 1.13  90/06/14  13:54:10  little
 *  more changes to integrate visulization and structure editors
 * 
 * Revision 1.12  90/06/06  18:04:52  little
 * parital update to support combination of visual and structural editor
 * 
 * Revision 1.11  90/05/21  13:34:00  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.10  90/05/01  11:38:57  rcs
 * changed something long ago and far away
 * 
 * Revision 1.9  90/03/19  16:44:20  rcs
 * sleazed ill file generation in export command
 * 
 * Revision 1.8  90/02/14  22:29:13  ejh
 * removed define for list_of_displays_prop
 * 
 * Revision 1.7  90/02/14  13:00:16  bmc
 * Now handles object instances;
 * 
 * Revision 1.6  89/12/18  15:32:55  little
 * fixed soome other shit
 * 
 * Revision 1.5  89/12/18  14:19:08  little
 * fixed some shit that was wrong
 * 
 * Revision 1.4  89/12/11  16:11:55  little
 * fix bin read/write and add bin & src file exists routined
 * 
 * Revision 1.3  89/11/29  10:11:22  bmc
 * Now uses ebi abstraction rather than uio abstraction
 * 
 * Revision 1.2  89/11/29  09:45:26  bmc
 * no changes
 * 
 * Revision 1.1  89/11/29  09:37:42  bmc
 * Initial revision
 * 

|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*---------------------------- Includes ------------------------------------*/

#define edg_main
#include "cus.h"
#include "edg.h"                /* dialogue abstract type	            */
#include "eob.h"
#include "c22.h"
#include "serpent.h"		/* General Serpent definitions		    */
#include "ubo.h"		/* Buffer definitions and operations	    */
#include "ebi.h"		/* Editor binary I/O routines         	    */
#include "u22.h"		/* General purpose Serpent routines	    */
#include "e22.h"
#include "evc_cmd.h"		/* Editor VC abstraction		    */
#include "evc.h"		/* Editor VC abstraction */
#include "eat.h"
#include "ea2.h"		/* Editor Global header file	 	    */
#include "est.h"		/* Editor symbol table routines 	    */
#include "esi.h"		/* Editor source I/O abstraction	    */
#include "eta.h"		/* Editor available technologies abstraction*/
#include "eoi.h"		/* Editor object instances abstraction      */
#include <string.h>		/* C string functions 			    */

/*---------------------------- Constants -----------------------------------*/

#undef DEBUG1
#undef DEBUG2
#undef DEBUG3
#define DEBUG1(statement) SERPENT_DEBUG1(edg_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(edg_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(edg_debug, statement)

#define END_OF_PROLOGUE			"|||"
#define END_OF_PROLOGUE_LEN		(sizeof (END_OF_PROLOGUE) - 1)
#define TOP_LEVEL_VC_PREFIX		"EVE"
#define EDG_DEFAULT_FILE_NAME           UNDEFINED_STRING

#define ILL_FILE_EXTENSION		".ill"
#define INCLUDE_LEAD_IN			'#'
#define INCLUDE_TOKEN			"include"

#define INCLUDE_START_FILE_NAME		"\"<"
#define INCLUDE_END_FILE_NAME		"\">"

#define CPP_INCLUDE_OPTION		"-I"
#define CPP_COMMAND			"scpp"

#define WHITE_SPACE			" \t\n\f"

#define SRC_EXT ".sl"
#define BIN_EXT ".slb"
#define make_file_name(file_name, dial_name, ext) \
   {\
   if ((file_name = make_node (strlen (dial_name) + strlen (ext) + 1)) != NULL)\
       {\
       (void) strcpy (file_name, dial_name); \
       (void) strcat (file_name, ext); \
       }\
   }


#define write_buffer(stream, buffer, the_selector) \
{  ebi_buffer a_buffer; \
   a_buffer.actual_buffer = buffer; \
   a_buffer.selector = the_selector; \
   if (!ebi_write_buffer (stream, a_buffer)) \
      {set_status (system_operation_failed); \
      return; \
      } \
}

#define write_open_record(stream) \
   if (!ebi_write_open_record (stream)) \
      {set_status (system_operation_failed); \
      return; \
      }

#define write_close_record(stream) \
   if (!ebi_write_close_record (stream)) \
      {set_status (system_operation_failed); \
      return; \
      }

#define bum_dialogue(the_status) \
   {set_status (the_status); \
   return (EDG_NO_DIALOGUE); \
   };

#define read_open_record(stream) \
   if (!(ebi_read_token_char (stream) == EBI_OPEN_RECORD)) \
      bum_dialogue (system_operation_failed);


#define read_close_record(stream) \
   if (!(ebi_read_token_char (stream) == EBI_CLOSE_RECORD)) \
      bum_dialogue (system_operation_failed);

#define read_buffer(stream, some_buffer, the_selector) \
   some_buffer = ebi_read_buffer (stream); \
   if (some_buffer.selector != the_selector) \
      bum_dialogue (system_operation_failed);



/*------------------------ Local Routines ----------------------------------*/
/*--------------------------------------------------------------------------*\
|  Routine: att_is_string_type
|
|  Description:
|     determines if the parameter att is of string type
\*--------------------------------------------------------------------------*/
static boolean att_is_string_type (candidate_att)
string candidate_att;
{

if (!strcmp (candidate_att, "border_color")  ||
    !strcmp (candidate_att, "background_color")  ||
    !strcmp (candidate_att, "method")  ||
    !strcmp (candidate_att, "foreground_color")  ||
    !strcmp (candidate_att, "font")  ||
    !strcmp (candidate_att, "label_text")  ||
    !strcmp (candidate_att, "text_buffer"))
   return (TRUE);
 else
   return (FALSE);
}					/* end att_is_string_type */


/*--------------------------------------------------------------------------*\
|  Routine: edg_create_dialogue
| 
|  Description:  
|     Creates a dialogue.
\*--------------------------------------------------------------------------*/
edg_dialogue edg_create_dialogue ()
{                                       /* local type definitions           */
    cus_sym dialogue_ste;
    buffer dummy_buffer;
  
  
    set_status (ok);
  
    cus_create_sym_tab (/* redefine_flag = */ TRUE);

    dialogue_ste = cus_gensym (TOP_LEVEL_VC_PREFIX, vc_sym, NO_SEED);
    rcheck_status ("edg_create_dialogue: can't generate symbol for VC.",
	EDG_NO_DIALOGUE);
  
  				    /* Initialize all fields */
  
    est_set_string_property (dialogue_ste,
			     file_name_prop,
			     EDG_DEFAULT_FILE_NAME);

			/* technologies_prop & object_instance_count_prop are
			   two parallel lists.  technologies_prop contains
			   the short names of all technologies the dialogue
			   "uses" and the corresponding entry on the
			   object_count_prop list is the number of object
			   instances & templates for that technology */
    cus_put_prop (dialogue_ste, technologies_prop, make_list ());
    cus_put_prop (dialogue_ste, object_count_prop, make_list ());

    edg_set_vc (dialogue_ste, evc_create_vc ());
    cus_put_prop (dialogue_ste, dialogue_element_type_prop, dialogue_element);

				    /*
				       It's okay to use a local buffer for the
				       buffer properties, since
				       est_set_buffer_property stores a copy
				       of the buffer, not the buffer itself.
				    */

    ubo_init_buffer (dummy_buffer);

    est_set_buffer_property (dialogue_ste,
			     prologue_comments_prop,
			     dummy_buffer);
    est_set_buffer_property (dialogue_ste, macros_prop,	dummy_buffer);
    est_set_buffer_property (dialogue_ste, externs_prop, dummy_buffer);
    est_set_buffer_property (dialogue_ste, shared_data_prop, dummy_buffer);

				    /*
				       Create unique context for dialogue.
				       Saving context handle is a hook for
				       possible future stuff.
				    */
  
    cus_put_prop (dialogue_ste,
		  dialogue_context_prop,
		  cus_create_nu_context ());

    edg_set_count (dialogue_ste, 0);

    return ( (edg_dialogue) dialogue_ste );
}                                       /* end edg_create_dialogue          */


/*--------------------------------------------------------------------------*\
|  Routine: edg_destroy_dialogue
| 
|  Description:  
|     Destroys an existing dialogue.
\*--------------------------------------------------------------------------*/
void edg_destroy_dialogue (dialogue, out_transaction)
edg_dialogue dialogue;
transaction_type out_transaction;
{
    evc_vc vc;
    LIST list;
    object_instance_sdd *obj_inst;


    set_status (ok);

    est_clear_buffer_property ((cus_sym) dialogue, prologue_comments_prop);
    est_clear_buffer_property ((cus_sym) dialogue, macros_prop);
    est_clear_buffer_property ((cus_sym) dialogue, externs_prop);
    est_clear_buffer_property ((cus_sym) dialogue, shared_data_prop);
    est_clear_string_property ((cus_sym) dialogue, file_name_prop);

    if ( (vc = edg_get_vc (dialogue)) != EVC_NO_VC )
	evc_destroy_vc (vc, EVC_ENTIRE_HIERARCHY, out_transaction);

    est_free_list_property ((cus_sym) dialogue, technologies_prop);

					/* Free dialogue record.  */
    free_node (dialogue);

    return;
}                                       /* end edg_destroy_dialogue         */


/*--------------------------------------------------------------------------*\
|  Routine: edg_is_dialogue
| 
|  Description:  
|     Returns true if the specified handle is a valid dialogue handle and
!     false if it is not.
\*--------------------------------------------------------------------------*/
boolean edg_is_dialogue (dialogue)
edg_dialogue dialogue;
{

    ea2_element_type element_type;

    set_status (ok); 

    element_type = (ea2_element_type) cus_get_prop (
      (cus_sym)dialogue, dialogue_element_type_prop
    );

    return (element_type == dialogue_element);
}                                       /* end edg_is_dialogue          */

/*--------------------------------------------------------------------------*\
|  Routine: edg_get_vc
| 
|  Description:  
|     Gets the top level vc associated with a given dialogue.
\*--------------------------------------------------------------------------*/
evc_vc edg_get_vc (dialogue)
edg_dialogue dialogue;
{

    set_status (ok); 

    return (evc_vc) cus_get_prop ((cus_sym) dialogue, eve_vc_prop);
}                                       /* end edg_get_vc          */


/*--------------------------------------------------------------------------*\
|  Routine: edg_set_vc
| 
|  Description:  
|     Sets the top level vc associated with a given dialogue.
\*--------------------------------------------------------------------------*/
void edg_set_vc (dialogue, vc)
edg_dialogue dialogue;
evc_vc vc;
{
    set_status (ok);

    cus_put_prop ((cus_sym) dialogue, eve_vc_prop, (caddr_t) vc);
    evc_set_dialogue (vc, dialogue);

    return;
}


/*--------------------------------------------------------------------------*\
|  Routine: edg_set_file_name
| 
|  Description:  
|     Sets the file name associated with a dialogue.
\*--------------------------------------------------------------------------*/
void edg_set_file_name (dialogue, file_name)
edg_dialogue dialogue;
string file_name;
{
    set_status (ok);

    est_set_string_property ((cus_sym) dialogue, file_name_prop, file_name);

    return;
}


/*--------------------------------------------------------------------------*\
|  Routine: edg_get_file_name
| 
|  Description:  
|     Gets the file name associated with a dialogue.
\*--------------------------------------------------------------------------*/
string edg_get_file_name (dialogue)
edg_dialogue dialogue;
{
    set_status (ok);

    return (string) cus_get_prop ((cus_sym) dialogue, file_name_prop);
}


/*--------------------------------------------------------------------------*\
|  Routine: edg_set_prologue
| 
|  Description:  
|     Sets the prologue associated with a dialogue.
\*--------------------------------------------------------------------------*/
void edg_set_prologue (dialogue, prologue)
edg_dialogue dialogue;
buffer prologue;
{
    set_status (ok);

    est_set_buffer_property ((cus_sym) dialogue,
			     prologue_comments_prop,
			     prologue);

    return;
}

/*--------------------------------------------------------------------------*\
|  Routine: edg_get_prologue
| 
|  Description:  
|     Gets the prologue associated with a dialogue.
\*--------------------------------------------------------------------------*/
buffer edg_get_prologue (dialogue)
edg_dialogue dialogue;
{
    set_status (ok);
    return est_get_buffer_property ((cus_sym) dialogue,
				    prologue_comments_prop);
}

/*--------------------------------------------------------------------------*\
|  Routine: edg_set_macros
| 
|  Description:  
|     Sets the macros buffer associated with a dialogue.
\*--------------------------------------------------------------------------*/
void edg_set_macros (dialogue, macros)
edg_dialogue dialogue;
buffer macros;
{
    set_status (ok);

    est_set_buffer_property ((cus_sym) dialogue, macros_prop, macros);

    return;
}

/*--------------------------------------------------------------------------*\
|  Routine: edg_get_macros
| 
|  Description:  
|     Gets the macros buffer associated with a dialogue.
\*--------------------------------------------------------------------------*/
buffer edg_get_macros (dialogue)
edg_dialogue dialogue;
{
    set_status (ok);
    return est_get_buffer_property ((cus_sym) dialogue, macros_prop);
}

/*--------------------------------------------------------------------------*\
|  Routine: edg_set_externs
| 
|  Description:  
|     Sets the externs buffer associated with a dialogue.
\*--------------------------------------------------------------------------*/
void edg_set_externs (dialogue, externs)
edg_dialogue dialogue;
buffer externs;
{
    set_status (ok);

    est_set_buffer_property ((cus_sym) dialogue, externs_prop, externs);

    return;
}

/*--------------------------------------------------------------------------*\
|  Routine: edg_get_externs
| 
|  Description:  
|     Gets the externs buffer associated with a dialogue.
\*--------------------------------------------------------------------------*/
buffer edg_get_externs (dialogue)
edg_dialogue dialogue;
{
    set_status (ok);
    return est_get_buffer_property ((cus_sym) dialogue, externs_prop);
}

/*--------------------------------------------------------------------------*\
|  Routine: edg_get_shared_data
| 
|  Description:  
|     Gets the shared data buffer associated with the dialogue.
\*--------------------------------------------------------------------------*/
buffer edg_get_shared_data (dialogue)
edg_dialogue dialogue;
{
    set_status (ok);
    return est_get_buffer_property ((cus_sym) dialogue, shared_data_prop);
}

/*--------------------------------------------------------------------------*\
|  Routine: edg_set_shared_data
| 
|  Description:  
|     Sets the shared data buffer associated with the dialogue.
\*--------------------------------------------------------------------------*/
void edg_set_shared_data (dialogue, shared_data)
edg_dialogue dialogue;
buffer shared_data;
{
    set_status (ok);

    est_set_buffer_property ((cus_sym) dialogue, shared_data_prop, shared_data);

    return;
}

/*--------------------------------------------------------------------------*\
|  Local routine: find_technology
| 
|  Description:
|     Locates a technology in a technology list.  Returns NULL if not found.
\*--------------------------------------------------------------------------*/
static string find_technology (list, technology_name)
LIST list;
string technology_name;
{
    boolean found;
    string list_element;


    found = false;
    list_element = (string) get_list_next (list, NULL);

    while ( (! found) && (list_element != NULL) )
	{
	if (strcmp (technology_name, list_element) == 0)
	    found = true;
	else
	    list_element = (string) get_list_next (list, list_element);
	}

    return (list_element);
}					/* end find_technology */


/*--------------------------------------------------------------------------*\
|  Routine: edg_add_technology
| 
|  Description:
|     Adds a technology to the technology list associated with the dialogue.
\*--------------------------------------------------------------------------*/
string edg_add_technology (dialogue, technology)
edg_dialogue dialogue;
string technology;
{
    string list_element;
    LIST list;


    set_status (ok);

    list_element = NULL;
    list = (LIST) cus_get_prop ((cus_sym) dialogue, technologies_prop);
    if (list == NULL)
	{
				    /*
				       No list in property.  Try to make
				       one.
				    */
	if ( (list = make_list ()) == (LIST) NULL)
	    set_status (out_of_memory);
	}

    if (list != NULL)
	{
				    /* Make sure it's not already in list. */
	if ((list_element = find_technology (list, technology)) == NULL)
	    {
				    /* 
				       Not there.  Add a copy of the string.
				       No need to do a put_prop, since list
				       pointer is already in symbol table.
				    */
	    if ((list_element = copy_string (technology)) != NULL )
		add_to_tail_val (list, list_element, 0);
	    }
	}

    return (list_element);
}

/*--------------------------------------------------------------------------*\
|  Routine: edg_remove_technology
| 
|  Description:
|     removes a technology to the technology list associated with the dialogue.
\*--------------------------------------------------------------------------*/
void edg_remove_technology (dialogue, technology)
edg_dialogue dialogue;
string technology;
{
    string list_element;
    LIST list;


    set_status (ok);

    list = (LIST) cus_get_prop ((cus_sym) dialogue, technologies_prop);
    if ((list_element = find_technology (list, technology)) != NULL )
	{delete_list_node (list, list_element);
	free_node (list_element);
	}

    return;
}

/*--------------------------------------------------------------------------*\
|  Routine: edg_is_a_technology
| 
|  Description:
|     Determines if a technology is on the technology list.
\*--------------------------------------------------------------------------*/
boolean edg_is_a_technology (dialogue, technology)
edg_dialogue dialogue;
string technology;
{
    LIST list;


    set_status (ok);

    list = (LIST) cus_get_prop ((cus_sym) dialogue, technologies_prop);
    return (find_technology (list, technology) == NULL ? false : true);
}


/*--------------------------------------------------------------------------*\
|  Routine: edg_get_ill_name
| 
|  Description:
|     Examines a line to see whether it is an include directive for an ILL
|     file.  If it is, it returns the name of the ill file (minus the .ill
|     extension) in an allocated string.  Otherwise, it returns NULL.
\*--------------------------------------------------------------------------*/
char *edg_get_ill_name (line)
char *line;
{
    char *token;
    char *s;
    int chars_to_skip;
    char *ill_name = NULL;


    s = line;
    if ( (*s == INCLUDE_LEAD_IN) &&
	 ((token = u22_get_token (++s, WHITE_SPACE, &chars_to_skip)) != NULL) &&
	 (strcmp (token, "include") == 0) )
	{
	/*
	** If the include line is badly formatted in any way, let the
	** preprocessor gag on it.
	*/

	free_node (token); /* we don't need it anymore */
	s += chars_to_skip;
	if ( ((token = u22_get_token (s,
				      WHITE_SPACE,
				      &chars_to_skip)) != NULL) &&

	     (index (INCLUDE_START_FILE_NAME, *token) != NULL) )
	    {
	    /* Looks like the start of a legal file name include directive. */

	    char *last;
	    char *file_name;

	    /* We need to save the token pointer so we can free it.  Use
	    ** a copy of the pointer for further parsing.
	    */
	    file_name = token;

	    file_name++;
	    last = file_name + strlen (file_name) - 1;
	    /*
	    ** Strictly speaking, we should make sure the delimiters are
	    ** properly paired when examining an include line.  However, the
	    ** preprocessor will catch the error when we invoke it, so we'll
	    ** take the easy way out.
	    */
	    if (index (INCLUDE_END_FILE_NAME, *last) != NULL)
		{
		/* We have a file name.  Is it an ILL file?  If so, is it
		** one of the technologies?  (We can destroy the token without
		** affecting the line it came from, since the token is a 
		** separately allocated string.
		*/
		char *file_root;
		char *file_ext;

		*last = '\0';
		file_root = u22_get_token (file_name, ".", &chars_to_skip);
		file_ext = file_name + chars_to_skip;

		if ( (file_ext != NULL) &&
		     (strcmp (file_ext, ILL_FILE_EXTENSION) == 0) )
		    {
		    /* We have an ill file. Let the caller worry about
		    ** its significance.
		    */
		    ill_name = file_root;
		    }
		}

	    free_node (token);
	    }
	}

    return (ill_name);
}


#define delete_a_prop_list_of_syms(the_sym, prop) \
   {LIST the_list; \
   cus_sym a_sym; \
		 \
   if ((the_list = (LIST) cus_get_prop (the_sym, prop))) \
      {loop_through_list (the_list, a_sym, cus_sym) \
         if (cus_is_sym (a_sym)) \
            cus_delete_sym (a_sym); \
      free_list (the_list, NOT_NODES); \
      } \
   }

/*--------------------------------------------------------------------------*\
|  Routine: edg_cleanup_import_symbol_table_vc
| 
|  Description:
|     removes most of the symbol table contents assocaited with an import
\*--------------------------------------------------------------------------*/
static void edg_cleanup_import_symbol_table_vc (the_vc)
cus_sym the_vc;
{  LIST stuff, objs, sub_vcs;
   cus_sym a_vc, an_obj, a_sym;

   delete_a_prop_list_of_syms (the_vc, variables_prop);
   if ((objs = (LIST) cus_get_prop (the_vc, objects_prop)))
      {loop_through_list (objs, an_obj, cus_sym)
         {delete_a_prop_list_of_syms (an_obj, attributes_prop);
         delete_a_prop_list_of_syms (an_obj, methods_prop);
         if ((stuff = (LIST) cus_get_prop (an_obj, fixed_method_snippet_prop)))
            esi_free_parse_tree (stuff);
         if ((a_sym = (cus_sym) cus_get_prop (an_obj, fixed_method_prop)))
            cus_delete_sym (a_sym);
         cus_delete_sym (an_obj);
         }
      free_list (objs, NOT_NODES);
      }
   if ((sub_vcs = (LIST) cus_get_prop (the_vc, sub_vcs_prop)))
      {loop_through_list (sub_vcs, a_vc, cus_sym)
         {edg_cleanup_import_symbol_table_vc (a_vc);
         cus_delete_sym (a_vc);
         }
      free_list (sub_vcs, NOT_NODES);
      }
   if ((stuff = (LIST) cus_get_prop (the_vc, cc_used_vars_prop)))
      free_list (stuff, NOT_NODES);
   if ((stuff = (LIST) cus_get_prop (the_vc, old_cc_used_vars_prop)))
      free_list (stuff, NOT_NODES);
   if ((stuff = (LIST) cus_get_prop (the_vc, cc_uses_new_prop)))
      free_list (stuff, NOT_NODES);
   if ((stuff = (LIST) cus_get_prop (the_vc, fixed_creation_condition_prop)))
      esi_free_parse_tree (stuff);
      
   return;
}				/* end edg_cleanup_import_symbol_table_vc */


/*--------------------------------------------------------------------------*\
|  Routine: edg_cleanup_import_symbol_table
| 
|  Description:
|     removes most of the symbol table contents assocaited with an import
\*--------------------------------------------------------------------------*/
static void edg_cleanup_import_symbol_table ()
{  LIST a_list, another_list;
   cus_sym a_sym;

			/* cleanup most of the symbol table */
   edg_cleanup_import_symbol_table_vc (car_god_vc);
			/* now do some god specific cleanup */
   delete_a_prop_list_of_syms (car_god_vc, ill_files_prop);
   if ((a_list = (LIST) cus_get_prop (car_god_vc, externs_prop)))
      {loop_through_list (a_list, a_sym, cus_sym)
         {if ((another_list = (LIST) cus_get_prop (a_sym,
					 parameter_conversion_functions_prop)))
            free_list (another_list, NOT_NODES);
         cus_delete_sym (a_sym); 
         }
      free_list (a_list, NOT_NODES);
      }

			/* now reset god in anticipation of another import */
   cus_put_prop (car_god_vc, parent_vc_prop, NULL);
   cus_put_prop (car_god_vc, externs_prop, make_list ());
   cus_put_prop (car_god_vc, ill_files_prop, make_list ());
   cus_put_prop (car_god_vc, creation_condition_prop, car_true_symbol);
   cus_put_prop (car_god_vc, cc_used_vars_prop, make_list ());

   return;
}				/* end edg_cleanup_import_symbol_table */


/*--------------------------------------------------------------------------*\
|  Routine: internal_src_read
| 
|  Description:
|     does the REAL src read work
\*--------------------------------------------------------------------------*/
static edg_dialogue internal_src_read (dial_name, file_name)
string dial_name;
string file_name;
{  char *line;
   char *ill_name;
   char *temp_file_name;
   char temp_file_buf [L_tmpnam];	    /* see tmpnam(3s) */
   edg_dialogue dialogue;
   FILE *stream;
   char *command;
   buffer prologue;
   char *include_list;
   char *data_path_setting;

   extern int unlink ();		    /* see unlink(2) */
   extern char *getenv ();

   set_status (ok);

   if (!(stream = fopen (file_name, "r")))
      {set_status (system_operation_failed);
      return (EDG_NO_DIALOGUE);
      }

   temp_file_name = tmpnam (temp_file_buf);

   dialogue = edg_create_dialogue ();
			/* setting this global here aint good s/w engineering
			   (but neither is this english) but the global needs
			   to be set for some of the "deep" routines necessary
			   for importation.  and seeing that is is 4 weeks before
			   version 1.0 delivery there aint no way i'm gonna go
			  change everything around */
   rcheck_status ("internal_src_read: can't create dialogue.", EDG_NO_DIALOGUE);
			/* need to let the evc stuff know about the dialogue
			   be constructed */
   e22_dialogue = dialogue;

   evc_imported_dialogue = dialogue;

   edg_set_file_name (dialogue, dial_name);
   rcheck_status ("internal_src_read: can't set file name in dialogue",
	EDG_NO_DIALOGUE);

   prologue.body   = "";
   prologue.type   = idd_string;
   prologue.length = 0;
				/* get the ill includes from the source file */

   while ( ((line = u22_read_line (stream)) != NULL) &&
	    (strncmp (line,  END_OF_PROLOGUE, END_OF_PROLOGUE_LEN) != 0) )
	{
	if ( (ill_name = edg_get_ill_name (line)) != NULL )
	    {
				/* We have an ill file name here.  Is it
				** one of the available technologies?
				*/
	    if (eta_is_valid (ill_name))
		edg_add_technology (dialogue, ill_name);
	    else
		{
				/* Save the include line in the prologue. */
		string old_prologue;

		old_prologue = prologue.body;
		prologue.body = u22_concat_string (prologue.body,
						   line,
						   "\n",
						   NULL);
		prologue.length = strlen (prologue.body);
		if (*old_prologue != '\0')
		    free_node (old_prologue);
		}
	    free_node (ill_name);
	    }
	} /* end while */

   if (prologue.length > 0)
	{
	edg_set_prologue (dialogue, prologue);
				/*
				** Free the buffer body.  Edg_set_prologue
				** stored a copy.
				*/
	free_node (prologue.body);
	}

   (void) fclose (stream);

				/* Build include list for C preprocessor */

   include_list = "";
   if ( (data_path_setting = getenv (DATA_PATH_ENV_VAR)) != NULL )
	{
	char *old_ptr;
	char *directory;
	int chars_to_skip;

	while ( ((directory = u22_get_token (data_path_setting,
					     WHITE_SPACE,
					     &chars_to_skip)) != NULL) &&
		(include_list != NULL) )
	    {
	    old_ptr = include_list;
	    include_list = u22_concat_string (" ",
					      include_list,
					      " ",
					      CPP_INCLUDE_OPTION,
					      /* no space! */
					      directory,
					      NULL);
	    if (*old_ptr != '\0')
		free_node (old_ptr);

	    data_path_setting += chars_to_skip;
	    }
	}

   if (include_list == NULL)
	{
	set_status (out_of_memory);
	return (EDG_NO_DIALOGUE);
	}

				/* Build C preprocessor command line.
				** Note that the include was was built with
				** a leading blank.
				*/

   command = u22_concat_string (CPP_COMMAND,
				 include_list,
				 " ",
				 file_name,
				 " >",
				 temp_file_name,
				 NULL);

   if (command == NULL)
	{
	set_status (out_of_memory);
	return (EDG_NO_DIALOGUE);
	}

   free_node (include_list); /* we don't need it any more */

   if (system (command) != 0)
	{set_status (system_operation_failed);
	return (EDG_NO_DIALOGUE);
	}

   free_node (command); /* we don't need it anymore */

			/*Compile the preprocessed Slang source file */
   stream = fopen (temp_file_name, "r");

   if (!c22_compile (stream, 1, FALSE, FALSE, TRUE, file_name))
	{set_status (invalid_dialogue);
	return (EDG_NO_DIALOGUE);
	}
			/* now get rid of the temp file */
   if (unlink (temp_file_name) < 0 )
	{fprintf (stderr,
	    "internal_src_read: WARNING: Can't remove temp file \"%s\"\n",
	    temp_file_name);
	}

   edg_set_externs (dialogue,
		esi_externs_2_text (cus_get_prop (car_god_vc,
						  externs_prop)));
   edg_cleanup_import_symbol_table ();

   return (dialogue);
}					/* end internal_src_read */


/*--------------------------------------------------------------------------*\
|  Routine: edg_src_read
|
|  Description:
|     Reads a source dialogue from a file
\*--------------------------------------------------------------------------*/
edg_dialogue edg_src_read (name)
string name;
{  string file_name;
   edg_dialogue the_dialogue;

   set_status (ok);

   make_file_name (file_name, name, SRC_EXT);
   if (file_name == NULL)
      bum_dialogue (system_operation_failed);

   the_dialogue = internal_src_read (name, file_name);

   free_node (file_name);

   return (the_dialogue);
}					/* end edg_src_read */


/*--------------------------------------------------------------------------*\
|  Local Routine: src_write_prologue
| 
|  Description:
|     writes out the prologue info in source format
\*-----------------------------------------------------------------------*/
static void src_write_prologue (stream, dialogue)
FILE *stream;
edg_dialogue dialogue;
{
    LIST tech_list;
    string tech_name;

   tech_list = (LIST) cus_get_prop ((cus_sym) dialogue, technologies_prop);
   if (tech_list != NULLlist)
      loop_through_list (tech_list, tech_name, string)
         fprintf (stream, "%c%s \"%s%s\"\n",
              INCLUDE_LEAD_IN, INCLUDE_TOKEN, tech_name, ILL_FILE_EXTENSION);
    else
			/* this is here to protect the app from a dialogue
			   which does not set any technologies */
       fprintf(stream, "\n#include \"sat.ill\"\n");

    esi_buffer_src_write (edg_get_prologue (dialogue), stream, FALSE);
    fprintf (stream, "\n\n%s\n\n", END_OF_PROLOGUE);

    return;
}					/* end src_write_prologue */


/*--------------------------------------------------------------------------*\
|  Local Routine: internal_src_write
| 
|  Description:
|     does the REAL src write work
\*-----------------------------------------------------------------------*/
static void internal_src_write (stream, dialogue)
FILE *stream;
edg_dialogue dialogue;
{
    set_status (ok);

    src_write_prologue (stream, dialogue);

    esi_buffer_src_write (edg_get_macros (dialogue), stream, FALSE);

    esi_buffer_src_write (edg_get_externs (dialogue), stream, FALSE);

    evc_src_write (edg_get_vc (dialogue), stream);

    fprintf (stream, "\n\n");

    return;
}					/* end internal_src_write */

/*--------------------------------------------------------------------------*\
|  Routine: edg_src_write
|
|  Description:
|     Write a source dialogue to a file
\*-----------------------------------------------------------------------*/
void edg_src_write (dialogue, name)
edg_dialogue dialogue;
string name;
{  string file_name;
   FILE *stream;

   set_status (ok);

   make_file_name (file_name, name, SRC_EXT);
   if (file_name == NULL)
      {
      set_status (out_of_memory);
      return;
      }

   if (!(stream = fopen (file_name, "w")))
      {
      set_status (system_operation_failed);
      return;
      }

   internal_src_write (stream, dialogue);

   free_node (file_name);
   fclose (stream);

   return;
}					/* end edg_src_write */


/*-----------------------------------------------------------------------*\
|  Description:
|     Reads a binary dialogue from a file
\*-----------------------------------------------------------------------*/
edg_dialogue edg_bin_read (name, transaction)
string name;
transaction_type transaction;
{  string the_token, file_name;
   edg_dialogue the_dialogue;
   ebi_buffer the_buffer;
   char the_char;
   evc_vc the_vc;
   FILE *stream;
   int count;

   set_status (ok);

   make_file_name (file_name, name, BIN_EXT);
   if (file_name == NULL)
      bum_dialogue (system_operation_failed);

   if (!(stream = ebi_open_dialogue_file (file_name, ebi_open_read)))
      bum_dialogue (file_not_found);

   if ((the_dialogue = edg_create_dialogue ()) == EDG_NO_DIALOGUE)
      bum_dialogue (system_operation_failed);
   e22_dialogue = the_dialogue;

   if (!ebi_read_magic_number (stream))
      bum_dialogue (invalid_bin_file);
   
   read_open_record (stream);
   if (((the_char = ebi_read_token_char (stream)) == EBI_BAD_CHAR) ||
       (the_char != EBI_DIALOGUE_SELECTOR))
      bum_dialogue (invalid_bin_file);

   if (!(the_token = ebi_read_token (stream)))
      bum_dialogue (invalid_bin_file);
   edg_set_file_name (the_dialogue, the_token);

   count = ebi_read_integer (stream);
   if (get_status () != ok)
      bum_dialogue (invalid_bin_file);
   edg_set_count (the_dialogue, count);

   read_open_record (stream);
   while ((the_token = ebi_read_token (stream)) &&
          (*the_token != EBI_CLOSE_RECORD))
      edg_add_technology (the_dialogue, the_token);

   read_buffer (stream, the_buffer, EBI_COMMENTS_SELECTOR);
   edg_set_prologue (the_dialogue, the_buffer.actual_buffer);

   read_buffer (stream, the_buffer, EBI_MACROS_SELECTOR);
   edg_set_macros (the_dialogue, the_buffer.actual_buffer);

   read_buffer (stream, the_buffer, EBI_EXTERNS_SELECTOR);
   edg_set_externs (the_dialogue, the_buffer.actual_buffer);

   read_buffer (stream, the_buffer, EBI_SHARED_DATA_SELECTOR);
   edg_set_shared_data (the_dialogue, the_buffer.actual_buffer);

			/* initialize the hash tables necessary for binary
			  reads */
   eob_init_template_hash ();
   eat_init_template_hash ();

   if ((the_vc = evc_bin_read (stream, transaction)) == EVC_NO_VC)
      bum_dialogue (invalid_bin_file);
   edg_set_vc (the_dialogue, the_vc);

			/* we are done with the binary read hash tables, get
			   rid of them */
   eob_free_template_hash ();
   eat_free_template_hash ();

   read_close_record (stream); /* end of dialogue */

   if (!ebi_read_checksum (stream))
      bum_dialogue (invalid_bin_file);

   free_node (file_name);

   return (the_dialogue);
}				/* end edg_bin_read		    */


/*--------------------------------------------------------------------------*\
|  Routine: edg_bin_write
| 
|  Description:
|     Writes a binary dialogue out to a file
\*--------------------------------------------------------------------------*/
void edg_bin_write (dialogue, name)
edg_dialogue dialogue;
string name;
{  string ill, file_name;
   FILE *stream;
   LIST guys;
   object_instance_sdd *obj_inst;

   set_status(ok);

   make_file_name (file_name, name, BIN_EXT);
   if (file_name == NULL)
      {set_status (out_of_memory);
      return;
      }

   if (!(stream = ebi_open_dialogue_file (file_name, ebi_open_write)))
      {set_status (system_operation_failed);
      return;
      }

   if (!ebi_write_magic_number (stream))
      {set_status (system_operation_failed);
      return;
      }

   write_open_record (stream);
   ebi_write_token_char (stream, EBI_DIALOGUE_SELECTOR);

   edg_set_file_name (dialogue, name);
   ebi_write_token (stream, name);

   ebi_write_integer (stream, edg_get_count (dialogue));

   write_open_record (stream);
   if ((guys = (LIST) cus_get_prop (dialogue, technologies_prop)))
      {loop_through_list (guys, ill, string)
         ebi_write_token (stream, ill);
      }
    else
      set_status (ok);
   write_close_record (stream);

   write_buffer (stream, edg_get_prologue (dialogue),
		    EBI_COMMENTS_SELECTOR);

   write_buffer (stream, edg_get_macros (dialogue),
		    EBI_MACROS_SELECTOR);

   write_buffer (stream, edg_get_externs (dialogue),
		    EBI_EXTERNS_SELECTOR);

   write_buffer (stream, edg_get_shared_data (dialogue),
		    EBI_SHARED_DATA_SELECTOR);

   evc_bin_write (edg_get_vc (dialogue), stream);

   write_close_record (stream);
   if (!ebi_write_checksum (stream))
      {set_status (system_operation_failed);
      return;
      }
   fclose (stream);

   free_node (file_name);

			/* ??? this is a kludge, could not find some 
			   "problems" somewhere earlier in the bin write
			   sequence, so just reset status for now (the
			   "problems" do not seem to do anthing bad */
   set_status(ok);

   return;
}					/*end edg_bin_write */


/*--------------------------------------------------------------------------*\
|  Routine: file_exists
|
|  Description:
|     determines if a file exists
\*--------------------------------------------------------------------------*/
static boolean file_exists (name, ext)
string name;
string ext;
{  string file_name;
   FILE *stream;
   boolean is_there;

   make_file_name (file_name, name, ext);
   if (file_name == NULL)
      {set_status (out_of_memory);
      return false;
      }
   if (stream = fopen (file_name, "r"))
      {is_there = TRUE;
      fclose (stream);
      }
     else
      is_there = FALSE;
   free_node (file_name);
   return (is_there);
}					/* end file_exists */


/*--------------------------------------------------------------------------*\
|  Routine: edg_src_file_exists
|
|  Description:
|     determines if a src file exists
\*--------------------------------------------------------------------------*/
boolean edg_src_file_exists (name)
string name;
{
   return (file_exists (name, SRC_EXT));
}					/* end edg_src_file_exists */


/*--------------------------------------------------------------------------*\
|  Routine: edg_bin_file_exists
|
|  Description:
|     determines if a bin file exists
\*--------------------------------------------------------------------------*/
boolean edg_bin_file_exists (name)
string name;
{
   return (file_exists (name, BIN_EXT));
}					/* end edg_bin_file_exists */


/*--------------------------------------------------------------------------*\
|  Routine: edg_set_count
| 
|  Description:  
|     Sets the counter associated with a dialogue.
\*--------------------------------------------------------------------------*/
void edg_set_count (dialogue, the_count)
edg_dialogue dialogue;
int the_count;
{
   set_status (ok);
   cus_put_prop (dialogue, count_prop, the_count);
   return;
}					/* end edg_set_count */


/*--------------------------------------------------------------------------*\
|  Routine: edg_get_count
| 
|  Description:  
|     Gets the counter associated with a dialogue.
\*--------------------------------------------------------------------------*/
int edg_get_count (dialogue)
edg_dialogue dialogue;
{  int the_count;

   the_count = (int) cus_get_prop (dialogue, count_prop);
			/* we use the intermediate value because get_prop
			   could set status ~ok, but that is really ok in 
			   this case, so we need to force status ok, got that*/
   set_status (ok);
   return (the_count);
}					/* end edg_get_count */


/*--------------------------------------------------------------------------*\
|  Routine: edg_increment_object_count
| 
|  Description:  
|     increments the object (instance & template) counter associated with a 
| 	specfic technology. adds the technology to the dialogue, if it is not
|	there.
\*--------------------------------------------------------------------------*/
void edg_increment_object_count (dialogue, the_tech)
edg_dialogue dialogue;
string the_tech;
{  LIST tech_list;
   int count;
   string local_tech;

   set_status (ok);
			/* ensure the technology is on the list,
			   add_tech will put it there if it was not there to
			   begin with.  now, add_tech puts a copy of the string
			   on the tech list, so we need to use that copy of
			   the string for get_node_val and change_list_val to
			   work.  we are using the value part of the node
			   to keep the count of object templates/instances
			   associated with each tech. */
   local_tech = edg_add_technology (dialogue, the_tech);
			/* now get & increment the correct count */
   tech_list = (LIST) cus_get_prop (dialogue, technologies_prop);
   count = get_node_val (tech_list, local_tech);
   count++;
   change_list_val (tech_list, local_tech, count);

			/* i know that this routine results in three
			   sequential searches down the technology list, but
			   it is not sorted (that's what lists are all about),
			   that's how the get/put val routines work in the
			   list package, and the list should be pretty short. */

   return;
}					/* end edg_increment_object_count */


/*--------------------------------------------------------------------------*\
|  Routine: edg_decrement_object_count
| 
|  Description:  
|     decrements the object (instance & template) counter associated with a 
| 	specfic technology.  removes the technology from the dialogue if the
|	count goes to zero.
\*--------------------------------------------------------------------------*/
void edg_decrement_object_count (dialogue, the_tech)
edg_dialogue dialogue;
string the_tech;
{  LIST tech_list;
   int count;
   string local_tech;

   set_status (ok);
   tech_list = (LIST) cus_get_prop (dialogue, technologies_prop);
   if (tech_list)
			/* there is a technologies_prop, go to town */
      {local_tech = find_technology (tech_list, the_tech);
			/* i know that this results in two or three sequential
			   searches down the technology list.  see the
			   disclamer in edg_increment_object_count */
      count = get_node_val (tech_list, local_tech);
      if ((count == LIST_DEFAULT_VALUE) || (count < 2))
			/* either the tech is not on technologies_prop list or
			   the count is 1 or less, in any case remove it.
			   edg_remove_technology can handle things if the tech
			   is not on the list */
         edg_remove_technology (dialogue, local_tech);
      else
			/* tech is on the list & count is greater than one,
			   so decrement it */
         {count--;
         change_list_val (tech_list, local_tech, count);
         }
      }

   return;
}					/* end edg_decrement_object_count */



