static char rcsid [] =
        "$Header: /u/04c/mnt/integ/source/de/src/RCS/ehf.c,v 1.4 90/05/21 13:34:18 little Exp $";
/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor Hash Functions (ehf)
|
|  Description:  
|     Constains the routines to support the various hash tables used
|     in the dialogue editor.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log:	ehf.c,v $
 * Revision 1.4  90/05/21  13:34:18  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.3  89/12/20  14:43:36  rcs
 * fixed debug problem
 * 
 * Revision 1.2  89/11/30  13:49:40  rcs
 * fixed spelling error
 * 
 * Revision 1.1  89/11/28  18:39:00  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*-------------------- Includes --------------------------------------------*/

#define ehf_main
#include "ehf.h"                /* hash functions                      */
#include "emp.h"
#include "se.h"
#include "serpent.h"
#include "hashPack.h"


/*-------------------- Constants -------------------------------------------*/

#define DEBUG1(statement) SERPENT_DEBUG1(ehf_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(ehf_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(ehf_debug, statement)

#define PARM_HASH_SIZE 11
#define MAX_HASH 257

/*-------------------- Type Definitions ------------------------------------*/

/*-------------------- Externals -------------------------------------------*/

/*-------------------- Retained Data ---------------------------------------*/

HASH parameter_table;		/* parameters associated with current trans */
HASH id_table;			/* hash of emp_map records by id	    */
HASH elem_table;		/* hash of emp_map records by dial elem     */

/*--------------------------------------------------------------------------*\
|  Routine: ehf_match_id
| 
|  Description:  
|     Routine to compare an identifier with the id in a shared data record.
\*--------------------------------------------------------------------------*/
int ehf_match_id(id, map)
id_type id;
emp_map *map;
{ 					/* local type definitions	    */
  set_status(ok);			/* begin 			    */
  DEBUG2(
    fprintf(stderr, "ehf_match_id: call.\n");
    fprintf(stderr, "ehf_match_id: identifer = %d.\n", id);
    fprintf(stderr, "ehf_match_id: identifer = %x.\n", map);
  );

  DEBUG2(
    fprintf(stderr, "ehf_match_id:  exiting.\n");
  );
  return (map->id == id);
} 					/* end ehf_match_id		    */

/*--------------------------------------------------------------------------*\
|  Routine: ehf_hash_id
| 
|  Description:  
|     Internal function which will convert an id into indehfes
|     in the array of hash lists.
|   
\*--------------------------------------------------------------------------*/
int ehf_hash_id(id)
id_type id;
{ 					/* local type definitions	    */
/*
   Initialize.
*/
  set_status(ok);			/* begin 			    */
  DEBUG2(
    fprintf(stderr, "ehf_hash_id call.\n");
  );
/*
   Return a value in the right range.
*/
  DEBUG2(
    fprintf(stderr, "ehf_hash_id exiting.\n");
  );
  return((int)id % MAX_HASH);
} 					/* end ehf_hash_id		    */

/*--------------------------------------------------------------------------*\
|  Routine: ehf_match_elem
| 
|  Description:  
|     Routine to compare a dialogue element with the dialogue element 
|      in the mapping record.
\*--------------------------------------------------------------------------*/
int ehf_match_elem(elem, map)
char *elem;
emp_map *map;
{ 					/* local type definitions	    */
  set_status(ok);			/* begin 			    */
  DEBUG2(
    fprintf(stderr, "ehf_match_elem: call.\n");
    fprintf(stderr, "ehf_match_elem: dialogue element = %d.\n", elem);
    fprintf(stderr, "ehf_match_elem: mapping record = %x.\n", map);
  );

  DEBUG2(
    fprintf(stderr, "ehf_match_elem:  exiting.\n");
  );
  return (map->dialogue_element == elem);
} 					/* end ehf_match_elem		    */

/*--------------------------------------------------------------------------*\
|  Routine: ehf_hash_elem
| 
|  Description:  
|     Internal function which will convert an elem into indehfes
|     in the array of hash lists.
\*--------------------------------------------------------------------------*/
int ehf_hash_elem(elem)
char *elem;
{ 					/* local type definitions	    */
/*
   Initialize.
*/
  set_status(ok);			/* begin 			    */
  DEBUG2(
    fprintf(stderr, "ehf_hash_elem call.\n");
  );
/*
   Return a value in the right range.
*/
  DEBUG2(
    fprintf(stderr, "ehf_hash_elem exiting.\n");
  );
  return((int)elem % MAX_HASH);
} 					/* end ehf_hash_elem		    */

/*--------------------------------------------------------------------------*\
|  Routine: ehf_match_parm
| 
|  Description:  
|     Routine to compare the parameter type with parameter type stored in 
|     the parameter shared data record.
\*--------------------------------------------------------------------------*/
int ehf_match_parm(id, parameter)
id_type id;
parameter_sdd *parameter;
{ 					/* local type definitions	    */
  set_status(ok);			/* begin 			    */
  DEBUG2(
    fprintf(stderr, "ehf_match_parm: call.\n");
    fprintf(stderr, "ehf_match_parm: id = %d.\n", id);
    fprintf(stderr, "ehf_match_parm: parameter = %x.\n", parameter);
    fprintf(
      stderr, "ehf_match_parm: parameter->self = %d.\n", parameter->self
    );
  );

  DEBUG2(
    fprintf(stderr, "ehf_match_parm:  exiting.\n");
  );
  return (parameter->self == id);
} 					/* end ehf_match_parm		    */

/*--------------------------------------------------------------------------*\
|  Routine: ehf_hash_parm
| 
|  Description:  
|     Internal function which will convert an parameter_type into indehfes
|     in the array of hash lists.
\*--------------------------------------------------------------------------*/
int ehf_hash_parm(id)
id_type id;
{ 					/* local type definitions	    */
/*
   Initialize.
*/
  set_status(ok);			/* begin 			    */
  DEBUG2(
    fprintf(stderr, "ehf_hash_parm call.\n");
  );
/*
   Return a value in the right range.
*/
  DEBUG2(
    fprintf(stderr, "ehf_hash_parm exiting.\n");
  );
  return((int)id % PARM_HASH_SIZE);
} 					/* end ehf_hash_parm		    */
