static char rcsid [] =
        "$Header: /u/04c/mnt/integ/source/de/src/RCS/eit.c,v 1.9 1991/06/27 17:10:34 dvk Exp $";

/*--------------------------------------------------------------------------*\
|   
|  Name:  ID Table
|
|  Description:  
|     The ID table is used by the dialogue editor to maintain a mapping
|     between the names of object instances and the ID of the realization
|     object used to represent it.
|
|  Component of:
|     Dialoge Editor Dialogue.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log: eit.c,v $
 * Revision 1.9  1991/06/27  17:10:34  dvk
 * No changes - put 'em in externs/arith.c
 *
 * Revision 1.8  1991/04/11  18:09:30  rcs
 * *** empty log message ***
 *
 * Revision 1.7  90/11/29  09:55:49  rcs
 * no change
 * 
 * Revision 1.6  90/09/25  21:40:26  rcs
 * some changes to support parenting
 * 
 * Revision 1.5  90/09/21  00:08:24  rcs
 *  fixed parent instantiation order problem
 * 
 * Revision 1.3  90/05/21  13:34:31  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.2  90/03/05  14:46:42  rcs
 * added a new routine and changed some names
 * 
 * Revision 1.1  89/11/29  09:45:01  trm
 * Initial revision
 * 
 * Revision 1.1  89/09/28  12:13:49  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*-------------------- Includes --------------------------------------------*/

#define eit_main
#include "eit.h"
#include "global.h"
#ifndef memoryPack
#define memoryPack
#endif
#include "hashPack.h"
#include <string.h>


/*-------------------- Constants -------------------------------------------*/

#define DEBUG1(statement) SERPENT_DEBUG1(eit_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(eit_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(eit_debug, statement)

#define MAX_HASH 32

/*-------------------- Type Definitions ------------------------------------*/

typedef struct {
  string name;			/* name of the instance			*/
  iid_id_type value;
/*  string type;		type of the object, i.e. command_widget */
} id_record;

/*-------------------- Retained Data ---------------------------------------*/

static HASH id2name_table;       /* hash table of ids 		     */

/*--------------------------------------------------------------------------*\
|  Routine: match_name
| 
|  Description:  
|     Routine to compare the names with user ids.
\*--------------------------------------------------------------------------*/
int match_name(name, id_rec)
string name;
id_record *id_rec;
{ 					/* local type definitions	    */
  set_status(ok);			/* begin 			    */

  DEBUG2(
    fprintf(stdout, "match_name: call.\n");
    fprintf(stdout, "name = %s.\n", name);
    fprintf(stdout, "id_rec = %x.\n", id_rec);
  );

  DEBUG3(
    fprintf(stdout, "id_rec->name = %s.\n", id_rec->name);
  );

  DEBUG2(
    fprintf(stdout, 
      "match_name:  match_name = %d.\n", 
      strcmp(name, id_rec->name) == 0
    );
    fprintf(stdout, "match_name:  exiting.\n");
  );

  return(strcmp(name, id_rec->name) == 0);
} 					/* end match_name	    */

/*--------------------------------------------------------------------------*\
|  Routine: hash_name
| 
|  Description:  
|     Internal function which will convert a string identifiers into indecies
|     in the array of hash lists.
|   
\*--------------------------------------------------------------------------*/
int hash_name(name)
string name;
{ 					/* local type definitions	    */
  register int hash_value = 0;
  register string pointer;

  set_status(ok);			/* begin 			    */
  DEBUG2(
    fprintf(stdout, "hash_name: call.\n");
  );
/*
   Loop through each character in the name string and add the character value
   to the accumulator.
*/
  for (pointer = name; *pointer != '\0'; pointer++) {
    hash_value += *pointer;
  }
/*
   Return a value in the right range.
*/
  DEBUG2(
    fprintf(stdout, "hash_name: exiting.\n");
  );
  return ((hash_value & 077777) % MAX_HASH);
} 					/* end hash_name		    */

/*--------------------------------------------------------------------------*\
|  Routine: create_id_table
| 
|  Description:
|     Creates an id_table to be used within the dialogue editor to 
|     provide a mapping between object instance names and realization ids.
\*--------------------------------------------------------------------------*/
void create_id_table()
{                                       /* local type definitions           */
/*
   Initialize.
*/
  set_status(ok);                       /* begin                            */
  DEBUG2(
    fprintf(stdout, "create_id_table: call.\n");
  );
/*
   Create the id table.
*/
  id2name_table = make_hashtable(MAX_HASH, hash_name, match_name);
/*
   Return.
*/
  DEBUG2(
    fprintf(stdout, "create_id_table:  exiting.\n");
  );
  return;
}                                       /* end create_id_table         */

/*--------------------------------------------------------------------------*\
|  Routine: add_object
| 
|  Description:
|     The add_object routine adds a new name to the hash table.
\*--------------------------------------------------------------------------*/
void add_object(name)
string name;
{                                       /* local type definitions           */
  id_record *id_rec;
/*
   Initialize.
*/
  set_status(ok);                       /* begin                            */
  DEBUG2(
    fprintf(stdout, "add_object: call.\n");
    fprintf(stdout, "add_object: name = %s.\n", name);
  );
/*
   Create id record and initialize id to null.
*/
  id_rec = (id_record *)make_node(sizeof(id_record));
  check_null(
    id_rec,
    "add_object:  out of memory during make_node.\n",
    out_of_memory
  );
  id_rec->value = iid_null_id;
/*
   Copy in name into the id record.
*/
  id_rec->name = (string)make_node(strlen(name)+1);
  check_null(
    id_rec->name,
    "add_object:  out of memory during make_node.\n",
    out_of_memory
  );
  strcpy(id_rec->name, name);
#if 0
/*
   Copy in type into the id record.
*/
  id_rec->type = (string)make_node(strlen(type)+1);
  check_null(
    id_rec->type,
    "add_object:  out of memory during make_node.\n",
    out_of_memory
  );
  strcpy(id_rec->type, type);
#endif
/*
   Add the id record to the id2name_table using the name as a hashing
   index.
*/
  add_to_hashtable(id2name_table, id_rec, name);
/*
   Return.
*/
  DEBUG2(
    fprintf(stdout, "add_object: id_rec = %x.\n", id_rec);
    fprintf(stdout, "add_object: exiting.\n");
  );
  return;
}                                       /* end add_object         */

/*--------------------------------------------------------------------------*\
|  Routine: remove_object
| 
|  Description:
|     The remove_object routine removes a name from the hash table.
\*--------------------------------------------------------------------------*/
void remove_object(name)
string name;
{                                       /* local type definitions           */
  id_record *id_rec;
/*
   Initialize.
*/
  set_status(ok);                       /* begin                            */
  DEBUG2(
    fprintf(stdout, "remove_object: call.\n");
    fprintf(stdout, "remove_object: name = %s.\n", name);
  );
/*
   
*/
/*
   If the id is in the hashtable, delete it and free the 
   memory associated with the node.
*/
  id_rec = (id_record *)delete_from_hashtable(
    id2name_table,
    name
  );
  if (id_rec != NULL) {
    free_node(id_rec->name);
    free_node(id_rec);
  }
/*
   Return.
*/
  DEBUG2(
    fprintf(stdout, "remove_object: exiting.\n");
  );
  return;
}                                       /* end remove_object         */

/*--------------------------------------------------------------------------*\
|  Routine: set_id
| 
|  Description:
|     The set_id routine sets the id for the given name to the specified id.
\*--------------------------------------------------------------------------*/
void set_id(name, id)
string name;
iid_id_type id;
{                                       /* local type definitions           */
  id_record *id_rec;
/*
   Initialize.
*/
  set_status(ok);                       /* begin                            */
  DEBUG2(
    fprintf(stdout, "set_id: call.\n");
    fprintf(stdout, "set_id: name = %s.\n", name);
    fprintf(stdout, "set_id: id = %d.\n", id);
  );
/*
  Find id name in hash table.
*/
  id_rec = (id_record *)get_from_hashtable(
    id2name_table,
    name
  );
  check_null(
    id_rec,
    "set_id: name not found.\n",
    not_found
  );
/*
   Change the id.
*/
  id_rec->value = id;
/*
   Return.
*/
  DEBUG2(
    fprintf(stdout, "set_id:  exiting.\n");
  );
  return;
}                                       /* end set_id         */

/*--------------------------------------------------------------------------*\
|  Routine: get_id
| 
|  Description:
|     The get_id routine returns the id associated with the given object
|     instance name.
\*--------------------------------------------------------------------------*/
iid_id_type get_id(name)
string name;
{                                       /* local type definitions           */
  id_record *id_rec;
  iid_id_type return_value = iid_null_id;
/*
   Initialize.
*/
  DEBUG2(
    fprintf(stdout, "get_id: call.\n");
    fprintf(stdout, "get_id: name = %s.\n", name);
  );
/*
  Find id name in hash table.
*/
  id_rec = (id_record *)get_from_hashtable(
    id2name_table,
    name
  );
/*
   If the record is found, set the return value to the id associated
   with the name.
*/
  if (id_rec != NULL) {
    return_value = id_rec->value;
  }
/*
   Return.
*/
  DEBUG2(
    fprintf(stdout, "get_id: exiting.\n");
    fprintf(stdout, "get_id: return value = %d .\n", return_value);
  );

  return return_value;
}                                       /* end get_id         */

#if 0
/*--------------------------------------------------------------------------*\
|  Routine: get_type
| 
|  Description:
|     The get_type routine returns the type associated with the given object
|     instance name.
\*--------------------------------------------------------------------------*/
string get_type(name)
string name;
{                                       /* local type definitions           */
  id_record *id_rec;
  string return_value = NULL;
/*
   Initialize.
*/
  DEBUG2(
    fprintf(stdout, "get_type: call.\n");
    fprintf(stdout, "get_type: name = %s.\n", name);
  );
/*
  Find id name in hash table.
*/
  id_rec = (id_record *)get_from_hashtable(
    id2name_table,
    name
  );
/*
   If the record is found, set the return value to the object type associated
   with the name.
*/
  if (id_rec != NULL) {
    return_value = id_rec->type;
  }
/*
   Return.
*/
  DEBUG2(
    fprintf(stdout, "get_type: exiting.\n");
    fprintf(stdout, "get_type: return value = %d .\n", return_value);
  );
  return return_value;
}                                       /* end get_type         */
#endif

/*--------------------------------------------------------------------------*\
|  Routine: object_exists
| 
|  Description:
|     The object_exists routine returns true if the name is in the table
|     and false if it is not.
\*--------------------------------------------------------------------------*/
boolean object_exists(name)
string name;
{                                       /* local type definitions           */
  id_record *id_rec;
  boolean exists;
/*
   Initialize.
*/
  set_status(ok);                       /* begin                            */
  DEBUG2(
    fprintf(stdout, "object_exists: call.\n");
    fprintf(stdout, "object_exists: name = %s.\n", name);
  );
/*
  Find id name in hash table.
*/
  id_rec = (id_record *)get_from_hashtable(
    id2name_table,
    name
  );
  if (id_rec == NULL) {
    exists = false;
  }
  else {
    exists = true;
  }
/*
   Return.
*/
  DEBUG2(
    fprintf(stdout, "object_exists: exists = %d.\n", exists);
    fprintf(stdout, "object_exists: exiting.\n");
  );
  return exists;
}                                       /* end object_exists         */

/*--------------------------------------------------------------------------*\
|  Routine: object_exists_and_not_null
| 
|  Description:
|     The object_exists routine returns true if the name is in the table
|     and the id is not the iid_null_id and false if it is not.
\*--------------------------------------------------------------------------*/
boolean object_exists_and_not_null(name)
string name;
{                                       /* local type definitions           */
  id_record *id_rec;
  boolean exists;
/*
   Initialize.
*/
  set_status(ok);                       /* begin                            */
  DEBUG2(
    fprintf(stdout, "object_exists_and_not_null: call.\n");
    fprintf(stdout, "object_exists_and_not_null: name = %s.\n", name);
  );
/*
  Find id name in hash table.
*/
  id_rec = (id_record *)get_from_hashtable(
    id2name_table,
    name
  );
/*
**  If object exists and is not null return true, otherwise false.
*/
  if (id_rec == NULL) {
    exists = false;
  }
  else {
    if (id_rec->value == iid_null_id) {
      exists = false;
    }
    else {
      exists = true;
    }
  }
/*
   Return.
*/
  DEBUG2(
    fprintf(stdout, "object_exists_and_not_null: exists = %d.\n", exists);
    fprintf(stdout, "object_exists_and_not_null: exiting.\n");
  );
  return exists;
}                                       /* end object_exists_and_not_null   */

/*--------------------------------------------------------------------------*\
|  Routine: destroy_id_record
|
|  Description:
|     The destroy element is used to destroy an existing element.
\*--------------------------------------------------------------------------*/
void destroy_id_record(nada, id_rec)
id_record *nada;
id_record *id_rec;
{                                       /* local type definitions           */
/*
   Initialization.
*/
  set_status(ok);                       /* begin                            */
  DEBUG2(
    fprintf(stdout, "destroy_id_record: call.\n");
    fprintf(stdout, "destroy_id_record: id_rec = %x.\n", id_rec);
  );
/*
   Free storage associated with id name.
*/
  free_node(id_rec->name);
/*
   Free storage associated with id record.
*/
  free_node(id_rec);
/*
   Return.
*/
  DEBUG2(
    fprintf(stdout, "destroy_id_record: exiting.\n");
  );
  return;
}                                       /* end destroy_id_record          */

/*--------------------------------------------------------------------------*\
|  Routine: destroy_id_table
| 
|  Description:  
|     The destroy_id_table routine is used to destroy the id hash
|     table and all the nodes.
\*--------------------------------------------------------------------------*/
void destroy_id_table()
{ 					/* local type definitions	    */
/*
   Initialization.
*/
  set_status(ok);			/* begin 			    */
  DEBUG2(
    fprintf(stdout, "destroy_id_table: call.\n");
  );
/*
   Destroy all id records in the hash table.
*/
  for_all_hashtable(id2name_table, NULL, destroy_id_record);
/*
   Free storage associated with hash table structure.
*/
  free_hashtable(id2name_table, NOT_NODES);
/*
   Return.
*/
  DEBUG2(
    fprintf(stdout, "destroy_id_table: exiting.\n");
  );
  return;
} 					/* end destroy_id_table  */
