static char rcsid [] =
"$Header: /u/04c/mnt/integ/de/src/RCS/emc_cmd.c,v 1.1 89/12/19 12:34:33 rcs Exp $";

/*--------------------------------------------------------------------------*\
|     
|  Name:  Dialogue Editor Method Command Definitions  (emc)
|
|  Description:  
|     definitions of the method associated commands supported by the DEA.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log:	emc_cmd.c,v $
 * Revision 1.1  89/12/19  12:34:33  rcs
 * Initial revision
 * 
 * Revision 1.4  89/12/13  13:34:12  rcs
 * fixed bug
 * 
 * Revision 1.3  89/12/12  17:01:32  little
 * change the order of the parameters fromthe dialogue to emc_get_method
 * 
 * Revision 1.2  89/12/12  15:15:30  little
 * - add emc_delete_method
 * - clean up some extreanous comments
 * ./
 * 
 * Revision 1.1  89/12/07  15:10:20  little
 * Initial revision
 * 
|
|   created 6 dec 1989
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute   (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|   (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


                        /* need DEBUG1 for check_parm_type to work, but do not
                           use any other debug statements, saber is a lot
                           better and definition must be before include of
			   ea2.h */
#define DEBUG1(statement) SERPENT_DEBUG1(emc_debug, statement)


#define emc_main
#include "emc_cmd.h"
#include "esm_sd.h"
#include "eob.h"
#include "eat.h"
#include "emp.h"
#include "ehf.h"
#include "ea2.h"
#include "serpent.h"
#include "idd.h"
#include "isc.h"
#ifndef memoryPack
#define memoryPack
#endif
#include "hashPack.h"
#include "se.h"


/*--------------------------------------------------------------------------*\
|  Routine:  emc_get_method
| 
|  Description:  
|     gets the specified (by a command parameter from the dialogue) method
|     from the abstraction and places it into a transaction
\*--------------------------------------------------------------------------*/
void emc_get_method (command, transaction)
command_sdd *command;
transaction_type transaction;
{  eob_object the_object;		/* parameter 1 */
   string att_name;			/* parameter 2 */
   eme_method method;
   parameter_sdd *parm;
   id_type obj_id;
   LIST atts;

   set_status (ok);

				/* get the first parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
						command-> parameters);
   check_parm_type (parm, idd_id,
		    "emc_get_method: first parameter is not an id\n");
   obj_id = *((id_type *) ((parm-> value).body));
   the_object = emp_get_de (obj_id);

				/* now, get the 2nd parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
						parm-> next);
   check_parm_type (parm, idd_string,
		    "emc_get_method: second parameter is not a string\n");
   att_name = (string) ((parm-> value).body);

				/* now find the method in the object, if not
				   there create one.  then place the info
				   associated with the method into a sd
				   record */
   atts = eob_get_methods (the_object);
   check_status("emc_get_method:  bad status from eob_get_methods.");

   if (method = (eme_method) ea2_in_list (att_name, atts, eme_get_name))
      if (emp_get_id (method) == null_id) {
         esm_create_sd(transaction, method);
         check_status(
           "emc_get_method:  bad status from esm_create_sd."
         );
      }
        else {
         esm_modify_sd (transaction, method);
         check_status(
           "emc_get_method:  bad status from esm_modify_sd."
         );
	}
     else {
      method = eme_create_method ();
      check_status("emc_get_method:  bad status from eme_create_method.");

      eme_set_name(method, att_name);
      check_status("emc_get_method:  bad status from eme_set_name.");

      eob_add_method (the_object, method);
      check_status("emc_get_method:  bad status from eob_add_method.");

      esm_create_sd (transaction, method);
      check_status("emc_get_method:  bad status from esm_create_sd.");
      }

   return;
} 					/* end emc_get_method */


/*--------------------------------------------------------------------------*\
|  Routine:  emc_delete_method
|
|  Description:
|     deletes the specified (by a command parameter from the dialogue) att and
|       places a remove sd request into the transaction
\*--------------------------------------------------------------------------*/
void emc_delete_method (command, transaction)
command_sdd *command;
transaction_type transaction;
{  id_type att_id;			/* parameter 1 */
   parameter_sdd *parm;
   eme_method method; 

   set_status(ok);

                                /* get the first parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
                                                command-> parameters);
   check_parm_type (parm, idd_id,
                    "emc_delete_method: first parameter is not an id\n");
   att_id = *((id_type *) ((parm-> value).body));
   method = emp_get_de (att_id);

                                /* delete the att from it's parent obj */
   eob_remove_method (method);
   check_status("emc_delete_method:  bad status from eob_remove_method.");
                                /* kill the att itself */
   eme_destroy_method (method);
   check_status("emc_delete_method:  bad status from eme_destroy_method.");
                                /* now place a delete request into the
                                   xaction */
   esm_remove_from_sd (transaction, method);
   check_status("emc_delete_method:  bad status from esm_remove_from_sd.");

   return;
}                                       /* end emc_delete_method */


