static char rcsid [] =
        "$Header: /u/04c/mnt/integ/source/de/src/RCS/eme.c,v 1.11 90/11/29 10:01:24 rcs Exp $";

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Attribute Abstraction (eme)
|
|  Description:  
|     The dialogue editor method abstraction provides operations on the
|     method abstract type.
|
|  Component of:
|     Dialogue Editor Layer.
|
$Log:	eme.c,v $
 * Revision 1.11  90/11/29  10:01:24  rcs
 * fixed symbol table problem
 * 
 * Revision 1.10  90/09/11  08:33:48  little
 * add eme_duplicate_method
 * add third parameter to esi_buffer_src_write calls
 * change how src_write works a little
 * 
 * Revision 1.9  90/09/07  18:26:43  little
 * add a terminating : and some line feeds in eme_src_write
 * 
 * Revision 1.8  90/05/21  13:34:48  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.7  89/12/18  15:11:40  bmc
 * eme_bin_read now sets comment property
 * 
 * Revision 1.6  89/12/18  14:19:44  little
 * fixed some shit that was wrong
 * 
 * Revision 1.5  89/12/15  20:30:36  bmc
 * Added binary read routine .
 * 
 * Revision 1.4  89/12/15  18:41:10  rcs
 * added set and get comment routines
 * 
 * Revision 1.3  89/12/15  15:12:01  bmc
 * no changes
 * 
 * Revision 1.2  89/12/15  11:55:11  little
 * add bin_write
 * 
 * Revision 1.1  89/11/29  09:38:12  bmc
 * Initial revision
 * 
|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


#define eme_main
#include "eme.h"
#include "eob.h"
#include "esi.h"
#include "est.h"
#include "ebi.h"
#include "ea2.h"
#include "cus.h"
#include "ubo.h"
#include "listPack.h"
#include <stdio.h>


#define DEBUG1(statement) SERPENT_DEBUG1(eme_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(eme_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(eme_debug, statement)

#define write_open_record(stream) \
   if (!ebi_write_open_record (stream)) \
      {set_status (system_operation_failed); \
      return; \
      }

#define write_close_record(stream) \
   if (!ebi_write_close_record (stream)) \
      {set_status (system_operation_failed); \
      return; \
      }

#define write_buffer(stream, buffer, the_selector) \
{  ebi_buffer a_buffer; \
   a_buffer.actual_buffer = buffer; \
   a_buffer.selector = the_selector; \
   if (!ebi_write_buffer (stream, a_buffer)) \
      {set_status (system_operation_failed); \
      return; \
      } \
}


/*--------------------------------------------------------------------------*\
|  Routine: eme_create_method
|
|  Description:
|     Creates a new method
\*--------------------------------------------------------------------------*/
eme_method eme_create_method ()
{  eme_method new_method;
   buffer new_buff;

   new_method = (eme_method) cus_gensym ("EDITOR_METH", method_sym, NO_SEED);
					/*
					   Okay to use a local buffer, since
					   est_set_buffer_property make a copy
					   before storing in the symbol table.
					*/
   init_buffer (new_buff);

   est_set_buffer_property ((cus_sym) new_method,
			    method_text_prop,
			    new_buff);

   est_set_buffer_property ((cus_sym) new_method,
                            object_comments_text_prop,
                            new_buff);

   est_set_string_property (new_method, name_text_prop, est_default_name);
   return (new_method);
}					/* end eme_create_method */


/*--------------------------------------------------------------------------*\
|  Routine: eme_destroy_method
|
|  Description:
|     Destroys a specified method and any sub-components.
\*--------------------------------------------------------------------------*/
void eme_destroy_method (the_method)
eme_method the_method;
{

   est_clear_buffer_property ((cus_sym) the_method, method_text_prop);
   est_clear_string_property ((cus_sym) the_method, name_text_prop);
   cus_delete_sym (the_method);
   return;
}					/* end eme_destroy_method */


/*--------------------------------------------------------------------------*\
|  Routine: eme_duplicate_method
|
|  Description:
|     duplicates the specified method
\*--------------------------------------------------------------------------*/
eme_method eme_duplicate_method (old_method)
eme_method old_method;
{  eme_method new_method;
   buffer new_buff;
			/* if we can get a new method, copy all properties from
			   "parent" to clone */
   if ((new_method = eme_create_method ()) != EME_NO_METHOD)
      {eme_set_snippet (new_method, eme_get_snippet (old_method));
      eme_set_comment (new_method, eme_get_comment (old_method));
      eme_set_name (new_method, eme_get_name (old_method));
      }
   return (new_method);
}					/* end eme_duplicate_method */


/*--------------------------------------------------------------------------*\
|  Routine: eme_set_snippet
|
|  Description:
|     squirrels away a snippet for a method
\*--------------------------------------------------------------------------*/
void eme_set_snippet (the_method, the_snip)
eme_method the_method;
buffer the_snip;
{
   est_set_buffer_property (the_method, method_text_prop, the_snip);
   return;
}					/* end eme_set_snippet */


/*--------------------------------------------------------------------------*\
|  Routine: eme_get_snippet
|
|  Description:
|      fetches the squirreled away snippet for a method
\*--------------------------------------------------------------------------*/
buffer eme_get_snippet (the_method)
eme_method the_method;
{  buffer the_snippet;

   the_snippet = est_get_buffer_property (the_method,
					  method_text_prop);
   return (the_snippet);
}					/* end eme_get_snippet */

/*--------------------------------------------------------------------------*\
|  Routine: eme_set_comment
|
|  Description:
|     squirrels away a comment for a method
\*--------------------------------------------------------------------------*/
void eme_set_comment (the_method, the_comment)
eme_method the_method;
buffer the_comment;
{
   est_set_buffer_property (the_method, object_comments_text_prop,
                            the_comment);
   return;
}                                       /* end eme_set_comment */

/*--------------------------------------------------------------------------*\
|  Routine: eme_get_comment
|
|  Description:
|      fetches the squirreled away comment for a method
\*--------------------------------------------------------------------------*/
buffer eme_get_comment (the_method)
eme_method the_method;
{  buffer the_comment;

   the_comment = est_get_buffer_property (the_method,
                                          object_comments_text_prop);
   return (the_comment);
}                                       /* end eme_get_comment */

/*--------------------------------------------------------------------------*\
|  Routine: eme_set_parent
|
|  Description:
|     squirrels away a parent for a method
\*--------------------------------------------------------------------------*/
void eme_set_parent (the_method, the_parent)
eme_method the_method;
eob_object the_parent;
{
   cus_put_prop (the_method, parent_object_prop, the_parent);
   return;
}					/* end eme_set_parent */


/*--------------------------------------------------------------------------*\
|  Routine: eme_get_parent
|
|  Description:
|      fetches the squirreled away parent for a method
\*--------------------------------------------------------------------------*/
eob_object eme_get_parent (the_method)
eme_method the_method;
{  eob_object the_parent;

   the_parent = cus_get_prop (the_method, parent_object_prop);
   return (the_parent);
}					/* end eme_get_parent */


/*--------------------------------------------------------------------------*\
|  Routine: eme_set_name
|
|  Description:
|     squirrels away a name for a method
\*--------------------------------------------------------------------------*/
void eme_set_name (the_method, the_name)
eme_method the_method;
string the_name;
{
   est_set_string_property (the_method, name_text_prop, the_name);
   return;
}					/* end eme_set_name */


/*--------------------------------------------------------------------------*\
|  Routine: eme_get_name
|
|  Description:
|      fetches the squirreled away name for a method
\*--------------------------------------------------------------------------*/
string eme_get_name (the_method)
eme_method the_method;
{  string the_name;

   the_name = cus_get_prop (the_method, name_text_prop);
   return (the_name);
}					/* end eme_get_name */


/*--------------------------------------------------------------------------*\
|  Routine: eme_parse_method_2_ed
|
|  Description:
|     convert the parser version of the method to editor version
\*--------------------------------------------------------------------------*/
eme_method eme_parse_method_2_ed (method)
cus_sym method;
{  eme_method ed_meth;
   LIST snippet;
   buffer ed_snippet;

   ed_meth = eme_create_method ();

   eme_set_name (ed_meth, cus_sym_representation (method));

   if (snippet = (LIST) cus_get_prop (method, value_snippet_prop))
      {ed_snippet = esi_snip_2_text (snippet);
      eme_set_snippet (ed_meth, ed_snippet);
      }

   return (ed_meth);
}					/* end eme_parse_method_2_ed */


/*--------------------------------------------------------------------------*\
|  Routine: eme_src_write
|
|  Description:
|     write a method to a source stream
\*--------------------------------------------------------------------------*/
void eme_src_write (the_method, stream)
eme_method the_method;
FILE *stream;
{  buffer the_snippet;

/* ???? check this out, different things are gonna happen 4 'empty' meth */
   the_snippet = eme_get_snippet (the_method);
   if (ea2_printable_buffer (the_snippet))
      {fprintf (stream, "%s:\n", eme_get_name (the_method));
      esi_buffer_src_write (the_snippet, stream, TRUE);
      }
   return;
}					/* end eme_src_write */


/*--------------------------------------------------------------------------*\
|  Routine: eme_bin_write
|
|  Description:
|     write a method to a binary stream
\*--------------------------------------------------------------------------*/
void eme_bin_write (the_method, stream)
eme_method the_method;
FILE *stream;
{

   write_open_record (stream);
   ebi_write_token_char (stream, EBI_METHOD_SELECTOR);

   ebi_write_token (stream, eme_get_name (the_method));

   ebi_write_token (stream, cus_sym_representation (the_method));

   write_buffer (stream, eme_get_comment (the_method), EBI_COMMENTS_SELECTOR);

   write_buffer (stream, eme_get_snippet (the_method), EBI_SNIPPET_SELECTOR);

   write_close_record (stream);

   return;

}					/* end eme_bin_write */


/*--------------------------------------------------------------------------*\
|  Routine: eme_bin_read
|
|  Description:
|     read a method from a binary stream
\*--------------------------------------------------------------------------*/
eme_method eme_bin_read (stream)
FILE *stream;
{
    eme_method method = EME_NO_METHOD;
    char tc;
    string unique_name = NULL;
    string name = NULL;
    ebi_buffer snippet;
    ebi_buffer comment;


    /*
    ** A binary method has the following form (line breaks are insignificant):
    **
    ** EBI_OPEN_RECORD
    **   EBI_METHOD_SELECTOR method_name unique_name method_snippet
    ** EBI_CLOSE_RECORD
    */
    set_status (ok);
    snippet.selector = EBI_INVALID_SELECTOR;
    comment.selector = EBI_INVALID_SELECTOR;

    if ((tc = ebi_read_token_char (stream)) == EBI_CLOSE_RECORD)
	{
	/* End of list.  Stop here.  Push the end of list character back
	** onto the input token stream.
	*/
	method = EME_NO_MORE_METHODS;
	if (! ebi_push_token_char (EBI_CLOSE_RECORD))
	    {
	    /* Assume status is already set. */
	    method = EME_NO_METHOD; /* to indicate error */
	    DEBUG1 (fprintf (stderr,
		"eme_bin_read: can't push `%c' back onto token stream\n",
		EBI_CLOSE_RECORD););
	    }
	}

    else if ( (method = eme_create_method ()) == EME_NO_METHOD )
	{
	set_status (out_of_memory);
	DEBUG1 (fprintf (stderr, "eme_bin_read: can't create method\n"););
	}

    else if (tc != EBI_OPEN_RECORD)
	{
	set_status (invalid_method);
	DEBUG1 (fprintf (stderr,
	    "eme_bin_read: missing open record `%c' character\n",
	    EBI_OPEN_RECORD););
	}

    else if ( (tc = ebi_read_token_char (stream)) != EBI_METHOD_SELECTOR )
	{
	set_status (invalid_method);
	DEBUG1 (fprintf (stderr,
	    "eme_bin_read: bad selector character -- `%c'\n", tc););
	}

    else if ( (name = ebi_read_token (stream)) == NULL )
	{
	set_status (invalid_method);
	DEBUG1 (fprintf (stderr, "eme_bin_read: missing method name\n"););
	}

    else if ( (unique_name = ebi_read_token (stream)) == NULL )
	{
	set_status (invalid_method);
	DEBUG1 (fprintf (stderr, "eme_bin_read: missing unique name\n"););
	}

    /*
    ** The commas in the following two expressions are INTENTIONAL, and
    ** represent perfectly legal C.  See K&R (edition 1), p. 192 for details.
    */
    else if (comment = ebi_read_buffer (stream),
	     comment.selector != EBI_COMMENTS_SELECTOR)
	{
	set_status (invalid_method);
	DEBUG1 (fprintf (stderr,
	    "eme_bin_read: Expected `%c' buffer, found `%c' buffer.\n",
	    EBI_COMMENTS_SELECTOR, snippet.selector););
	}

    else if (snippet = ebi_read_buffer (stream),
	     snippet.selector != EBI_SNIPPET_SELECTOR)
	{
	set_status (invalid_method);
	DEBUG1 (fprintf (stderr,
	    "eme_bin_read: Expected `%c' buffer, found `%c' buffer.\n",
	    EBI_SNIPPET_SELECTOR, snippet.selector););
	}

    else if (ebi_read_token_char (stream) != EBI_CLOSE_RECORD)
	{
	set_status (invalid_method);
	DEBUG1 (fprintf (stderr,
	    "eme_bin_read: Missing close record `%c' character\n",
	    EBI_CLOSE_RECORD););
	}

    else
	{
	/*
	** We have all the pieces.  Store 'em.  (We're ignoring the
	** unique name for now.)
	*/
	eme_set_name (method, name);
	eme_set_snippet (method, snippet.actual_buffer);
	eme_set_comment (method, comment.actual_buffer);
	}

    if (get_status () != ok)
	{
	/* Problem.  Release storage. */

	if ( (method != EME_NO_METHOD) && (method != EME_NO_MORE_METHODS) )
	    {
	    eme_destroy_method (method);
	    method = EME_NO_METHOD;
	    }

	ea2_free_node (name);
	ea2_free_node (unique_name);

	if (snippet.selector != EBI_INVALID_SELECTOR)
	    ubo_destroy_buffer (snippet.actual_buffer);
	if (comment.selector != EBI_INVALID_SELECTOR)
	    ubo_destroy_buffer (comment.actual_buffer);
	}

    return (method);
}
