static char rcsid [] =
        "$Header: /u/04c/mnt/integ/de/src/RCS/emp.c,v 1.3 89/12/12 09:17:50 rcs Exp $";
/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor Mapping Abstraction (emp)
|
|  Description:  
|     The dialogue editor mapping abstraction creates and destroys structures
|     that provide a mapping between shared data elements and actual dialogue
|     elements.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Related files:
|     emp.c - contains the implementation of the dialogue editor mapping 
|             abstraction.
|     emp.h - contains the definition of the diaalogue editor mapping 
|             abstraction.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Robert C. Seacord
|     rcs@sei.cmu.edu  
|
|  Change History:
|     May 2, 1988 : Module created.
|     $Log:	emp.c,v $
 * Revision 1.3  89/12/12  09:17:50  rcs
 * fixed bugs found during integration testing
 * 
 * Revision 1.2  89/11/30  13:51:02  rcs
 * fixed spelling error
 * 
 * Revision 1.1  89/11/28  18:39:20  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*-------------------- Includes --------------------------------------------*/

#define emp_main
#include "emp.h"			/* application data structures	    */

/*-------------------- Constants -------------------------------------------*/

#define DEBUG1(statement) SERPENT_DEBUG1(emp_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(emp_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(emp_debug, statement)

/*-------------------- Type Definitions ------------------------------------*/

/*-------------------- Retained Data ---------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: emp_base_create_map
| 
|  Description:  
|     Creates a mapping between a shared data element to a dialogue element. 
|     And adds it to the mapping table.
|
|     Note:  Since the dialogue element may be of any type the procedure
|            definition forces a type cast to char *.
\*--------------------------------------------------------------------------*/
void emp_base_create_map(id, dialogue_element)
iid_id_type id;
char *dialogue_element;
{                                       /* local type definitions           */
  emp_map *map;
/*
   Initialize.
*/
  set_status(ok);                       /* begin                            */
  DEBUG2(
    fprintf(stderr, "emp_base_create_map: call.\n");
  );
/*
   Allocate memory for the map record.
*/
  map = (emp_map *)make_node(sizeof(emp_map));
  check_null(
    map,
    "emp_base_create_map: out_of_memory during make_node.\n",
    out_of_memory
  );
/*
   Set values.
*/
  map->id = id;
  map->dialogue_element = dialogue_element;
/*
   Add the map to both hashtables.
*/
  add_to_hashtable(id_table, map, id);
  add_to_hashtable(elem_table, map, dialogue_element);
/*
   Return.
*/
  DEBUG2(
    fprintf(stderr, "emp_base_create_map exiting.\n");
  );
  return;
}                                       /* end emp_base_create_map          */

/*--------------------------------------------------------------------------*\
|  Routine: emp_get_de
| 
|  Description:  
|     Gets the dialogue element associated with a particular id.
\*--------------------------------------------------------------------------*/
char *emp_get_de(id)
iid_id_type id;
{                                       /* local type definitions           */
  emp_map *map;
/*
   Initialize.
*/
  set_status(ok);                       /* begin                            */

  DEBUG2(
    fprintf(stderr, "emp_get_de: call.\n");
    fprintf(stderr, "emp_get_de: id = %d.\n", id);
  );
/*
   Get the map structure from the id_table.
*/
  map = (emp_map *)get_from_hashtable(
    id_table,
    id
  );
  DEBUG3(
    fprintf(stderr, "emp_get_de: map = %x.\n", map);
  );
/*
   Set status to not_found and return if mapping element does not 
   exist.
*/
  if (map == NULL) {
    set_status(not_found);
    return NULL;
  }
/*
   Return.
*/
  DEBUG2(
    fprintf(stderr, "emp_get_de: map->dialogue_element = %x.\n", 
      map->dialogue_element
    );
    fprintf(stderr, "emp_get_de: exiting.\n");
  );
  return map->dialogue_element;
}                                       /* end emp_get_de     */

/*--------------------------------------------------------------------------*\
|  Routine: emp_get_id
| 
|  Description:  
|     Gets the id associated with a particular dialogue element.
|     Status is set to not_found if mapping record not found.
\*--------------------------------------------------------------------------*/
id_type emp_get_id(dialogue_element)
char *dialogue_element;
{                                       /* local type definitions           */
  emp_map *map;
/*
   Initialize.
*/
  set_status(ok);                       /* begin                            */
  DEBUG2(
    fprintf(stderr, "emp_get_id: call.\n");
    fprintf(stderr, "emp_get_id: dialogue_element = %x.\n", dialogue_element);
  );
/*
   Get the map structure from the element table.
*/
  map = (emp_map *)get_from_hashtable(
    elem_table,
    dialogue_element
  );
  DEBUG3(
    fprintf(stderr, "emp_get_id: map = %x.\n", map);
  );
/*
   Set status to not_found and return of mapping element does not 
   exist.
*/
  if (map == NULL) {
    set_status(not_found);
    return null_id;
  }
/*
   Return.
*/
  DEBUG2(
    fprintf(stderr, "emp_get_id:  map->id = %d.\n", map->id);
    fprintf(stderr, "emp_get_id:  exiting.\n");
  );
  return map->id;
}                                       /* end emp_get_id     */

/*--------------------------------------------------------------------------*\
|  Routine: emp_destroy_map_by_id
| 
|  Description:  
|     Destroys the map corresponding to the id.
\*--------------------------------------------------------------------------*/
void emp_destroy_map_by_id(id)
iid_id_type id;
{                                       /* local type definitions           */
  char *dialogue_element;
  emp_map *map;
/*
   Initialize.
*/
  set_status(ok);                       /* begin                            */
  DEBUG2(
    fprintf(stderr, "emp_destroy_map_by_id: call.\n");
    fprintf(stderr, "emp_destroy_map_by_id: id = %d.\n", id);
  );
/*
   Get the dialogue_element as well as the id.
*/
  dialogue_element = emp_get_de(id);
  check_status("emp_destroy_map_by_id: bad status from emp_get_de.");
/*
   Delete the map structure from the id_table.
*/
  map = (emp_map *)delete_from_hashtable(
    id_table,
    id
  );
  DEBUG3(
    fprintf(stderr, "emp_destroy_map_by_id: map = %x.\n", map);
  );
/*
   Delete the map structure from the elem_table.
*/
  map = (emp_map *)delete_from_hashtable(
    elem_table,
    dialogue_element
  );
  DEBUG3(
    fprintf(stderr, "emp_destroy_map_by_id: map = %x.\n", map);
  );
/*
   Free map record storage.
*/
  free_node(map);
/*
   Return.
*/
  DEBUG2(
    fprintf(stderr, "emp_destroy_map_by_id:  exiting.\n");
  );
  return;
}                                       /* end emp_destroy_map_by_id        */

/*--------------------------------------------------------------------------*\
|  Routine: emp_destroy_map_by_de
| 
|  Description:  
|     Destroys the map corresponding to the dialogue element.
\*--------------------------------------------------------------------------*/
void emp_destroy_map_by_de(dialogue_element)
char *dialogue_element;
{                                       /* local type definitions           */
  id_type id;
  emp_map *map;
/*
   Initialize.
*/
  set_status(ok);                       /* begin                            */
  DEBUG2(
    fprintf(stderr, "emp_destroy_map_by_de: call.\n");
    fprintf(stderr, "emp_destroy_map_by_de: dialogue_element = %x.\n", 
      dialogue_element
    );
  );
/*
   Get the id as well as the dialogue element.
*/
  id = emp_get_id(dialogue_element);
  check_status("emp_destroy_map_by_de: bad status from emp_get_id.");
/*
   Delete the map structure from the element table.
*/
  map = (emp_map *)delete_from_hashtable(
    elem_table,
    dialogue_element
  );
  DEBUG3(
    fprintf(stderr, "emp_destroy_map_by_de: map = %x.\n", map);
  );
/*
   Delete the map structure from the id table.
*/
  map = (emp_map *)delete_from_hashtable(
    id_table,
    id
  );
  DEBUG3(
    fprintf(stderr, "emp_destroy_map_by_de: map = %x.\n", map);
  );
/*
   Free map record storage.
*/
  free_node(map);
/*
   Return.
*/
  DEBUG2(
    fprintf(stderr, "emp_destroy_map_by_de:  exiting.\n");
  );
  return;
}                                       /* end emp_destroy_map_by_de        */
