static char rcsid [] =
	"$Header: /usr/users/little/serpent/mips/de/src/RCS/eoc_cmd.c,v 1.6 1991/05/07 17:48:46 little Exp $";

/*--------------------------------------------------------------------------*\
|     
|  Name:  Dialogue Editor Object Commands (eoc_cmd.c)
|
|  Description:  
|     The dialogue editor objects command module contains the object
|     command handling routines for the dialogue editor command-interpreter.
|
|  Component of:
|     Dialogue Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213

$Log: eoc_cmd.c,v $
 * Revision 1.6  1991/05/07  17:48:46  little
 * remove the DEBUG statements so dbx does not get confused
 * add object counting functionality
 *
 * Revision 1.5  1990/09/17  22:19:31  little
 * fix duplication
 *
 * Revision 1.4  90/07/05  15:06:30  little
 *  more structure and visualizer parts integration
 * 
 * Revision 1.3  90/05/21  13:35:34  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.2  89/12/13  09:26:23  bmc
 * Now uses ea2_check_vc to check whether VC is really a VC or the dialogue.
 * 
 * Revision 1.1  89/12/12  12:59:11  bmc
 * Initial revision
 * 

|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*-------------------- Constants -------------------------------------------*/

	/* These definitions MUST appear before the include for "ea2.h" */

#define DEBUG1(statement) SERPENT_DEBUG1(eoc_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(eoc_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(eoc_debug, statement)

/*-------------------- Includes --------------------------------------------*/

#define eoc_main

#include "serpent.h"			/* Serpent misc. definitions	    */
#include "isc.h"			/* status codes			    */

#include "eoc_cmd.h"			/* eoc definitions 		    */
#include "eob.h"			/* object abstraction		    */
#include "evc_cmd.h"			/* VC abstraction		    */
#include "eso_sd.h"			/* object sdd declarations	    */
#include "emp.h"			/* editor mapping abstraction	    */
#include "ehf.h"			/* editor hash functions	    */
#include "ea2.h"			/* editor general purpose defs	    */
#include "se.h"				/* dialogue editor shared data defs */

#ifndef memoryPack
#define memoryPack
#endif

#include "listPack.h"			/* Zimmermann list functions	    */
#include "hashPack.h"			/* Zimmermann list functions	    */

/*--------------------------------------------------------------------------*\
|  Routine:  eoc_add_object
| 
|  Description:  
|     The insert_object routine is used to create and insert an object into an 
|     existing VC.
\*--------------------------------------------------------------------------*/
void eoc_add_object (command, transaction)
command_sdd *command;
transaction_type transaction;
{
    parameter_sdd *parm;

    eob_object the_object;
    evc_vc parent_vc;
    string technology_name;
    string object_type_name;

    id_type parent_id;


    set_status(ok);

    /*
    ** Get the parameters.  This routine expects the following parameters,
    ** in the order listed:
    **
    **    1 - ID of VC to receive object
    **    2 - Name of technology associated with object
    **    3 - Type of object (e.g., "command_widget")
    */
    parm = (parameter_sdd *) get_from_hashtable (parameter_table, 
						 command->parameters);
    check_parm_type (parm, idd_id,
		     "eoc_add_object: VC parameter is not an ID.");
    parent_id = *((id_type *) ((parm->value).body));

    parm = (parameter_sdd *) get_from_hashtable (parameter_table, parm->next);
    check_parm_type (parm, idd_string,
		     "eoc_add_object: technology parameter is not a string.");
    technology_name = ((string) ((parm->value).body));

    parm = (parameter_sdd *) get_from_hashtable (parameter_table, parm->next);
    check_parm_type (parm, idd_string,
		     "eoc_add_object: object type parameter is not a string.");
    object_type_name = ((string) ((parm->value).body));

    /*
    ** Map parent VC id into actual parent VC handle.
    */

    parent_vc = (evc_vc) emp_get_de (parent_id);
    check_status ("eoc_add_object: can't map VC id into dialogue element.");
    ea2_check_vc (parent_vc);

    if ( (the_object = eob_create_object (object_type_name,
					  technology_name)) == EOB_NO_OBJECT )
	fprintf (stderr, "eoc_add_object: can't create object\n");

    else
	{evc_add_object (parent_vc, the_object);
	eso_create_sd (transaction, the_object);
	}

    return;
} 						/* end eoc_add_object	    */

/*--------------------------------------------------------------------------*\
|  Routine:  eoc_copy_object
| 
|  Description:  
|     The eoc_copy_object routine copies an object to a parent vc.
\*--------------------------------------------------------------------------*/
void eoc_copy_object (command, transaction)
command_sdd *command;
transaction_type transaction;
{
    parameter_sdd *parm;

    id_type target_vc_id;
    id_type object_id;

    evc_vc target_vc;
    eob_object the_object;
    eob_object object_copy;


    set_status(ok);

    /*
    ** Get the parameters.  This routine expects the following parameters,
    ** in the order listed:
    **
    **    1 - ID of object to be copied.
    **    2 - ID of VC to receive copy.
    */
    parm = (parameter_sdd *) get_from_hashtable (parameter_table, 
						 command->parameters);
    check_parm_type (parm, idd_id,
		     "eoc_copy_object: object parameter is not an ID.");
    object_id = *((id_type *) ((parm->value).body));

    parm = (parameter_sdd *) get_from_hashtable (parameter_table, parm->next);
    check_parm_type (parm, idd_id,
		     "eoc_copy_object: VC parameter is not an ID.");
    target_vc_id = *((id_type *) ((parm->value).body));

    /*
    ** Get the actual dialogue elements corresponding to the ids.
    */
    target_vc = (evc_vc) emp_get_de (target_vc_id);
    check_status ("eoc_copy_object: can't map VC id into dialogue element.");
    ea2_check_vc (target_vc);

    the_object = (eob_object) emp_get_de (object_id);

    /* Make the duplicate. */

    if ( (object_copy = eob_duplicate_object (the_object)) == EOB_NO_OBJECT )
	fprintf (stderr, "eoc_copy_object: can't copy object %s\n",
		eob_get_name (the_object));

    else
	{
	/*
	** Add the copy to the specified parent and create a shared data
	** element for it.
	*/
	evc_add_object (target_vc, object_copy);
	check_status ("eoc_copy_object: can't add new object to VC");
	eso_create_sd (transaction, object_copy);
	check_status ("eoc_copy_object: can't add new object to shared data");
	}

    return;
} 					/* end eoc_copy_object		    */

/*--------------------------------------------------------------------------*\
|  Routine:  eoc_move_object
| 
|  Description:  
|     The eoc_copy_object routine moves an object from one VC to another.
\*--------------------------------------------------------------------------*/
void eoc_move_object (command, transaction)
command_sdd *command;
transaction_type transaction;

{   parameter_sdd *parm;
    id_type new_parent_vc_id;
    id_type object_id;
    evc_vc new_parent_vc;
    eob_object the_object;

    set_status(ok);

    /*
    ** Get the parameters.  This routine expects the following parameters,
    ** in the order listed:
    **
    **    1 - ID of object to be copied.
    **    2 - ID of VC to which to move object.
    */
    parm = (parameter_sdd *) get_from_hashtable (parameter_table, 
						 command->parameters);
    check_parm_type (parm, idd_id,
		     "eoc_move_object: object parameter is not an ID.");
    object_id = *((id_type *) ((parm->value).body));

    parm = (parameter_sdd *) get_from_hashtable (parameter_table, parm->next);
    check_parm_type (parm, idd_id,
		     "eoc_move_object: VC parameter is not an ID.");
    new_parent_vc_id = *((id_type *) ((parm->value).body));

    /*
    ** Get the actual dialogue elements corresponding to the ids.
    */
    new_parent_vc = (evc_vc) emp_get_de (new_parent_vc_id);
    check_status ("eoc_move_object: can't map VC id into dialogue element.");
    ea2_check_vc (new_parent_vc);

    the_object    = (eob_object) emp_get_de (object_id);

    /*
    ** Move the object by removing it from its current parent and adding
    ** it to the new parent.   Note that we don't need to determine the
    ** current parent, as evc_remove_object figures that out for itself.
    */
    (void) evc_remove_object (the_object);
    check_status ("eoc_move_object: can't remove object from current parent");
    evc_add_object (new_parent_vc, the_object);
    check_status ("eoc_move_object: can't add object to new parent");

    /* Modify the shared data record to account for the changed parent. */

    eso_modify_sd (transaction, the_object);
    check_status ("eoc_move_object: can't modify shared data");

    return;
} 					/* end eoc_copy_object		    */


/*--------------------------------------------------------------------------*\
|  Routine:  eoc_delete_object
| 
|  Description:  
|     The eoc_delete_object removes an object from its parent VC and destroys
|     it.
\*--------------------------------------------------------------------------*/
void eoc_delete_object (command, transaction)
command_sdd *command;
transaction_type transaction;
{
    parameter_sdd *parm;
    id_type object_id;
    eob_object the_object;


    set_status(ok);

    /*
    ** Get the parameters.  This routine expects the following parameters,
    ** in the order listed:
    **
    **    1 - ID of object to be deleted.
    */
    parm = (parameter_sdd *) get_from_hashtable (parameter_table, 
						 command->parameters);
    check_parm_type (parm, idd_id,
		     "eoc_delete_object: object parameter is not an ID.");
    object_id = *((id_type *) ((parm->value).body));

    /*
    ** Get the actual dialogue element corresponding to the id.
    */
    the_object = (eob_object) emp_get_de (object_id);

    /*
    ** Remove the object from its parent VC.  Note that evc_remove_object
    ** figures out who the parent is.
    */
    evc_remove_object (the_object);
    check_status ("eoc_delete_object: can't remove object from parent.");

    /*
    ** Remove the object from shared data.
    */
    eso_remove_from_sd (transaction, the_object);

			/* finally destroy the object itself */
    eob_destroy_object (the_object, NULL);
    return;
} 					/* end eoc_delete_object	  */

/*--------------------------------------------------------------------------*\
|  Routine:  eoc_get_objects
|
|  Description:
|     Gets the objects associated with a VC and places them into shared data.
|     The VC is specified via a command from the dialogue.
|     it.
\*--------------------------------------------------------------------------*/
void eoc_get_objects (command, transaction)
command_sdd *command;
transaction_type transaction;
{
    evc_vc the_vc;
    LIST objects;
    parameter_sdd *parm;
    id_type vc_id;
    eob_object the_object;


    set_status (ok);

    /*
    ** Get the parameters.  This routine expects the following parameters,
    ** in the order listed:
    **
    **    1 - ID of VC for which objects are to be placed in shared data.
    */
    parm = (parameter_sdd *) get_from_hashtable (parameter_table, 
						 command->parameters);
    check_parm_type (parm, idd_id,
		     "eoc_get_objects: VC parameter is not an ID.");
    vc_id = *((id_type *) ((parm->value).body));

    the_vc = emp_get_de (vc_id);
    check_status ("eoc_get_objects: can't map VC id into dialogue element.");
    ea2_check_vc (the_vc);

    /*
    ** Get the list of objects in this VC, and loop through them, placing
    ** each one into shared data if it's not already there.
    */
    objects = evc_get_objects (the_vc);
    check_status ("eoc_get_objects: can't get list of VC objects.");

    loop_through_list (objects, the_object, eob_object)
	{
	if (emp_get_id (the_object) == null_id)
	    {
	    /* Object is not in shared data.  Put it in there. */

	    eso_create_sd (transaction, the_object);
	    check_status ("eoc_get_objects: can't place object in shared data");
	    }
	}

    return;
}

