static char rcsid [] =
"$Header: /u/04c/mnt/integ/source/de/src/RCS/esa_sd.c,v 1.8 90/10/23 14:09:58 little Exp $";

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor Shared Data Att Abstraction  (esa)
|
|  Description:  
|     provides routines to create,modify, & delete shared data records 
|     based on attributes.
|     
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
$Log:	esa_sd.c,v $
 * Revision 1.8  90/10/23  14:09:58  little
 * use glue to get att instance type in esa_make_inst_sd_struct
 * 
 * Revision 1.7  90/07/05  15:07:43  little
 *  more structure and visualizer parts integration
 * 
 * Revision 1.6  90/06/14  13:55:14  little
 *  more changes to integrate visulization and structure editors
 * 
 * Revision 1.5  90/05/21  13:36:12  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.4  89/12/21  11:22:25  rcs
 * hjk
 * 
 * Revision 1.2  89/12/12  15:17:45  little
 * - change the strcpy of the name to use strncpy
 * - add the destroy_map to the remove function
 * 
 * Revision 1.1  89/12/07  15:11:06  little
 * Initial revision
 * 
|     6 dec 1989: Module created.
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute   (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|   (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*-------------------- Includes --------------------------------------------*/

#define esa_main
#include "esa_sd.h"
#include "eat.h"
#include "eob.h"
#include "emp.h"
#include "se.h"
#include "glue_intf.h"
#include "isc.h"
#include <string.h>

			/* need DEBUG1 for check_status to work, but do not
			   use any other debug statements, saber is a lot
			   better */
#define DEBUG1(statement) SERPENT_DEBUG1(esa_debug, statement)

/*--------------------------------------------------------------------------*\
|  Routine: esa_make_inst_sd_struct
|
|  Description:
|     Creates a shared data structure for an object instance
\*--------------------------------------------------------------------------*/
object_instance_attribute_sdd *esa_make_inst_sd_struct (att)
eat_attribute att;
{  object_instance_attribute_sdd *instance;
   ATTR *glue_attr;
   string att_name;

   instance = (object_instance_attribute_sdd *)
                   make_node (sizeof (object_instance_attribute_sdd));
   if (!instance)
      {fprintf (stderr, "esa_make_inst_sd_struct: outta memory\n");
      set_status (out_of_memory);
      return (NULL);
      }

   att_name = eat_get_name (att);
   strcpy (instance-> name, att_name);

   glue_attr = glue_get_attribute (eob_get_widget (eat_get_parent (att)),
                                   att_name, SERP_NAMES);
   instance-> type = glue_attr-> x_type;

   instance-> template = (id_type) att;
			/* object component cannot be set here, it nust
			   be an id */
   eat_set_instance (att, instance);

   return (instance);
}                                       /* end esa_make_inst_sd_struct */


/*--------------------------------------------------------------------------*\
|  Routine: esa_make_sd_struct
| 
|  Description:  
|     creates an sd structure for an attribute
\*--------------------------------------------------------------------------*/
attribute_sdd esa_make_sd_struct (attribute)
eat_attribute attribute;
{  attribute_sdd attribute_sd;

   set_status (ok);

				/* put att name into sd struct */
   strncpy (attribute_sd.name, eat_get_name (attribute),
            sizeof (attribute_sd.name));

				/* put the att parent into sd struct */
   attribute_sd.parent = emp_get_id (eat_get_parent (attribute));
   rcheck_status ("esa_make_sd_struct:  bad status from emp_get_id.",
		  attribute_sd);
				/* put the att comment into sd struct */
   attribute_sd.comment = eat_get_comment (attribute);
   rcheck_status ("esa_make_sd_struct:  bad status from eat_get_comment.",
		  attribute_sd);

				/* put the att snippet into sd struct */
   attribute_sd.value = eat_get_snippet (attribute);
   rcheck_status ("esa_make_sd_struct:  bad status from eat_get_snippet.",
		  attribute_sd);

   return (attribute_sd);
}                                       /* end esa_make_sd_struct */


/*--------------------------------------------------------------------------*\
|  Routine: esa_create_sd
|
|  Description:
|     Creates a shared data element for a specified attribute and adds it to 
|     the specified transaction.  this moves data from application to dialogue
\*--------------------------------------------------------------------------*/
void esa_create_sd (transaction, attribute)
transaction_type transaction;
eat_attribute attribute;
{  attribute_sdd attribute_sd;
   id_type id;

   set_status (ok);

			/* Create shared data element for specified attribute */
   attribute_sd = esa_make_sd_struct (attribute);
   check_status("esa_create_sd:  bad status from esa_make_sd_struct.");
 
			/* add the new att sd element to transaction */
   id = add_shared_data (transaction, "attribute_sdd", NULL, &attribute_sd);
   check_status ("esa_create_sd: bad status from add_shared_data");

    			/* create mapping record */
   emp_create_map (id, attribute);
   check_status("esa_create_sd:  bad status from emp_create_map.");

   return;
}                                       /* end esa_create_sd */


/*--------------------------------------------------------------------------*\
|  Routine: esa_modify_sd
| 
|  Description:
|     modifies a shared data element for a specified attribute.  this moves
|	data from application to dialogue
\*--------------------------------------------------------------------------*/
void esa_modify_sd (transaction, attribute)
transaction_type transaction;
eat_attribute attribute;
{  attribute_sdd attribute_sd;
   id_type attribute_id;

   set_status (ok);

				/* Get the att id */
   attribute_id = emp_get_id (attribute);
   check_status("esa_modify_sd:  bad status from emp_get_id.");

				/* Create shared data struct for specified
				   attribute */
   attribute_sd = esa_make_sd_struct (attribute);
   check_status ("esa_modify_sd: bad status from esa_make_sd_struct");
 
				/* add to transaction */
   put_shared_data (transaction, attribute_id, "attribute_sdd", NULL,
		    &attribute_sd);
   check_status ("esa_modify_sd:  bad status from add_shared_data.");

   return;

}                                       /* end esa_modify_sd */


/*--------------------------------------------------------------------------*\
|  Routine: esa_remove_from_sd
|
|  Description:
|     Removes an att from the shared database.  this moves information from
|	application to dialogue
\*--------------------------------------------------------------------------*/
void esa_remove_from_sd (transaction, attribute)
transaction_type transaction;
eat_attribute attribute;
{  id_type attribute_id;
   attribute_sdd *attribute_sd;

   set_status (ok);

				/* Remove the attribute from the shared data
				   base if it's in there.  */
   if ((attribute_id = emp_get_id (attribute)) != null_id)
      {remove_shared_data (transaction, "attribute_sdd", attribute_id);
      check_status("esa_remove_from_sd: bad status from remove_shared_data.");

      emp_destroy_map_by_id (attribute_id);
      check_status (
        "esa_remove_from_sd: bad status from emp_destroy_map_by_id.");     
      }
    else
      set_status (ok);

   return;
}                                       /* end esa_remove_from_sd */


/*--------------------------------------------------------------------------*\
|  Routine: esa_modify_attribute
| 
|  Description:  
|     Modifies the various parts of an existing attribute as specified by a
|     shared data record.  this moves data from dialogue to application.
|
|     Note:  This routine can not be used to reparent a attribute.
|
\*--------------------------------------------------------------------------*/
void esa_modify_attribute (transaction, attribute, attribute_sd)
transaction_type transaction;
eat_attribute attribute;
attribute_sdd attribute_sd;
{  object_instance_attribute_sdd *att_instance;
   id_type id;

   set_status (ok);

				/* Copy data from the attribute_sdd to the
				   attribute */
   eat_set_name (attribute, attribute_sd.name);
   check_status("esa_modify_attribute: bad status from eat_set_name.");

   eat_set_comment (attribute, attribute_sd.comment);
   check_status("esa_modify_attribute: bad status from eat_set_comment.");
				/* parent is not set, it cannot be changed
				   by the dialogue editor person directly */
   eat_set_snippet (attribute, attribute_sd.value);
   check_status ("esa_modify_attribute: bad status from eat_set_snippet.");
   eat_clear_ghost (attribute);
   check_status ("esa_modify_attribute: bad status from eat_set_ghost");

                        /* if there is an instance try & update it */
   if ((att_instance = eat_get_instance (attribute)) &&
      eat_snippet_2_constant (attribute, att_instance-> value,
			       sizeof (att_instance-> value)))
			/* send new instance value back to the dialogue */
      {id = emp_get_id (att_instance);
      put_shared_data (transaction, id, "object_instance_attribute_sdd",
		       "value", att_instance-> value);
      }

   return ;
}                                       /* end esa_modify_attribute */


