static char rcsid [] =
        "$Header: /u/04c/mnt/integ/source/de/src/RCS/esd.c,v 1.12 91/03/31 11:02:18 little Exp $";
/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor Dialogue Shared Data Abstraction (esd)
|
|  Description:  
|     The dialogue editor dialogue sahred data abstraction provides
|     conversions and mappings between dialogue elements and shared
|     data.
|     
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log:	esd.c,v $
 * Revision 1.12  91/03/31  11:02:18  little
 * esd_add_to_shared_data temporarily puts each component into sd individually
 * change back when add_to_shared_data is fixed
 * 
 * Revision 1.11  90/12/05  19:38:50  little
 * add esd_incr_count
 * 
 * Revision 1.10  90/09/19  16:41:20  rcs
 * modified esd remove from shared data to remove subvcs instead of 
 * subdialogues
 * 
 * Revision 1.9  90/09/07  18:28:40  little
 * fix macros stuff everywhere
 * 
 * Revision 1.8  90/08/29  18:30:32  little
 * add the counter to the dialogue sd record
 * 
 * Revision 1.7  90/08/24  16:47:33  rcs
 * no changse
 * 
 * Revision 1.6  90/07/05  15:08:12  little
 *  more structure and visualizer parts integration
 * 
 * Revision 1.5  90/06/14  13:55:39  little
 *  more changes to integrate visulization and structure editors
 * 
 * Revision 1.4  90/05/21  13:36:34  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.3  89/12/12  09:18:08  rcs
 * fixed bugs found during integration testing
 * 
 * Revision 1.2  89/11/30  13:50:19  rcs
 * fixed spelling error
 * 
 * Revision 1.1  89/11/28  18:39:40  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*-------------------- Includes --------------------------------------------*/

#define esd_main
#include "esd.h"			/* application data structures	    */
#include "edg.h"
#include "evc.h"
#include "emp.h"
#include "esv.h"
#include "esx_sd.h"
#include "ea2.h"
#ifndef memoryPack
#define memoryPack
#endif
#include "hashPack.h"
#include <string.h>


/*-------------------- Constants -------------------------------------------*/

#define DEBUG1(statement) SERPENT_DEBUG1(esd_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(esd_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(esd_debug, statement)

/*-------------------- Type Definitions ------------------------------------*/

/*-------------------- Externals -------------------------------------------*/

extern HASH id_table;
extern HASH elem_table;

/*-------------------- Retained Data ---------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: esd_set_dialogue_sdd
| 
|  Description:  
|     Creates a dialogue shared data record for a specified dialogue.
\*--------------------------------------------------------------------------*/
dialogue_sdd esd_set_dialogue_sdd (dialogue)
edg_dialogue dialogue;
{  dialogue_sdd dialogue_sd;
   evc_vc dialogue_vc;
   string file_name;
 
   set_status(ok);
			/* the god vc associated w/dialogue */ 
   dialogue_vc = edg_get_vc (dialogue);

 			/* Copy dialogue name from dialogue to the dialogue
 			   shared data record.  */
   if ((file_name = edg_get_file_name (dialogue)))
      strcpy (dialogue_sd.name, file_name );

			/* Copy the snippets.  */
   dialogue_sd.prologue = edg_get_prologue (dialogue);
   dialogue_sd.macros = edg_get_macros (dialogue);
   dialogue_sd.externs = edg_get_externs (dialogue);
   dialogue_sd.shared_data = edg_get_shared_data (dialogue);
   dialogue_sd.count = edg_get_count (dialogue);

   dialogue_sd.initialization = evc_get_create_actions (dialogue_vc);
   dialogue_sd.termination = evc_get_destroy_actions (dialogue_vc);


   return dialogue_sd;
}                                       /* end esd_set_dialogue_sdd      */


/*--------------------------------------------------------------------------*\
|  Routine: esd_modify_dialogue
| 
|  Description:  
|     Modifies the various parts of an existing dialogue based as specified by
|     a shared data record and destroys the shared data record.
\*--------------------------------------------------------------------------*/
void esd_modify_dialogue (dialogue, dialogue_sd)
edg_dialogue dialogue;
dialogue_sdd dialogue_sd;
{ 

   set_status(ok);

   edg_set_file_name (dialogue, dialogue_sd.name);
   edg_set_prologue (dialogue, dialogue_sd.prologue);
   edg_set_macros (dialogue, dialogue_sd.macros);
   edg_set_externs (dialogue, dialogue_sd.externs);
   edg_set_shared_data (dialogue, dialogue_sd.shared_data);
   edg_set_count (dialogue, dialogue_sd.count);

   evc_set_create_actions (dialogue, dialogue_sd.initialization);
   evc_set_destroy_actions (dialogue, dialogue_sd.termination);

   return ;
}                                       /* end esd_modify_dialogue          */

 
/*--------------------------------------------------------------------------*\
|  Routine: esd_add_to_shared_data
| 
|  Description:
|     Creates a shared data element for a specified dialogue and adds it to 
|     the specified transaction.  Shared data elements are also created
|     for subvcs, objects, and variables if the appropriate flags are true.  
|     Change type for shared elements is isd_create.
\*--------------------------------------------------------------------------*/
void esd_add_to_shared_data(transaction, node_selection, dialogue)
transaction_type transaction;
ea2_node_selection node_selection;
edg_dialogue dialogue;
{  dialogue_sdd dialogue_sd;
   id_type id;
   emp_map *map;
   evc_vc vc, subvc;
   eva_variable variable;

   set_status(ok);

			/* Look for dialogue in shared data mapping.  */
   map = (emp_map *) get_from_hashtable (elem_table, dialogue);

			/* Add dialogue to shared data if it is not already
			   there.  */
   if (map == NULL)
			/* dialogue not already in shared data, Create shared
			   data element for specified dialogue, add to
			   transaction, and create mapping record.  */
      {dialogue_sd = esd_set_dialogue_sdd(dialogue);
      check_status("esd_add_to_sd: bad status from esd_set_dialogue_sdd.");

#if 0
/* ????? put the components in explicitly until the interface gets fixed
	 such that add_shared_data works for a non-NULL element parameter
*/
      id = add_shared_data(transaction, "dialogue_sdd", NULL, &dialogue_sd);
      check_status("esd_add_to_shared_data:  bad status from add_shared_data.");

#endif

      id = add_shared_data (transaction, "dialogue_sdd", NULL, NULL);
      check_status("esd_add_to_shared_data:  bad status from add_shared_data.");
      put_shared_data (transaction, id, "dialogue_sdd",
		       "name", dialogue_sd.name);
      put_shared_data (transaction, id, "dialogue_sdd",
		       "prologue", &dialogue_sd.prologue);
      put_shared_data (transaction, id, "dialogue_sdd",
		       "macros", &dialogue_sd.macros);
      put_shared_data (transaction, id, "dialogue_sdd",
		       "externs", &dialogue_sd.externs);
      put_shared_data (transaction, id, "dialogue_sdd",
		       "shared_data", &dialogue_sd.shared_data);
      put_shared_data (transaction, id, "dialogue_sdd",
		       "count", &dialogue_sd.count);
      put_shared_data (transaction, id, "dialogue_sdd",
		       "initialization", &dialogue_sd.initialization);
      put_shared_data (transaction, id, "dialogue_sdd",
		       "termination", &dialogue_sd.termination);

      emp_create_map(id, dialogue);
      check_status("esd_add_to_shared_data: bad status from emp_create_map.");
      }			/* end if dialogue not already in shared data	    */

			/* Get the dialogue vc.  */
   vc = edg_get_vc(dialogue);
   check_status("esd_add_to_shared_data: bad status from edg_get_vc.");

			/* Send entire hierarchy if specified.  */
   if (node_selection == EA2_SUBTREE)
      {loop_through_list(evc_get_vcs(vc), subvc, evc_vc)
         {esv_add_to_shared_data(transaction, EA2_SUBTREE, subvc);
         check_status (
             "esd_add_to_shared_data:bad status from esv_add_to_shared_data.");
         }
      loop_through_list(evc_get_variables(vc), variable, eva_variable)
         {esx_modify_sd(transaction, variable);
         check_status("esd_add_to_sd:  bad status from esx_modify_sd.");
         }

/* ?? what about objects */
/* ?? what about instances */

   }				/* end if entire hierarchy		    */

   return;
}                                       /* end esd_add_to_shared_data       */


/*--------------------------------------------------------------------------*\
|  Routine: esd_put_into_shared_data
| 
|  Description:
|     Creates a shared data element for a specified dialogue and adds it to 
|     the specified transaction.  Shared data elements are also created
|     for subvcs and objects if the appropriate flags are true.  Change 
|     type  for shared elements is isd_create.
\*--------------------------------------------------------------------------*/
void esd_put_into_shared_data(transaction, node_selection, dialogue)
transaction_type transaction;
ea2_node_selection node_selection;
edg_dialogue dialogue;

{ dialogue_sdd dialogue_sd;
  evc_vc vc, subvc;
  eva_variable variable;
  id_type dialogue_id;

  set_status(ok);

/*
   Get the id of dialogue element from the mapping.
*/
  dialogue_id = emp_get_id(dialogue);
  check_status("esd_put_into_shared_data: bad status from emp_get_id.");
/*
   Create shared data element for specified dialogue,  add to transaction, and
   create mapping record.
*/
  dialogue_sd = esd_set_dialogue_sdd(dialogue);
  check_status("esd_put_into_shared_data: bad status from esd_set_dialogue_sdd.");
 
  put_shared_data(transaction, dialogue_id, "dialogue_sdd", NULL, &dialogue_sd);
  check_status("esd_put_into_shared_data:  bad status from add_shared_data.");
/*
   Get the dialogue vc.
*/
  vc = edg_get_vc(dialogue);
  check_status("esd_put_into_shared_data: bad status from edg_get_vc.");
/*
   Send entire hierarchy if specified.
*/
  if (node_selection == EA2_SUBTREE) {	 /* entire hierarchy        */

    loop_through_list(evc_get_vcs(vc), subvc, evc_vc) {
      esd_put_into_shared_data(transaction, EA2_SUBTREE, subvc);
      check_status("esd_put_into_sd:  bad status from esv_put_into_sd.");
    }

    loop_through_list(evc_get_variables(vc), variable, eva_variable) {
      esx_modify_sd(transaction, variable);
      check_status("esd_put_into_sd:  bad status from esx_modify_sd.");
    }

#if 0
    loop_through_list(esd_get_objects(vc), object, eob_object) {
      esd_put_into_shared_data(transaction, EA2_SUBTREE, object);
      check_status("esd_put_into_sd:  bad status from esd_put_into_sd.");
    }
#endif

  }                             /* end if entire hierarchy                  */

  return;
}                                       /* end esd_put_into_shared_data     */

/*--------------------------------------------------------------------------*\
|  Routine: esd_remove_from_shared_data
|
|  Description:
|     Removes a dialogue from the shared database.  If the entire
|     hierarchy is specified it also removes any displayed children
!     of the specified dialogue.
\*--------------------------------------------------------------------------*/
void esd_remove_from_shared_data(transaction, node_selection, dialogue)
transaction_type transaction;
ea2_node_selection node_selection;
edg_dialogue dialogue;
{ id_type dialogue_id;
  evc_vc vc, subvc;
  LIST subvc_list;

  set_status(ok);                       /* begin                            */
/*
   Get the id of dialogue element from the mapping.
*/
  dialogue_id = emp_get_id(dialogue);
/*
   Remove the dialogue from the shared data base and delete the mapping
   if it exists.
*/
  if (dialogue_id != null_id) {

    remove_shared_data(transaction, "dialogue_sdd", dialogue_id);
    check_status("esd_remove_from_sd:  bad status from remove_shared_data.");

    emp_destroy_map_by_de(dialogue);
    check_status("esd_remove_from_sd:  bad status from emp_destroy_map.");

  }
/*
   Get the dialogue vc.
*/
  vc = edg_get_vc(dialogue);
  check_status("esd_remove_from_shared_data: bad status from edg_get_vc.");
/*
   Send entire hierarchy if specified.
*/
  if (node_selection == EA2_SUBTREE) {	 /* entire hierarchy        */

    subvc_list = evc_get_vcs(vc);
    check_status("esd_remove_from_sd:  bad status from evc_get_vcs.");

    loop_through_list(subvc_list, subvc, evc_vc) {
      esv_remove_from_shared_data(transaction, EA2_SUBTREE, subvc);
      check_status("esd_remove_from_sd:  bad status from esd_put_into_sd.");
    }

    free_list(subvc_list, NOT_NODES);

  }				/* end if entire hierarchy		    */

  return;
}                                       /* end esd_remove_from_sd       */


/*--------------------------------------------------------------------------*\
|  Routine: esd_incr_count
| 
|  Description:
|	increments the app internal version of the dialogue's count attribute
|	and notifies the dialogue of the new value.
\*--------------------------------------------------------------------------*/
int esd_incr_count (dialogue, transaction)
edg_dialogue dialogue;
transaction_type transaction;
{  int count;
   id_type dialogue_id;

   set_status(ok);

			/* Get the id of dialogue element from the mapping. */
   dialogue_id = emp_get_id (dialogue);
   check_status ("esd_incr_count: bad status from emp_get_id.");

			/* create new count and save it */
   count = edg_get_count (dialogue);
   count++;
   edg_set_count (dialogue, count);

			/* tell dialogue about new count */
   put_shared_data (transaction, dialogue_id, "dialogue_sdd", "count",
		    &count);
   check_status("esd_incr_count:  bad status from add_shared_data.");

   return (count);
}                                       /* end esd_incr_count     */


