static char rcsid [] =
        "$Header: /u/04c/mnt/integ/source/de/src/RCS/esi.c,v 1.13 91/01/02 17:57:08 dvk Exp $";

/*--------------------------------------------------------------------------*
|   
| Name:  Dialoge Editor Source I/O (esi)
|
|  Description:  
|	provides source input and output functions for the various
|	editor application abstractions
|
|
|
$Log:	esi.c,v $
 * Revision 1.13  91/01/02  17:57:08  dvk
 * Eliminated label and goto code (went away with new compiler and interpreter)
 * 
 * Revision 1.12  90/09/11  08:35:46  little
 * add third parameter to esi_buffer_src_write and use it
 * 
 * Revision 1.11  90/09/07  18:29:51  little
 * fix snippet_write to cortrectly handle a "constant" snippet
 * which does not have a tree after the snippet identifier
 * 
 * Revision 1.10  90/08/30  17:36:53  little
 * make those who call get_prop & donot care about status, set it ok
 * 
 * Revision 1.9  90/08/09  11:36:33  little
 * change esi_buffer_src_write to stop on a null character
 * 
 * Revision 1.8  90/08/06  16:39:04  little
 * add the free parse tree stuff, to aid in freeing up space used by dialogue
 * importation.  also, put in an explicit non-printing character for the
 * terminator of the the temp files used to convert parse trees into text.
 * 
 * Revision 1.7  90/06/16  12:34:11  little
 * fix up the externals src out to work correctly
 * 
 * fix the snippet tree to string guys so that the output string looks a little
 * "nicer"
 * 
 * use the new parser editor import flag so the parser does not do near as
 * much clobbering to the snippet so i will bette rrepresent the original
 * input string.
 * 
 * Revision 1.6  90/05/21  13:36:53  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.5  90/03/19  13:56:26  rcs
 * temp fix for export string problem
 * 
 * Revision 1.4  89/12/18  16:52:17  bmc
 * Fixed write_buffer() macro.
 * 
 * Revision 1.3  89/12/11  11:19:53  little
 * add a check to esi_buffer_src_write to enssure that the buffer is of 
 * string type
 * 
 * Revision 1.2  89/12/11  11:02:21  little
 * fix esi_buffer_src_write to work with the new typed buffers
 * 
 * Revision 1.1  89/11/29  09:39:12  bmc
 * Initial revision
 * 
|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*-------------------- Includes --------------------------------------------*/

#define esi_main
#include "esi.h"
#include "cus.h"
#include "c22.h"
#include "idd.h"
#include "isc.h"
#include "global.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

/*-------------------- Constants -------------------------------------------*/

#undef DEBUG1
#undef DEBUG2
#undef DEBUG3
#define DEBUG1(statement) SERPENT_DEBUG1(esi_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(esi_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(esi_debug, statement)
#define STRING_FUDGE_FACTOR 10
#define EOT '\004'

#define open_tmp_file(file_name, stream, err_msg) \
   sprintf (file_name, "/tmp/FARKLE_%d", getpid ()); \
   stream = fopen (file_name, "w"); \
   if (!stream) \
      {fprintf (stderr, "%s: CANNOT OPEN TEMPORARY FILE\n\n", err_msg); \
      exit (1); \
      }

#define tmp_file_2_buffer(file_name, stream, buffer) \
   putc (EOT, stream); \
   fclose (stream); \
   buffer.body = esi_file_2_text (file_name); \
   buffer.length = strlen (buffer.body) + 1; \
   buffer.type = idd_string;



static void esi_statement_src_write ();
static void esi_expression_src_write ();
static void esi_snippet_src_write ();


/*--------------------------------------------------------------------------*\
|  Routine: esi_free_parse_tree
|
|  Description:
|     frees up the space used by a parse tree
\*--------------------------------------------------------------------------*/
void esi_free_parse_tree (the_tree)
LIST the_tree;
{  caddr_t branch;

   if (the_tree && (!cus_is_sym (the_tree)))
      {loop_through_list (the_tree, branch, caddr_t); 
         esi_free_parse_tree (branch);
      free_list (the_tree, NOT_NODES);
      }
   return;
}                                       /* end esi_free_parse_tree */


/*--------------------------------------------------------------------------*\
|  Routine: esi_file_2_text
|
|  Description:
|     read a file back into a text string
\*--------------------------------------------------------------------------*/
static string esi_file_2_text (file_name)
string file_name;
{  char *text, *cur_text;
   struct stat file_info;
   FILE *stream;

   stat (file_name, &file_info);
   text = (char *) make_node (file_info.st_size + STRING_FUDGE_FACTOR);
   cur_text = text;
   stream = fopen (file_name, "r"); 
   while ((*cur_text = getc (stream)) != EOT)
      cur_text++;
   *cur_text = '\0';
   fclose (stream);

   return (text);
}					/* end esi_file_2_text */


/*--------------------------------------------------------------------------*\
|  Routine: esi_exp_2_text
|
|  Description:
|     convert an exp to text
\*--------------------------------------------------------------------------*/
static buffer esi_exp_2_text (exp)
LIST exp;
{  FILE *stream;
   char tmp_file_name [257];
   buffer the_buff;

   DEBUG2 (fprintf (stderr, "esi_exp_2_text enter: exp = \n");
           cud_print_tree (stderr, exp, "\t"););

   open_tmp_file (tmp_file_name, stream, "\n\nesi_exp_2_text");

   esi_expression_src_write (stream, exp);
   DEBUG3 (esi_expression_src_write (stderr, exp););

   tmp_file_2_buffer (tmp_file_name, stream, the_buff);

   DEBUG2 (fprintf (stderr, "esi_exp_2_text exit\n");
           fprintf (stderr, "\ttext = %s\n", the_buff.body););
   return (the_buff);
}					/* end esi_exp_2_text */


/*--------------------------------------------------------------------------*\
|  Routine: esi_data_type_write
|
|  Description:
|     converts internal data type info into string form
\*--------------------------------------------------------------------------*/
static void esi_data_type_write (data_type, stream)
idd_data_types data_type;
FILE *stream;
{   string print_string;

   switch (data_type)
      {case idd_no_data_type:
         print_string= "ADI";
         break;
      case idd_boolean:
         print_string= "BOOLEAN";
         break;
      case idd_buffer:
         print_string= "BUFFER";
         break;
      case idd_id:
         print_string= "ID";
         break;
      case idd_integer:
         print_string= "INTEGER";
         break;
      case idd_real:
         print_string= "REAL";
         break;
      case idd_string:
         print_string= "STRING";
         break;
      default:
         fprintf (stderr, "\n\nBAD DATA TYPE IN esi_data_type_write\n");
         print_string= "NO GOOD DATA TYPE ";
      }
   fprintf (stream, "%s ", print_string);
   return;
}                                       /* end esi_data_type_write */


/*--------------------------------------------------------------------------*\
|  Routine: esi_externs_2_text
|
|  Description:
|     convert the dialogue externs to text
\*--------------------------------------------------------------------------*/
buffer esi_externs_2_text (externs_list)
LIST externs_list;
{  cus_sym the_extern;
   LIST parm_list;
   idd_data_types parm;
   FILE *stream;
   char tmp_file_name [257];
   buffer the_buff;
   int list_len, cntr;

   open_tmp_file (tmp_file_name, stream, "\n\nesi_externs_2_text");

   if (externs_list && (list_length (externs_list) > 0))
      {loop_through_list (externs_list, the_extern, cus_sym)
         {fprintf (stream, "EXTERNALS: ");
         fprintf (stream, "%s ",
                  cus_get_prop (the_extern, external_type_string));
         fprintf (stream, "%s (",
                  cus_get_prop (the_extern, c_function_name_prop));
         parm_list = (LIST) cus_get_prop (the_extern,
                                   parameter_conversion_functions_prop);
			/* we cannot use get_list_next to go down the list
			   because NULL or 0 is a valid list element */
         if (parm_list && ((list_len = list_length (parm_list)) > 0))
            {parm = (idd_data_types) get_list_pos (parm_list, 1);
            esi_data_type_write (parm, stream);
            for (cntr = 2; cntr <= list_len; cntr++)
               {fprintf (stream, ", ");
               parm = (idd_data_types) get_list_pos (parm_list, cntr);
               esi_data_type_write (parm, stream);
               }
            free_list (parm_list, NOT_NODES);
            }
         fprintf (stream, ");\n");
         }
      free_list (externs_list, NOT_NODES);
      }

   tmp_file_2_buffer (tmp_file_name, stream, the_buff);

   return (the_buff);
}					/* end esi_externs_2_text */


/*--------------------------------------------------------------------------*\
|  Routine: esi_snip_2_text
|
|  Description:
|     convert a snippet to text
\*--------------------------------------------------------------------------*/
buffer esi_snip_2_text (snippet)
LIST snippet;
{  FILE *stream;
   char tmp_file_name [257];
   buffer the_buff;

   DEBUG2 (fprintf (stderr, "esi_snip_2_text: snippet = \n");
           cud_print_tree (stderr, snippet, "\t"););

   open_tmp_file (tmp_file_name, stream, "\n\nesi_snip_2_text");

   esi_snippet_src_write (stream, snippet);
   DEBUG3 (esi_snippet_src_write (stderr, snippet););

   tmp_file_2_buffer (tmp_file_name, stream, the_buff);

   esi_free_parse_tree (snippet);
   DEBUG2 (fprintf (stderr, "esi_snip_2_text exit\n");
           fprintf (stderr, "\ttext = %s\n", the_buff.body););
   return (the_buff);
}					/* end esi_snip_2_text */


/*--------------------------------------------------------------------------*\
|  Routine: esi_atom_src_write
|
|  Description:
|     Write an atom to a source stream.
\*--------------------------------------------------------------------------*/
static void esi_atom_src_write (stream, atom)
FILE *stream;
caddr_t atom;
{  char *value;
   double *real_pointer;

   if (cus_get_prop (atom, is_constant_prop))
      {switch ((idd_data_types) cus_get_prop (atom, current_type_prop))
         {case idd_integer:
            fprintf (stream, "%d ", cus_get_prop (atom, current_value_prop));
            break;
         case idd_real:
            real_pointer = (double *)cus_get_prop (atom, current_value_prop);
            fprintf (stream, "%f ", *real_pointer);
            break;
         case idd_string:
            fprintf (stream, "\"%s\" ",cus_get_prop(atom, current_value_prop));
            break;
         case idd_boolean:
            value = cus_get_prop (atom, current_value_prop);
            if (value)
               fprintf (stream, "TRUE ");
             else
               fprintf (stream, "FALSE ");
            break;
         case idd_id:
            fprintf (stream, "%s ", cus_sym_representation (atom));
            break;
         default:
            fprintf (stream,
              "UNKNOWN TYPE OF %d", cus_get_prop (atom, current_type_prop));
            break;
         }
      }
    else if (atom == (caddr_t) car_unary_minus_symbol)
      fprintf (stream, "- ");
    else
      fprintf (stream, "%s ", cus_sym_representation (atom));
   return;
}					/* end esi_atom_src_write */


/*--------------------------------------------------------------------------*\
|  Routine:   esi_dot_src_write
|
|  Description:
|     Write a dot "expression" to a source stream.
\*--------------------------------------------------------------------------*/
static void esi_dot_src_write (stream, exp)
FILE *stream;
caddr_t exp;
{  int cnt, last_var;

   last_var = list_length (exp);
   fprintf (stream, "%s", cus_sym_representation (get_list_pos (exp, 2)));
   for (cnt = 3; cnt <= last_var; cnt++)
      {fprintf (stream, ".");
      fprintf (stream, "%s", cus_sym_representation (get_list_pos (exp, cnt)));
      }
   return;
}


/*--------------------------------------------------------------------------*\
|  Routine: esi_boolean_expression_src_write
|
|  Description:
|     Write a boolean expression to a source stream.
\*--------------------------------------------------------------------------*/
static void esi_boolean_expression_src_write (stream, exp)
FILE *stream;
LIST exp;
{  
   fprintf (stream, "(");
   esi_expression_src_write (stream, exp);
   fprintf (stream, ") ");
   return;
}


/*--------------------------------------------------------------------------*\
|  Routine: esi_expression_src_write
|
|  Description:
|     Write an expression to a source stream.
\*--------------------------------------------------------------------------*/
static void esi_expression_src_write (stream, exp)
FILE *stream;
caddr_t exp;
{  cus_sym pseudo_fun_name;
   int cnt, num_operands;

   if (cus_is_sym (exp))
      esi_atom_src_write (stream, exp);
    else
      {pseudo_fun_name = (cus_sym) get_list_pos (exp, 1);
      if ((pseudo_fun_name == car_dot_symbol) ||
          (pseudo_fun_name == car_cc_dot_symbol))
         esi_dot_src_write (stream, exp);
       else if (cus_get_prop (pseudo_fun_name, unary_operator_prop))
         {fprintf (stream, "(");
         if (pseudo_fun_name == car_unary_minus_symbol)
            fprintf (stream, "- ");
           else
            fprintf (stream, "%s ", cus_sym_representation (pseudo_fun_name));
         esi_expression_src_write (stream, get_list_pos (exp, 2));
         fprintf (stream, ") ");
         }
       else if (cus_get_prop (pseudo_fun_name, binary_operator_prop))
         {fprintf (stream, "(");
         esi_expression_src_write (stream, get_list_pos (exp, 2));
         fprintf (stream, "%s ", cus_sym_representation (pseudo_fun_name));
         esi_expression_src_write (stream, get_list_pos (exp, 3));
         fprintf (stream, ") ");
         }
       else
         {num_operands = list_length (exp) - 2;
         fprintf (stream,"%s (",cus_sym_representation(get_list_pos (exp, 2)));
         if (num_operands > 0)
            {esi_expression_src_write (stream, get_list_pos (exp, 3));
            for (cnt = 1; cnt < num_operands; cnt++)
               {fprintf (stream, ", ");
               esi_expression_src_write (stream, get_list_pos (exp, 3 + cnt));
               }
            }
         fprintf (stream, ") ");
         }
      }
   return;
}					/* end esi_expression_src_write */


/*--------------------------------------------------------------------------*\
|  Routine: esi_comment_src_write
|
|  Description:
|     Write comments to a source stream.
\*--------------------------------------------------------------------------*/
static void esi_comment_src_write (comment_tree, stream)
LIST comment_tree;
FILE *stream;
{
   int cnt, len;

   if (comment_tree)
				/* get first comment string which is just after
				   the comment symbol
				*/
      {len = list_length (comment_tree);
      DEBUG3 (fprintf (stderr, "esi_comment_src_write: list len = %d\n",len););
      for (cnt = 2; cnt <= len; cnt = cnt + 2)
         {fprintf (stream, "!* ");
         fprintf (stream, "%s ", cus_get_prop (
					get_list_pos (comment_tree, cnt),
					current_value_prop));
         fprintf (stream, "*!\n");
				/* get next comment string, the strings are
				   alternating with the comment symbol on
				   the list
				 */
         }
      }
   return;
}					/* end esi_comment_src_write */


/*--------------------------------------------------------------------------*\
|  Routine: esi_cc_2_text
|
|  Description:
|     convert a cc to text
\*--------------------------------------------------------------------------*/
buffer esi_cc_2_text (cc)
LIST cc;
{  FILE *stream;
   char tmp_file_name [257];
   buffer the_buff;

   open_tmp_file (tmp_file_name, stream, "\n\nesi_cc_2_text");

   esi_boolean_expression_src_write (stream, cc);

   tmp_file_2_buffer (tmp_file_name, stream, the_buff);

   esi_free_parse_tree (cc);

   return (the_buff);
}					/* end esi_cc_2_text */


/*--------------------------------------------------------------------------*\
|  Routine: esi_if_src_write
|
|  Description:
|     Write statement to a source stream.
\*--------------------------------------------------------------------------*/
static void esi_if_src_write (stream, if_statement)
FILE *stream;
caddr_t if_statement;
{  LIST statement, else_statements;
   LIST if_part;

   fprintf (stream, "if ");

			/* skip the if symbol */
   if_part = (LIST) get_list_next (if_statement, NULL);

			/* get the condition */
   if_part = (LIST) get_list_next (if_statement, if_part);
   esi_boolean_expression_src_write (stream, if_part);

				/* now get the then, there is always one */
   if_part = (LIST) get_list_next (if_statement, if_part);
   fprintf (stream, "then\n");
   loop_through_list (if_part, statement, LIST)
      esi_statement_src_write (stream, statement);

			/* now go thorough the rest of the if, there can
			   be a variable number of elsifs (including zero)
			   followed by an option al else */
   if_part = (LIST) get_list_next (if_statement, if_part);
   while (if_part != NULL)
      {if ((cus_sym) get_list_pos (if_part, 1) == car_elsif_symbol)
         {fprintf (stream, "elsif ");
         esi_boolean_expression_src_write (stream, get_list_pos (if_part, 2));
         fprintf (stream, "then\n");
         loop_through_list ((LIST) get_list_pos (if_part, 3), statement, LIST)
            esi_statement_src_write (stream, statement);
         }
       else
         {fprintf (stream, "else\n");
         loop_through_list (if_part, statement, LIST)
            esi_statement_src_write (stream, statement);
         }
      if_part = (LIST) get_list_next (if_statement, if_part);
      }

   fprintf (stream, "end if\n");
   return;
}


/*--------------------------------------------------------------------------*\
|  Routine: esi_while_src_write
|
|  Description:
|     Write while to a source stream.
\*--------------------------------------------------------------------------*/
static void esi_while_src_write (stream, while_statement)
FILE *stream;
caddr_t while_statement;
{  LIST statements, statement;

   fprintf (stream, "while");
   esi_boolean_expression_src_write (stream,
				     (LIST) get_list_pos (while_statement, 2));
   if (statements = (LIST) get_list_pos (while_statement, 3))
      {loop_through_list ((LIST) statements, statement, LIST)
   	 esi_statement_src_write (stream, statement);
      }
   fprintf (stream, "end while\n");
   return;
}					/* end esi_while_src_write */


/*--------------------------------------------------------------------------*\
|  Routine: esi_imperative_src_write
|
|  Description:
|     Write fun_call to a source stream.
\*--------------------------------------------------------------------------*/
static void esi_imperative_src_write (stream, fun_call)
FILE *stream;
caddr_t fun_call;
{

   esi_expression_src_write (stream, fun_call);
   return;
}					/* end esi_imperative_src_write */


#if 0
/*--------------------------------------------------------------------------*\
|  Routine: esi_goto_src_write
|
|  Description:
|     Write a goto to the source stream
\*--------------------------------------------------------------------------*/
static void esi_goto_src_write (stream, the_goto)
FILE *stream;
caddr_t the_goto;
{
   fprintf (stderr, "esi_goto_src_write: should not be here\n");
   cud_print_tree (stderr, the_goto, "  ");
   return;
}					/* end esi_goto_src_write */


/*--------------------------------------------------------------------------*\
|  Routine: esi_label_src_write
|
|  Description:
|     Write a label to the source stream
\*--------------------------------------------------------------------------*/
static void esi_label_src_write (stream, label)
FILE *stream;
caddr_t label;
{

   fprintf (stderr, "esi_label_src_write: should not be here\n");
   cud_print_tree (stderr, label, "  ");
   return;
}					/* end esi_label_src_write */
#endif


/*--------------------------------------------------------------------------*\
|  Routine: esi_gets_src_write
|
|  Description:
|     Write gets to a source stream.
\*--------------------------------------------------------------------------*/
static void esi_gets_src_write (stream, gets)
FILE *stream;
caddr_t gets;
{
				/* the lhs it not really an expression, but
				   since it has already be recognized as being
				   correct by the parser, it is safe to use
				   the expression writer to kick it out.  lhs's
				   are really a proper sub-set of expressions.
				*/
   esi_expression_src_write (stream, get_list_pos (gets, 2));
   fprintf (stream, " := ");
				/* the rhs is really an expression */
   esi_expression_src_write (stream, get_list_pos (gets, 3));
   return;
}


/*--------------------------------------------------------------------------*\
|  Routine: esi_statement_src_write
|
|  Description:
|     Write statement to a source stream.
\*--------------------------------------------------------------------------*/
static void esi_statement_src_write (stream, statement)
FILE *stream;
caddr_t statement;
{  cus_sym head;

			/*  if we got a symbol, write it straight out */

   if (cus_is_sym (statement))
      {esi_atom_src_write (stream, statement);
      fprintf (stream, ";\n");
      return;
      }

   head = (cus_sym) get_list_head (statement);
   DEBUG3 (fprintf (stderr, "esi_statement_src_write: statement = \n");
           cud_print_tree (stderr, statement, "\t"););
   if (head == car_comment_symbol)
      esi_comment_src_write ((LIST) statement, stream);
    else
      {if (head == car_if_symbol)
         esi_if_src_write (stream, statement);
       else if (head == car_while_symbol)
         esi_while_src_write (stream, statement);
       else if (head == car_function_call_symbol)
         esi_imperative_src_write (stream, statement);
       else if (head ==  car_gets_symbol)
         esi_gets_src_write (stream, statement);
#if 0
       else if (head == car_goto_symbol)
         esi_goto_src_write (stream, statement);
       else if (head == car_label_symbol)
         esi_label_src_write (stream, statement);
#endif
       else
                        /*if it is not a known statement assume it is an
			   expression */
         esi_expression_src_write (stream, statement);
      fprintf (stream, ";\n");
      }
   return;
}


/*--------------------------------------------------------------------------*\
|  Routine: esi_snippet_src_write
|
|  Description:
|     Write snippet snippet to a source stream.
\*--------------------------------------------------------------------------*/
static void esi_snippet_src_write (stream, snippet)
FILE *stream;
LIST snippet;
{  caddr_t statement;
   boolean snippet_p;

				/* note, we are skipping around the
				   first item on the list, which is the
				   snippet id */
   statement = get_list_next (snippet, NULL);
			/* find out if this is a snippet or an expression 
			   if the snippet id is of type exp_only_sym 
			   this is just an expression and does not need the
			   snippet delimiters */
   snippet_p = ((cus_sym_type) cus_get_prop (statement, type_prop) !=
			       exp_only_sym);

   if (snippet_p)
      fprintf (stream, "{");

   statement = get_list_next (snippet, statement);
   while (statement != NULL)
      {esi_statement_src_write (stream, statement);
      statement = get_list_next (snippet, statement);
      }

   if (snippet_p)
      fprintf (stream, "}\n");
   return;
}					/* end esi_snippet_src_write */


/*--------------------------------------------------------------------------*\
|  Routine: esi_buffer_src_write
|
|  Description:
|     Write a buffer to a source stream.
\*--------------------------------------------------------------------------*/
void esi_buffer_src_write (the_buffer, stream, need_semi)
buffer the_buffer;
FILE *stream;
boolean need_semi;
{  int i; 
   char *ptr, *a_semi = NULL, *white_stuff = NULL, *non_white_stuff = NULL;

   if (the_buffer.type != idd_string)
      {DEBUG3 (fprintf (stderr,
		"esi_buffer_src_write: did not get a string buffer.  ");
	       fprintf (stderr,  "it was a %d type\n", the_buffer.type););
      set_status (invalid_buffer);
      return;
      }
				/* note that counting starts at 1 and
				   proceeds to <  the_buffer.length.  this
				   takes into account that the length of a
				   string buffer includes the terminating null
				   byte */

   for (i = 1, ptr = the_buffer.body; 
        i <  the_buffer.length; 
        i++, ptr++) 
			/* stop whenever a null character is found */
      {if (! (*ptr))
         break;
			/* save the location of last ; or } found */
      if ((*ptr == ';') || (*ptr == '}'))
         a_semi = ptr;
			/* save location of last white space found */
      else if (strchr ("\011\012\013\014\015\040", *ptr))
         white_stuff = ptr;
			/* save location of last non-white space found */
      else
         non_white_stuff = ptr;
      putc (*ptr, stream); 
      }

			/* if a ; was needed and non-white space was put out
			   after the last ; or ], force out another ; */
   if (need_semi && (non_white_stuff > a_semi))
      {putc (';', stream);
      putc ('\n', stream);
      }

   return;
}					/* end esi_buffer_src_write */


/*--------------------------------------------------------------------------*\
|  Routine: esi_comments_2_text
|
|  Description:
|     convert a snippet to text
\*--------------------------------------------------------------------------*/
buffer esi_comments_2_text (comments)
LIST comments;
{  FILE *stream;
   char tmp_file_name [257];
   buffer comments_text;
   caddr_t comment;

   DEBUG2 (fprintf (stdout, "esi_comments_2_text: enter\n");
   	   cud_print_tree (stdout, comments, "");
           fprintf (stdout, "\n\n"););

   open_tmp_file (tmp_file_name, stream, "\n\nesi_comments_2_text");

   fprintf (stream, "!* ");
   if (comments)
				/* comments list is acutally a list of pairs,
				   1st element is the global comment symbol &
				   2nd element is a string type symbol whose
				   current_value_prop is the aacutal comment
				*/
      {comment = get_list_next (comments, NULL);
      while (comment != NULL)
         {comment = get_list_next (comments, comment);
         fprintf (stream, "%s\n", cus_get_prop (comment, current_value_prop));
         comment = get_list_next (comments, comment);
         }
      }

                                /* the trailing space in the format is
                                   important
                                */
   fprintf (stream, " *!  \n");

   tmp_file_2_buffer (tmp_file_name, stream, comments_text);

   esi_free_parse_tree (comments);

   return (comments_text);
}


