static char rcsid [] =
"$Header: /u/04c/mnt/integ/source/de/src/RCS/esm_sd.c,v 1.2 90/05/21 13:37:10 little Exp $";

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor Shared Data Method Abstraction  (esm)
|
|  Description:  
|     provides routines to create,modify, & delete shared data records 
|     based on methods.
|     
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
$Log:	esm_sd.c,v $
 * Revision 1.2  90/05/21  13:37:10  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.1  89/12/19  12:35:03  rcs
 * Initial revision
 * 
 * Revision 1.2  89/12/12  15:17:45  little
 * - change the strcpy of the name to use strncpy
 * - add the destroy_map to the remove function
 * 
 * Revision 1.1  89/12/07  15:11:06  little
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute   (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|   (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*-------------------- Includes --------------------------------------------*/

#define esm_main
#include "esm_sd.h"
#include "eme.h"
#include "emp.h"
#include "isc.h"
#include <string.h>

			/* need DEBUG1 for check_status to work, but do not
			   use any other debug statements, saber is a lot
			   better */
#define DEBUG1(statement) SERPENT_DEBUG1(esm_debug, statement)

/*--------------------------------------------------------------------------*\
|  Routine: esm_make_sd_struct
| 
|  Description:  
|     creates an sd structure for an method
\*--------------------------------------------------------------------------*/
method_sdd esm_make_sd_struct (method)
eme_method method;
{  method_sdd method_sd;

   set_status (ok);

				/* put att name into sd struct */
   strncpy (method_sd.name, eme_get_name (method),
            sizeof (method_sd.name));

				/* put the att parent into sd struct */
   method_sd.parent = emp_get_id (eme_get_parent (method));
   rcheck_status("esm_make_sd_struct:  bad status from emp_get_id.", method_sd);
				/* put the att comment into sd struct */
   method_sd.comment = eme_get_comment(method);
   rcheck_status("esm_make_sd_struct:  bad status from eme_get_comment.", method_sd);

				/* put the att snippet into sd struct */
   method_sd.actions = eme_get_snippet (method);
   rcheck_status("esm_make_sd_struct:  bad status from eme_get_snippet.", method_sd);

   return (method_sd);
}                                       /* end esm_make_sd_struct */


/*--------------------------------------------------------------------------*\
|  Routine: esm_create_sd
|
|  Description:
|     Creates a shared data element for a specified method and adds it to 
|     the specified transaction.
\*--------------------------------------------------------------------------*/
void esm_create_sd (transaction, method)
transaction_type transaction;
eme_method method;
{  method_sdd method_sd;
   id_type id;

   set_status (ok);

				/* Create shared data element for specified
				   method */

   method_sd = esm_make_sd_struct (method);
   check_status("esm_create_sd:  bad status from esm_make_sd_struct.");
 
			/* add the new att sd element to transaction */
   id = add_shared_data (transaction, "method_sdd", NULL, &method_sd);
   check_status ("esm_create_sd: bad status from add_shared_data");

    				/* create mapping record */
   emp_create_map (id, method);
   check_status("esm_create_sd:  bad status from emp_create_map.");

   return;
}                                       /* end esm_create_sd */


/*--------------------------------------------------------------------------*\
|  Routine: esm_modify_sd
| 
|  Description:
|     modifies a shared data element for a specified method
\*--------------------------------------------------------------------------*/
void esm_modify_sd (transaction, method)
transaction_type transaction;
eme_method method;
{  method_sdd method_sd;
   id_type method_id;

   set_status (ok);

				/* Get the att id */
   method_id = emp_get_id (method);
   check_status("esm_modify_sd:  bad status from emp_get_id.");

				/* Create shared data struct for specified
				   method */
   method_sd = esm_make_sd_struct (method);
   check_status ("esm_modify_sd: bad status from esm_make_sd_struct");
 
				/* add to transaction */
   put_shared_data (transaction, method_id, "method_sdd", NULL,
		    &method_sd);
   check_status ("esm_modify_sd:  bad status from add_shared_data.");

   return;
}                                       /* end esm_modify_sd */


/*--------------------------------------------------------------------------*\
|  Routine: esm_remove_from_sd
|
|  Description:
|     Removes an att from the shared database.
\*--------------------------------------------------------------------------*/
void esm_remove_from_sd (transaction, method)
transaction_type transaction;
eme_method method;
{  id_type method_id;

   set_status (ok);

				/* Remove the method from the shared data
				   base if it's in there.  */
   if ((method_id = emp_get_id (method)) != null_id)
      {
      
      remove_shared_data (transaction, "method_sdd", method_id);
      check_status("esm_remove_from_sd: bad status from remove_shared_data.");

      emp_destroy_map_by_id (method_id);
      check_status(
        "esm_remove_from_sd: bad status from emp_destroy_map_by_id."
      );     

      }

   return;
}                                       /* end esm_remove_from_sd */


/*--------------------------------------------------------------------------*\
|  Routine: esm_modify_method
| 
|  Description:  
|     Modifies the various parts of an existing method as specified by a
|     shared data record and then destroys the shared data record.
|
|     Note:  This routine can not be used to reparent a method.
|
\*--------------------------------------------------------------------------*/
void esm_modify_method (method, method_sd)
eme_method method;
method_sdd method_sd;
{

   set_status (ok);

				/* Copy data from the method_sdd to the
				   method */
   eme_set_name (method, method_sd.name);
   check_status("esm_modify_method: bad status from eme_set_name.");

   eme_set_comment (method, method_sd.comment);
   check_status("esm_modify_method: bad status from eme_set_comment.");
				/* parent is not set, it cannot be changed
				   by the dialogue editor person directly */
   eme_set_snippet (method, method_sd.actions);
   check_status("esm_modify_method: bad status from eme_set_snippet.");

   return ;
}                                       /* end esm_modify_method */


