static char rcsid [] =
    "$Header: /usr/users/little/serpent/mips/de/src/RCS/eso_sd.c,v 1.9 1991/05/07 17:50:00 little Exp $";

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Shared Data Object Abstraction (eso_sd.c)
|
|  Description:  
|     The dialogue editor shared data abstraction provides routines
|     to create shared data records based on current object data,
|     modify object data based on shared data values and destroy
|     shared data records.
|     
|  Component of:
|     Dialogue Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|

$Log: eso_sd.c,v $
 * Revision 1.9  1991/05/07  17:50:00  little
 * remove the DEBUG statements so dbx does not get confused
 * add object counting functionality
 *
 * Revision 1.8  1990/12/05  19:39:33  little
 * in eso_make_inst_sd_struct:  change name to be unique by
 *   using the count guy from the dialogue entity.
 *   set the new mailbox component in the object_instance_sdd
 *
 * Revision 1.7  90/10/23  14:10:46  little
 * no changes
 * 
 * Revision 1.6  90/07/05  15:08:35  little
 *  more structure and visualizer parts integration
 * 
 * Revision 1.5  90/06/14  13:55:55  little
 *  more changes to integrate visulization and structure editors
 * 
 * Revision 1.4  90/05/30  15:31:10  clc
 * set user_created to false always
 * 
 * Revision 1.3  89/12/20  14:44:00  rcs
 * integratino testing
 * 
 * Revision 1.2  89/12/13  09:25:15  bmc
 * Removal of object shared data now removes methods and attributes as well.
 * 
 * Revision 1.1  89/12/12  12:58:11  bmc
 * Initial revision
 * 

|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*-------------------- Includes --------------------------------------------*/

#include <string.h>			/* C string functions		    */

#define eso_main
#include "eso_sd.h"			/* object shared data stuff	    */
#include "esa_sd.h"			/* attribute shared data stuff	    */

#include "evc_cmd.h"
#include "eob.h"			/* editor object abstraction	    */
#include "eat.h"			/* editor attribute abstraction	    */
#include "eme.h"			/* editor method abstraction	    */
#include "emp.h"			/* editor mapping abstraction	    */
#include "e22.h"
#include "se.h"
#include "serpent.h"			/* general Serpent declarations     */

/*-------------------- Constants -------------------------------------------*/

#define DEBUG1(statement) SERPENT_DEBUG1(eso_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(eso_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(eso_debug, statement)


			/* set big enuf to hold almost anything. */
#define name_buff_size 1024


/*--------------------------------------------------------------------------*\
|  Routine: eso_make_inst_sd_struct
|
|  Description:
|     Creates a shared data structure for an object instance
\*--------------------------------------------------------------------------*/
object_instance_sdd *eso_make_inst_sd_struct (object, out_trans)
eob_object object;
transaction_type out_trans;
{  object_instance_sdd *instance;
   int count;
   char name_buff [name_buff_size];

   instance = (object_instance_sdd *) make_node (sizeof (object_instance_sdd));
   if (!instance)
      {fprintf (stderr, "eso_make_inst_sd_struct: outta memory\n");
      set_status (out_of_memory);
      return (NULL);
      }
			/* set the template to point at the instance */
   instance-> template = (id_type) object;
			/* this is using the external named dialogue from the
			   eci.c file.  do bitch at me (reed), i didn't name
			   it.  */
   count = esd_incr_count (e22_dialogue, out_trans);
			/* now build a unique name based on the name in the
			   template and the unique counter */
   sprintf (name_buff, "%s_%d", eob_get_name (object), count);
			/* put the name in the sd record.  the size of the
			   name component is 33 in the se.sdd
			   file.  this should change if se.sdd changes */
   strncpy (instance-> name, name_buff, 32);
			/* set up the rest of the values in the instance */
   strcpy (instance-> type, eob_get_type (object));
   instance-> visible = FALSE;
   strcpy (instance-> tech, eob_get_technology (object));
   strcpy (instance-> mailbox, eta_get_mailbox (instance-> tech));

   eob_set_instance (object, instance);

   return (instance);
}					/* end eso_make_inst_sd_struct */


/*--------------------------------------------------------------------------*\
|  Routine: eso_make_sd_struct
|
|  Description:
|     Creates a shared data structure for an object template.
\*--------------------------------------------------------------------------*/
object_sdd eso_make_sd_struct (object)
eob_object object;
{  object_sdd object_sd;
   caddr_t parent;
   caddr_t dialogue;

   set_status (ok);

   strncpy (object_sd.name, eob_get_name (object), sizeof (object_sd.name) - 1);
   strncpy (object_sd.type, eob_get_type (object), sizeof (object_sd.type) - 1);
   strncpy (object_sd.technology, eob_get_technology (object),
	    sizeof (object_sd.technology) - 1);
   object_sd.comment = eob_get_comments (object);
   object_sd.user_created = false;

   parent = (caddr_t) eob_get_parent (object);

			/* Check the parent's parent.  If it's a dialogue, then
			   the parent VC is Eve.  In that case, set the parent
			   element of the structure to the dialogue, not to its
			   pseudo-VC.  Otherwise, it's okay to use the VC.  */
   if ((dialogue = evc_get_dialogue ((evc_vc) parent)) != EDG_NO_DIALOGUE )
      parent = (caddr_t) dialogue;

   object_sd.parent = emp_get_id (parent);
   return (object_sd);
}					/* end eso_make_sd_struct	    */


/*--------------------------------------------------------------------------*\
|  Routine: eso_create_sd
|
|  Description:
|     Creates a shared data element for a specified object and adds it to
|     the specified transaction.  this sends information from app to dialogue
\*--------------------------------------------------------------------------*/
void eso_create_sd (transaction, object)
transaction_type transaction;
eob_object object;
{  id_type id;
   object_sdd object_sd;

   set_status (ok);

			/* Initialize a shared data record for the specified
			   object.  */
   object_sd = eso_make_sd_struct (object);
   check_status ("eso_create_sd: can't initialize shared data structure.");

			/* Add the new object shared data record to the
			   transaction.  */
   id = add_shared_data (transaction, "object_sdd", NULL, &object_sd);
   check_status ("eso_create_sd: can't add object to transaction");

			/* Create an editor mapping record.  */
   emp_create_map (id, object);

   return;
}					/* end eso_create_sd		    */


/*--------------------------------------------------------------------------*\
|  Routine: eso_modify_sd
|
|  Description:
|     Modifies a shared data element for a specified object.  this sends
|	information from app to dialogue.
\*--------------------------------------------------------------------------*/
void eso_modify_sd (transaction, object)
transaction_type transaction;
eob_object object;
{
    id_type id;
    object_sdd object_sd;


    set_status (ok);

    /*
    ** Map the object into its associated id.
    */
    id = emp_get_id (object);

    /*
    ** Initialize a shared data record for the object.
    */
    object_sd = eso_make_sd_struct (object);
    check_status ("eso_modify_sd: can't initialize shared data structure.");

    /*
    ** Put item into transaction.
    */
    put_shared_data (transaction, id, "object_sdd", NULL, &object_sd);
    check_status ("eso_modify_sd: can't put object into transaction.");

    return;
}					/* end eso_modify_sd		    */

/*--------------------------------------------------------------------------*\
|  Routine: eso_remove_from_sd
|
|  Description:
|     Removes an object from the shared database.  this sends information from
|	app to dialogue.
\*--------------------------------------------------------------------------*/
void eso_remove_from_sd (transaction, the_object)
transaction_type transaction;
eob_object the_object;
{
    id_type id;


    set_status (ok);

    /*
    ** Remove the object from the shared data base and from mapping data
    ** base.  Removing the object implies removing all its attributes and
    ** methods as well.
    */

    if ( (id = emp_get_id (the_object)) != null_id )
	{
	LIST list;
	eat_attribute attribute;
	eme_method method;


	/* Remove any and all attributes. */

	list = eob_get_attributes (the_object);
	check_status ("eso_remove_from_sd: can't get list of attributes");
	loop_through_list (list, attribute, eat_attribute)
	    {esa_remove_from_sd (transaction, attribute);
	    check_status ("eso_remove_from_sd: failed to remove attribute");
	    }

	/* Remove any and all methods. */

	list = eob_get_methods (the_object);
	check_status ("eso_remove_from_sd: can't get list of methods");
	loop_through_list (list, method, eme_method)
	    {
/* ???? why are we not removing methods in eso_remove_from_sd */
#if 0
	    esm_remove_from_sd (transaction, method);
	    check_status ("eso_remove_from_sd: failed to remove method");
#endif
	    }


	/* Remove the object itself. */

	remove_shared_data (transaction, "object_sdd", id);
	check_status ("eso_remove_from_sd: can't remove object from s.d.");

	/* Finally, destroy the mapping entry. */

	emp_destroy_map_by_id (id);
	check_status ("eso_remove_from_sd: can't remove object from map");
	}

    return;
}					/* end eso_remove_from_sd	    */


/*--------------------------------------------------------------------------*\
|  Routine: eso_modify_object
|
|  Description:
|     Modifies the various parts of an existing object as specified by a
|     	shared data record and then destroys the shared data record.  this
|	sends information from dialogue to app.
|
|     Note:  This routine can not be used to reparent an object.
\*--------------------------------------------------------------------------*/
void eso_modify_object (transaction, object, object_sd)
transaction_type transaction;
eob_object object;
object_sdd object_sd;
{  object_instance_sdd *obj_instance;
   id_type id;

   set_status (ok);


			/* Copy data from the shared data record to the object.
			   Note that this routine can't set the technology or
			   type (since these values can only be specified at
			   object creation) and won't set the parent.  */
   eob_set_name (object, object_sd.name);
   eob_set_comments (object, object_sd.comment);

			/* if there is an instance, update its name */
   if ((obj_instance = eob_get_instance (object)))
      {strcpy (obj_instance-> name, object_sd.name);
      id = emp_get_id (obj_instance);
      put_shared_data (transaction, id, "object_instance_sdd", "name",
		       obj_instance-> name);
      }

   return;
}					/* end eso_modify_object	    */


