static char rcsid [] =
	"$Header: /u/04c/mnt/integ/source/de/src/RCS/est.c,v 1.2 90/08/30 17:37:27 little Exp $";

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Symbol Table Routines
|
|  Description:  
|     This package provides additional symbol table routines not already
|     provided by "cus".  They are completely compatible with the "cus"
|     routines and are intended to be used in tandem with "cus", rather than
|     in lieu of it.
|
|  Component of:
|     Dialogue Editor Layer.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:  
|     Len Bass
|     ljb@sei.cmu.edu
|

$Log:	est.c,v $
 * Revision 1.2  90/08/30  17:37:27  little
 * make those who call get_prop & donot care about status, set it ok
 * 
 * Revision 1.1  89/11/29  09:39:39  bmc
 * Initial revision
 * 

|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Includes: est
|
|  Description:
|     Necessary includes for the est package.
|
\* Definition:                                                              */

#include <listPack.h>           /* C Toolbox - listPack                     */
#define memoryPack              /* use memoryPack rather than mallocPack    */
#include "serpent.h"		/* General Serpent defs 		    */
#include "global.h"             /* Global type defs and constants           */
#include "est.h"                /* package definitions			    */
#include "cus.h"		/* symbol table utility routines and defs   */
#include "ubo.h"		/* buffer definitions and operations 	    */
#include "u22.h"		/* general purpose Serpent routines 	    */
#include <string.h>		/* C string functions			    */

/*
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
|  Routine: est_set_buffer_property
|
|  Description:
|     Provides a convenient way to store a buffer in the symbol table.
|     This routine makes a copy of a buffer and stores the copy in the
|     symbol table.
\*--------------------------------------------------------------------------*/

void est_set_buffer_property (ste, property_name, the_buffer)
cus_sym ste;
cus_property_type property_name;
buffer the_buffer;
{
    buffer *stored_buffer;


				    /*
				       We need to store a copy of the buffer,
				       not the actual buffer, since the actual
				       buffer may reside on the stack.  Re-use
				       the buffer already in the property if
				       there is one.  Otherwise, create a one.
				    */

    set_status (ok);
    stored_buffer = (buffer *) cus_get_prop (ste, property_name);
			/* do not care is prop not there */
    set_status (ok);
    if (stored_buffer == NULL_BUFFER)
	stored_buffer = ubo_create_buffer ();

    if (stored_buffer != NULL_BUFFER)
	{
	ubo_buffcpy (stored_buffer, &the_buffer);
	cus_put_prop (ste, property_name, (caddr_t) stored_buffer);
	}

    return;
}

/*--------------------------------------------------------------------------*\
|  Routine: est_get_buffer_property
|
|  Description:
|     Provides a convenient way to retrieve a buffer property from the
|     symbol table. 
\*--------------------------------------------------------------------------*/

buffer est_get_buffer_property (ste, property_name)
cus_sym ste;
cus_property_type property_name;
{
    buffer the_buffer;
    buffer *buf_ptr;


    set_status (ok);
    ubo_init_buffer (the_buffer);

    buf_ptr = (buffer *) cus_get_prop (ste, property_name);
                        /* do not care is prop not there */
    set_status (ok);

    if ( (get_status () == ok) && (buf_ptr != NULL_BUFFER) )
	the_buffer = *buf_ptr;

    return (the_buffer);
}

/*--------------------------------------------------------------------------*\
|  Routine: est_clear_buffer_property
|
|  Description:
|     Clears a symbol table buffer property previously set via
|     est_set_buffer_property, freeing all necessary storage.
\*--------------------------------------------------------------------------*/

void est_clear_buffer_property (ste, property_name)
cus_sym ste;
cus_property_type property_name;
{
    buffer *the_buffer;


    set_status (ok);
    the_buffer = (buffer *) cus_get_prop (ste, property_name);
                        /* do not care is prop not there */
    set_status (ok);

    if ( the_buffer != NULL_BUFFER )
	{
	ubo_destroy_buffer (the_buffer);
	cus_put_prop (ste, property_name, NULL_BUFFER);
	}

    return;
}

/*--------------------------------------------------------------------------*\
|  Routine: est_set_string_property
|
|  Description:
|     Provides a convenient way to store a string in the symbol table.
|     This routine makes a copy of a string and stores the copy in the
|     symbol table.
\*--------------------------------------------------------------------------*/

void est_set_string_property (ste, property_name, the_string)
cus_sym ste;
cus_property_type property_name;
string the_string;
{
    string new_string;


    set_status (ok);
				    /* Clear the property first.  */

    est_clear_string_property (ste, property_name);
    if (the_string != NULL)
	{
				    /*
				       We need to store a copy of the string,
				       not the actual string, since the actual
				       string could reside anywhere.
				    */
	if ( (new_string = copy_string (the_string)) == NULL )
	    set_status (out_of_memory);

	else
	    cus_put_prop (ste, property_name, new_string);
	}

    return;
}

/*--------------------------------------------------------------------------*\
|  Routine: est_clear_string_property
|
|  Description:
|     Clears a symbol table string property previously set via
|     est_set_string_property, freeing all necessary storage.
\*--------------------------------------------------------------------------*/

void est_clear_string_property (ste, property_name)
cus_sym ste;
cus_property_type property_name;
{
    string *the_string;


    set_status (ok);
    the_string = (string *) cus_get_prop (ste, property_name);
                        /* do not care is prop not there */
    set_status (ok);

    if ( the_string != NULL )
	{
	free_node (the_string);
	cus_put_prop (ste, property_name, NULL);
	}

    return;
}

/*--------------------------------------------------------------------------*\
|  Routine: est_free_list_property
|
|  Description:
|     Frees a symbol table list property.
\*--------------------------------------------------------------------------*/

void est_free_list_property (ste, property_name)
cus_sym ste;
cus_property_type property_name;
{
    LIST the_list;


    set_status (ok);
    the_list = (LIST) cus_get_prop (ste, property_name);
                        /* do not care is prop not there */
    set_status (ok);

    if ( the_list != NULL )
	{
	free_list (the_list, NOT_NODES);
	cus_put_prop (ste, property_name, NULL);
	}

    return;
}
