static char rcsid [] =
        "$Header: /u/04c/mnt/integ/source/de/src/RCS/esv.c,v 1.7 90/07/05 15:08:57 little Exp $";
/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor Shared Data VC Abstraction (esv)
|
|  Description:  
|     The dialogue editor shared data abstraction provides routines
|     to create shared data records based on current view controller data,
|     modify view controller data based on shared data values and destroy
|     shared data records.
|     
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log:	esv.c,v $
 * Revision 1.7  90/07/05  15:08:57  little
 *  more structure and visualizer parts integration
 * 
 * Revision 1.6  90/06/16  12:29:21  little
 * make add user_created sd components false
 * 
 * Revision 1.5  90/05/21  13:37:32  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.4  89/12/20  14:44:56  rcs
 * integration testing
 * 
 * Revision 1.3  89/12/12  07:56:03  rcs
 * fixed bugs found during integration testing
 * 
 * Revision 1.2  89/11/30  13:50:39  rcs
 * fixed spelling error
 * 
 * Revision 1.1  89/11/28  18:40:00  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*-------------------- Includes --------------------------------------------*/

#define esv_main
#include "esv.h"			/* application data structures	    */
#include "eso_sd.h"
#include "esx_sd.h"
#include "edg.h"
#include "evc.h"
#include "eva.h"
#include "emp.h"
#include <string.h>


/*-------------------- Constants -------------------------------------------*/

#define DEBUG1(statement) SERPENT_DEBUG1(esv_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(esv_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(esv_debug, statement)

/*-------------------- Type Definitions ------------------------------------*/

/*-------------------- Externals -------------------------------------------*/


/*-------------------- Retained Data ---------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: esv_set_vc_sdd
| 
|  Description:  
|     Sets a vc shared data record for a specified vc.
\*--------------------------------------------------------------------------*/
vc_sdd esv_set_vc_sdd(vc)
evc_vc vc;
{ vc_sdd vc_sd;
  evc_vc parent_vc;
  edg_dialogue dialogue;
  id_type map;

   set_status (ok);
   DEBUG2 ( fprintf(stderr, "esv_set_vc_sdd: call.\n"););

			/* Copy vc name from vc to the vc shared data record*/
   strcpy(vc_sd.name, evc_get_name(vc));

			/* Get the parent vc.  */
   parent_vc = evc_get_parent(vc);

			/* Determine the parent id.  If the parent_vc is Eve
			   then get use the id of the dialogue, otherwise use
			   the id of the view controller.  */
   dialogue = evc_get_dialogue(parent_vc);

   DEBUG3 (fprintf(stderr, "esv_set_vc_sdd: dialogue = %x.\n", dialogue););

   if (dialogue != EDG_NO_DIALOGUE)
			/* vc is Eve */
      map = emp_get_id (dialogue);    
    else
			/* vc is not eve		*/
      map = emp_get_id  (parent_vc);

   vc_sd.parent = map;

			/* Copy the snippets.  */
  vc_sd.comment = evc_get_comments(vc);
  vc_sd.create_condition = evc_get_creation_condition(vc);

  DEBUG3 (fprintf(stderr,"esv_set_vc_sdd: vc_sd.create_condition.type = %d.\n",
      		  vc_sd.create_condition.type);
    fprintf (stderr, "esv_set_vc_sdd: vc_sd.create_condition.length = %d.\n",
             vc_sd.create_condition.length);
    fprintf (stderr, "esv_set_vc_sdd: vc_sd.create_condition.length = %x.\n",
      	     vc_sd.create_condition.body););

  vc_sd.create_action = evc_get_create_actions(vc);
  vc_sd.destroy_action = evc_get_destroy_actions(vc);

  DEBUG3(
    fprintf(stderr, "esv_set_vc_sdd: vc_sd.destroy_action.type = %d.\n",
      vc_sd.destroy_action.type
    );
    fprintf(stderr, "esv_set_vc_sdd: vc_sd.destroy_action.length = %d.\n",
      vc_sd.destroy_action.length
    );
    fprintf(stderr, "esv_set_vc_sdd: vc_sd.destroy_action.length = %x.\n",
      vc_sd.destroy_action.body
    );
  );

  vc_sd.user_created = FALSE;

  DEBUG2 (fprintf(stderr, "esv_set_vc_sdd exiting.\n"););

  return vc_sd;
}                                       /* end esv_set_vc_sdd          */


/*--------------------------------------------------------------------------*\
|  Routine: esv_add_to_shared_data
| 
|  Description:
|     Creates a shared data element for a specified vc and adds it to 
|     the specified transaction.  Shared data elements are also created
|     for subvcs and objects if the appropriate flags are true.  Change 
|     type  for shared elements is isd_create.
\*--------------------------------------------------------------------------*/
void esv_add_to_shared_data(transaction, node_selection, vc)
transaction_type transaction;
evc_node_selection node_selection;
evc_vc vc;
{  vc_sdd vc_sd;
   id_type id, map;
   evc_vc subvc;

   set_status (ok);
   DEBUG2 (fprintf(stderr, "esv_add_to_shared_data: call.\n");
          fprintf (stderr, "esv_add_to_shared_data: transaction = %x.\n", 
      		   transaction);
          fprintf (stderr, "esv_add_to_shared_data: node_selection = %d.\n", 
      		   node_selection);
          fprintf (stderr, "esv_add_to_shared_data: vc = %x.\n", vc););

			/* Look for VC in shared data mapping.  */
   map = emp_get_id (vc);

			/* Add VC to shared data if it is not already there. */
   if (map == null_id)
			/* vc not already in shared data	*/
      {
			/* Create shared data element for specified vc, add to
			   transaction, and create mapping record.  */
      vc_sd = esv_set_vc_sdd(vc);
      check_status("esv_add_to_shared_data: bad status from esv_set_vc_sdd.");
 
      id = add_shared_data(transaction, "vc_sdd", NULL, &vc_sd);
      check_status("esv_add_to_shared_data:  bad status from add_shared_data.");

      emp_create_map(id, vc);
      check_status("esv_add_to_shared_data: bad status from emp_create_map.");

      }				/* end if vc not already in shared data	    */

			/* Send entire hierarchy if specified.  */
   if (node_selection == EVC_ENTIRE_HIERARCHY)
			/* entire hierarchy        */
      {loop_through_list(evc_get_vcs(vc), subvc, evc_vc)
         esv_add_to_shared_data(transaction, EVC_ENTIRE_HIERARCHY, subvc);
       }

  DEBUG2 (fprintf(stderr, "esv_add_to_shared_data:  exiting.\n"););
  return;
}                                       /* end esv_add_to_shared_data       */


/*--------------------------------------------------------------------------*\
|  Routine: esv_put_into_shared_data
| 
|  Description:
|     Creates a shared data element for a specified vc and adds it to 
|     the specified transaction.  Shared data elements are also created
|     for subvcs and objects if the appropriate flags are true.  Change 
|     type  for shared elements is isd_create.
\*--------------------------------------------------------------------------*/
void esv_put_into_shared_data(transaction, node_selection, vc)
transaction_type transaction;
evc_node_selection node_selection;
evc_vc vc;
{ vc_sdd vc_sd;
  eva_variable variable;
  eob_object object;
  evc_vc subvc;
  id_type vc_id;

  set_status(ok);
  DEBUG2(
    fprintf(stderr, "esv_put_into_shared_data: call.\n");
    fprintf(
      stderr, 
      "esv_put_into_shared_data: transaction = %x.\n", 
      transaction
    );
    fprintf(stderr, "esv_put_into_shared_data: node_selection = %d.\n", 
      node_selection
    );
    fprintf(stderr, "esv_put_into_shared_data: vc = %x.\n", vc);
  );
/*
   Get the id of dialogue element from the mapping.
*/
  vc_id = emp_get_id(vc);
  check_status("esv_put_into_shared_data: bad status from emp_get_id.");
/*
   Create shared data element for specified vc,  add to transaction, and
   create mapping record.
*/
  vc_sd = esv_set_vc_sdd(vc);
  check_status("esv_put_into_shared_data: bad status from esv_set_vc_sdd.");
 
  put_shared_data(transaction, vc_id, "vc_sdd", NULL, &vc_sd);
  check_status("esv_put_into_shared_data:  bad status from add_shared_data.");
/*
   Send entire hierarchy if specified.
*/
  if (node_selection == EVC_ENTIRE_HIERARCHY) {	 /* entire hierarchy        */

    loop_through_list(evc_get_vcs(vc), subvc, evc_vc) {
      esv_put_into_shared_data(transaction, EVC_ENTIRE_HIERARCHY, subvc);
      check_status("esv_put_into_sd:  bad status from esv_put_into_sd.");
    }

    loop_through_list(evc_get_variables(vc), variable, eva_variable) {
      esx_modify_sd(transaction, variable);
      check_status("esv_put_into_sd:  bad status from esx_modify_sd.");
    }

    loop_through_list(evc_get_objects(vc), object, eob_object) {
      eso_modify_sd(transaction, object);
      check_status("esv_put_into_sd:  bad status from esv_put_into_sd.");
    }

  }                             /* end if entire hierarchy                  */
/*
   Return.
*/
  DEBUG2(
    fprintf(stderr, "esv_put_into_shared_data:  exiting.\n");
  );
  return;
}                                       /* end esv_put_into_shared_data     */

/*--------------------------------------------------------------------------*\
|  Routine: esv_remove_from_shared_data
|
|  Description:
|     Removes a view controller from the shared database.  If the entire
|     hierarchy is specified it also removes any displayed children
!     of the specified VC.
\*--------------------------------------------------------------------------*/
void esv_remove_from_shared_data(transaction, node_selection, vc)
transaction_type transaction;
evc_node_selection node_selection;
evc_vc vc;
{ id_type vc_id;
  evc_vc subvc;
  eva_variable variable;
  eob_object object;
  LIST subvc_list;

  set_status(ok);
  DEBUG2(
    fprintf(stderr, "esv_remove_from_sd: call.\n");
    fprintf(
      stderr, 
      "esv_remove_from_sd: transaction = %x.\n", 
      transaction
    );
    fprintf(stderr, "esv_remove_from_sd: node_selection = %d.\n", 
      node_selection
    );
    fprintf(stderr, "esv_remove_from_sd: vc = %x.\n", vc);
  );
/*
   Get the id of dialogue element from the mapping.
*/
  vc_id = emp_get_id(vc);
/*
   Remove the vc from the shared data base and delete the mapping
   if it exists.
*/
  if (vc_id != null_id) {

    remove_shared_data(transaction, "vc_sdd", vc_id);
    check_status("esv_remove_from_sd:  bad status from remove_shared_data.");

    emp_destroy_map_by_de(vc);
    check_status("esv_remove_from_sd:  bad status from emp_destroy_map.");

  }
/*
   Send entire hierarchy if specified.
*/
  if (node_selection == EVC_ENTIRE_HIERARCHY) {	 /* entire hierarchy        */

    subvc_list = evc_get_vcs(vc);
    check_status("esv_remove_from_sd:  bad status from evc_get_vcs.");

    DEBUG3(
      fprintf(stderr, "esv_remove_from_sd: subvc_list = %x.\n", subvc_list);
    );

    loop_through_list(subvc_list, subvc, evc_vc) {
      esv_remove_from_shared_data(transaction, EVC_ENTIRE_HIERARCHY, subvc);
      check_status("esv_remove_from_sd:  bad status from esv_put_into_sd.");
    }

    loop_through_list(evc_get_variables(vc), variable, eva_variable) {
      esx_remove_from_sd(transaction, variable);
      check_status("esv_remove_from_sd:  bad status from esx_remove_from_sd.");
    }

    loop_through_list(evc_get_objects(vc), object, eob_object) {
      eso_remove_from_sd(transaction, object);
      check_status("esv_remove_from_sd:  bad status from esv_put_into_sd.");
    }

    free_list(subvc_list, NOT_NODES);

  }				/* end if entire hierarchy		    */
/*
   Return.
*/
  DEBUG2(
    fprintf(stderr, "esv_remove_from_sd:  exiting.\n");
  );
  return;
}                                       /* end esv_remove_from_sd       */

/*--------------------------------------------------------------------------*\
|  Routine: esv_modify_vc
| 
|  Description:  
|     Modifies the name, create condition, create & destroy actions of
|     an existing vc based as specified by a shared data record 
|     and destroys the shared data record.
|
|     Note:  This routine can not be used to reparent a vc.
|
\*--------------------------------------------------------------------------*/
void esv_modify_vc(vc, vc_sd)
evc_vc vc;
vc_sdd vc_sd;
{                                       /* local type definitions           */
/*
   Initialize.
*/
  set_status(ok);                       /* begin                            */
  DEBUG2(
    fprintf(stderr, "esv_modify_vc: call.\n");
  );
/*
   Copy data from the vc_sdd to the vc.
*/
  
  evc_set_name(vc, vc_sd.name);
  check_status("esv_modify_vc:  bad status from evc_set_name.");

  evc_set_comments(vc, vc_sd.comment);
  check_status("esv_modify_vc:  bad status from evc_set_comments.");

  evc_set_creation_condition(vc, vc_sd.create_condition);
  check_status("esv_modify_vc:  bad status from evc_set_creation_condition.");

  evc_set_create_actions(vc, vc_sd.create_action);
  check_status("esv_modify_vc:  bad status from evc_set_create_actions.");

  evc_set_destroy_actions(vc, vc_sd.destroy_action);
  check_status("esv_modify_vc:  bad status from evc_set_destroy_actions.");
/*
   Return.
*/
  DEBUG2(
    fprintf(stderr, "esv_modify_vc:  exiting.\n");
  );
  return ;
}                                       /* end esv_modify_vc          */
