static char rcsid [] =
"$Header: /u/04c/mnt/integ/source/de/src/RCS/esx_sd.c,v 1.9 90/05/21 13:37:57 little Exp $";

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialoge Editor Variable Shared Data 
|
|  Description:  
|     provides routines to create,modify, & delete shared data records based on
|     variables
|     
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
$Log:	esx_sd.c,v $
 * Revision 1.9  90/05/21  13:37:57  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.8  89/12/21  16:28:17  rcs
 * dicked around with user_created field in variable_sdd
 * 
 * Revision 1.7  89/12/20  14:45:49  rcs
 * integration testing
 * 
 * Revision 1.6  89/12/12  17:01:50  little
 * change strcpy to strncpy
 * 
 * Revision 1.5  89/12/12  10:57:19  little
 * redo the mevore parameters yet again
 * 
 * Revision 1.4  89/12/12  10:33:58  little
 * make the parameters to exs_remove_sd correct
 * 
 * Revision 1.3  89/12/11  16:34:30  little
 * fix up some things i cannot remember right now
 * ,
 * 
 * Revision 1.2  89/12/07  17:26:45  rcs
 * soiled reeds abstraction
 * 
 * Revision 1.1  89/12/07  15:12:07  little
 * Initial revision
 * 
|     6 dec 1989: Module created.
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute   (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|   (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*-------------------- Includes --------------------------------------------*/

#define esx_main
#include "esx_sd.h"
#include "eva.h"
#include "emp.h"
#include "isc.h"
#include "idd.h"
#include <string.h>


			/* need DEBUG1 for check_status to work, but do not
			   use any other debug statements, saber is a lot
			   better */
#define DEBUG1(statement) SERPENT_DEBUG1(esx_debug, statement)

/*--------------------------------------------------------------------------*\
|  Routine: esx_make_sd_struct
| 
|  Description:  
|     creates an sd structure for an variable
\*--------------------------------------------------------------------------*/
variable_sdd esx_make_sd_struct (variable)
eva_variable variable;
{  variable_sdd variable_sd;
   caddr_t parent, test_lookie;

   set_status (ok);

				/* put var name into sd struct */
   strncpy (variable_sd.name, eva_get_name (variable),
	   sizeof (variable_sd.name) - 1);

				/* put the var parent into sd struct */
   parent = (caddr_t) eva_get_parent (variable);
				/* parent vc is eve, the pseudo-vc does not
				   have a shared data entry, but the dialogue
				   does, so get it */
   if ((test_lookie = evc_get_dialogue (parent)) != EDG_NO_DIALOGUE)
      parent = test_lookie;

   variable_sd.parent = emp_get_id (parent);

				/* put the var comment into sd struct */
   variable_sd.comment = eva_get_comment (variable);

				/* put the var snippet into sd struct */
   variable_sd.value = eva_get_snippet (variable);

				/* default this off */
   variable_sd.user_created = false;

   return (variable_sd);
}                                       /* end esx_make_sd_struct */


/*--------------------------------------------------------------------------*\
|  Routine: esx_create_sd
|
|  Description:
|     Creates a shared data element for a specified variable and adds it to 
|     the specified transaction.
\*--------------------------------------------------------------------------*/
void esx_create_sd (transaction, variable)
transaction_type transaction;
eva_variable variable;
{  variable_sdd variable_sd;
   id_type id;

   set_status (ok);

				/* Create shared data element for specified
				   variable */

   variable_sd = esx_make_sd_struct (variable);
 
				/* add the new var sd element to transaction */
   id = add_shared_data (transaction, "variable_sdd", NULL, &variable_sd);
   check_status ("esx_create_sd: bad status from add_shared_data");

    				/* create mapping record */
   emp_create_map (id, variable);

   return;
}                                       /* end esx_create_sd */


/*--------------------------------------------------------------------------*\
|  Routine: esx_modify_sd
| 
|  Description:
|     modifies a shared data element for a specified variable
\*--------------------------------------------------------------------------*/
void esx_modify_sd (transaction, variable)
transaction_type transaction;
eva_variable variable;
{  variable_sdd variable_sd;
   id_type variable_id;

   set_status (ok);

				/* Get the var id */
   variable_id = emp_get_id (variable);
   if (variable_id == iid_null_id)
      {set_status (invalid_id);
      return;
      }

				/* Create shared data struct for specified
				   variable */
   variable_sd = esx_make_sd_struct (variable);
   check_status ("esx_modify_sd: bad status from esx_make_sd_struct");
 
				/* add to transaction */
   put_shared_data (transaction, variable_id, "variable_sdd", NULL,
		    &variable_sd);
   check_status ("esx_modify_sd:  bad status from add_shared_data.");

   return;
}                                       /* end esx_modify_sd */


/*--------------------------------------------------------------------------*\
|  Routine: esx_remove_from_sd
|
|  Description:
|     Removes an var from the shared database.
\*--------------------------------------------------------------------------*/
void esx_remove_from_sd (transaction, variable)
transaction_type transaction;
eva_variable variable;
{  id_type variable_id;

   set_status (ok);
				/* Remove the variable from the shared data
				   base if it's in there.  */
   if ((variable_id = emp_get_id (variable)) != null_id)
      {remove_shared_data (transaction, "variable_sdd", variable_id);
      check_status ("esx_remove_from_sd: bad status from remove_shared_data.");
      emp_destroy_map_by_id (variable_id);
      }

   return;
}                                       /* end esx_remove_from_sd */


/*--------------------------------------------------------------------------*\
|  Routine: esx_modify_variable
| 
|  Description:  
|     Modifies the various parts of an existing variable as specified by a
|     shared data record and then destroys the shared data record.
|
|     Note:  This routine can not be used to reparent a variable.
|
\*--------------------------------------------------------------------------*/
void esx_modify_variable (variable, variable_sd)
eva_variable variable;
variable_sdd variable_sd;
{

   set_status (ok);

				/* Copy data from the variable_sdd to the
				   variable */
   eva_set_name (variable, variable_sd.name);
   eva_set_comment (variable, variable_sd.comment);
				/* parent is not set, it cannot be changed
				   by the dialogue editor person directly */
   eva_set_snippet (variable, variable_sd.value);

   return ;
}                                       /* end esx_modify_variable */


