static char rcsid [] =
	"$Header: /u/04c/mnt/integ/source/de/src/RCS/eta.c,v 1.11 90/12/21 14:31:39 ejh Exp $";

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Available Technologies Abstraction (eta)
|
|  Description:  
|     This abstraction provides operations to determine what technologies
|     (i.e., toolkits) are available to the dialogue writer.
|
|  Component of:
|     Dialogue Editor Layer.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|

$Log:	eta.c,v $
 * Revision 1.11  90/12/21  14:31:39  ejh
 * strings.h => string.h
 * 
 * Revision 1.10  90/12/05  19:40:57  little
 * add eta_get_mailbox
 * 
 * Revision 1.9  90/11/29  10:03:08  rcs
 * fixed symbol table problem
 * 
 * Revision 1.8  90/10/23  14:11:18  little
 * cahnge things to use real tx files and glue instead of the hokey tx files
 * 
 * Revision 1.7  90/08/29  21:57:24  rcs
 * *** empty log message ***
 * 
 * Revision 1.6  90/08/07  16:18:06  little
 * no changes
 * 
 * Revision 1.5  90/05/21  13:38:24  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.4  90/02/01  19:59:13  bmc
 * no changes.  (oops)
 * 
 * Revision 1.3  89/12/20  14:45:17  rcs
 * integration testing
 * 
 * Revision 1.2  89/12/12  07:55:29  rcs
 * fixed bugs found during integration testing
 * 
 * Revision 1.1  89/11/29  09:40:09  bmc
 * Initial revision
 * 

|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*---------------------------- Includes ------------------------------------*/

#include <stdio.h>		/* C standard I/O declarations 		    */
#include <string.h>		/* C string functions 			    */
#include <varargs.h>		/* C variable argument package		    */
#include <values.h>		/* Unix portability constants		    */

#define eta_main
#include "eta.h"                /* available technologies abstraction	    */
#include "ubo.h"		/* Buffer definitions and operations	    */
#include "u22.h"		/* General purpose Serpent routines	    */
#include "ufa.h"		/* Utility File Abstraction		    */
#include "isc.h"		/* Interface Status Code abstraction	    */
#include "glue_intf.h"

#ifndef memoryPack
#define memoryPack
#endif
#include "memoryPack.h"		/* Zimmermann memory allocation functions   */
#include "listPack.h"		/* Zimmermann linked list functions	    */
#include "hashPack.h"		/* Zimmermann hash table functions	    */


/*---------------------------- Constants -----------------------------------*/

#undef DEBUG1
#undef DEBUG2
#undef DEBUG3
#define DEBUG1(statement) SERPENT_DEBUG1(eta_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(eta_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(eta_debug, statement)

#define FILE_LINE_LENGTH	(255)		/* arbitrary */
#define COMMENT_CHAR		'#'

/*
** The following constants are defined as static variables so that they
** are only stored in one place.  (They are referenced many times.)  Their
** values should not be changed under any circumstances!
*/

static char *METHOD_KEYWORD 	= "METHOD";
static char *ATTRIBUTE_KEYWORD	= "ATTRIBUTE";
static char *OBJECT_KEYWORD	= "OBJECT";
static char *TOKEN_DELIMS	= "\t ";

#define TECH_HASH_TABLE_SIZE	(11)		/* prime number */
#define OBJECT_HASH_TABLE_SIZE	(29)		/* prime number */

#define NO_HASH			((HASH) NULL)

/*-------------------------- Local Types -----------------------------------*/


typedef struct
   {eta_technology_record public;
   boolean objects_got;
   string aux_file_name;
   TECH *glue_tech;
   } tech_entry;

#define NULL_TECH               ((tech_entry *) NULL)

typedef struct
   {char *file_name;			/* for diagnostics */
   unsigned line_number;		/* for diagnostics */
   FILE *stream;
   } eta_file;

#define NO_FILE			((eta_file *) NULL)

/*-------------------------- Global Data -----------------------------------*/

static HASH technology_table;
boolean technologies_got = false;

/*----------------------------- Macros -------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Macro: eta_make_technology
| 
|  Description:
|     Allocates storage for a technology
\*                                                                          */
#define eta_make_technology() \
    (tech_entry *) make_node (sizeof (tech_entry))


/*--------------------------------------------------------------------------*\
|  Macro: eta_free
| 
|  Description:
|     Frees an object and zeroes the pointer.  Type is the pointer type, not
|     the base type.  (e.g., eta_free (fred, char *) frees a pointer to
|     character.)
\*                                                                          */
#define eta_free(thing,type) \
    {free_node ((char *) thing); \
    thing = (type) NULL; \
    }


/*--------------------------------------------------------------------------*\
|  Macro: eta_find_tech
| 
|  Description:
|     Simplifies looking up a technology in its hash table.
\*                                                                          */
#define eta_find_tech(tech_name) \
    ((tech_entry *) get_from_hashtable (technology_table, tech_name))


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_calc_a_hash
| 
|  Description:
|     Calculates the hash index for a string.
\*--------------------------------------------------------------------------*/
static int eta_calc_a_hash (the_string, hash_size)
string the_string;
int hash_size;
{
    register int hash_index = 0;
    register char *s;

    s = the_string;
    while (*s != '\0') 
	hash_index += (hash_index << 1) + *s++;

    return ( (hash_index & (~HIBITI)) % hash_size );
}					/* end eta_calc_a_hash */


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_calc_tech_hash
| 
|  Description:
|     Calculates the hash index for the technology hash table.  It's a
|     separate routine because that's what the Zimmermann hash routines
|     require.
\*--------------------------------------------------------------------------*/
static int eta_calc_tech_hash (name)
char *name;
{
    return (eta_calc_a_hash (name, TECH_HASH_TABLE_SIZE));
}					/* end eta_calc_tech_hash */


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_match_tech
| 
|  Description:
|     Determines whether a technology name matches an existing technology.
|     Used by the hash table functions.
\*--------------------------------------------------------------------------*/
static boolean eta_match_tech (tech_short_name, entry)
string tech_short_name;
tech_entry *entry;
{
    return (strcmp (tech_short_name, entry->public.short_name ) == 0 );
}					/* end eta_match_tech */


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_free_technology
| 
|  Description:  
|     Frees the storage associated with a technology buffer.
\*--------------------------------------------------------------------------*/
static void eta_free_technology (dummy, tech)
caddr_t dummy;
tech_entry *tech;
{   
    set_status (ok);

    eta_free (tech->public.short_name, char *);
    eta_free (tech->public.ill_file_name, char *);
    eta_free (tech->public.long_name, char *);
    eta_free (tech->aux_file_name, char *);
			/* if there were a glue cleanup it would be done
			   here with tech-> glue_tech */
    eta_free (tech, tech_entry *);

    return;
}					/* end eta_free_technology */


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_read_line (ef)
|
|  Description:
|     Reads a line from the technologies file.
|
|  Returns:
|     A pointer to a (static) buffer containing the input line, or NULL on
|     error or EOF.  Any trailing newline is excised from the buffer before
|     returning.
\*--------------------------------------------------------------------------*/
static char *eta_read_line (ef)
eta_file *ef;
{
    static char line_buffer [FILE_LINE_LENGTH];
    char *s;
    char *result = line_buffer;


    if (fgets (line_buffer, FILE_LINE_LENGTH, ef->stream) == NULL)
        /* EOF */
        result = NULL;

    else
        {
        ef->line_number++;
        s = & (line_buffer [strlen (line_buffer) - 1]);
        if ( *s == '\n' )
            *s = '\0';
        }

    return (result);
}					/* end eta_read_line */


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_read_valid_line
|
|  Description:
|     Reads the next valid line from a technology or auxiliary file,
|     skipping blank lines and comments.  (Note that this assumes that
|     comment lines are identified the same way in both files.)
|
|  Returns:
|     NULL on end of file; otherwise returns a pointer to an allocated
|     buffer containing the line.  The buffer is allocated via make_node().
|     Freeing it is the caller's responsibility.
\*--------------------------------------------------------------------------*/
static char *eta_read_valid_line (ef)
eta_file *ef;
{
    char *line;
    char *s;
    boolean comment_line = true;


    while (comment_line && ((line = eta_read_line (ef)) != NULL) )
        {
        /* Find first non-white character in the line.  If it's the
        ** comment character (or if there are no non-white characters)
        ** ignore the line.
        */
        s = line + strspn (line, TOKEN_DELIMS);
        if ( (*s !=  '\0') && (*s !=  COMMENT_CHAR) )
            comment_line = false;

        } /* end while */

    return (line);
}					/* end eta_read_valid_line */


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_open_file
| 
|  Description:
|     Locates and opens a the techology list file.  Returns NULL pointer
|     on error.  (Does not set status.)
\*--------------------------------------------------------------------------*/
static eta_file *eta_open_file (file_name)
string file_name;
{  char *data_path_setting;
   char *full_file_name;
   eta_file *ef = NO_FILE;
   FILE *f;

   extern char *getenv ();

/* ???? is data_path_setting used */

   if ((data_path_setting = getenv (DATA_PATH_ENV_VAR)) == NULL )
	data_path_setting = ".";

   if ((full_file_name = ufa_full_file_name (getenv (DATA_PATH_ENV_VAR),
					       file_name)) == NULL )
      fprintf (stderr, "eta_open_file: can't find file \"%s\"\n", file_name);

   else if ( (f = fopen (full_file_name, "r")) == (FILE *) NULL )
      fprintf (stderr, "eta_open_file: can't open \"%s\"\n", file_name);

   else if ((ef = (eta_file *) make_node (sizeof (eta_file))) == NO_FILE)
      {fclose (f);
      fprintf (stderr,
	       "eta_open_file: can't allocate storage for file descriptor\n");
      }

   else 
      {ef->stream      = f;
      ef->file_name   = full_file_name;
      ef->line_number = 0;
      }

   return (ef);
}					/* end eta_open_file */


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_close_file
| 
|  Description:
|     Closes a file opened with eta_open_file, releasing all associated
|     storage.
\*--------------------------------------------------------------------------*/
static void eta_close_file (ef)
eta_file *ef;
{
   if (ef != NO_FILE)
      {if (ef->stream != (FILE *) NULL)
         fclose (ef->stream);

      if (ef->file_name != NULL)
         eta_free (ef->file_name, char *);
      ef->stream = (FILE *) NULL;
      ef->line_number = 0;
      }
}					/* end eta_close_file */


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_file_error (ef, format, args, ...)
| 
|  Description:
|     Prints out an error message for an open eta_file.  Mainly, this routine
|     exists to provide consistency between error messages.
|
|  Parameters:
|     ef     - Pointer to eta_file structure describing open file.
|     format - Printf format string to use to format error message
|     args   - Arguments necessary to satisfy format string.
\*--------------------------------------------------------------------------*/
static void eta_file_error (va_alist)
va_dcl
{  va_list ap;
   eta_file *ef;
   char *format;


   va_start (ap);

   ef = (eta_file *) va_arg (ap, eta_file *);
   format = (char *) va_arg (ap, char *);

   fprintf (stderr, "File %s, line %d: ", ef->file_name, ef->line_number);
   vfprintf (stderr, format, ap);

   va_end (ap);
}					/* end eta_file_error */


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_parse_technology
| 
|  Description:
|     Parses a technology line, filling the technology buffer accordingly.
\*--------------------------------------------------------------------------*/
static boolean eta_parse_technology (ef, line, tech)
eta_file *ef; /* for diagnostics */
char *line;
tech_entry *tech;
{  boolean success = true;
   char *long_name;
   int chars_to_skip;


   tech->public.short_name = u22_get_token (line,
					     TOKEN_DELIMS,
					     &chars_to_skip);
   line += chars_to_skip;
   tech->public.ill_file_name = u22_get_token (line,
						TOKEN_DELIMS,
						&chars_to_skip);
   line += chars_to_skip;
   tech->aux_file_name = u22_get_token (line,
					 TOKEN_DELIMS,
					 &chars_to_skip);
   line += chars_to_skip;

   if (tech->public.short_name == NULL )
      {success = false;
      eta_file_error (ef, "missing short name for technology\n");
      }
    
   else if (tech->public.ill_file_name == NULL)
      {success = false;
      eta_file_error (ef, "missing technology ILL file name\n");
      }
    
   else if (tech->aux_file_name == NULL)
      {success = false;
      eta_file_error (ef, "missing technology auxiliary file name\n");
      }
    
   else 
			/* The long name consists of the rest of the line.
			   Just skip the leading white space.  */
      {long_name = line + strspn (line, TOKEN_DELIMS);
      if ( *long_name == '\0' )
         {success = false;
         eta_file_error (ef, "missing technology descriptive name\n");
         }

      else
			/* We need to save allocated strings in the database.
			   u22_get_token returns allocated strings, so the only
			   string we need to copy here is long_name, the one
			   u22_get_token didn't parse */
         {if ( (tech->public.long_name = u22_copy_string (long_name)) == NULL)
            {success = false;
            eta_file_error (ef, "can't allocate memory to save long name!\n");
            }

         } /* end else */
      } /* end else */

   return (success);
}					/* end eta_parse_technology */


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_read_a_technology
| 
|  Description:
|     Reads a technology from an open stream into the internal database.
\*--------------------------------------------------------------------------*/
static boolean eta_read_a_technology (ef)
eta_file *ef;
{  char *line;
   tech_entry *tech;
   boolean success = true;

    /*
    ** lines in the file are of the form:
    **
    **     short_name ILL_file_name auxiliary_file_name description ...
    */

   if ((line = eta_read_valid_line (ef)) == NULL )
			/* Error already set */
      success = false;

   else if ( (tech = eta_make_technology ()) == NULL_TECH )
      {success = false;
      set_status (out_of_memory);
      eta_file_error (ef, "can't allocate memory to store technology\n");
      }

   else if (! eta_parse_technology (ef, line, tech) )
			/* Error already set */
      success = false;

/* ????  check, ensure this works for new glue stuff */
   else if (!(tech-> glue_tech = glue_get_technology(tech->aux_file_name)))
      {fprintf (stderr,
 	   "eta_read_a_technology: did not file %s\n", tech->aux_file_name);
      success = false;
      }

   else
			/* Everything worked fine.  Add the sucker to the hash
			   table. */
      {tech->objects_got = false;
      add_to_hashtable (technology_table, tech, tech->public.short_name);
      }

   return (success);
}					/* end eta_read_a_technology */


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_read_technology_file
| 
|  Description:
|     Reads the available technologies file into the internal database
\*--------------------------------------------------------------------------*/
static boolean eta_read_technology_file (file_name)
char *file_name;
{  boolean success = true;
   eta_file *ef;


   if ((ef = eta_open_file (file_name)) == NO_FILE )
	/* Error already set */
      success = false;
   else
      {while (eta_read_a_technology (ef))
         1;
      if (get_status () != ok)
			/* error already set */
         success = false;
      eta_close_file (ef);
      }
   return (success);
}					/* end eta_read_technology_file */


/*--------------------------------------------------------------------------*\
|  Local Routine: eta_save_tech_for_export
| 
|  Description:  
|     Saves the public portion of a technology in a list.
\*--------------------------------------------------------------------------*/
static void eta_save_tech_for_export (list, tech)
LIST list;
tech_entry *tech;
{
			/* Be sure to use the '&' operator, since the public
			   field is a structure */
   add_to_tail (list, (NODE) & (tech->public));
}					/* end eta_save_tech_for_export */


/*--------------------------------------------------------------------------*\
|  Routine: eta_init
| 
|  Description:  
|     Initializes the available technologies database
\*--------------------------------------------------------------------------*/
boolean eta_init (file_name)
string file_name;
{  boolean success = true;

   set_status (ok);

			/* First create the technologies hash table. */

   technology_table = make_hashtable (TECH_HASH_TABLE_SIZE,
				      eta_calc_tech_hash,
				      eta_match_tech);

   if (technology_table == NO_HASH)
      {success = false;
      set_status (out_of_memory);
      fprintf (stderr, "eta_init: can't create internal hash table.\n");
      }

   else if (! eta_read_technology_file (file_name))
			/* Error already set */
      success = false;

   return (success);
}					/* end eta_init */


/*--------------------------------------------------------------------------*\
|  Routine: eta_cleanup
| 
|  Description:  
|     Cleans up the available technologies database, releasing any allocated
|     storage
\*--------------------------------------------------------------------------*/

void eta_cleanup ()
{

    set_status (ok);
    for_all_hashtable (technology_table, NULL, eta_free_technology);
    free_hashtable (technology_table, NOT_NODES);
}					/* end eta_cleanup */


/*--------------------------------------------------------------------------*\
|  Routine: eta_get_technologies
| 
|  Description:
|     Returns a list of all available technologies
\*--------------------------------------------------------------------------*/
LIST eta_get_technologies (get_once)
boolean get_once;
{  LIST list;



   set_status (ok);
			/* If get once and technologies already gotten return
			   NULL.  */
   if (get_once && technologies_got)
      return (NULLlist);

   if ((list = make_list ()) == NULLlist)
      {set_status (out_of_memory);
      fprintf(stderr, "eta_get_technologies: can't create list\n");
      }

   else
      {technologies_got = true;
      for_all_hashtable (technology_table, (NODE) list,
			 eta_save_tech_for_export);
      }

   return (list);
}					/* end eta_get_technologies */


/*--------------------------------------------------------------------------*\
|  Routine: eta_get_objects
| 
|  Description:
|     Returns a list of all objects for a given technology.
\*--------------------------------------------------------------------------*/
ALIST eta_get_objects (short_name, get_once)
string short_name;
boolean get_once;
{  tech_entry *tech;
   ALIST glue_wid_alist = (ALIST) NULLlist;

   set_status (ok);

   if ((tech = eta_find_tech (short_name)) == NULL_TECH )
      {set_status (not_found);
      fprintf (stderr,
               "eta_get_objects: unknown technology -- %s\n", short_name);
      }

   else if (get_once && tech->objects_got)
			/* we fall through to the return */
      1;

   else
      {tech->objects_got = true;
      if (!(glue_wid_alist = tech->glue_tech->widgets))
         {set_status (not_found);
         fprintf (stderr,
               "eta_get_objects: no objects for tech %s\n", short_name);
         }
      }

   return (glue_wid_alist);
}					/* end eta_get_objects */


/*--------------------------------------------------------------------------*\
|  Routine: eta_is_valid
| 
|  Description:
|     Determines whether a specified technology name represents one of the
|     available technologies in the database.
\*--------------------------------------------------------------------------*/

boolean eta_is_valid (short_name)
string short_name;
{
    return (eta_find_tech (short_name) != NULL_TECH);
}					/* end eta_is_valid */


/*--------------------------------------------------------------------------*\
|  Routine: eta_get_attributes
| 
|  Description:
|     Returns a list of attributes for a specific object type from a
|     technology.
|
|     WARNING: This routine returns a copied list.  When finished with the list
|	       it should be freed to prevent memory leaks.
\*--------------------------------------------------------------------------*/
ALIST eta_get_attributes (tech_short_name, object_name, get_once)
string tech_short_name;
string object_name;
boolean get_once;
{  tech_entry *tech;
   ALIST glue_att_alist, ret_list = (ALIST) NULLlist;
   WIDG *the_widget;
   ATTR *the_att;

   set_status (ok);
   if ((tech = eta_find_tech (tech_short_name)) == NULL_TECH )
      {set_status (not_found);
      fprintf (stderr,
               "eta_get_attributes: unknown technology %s\n",
	       tech_short_name);
      }

   else if (!(the_widget = glue_get_widget (tech-> glue_tech, object_name)))
      {set_status (not_found);
      fprintf (stderr,
               "eta_get_attributes: unknown object %s in tech %s\n",
	       object_name, tech_short_name);
      }

   else if (get_once && the_widget-> attributes_got)
			/* fall through to the return */
      1;

   else if (!the_widget-> attributes)
      {set_status (not_found);
      fprintf (stderr,
       "eta_get_attributes: can't find attributes for object %s in tech %s\n",
            object_name, tech_short_name);
      }

   else
      {the_widget-> attributes_got = 1;
      ret_list = make_alist ();
      loop_through_alist (the_widget-> attributes, the_att, ATTR *)
			/* if att name is NOT method, add it to the list */
         {if (strcmp (the_att-> s_name, "method"))
            add_to_atail (ret_list, the_att-> s_name);
         }
      }

    return (ret_list);

}					/* end eta_get_attributes */


/*--------------------------------------------------------------------------*\
|  Routine: eta_get_methods
| 
|  Description:
|     Returns a list of methods for a specific object type from a
|     technology.
\*--------------------------------------------------------------------------*/
ALIST eta_get_methods (tech_short_name, object_name, get_once)
string tech_short_name;
string object_name;
boolean get_once;
{  tech_entry *tech;
   ALIST glue_meth_alist = (ALIST) NULLlist;
   WIDG *the_widget;

   if ((tech = eta_find_tech (tech_short_name)) == NULL_TECH )
      {set_status (not_found);
      fprintf (stderr,
               "eta_get_methods: unknown technology %s\n",
	       tech_short_name);
      }

   else if (!(the_widget = glue_get_widget (tech-> glue_tech, object_name)))
      {set_status (not_found);
      fprintf (stderr,
               "eta_get_methods: unknown object %s in tech %s\n",
	       object_name, tech_short_name);
      }

   else if (get_once && the_widget-> methods_got)
			/* fall through to the return */
      1;

   else if (!(glue_meth_alist = glue_alist_methods (tech-> glue_tech,
						    object_name)))
      {set_status (not_found);
      fprintf (stderr,
	       "eta_get_methods: can't find methods for object %s in tech %s\n",
	       object_name, tech_short_name);
      }

   else
      the_widget-> methods_got = true;

   return (glue_meth_alist);
}					/* end eta_get_methods */


/*--------------------------------------------------------------------------*\
|  Routine: eta_get_glue_widget
| 
|  Description:
|     Returns the glue widget for a specific object type from a
|     technology.
\*--------------------------------------------------------------------------*/
WIDG *eta_get_glue_widget (tech_short_name, widget_name)
string tech_short_name;
string widget_name;
{  tech_entry *tech;
   WIDG *the_widget = NULL;

   set_status (ok);

   if ((tech = eta_find_tech (tech_short_name)) == NULL_TECH )
      {set_status (not_found);
      fprintf (stderr,
               "eta_get_glue_widget: unknown technology %s\n", tech_short_name);
      }

   else if (!(the_widget = glue_get_widget (tech-> glue_tech, widget_name)))
      {set_status (not_found);
      fprintf (stderr,
               "eta_get_glue_widget: unknown widget %s requested for technology %s\n",
	       widget_name, tech_short_name);
      }

   return (the_widget);
}					/* end eta_get_glue_widget */


/*--------------------------------------------------------------------------*\
|  Routine: eta_get_mailbox
| 
|  Description:
|     Returns the glue widget for a specific object type from a
|     technology.
\*--------------------------------------------------------------------------*/
string eta_get_mailbox (tech_short_name)
string tech_short_name;
{  tech_entry *tech;

   set_status (ok);

   if ((tech = eta_find_tech (tech_short_name)) == NULL_TECH )
      {set_status (not_found);
      fprintf (stderr,
               "eta_get_mailbox: unknown technology %s\n", tech_short_name);
      }

   return (glue_get_mailbox (tech-> glue_tech));
}					/* end eta_get_mailbox */

