static char rcsid [] =
        "$Header: /u/04c/mnt/integ/source/de/src/RCS/eva.c,v 1.10 90/11/29 10:03:29 rcs Exp $";

/*--------------------------------------------------------------------------*\
|   
|  Name:  Dialogue Editor Variable Abstraction (eva)
|
|  Description:  
|     The dialogue editor variable abstraction provides operations on the
|     variable abstract type.
|
|  Component of:
|     Dialogue Editor Layer.
|
$Log:	eva.c,v $
 * Revision 1.10  90/11/29  10:03:29  rcs
 * fixed symbol table problem
 * 
 * Revision 1.9  90/09/11  08:36:22  little
 * add third parameter to esi_buffer_src_write calls
 * 
 * Revision 1.8  90/09/07  18:32:21  little
 * put ; and line feed in the source out
 * 
 * Revision 1.7  90/07/05  15:09:39  little
 *  more structure and visualizer parts integration
 * 
 * Revision 1.6  89/12/18  15:12:07  bmc
 * eva_bin_read now sets comment property
 * 
 * Revision 1.5  89/12/18  14:20:52  little
 * fixed some shit that was wrong
 * 
 * Revision 1.4  89/12/15  20:30:55  bmc
 * Added binary read routine.
 * 
 * Revision 1.3  89/12/15  11:58:14  little
 * add bin_write
 * 
 * Revision 1.2  89/12/08  17:45:43  rcs
 * fixed eva_set_name to use est_set_string_property
 * 
 * Revision 1.1  89/11/29  09:40:41  bmc
 * Initial revision
 * 
|
|-----------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


#define eva_main
#include "eva.h"
#include "evc.h"
#include "esi.h"
#include "est.h"
#include "ebi.h"
#include "ea2.h"
#include "cus.h"
#include "ubo.h"
#include "listPack.h"
#include <stdio.h>


#define DEBUG1(statement) SERPENT_DEBUG1(eva_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(eva_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(eva_debug, statement)

#define write_buffer(stream, buffer, the_selector) \
{  ebi_buffer a_buffer; \
   a_buffer.actual_buffer = buffer; \
   a_buffer.selector = the_selector; \
   if (!ebi_write_buffer (stream, a_buffer)) \
      {set_status (system_operation_failed); \
      return; \
      } \
}

#define write_open_record(stream) \
   if (!ebi_write_open_record (stream)) \
      {set_status (system_operation_failed); \
      return; \
      }

#define write_close_record(stream) \
   if (!ebi_write_close_record (stream)) \
      {set_status (system_operation_failed); \
      return; \
      }

/*--------------------------------------------------------------------------*\
|  Routine: eva_create_variable
|
|  Description:
|     Creates a new variable
\*--------------------------------------------------------------------------*/
eva_variable eva_create_variable ()
{
   eva_variable new_variable;
   buffer buf;

   new_variable = (eva_variable)cus_gensym("EDITOR_VAR",variable_sym,NO_SEED);
   init_buffer (buf);
					/*
					   Okay to use a buffer on the stack
					   since est_set_buffer_property makes
					   a copy.
					*/
   est_set_buffer_property (new_variable, initial_value_snippet_text_prop, buf);
   est_set_buffer_property (new_variable, variable_comments_text_prop, buf);
   est_set_string_property (new_variable, name_text_prop, est_default_name);
   return (new_variable);
}					/* end eva_create_variable */


/*--------------------------------------------------------------------------*\
|  Routine: eva_destroy_variable
|
|  Description:
|     Destroys a specified variable and any sub-components.
\*--------------------------------------------------------------------------*/
void eva_destroy_variable (the_variable)
eva_variable the_variable;
{
   est_clear_buffer_property ((cus_sym) the_variable,
			      initial_value_snippet_text_prop);
   est_clear_buffer_property ((cus_sym) the_variable,
			      variable_comments_text_prop);
   est_clear_string_property ((cus_sym) the_variable, name_text_prop);

   cus_delete_sym (the_variable);
   return;
}					/* end eva_destroy_variable */


/*--------------------------------------------------------------------------*\
|  Routine: eva_duplicate_variable
|
|  Description:
|     Destroys a specified variable and any sub-components.
\*--------------------------------------------------------------------------*/
eva_variable eva_duplicate_variable (the_variable)
eva_variable the_variable;
{  eva_variable new_variable;

   if ( (new_variable = eva_create_variable ()) != EVA_NO_VARIABLE )
      {
      est_copy_buffer_property ((cus_sym) new_variable,
				(cus_sym) the_variable,
				initial_value_snippet_text_prop);
      est_copy_buffer_property ((cus_sym) new_variable,
				(cus_sym) the_variable,
				variable_comments_text_prop);
      est_copy_string_property ((cus_sym) new_variable,
				(cus_sym) the_variable,
				name_text_prop);
      }

   return (new_variable);
}

/*--------------------------------------------------------------------------*\
|  Routine: eva_set_snippet
|
|  Description:
|     squirrels away a snippet for a variable
\*--------------------------------------------------------------------------*/
void eva_set_snippet (the_variable, the_snip)
eva_variable the_variable;
buffer the_snip;
{
   est_set_buffer_property (the_variable, initial_value_snippet_text_prop,
			    the_snip);
   return;
}					/* end eva_set_snippet */


/*--------------------------------------------------------------------------*\
|  Routine: eva_get_snippet
|
|  Description:
|      fetches the squirreled away snippet for a variable
\*--------------------------------------------------------------------------*/
buffer eva_get_snippet (the_variable)
eva_variable the_variable;
{  buffer the_snippet;

   the_snippet = est_get_buffer_property (the_variable,
					  initial_value_snippet_text_prop);
   return (the_snippet);
}					/* end eva_get_snippet */


/*--------------------------------------------------------------------------*\
|  Routine: eva_set_comment
|
|  Description:
|     squirrels away a comment for a variable
\*--------------------------------------------------------------------------*/
void eva_set_comment (the_variable, the_comment)
eva_variable the_variable;
buffer the_comment;
{
   est_set_buffer_property (the_variable, variable_comments_text_prop,
			    the_comment);
   return;
}					/* end eva_set_comment */


/*--------------------------------------------------------------------------*\
|  Routine: eva_get_comment
|
|  Description:
|      fetches the squirreled away comment for a variable
\*--------------------------------------------------------------------------*/
buffer eva_get_comment (the_variable)
eva_variable the_variable;
{  buffer the_comment;

   the_comment = est_get_buffer_property (the_variable,
					  variable_comments_text_prop);
   return (the_comment);
}					/* end eva_get_comment */


/*--------------------------------------------------------------------------*\
|  Routine: eva_set_parent
|
|  Description:
|     squirrels away a parent for a variable
\*--------------------------------------------------------------------------*/
void eva_set_parent (the_variable, the_parent)
eva_variable the_variable;
evc_vc the_parent;
{
   cus_put_prop (the_variable, parent_vc_prop, the_parent);
   return;
}					/* end eva_set_parent */


/*--------------------------------------------------------------------------*\
|  Routine: eva_get_parent
|
|  Description:
|      fetches the squirreled away parent for a variable
\*--------------------------------------------------------------------------*/
evc_vc eva_get_parent (the_variable)
eva_variable the_variable;
{  evc_vc the_parent;

   the_parent = cus_get_prop (the_variable, parent_vc_prop);
   return (the_parent);
}					/* end eva_get_parent */


/*--------------------------------------------------------------------------*\
|  Routine: eva_set_name
|
|  Description:
|     squirrels away a name for a variable
\*--------------------------------------------------------------------------*/
void eva_set_name (the_variable, the_name)
eva_variable the_variable;
string the_name;
{
   est_set_string_property (the_variable, name_text_prop, the_name);
   return;
}					/* end eva_set_name */


/*--------------------------------------------------------------------------*\
|  Routine: eva_get_name
|
|  Description:
|      fetches the squirreled away name for a variable
\*--------------------------------------------------------------------------*/
string eva_get_name (the_variable)
eva_variable the_variable;
{  string the_name;

   the_name = cus_get_prop (the_variable, name_text_prop);
   return (the_name);
}					/* end eva_get_name */


/*--------------------------------------------------------------------------*\
|  Routine: eva_parse_variable_2_ed
|
|  Description:
|     convert the parser version of the variable to editor version
\*--------------------------------------------------------------------------*/
eva_variable eva_parse_variable_2_ed (variable)
cus_sym variable;
{  eva_variable ed_var;
   LIST snippet, comment;
   buffer ed_snippet, ed_comment;

   ed_var = eva_create_variable ();

   eva_set_name (ed_var, cus_sym_representation (variable));

   if (snippet = (LIST) cus_get_prop (variable, initial_value_snippet_prop))
      {ed_snippet = esi_snip_2_text (snippet);
      eva_set_snippet (ed_var, ed_snippet);
      }

   if (comment = (LIST) cus_get_prop (variable, variable_comments_prop))
      {ed_comment = esi_comments_2_text (comment);
      eva_set_comment (ed_var, ed_comment);
      }

   return (ed_var);
}					/* end eva_parse_variable_2_ed */


/*--------------------------------------------------------------------------*\
|  Routine: eva_src_write
|
|  Description:
|     write a variable to a source stream
\*--------------------------------------------------------------------------*/
void eva_src_write (the_variable, stream)
eva_variable the_variable;
FILE *stream;
{  buffer the_snippet;

   fprintf (stream, "%s", eva_get_name (the_variable));

   the_snippet = eva_get_snippet (the_variable);
   if (ea2_printable_buffer (the_snippet))
      {fprintf (stream, ": ");
      esi_buffer_src_write (the_snippet, stream, TRUE);
      }

#if 0
/* ?? variable comments only exist in the legal slang source for variables
      which are defined via expressions as opposed to complete snippets.  well,
      the parser converts the expression version to snippet version before we
      get it.  soooooo there is nowhere to legally put out variable comments
*/

   the_comment = eva_get_comment (the_variable);
   if (ea2_printable_buffer (the_snippet))
      esi_buffer_src_write (the_comment, stream, FALSE);
#endif

   return;
}					/* end eva_src_write */


/*--------------------------------------------------------------------------*\
|  Routine: eva_bin_write
|
|  Description:
|     write a variable to a binary stream
\*--------------------------------------------------------------------------*/
void eva_bin_write (the_variable, stream)
eva_variable the_variable;
FILE *stream;
{

   write_open_record (stream);
   ebi_write_token_char (stream, EBI_VARIABLE_SELECTOR);

   ebi_write_token (stream, eva_get_name (the_variable));

   ebi_write_token (stream, cus_sym_representation (the_variable));

   write_buffer (stream, eva_get_comment (the_variable), EBI_COMMENTS_SELECTOR);

   write_buffer (stream, eva_get_snippet (the_variable), EBI_SNIPPET_SELECTOR);

   write_close_record (stream);

   return;
}					/* end eva_bin_write */


/*--------------------------------------------------------------------------*\
|  Routine: eva_bin_read
|
|  Description:
|     read a variable from a binary stream
\*--------------------------------------------------------------------------*/
eva_variable eva_bin_read (stream)
FILE *stream;
{
    eva_variable var = EVA_NO_VARIABLE;
    char tc;
    string unique_name = NULL;
    string name = NULL;
    ebi_buffer snippet;
    ebi_buffer comment;


    /*
    ** A binary variable has the following form (line breaks are insignificant):
    **
    ** EBI_OPEN_RECORD
    **   EBI_VARIABLE_SELECTOR variable_name unique_name variable_snippet
    ** EBI_CLOSE_RECORD
    */
    set_status (ok);
    snippet.selector = EBI_INVALID_SELECTOR;
    comment.selector = EBI_INVALID_SELECTOR;

    if ((tc = ebi_read_token_char (stream)) == EBI_CLOSE_RECORD)
	{
	/* End of list.  Stop here.  Push the end of list character back
	** onto the input token stream.
	*/
	var = EVA_NO_MORE_VARIABLES;
	if (! ebi_push_token_char (EBI_CLOSE_RECORD))
	    {
	    /* Assume status is already set. */
	    var = EVA_NO_VARIABLE; /* to indicate error */
	    DEBUG1 (fprintf (stderr,
		"eva_bin_read: can't push `%c' back onto token stream\n",
		EBI_CLOSE_RECORD););
	    }
	}

    else if ( (var = eva_create_variable ()) == EVA_NO_VARIABLE )
	{
	set_status (out_of_memory);
	DEBUG1 (fprintf (stderr, "eva_bin_read: can't create variable\n"););
	}

    else if (tc != EBI_OPEN_RECORD)
	{
	set_status (invalid_variable);
	DEBUG1 (fprintf (stderr,
	    "eva_bin_read: missing open record `%c' character\n",
	    EBI_OPEN_RECORD););
	}

    else if ( (tc = ebi_read_token_char (stream)) != EBI_VARIABLE_SELECTOR )
	{
	set_status (invalid_variable);
	DEBUG1 (fprintf (stderr,
	    "eva_bin_read: bad selector character -- `%c'\n", tc););
	}

    else if ( (name = ebi_read_token (stream)) == NULL )
	{
	set_status (invalid_variable);
	DEBUG1 (fprintf (stderr, "eva_bin_read: missing variable name\n"););
	}

    else if ( (unique_name = ebi_read_token (stream)) == NULL )
	{
	set_status (invalid_variable);
	DEBUG1 (fprintf (stderr, "eva_bin_read: missing unique name\n"););
	}

    /*
    ** The commas in the following two expressions are INTENTIONAL, and
    ** represent perfectly legal C.  See K&R (edition 1), p. 192 for details.
    */

    else if (comment = ebi_read_buffer (stream),
             comment.selector != EBI_COMMENTS_SELECTOR)
	{
	set_status (invalid_variable);
	DEBUG1 (fprintf (stderr,
	    "eva_bin_read: Expected `%c' buffer, found `%c' buffer.\n",
	    EBI_COMMENTS_SELECTOR, snippet.selector););
	}

    else if (snippet = ebi_read_buffer (stream),
             snippet.selector != EBI_SNIPPET_SELECTOR)
	{
	set_status (invalid_variable);
	DEBUG1 (fprintf (stderr,
	    "eva_bin_read: Expected `%c' buffer, found `%c' buffer.\n",
	    EBI_SNIPPET_SELECTOR, snippet.selector););
	}

    else if (ebi_read_token_char (stream) != EBI_CLOSE_RECORD)
	{
	set_status (invalid_variable);
	DEBUG1 (fprintf (stderr,
	    "eva_bin_read: Missing close record `%c' character\n",
	    EBI_CLOSE_RECORD););
	}

    else
	{
	/*
	** We have all the pieces.  Store 'em.  (We're ignoring the
	** unique name for now.)
	*/
	eva_set_name (var, name);
	eva_set_snippet (var, snippet.actual_buffer);
	eva_set_comment (var, comment.actual_buffer);
	}

    if (get_status () != ok)
	{
	/* Problem.  Release storage. */

	if ( (var != EVA_NO_VARIABLE) && (var != EVA_NO_MORE_VARIABLES) )
	    {
	    eva_destroy_variable (var);
	    var = EVA_NO_VARIABLE;
	    }

	ea2_free_node (name);
	ea2_free_node (unique_name);

	if (snippet.selector != EBI_INVALID_SELECTOR)
	    ubo_destroy_buffer (snippet.actual_buffer);
	if (comment.selector != EBI_INVALID_SELECTOR)
	    ubo_destroy_buffer (comment.actual_buffer);
	}

    return (var);
}
