static char rcsid [] =
        "$Header: /usr/users/little/serpent/mips/de/src/RCS/evc_cmd.c,v 1.4 1991/05/07 17:51:15 little Exp $";

/*--------------------------------------------------------------------------*\
|     
|  Name:  View Controller Command Definitions
|
|  Description:  
|     The dialogue editor command definitions contains the definition
|     of the commands supported by the DEA.  This component accesses
|     retained data stored in the dialogue editor command interpreter
|     component.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log: evc_cmd.c,v $
 * Revision 1.4  1991/05/07  17:51:15  little
 * use e22.h for the e22_dialogue external
 *
 * Revision 1.3  1990/09/17  22:20:17  little
 * fix duplication
 *
 * Revision 1.2  90/08/06  16:48:53  little
 * added a parameter to the call of evc_destroy_vc, it needed a transaction.
 * 
 * Revision 1.1  90/07/10  17:15:49  little
 * Initial revision
 * 
|
|
|
log when file was named ecv.c

 * Revision 1.2  90/05/21  13:33:43  little
 * fix up for multiple includes & start mods for visu/struc editor integration
 * 
 * Revision 1.1  89/12/12  09:14:41  rcs
 * Initial revision
 * 
 * Revision 1.2  89/11/30  12:55:10  rcs
 * fixed spelling error
 * 
 * Revision 1.1  89/11/28  18:00:16  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*-------------------- Includes --------------------------------------------*/

#include "evc_cmd.h"			/* command definitions		    */
#include "edg.h"
#include "emp.h"
#include "ehf.h"
#include "esv.h"
#include "ea2.h"
#include "cmd_defs.h"
#include "se.h"
#ifndef memoryPack
#define memoryPack
#endif
#include "listPack.h"
#include "serpent.h"

/*-------------------- Constants -------------------------------------------*/

#undef DEBUG1
#define DEBUG1(statement) SERPENT_DEBUG1(evc_debug, statement)

#define VC_ONLY evc_vc_only
#define SUBTREE evc_entire_hierarchy



/*--------------------------------------------------------------------------*\
|  Routine:  evc_insert_vc_cmd
| 
|  Description:  
|     used to create and insert a VC into an existing VC.
\*--------------------------------------------------------------------------*/
void evc_insert_vc_cmd (command, transaction)
command_sdd *command;
transaction_type transaction;
{ parameter_sdd *parm1;
  evc_vc new_vc, parent_vc;
  emp_map *parent_vc_map; 
  id_type parent_id;

  set_status(ok);

			/* Create a VC.  */
  new_vc = evc_create_vc();
  check_status ("evc_insert_vc_cmd:  bad status from evc_create_vc.");

			/* Get the parameter record and value.  */
  parm1 = (parameter_sdd *) get_from_hashtable (parameter_table, 
    						command->parameters);
  parent_id = *((id_type *) ((parm1->value).body));

			/* Get the mapping element that maps the id of the
			   parent vc to the actual parent vc handle.  */
  parent_vc_map = (emp_map *) get_from_hashtable (id_table, parent_id);

			/* If dialogue element is a dialogue_element then get
			   the vc, otherwise just it is the vc.  */
  if (edg_is_dialogue (parent_vc_map->dialogue_element))
    parent_vc = edg_get_vc(parent_vc_map->dialogue_element);
   else
    parent_vc = parent_vc_map->dialogue_element;

			/* Perform the insertion.  */
  evc_add_vc (parent_vc, new_vc);
  check_status ("evc_insert_vc_cmd:  bad status from evc_add_vc.");

			/* Add copy of vc to shared data */
  esv_add_to_shared_data  (transaction, EVC_VC_ONLY, new_vc);
  check_status ("evc_insert_vc_cmd:  bad status from evc_add_to_sd.");

  return;
} 					/* end evc_insert_vc_cmd */


/*--------------------------------------------------------------------------*\
|  Routine:  evc_copy_vc_cmd
| 
|  Description:  
|     copies a vc to a parent vc.
\*--------------------------------------------------------------------------*/
void evc_copy_vc_cmd (command, transaction)
command_sdd *command;
transaction_type transaction;
{ parameter_sdd *parm1;
  parameter_sdd *parm2;
  parameter_sdd *parm3;
  id_type source_vc_id, target_vc_id;
  evc_vc source_vc, target_vc, copy_of_vc;
  evc_node_selection how_to_dup;

  set_status (ok);

			/* Get the parameter records and values.  */
  parm1 = (parameter_sdd *) get_from_hashtable (parameter_table, 
    						command->parameters);
  source_vc_id = *((id_type *) ((parm1->value).body));


  parm2 = (parameter_sdd *) get_from_hashtable (parameter_table, 
    						parm1->next);

			/* Set how_to_dup value based on value of parameter.  */   
  if (*((boolean *) ((parm2->value).body)))
    how_to_dup = EVC_ENTIRE_HIERARCHY;
  else
    how_to_dup = EVC_VC_ONLY;

  parm3 = (parameter_sdd *) get_from_hashtable (parameter_table, 
    						parm2->next);
  target_vc_id = *((id_type *) ((parm3->value).body));

			/* Get the actual dialogue elements corresponding to
			  the ids.  */
  source_vc = emp_get_de (source_vc_id);
  target_vc = emp_get_de (target_vc_id);

			/* If the target_vc is a dialogue_element then get
			   the vc.  */
  if (edg_is_dialogue (target_vc))
    target_vc = edg_get_vc(target_vc);

			/* Duplicate the VC.  */
  copy_of_vc = evc_duplicate_vc (source_vc, how_to_dup);
  check_status ("evc_copy_vc_cmd:  bad status from evc_duplicate_vc.");

			/* Add the copy to the specified parent.  */
  evc_add_vc (target_vc, copy_of_vc);
  check_status ("evc_copy_v_cmdc:  bad status from evc_add_vc.");

			/* Add a copy of topmost vc copied to the shared data */
  esv_add_to_shared_data (transaction, EVC_VC_ONLY, copy_of_vc);
  check_status ("evc_copy_vc_cmd:  bad status from evc_add_to_sd.");

  return;  
} 					/* end evc_copy_vc_cmd */


/*--------------------------------------------------------------------------*\
|  Routine:  evc_move_vc_cmd
| 
|  Description:  
|     moves a vc to a parent vc.
\*--------------------------------------------------------------------------*/
void evc_move_vc_cmd (command, transaction)
command_sdd *command;
transaction_type transaction;
{ parameter_sdd *parm1;
  parameter_sdd *parm2;
  parameter_sdd *parm3;
  id_type source_vc_id, target_vc_id;
  evc_vc source_vc, target_vc, copy_of_vc;
  evc_node_selection node_selection;

  set_status (ok);

			/* Get the parameter records and values.  */
  parm1 = (parameter_sdd *) get_from_hashtable (parameter_table, 
    						command->parameters);
  source_vc_id = *((id_type *) ((parm1->value).body));

  parm2 = (parameter_sdd *) get_from_hashtable (parameter_table, 
    						parm1->next);

			/* Set node_selection value based on value o
			    parameter.  */   
  if (*((boolean *) ((parm2->value).body)))
    node_selection = EVC_ENTIRE_HIERARCHY;
   else
    node_selection = EVC_VC_ONLY;

  parm3 = (parameter_sdd *) get_from_hashtable (parameter_table, 
    						parm2->next);
  target_vc_id = *((id_type *) ((parm3->value).body));

			/* Get the actual dialogue elements corresponding to
			   the ids.  */
  source_vc = emp_get_de (source_vc_id);
  target_vc = emp_get_de (target_vc_id);

			/* If the target_vc is a dialogue_element then get
			   the vc.  */
  if (edg_is_dialogue (target_vc))
    target_vc = edg_get_vc(target_vc);

			/* Remove the vc from the dialogue structure.  */
  copy_of_vc = evc_remove_vc (source_vc, node_selection);
  check_status ("evc_move_vc_cmd:  bad status from evc_remove_vc.");

			/* Move the copy to the specified parent.  */
  evc_add_vc (target_vc, copy_of_vc);
  check_status ("evc_move_vc_cmd:  bad status from evc_add_vc.");

			/* Put a new copy of the topmost vc into the database
			   since this is the only thing which actually
			   changed.  */
  esv_put_into_shared_data (transaction, EVC_VC_ONLY, copy_of_vc);
  check_status ("evc_move_vc_cmd:  bad status from esv_put_into_shared_data.");

  return;
} 					/* end evc_move_vc_cmd */


/*--------------------------------------------------------------------------*\
|  Routine:  evc_delete_vc_cmd
| 
|  Description:  
|     deletes a vc to a parent vc.
\*--------------------------------------------------------------------------*/
void evc_delete_vc_cmd (command, transaction)
command_sdd *command;
transaction_type transaction;
{ parameter_sdd *parm1;
  parameter_sdd *parm2;
  id_type source_vc_id;
  evc_vc source_vc;
  evc_node_selection node_selection;

  set_status (ok);

			/* Get the parameter records and values.  */
  parm1 = (parameter_sdd *) get_from_hashtable (parameter_table, 
    						command->parameters);
  source_vc_id = *((id_type *) ((parm1->value).body));

  parm2 = (parameter_sdd *) get_from_hashtable (parameter_table, 
    						parm1->next);

			/* Set node_selection value based on value
			   of parameter.  */   
  if (*((boolean *) ((parm2->value).body)))
    node_selection = EVC_ENTIRE_HIERARCHY;
   else
    node_selection = EVC_VC_ONLY;

			/* Get the actual dialogue elements corresponding
			   to the vc id.  */
  source_vc = emp_get_de (source_vc_id);

			/* If the source_vc is a dialogue_element then get
			   the vc.  */
  if (edg_is_dialogue (source_vc))
    source_vc = edg_get_vc(source_vc);

			/* Remove the vc from vc to shared data. */
  esv_remove_from_shared_data  (transaction, node_selection, source_vc);
  check_status ("evc_delete_vc_cmd:  bad status from esv_remove_from_sd.");

			/* Destroy the vc.  */
  evc_destroy_vc (source_vc, node_selection, transaction);
  check_status ("evc_delete_vc_cmd:  bad status from evc_destroy_vc");

  return;
} 					/* end evc_delete_vc_cmd */


/*--------------------------------------------------------------------------*\
|  Routine:  evc_get_subvcs_cmd
| 
|  Description:  
|     Gets the list of subvcs for a given vc.
\*--------------------------------------------------------------------------*/
void evc_get_subvcs_cmd (command, transaction)
command_sdd *command;
transaction_type transaction;
{ parameter_sdd *parm1;
  evc_vc vc, subvc;
  LIST subvc_list;
  id_type vc_id;

  set_status (ok);

			/* Get the vc handle from the parameter record.  */
  parm1 = (parameter_sdd *) get_from_hashtable (parameter_table, 
    						command->parameters);
  vc_id = *((id_type *) ((parm1->value).body));
  vc = emp_get_de(vc_id);

			/* If the "vc" is acutally a dialogue_element then get
			   the vc, otherwise just it is the vc.  */
  if (edg_is_dialogue (vc))
    vc = edg_get_vc(vc);

			/* Add list of subvcs to shared memory.  */
  subvc_list = evc_get_vcs (vc);

  loop_through_list(subvc_list, subvc, evc_vc)
    esv_add_to_shared_data(transaction, EVC_VC_ONLY, subvc);

  return;
} 					/* end evc_get_subvcs_cmd */


/*--------------------------------------------------------------------------*\
|  Routine:  evc_instantiate_guys_cmd
|
|  Description:
|     handle the object templates instantiate commands
\*--------------------------------------------------------------------------*/
void evc_instantiate_guys_cmd (command, out_transaction)
command_sdd *command;
transaction_type out_transaction;
{  parameter_sdd *parm;
   id_type vc_id;
   evc_vc the_vc;
   evc_node_selection node_selection;

   set_status(ok);
			/* Get the vc template in question */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
						 command->parameters);
   check_parm_type (parm, idd_id,
                    "evc_instantiate_guys_cmd:first parameter is not an id\n");
   vc_id = *((id_type *) ((parm->value).body));
   the_vc = emp_get_de (vc_id);

			/* now get the node/subtree indicator */ 
   parm = (parameter_sdd *) get_from_hashtable (parameter_table, parm->next);
   check_parm_type (parm, idd_boolean,
              "evc_instantiate_guys_cmd: second parameter is not a boolean\n");
   if (*((boolean *)((parm->value).body)))
     node_selection = EVC_ENTIRE_HIERARCHY;
   else
     node_selection = EVC_VC_ONLY;

			/* If the vc is the dialogue then get the "real" vc */
   if (edg_is_dialogue (the_vc))
      the_vc = edg_get_vc (the_vc);

   evc_instantiate_guys (the_vc, node_selection, command-> type,
                         out_transaction); 

   return;
}
 					/* end evc_instantiate_guys_cmd */
 
