static char rcsid [] =
"$Header: /u/04c/mnt/integ/source/de/src/RCS/exc_cmd.c,v 1.7 90/07/05 15:11:57 little Exp $";

/*--------------------------------------------------------------------------*\
|     
|  Name:  Dialogue Editor Variable Command Definitions  (exc)
|
|  Description:  
|     definitions of the variable associated commands supported by the DEA.
|
|  Component of:
|     Dialoge Editor Layer.
|
|  Project:
|     Serpent
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Change History:
|     $Log:	exc_cmd.c,v $
 * Revision 1.7  90/07/05  15:11:57  little
 *  more structure and visualizer parts integration
 * 
 * Revision 1.6  89/12/12  16:59:10  bmc
 * Moved check_vc to ea2.h and changed its name to ea2_check_vc.
 * 
 * Revision 1.5  89/12/12  10:58:00  little
 * fix the parameters to remove_sd
 * 
 * Revision 1.4  89/12/11  16:36:44  little
 * - add copy and move
 * 
 * Revision 1.3  89/12/08  13:18:57  little
 * add exc_add_variable 
 * 
 * Revision 1.2  89/12/07  17:45:22  little
 * changed exc_get_variable to exc_get_variables and the associated
 * functionality
 * 
 * Revision 1.1  89/12/07  15:12:34  little
 * Initial revision
 * 
|
|   created 7 dec 1989
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute   (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|   (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


                        /* need DEBUG1 for check_parm_type to work, but do not
                           use any other debug statements, saber is a lot
                           better and definition must be before of include of
                           ea2.h */
#define DEBUG1(statement) SERPENT_DEBUG1(exc_debug, statement)

#define exc_main
#include "exc_cmd.h"
#include "esx_sd.h"
#include "evc.h"
#include "eva.h"
#include "emp.h"
#include "ehf.h"
#include "ea2.h"
#include "serpent.h"
#include "idd.h"
#include "isc.h"
#ifndef memoryPack
#define memoryPack
#endif
#include "hashPack.h"
#include "se.h"


/*--------------------------------------------------------------------------*\
|  Routine:  exc_add_variable
|
|  Description:
|     adds a newly created variable to the specified (by a command parameter
|	from the dialogue) vc and places it into a transaction
\*--------------------------------------------------------------------------*/
void exc_add_variable (command, transaction)
command_sdd *command;
transaction_type transaction;
{  evc_vc the_vc;                       /* parameter 1 */
   parameter_sdd *parm;
   id_type vc_id;
   eva_variable the_var;

                                /* get the first parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
                                                command-> parameters);
   check_parm_type (parm, idd_id,
                    "exc_add_variable: first parameter is not an id\n");
   vc_id = *((id_type *) ((parm-> value).body));
   the_vc = emp_get_de (vc_id);
                                /* if the thing is a dialogue, get the vc,
                                   eve, associated with it */
   ea2_check_vc (the_vc);

   the_var = eva_create_variable ();
   evc_add_variable (the_vc, the_var);
   esx_create_sd (transaction, the_var);

   return;
}					/* end exc_add_variable */


/*--------------------------------------------------------------------------*\
|  Routine:  exc_delete_variable
|
|  Description:
|     deletes the specified (by a command parameter from the dialogue) var and
|	places a remove sd request into the transaction
\*--------------------------------------------------------------------------*/
void exc_delete_variable (command, transaction)
command_sdd *command;
transaction_type transaction;
{  eva_variable the_var;		/* parameter 1 */
   parameter_sdd *parm;
   id_type var_id;
                                /* get the first parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
                                                command-> parameters);
   check_parm_type (parm, idd_id,
                    "exc_delete_variable: first parameter is not an id\n");
   var_id = *((id_type *) ((parm-> value).body));
   the_var = emp_get_de (var_id);

				/* delete the var from it's parent vc */
   evc_remove_variable (the_var);
				/* kill the var itself */
   eva_destroy_variable (the_var);
				/* now place a delete request into the
				   xaction */
   esx_remove_from_sd (transaction, the_var);

   return;
}					/* end exc_delete_variable */


/*--------------------------------------------------------------------------*\
|  Routine:  exc_copy_variable
|
|  Description:
|     generates a copy of the specified variable and adds it to the target vc
\*--------------------------------------------------------------------------*/
void exc_copy_variable (command, transaction)
command_sdd *command;
transaction_type transaction;
{  eva_variable the_var;                /* parameter 1 */
   evc_vc target_vc;                	/* parameter 2 */
   parameter_sdd *parm;
   id_type var_id, target_vc_id;
   eva_variable new_var;

                                /* get the first parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
                                                command-> parameters);
   check_parm_type (parm, idd_id,
                    "exc_copy_variable: first parameter is not an id\n");
   var_id = *((id_type *) ((parm-> value).body));
   the_var = emp_get_de (var_id);

                                /* get the second parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
                                                parm-> next);
   check_parm_type (parm, idd_id,
                    "exc_copy_variable: second parameter is not an id\n");
   target_vc_id = *((id_type *) ((parm-> value).body));
   target_vc = emp_get_de (target_vc_id);
                                /* if the thing is a dialogue, get the vc,
                                   eve, associated with it */
   ea2_check_vc (target_vc);

   new_var = eva_duplicate_variable (the_var);

   evc_add_variable (target_vc, new_var);
   esx_create_sd (transaction, new_var);

   return;
}					/* end exc_copy_variable */


/*--------------------------------------------------------------------------*\
|  Routine:  exc_move_variable
|
|  Description:
|     moves the specified variable to the specified vc
\*--------------------------------------------------------------------------*/
void exc_move_variable (command, transaction)
command_sdd *command;
transaction_type transaction;
{  eva_variable the_var;                /* parameter 1 */
   evc_vc target_vc;                    /* parameter 2 */
   parameter_sdd *parm;
   id_type var_id, target_vc_id;

                                /* get the first parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
                                                command-> parameters);
   check_parm_type (parm, idd_id,
                    "exc_move_variable: first parameter is not an id\n");
   var_id = *((id_type *) ((parm-> value).body));
   the_var = emp_get_de (var_id);

                                /* get the second parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
                                                parm-> next);
   check_parm_type (parm, idd_id,
                    "exc_move_variable: second parameter is not an id\n");
   target_vc_id = *((id_type *) ((parm-> value).body));
   target_vc = emp_get_de (target_vc_id);
                                /* if the thing is a dialogue, get the vc,
                                   eve, associated with it */
   ea2_check_vc (target_vc);

				/* now reparent the var */
   evc_remove_variable (eva_get_parent (the_var), the_var);
   evc_add_variable (target_vc, the_var);

				/* now put into xaction */
   esx_modify_sd (transaction, the_var);

   return;
}					/* end exc_move_variable */


/*--------------------------------------------------------------------------*\
|  Routine:  exc_get_variables
| 
|  Description:  
|     gets the specified (by a command parameter from the dialogue) variable
|     from the abstraction and places it into a transaction
\*--------------------------------------------------------------------------*/
void exc_get_variables (command, transaction)
command_sdd *command;
transaction_type transaction;
{  evc_vc the_vc;			/* parameter 1 */
   LIST vars;
   parameter_sdd *parm;
   id_type vc_id;
   eva_variable var;

   set_status (ok);

				/* get the first parameter */
   parm = (parameter_sdd *) get_from_hashtable (parameter_table,
						command-> parameters);
   check_parm_type (parm, idd_id,
		    "exc_get_variables: first parameter is not an id\n");
   vc_id = *((id_type *) ((parm-> value).body));
   the_vc = emp_get_de (vc_id);
				/* if the thing is a dialogue, get the vc,
				   eve, associated with it */
   ea2_check_vc (the_vc);

				/* get the vc's vars and loop through them */
   vars = evc_get_variables (the_vc);
   loop_through_list (vars, var, eva_variable)
				/* if var is not in shared data, put it in */
      if (emp_get_id (var) == null_id)
         esx_create_sd (transaction, var);

   return;
} 					/* end exc_get_variable */



