/* $Header: /u/04c/mnt/integ/source/de/src/RCS/fontsel.c,v 1.6 90/12/26 14:37:47 ejh Exp $ */
/*--------------------------------------------------------------------------*\
|  
|  Name:  fontsel.c
|
|  Description:  
|     This file contains dialogue callable c functions that are used
|     to support the font selector within the dialogue editor.
|
|  Component of:
|     Dialogue editor dialogue.
|
|  Related files:
|     fontsel.h - contains the definition of the interface.
|
|  Project:
|     Serpent
|     User Interface Project
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Daniel V. Klein
|     dvk@sei.cmu.edu  
|     Robert C. Seacord
|     rcs@sei.cmu.edu  
|
|  Change History:
|     June 20, 1990 : Module created.
|     $Log:	fontsel.c,v $
 * Revision 1.6  90/12/26  14:37:47  ejh
 * fixed a couple of syntax errors on hp
 * 
 * Revision 1.5  90/12/26  14:35:15  ejh
 * fixed re_comp and re_exec for hpux
 * 
 * Revision 1.4  90/12/21  14:46:28  ejh
 * got rid of strings.h
 * 
 * Revision 1.3  90/08/27  20:47:30  rcs
 * removed debug print
 * 
 * Revision 1.2  90/08/22  18:32:50  rcs
 * *** empty log message ***
 * 
 * Revision 1.1  90/06/20  11:12:54  rcs
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*-------------------- Includes --------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include "treePack.h"
#define fs_main
#include "fontsel.h"
#include "serpent.h"

/*-------------------- Constants -------------------------------------------*/

#define DEBUG1(statement) SERPENT_DEBUG1(fs_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(fs_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(fs_debug, statement)

#define MAXNAMES 	500
#define PAT_LEN		200

/*-------------------- Retained Data ---------------------------------------*/

static char **Xfont_list;
static TREE font_tree;
static int Xfont_count;
static char **font_names;
static char **font_sizes;
static char **font_slants;
static char **font_weights;
static int name_len;
static int size_len;
static int slant_len;
static int weight_len;
static TREE name_tree;
static TREE size_tree;
static TREE slant_tree;
static TREE weight_tree;
static Display *display;

static short Initialized = 0;

static char *font_pattern = "-*-%s-%s-%s-*-*-%s-*-*-*-*-*-*-*";
static char *check_pattern = "-%s-%s-%s-%s-";

/*--------------------------------------------------------------------------*\
|  Routine: 
|
|  Description:
|     
\*--------------------------------------------------------------------------*/
static int hash(s)
char *s;
{
    return *s % 26;
}

/*--------------------------------------------------------------------------*\
|  Routine: 
|
|  Description:
|     
\*--------------------------------------------------------------------------*/
static int compare(s1, s2)
char *s1, *s2;
{
    if (*s1 >= '0' && *s1 <= '9' && *s2 >= '0' && *s2 <= '9')
	return atoi(s1) - atoi(s2);
    else
	return strcmp (s1, s2);
}

/*--------------------------------------------------------------------------*\
|  Routine: 
|
|  Description:
|     
\*--------------------------------------------------------------------------*/
static void addit(what, str)
char **what;
char *str;
{
  DEBUG3(
    fprintf(stderr, "string = %s\n", str);
  );

  while (*what) {
    what++;
  }

  *what = str;
}

/*--------------------------------------------------------------------------*\
|  Routine: 
|
|  Description:
|     
\*--------------------------------------------------------------------------*/
static char *mystrtok(str, sepset)
char *str, *sepset;
{
    register char *p, *r;
    static char	*savept;

    /*first or subsequent call*/
    p = (str == NULL)? savept: str;
    if (p == 0)					/* no tokens remaining */
	return(NULL);
    if ((r=strpbrk(p, sepset)) == NULL)		/* move past token */
	savept = 0;				/* last token */
    else {
	*r = '\0';
	savept = ++r;
	}
    return(p);
}

/*--------------------------------------------------------------------------*\
|  Routine: collect_fonts
|
|  Description:
|     
\*--------------------------------------------------------------------------*/
static void collect_fonts()
{
    char all_pattern[PAT_LEN];
    char tmp[PAT_LEN];
    int i;
    char *cp, *xp, *name_p, *size_p, *slant_p, *weight_p;
    
    Initialized++;
    display = XOpenDisplay(NULL);
    (void) sprintf(all_pattern, font_pattern, "*", "*", "*", "*");
    for (i=1; /*null*/; i++) {
	Xfont_list = XListFonts(display, all_pattern, i*MAXNAMES, &Xfont_count);
	if (Xfont_count <= i*MAXNAMES)
	    break;			/* It fit - go on... */
	else			
	    XFreeFontNames (Xfont_list);	/* Free incomplete list, try again */
	}
    font_tree = make_bintree (compare);
    name_tree = make_bintree (compare);
    size_tree = make_bintree (compare);
    slant_tree = make_bintree (compare);
    weight_tree = make_bintree (compare);
/*
**  For each font add unique stuff that we use to binary trees.
*/
    for (i=0; i<Xfont_count; i++) {
	(void) strcpy (tmp, Xfont_list[i]);
	(void) mystrtok(tmp, "-");				/* Dummy */
	(void) mystrtok(NULL, "-");				/* Foundry */
	cp = mystrtok(NULL, "-");				/* Family */
	name_p = strcpy (make_node (strlen(cp)+1), cp);
	if (*cp != '\0')
	    (void) add_unique_to_bintree (name_tree, name_p, name_p);
	cp = mystrtok(NULL, "-");				/* Weight */
	weight_p = strcpy (make_node (strlen(cp)+1), cp);
	if (*cp != '\0')
	    (void) add_unique_to_bintree (weight_tree, weight_p, weight_p);
	cp = mystrtok(NULL, "-");				/* Slant */
	slant_p = strcpy (make_node (strlen(cp)+1), cp);
	if (*cp != '\0')
	    (void) add_unique_to_bintree (slant_tree, slant_p, slant_p);
	(void) mystrtok(NULL, "-");			/* Width */
	(void) mystrtok(NULL, "-");			/* Add Style */

	cp = mystrtok(NULL, "-");			/* Pixel Size */
	size_p = strcpy (make_node (strlen(cp)+1), cp);
	if (*cp != '\0')
	    (void) add_unique_to_bintree (size_tree, size_p, size_p);

	xp = make_node (strlen(name_p)+
			strlen(size_p)+
			strlen(weight_p)+
			strlen(slant_p)+
			5+			/* 5 '-'s in the string */
			1);			/* 1 trailing null */
	(void) sprintf (xp, check_pattern, name_p, weight_p, slant_p, size_p);
	(void) add_unique_to_bintree (font_tree, xp, xp);
	}

/*
**  Create array of font names.
*/
    name_len = size_of_bintree (name_tree)+1;
    font_names = (char **)make_node(name_len*sizeof(cp));
    font_names[0] = "*";
    for (i=1; i<size_of_bintree(name_tree); i++)
	font_names[i] = NULL;
    for_all_bintree (name_tree, font_names, addit);
    name_len = size_of_bintree (name_tree)+1;
    (void) free_bintree (name_tree, NOT_NODES);
/*
**  Create array of font weights.
*/
    weight_len = size_of_bintree (weight_tree)+1;
    font_weights = (char **)make_node(weight_len*sizeof(cp));
    font_weights[0] = "*";
    for (i=1; i<size_of_bintree(weight_tree); i++)
	font_weights[i] = NULL;
    for_all_bintree (weight_tree, font_weights, addit);
    (void) free_bintree (weight_tree, NOT_NODES);
/*
**  Create array of font slants.
*/
    slant_len = size_of_bintree (slant_tree)+1;
    font_slants = (char **)make_node(slant_len*sizeof(cp));
    font_slants[0] = "*";
    for (i=1; i<size_of_bintree(slant_tree); i++)
	font_slants[i] = NULL;
    for_all_bintree (slant_tree, font_slants, addit);
    (void) free_bintree (slant_tree, NOT_NODES);
/*
**  Create array of font sizes.
*/
    size_len = size_of_bintree (size_tree)+1;
    font_sizes = (char **)make_node(size_len*sizeof(cp));
    font_sizes[0] = "*";
    for (i=1; i<size_of_bintree(size_tree); i++)
	font_sizes[i] = NULL;
    for_all_bintree (size_tree, font_sizes, addit);
    (void) free_bintree (size_tree, NOT_NODES);

    DEBUG3(
      fprintf(stdout, "collect_fonts: font_sizes[19] = %s\n", font_sizes[19]);
    );

    XFreeFontNames (Xfont_list);

    Xfont_count = size_of_bintree (font_tree);
    Xfont_list = (char **)make_node(Xfont_count*sizeof(cp));
    for (i=0; i<Xfont_count; i++)
	Xfont_list[i] = NULL;
    for_all_bintree (font_tree, Xfont_list, addit);
    (void) free_bintree (font_tree, NOT_NODES);
}

/*--------------------------------------------------------------------------*\
|  Routine: howmany_of
|
|  Description:
|     
\*--------------------------------------------------------------------------*/
int howmany_of(type)
int type;
{
    if (!Initialized) {
	collect_fonts();
    }

    DEBUG3(
      fprintf(stdout, "howmany_of: font_sizes[19] = %s\n", font_sizes[19]);
    );

    switch (type) {
	case FONT_NAME:
	    return name_len;

	case FONT_SIZE:
          DEBUG3(
            fprintf(
              stdout, "howmany_of: font_sizes[19] = %s\n", font_sizes[19]
            );
          );
	  return size_len;

	case FONT_SLANT:
	    return slant_len;
	case FONT_WEIGHT:
	    return weight_len;
	default:
          DEBUG1(
	    fprintf(stdout, "Auggghhh!! - howmany_of()\n");
          );
          return 0;
	}
}

/*--------------------------------------------------------------------------*\
|  Routine: fetch_val
|
|  Description:
|     
\*--------------------------------------------------------------------------*/
string fetch_val(type,idx)
int type, idx;
{

    DEBUG2(
      fprintf(stdout, "fetch_val: call.\n");
      fprintf(stdout, "fetch_val: type = %d.\n", type);
      fprintf(stdout, "fetch_val: idx = %d.\n", idx);
    );

    if (!Initialized) {
	collect_fonts();
    }

    DEBUG3(
      fprintf(stdout, "fetch_val: font_sizes[19] = %s\n", font_sizes[19]);
    );

    switch (type) {
	case FONT_NAME:
	    return idx < name_len ? font_names[idx] : "";

	case FONT_SIZE:
          DEBUG3(
            fprintf(stdout,"fetch_val: font_sizes[19] = %s\n",font_sizes[19]);
            fprintf(stdout, "fetch_val: size_len = %d\n", size_len);
            fprintf(
              stdout, 
              "fetch_val: return = %s.\n", 
              idx < size_len ? font_sizes[idx] : ""
            );
          );

	  return idx < size_len ? font_sizes[idx] : "";

	case FONT_SLANT:
	    return idx < slant_len ? font_slants[idx] : "";
	case FONT_WEIGHT:
	    return idx < weight_len ? font_weights[idx] : "";
	default:
          DEBUG1(
	    fprintf(stdout, "Auggghhh!! - fetch_val()\n");
          );
          return "";
	}
}

/*--------------------------------------------------------------------------*\
|  Routine: 
|
|  Description:
|     
\*--------------------------------------------------------------------------*/
boolean font_exists(name,size,slant,weight)
string name;
string size;
string slant;
string weight;
{
    char test_pattern[200];
    char *err;
#ifdef hpux
    char *regcmp (), *regex ();
#else
    char *re_comp ();
#endif
    int i;

    if (!Initialized)
	collect_fonts();
    (void) sprintf (test_pattern, check_pattern,
			strcmp (name, "*") ? name : "[^-]*" ,
			strcmp (weight, "*") ? weight : "[^-]*" ,
			strcmp (slant, "*") ? slant : "[^-]*" ,
			strcmp (size, "*") ? size : "[^-]*" );
#ifdef hpux
    err = regcmp (test_pattern, 0);
    for (i=0; i<Xfont_count; i++)
	if (regex (err, Xfont_list[i], test_pattern))
	    return true;
#else
    err = re_comp (test_pattern);
    for (i=0; i<Xfont_count; i++)
	if (re_exec (Xfont_list[i]))
	    return true;
#endif
    return false;
}

/*--------------------------------------------------------------------------*\
|  Routine: 
|
|  Description:
|     
\*--------------------------------------------------------------------------*/
string font_str(name,size,slant,weight)
string name;
string size;
string slant;
string weight;
{
    static char out_pattern[200];

    (void) sprintf (out_pattern, font_pattern, name, weight, slant, size);
    return out_pattern;
}

/*--------------------------------------------------------------------------*\
|  Routine: 
|
|  Description:
|     
\*--------------------------------------------------------------------------*/
int int_max(l,r)
int l,r;
{
    return (l>r ? l : r);
}
