/* $Header: /u/04b/mnt/ljb/RCS/int.sl,v 1.5 1991/04/23 17:57:38 ljb Exp $ */
/*--------------------------------------------------------------------------*\
|  
|  Name:  Integer Xtype Editor (int.sl)
|
|  Description:  
|     This file contains the Slang code to implement the edit 
|     field for intgeer attributes. 
|
|  Component of:
|     Dialogue editor dialogue.
|
|  Related files:
|     gw.slang - generic widget code.
|
|  Project:
|     Serpent
|     User Interface Project
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Robert C. Seacord
|     rcs@sei.cmu.edu  
|
|  Change History:
|     Nov 21, 1990 : Module created.
|     $Log: int.sl,v $
% Revision 1.5  1991/04/23  17:57:38  ljb
% change background to form rather than box
%
% Revision 1.4  1991/04/11  18:06:08  rcs
% *** empty log message ***
%
% Revision 1.3  91/03/21  17:15:11  rcs
% *** empty log message ***
% 
% Revision 1.2  91/03/19  19:59:43  rcs
% delete shared data instance in on delete
% 
% Revision 1.1  90/11/29  18:42:46  rcs
% Initial revision
% 
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*------------------- constants --------------------------------------------*/

#undef DEBUG1
#undef DEBUG2
#undef DEBUG3

#define DEBUG1(statement) SERPENT_DEBUG1(int_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(int_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(int_debug, statement)

/*--------------------------------------------------------------------------*\
|  Name:  gt_int_vc
|
|  Description:
|    The int x type vc supports the editing of int attributes.
\*--------------------------------------------------------------------------*/
VC: gt_int_vc

Creation Condition: (new("int_xtype_sdd"))

Variables:
  int_help_id: true;
  int_attr_error_id;		/* id of error sdd	*/
  attr_name: get_sd_value(int_xtype_sdd.attribute,"name");
  attr_value: get_sd_value(int_xtype_sdd.attribute,"value");

Objects:

  attribute_box(int)

  attribute_label(int)

  int_attr_text: XawTextentry {
    Attributes: 
      parent: int_attr_box; 
      width: ATTR_WIDTH; 
      height: ATTRIBUTE_HEIGHT; 
      borderColor: "white"; 
      horizDistance: int_attr_label.width + 3;
      textBuffer: { 
        if (attr_value = UNDEFINED) then 
          DEBUG3(printf("int_attr_text: attr value = %s.\n",attr_value))
          textBuffer := glue_get_attr_default( 
            widget_handle,attr_name,SERPENT_DEFAULT 
          ); 
        else
          DEBUG3(printf("int_attr_text.textBuffer = %s.\n",attr_value))
          textBuffer := attr_value;
        end if; 
      } 
    Methods: 
/*
**  Send method.
*/
      send: {
        DEBUG2(
          printf("int_attr_text: send method called.\n",""); 
          printf("int_attr_text: name = %s.\n",attr_name);
          printf("int_attr_text: textBuffer = %s.\n",textBuffer); 
        ) 
/*
**  Destroy existing error message, if it exists.
*/
        if (id_exists(int_attr_error_id)) then
          destroy_sd_instance(int_attr_error_id); 
        end if; 
/*
**  If text buffer is Null then an error condition exists.
*/
        if (string_lower(textBuffer) = "null") then

          error_flag := true; 

          int_attr_error_id := create_sd_instance("error_sdd","DM_BOX"); 
          put_sd_value(int_attr_error_id,"title",attr_name); 
          put_sd_value(int_attr_error_id,"message",
            "NULL is an invalid value for this attribute."
          ); 
          put_sd_value(int_attr_error_id,"height",
            string_count_chars(
              "NULL is an invalid value for this attribute.",
              "\n"
            ) + 1
          ); 
          put_sd_value(int_attr_error_id,"self",int_attr_error_id); 

          int_attr_label.background := "black"; 
          int_attr_label.foreground := "white"; 
/*
**  If text buffer is not an integer an error condition exists.
*/
        elsif (NOT string_is_integer(textBuffer)) then

          error_flag := true; 

          int_attr_error_id := create_sd_instance("error_sdd","DM_BOX"); 
          put_sd_value(int_attr_error_id,"title",attr_name); 
          put_sd_value(int_attr_error_id,"message",
            "The specified attribute value is not an integer."
          ); 
          put_sd_value(int_attr_error_id,"height",
            string_count_chars(
              "The specified attribute value is not an integer.",
              "\n"
            ) + 1
          ); 
          put_sd_value(int_attr_error_id,"self",int_attr_error_id); 

          int_attr_label.background := "black"; 
          int_attr_label.foreground := "white"; 
/*
**  Valid integer input.
*/
        else
/*
**  If null string specified, set attribute to undefined.
*/
          if (textBuffer = "") then
/*
**  Update the application value.
*/
            put_sd_value(int_xtype_sdd.attribute,"value",UNDEFINED); 
/*
**  Update the display value.
*/
            if (id_exists(object_id)) then
              put_sd_value(object_id,attr_name,UNDEFINED);
            end if;
/*
**  Actual integer value.
*/
          else
/*
**  Update the application value.
*/
            put_sd_value(int_xtype_sdd.attribute,"value",textBuffer); 
/*
**  Update the display value.
*/
            if (id_exists(object_id)) then
              put_sd_value(object_id,attr_name,textBuffer);
            end if;

  DEBUG3(
    printf("int_attr_text: new defined value.\n",""); 
    printf("int_attr_text: name = %s.\n",attr_value); 
    printf("int_attr_text: textBuffer = %s.\n",textBuffer); 
  ) 

          end if; 
/*
**  Reset any existing error hiliting.
*/
          int_attr_label.background := "white"; 
          int_attr_label.foreground := "black"; 
        end if; 

        DEBUG2(
          printf("int_attr_text: exit.\n",""); 
        ) 
      }
/*
** Focus method.
*/
      focus: {
        if (id_exists(current_focus)) then
          put_sd_value(current_focus,"font","6x13");
        end if;
        int_attr_box.focus := SELF;
        current_focus := SELF;
        font := "*";
      }
  }

  On Destroy: {
    destroy_sd_instance(int_xtype_sdd);
  }

end VC gt_int_vc;
