/* $Header: /u/04b/mnt/ljb/RCS/parent.sl,v 1.12 1991/04/23 18:01:21 ljb Exp $ */
/*--------------------------------------------------------------------------*\
|
|  Name:  Parent Xtype Editor (parent.sl)
|
|  Description:
|     This file contains the Slang code to implement the edit
|     field for intgeer attributes.
|
|  Component of:
|     Dialogue editor dialogue.
|
|  Related files:
|     gw.sl - generic parent code.
|
|  Project:
|     Serpent
|     User Interface Project
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:
|     Robert C. Seacord
|     rcs@sei.cmu.edu
|
|  Change History:
|     Mar 26, 1991 : Module created.
|     $Log: parent.sl,v $
% Revision 1.12  1991/04/23  18:01:21  ljb
% change background to form rather than box
%
% Revision 1.11  1991/04/11  18:06:45  rcs
% c
%
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*------------------- constants --------------------------------------------*/

#undef DEBUG1
#undef DEBUG2
#undef DEBUG3

#define DEBUG1(statement) SERPENT_DEBUG1(parent_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(parent_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(parent_debug, statement)

/*--------------------------------------------------------------------------*\
|  Name:  gt_parent_vc
|
|  Description:
|    The parent x type vc supports the editing of parent attributes.
\*--------------------------------------------------------------------------*/
VC: gt_parent_vc

Creation Condition: (new("parent_xtype_sdd"))

Variables:
  last_parent: "null";  /* the last parent reference, "null" if none        */

  parent_help_id: true;
  parent_attr_error_id;         /* id of error sdd      */
  attr_name: get_sd_value(parent_xtype_sdd.attribute,"name");
  attr_value: get_sd_value(parent_xtype_sdd.attribute,"value");
  temp_value;

Objects:

  attribute_box(parent)

  attribute_label(parent)

  parent_attr_text: XawTextentry {
    Attributes:
      parent: parent_attr_box;
      width: ATTR_WIDTH;
      height: ATTRIBUTE_HEIGHT;
      borderColor: "white";
      horizDistance: parent_attr_label.width + 3;
      textBuffer: {
        if (attr_value = UNDEFINED) then
          textBuffer := glue_get_attr_default(
            widget_handle,attr_name,SERPENT_DEFAULT
        );
        else
          textBuffer := attr_value;
        end if;
      }
    Methods:
      send: {

        DEBUG2(
          printf("parent_attr_text: send method call.\n", "");
          printf("parent_attr_text: name = %s.\n",attr_name);
          printf("parent_attr_text: textBuffer = %s.\n", textBuffer);
          printf(
            "parent_attr_text: parent id = %s.\n", 
            get_id(object_instance_sdd.name)
          );
        )
/*
**  If the object instance name specified in the text buffer is
**  a valid object, or null or undefined update the application
**  and display values.  Otherwise raise an error condition.
*/
        if (
          object_exists(textBuffer) OR
          string_lower(textBuffer) = "undefined" OR
          string_lower(textBuffer) = "null"
        ) then

          DEBUG3(
            printf("parent_attr_text: parent exists.\n", "");
            printf(
              "parent_attr_text: parent id =  %s.\n",
              get_id(object_instance_sdd.name)
            );
          )

          last_parent := attr_value;
/* 
**  If the parent name has changed change update the application and
**  display values and set the flag to cause reparenting to occur. 
*/ 
        if ( 			
          string_lower(textBuffer) <> string_lower(last_parent) 
        ) then 			/* parent name has changed	*/

          DEBUG3(printf("parent_attr_text:  parent name has changed.\n", ""))

/*
**  Update the application value.
*/
          put_sd_value(parent_xtype_sdd.attribute,"value",textBuffer);
/*
**  Update the display value.
*/
          if (id_exists(object_id)) then
            put_sd_value(object_id,attr_name,get_id(textBuffer));
          end if;

          flip_flop_flag := NOT flip_flop_flag; 
        end if;			/* end if parent name has changed	*/
/* 
**  Set the background and foreground colors to normal in case 
**  there was a previous error condition. 
*/ 
        parent_attr_text.background := "white"; 
        parent_attr_text.foreground := "black"; 
/* 
**  If ID for error_sdd exists, destroy the shared data instance. 
*/ 
        if (id_exists(parent_attr_error_id)) then  /* error id exists 	*/
          destroy_sd_instance(parent_attr_error_id); 
        end if; 			/* end if error id exists 	*/
/*
**  If the object is not defined or undefined or null then an 
**  error condition exists.
*/
    else			/* object not defined nor null or undef */ 
/* 
**  Set error flag and create a help form for the attribute. 
*/ 
      error_flag := true; 

      parent_attr_error_id := create_sd_instance("error_sdd", "DM_BOX"); 
      put_sd_value(parent_attr_error_id, "title", "Parent"); 
      put_sd_value( 
        parent_attr_error_id, 
        "message", 
        "The specified parent is not defined." 
      ); 
      put_sd_value( 
        parent_attr_error_id, 
        "height", 
        string_count_chars( 
          "The specified parent is not defined.", "\n" 
        ) + 1 
      ); 
      put_sd_value(parent_attr_error_id, "self", parent_attr_error_id); 
/* 
**  Provide error highlighting. 
*/ 
      parent_attr_text.background := "black"; 
      parent_attr_text.foreground := "white"; 
    end if; 

/* --------  Determine the correct coordinate values. -------------------*/ 

    DEBUG3(
      printf("parent_attr_text: error_flag = %s.\n", error_flag);
      printf("parent_attr_text: last_parent = %s.\n", last_parent);
      printf(
        "parent_attr_text: parent = %s.\n",
        attr_value
      );
    )
/*
**  If no error occured in the previus part and a change occured
**  determine the exact change in parental status and adjust the coordinates.
*/
    if (
      (NOT error_flag) AND
      (string_lower(last_parent) <> string_lower(attr_value)) 
    ) then
/* 
**  New Parent:  subtract the coordinate of the new parent. 
*/ 
      if ( 
	(string_lower(last_parent) = "null") AND 
        (string_lower(attr_value) <> "null") 
      ) then 

        DEBUG3(
          printf("coordinate_value: new parent.\n", "");
          printf("coordinate_value: attr = %s.\n",current_name);
          printf("coordinate_value: parent x = %s.\n",current_value);
        )

        current_id := object_instance_sdd.attr_list;
/*
**  Search for affected attributes.
*/
        while (current_id <> NULL) do
/*
**  If the application attribute shared data is either x, y, horizdistance,
**  or vertdistance modify the application data and the values displayed
**  on the edit attributes form, if present.
*/
          if (
            (current_name = "x") OR (current_name = "horizdistance") OR
            (current_name = "y") OR (current_name = "vertdistance")
          ) then
/*
**  Update the application.
*/
            DEBUG3(
              printf("gw_methods var: ABOUT TO ASSIGN APP VALUE.\n","");
              printf("gw_methods var: current_id = %s.\n",current_id);
              printf(
                "gw_methods var: value = %s.\n",
                get_sd_value(object_id, current_name)
              );
            )

            temp_value :=  get_sd_value(current_id,"value") - 
              get_sd_value(get_id(attr_value), current_name);
/*
**  Update the application value.
*/
            put_sd_value(current_id,"value",temp_value);
/*
**  Update the display value.
*/
            if (id_exists(object_id)) then
              put_sd_value(object_id,attr_name,get_id(textBuffer));
            end if;

          end if;               /* x, y, horiz, or vert attribute       */
          current_id := get_sd_value(current_id,"attr_list");
        end while;

      end if;
/* 
**  Un-Parent: adding back the coordinate of the old parent form. 
*/ 
      if ( 
        (string_lower(last_parent) <> "null") AND 
        (string_lower(attr_value) = "null") 
      ) then 

        DEBUG3(printf("coordinate_value: un parent.\n", ""))
/*
**  Search for affected attributes.
*/
        while (current_id <> NULL) do
/*
**  If the application attribute shared data is either x, y, horizdistance,
**  or vertdistance modify the application data and the values displayed
**  on the edit attributes form, if present.
*/
          if (
            (current_name = "x") OR (current_name = "horizdistance") OR
            (current_name = "y") OR (current_name = "vertdistance")
          ) then
/*
**  Update the application.
*/
            DEBUG3(
              printf("gw_methods var: current_id = %s.\n",current_id);
              printf(
                "gw_methods var: value = %s.\n",
                get_sd_value(object_id, current_name)
              );
            )

            temp_value :=  get_sd_value(current_id,"value") + 
              get_sd_value(get_id(last_parent), current_name);
/*
**  Update the application value.
*/
            put_sd_value(current_id,"value",temp_value);
/*
**  Update the display value.
*/
            if (id_exists(object_id)) then
              put_sd_value(object_id,attr_name,get_id(textBuffer));
            end if;

          end if;               /* x, y, horiz, or vert attribute       */
          current_id := get_sd_value(current_id,"attr_list");
        end while;

#if 0		/* the way things used to get done */
       put_sd_value(x_sdi, "value", 
         get_sd_value(x_sdi, "value") + get_sd_value(get_id(last_parent), "x")
       ); 
#endif

      end if;
/*
**  NOTE: Differet Parent:  If the parents changed there is no need to 
**  reevaluate the coordinates since they should remain at the same offset.
*/
    end if;		/* no error occured			*/

    DEBUG2(
      printf(
        "parent_attr_text: parent id =  %s.\n",
        get_id(object_instance_sdd.name)
      );
      printf("parent_method: exit. \n", "");
    )
      }
/*
** Focus method.
*/
      focus: {
        if (id_exists(current_focus)) then
          put_sd_value(current_focus,"font","6x13");
        end if;
        parent_attr_box.focus := SELF;
        current_focus := SELF;
        font := "*";
      }
  }

  On Destroy: {
    destroy_sd_instance(parent_xtype_sdd);
  }

end VC gt_parent_vc;
