/* $Header: /u/04b/mnt/ljb/RCS/widget.sl,v 1.2 1991/04/23 17:58:33 ljb Exp $ */
/*--------------------------------------------------------------------------*\
|
|  Name:  Widget Xtype Editor (widget.sl)
|
|  Description:
|     This file contains the Slang code to implement the edit
|     field for intgeer attributes.
|
|  Component of:
|     Dialogue editor dialogue.
|
|  Related files:
|     gw.sl - generic widget code.
|
|  Project:
|     Serpent
|     User Interface Project
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:
|     Robert C. Seacord
|     rcs@sei.cmu.edu
|
|  Change History:
|     Mar 26, 1991 : Module created.
|     $Log: widget.sl,v $
# Revision 1.2  1991/04/23  17:58:33  ljb
# change background to form rather than box
#
# Revision 1.1  1991/04/11  18:08:30  rcs
# Initial revision
#
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*------------------- constants --------------------------------------------*/

#undef DEBUG1
#undef DEBUG2
#undef DEBUG3

#define DEBUG1(statement) SERPENT_DEBUG1(widget_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(widget_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(widget_debug, statement)

/*--------------------------------------------------------------------------*\
|  Name:  gt_widget_vc
|
|  Description:
|    The widget x type vc supports the editing of widget attributes.
\*--------------------------------------------------------------------------*/
VC: gt_widget_vc

Creation Condition: (new("widget_xtype_sdd"))

Variables:
  widget_help_id: true;
  widget_attr_error_id;         /* id of error sdd      */
  attr_name: get_sd_value(widget_xtype_sdd.attribute,"name");
  attr_value: get_sd_value(widget_xtype_sdd.attribute,"value");

Objects:

  attribute_box(widget)

  attribute_label(widget)

  widget_attr_text: XawTextentry {
    Attributes:
      parent: widget_attr_box;
      width: ATTR_WIDTH;
      height: ATTRIBUTE_HEIGHT;
      borderColor: "white";
      horizDistance: widget_attr_label.width + 3;
      textBuffer: {
        if (attr_value = UNDEFINED) then
          textBuffer := glue_get_attr_default(
            widget_handle,attr_name,SERPENT_DEFAULT
        );
        else
          textBuffer := attr_value;
        end if;
      }
    Methods:
      send: {

        DEBUG2(
          printf("widget_attr_text: send method call.\n", "");
          printf("widget_attr_text: name = %s.\n",attr_name);
          printf("widget_attr_text: textBuffer = %s.\n", textBuffer);
          printf(
            "widget_attr_text: widget id = %s.\n", 
            get_id(object_instance_sdd.name)
          );
        )
/* 
**  If the object instance name specified in the text buffer is 
**  a valid object, or null or undefined update the application 
**  and display values.  Otherwise raise an error condition. 
*/ 
        if (
          object_exists(textBuffer) OR
          string_lower(textBuffer) = "undefined" OR 
          string_lower(textBuffer) = "null" 
        ) then 

          DEBUG3(
            printf("widget_attr_text: widget exists.\n", "");
            printf(
              "widget_attr_text: widget id =  %s.\n",
              get_id(object_instance_sdd.name)
            );
          )
/*
**  Update the application value.
*/
          put_sd_value(widget_xtype_sdd.attribute,"value",textBuffer);
/*
**  Update the display value.
*/
          if (id_exists(object_id)) then
            put_sd_value(object_id,attr_name,textBuffer);
          end if;
/* 
**  Set the background and foreground colors to normal in case 
**  there was a previous error condition. 
*/ 
          widget_attr_text.background := "white"; 
          widget_attr_text.foreground := "black"; 
/* 
**  If ID for error_sdd exists, destroy the shared data instance. 
*/ 
          if (id_exists(widget_attr_error_id)) then 	/* error id exists */
            destroy_sd_instance(widget_attr_error_id); 
          end if; 			/* end if error id exists 	*/
/*
**  Error condition.
*/
    else			/* object not defined nor null or undef */ 
/* 
**  Set error flag and create a help form for the attribute. 
*/ 
      error_flag := true; 

      widget_attr_error_id := create_sd_instance("error_sdd", "DM_BOX"); 
      put_sd_value(widget_attr_error_id, "title", "Parent"); 
      put_sd_value( 
        widget_attr_error_id, 
        "message", 
        "The specified widget is not defined." 
      ); 
      put_sd_value( 
        widget_attr_error_id, 
        "height", 
        string_count_chars( 
          "The specified widget is not defined.", "\n" 
        ) + 1 
      ); 
      put_sd_value(widget_attr_error_id, "self", widget_attr_error_id); 
/* 
**  Provide error highlighting. 
*/ 
      widget_attr_text.background := "black"; 
      widget_attr_text.foreground := "white"; 
    end if; 

    DEBUG2(
      printf("widget_attr_test: send method exit. \n", "");
    )

      }
/*
** Focus method.
*/
      focus: {
        if (id_exists(current_focus)) then
          put_sd_value(current_focus,"font","6x13");
        end if;
        widget_attr_box.focus := SELF;
        current_focus := SELF;
        font := "*";
      }
  }

  On Destroy: {
    destroy_sd_instance(widget_xtype_sdd);
  }

end VC gt_widget_vc;
