#include "sat.ill"

|||

#define MAX_SLIDES 19
#define BACKGROUND_COLOR "blue"
#define BODY_COLOR "goldenrod"
#define TITLE_COLOR "yellow"
#define LOGO_COLOR "white"

#ifdef mc68000

#define DISPLAY_WIDTH 1152
#define DISPLAY_HEIGHT 900

#else

#define DISPLAY_WIDTH 1000
#define DISPLAY_HEIGHT 800

#endif

#define TITLE_FONT "*times-bold-r-n*--34*"
#define BULLET_FONT "*times-bold-r-n*--34*"
#define SUBBULLET_FONT "*times-bold-r-n*--25*"

#define FONT_20 "*times-bold-r-n*--20*"
#define FONT_24 "*times-bold-r-n*--24*"
#define FONT_25 "*times-bold-r-n*--25*"

#define BULLET_OFFSET 50
#define SUBBULLET_OFFSET 100

#define LEFT 0
#define CENTER 1
#define RIGHT 2

#define cnext(fname,fparent,fhoriz) \
fname: XawCommand {\
Attributes:\
  parent: fparent;\
  label: "Next";\
  font: TITLE_FONT;\
  width: 170;\
  height: 50;\
  vertDistance: DISPLAY_HEIGHT - 55;\
  horizDistance: fhoriz;\
  borderColor: "white";\
  background: BACKGROUND_COLOR;\
  foreground: BODY_COLOR;\
 Methods:\
  notify: {\
    if (slide < MAX_SLIDES) then\
      slide := slide + 1;\
    endif;\
  }\
}

#define cprev(fname,fparent,fhoriz) \
fname: XawCommand {\
Attributes:\
  parent: fparent;\
  label: "Previous";\
  font: TITLE_FONT;\
  width: 170;\
  height: 50;\
  vertDistance: DISPLAY_HEIGHT - 55;\
  horizDistance: fhoriz;\
  borderColor: "white";\
  foreground: BODY_COLOR;\
  background: BACKGROUND_COLOR;\
 Methods:\
  notify: {\
    if (slide > 0) then\
      slide := slide - 1;\
    endif;\
  }\
}

#define cquit(fname,fparent,fhoriz) \
fname: XawCommand {\
Attributes:\
  parent: fparent;\
  label: "Quit";\
  font: TITLE_FONT;\
  width: 170;\
  height: 50;\
  vertDistance: DISPLAY_HEIGHT - 55;\
  horizDistance: fhoriz;\
  borderColor: "white";\
  foreground: BODY_COLOR;\
  background: BACKGROUND_COLOR;\
 Methods:\
  notify: {\
    exit();\
  }\
}


#define cform(fname) \
fname: XawBboard {\
Attributes:\
  parent: toplevel;\
  height: DISPLAY_HEIGHT;\
  width: DISPLAY_WIDTH;\
  background: BACKGROUND_COLOR;\
}

#define major_heading(fname, fparent, flabel, ffy) \
fname: XawLabel {\
Attributes:\
  parent: fparent;\
  font: TITLE_FONT;\
  label: flabel;\
  vertDistance: ffy;\
  horizDistance: 0;\
  borderColor: BACKGROUND_COLOR;\
  height: 50;\
  width: DISPLAY_WIDTH;\
  foreground: BODY_COLOR;\
  background: BACKGROUND_COLOR;\
}

#define minor_heading(fname, fparent, flabel, ffy, ffont) \
fname: XawLabel {\
Attributes:\
  parent: fparent;\
  font: ffont;\
  label: flabel;\
  vertDistance: ffy;\
  horizDistance: 0;\
  borderColor: BACKGROUND_COLOR;\
  height: 50;\
  width: DISPLAY_WIDTH;\
  background: BACKGROUND_COLOR;\
  foreground: BODY_COLOR;\
}

#define title(fname,flabel) \
fname: XawLabel {\
Attributes:\
  parent: slide_form;\
  font: TITLE_FONT;\
  label: flabel;\
  vertDistance: 150;\
  borderColor: BACKGROUND_COLOR;\
  height: 50;\
  width: DISPLAY_WIDTH;\
  background: BACKGROUND_COLOR;\
  foreground: TITLE_COLOR;\
}

#define bullet(fname, flabel, ffy)\
fname: XawLabel {\
Attributes:\
  parent: slide_form;\
  font: BULLET_FONT;\
  label: flabel;\
  vertDistance: ffy;\
  horizDistance: BULLET_OFFSET;\
  borderColor: BACKGROUND_COLOR;\
  background: BACKGROUND_COLOR;\
  foreground: BODY_COLOR;\
  height: 50;\
}

#define subbullet(fname, flabel, ffy)\
fname: XawLabel {\
Attributes:\
  parent: slide_form;\
  font: SUBBULLET_FONT;\
  label: flabel;\
  vertDistance: ffy;\
  horizDistance: SUBBULLET_OFFSET;\
  borderColor: BACKGROUND_COLOR;\
  foreground: BODY_COLOR;\
  background: BACKGROUND_COLOR;\
}

variables:

  main_talk : false;

objects:

    slide_form: XawBboard {
      Attributes:
        height: DISPLAY_HEIGHT;
        width: DISPLAY_WIDTH;
        background: BACKGROUND_COLOR;
    }

vc: title_slide_vc

  creation condition:  (NOT main_talk)

  objects:

    major_heading(j1, slide_form, "Serpent, A User Interface", 200)
    major_heading(j2, slide_form, "Management System", 260)
    minor_heading(j3, slide_form, "Software Engineering Institute", 350, FONT_25)
    minor_heading(j4, slide_form, "Carnegie Mellon University", 400, FONT_20)
    minor_heading(j5, slide_form, "Pittsburgh, PA 15213", 450, FONT_20)
    minor_heading(j6, slide_form, "Sponsored by the U.S. Department of Defense", 600, FONT_25)

    next_button: XawCommand {
      Attributes:
       parent: slide_form;
       label: "Next";
       font: BULLET_FONT;
       width: 170;
       height: 50;
       vertDistance: DISPLAY_HEIGHT - 55;
       horizDistance: DISPLAY_WIDTH - 220;
       borderColor: "white";
       foreground: "white";
       background: BACKGROUND_COLOR;

     Methods:
       notify: {
         main_talk := true;
       }
    }

end vc title_slide_vc;


vc: main_talk_vc

  creation condition:  (main_talk)

  variables:

   slide : 1;

  objects:

#if 0
    death_star: pixmap_object {
      Attributes:
        parent: slide_form;
        x: 10;
        y: 10;
        bitmap_file: "death_star.bitmap";
    }
#endif

    cmu_logo: XawLabel {
      Attributes:
        parent: slide_form;
        font: FONT_20;
        label: "Carnegie Mellon University";
        vertDistance: 	25;
        horizDistance: 150;
        borderColor: BACKGROUND_COLOR;
        height: 30;
        justify: LEFT;
        foreground: LOGO_COLOR;
        background: BACKGROUND_COLOR;
    }

    sei_logo: XawLabel {
      Attributes:
        parent: slide_form;
        font: FONT_25;
        label: "Software Engineering Institute";
        vertDistance: 	55;
        horizDistance: 150;
        borderColor: BACKGROUND_COLOR;
        height: 30;
        justify: LEFT;
        foreground: LOGO_COLOR;
        background: BACKGROUND_COLOR;
    }

    bar: XawLabel {
      Attributes:
        parent: slide_form;
        label: "";
        vertDistance: 	87;
        horizDistance: 150;
        height: 2;
        width: DISPLAY_WIDTH - 250;
    }

    cnext(next, slide_form, DISPLAY_WIDTH - 200)
    cprev(prev, slide_form, DISPLAY_WIDTH - 400)
    cquit(quit, slide_form, DISPLAY_WIDTH - 985)
/*--------------------- Introduction ---------------------------------------*/

vc: slide1_vc

  creation condition:  (slide = 1)

  objects:

    title(j1, "Introduction")
    bullet(j2, "* Motivation", 250)
    bullet(j3, "* Requirements", 320)
    bullet(j4, "* Serpent UIMS", 390)
    subbullet(j5, "- use as prototyping tool", 460)
    subbullet(j6, "- features to support production", 510)
    subbullet(j7, "- editor", 560)

end vc slide1_vc

/*--------------------- Current User Interface Life Cycle ------------------*/

vc: slide2_vc

  creation condition:  (slide = 2)

  objects:

    title(j1, "Current User Interface Life Cycle")
    bullet(j2, "* Prototype displays using prototyping tool", 250)
    bullet(j3, "* Implement prototype using user interface toolkit", 320)
    bullet(j4, "* Modify implementation to reflect errors in original", 390)
    bullet(j5, "  prototype or user enhancements", 450)
    bullet(j6, "* Best case scenario.  In reality, prototyping not", 520)
    bullet(j7, "  always done.", 580)

end vc slide2_vc

/*--------------------- Problems of Current Approach -----------------------*/

vc: slide3_vc

  creation condition:  (slide = 3)

  objects:

    title(j1, "Problems of Current Approach")
    bullet(j2, "* Prototype is thrown away after serving as model", 250)
    bullet(j3, "  for implementation", 310)
    bullet(j4, "* Mismatch between prototyping tool and", 380)
    bullet(j5, "  implementation toolkit", 440)
    bullet(j6, "* User Interface toolkits are complicated and difficult to use", 510)
    bullet(j8, "* Integration of new input/output media usually", 570)
    bullet(j9, "  requires total re-engineering of application", 640)

end vc slide3_vc

/*--------------------- Incremental Development ----------------------------*/

vc: slide4_vc

  creation condition:  (slide = 4)

  objects:

    title(j1, "Incremental Development")
    bullet(j2, "* Use same tool for prototyping and for production", 250)
    subbullet(j3, "- prototype is not thrown away", 320)
    subbullet(j4, "- constrains prototypes to easily implementable solutions", 370)
    subbullet(j5, "- requires tool that supports both aspects", 420)
    bullet(j6, "* Integrate new media to tool", 490)
    subbullet(j7, "- requires portion of application system be media independent", 560)
    subbullet(j8, "- requires extensible tool", 610)
    subbullet(j9, "- evolution from one presentation mechanism to another", 660)

end vc slide4_vc

/*--------------------- Requirement for UI Tool ----------------------------*/

vc: slide5_vc

  creation condition: (slide = 5)

  objects:

    title(j1, "Requirement for UI Tool")
    bullet(j2, "* For use as prototyper", 250)
    subbullet(j3, "- interactive drawing editor", 320)
    subbullet(j4, "- simulation of behavior", 370)
    bullet(j5, "* For use in production", 440)
    subbullet(j6, "- support for abstraction", 510)  
    subbullet(j7, "- inclusion of arbitrary application functionality", 560)  
    subbullet(j8, "- extensible in term of media", 610)
    subbullet(j9, "- reasonable performance", 660)  

end vc slide5_vc

/*--------------------- Serpent --------------------------------------------*/

vc: slide6_vc

  creation condition: (slide = 6)

  objects:

    title(j1, "Serpent")
    bullet(j2, "* General purpose user interface management system", 250)
    bullet(j3, "* Supports incremental development of user interface", 320)
    bullet(j4, "* Multiple media for presentation", 390)
    bullet(j5, "* Language for specification of user interface", 460)  
    bullet(j6, "* Interactive editor (under development) using ", 530)  
    bullet(j7, "  Serpent language for UI specification and execution", 590)
    bullet(j8, "* X11 Athena Widget Toolkit set currently used for presentation", 660)
/*    bullet(j9, "  presentation", 720)*/

end vc slide6_vc

/*--------------------- Serpent as Prototyping Tool ------------------------*/

vc: slide7_vc
  creation condition: (slide = 7)
  objects:

    title(j1, "Serpent as Prototyping Tool")
    bullet(j2, "* Editor includes drawing component", 250)
    bullet(j3, "* Editor outputs Serpent language", 320)
    bullet(j4, "* Language has facility for recording user", 390)
    bullet(j5, "  performance data", 450)

end vc slide7_vc

/*--------------------- Serpent as Production Tool -------------------------*/

vc: slide8_vc
  creation condition: (slide = 8)
  objects:

    title(j1, "Serpent as Production Tool")
    bullet(j2, "* Supports arbitrary applications", 250)
    bullet(j3, "* Supports multiple media, simultaneously, for user", 320)
    bullet(j35, "  interactions", 380)
    bullet(j4, "* Extensible in input/output media supported", 450)
    bullet(j5, "* Supports", 520)
    subbullet(j6, "- fixed application programmer interface", 590)  
    subbullet(j7, "- user control of task sequencing", 640)  
    subbullet(j8, "- consistency of data presentation", 690)  

end vc slide8_vc

/*--------------------- Slang, UI Specification Language -------------------*/

vc: slide9_vc

  creation condition: (slide = 9)

  objects:

    title(j1, "Slang, UI Specification Language")
    bullet(j2, "* Based on production model", 250)
    subbullet(j3, "- data driven", 320)  
    subbullet(j4, "- allows multiple threads of control", 370)  
    bullet(j5, "* Multiple views of the same data", 440)
    subbullet(j6, "- implemented with constraint mechanism", 510)  
    subbullet(j7, "- dependent values automatically re-evaluated", 560)  
    subbullet(j8, "  when independent values modified", 600)  

end vc slide9_vc

/*--------------------- Interaction Objects --------------------------------*/

vc: slide10_vc

  creation condition: (slide = 10)

  objects:

    title(j1, "Interaction Objects")
    bullet(j2, "* Slang provides language for manipulating arbitrary", 235)
    bullet(j3, "  interaction objects", 285)
    bullet(j4, "* Interaction objects contain", 350)
    subbullet(j5, "- attributes for presentation", 420)  
    subbullet(j6, "- methods for interaction", 470)  
    bullet(j7, "* Slang allows for extensible presentation technologies", 540)
    subbullet(j8, "- X11 Athena Widget Toolkit and digital mapping", 590)  
    subbullet(j9, "  system currently supported", 640)  
    subbullet(j10, "- video and gesturing system planned", 690)  

end vc slide10_vc

/*--------------------- Serpent Architecture -------------------------------*/

vc: slide14_vc

  creation condition:  (slide = 11)

  objects:

    title(j1, "Serpent Architecture")

    application_label: XawLabel {
      Attributes:
        parent: slide_form;
        font: FONT_20;
        label: "Application Layer";
        vertDistance: 240;
        horizDistance: DISPLAY_WIDTH / 2 - 200;
        width: 400;
        height: 75;
        foreground: BODY_COLOR;
        background: BACKGROUND_COLOR;
        borderColor: "white";
    }

    arrow1: XawLabel {
      Attributes:
        parent: slide_form;
        label: "";
        vertDistance: 	315;
        horizDistance: DISPLAY_WIDTH / 2;
        height: 25;
        width: 6;
    }

    serpent_form: XawForm {
      Attributes:
        parent: slide_form;
        height: 400;
        width: 600;
        vertDistance: 340;
        horizDistance: DISPLAY_WIDTH / 2 - 300;
        background: BACKGROUND_COLOR;
        borderColor: "white";
    }

    arrow2: XawLabel {
      Attributes:
        parent: serpent_form;
        label: "";
        vertDistance: 	0;
        horizDistance: 299;
        height: 25;
        width: 6;
    }

    dialogue_label: XawLabel {
      Attributes:
        parent: serpent_form;
        font: FONT_20;
        label: "Dialogue Layer";
        vertDistance: 25;
        horizDistance: 100;
        width: 400;
        height: 75;
        foreground: BODY_COLOR;
        background: BACKGROUND_COLOR;
        borderColor: "white";
    }

    arrow3: XawLabel {
      Attributes:
        parent: serpent_form;
        label: "";
        vertDistance: 	100;
        horizDistance: 300;
        height: 50;
        width: 6;
    }

    presentation_label: XawLabel {
      Attributes:
        parent: serpent_form;
        font: FONT_20;
        label: "Presentation Layer";
        vertDistance: 150;
        horizDistance: 100;
        width: 400;
        height: 75;
        foreground: BODY_COLOR;
        background: BACKGROUND_COLOR;
        borderColor: "white";
    }

    x_label: XawLabel {
      Attributes:
        parent: serpent_form;
        font: FONT_20;
        label: "X Window System";
        vertDistance: 225;
        horizDistance: 100;
        width: 200;
        height: 75;
        foreground: BODY_COLOR;
        background: BACKGROUND_COLOR;
        borderColor: "white";
    }

    other_label: XawLabel {
      Attributes:
        parent: serpent_form;
        font: FONT_20;
        label: "I/O Technologies";
        vertDistance: 225;
        horizDistance: 300;
        width: 200;
        height: 75;
        foreground: BODY_COLOR;
        background: BACKGROUND_COLOR;
        borderColor: "white";
    }

    serpent_label: XawLabel {
      Attributes:
        parent: serpent_form;
        font: FONT_20;
        label: "Serpent";
        vertDistance: 325;
        horizDistance: 0;
        width: 600;
        height: 50;
        borderColor: BACKGROUND_COLOR;
        foreground: BODY_COLOR;
        background: BACKGROUND_COLOR;
    }

end vc slide14_vc

/*--------------------- Serpent Editor - 1 -------------------------------*/

vc: slide17_vc

  creation condition: (slide = 12)

  objects:

    title(j1, "Serpent Editor - 1")
    bullet(j2, "* Layouts of user interface are best specified or", 250)
    bullet(j3, "  examined graphically", 300)
    bullet(j4, "* Logic, dependencies and calculations are best", 370)
    bullet(j5, "  specified textually", 420)
    bullet(j6, "* Serpent editor has two portions", 490)
    subbullet(j7, "- structure editor for textual specification", 560)
    subbullet(j8, "- graphical editor for visualization of layout", 610)
    bullet(j9, "* Mechanism for moving between portions", 680)

end vc slide17_vc

/*--------------------- Serpent Editor - 2 -------------------------------*/

vc: slide18_vc

  creation condition: (slide = 13)

  objects:

    title(j1, "Serpent Editor - 2")
    bullet(j2, "* Implemented using Serpent", 250)
    bullet(j3, "* Currently over 5000 lines of Slang", 320)
    bullet(j4, "* Application layer", 390)
    subbullet(j5, "- has commands such as create vc, add object to vc, etc.", 460)
    subbullet(j6, "- saves and restores user interface specifications", 510)
    bullet(j7, "* User interface portion", 580)
    subbullet(j8, "- has forms for structured input", 650)
    subbullet(j9, "- manages layout for realization", 700)

end vc slide18_vc

/*--------------------- Summary -------------------------------*/

vc: slide19_vc

  creation condition: (slide = 14)

  objects:

    title(j1, "Summary")
    bullet(j2, "* Serpent  is a media independent UIMS which", 250)
    bullet(j3, "  supports incremental development of user", 300)
    bullet(j35, "  interfaces through", 350)
    subbullet(j4, "- features which support prototyping", 420)
    subbullet(j5, "- power which supports production", 470)
    subbullet(j6, "- providing an application programming interface", 520)
    subbullet(j7, "  which is constant through changes in the user", 560)
    subbullet(j8, "  interface or in input/output technology", 600)
    subbullet(j9, "- extensibility in user interface technologies", 650)


end vc slide19_vc

end vc main_talk_vc;
