#include "u22_rt.h"

#define MAGIC      (int)0xAAAAAAAA
#define UNDEFINED_ID (int)0xAAAAAAAA

typedef struct line {
   short dx, dy;
   struct line *link;
   int magic;
 } polyline;

struct polygon {
   struct {
     polyline *head, *tail;
   }lower, upper;
 };

typedef struct tnode {
   int width, height, border;
   struct polygon contour;
 } tree;
   
int parent_distance;

 /*--------------------------------------------------------------------------*\
|  Routine: put_shared_integer_data
|
|  Description:
|     do the necessary conversions and call the correct routines
|     to put a value into a shared data element
|     
|  Parameters:
|     id: id of shared data element
|     component_name: name of component to place value into
|     value: value of type integer
|     
|  Result: 
|     none
\*                                                                          */
void put_shared_integer_data(id, component_name, value)
iid_id_type id;
int value;
string component_name;
{
     dra_put_component_value (
        dob_make_id_constant_adi (id), 
        dob_make_string_constant_adi (component_name),
        dob_make_integer_CONSTANT_adi (value)
        );

   return;
 }

 /*--------------------------------------------------------------------------*\
|  Routine: get_shared_integer_data
|
|  Description:
|     do the necessary conversions and call the correct routines
|     to get a value of type integer from a shared data element
|     
|  Parameters:
|     id: id of shared data element
|     component_name: name of component to place value into
|     
|  Result: 
|     value of type integer
\*                                                                          */
int get_shared_integer_data(id, component_name)
iid_id_type id;
string component_name;
{
int value;

     value = drc_get_atv( dra_get_component_value(
        dob_make_id_constant_adi (id),
        dob_make_string_constant_adi (component_name)),
        idd_integer);

   return value;
 }

 /*--------------------------------------------------------------------------*\
|  Routine: put_shared_id_data
|
|  Description:
|     do the necessary conversions and call the correct routines
|     to put a value into a shared data element
|     
|  Parameters:
|     id: id of shared data element
|     component_name: name of component to place value into
|     value: value of type id
|     
|  Result: 
|     none
\*                                                                          */
void put_shared_id_data(id, component_name, value)
iid_id_type id, value;
string component_name;
{
     dra_put_component_value (
        dob_make_id_constant_adi (id), 
        dob_make_string_constant_adi (component_name),
        dob_make_id_constant_adi (value)
        );

   return;
 }

 /*--------------------------------------------------------------------------*\
|  Routine: get_shared_id_data
|
|  Description:
|     do the necessary conversions and call the correct routines
|     to get a value of type id from a shared data element
|     
|  Parameters:
|     id: id of shared data element
|     component_name: name of component to place value into
|     
|  Result: 
|     value of type id
\*                                                                          */
iid_id_type get_shared_id_data(id, component_name)
iid_id_type id;
string component_name;
{
iid_id_type value;

     value = drc_get_atv( dra_get_component_value(
        dob_make_id_constant_adi (id),
        dob_make_string_constant_adi (component_name)),
        idd_id);

   return value;
 }

/*--------------------------------------------------------------------------*\
|  Routine: dump_tree
|
|  Description:
|     print out tree structure
|
|  Parameters:
|     t: tree to be dumped
|     
|  Result: 
|     none
\*                                                                          */
void dump_tree(t)
iid_id_type t;
{
  tree *t_tree;
  polyline *node;
  iid_id_type next;
   
  if (t == iid_null_id || t == UNDEFINED_ID) return;

  printf("\n>****************************************************\n");
  printf("shared data node: %x\n",t);
  printf("  parent: %x\n",get_shared_id_data(t,"parent"));
  printf("  sibling: %x\n",get_shared_id_data(t,"sibling"));
  printf("  child: %x\n",get_shared_id_data(t,"child"));
  printf("  predecessor: %x\n",get_shared_id_data(t,"predecessor"));
  printf("  last child: %x\n",get_shared_id_data(t,"last_child"));

  printf("  x_offset: %d\n",get_shared_integer_data(t,"x_offset"));
  printf("  y_offset: %d\n",get_shared_integer_data(t,"y_offset"));
  printf("  x_absolute: %d\n",get_shared_integer_data(t,"x_absolute"));
  printf("  y_absolute: %d\n",get_shared_integer_data(t,"y_absolute"));

/*
  t_tree = (tree *)get_shared_integer_data(t,"node_info");
  printf("\nC data structure pointer: %x\n",t_tree);
  printf("  width, height, border: %d,%d,%d\n",t_tree->width,
                                               t_tree->height,
                                               t_tree->border);

  printf("\n contour upper head: \n");
  node = t_tree->contour.upper.head;
  while (node && node->magic == MAGIC) {
      printf("  next link: %x\n",node);
      printf("  dx,dy: %d,%d\n",node->dx,node->dy);
      node = node->link;
    }

  printf("\n contour upper tail: \n");
  node = t_tree->contour.upper.tail;
  while (node && node->magic == MAGIC) {
      printf("  next link: %x\n",node);
      printf("  dx,dy: %d,%d\n",node->dx,node->dy);
      node = node->link;
    }

  printf("\n contour lower head: \n");
  node = t_tree->contour.lower.head;
  while (node && node->magic == MAGIC) {
      printf("  next link: %x\n",node);
      printf("  dx,dy: %d,%d\n",node->dx,node->dy);
      node = node->link;
    }

  printf("\n contour lower tail: \n");
  node = t_tree->contour.lower.tail;
  while (node && node->magic == MAGIC) {
      printf("  next link: %x\n",node);
      printf("  dx,dy: %d,%d\n",node->dx,node->dy);
      node = node->link;
    }
*/
  printf("<***************************************************\n");

/* this dumps the current node, now we dump the sibling*/

  next = (iid_id_type)get_shared_id_data(t,"sibling");
  if (next != iid_null_id && t != UNDEFINED_ID) dump_tree(next);

/* finally we dump the eldest child */

  next = (iid_id_type)get_shared_id_data(t,"child");
  if (next != iid_null_id && t != UNDEFINED_ID) dump_tree(next);

  return;
}

/*--------------------------------------------------------------------------*\
|  Routine: line
|
|  Description:
|     create structure with line offsets and pointer to next
|     
|  Parameters:
|     dx: x offset of end of line from predecessor
|     dy: y offset of end of line from predecessor
|     link: pointer to next node of line
|     
|  Result: 
|     returns point to structure created
\*                                                                          */
polyline *line(dx,dy,link)
int dx,dy;
polyline *link;
{
   polyline *result;

   result = (polyline *)make_node (sizeof(polyline));
   result->dx = dx;
   result->dy = dy;
   result->link = link;
   result->magic = MAGIC;

   return result;
 }

/*--------------------------------------------------------------------------*\
|  Routine: offset
|
|  Description:
|     determine the offset between point and contours
|    
|  Parameters
|     p1: x of point 
|     p2: y of point 
|     a1: distance from upper x
|     a2: distance from upper y
|     b1: distance from lower x
|     b2: distance from lower y
|     
|  Result: 
|     distance
\*                                                                          */
int offset (p1,p2,a1,a2,b1,b2)
int p1, p2, a1, a2, b1, b2;
{
   int d, s, t;

   if (b1 <= p1 || p1+a1 <= 0) return 0;

   t = b1*a2 - a1*b2;
   if (t > 0)
      if (p1 < 0) 
         s = p1 * a2, d = s / a1 - p2;
      else if (p1 > 0)
         s = p1 * b2, d = s/b1 - p2;
      else d = -p2;
   else if (b1 < p1+a1)
         s = (b1-p1) * a2, d = b2 - (p2 + s/a1);
   else if (b1 > p1 + a1)
         s = (a1 + p1) * b2, d = s / b1 - (p2+a2);
   else d = b2 - (p2+a2);
   if (d <0) return 0;
   return d;
}

/*--------------------------------------------------------------------------*\
|  Routine: bridge
|
|  Description:
|     construct a bridge between two lines
|     
|  Parameters:
|     line1: pointer to first line
|     x1: offset in x
|     y1: offset in y
|     line2: pointer to second line
|     x2: offset in x
|     y2: offset in y
|     
|  Result: 
|     polyline that contains all of the lines bridged
\*                                                                          */
polyline *bridge(line1,x1,y1,line2,x2,y2)
polyline *line1, *line2;
int x1, y1, x2, y2;
{

   int dy, dx, s;
   polyline *r;

   dx = x2 + line2->dx - x1;
   if (line2->dx == 0) dy = line2->dy;
   else  s = dx * line2->dy, dy = s /line2->dx;
   r = line(dx,dy,line2->link);
   line1->link = line (0,y2 + line2->dy - dy - y1, r);
   return r;
}

/*--------------------------------------------------------------------------*\
|  Routine: merge
|
|  Description:
|     merge two polygons
|     
|  Parameters:
|     c1: the first polygon
|     c2: the second polygon
|     
|  Result: 
|     the width of the two polygons
\*                                                                          */
int merge(c1,c2)
struct polygon *c1,*c2;
{
   int x, y, total, d;
   polyline *lower,*upper,* b;

   x = y = total = 0;
   upper = c1->lower.head;
   lower = c2->upper.head;

   while (lower && upper) {

     d = offset(x,y,lower->dx,lower->dy,upper->dx,upper->dy);
     y += d;
     total += d;

     if (x + lower->dx <= upper->dx) {
         y += lower->dy;
         x += lower->dx;
         lower = lower->link;
       }
     else {
         y -= upper->dy;
         x -= upper->dx;
         upper = upper->link;
       }
   }

   if (lower) {
      b = bridge(c1->upper.tail,0,0,lower,x,y);
      c1->upper.tail = (b->link) ? c2->upper.tail : b;
      c1->lower.tail = c2->lower.tail;
    }
   else {
      b = bridge (c2->lower.tail,x,y,upper,0,0);
      if (!b->link) c1->lower.tail = b;
    }
   c1->lower.head = c2->lower.head;

   return total;
 }

/*--------------------------------------------------------------------------*\
|  Routine: join
|
|  Description:
|     merge the contours for a sub tree
|     
|  Parameters:
|     t: root of the subtree (shared data id)
|     
|  Result: 
|     none
\*                                                                          */
join(t)
iid_id_type t;
{
   iid_id_type c;
   int d, h, sum;
   tree *tree_t, *tree_c;

   c = get_shared_id_data(t, "child");
   tree_t = (tree *)get_shared_integer_data (t, "node_info");
   tree_c = (tree *)get_shared_integer_data (c, "node_info");
   tree_t->contour = tree_c->contour;
   sum = h = tree_c->height + 2*tree_c->border;
   c = get_shared_id_data (c, "sibling");
   while (c != iid_null_id && c != UNDEFINED_ID) {
     tree_c = (tree *) get_shared_integer_data (c, "node_info");
     d = merge(&tree_t->contour, &tree_c->contour);
     put_shared_integer_data (c, "y_offset", d+h);
     put_shared_integer_data (c, "x_offset", 0);
     h = (tree_c->height) + 2*(tree_c->border);
     sum += d+h;
     c = get_shared_id_data (c, "sibling");
   }
   return sum;
 }

/*--------------------------------------------------------------------------*\
|  Routine: layout_leaf
|
|  Description:
|     create contour for the leaf sent as parameter
|     
|  Parameters:
|     t: node for which parameter is to be created
|     
|  Result: 
|     returns nothing
\*                                                                          */
void layout_leaf(t)
tree *t;
{

   t->contour.upper.tail = line(t->width + 2*t->border,0,0);
   t->contour.upper.head = t->contour.upper.tail;
   t->contour.lower.tail = line(0,-t->height-2*t->border,0);
   t->contour.lower.head = 
       line(t->width + 2*t->border,0,t->contour.lower.tail);

   return;
 }

/*--------------------------------------------------------------------------*\
|  Routine: rubout
|
|  Description:
|     remove all memory allocated by add_new_node
|
|  Parameters:
|     node_id: id of vc that owns the node of the tree
|  Result: 
|     none
\*                                                                          */
void rubout(t)
iid_id_type t;
{
tree *t_tree;
polyline *node, *old_node;
/* first we get the pointer to the data structures for the node*/
   t_tree = (tree *)get_shared_integer_data(t,"node_info");

/* now we free the list of line segments */
   node = t_tree->contour.upper.head;
   t_tree->contour.upper.head = 0;
   while (node && node->magic == MAGIC){
     node->magic = 0;
     free_node(node);
     node = node->link;
   }

    node = t_tree->contour.upper.tail;
    t_tree->contour.upper.tail = 0;
    while (node && node->magic == MAGIC){
     node->magic = 0;
       free_node(node);
       node = node->link;
      }

   node = t_tree->contour.lower.head;
   t_tree->contour.lower.head = 0;
   while (node && node->magic == MAGIC){
     node->magic = 0;
     free_node(node);
     node = node->link;
   }

    node = t_tree->contour.lower.tail;
    t_tree->contour.lower.tail = 0;
    while (node && node->magic == MAGIC){
      node->magic = 0;
      free_node(node);
      node = node->link;
    }

   return;
}
/*--------------------------------------------------------------------------*\
|  Routine: add_new_leaf
|
|  Description:
|     Do the initialization for a leaf in the tree.  This will always 
|       be a new node.  creates contour for leaf and sets up other pointers for
|     use by subsequent routines
|
|  Parameters:
|     node_id: id of vc that owns the node of the tree
|     height: height of node
|     width: width of node
|     border: border around node for spacing
|     parent_dist: global variable of parent distance
|     
|  Result: 
|     returns pointer to structure that describes node
\*                                                                          */
     tree * add_new_leaf(node_id, height, width, border, parent_dist)
     int height, width, border, parent_dist;
     int node_id;
{
     tree *result;

     parent_distance = parent_dist;

     /* make and initialize the node for this leaf */
     result = (tree *) make_node(sizeof(tree));
     result->width = width;
     result->height = height;
     result->border = border;

     layout_leaf (result);

     return result;
}

/*--------------------------------------------------------------------------*\
|  Routine: attach parent
|
|  Description:
|     merge the contours of the parent of a sub tree into
|     the contours of the rest of the subtree
|     
|  Parameters:
|     t: subtree parent (from shared data)
|     h: height of contours of subtree
|     
|  Result: 
|     none
\*                                                                          */
void attach_parent(t,h)
iid_id_type t;
int h;
{
   tree *t_tree;
   int x, y1, y2;
   iid_id_type temp;

   t_tree = (tree *)get_shared_integer_data(t,"node_info");
   x = t_tree->border + parent_distance;
   y2 = (h - t_tree->height) / 2 - t_tree->border;
   y1 = y2 + t_tree->height + 2*(t_tree->border) - h;
   temp = get_shared_id_data (t, "child");
   put_shared_integer_data (temp, "x_offset", x + t_tree->width);
   put_shared_integer_data (temp, "y_offset", y1);
   t_tree->contour.upper.head = 
        line(t_tree->width, 0, line(x, y1, t_tree->contour.upper.head));
   t_tree->contour.lower.head = 
        line(t_tree->width, 0, line(x, y2, t_tree->contour.lower.head));
   return;
 }

/*--------------------------------------------------------------------------*\
|  Routine: layout_tree
|
|  Description:
|     construct contours of tree
|     
|  Parameters:
|     t: tree parent (from shared data)
|     
|  Result: 
|     none
\*                                                                          */
void layout_tree(t)
iid_id_type t;
{
iid_id_type c;
tree *t_tree;

   c = get_shared_id_data(t, "child");
   while (c != iid_null_id && c != UNDEFINED_ID) {
     layout_tree(c);
     c = get_shared_id_data(c,"sibling");
   }

   c = get_shared_id_data(t, "child");
   if (c != iid_null_id && c != UNDEFINED_ID) 
       attach_parent(t,join(t));
   else {
       t_tree = (tree *)get_shared_integer_data(t, "node_info");
       layout_leaf(t_tree);
     }

   return;
}

/*--------------------------------------------------------------------------*\
|  Routine: unlayout_tree
|
|  Description:
|     remove contours of tree
|     
|  Parameters:
|     t: tree parent (from shared data)
|     
|  Result: 
|     none
\*                                                                          */
void unlayout_tree(t)
iid_id_type t;
{
iid_id_type c;
 
   c = get_shared_id_data(t, "child");
   while (c != iid_null_id && c != UNDEFINED_ID) {
     unlayout_tree(c);
     c = get_shared_id_data(c,"sibling");
   }

   c = get_shared_id_data(t, "child");
   if (c == iid_null_id || c == UNDEFINED_ID) 
       rubout(t);

   return;
}


/*--------------------------------------------------------------------------*\
|  Routine: remove_leaf
|
|  Description:
|       remove all memory used by a leaf
|
|  Parameters:
|     node_id: id of of sdd for leaf
|     
|  Result: 
|     none
\*                                                                          */
     void remove_leaf(t)
iid_id_type t;
{
tree *t_tree;

/* first we remove lines*/

   rubout(t);

/*  now remove C data structure*/

   t_tree = (tree *)get_shared_integer_data(t,"node_info");
   free_node(t_tree);
   return;
}
