#include "monitorc.ill"
#include "sat.ill"

#define XMAX 750
#define YMAX 600
#define DM_HEIGHT 30
#define INFO_HT 25
#define PARTN_LABEL_HT 15
#define PARTN_FORM_HT YMAX - DM_HEIGHT - INFO_HT - PARTN_LABEL_HT
#define BUTTON_HT 15
#define BORDER 1
#define SPACING 10	/* Margin */
#define PSPACING 10	/* Spacing between Partns */
#define HDIST 100	/* Spacing over of trans and elems */
#define LEFT_JUSTIFY 0
#define CHAR_WTH 7
#define WIDTH(len) (CHAR_WTH * (len + 1))
#define XawForm XawBboard
|||

#include "sstring.ext"
#include "sstring.h"

EXTERNALS:  void ipb_start_recording (string, string);
EXTERNALS:  string buffer_to_ascii_hex (buffer);
EXTERNALS:  integer strcmp (string, string);
EXTERNALS:  integer strncmp (string, string, integer);

#define record ipb_start_recording
#define idd_buffer 6

VC: Input

  CREATION CONDITION: (new ("input_sdd"))

  VARIABLES:

    display_get_file_name: {
      if (input_sdd.ask_for_file_name) then
        display_get_file_name := True;
      endif;}
    file_name_ok: False;
    recording: False;
    on_or_off: {
      if (recording) then
        on_or_off := "ON";
      else
        on_or_off := "OFF";
      endif;
    }
    scroll_vert: 0;
    pnum: 0;
    tnum: 0;

  VC: get_file_name

   CREATION CONDITION: (display_get_file_name = True)

    OBJECTS:

    file_name_window: XawForm {

      ATTRIBUTES:

        maintainSize: False;
	width: 310;
	height: 75;
    }

    display: XawLabel {

      ATTRIBUTES:

        parent: file_name_window;
        width: 300;
        label: {
          if (string_length (input_sdd.file_name) = 0) then
            label := "Please choose a journaling file to monitor.";
          elsif (input_sdd.ask_for_file_name) then
            label := string_append (input_sdd.file_name,
	                      " is invalid.  Please choose another.");
          else
            label := "Attempting to use file.";
          endif;
	}
    }

    file_name_button: XawText {

      ATTRIBUTES:

        parent: file_name_window;
        vertDistance: 25;
        width: 300;
        textBuffer: "";
        editType: 2;
        sendBuffer: False;

      METHODS:

        send: {
          input_sdd.file_name := textBuffer;
          input_sdd.ask_for_file_name := False;
          sendBuffer := False;
	}
    }

    ok_button: XawCommand {

      ATTRIBUTES:

        vertDistance: 50;
        parent: file_name_window;
        label: "OK";
        width: WIDTH(2);

      METHODS:

        notify: {
          file_name_button.sendBuffer := True;
	}
    }

    exit_button: XawCommand {

      ATTRIBUTES:

	parent: file_name_window;
        vertDistance: 50;
        horizDistance: 50;
	label: "EXIT MONITOR";

      METHODS:

        notify: {
          exit();
	}
    }

  END VC get_file_name

  VC: control_window

   CREATION CONDITION: (file_name_ok)

   OBJECTS:

    the_input_window: XawForm {

      ATTRIBUTES:

        maintainSize: False;
        width: 100;
        height: 100;

    }

    exit_button: XawCommand {

      ATTRIBUTES:

	parent: the_input_window;
	label: "EXIT";

      METHODS:

        notify: {
          exit();
	}
    }

    stop_button: XawCommand {

      ATTRIBUTES:

        parent: the_input_window;
        horizDistance: 50;
        label: "STOP";
        sensitive: (strcmp (input_sdd.speed, "DONE") <> 0);

      METHODS:

        notify: {
          input_sdd.speed := "STOP";
	}
    }

    step_button: XawCommand {

      ATTRIBUTES:

        parent: the_input_window;
        horizDistance: 100;
        label: "STEP";
        sensitive: (strcmp (input_sdd.speed, "STOP") = 0);

      METHODS:

	notify: {
          step_size_button.sendBuffer := True;
/*
   The send method in step_size_button sets the step size,
   and then sends a STEP message to the application
*/
	}
    }

    run_button: XawCommand {

      ATTRIBUTES:

        parent: the_input_window;
	horizDistance: 150;
	label: "RUN";
        sensitive: (strcmp (input_sdd.speed, "STOP") = 0);

      METHODS:

	notify: {
          input_sdd.speed := "RUN";
        }
    }

    speed_display: XawLabel {

      ATTRIBUTES:

        parent: the_input_window;
        vertDistance: 25;
        label: string_append ("Speed: ", input_sdd.speed);
    }

    step_size_label: XawLabel {

      ATTRIBUTES:

        parent: the_input_window;
        width: WIDTH(9);
        vertDistance: 25;
        horizDistance: 100;
        label: "Stepsize:";
    }

    step_size_button: XawText {

      ATTRIBUTES:

        sensitive: (strcmp (input_sdd.speed, "DONE") <> 0);
        parent: the_input_window;
        width: WIDTH(4);
        vertDistance: 25;
        horizDistance: 100 + step_size_label.width;
        textBuffer: string_append (" ", input_sdd.step_size);
        editType: 2;
        sendBuffer: False;

      METHODS:

        send: {
          input_sdd.step_size := textBuffer;
          if (input_sdd.step_size < 1) then
            input_sdd.step_size := 1;
          endif;
          sendBuffer := False;
/*
           This method is requested when the STEP button is clicked.  To
           ensure that the correct step_size is sent in the same transaction
           as the STEP request, we set the speed to STEP here, instead of in
           the STEP_button's notify method.
*/
          input_sdd.speed := "STEP";
	}
    }

    record_button: XawCommand {

      ATTRIBUTES:

        parent: the_input_window;
        vertDistance: 50;
        label: string_append ("Recording: ", on_or_off);
        sensitive: (NOT (recording));

      METHODS:

        notify: {
	  if (NOT (recording)) then
	    record ("monitor.record", "Recording monitor from the DM");
	    recording := True;
	  endif;
        }
    }

    tnum_button: XawLabel {

      ATTRIBUTES:

        parent: the_input_window;
        width: WIDTH(10);
        vertDistance: 50;
        horizDistance: 125;
        label: string_append ("Total: ", tnum);
    }

  END VC control_window

  VC: Output

    CREATION CONDITION: (new ("scalars_sdd"))

    VARIABLES:

      text1: string_append ("Dialogue name: ", scalars_sdd.dialogue_name);
      text2: string_append ("   Message: ", scalars_sdd.dialogue_message);
      info_text: string_append (text1, text2);

    OBJECTS:

      the_window: XawForm {

        ATTRIBUTES:

          width: XMAX + 5;
	  height: YMAX;

      }

      info_box: XawLabel {

        ATTRIBUTES:

	  parent: the_window;
	  width: XMAX;
	  height: INFO_HT - 5;
	  label: info_text;

      }

      dm_box: XawLabel {

        ATTRIBUTES:

	  parent: the_window;
	  width: XMAX;
	  height: DM_HEIGHT - 10;
	  vertDistance: INFO_HT + 5;
	  label: "This box represents the Dialogue Manager";

    }

  ON CREATE:
    {
       file_name_ok := True;
       display_get_file_name := False;
    }

    VC: partn

      CREATION CONDITION: (new ("participant_sdd") and
                  (strcmp (participant_sdd.mailbox_name, "DM_BOX") <> 0))

      VARIABLES:

        pindex;
        pwidth: {
          if (pnum = 0) then
            pwidth := 100;
          else
            pwidth := ((XMAX - (2 * SPACING) + PSPACING) / pnum) - PSPACING;
          end if;
        }
        pxpos: SPACING + ((pindex - 1) * (pwidth + PSPACING));

      OBJECTS:

        partn_form:  XawForm {

          ATTRIBUTES:

            parent: the_window;
            width: pwidth;
            height: PARTN_FORM_HT;
            horizDistance: pxpos;
            vertDistance: INFO_HT + DM_HEIGHT;

        }

        partn_label: XawLabel {

          ATTRIBUTES:

            parent: the_window;
            width: pwidth;
            height: PARTN_LABEL_HT;
            horizDistance: pxpos;
            vertDistance: INFO_HT + DM_HEIGHT + PARTN_FORM_HT;
            label: participant_sdd.mailbox_name;

        }

        scroll_form:  XawForm {

          ATTRIBUTES:

            parent: partn_form;
            width: pwidth - 1;
            height: {
              if (tnum < 32) then
                height := PARTN_FORM_HT - 1;
              else
                height := (tnum * (BUTTON_HT + 1) + (2 * PSPACING));
              endif;
	    }
            horizDistance: 0;
            vertDistance: scroll_vert;
            x: 0;
            y: scroll_vert;
            allowUserMove: True;

          METHODS:

            move: {
              scroll_vert := y;
              x := 0;
	    }
	}

      ON CREATE:
        {
          pnum := pnum + 1;
          pindex := pnum;
        }

    END VC partn

    VC: trans

      CREATION CONDITION: (new ("transaction_sdd"))

      VARIABLES:

        enum: 0;
        tindex;
        from_dm;
        sender_sd: transaction_sdd.sender_id;
        sender_name: get_sd_value (sender_sd, "mailbox_name");
        receiver_sd: transaction_sdd.receiver_id;
/*      receiver_name: get_sd_value (receiver_sd, "mailbox_name"); */
        parent_sd: {
          if ((strcmp (sender_name, "DM_BOX")) = 0) then
            from_dm := True;
            parent_sd := receiver_sd;
          else
            from_dm := False;
            parent_sd := sender_sd;
          end if;
	}
        parent_vc: get_vc (parent_sd, "partn");
        t_scroll_form: get_object (parent_vc, "scroll_form");
        t_partn_form: get_object (parent_vc, "partn_form");
        text1: string_append ("#", tindex);
        text2: {
          if (from_dm) then
            text2 := " \/   ";
          else
            text2 := "   /\ ";
          endif;
	}
        text3: string_append (text1, text2);
        text: string_append (text3, transaction_sdd.timestamp);
        have_elements: False;
        show_elements: False;
        attached: True;
        e_form;

      OBJECTS:

        t_button:  XawCommand {

          ATTRIBUTES:

            parent: t_scroll_form;
            width: get_variable_value (parent_vc, "pwidth") - (2 * PSPACING);
            height: BUTTON_HT;
            horizDistance: PSPACING;
            vertDistance: PSPACING + ((tindex - 1) * (BUTTON_HT + 1));
            justify: LEFT_JUSTIFY;
            label: text;

          METHODS:

            notify: {
              if (NOT (have_elements)) then
                input_sdd.transaction_num := tindex;
                have_elements := True;
              endif;

              if (NOT (show_elements)) then
                attached := True;
                show_elements := True;
              else
                show_elements := False;
              endif;
            }
        }

      ON CREATE:
        {
            tnum := tnum + 1;
            input_sdd.tcount := tnum;
            tindex := tnum;
        }

      VC: show_elements_vc

        CREATION CONDITION: (have_elements)

        OBJECTS:

          form: XawForm {

            ATTRIBUTES:

              mappedWhenManaged: show_elements;
              maintainSize: False;
              allowUserMove: True;
              allowUserResize: True;

              parent: the_window;
              vertDistance: {
                if (attached) then
                   vertDistance := t_button.vertDistance
                         + get_sd_value (t_scroll_form, "vertDistance")
		         + get_sd_value (t_partn_form, "vertDistance");
                endif;
	      }
              horizDistance: {
                if (attached) then
                   horizDistance := t_button.horizDistance + HDIST
	                 + get_sd_value (t_partn_form, "horizDistance");
                endif;
	      }
              x: horizDistance;
              y: vertDistance;
              width: 230;
              height: {
          	if (enum = 0) then
          	  height := BUTTON_HT;
          	else
          	  height := ((BUTTON_HT + 1) * enum);
          	endif;
              }

            METHODS:

              move: {
                attached := False;
                vertDistance := y;
                horizDistance := x;
	      }
          }

          working: XawLabel {

            ATTRIBUTES:

              mappedWhenManaged: (enum = 0);
              parent: form;
              height: BUTTON_HT;
              vertDistance: 0;
              horizDistance: 0;
              label: "WORKING";
          }

        ON CREATE:
          {
            e_form := form;
          }

      END VC show_elements_vc

    END VC trans

    VC: element

      CREATION CONDITION: (new ("element_sdd"))

      VARIABLES:

        cnum: 0;
        eindex;
        trans_vc: get_vc (element_sdd.trans_id, "trans");
        tindex: get_variable_value (trans_vc, "tindex");
        e_form: get_variable_value (trans_vc, "e_form");
        text1: string_append (eindex, ": ");
        text2: string_append (element_sdd.change_type_str, " ");
        text3: string_append (text1, text2);
        text4: string_append (element_sdd.element_name, " ");
        text5: string_append (text3, text4);
        text: string_append (text5, element_sdd.id_num);
        have_components: False;
        show_components: False;
        attached: True;
        c_form;

      OBJECTS:

        e_button:  XawCommand {

          ATTRIBUTES:

            parent: e_form;
            height: BUTTON_HT;
            vertDistance: ((eindex - 1) * (BUTTON_HT + 1));
            horizDistance: 0;
            sensitive: ((strcmp(element_sdd.change_type_str, "remove")) <> 0);
            label: text;

          METHODS:

            notify: {
              if (NOT (have_components)) then
                input_sdd.transaction_num := tindex;
                input_sdd.element_num := eindex;
                have_components := True;
              endif;

              if (NOT (show_components)) then
                attached := True;
                show_components := True;
              else
                show_components := False;
              endif;

            }
        }

      ON CREATE:
          {
            trans_vc := get_vc (element_sdd.trans_id, "trans");
            eindex := get_variable_value (trans_vc, "enum") + 1;
            put_variable_value (trans_vc, "enum", eindex);
            eindex := element_sdd.e_index;
          }

      VC: show_components_vc

        CREATION CONDITION: (have_components)

        OBJECTS:

          form: XawForm {

            ATTRIBUTES:

              mappedWhenManaged: show_components;
              maintainSize: False;
              allowUserMove: True;
              allowUserResize: True;

              parent: the_window;
              vertDistance: {
                if (attached) then
                  vertDistance := e_button.vertDistance
                                   + get_sd_value (e_form, "vertDistance");
                endif;
	      }
              horizDistance: {
                if (attached) then
                  horizDistance := get_sd_value (e_form, "horizDistance")
                                    + HDIST;
                endif;
	      }
              width: 250;
              x: horizDistance;
              y: vertDistance;
              height: {
                if (cnum = 0) then
                  height := BUTTON_HT;
                else
                  height := ((BUTTON_HT + 1) * cnum);
                endif;
              }

            METHODS:

              move: {
                attached := False;
                vertDistance := y;
                horizDistance := x;
	      }
          }

          working: XawLabel {

            ATTRIBUTES:

              mappedWhenManaged: (cnum = 0);
              parent: form;
              height: BUTTON_HT;
              vertDistance: 0;
              horizDistance: 0;
              label: "WORKING";
          }

        ON CREATE:
            {
              c_form := form;
            }

      END VC show_components_vc

    END VC element

    VC: component

      CREATION CONDITION: (new ("component_sdd"))

      VARIABLES:

        cindex;
        element_vc: get_vc (component_sdd.element_id, "element");
        c_form: get_variable_value (element_vc, "c_form");
        text1: string_append (cindex, ": ");
        text2: string_append (component_sdd.component_name, " [");
        text3: string_append (component_sdd.component_type, "]");
        text: string_append (string_append (text1, text2), text3);
        show_data: True;

      OBJECTS:

        c_button:  XawCommand {

          ATTRIBUTES:

            parent: c_form;
            height: BUTTON_HT;
            width: WIDTH(string_length (text) + 1);
            vertDistance: ((cindex - 1) * (BUTTON_HT + 1));
            horizDistance: 0;
            label: text;

          METHODS:

            notify: {
              /* Toggle showing of data */
              show_data := (NOT (show_data));
            }
        }

        data: XawLabel {

          ATTRIBUTES:

            mappedWhenManaged: show_data;

            parent: c_form;
            height: BUTTON_HT;
            horizDistance: c_button.width;
            vertDistance: c_button.vertDistance;
            label: {
              if (component_sdd.is_buffer) then
                label :=
		    string_append ("Type: ", component_sdd.buffer_type);
              else
                label := component_sdd.data_buffer;
              endif;
            }
        }

      ON CREATE:
          {
            element_vc := get_vc (component_sdd.element_id, "element");
            cindex := get_variable_value (element_vc, "cnum") + 1;
            put_variable_value (element_vc, "cnum", cindex);
          }

      VC: buffer_display

        CREATION CONDITION: (component_sdd.is_buffer = True)

        VARIABLES:

          text: {
            if (strcmp (component_sdd.buffer_type, "buffer") = 0) then
              text := buffer_to_ascii_hex (component_sdd.data_buffer);
            else
              text := component_sdd.data_buffer;
            endif;
	  }

        OBJECTS:

          display: XawLabel {

            ATTRIBUTES:

              mappedWhenManaged: show_data;

              parent: c_form;
              height: BUTTON_HT;
              horizDistance: c_button.width +
	          (CHAR_WTH * (2 + string_length (data.label)));
              vertDistance: c_button.vertDistance;
              label: text;
	  }

      END VC buffer_display

    END VC component

  END VC Output

END VC Input
