/*--------------------------------------------------------------------------*\
|   
|  Name:  journaling file monitor
|
|  Description:  
|     formats and monitors journaling file.
|     
|
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|     Chris Cook
|     clc@sei.cmu.edu
|
|  Author:  
|     Chris Cook
|     clc@sei.cmu.edu  
|
|  Change History:
|     Novermber 15, 1988 : Module created.
$Log:	monitorc.c,v $
 * Revision 1.7  90/06/07  11:12:44  clc
 * Removed the call to ipb_start_recording.
 * 
 * Revision 1.6  90/06/06  17:48:46  clc
 * set up monitorc to ask for a filename, or use on given on the command line
 * allows ipb_format_playback to decide if it wants to use stdin.
 * 
 * Revision 1.5  90/04/25  00:36:18  clc
 * Added handshaking and additional input from dm
 * 
 * Revision 1.4  90/03/15  17:39:17  clc
 * improved the interrupt error message
 * 
 * Revision 1.3  90/01/26  21:02:14  clc
 * Sent over a fake id_num for the elements
 * 
 * Revision 1.2  90/01/26  19:12:46  clc
 * Added a check for flagged buffers being passed as data from ipb routines
 * 
 * Revision 1.1  90/01/23  16:37:50  clc
 * Initial revision
 * 
 * Revision 1.1  89/10/09  11:09:27  serpent
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*-------------------- Includes --------------------------------------------*/

#define memoryPack

#include <stdio.h>
#include <signal.h>
#include <strings.h>
#include <alistPack.h>
#include "serpent.h"
#include "ipb2.h"	/* Definition of ipb_submit_type */
#include "monitorc.h"   /* Shared Data Definition */

/* From ufa.h and u22.h respectively: */
string ufa_full_file_name ();
string u22_copy_string ();
/*-------------------- Constants -------------------------------------------*/

/*-------------------- Type Definitions ------------------------------------*/

/* Typedefs */
typedef struct entry {
    NODE	sdd_record;
    id_type	ID;
    ALIST	list;
} Alist_entry;

/* Copied from ipb2.h for reference. 
* typedef enum {
*	JRNL_FILE_NAME,
*	ANNOUNCE_HEADER,
*	DIALOGUE_NAME,
*	JRNL_MESSAGE,
*	ANNOUNCE_PARTN,
*	ILL_FILE_NAME,
*	MAILBOX_NAME,
*	ANNOUNCE_TRANS,
*	TIMESTAMP,
*	SENDER,
*	RECEIVER,
*	ANNOUNCE_END_OF_TRANS
*	ELEMENT_NAME,
*	CHANGE_TYPE_STR,
*	ELEMENT_ID,
*	COMP_NAME_TYPE,
*	COMPONENT_DATA,
*	PRINT_COMPONENT,
* } ipb_submit_type;
*/

typedef enum {
	STORE_PARTN,
	STORE_TRANS,
	STORE_ELEMENT,
	STORE_COMPONENT,
	GET_PARTN_ID,
	SEND_DATA
} do_type;

/*-------------------- Retained Data ---------------------------------------*/

/* Retained Data */
char *do_type_string[] = {
	"STORE_PARTN",
	"STORE_TRANS",
	"STORE_ELEMENT",
	"STORE_COMPONENT",
	"GET_PARTN_ID",
	"SEND_DATA"
	};

char *submit_type_string[] = {
	"JRNL_FILE_NAME",
	"ANNOUNCE_HEADER",
	"DIALOGUE_NAME",
	"JRNL_MESSAGE",
	"ANNOUNCE_PARTN",
	"ILL_FILE_NAME",
	"MAILBOX_NAME",
	"ANNOUNCE_TRANS",
	"TIMESTAMP",
	"SENDER",
	"RECEIVER",
	"ANNOUNCE_END_OF_TRANS",
	"ELEMENT_NAME",
	"CHANGE_TYPE_STR",
	"ELEMENT_ID",
	"COMP_NAME_TYPE",
	"COMPONENT_DATA",
	"PRINT_COMPONENT"
	};


/*-------------------- Global variables ------------------------------------*/

id_type		Input_ID;
input_sdd	Input;

/*-------------------- Processing routine and subroutines ------------------*/
/*- save_ID, ship, compare_to_mbx_name, maintain_data,
  - eval_transaction, and process */

#include <varargs.h>

int save_ID (ID_ptr, entry)
id_type *ID_ptr;
Alist_entry *entry;
{
    ((component_sdd *) entry -> sdd_record) -> element_id = *ID_ptr;
    return (1);
}

int ship (trans_ptr, sdd_str, entry)
transaction_type *trans_ptr;
char *sdd_str;
Alist_entry *entry;
{
    id_type ID;

    ID = add_shared_data (*trans_ptr, sdd_str, NULL, entry -> sdd_record);
    if (!strcmp (sdd_str, "element_sdd"))
	(void) for_all_alist (entry->list, &ID, save_ID);
    return (1);
}

int compare_to_mbx_name (str, entry)
char *str;
Alist_entry *entry;
{
    if (!strcmp (((participant_sdd *)(entry -> sdd_record)) -> mailbox_name,
		 str))
	return ((int) True);
    return ((int) False);
}

void maintain_data (what_to_do, ptr, ID_ptr)
do_type what_to_do;
NODE ptr;
id_type *ID_ptr;
{
    Alist_entry *entry;

    static ALIST partn_alist;
    static ALIST trans_alist;
    static ALIST element_alist;
    static ALIST comp_alist;
    static boolean first_time_through = True;

    if (first_time_through)
    {
	partn_alist = make_alist();
	trans_alist = make_alist();
	first_time_through = False;
    }
/*
    fprintf (stderr, "In maintain_data with: %s\n",
	do_type_string [(int) what_to_do]);
*/
    if (what_to_do == SEND_DATA)
    {
	int num;
	transaction_type trans;
	trans = start_transaction ();

	/* Get the Nth transaction */
	entry = (Alist_entry *) get_alist_pos (trans_alist,
		Input.transaction_num);

	if (entry == NULL) {
	    fprintf (stderr, "Transaction #%d not in database.\n",
		     Input.transaction_num);
	    return;
	}

	if (Input.element_num == 0)
	{
	    /* Ship all of the transaction elements to shared_data */
	    num = for_all_2alist (entry->list, &trans, "element_sdd", ship);
	    fprintf (stderr, "Shipped %d element(s) for Transaction #%d.\n",
                      num, Input.transaction_num);
	}
	else
	{
	    /* Get the Nth element */
	    entry = (Alist_entry *) get_alist_pos (entry->list,
		    Input.element_num);
	    if (entry == NULL) {
		fprintf (stderr,
			"Transaction #%d, Element #%d not in database\n", 
			Input.transaction_num,
			Input.element_num);
                return;
	    }

	    /* Ship all of the element components to shared_data */
	    num = for_all_2alist (entry->list, &trans, "component_sdd", ship);
	    fprintf (stderr,
              "Shipped %d component(s) for Transaction #%d, Element #%d.\n",
              num, Input.transaction_num,
	      Input.element_num);
	}

	Input.transaction_num = 0;
	Input.element_num = 0;
	put_shared_data (trans, Input_ID, "input_sdd", NULL,
		&Input);

	commit_transaction (trans);
	return;
    }

    if (ptr == NULL)
    {
	fprintf (stderr, "NULL pointer passed to maintain_data\n");
	return;
    }

    if (what_to_do == GET_PARTN_ID)
    {
	*ID_ptr = NULL;

	entry = (Alist_entry *) search_alist (partn_alist, ptr,
		compare_to_mbx_name);
	if (entry == NULL) {
	    fprintf (stderr, "Participant mailbox not in database: %s\n",
		     ptr);
	    return;
	}

	*ID_ptr = entry -> ID;
/*
	fprintf (stderr, "Returned ID: %d\n", *ID_ptr);
*/
	return;
    }
/*
    fprintf (stderr, "Creating Alist_entry");
*/
    entry = (Alist_entry *) make_node (sizeof (Alist_entry));
    if (entry == NULL) {
	fprintf (stderr, "Error in make_node for maintain_data\n");
	return;
    }
    entry -> sdd_record = ptr;
    entry -> ID = NULL;
    entry -> list = NULL;
    if (ID_ptr != NULL)
	entry -> ID = *ID_ptr;

    switch (what_to_do)
    {
	case STORE_PARTN:
	    add_to_atail (partn_alist, entry);
	    break;

	case STORE_TRANS:
	    entry -> list = element_alist = make_alist ();
	    add_to_atail (trans_alist, entry);
	    break;

	case STORE_ELEMENT:
	    entry -> list = comp_alist = make_alist ();
	    add_to_atail (element_alist, entry);
	    break;

	case STORE_COMPONENT:
	    add_to_atail (comp_alist, entry);
	    break;

	default:
	    fprintf (stderr, "Unknown do_type in maintain_data\n");
	    break;
    }

    return;
}

void eval_transaction (transaction)
transaction_type transaction;
{
    id_type id;

    if (get_status() == not_available)
	return;
/*
    fprintf (stderr, "Evaluating a transaction:\n");
*/
    Input.transaction_num = 0;
    Input.element_num = 0;
    id = get_first_changed_element (transaction);

    while (id != null_id)
    {
	if (id == Input_ID)
	   incorporate_changes (transaction, id, &Input);
	else
	   fprintf (stderr, "changed element ID is not Input_ID: %d", id);

	id = get_next_changed_element (transaction);
    }

    if (Input.transaction_num != 0)
	maintain_data (SEND_DATA, NULL, NULL);

    purge_transaction (transaction);
    return;
}

void process (info_type, format, args)
ipb_submit_type info_type;
char *format;
va_list args;
{
    char   *str;
    static  int	local_eindex, local_tcount = 0, step_count = 0;

    static  transaction_type 	Trans;
    static  id_type 		ID;

    static  scalars_sdd 	*Scalar_rec;
    static  participant_sdd 	*Partn_rec;
    static  transaction_sdd 	*Trans_rec;
    static  element_sdd 	*Element_rec;
    static  component_sdd 	*Comp_rec;

/*
    fprintf (stderr, "Entering process with: %s\n",
	submit_type_string[(int) info_type]);
    fflush (stderr);
*/

    switch (info_type) {

	case ANNOUNCE_HEADER: 
	case ANNOUNCE_TRANS: 
	case ANNOUNCE_PARTN: 
	case ANNOUNCE_END_OF_TRANS:
	    /* No data associated. */
	    break;

	case TIMESTAMP: 
	case CHANGE_TYPE_STR: 
	case ELEMENT_ID: 
	case COMPONENT_DATA: 
	    /* Data passed is not a straight string. */
	    break;

	default: 
	    /* Data passed is a string. */
	    str = va_arg (args, char *);
	    break;
    }

    switch (info_type) {

/* Journal file stuff */

	case JRNL_FILE_NAME: 
	    Trans = start_transaction ();
	    Scalar_rec = (scalars_sdd *) make_node (sizeof (scalars_sdd));
	    (void) strcpy (Scalar_rec->file_name, str);
	    break;

	case ANNOUNCE_HEADER: 
	    break;

	case DIALOGUE_NAME: 
	    (void) strcpy (Scalar_rec->dialogue_name, str);
	    break;

	case JRNL_MESSAGE: 
	    (void) strcpy (Scalar_rec->dialogue_message, str);

	    (void) add_shared_data (Trans, "scalars_sdd", NULL, Scalar_rec);
	    commit_transaction (Trans);
	    break;

/* Participant stuff */

	case ANNOUNCE_PARTN: 
	    Trans = start_transaction ();
	    Partn_rec = (participant_sdd *)
	 	    make_node (sizeof (participant_sdd));
	    break;

	case ILL_FILE_NAME: 
	    (void) strcpy (Partn_rec->ill_file_name, str);
	    break;

	case MAILBOX_NAME: 
	    (void) strcpy (Partn_rec->mailbox_name, str);

	    ID =
		add_shared_data (Trans, "participant_sdd", NULL, Partn_rec);
	    commit_transaction (Trans);

	    maintain_data (STORE_PARTN, Partn_rec, &ID);
	    break;

/* Transaction stuff */

	case ANNOUNCE_TRANS:
	    Trans = start_transaction ();
	    Trans_rec = (transaction_sdd *)
		    make_node (sizeof (transaction_sdd));

            local_eindex = 0;
	    break;

	case TIMESTAMP: 
	    (void) vsprintf (Trans_rec->timestamp, format, args);
	    break;

	case SENDER: 
	    maintain_data (GET_PARTN_ID, str, &ID);
	    Trans_rec->sender_id = ID;
	    break;

	case RECEIVER: 
	    maintain_data (GET_PARTN_ID, str, &ID);
	    Trans_rec->receiver_id = ID;

	    ID =
		add_shared_data (Trans, "transaction_sdd", NULL, Trans_rec);

	    maintain_data (STORE_TRANS, Trans_rec, &ID);
	    break;

	case ANNOUNCE_END_OF_TRANS:
                /* If STEPPING then either get the initial step_size, or    */
		/* decrement the step_count.  If the step_count is 0, send  */
		/* a STOP signal.  					    */
	    if (!strcmp (Input.speed, "STEP")) {
            /*  We assume that the dialogue enforces step_size > 0  */
                if (step_count < 1)
                    step_count = Input.step_size;
                if (--step_count == 0) {
		    (void) strcpy (Input.speed, "STOP");

		    put_shared_data (Trans, Input_ID, "input_sdd",
			    "speed", "STOP");
		}
	    }

	    commit_transaction (Trans);
            local_tcount++;
	    break;


/* Element stuff */

	case ELEMENT_NAME: 
	    Element_rec = (element_sdd *) make_node (sizeof (element_sdd));
	    /* Store the latest transaction ID */
	    Element_rec->trans_id = ID;
	    (void) strcpy (Element_rec->element_name, str);
	    break;

	case CHANGE_TYPE_STR: 
	    (void) strcpy (Element_rec->change_type_str, (format + 15));
	    break;

	case ELEMENT_ID: 
	    Element_rec->id_num = va_arg (args, id_type);

            Element_rec->e_index = ++local_eindex;
	    maintain_data (STORE_ELEMENT, Element_rec, NULL);
	    break;

/* Component stuff */

	case COMP_NAME_TYPE: 
	    Comp_rec = (component_sdd *) make_node (sizeof (component_sdd));
	    (void) strcpy (Comp_rec->component_name, str);

	    /* Get the second string, the type */
	    str = va_arg (args, char *);
	    (void) strcpy (Comp_rec->component_type, str);
	    break;

	case COMPONENT_DATA: 
	    Comp_rec->data_buffer = *(va_arg (args, idd_buffer_type *));
	    Comp_rec->data_buffer.body =
		(NODE) memcpy (make_node (Comp_rec->data_buffer.length),
			Comp_rec->data_buffer.body,
			Comp_rec->data_buffer.length);
	    if (!strcmp (format, "Buffer being passed"))
	    {
		Comp_rec->is_buffer = True;
                (void) strcpy (Comp_rec->buffer_type,
		    idd_get_type_string (Comp_rec->data_buffer.type));
	    }

	    maintain_data (STORE_COMPONENT, Comp_rec, NULL);
	    break;

	default: 
	    fprintf (stderr, "process: bad ipb_submit_type: %d\n",
		    info_type);
	    break;
    }

    eval_transaction (get_transaction_no_wait ());

/* Wait for graphics to catch up to output */
    while (local_tcount > Input.tcount)
        eval_transaction (get_transaction ());

    while (!strcmp (Input.speed, "STOP"))
	eval_transaction (get_transaction ());

/*
    fprintf (stderr, "Returning from process call\n");
    fflush (stderr);
*/
    return;
}


/*---------- Above is the process routine, which is referenced ---------
------------   by the ipb routines, and process' subroutines. --------*/

/*----------- Below are the main routine and its subroutines ---------*/

void interrupt_handler ()
{
   (void) fprintf (stderr, "\nMonitorc exiting.\n");
   (void) fflush (stdout);
   (void) fflush (stderr);
   (void) serpent_cleanup ();
   (void) exit (0);
}

main (argc, argv)
int argc;
char *argv[];
{
/*   Local variables  */
  transaction_type	transaction;

/*
   Set up the interrupt_handler and intitialize communication with serpent.
*/
  signal (SIGINT, interrupt_handler);
  serpent_init (MAIL_BOX, ILL_FILE);

  printf ("Welcome to the Serpent monitor application (Monitorc).\n");

/*
   Send over the input shared data.  Changes to Input drive
   the actions of this application.
*/
  transaction = start_transaction ();
    (void) strcpy (Input.speed, "STEP");
    Input.step_size = 1;
    Input.transaction_num = 0;
    Input.element_num = 0;
    Input.tcount = 0;

    *(Input.file_name) = '\0';
    Input.ask_for_file_name = True;

    if (argc > 1)
    {
      (void) strncpy (Input.file_name, argv[1], 255);
      Input.file_name[255] = '/0';
      Input.ask_for_file_name = False;
    }

  Input_ID =
    add_shared_data (transaction, "input_sdd", NULL, &Input);
  commit_transaction (transaction);

/*
   If a file was not given on the command line, and we are asking the user
   to enter a filename, then wait for the user's input.
*/
  if (Input.ask_for_file_name)
    eval_transaction (get_transaction ());
  ipb_format_playback_to_subroutine (Input.file_name, process);

/*
   If format_playback can't deal with the name, ask the user for a valid name
*/
  while (get_status() == bad_parameter)
  {
    transaction = start_transaction ();
    Input.ask_for_file_name = True;
    put_shared_data (transaction, Input_ID, "input_sdd", NULL, &Input);

    commit_transaction (transaction);
    eval_transaction (get_transaction ());
    ipb_format_playback_to_subroutine (Input.file_name, process);
  }

  if (get_status() && get_status() != not_available)
    print_status ("Monitorc: Error formatting journal file");

  (void) strcpy (Input.speed, "DONE");

  transaction = start_transaction ();
  put_shared_data (transaction, Input_ID, "input_sdd", "speed", "DONE");
  commit_transaction (transaction);

/*
   Execute all successive input, allowing the user to decide when to exit.
*/

  while (True)
    eval_transaction (get_transaction());

/*
   The interrupt_handler should take care of cleanup and exiting.
*/
}
