#include "sat.ill"
#include "proto.ill"

|||
/*
This is the slang program to demonstrate a simple prototyper for command 
widgets.  It allows the end user to create command widgets and manipulate
them based on textual input into a table.

  
Initially created are a form upon which the command widgets are positioned
and a button which when pressed will create another command widget to be
position.

There is a view controller template which is instantiated for each command
widget created.  It controls the input of the attributes and the assigning
of those attributes to the command widgets displayed for modification.

This slang program is not very general.  That is, the addition of something
beside a command widget would require essentially replicating the entire
program.  It is intended as an example to demonstrate the use of Serpent
with an application.
*/

/*--------------------------------------------------------------------------*\
|  two objects are defined here.  
|   1. a form for the background of all of the objects
|   2. a button to create a new command widget for management
\*--------------------------------------------------------------------------*/

   OBJECTS:

        main_background:     XawBboard
        {
         ATTRIBUTES:
           width:  765;
           height: 645;
         }   

	create_btn: XawCommand
	{
         ATTRIBUTES:
           parent: main_background;
	   label: "Create command widget";
	   width: 140;
	   height: 20;
           vertDistance: 10;
           horizDistance: 600;

         METHODS:
          notify: {
            create_element("command", "DM_BOX");
          }
	}

	quit_widget: XawCommand
	{
         ATTRIBUTES:
           parent: main_background;
	   label: "Quit";
	   width: 140;
	   height: 20;
           vertDistance: 600;
           horizDistance: 600;

         METHODS:
          notify: {
            exit();
          }
	}

/*-------------------------------------------------------------------------
|   The "new" function will create a view controller instance whenever
|   a new instance of the command element is added to shared data.
-------------------------------------------------------------------*/

  VC: command_instance

    CREATION CONDITION:  (new("command"))

/*
    the components being set represent those variables which can be
    modified through the attribute form.  The settings are default values to
    be modified.  The modifications are automatically
    displayed in the realization window
*/
    VARIABLES:

     send_flag : 0;
     local_variable : 1;
     command_x : 0;
     command_y : 0;
     command_w : 100;
     command_h : 20;

     OBJECTS:

/*-----------------------------------------------------------------------
     The following objects form a table which allows the end user
     to input attributes textually
------------------------------------------------------------------------*/

        att_background:     XawBboard
        {
         ATTRIBUTES:
           width:  200;
           height: 200;
         }

    /*----------------------------------------------------------------------*\
    | the ok_button signals the text widgets with their attributes
    | to send their current values
    \*----------------------------------------------------------------------*/

      ok_button: XawCommand
	{
         ATTRIBUTES:
           parent: att_background;  
	   justify: 1;	/* center */
	   label: "OK";
	   width:  50;
	   height: 20;
           fromHoriz: NULL;
           horizDistance: 75;
           fromVert: NULL;
           vertDistance: 160;

         METHODS:
          notify: {
            send_flag := 1;
	  }
       }

    /*----------------------------------------------------------------------*\
    |  The next two objects represent an entry in the
    |  table which provides the text: "text" and the value
    |  This allows modification of the text within the constructed widget
    \*----------------------------------------------------------------------*/

      att_text_label: XawLabel
       {
        ATTRIBUTES:

         parent: att_background;
         justify: 0;	/* left */
         label: "text:";
         fromHoriz: NULL;
         fromVert: NULL;
         borderColor: "white";
         vertDistance: 0;
         horizDistance: 0;
         height: 25;
         width: 50;
       }

      att_text_value: XawText
       {
        ATTRIBUTES:

         parent: att_background;
         width: 175;
         height: 25;
         fromHoriz: NULL;
         fromVert: NULL;
         borderColor: "white";
         vertDistance: 5;
         horizDistance: 50;
         editType: 2;
         sendBuffer: send_flag;   /* value of 1 is message to send text*/
         textBuffer: "<label>";

        METHODS:
         send: {
           command.text := textBuffer;
           send_flag := 0;
          }
        }

    /*----------------------------------------------------------------------*\
    |  The next two objects represent an entry in the
    |  table which provides the text: "height" and the value
    |  This allows modification of the size of the constructed widget
    \*----------------------------------------------------------------------*/

      att_height_label: XawLabel
       {
        ATTRIBUTES:

         parent: att_background;
         justify: 0;
         label: "height        :";
         fromHoriz: NULL;
         fromVert: NULL;
         borderColor: "white";
         vertDistance: 25;
         horizDistance: 0;
         height: 25;
         width: 105;
       }

      att_height_value: XawText
       {
        ATTRIBUTES:

         parent: att_background;
         width: 120;
         height: 25;
         fromHoriz: NULL;
         fromVert: NULL;
         borderColor: "white";
         vertDistance: 30;
         horizDistance: 105;
         editType: 2;
         sendBuffer: send_flag;   /* value of 1 is message to send text*/
         textBuffer: command_h ;

        METHODS:
         send: {
           command.height := textBuffer;
           send_flag := 0;
          }
        }

    /*----------------------------------------------------------------------*\
    |  The next two objects represent an entry in the
    |  table which provides the text: "width" and the value
    |  This allows modification of the size of the constructed widget
    \*----------------------------------------------------------------------*/

      att_width_label: XawLabel
       {
        ATTRIBUTES:

         parent: att_background;
         justify: 0;
         label: "width         :";
         fromHoriz: NULL;
         fromVert: NULL;
         borderColor: "white";
         vertDistance: 50;
         horizDistance: 0;
         height: 25;
         width: 105;
       }

      att_width_value: XawText
       {
        ATTRIBUTES:

         parent: att_background;
         width: 120;
         height: 25;
         fromHoriz: NULL;
         fromVert: NULL;
         borderColor: "white";
         vertDistance: 55;
         horizDistance: 105;
         editType: 2;
         sendBuffer: send_flag;   /* value of 1 is message to send text*/
         textBuffer: command_w;

        METHODS:
         send: {
           command.width := textBuffer;
           send_flag := 0;
          }
        }

    /*----------------------------------------------------------------------*\
    |  The next two objects represent an entry in the
    |  table which provides the text: "horizDistance" and the value
    |  This allows modification of the position of the constructed widget
    \*----------------------------------------------------------------------*/

      att_horiz_label: XawLabel
       {
        ATTRIBUTES:

         parent: att_background;
         justify: 0;
         label: "x             :";
         fromHoriz: NULL;
         fromVert: NULL;
         borderColor: "white";
         vertDistance: 75;
         horizDistance: 0;
         height: 25;
         width: 105;
       }

      att_horiz_value: XawText
       {
        ATTRIBUTES:

         parent: att_background;
         width: 120;
         height: 25;
         fromHoriz: NULL;
         fromVert: NULL;
         borderColor: "white";
         vertDistance: 80;
         horizDistance: 105;
         editType: 2;
         sendBuffer: send_flag;   /* value of 1 is message to send text*/
         textBuffer: command_x;
	 length: 10 ;

        METHODS:
         send: {
           command.horiz_distance := textBuffer;
           send_flag := 0;
          }
        }

    /*----------------------------------------------------------------------*\
    |  The next two objects represent an entry in the
    |  table which provides the text: "vertDistance" and the value
    |  This allows modification of the position of the constructed widget
    \*----------------------------------------------------------------------*/

      att_vert_label: XawLabel
       {
        ATTRIBUTES:

         parent: att_background;
         justify: 0;
         label: "y             :";
         fromHoriz: NULL;
         fromVert: NULL;
         borderColor: "white";
         vertDistance: 100;
         horizDistance : 0;
         height: 25;
         width: 105;
       }

      att_vert_value: XawText
       {
        ATTRIBUTES:

         parent: att_background;
         width: 120;
         height: 25;
         fromHoriz: NULL;
         fromVert: NULL;
         borderColor: "white";
         vertDistance: 105;
         horizDistance: 105;
         editType: 2;
         sendBuffer: send_flag;   /* value of 1 is message to send text*/
         textBuffer: command_y;

        METHODS:
         send: {
           command.vert_distance := textBuffer;
           send_flag := 0;
         }
       }

    /*----------------------------------------------------------------------*\
    |  The following object actually displays the
    |  widget which is created and which has
    |  attributes specified from the table
    \*----------------------------------------------------------------------*/

    VC: XawCommand_instance

      Creation Condition: (local_variable = 1)

      Objects:

	command_real: XawCommand
	{
         ATTRIBUTES:

           parent: main_background;
  	   label: att_text_value.textBuffer;
           width: att_width_value.textBuffer;
           height: att_height_value.textBuffer;
	   justify: 1;		/* center */
           vertDistance: att_vert_value.textBuffer;
           horizDistance: att_horiz_value.textBuffer;
           x: horizDistance;
           y: vertDistance;
           allowUserResize: TRUE;
           allowUserMove: TRUE;

        METHODS:
	  move: {
            command_x := x;
            command_y := y;
          }
	  resize: {
            command_x := x;
            command_y := y;
            command_w := width;
            command_h := height;
          }
	}
    ENDVC XawCommand_instance

  ENDVC command_instance



