/*
 * Clock demo - second pass
 *
 * This demo program shows the clock demo in its second stage of development,
 * where the controls actually do things, but the time of day and date fields
 * are still hardwired.  There is no application program yet, but there is
 * a measure of dialogue interaction. Some of the dialogue interaction is
 * handled through an external routine, which formats the hours field to two
 * character strings with leading zeroes.
 */
#include "smo.ill"

|||

#include "glueXm.h"

EXTERNALS :
    string i2str(integer);

VARIABLES :
    hours_text : "13";
    minutes_text : "05";
    DatePlus : {
	IF (Show_Date.set) THEN
	    DatePlus := 50;
	ELSE
	    DatePlus := 0;
	ENDIF;
	}
    SetPlus : {
	IF (Set_Time.set) THEN
	    SetPlus := 12;
	ELSE
	    SetPlus := 0;
	ENDIF;
	}
    MilitaryTime : true;

OBJECTS :

    shell : XmApplicationShell {
	ATTRIBUTES :
	    height : palette.height;
	    width : palette.width;
	    allowShellResize : true;
	    }

    palette : XmBulletinBoard {
	ATTRIBUTES :
	    parent : shell;
	    height : 150 + DatePlus;
	    width : 375;
	    }

    quit : XmPushButton {
	ATTRIBUTES :
	    parent : palette;
	    labelstring : "Quit";
	    borderwidth : 1;
	    shadowthickness : 3;
	    height : 35;
	    width : 40;
	    x : palette.width - width - 10;
	    y : palette.height - height - 10;
	METHODS :
	    notify : { exit(); }
	    }

    FormatSelector : XmRowColumn {
	ATTRIBUTES :
	    parent : palette;
	    height : 100;
	    width : 200;
	    x : 95;
	    y : 65;
	    packing : XmPACK_COLUMN;
	    numcolumns : 1;
	    orientation : XmVERTICAL;
	    borderwidth : 0;
	    radiobehavior : true;
	    radioalwaysone : true;
	    }

    Button1 : XmToggleButton {
	ATTRIBUTES :
	    parent : FormatSelector;
	    labelString : "12 Hour";
	    indicatortype : XmONE_OF_MANY;
	    width : 75;
	    height : 30;
	    set : false;
	METHODS:
	    toggle : { MilitaryTime := not set; }
	    }

    Button2 : XmToggleButton {
	ATTRIBUTES :
	    parent : FormatSelector;
	    labelString : "24 Hour";
	    indicatortype : XmONE_OF_MANY;
	    width : 75;
	    height : 30;
	    set : true;
	METHODS:
	    toggle : { MilitaryTime := set; }
	    }

    Show_Date : XmToggleButton {
	ATTRIBUTES :
	    parent : palette;
	    labelString : "Show Date";
	    x : 200;
	    y : 80;
	    width : 80;
	    height : 20;
	    set : false;
	    }
    
    Set_Time : XmToggleButton {
	ATTRIBUTES :
	    parent : palette;
	    labelString : "Set";
	    x : 320;
	    y : 20;
	    width : 40;
	    height : 20;
	    set : false;
	    }

    hours : XmLabel {
	ATTRIBUTES :
	    parent : palette;
	    fontlist : "*-courier-bold-r-*-*-34-*";
	    labelstring : {
		IF (MilitaryTime) THEN
		    labelstring := i2str(hours_text);
		ELSIF (hours_text = 0) THEN
		    labelstring := 12;
		ELSIF (hours_text <= 12) THEN
		    labelstring := hours_text;
		ELSE
		    labelstring := hours_text - 12;
		ENDIF;
		}
	    recomputesize : false;
	    alignment : XmALIGNMENT_END;
	    height : 25;
	    width : 40;
	    x : 115;
	    y : 15;
	    }

    colon : XmLabel {
	ATTRIBUTES :
	    parent : palette;
	    fontlist : "*-courier-bold-r-*-*-34-*";
	    labelstring : ":";
	    recomputesize : false;
	    height : 25;
	    width : 15;
	    x : 155;
	    y : 15;
	    }

    minutes : XmLabel {
	ATTRIBUTES :
	    parent : palette;
	    fontlist : "*-courier-bold-r-*-*-34-*";
	    labelstring : i2str(minutes_text);
	    recomputesize : false;
	    height : 25;
	    width : 40;
	    x : 170;
	    y : 15;
	    }

    /*=====================================================*/

    VC : AM_PM
	CREATION CONDITION : (not MilitaryTime)

	OBJECTS :
	    AM_PM : XmLabel {
		ATTRIBUTES :
		    parent : palette;
		    fontlist : "*-courier-bold-r-*-*-24-*";
		    labelstring : {
			IF (hours_text >= 12) THEN
			    labelstring := "PM";
			ELSE
			    labelstring := "AM";
			ENDIF;
			}
		    recomputesize : false;
		    height : 23;
		    width : 40;
		    x : 215 + SetPlus;
		    y : 17;
		    }
	ENDVC AM_PM

    /*=====================================================*/

    VC : today
	CREATION CONDITION : (Show_Date.set)

	OBJECTS:
	    date : XmLabel {
		ATTRIBUTES :
		    parent : palette;
		    fontlist : "*-times-bold-r-*-*-20-*";
		    labelstring : "February 12, 1991";
		    recomputesize : false;
		    borderwidth : 1;
		    height : 35;
		    width : 200;
		    x : 85;
		    y : 130;
		    }
	ENDVC today
    
    /*=====================================================*/

    VC : set_buttons
	CREATION CONDITION : (Set_Time.set)

	OBJECTS :
	    hrs_up : XmArrowButton {
		ATTRIBUTES :
		    parent : palette;
		    arrowdirection : XmARROW_UP;
		    shadowthickness : 0;
		    height : 12;
		    width : 12;
		    x : 100;
		    y : 15;
		METHODS :
		    notify : {
			hours_text := hours_text + 1;
			IF (hours_text > 23) THEN
			    hours_text := 0;
			ENDIF;
			}
		    }

	    hrs_down : XmArrowButton {
		ATTRIBUTES :
		    parent : palette;
		    arrowdirection : XmARROW_DOWN;
		    shadowthickness : 0;
		    height : 12;
		    width : 12;
		    x : 100;
		    y : 30;
		METHODS :
		    notify : {
			hours_text := hours_text - 1;
			IF (hours_text < 0) THEN
			    hours_text := "23";
			ENDIF;
			}
		    }

	    mins_up : XmArrowButton {
		ATTRIBUTES :
		    parent : palette;
		    arrowdirection : XmARROW_UP;
		    shadowthickness : 0;
		    height : 12;
		    width : 12;
		    x : 213;
		    y : 15;
		METHODS :
		    notify : {
			minutes_text := minutes_text + 1;
			IF (minutes_text > 59) THEN
			    minutes_text := 0;
			ENDIF;
			}
		    }

	    mins_down : XmArrowButton {
		ATTRIBUTES :
		    parent : palette;
		    arrowdirection : XmARROW_DOWN;
		    shadowthickness : 0;
		    height : 12;
		    width : 12;
		    x : 213;
		    y : 30;
		METHODS :
		    notify : {
			minutes_text := minutes_text - 1;
			IF (minutes_text < 0) THEN
			    minutes_text := "59";
			ENDIF;
			}
		    }
	ENDVC set_buttons
