#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include "treePack.h"
#include "fonts.h"
#include "serpent.h"

#define MAXNAMES 	500
#define PAT_LEN		200

static char **Xfont_list;
static TREE font_tree;
static int Xfont_count;
static char **font_names;
static char **font_sizes;
static char **font_slants;
static char **font_weights;
static int name_len;
static int size_len;
static int slant_len;
static int weight_len;
static TREE name_tree;
static TREE size_tree;
static TREE slant_tree;
static TREE weight_tree;
static Display *display;

static short Initialized = 0;

static char *font_pattern = "-*-%s-%s-%s-*-*-%s-*-*-*-*-*-*-*";
static char *check_pattern = "-%s-%s-%s-%s-";

static int hash(s)
char *s;
{
    return *s % 26;
}

static int compare(s1, s2)
char *s1, *s2;
{
    if (*s1 >= '0' && *s1 <= '9' && *s2 >= '0' && *s2 <= '9')
	return atoi(s1) - atoi(s2);
    else
	return strcmp (s1, s2);
}

static void addit(what, str)
char **what;
char *str;
{
    while (*what)
	what++;
    *what = str;
}

static char *mystrtok(str, sepset)
char *str, *sepset;
{
    register char *p, *r;
    static char	*savept;

    /*first or subsequent call*/
    p = (str == NULL)? savept: str;
    if (p == 0)					/* no tokens remaining */
	return(NULL);
    if ((r=strpbrk(p, sepset)) == NULL)		/* move past token */
	savept = 0;				/* last token */
    else {
	*r = '\0';
	savept = ++r;
	}
    return(p);
}

static void collect_fonts()
{
    char all_pattern[PAT_LEN];
    char tmp[PAT_LEN];
    int i;
    char *cp, *xp, *name_p, *size_p, *slant_p, *weight_p;
    
    Initialized++;
    display = XOpenDisplay(NULL);
    (void) sprintf(all_pattern, font_pattern, "*", "*", "*", "*");
    for (i=1; /*null*/; i++) {
	Xfont_list = XListFonts(display, all_pattern, i*MAXNAMES, &Xfont_count);
	if (Xfont_count <= i*MAXNAMES)
	    break;			/* It fit - go on... */
	else			
	    XFreeFontNames (Xfont_list);	/* Free incomplete list, try again */
	}
    font_tree = make_bintree (compare);
    name_tree = make_bintree (compare);
    size_tree = make_bintree (compare);
    slant_tree = make_bintree (compare);
    weight_tree = make_bintree (compare);
    for (i=0; i<Xfont_count; i++) {
	(void) strcpy (tmp, Xfont_list[i]);
	(void) mystrtok(tmp, "-");				/* Dummy */
	(void) mystrtok(NULL, "-");				/* Foundry */
	cp = mystrtok(NULL, "-");				/* Family */
	name_p = strcpy (make_node (strlen(cp)+1), cp);
	if (*cp != '\0')
	    (void) add_unique_to_bintree (name_tree, name_p, name_p);
	cp = mystrtok(NULL, "-");				/* Weight */
	weight_p = strcpy (make_node (strlen(cp)+1), cp);
	if (*cp != '\0')
	    (void) add_unique_to_bintree (weight_tree, weight_p, weight_p);
	cp = mystrtok(NULL, "-");				/* Slant */
	slant_p = strcpy (make_node (strlen(cp)+1), cp);
	if (*cp != '\0')
	    (void) add_unique_to_bintree (slant_tree, slant_p, slant_p);
	(void) mystrtok(NULL, "-");				/* Width */
	(void) mystrtok(NULL, "-");				/* Add Style */
	cp = mystrtok(NULL, "-");				/* Pixel Size */
	size_p = strcpy (make_node (strlen(cp)+1), cp);
	if (*cp != '\0')
	    (void) add_unique_to_bintree (size_tree, size_p, size_p);
	xp = make_node (strlen(name_p)+
			strlen(size_p)+
			strlen(weight_p)+
			strlen(slant_p)+
			5+			/* 5 '-'s in the string */
			1);			/* 1 trailing null */
	(void) sprintf (xp, check_pattern, name_p, weight_p, slant_p, size_p);
	(void) add_unique_to_bintree (font_tree, xp, xp);
	}
    name_len = size_of_bintree (name_tree)+1;
    font_names = (char **)make_node(name_len*sizeof(cp));
    font_names[0] = "*";
    for (i=1; i<size_of_bintree(name_tree); i++)
	font_names[i] = NULL;
    for_all_bintree (name_tree, font_names, addit);
    name_len = size_of_bintree (name_tree)+1;
    (void) free_bintree (name_tree, NOT_NODES);

    weight_len = size_of_bintree (weight_tree)+1;
    font_weights = (char **)make_node(weight_len*sizeof(cp));
    font_weights[0] = "*";
    for (i=1; i<size_of_bintree(weight_tree); i++)
	font_weights[i] = NULL;
    for_all_bintree (weight_tree, font_weights, addit);
    (void) free_bintree (weight_tree, NOT_NODES);

    slant_len = size_of_bintree (slant_tree)+1;
    font_slants = (char **)make_node(slant_len*sizeof(cp));
    font_slants[0] = "*";
    for (i=1; i<size_of_bintree(slant_tree); i++)
	font_slants[i] = NULL;
    for_all_bintree (slant_tree, font_slants, addit);
    (void) free_bintree (slant_tree, NOT_NODES);

    size_len = size_of_bintree (size_tree)+1;
    font_sizes = (char **)make_node(size_len*sizeof(cp));
    font_sizes[0] = "*";
    for (i=1; i<size_of_bintree(size_tree); i++)
	font_sizes[i] = NULL;
    for_all_bintree (size_tree, font_sizes, addit);
    (void) free_bintree (size_tree, NOT_NODES);

    XFreeFontNames (Xfont_list);

    Xfont_count = size_of_bintree (font_tree);
    Xfont_list = (char **)make_node(Xfont_count*sizeof(cp));
    for (i=0; i<Xfont_count; i++)
	Xfont_list[i] = NULL;
    for_all_bintree (font_tree, Xfont_list, addit);
    (void) free_bintree (font_tree, NOT_NODES);
}

int howmany_of(type)
int type;
{
    if (!Initialized)
	collect_fonts();
    switch (type) {
	case FONT_NAME:
	    return name_len;
	case FONT_SIZE:
	    return size_len;
	case FONT_SLANT:
	    return slant_len;
	case FONT_WEIGHT:
	    return weight_len;
	default:
	    fprintf(stderr, "Auggghhh!! - howmany_of()\n");
	    return 0;
	}
}

string fetch_val(type,idx)
int type, idx;
{
    if (!Initialized)
	collect_fonts();
    switch (type) {
	case FONT_NAME:
	    return idx < name_len ? font_names[idx] : "";
	case FONT_SIZE:
	    return idx < size_len ? font_sizes[idx] : "";
	case FONT_SLANT:
	    return idx < slant_len ? font_slants[idx] : "";
	case FONT_WEIGHT:
	    return idx < weight_len ? font_weights[idx] : "";
	default:
	    fprintf(stderr, "Auggghhh!! - fetch_val()\n");
	    return "";
	}
}

boolean font_exists(name,size,slant,weight)
string name;
string size;
string slant;
string weight;
{
    char test_pattern[200];
    char *err;
#ifdef hpux
    char *regcmp (), *regex (), *dummy;
#else
    char *re_comp ();
#endif
    int i;

    if (!Initialized)
	collect_fonts();
    (void) sprintf (test_pattern, check_pattern,
			strcmp (name, "*") ? name : "[^-]*" ,
			strcmp (weight, "*") ? weight : "[^-]*" ,
			strcmp (slant, "*") ? slant : "[^-]*" ,
			strcmp (size, "*") ? size : "[^-]*" );
#ifdef hpux
    err = regcmp (test_pattern, 0);
#else
    err = re_comp(test_pattern);
#endif
    for (i=0; i<Xfont_count; i++)
#ifdef hpux
	if (regex (err, Xfont_list [i], dummy))
	    {
	    free (err);
	    return (true);
	    }
	free (err);
#else
	if (re_exec(Xfont_list[i]))
	    return (true);
#endif
    return (false);
}

string font_str(name,size,slant,weight)
string name;
string size;
string slant;
string weight;
{
    static char out_pattern[200];

    (void) sprintf (out_pattern, font_pattern, name, weight, slant, size);
    return out_pattern;
}

int int_max(l,r)
int l,r;
{
    return (l>r ? l : r);
}
