#include "smo.ill"
#include "group.ill"

||| 

/* This is the slang program to demonstrate use of Serpent in grouping
objects for an interactive editor.  

Initially created is a Form with a menu of Commands.

There is a view controller template which is instantiated for each Command
widget created.  It controls the position of that particular Command widget

There is another view controller template which is instantiated for each
group created.

*/


#define LEFT 0
#define CENTER 1
#define RIGHT 2

EXTERNALS:  string cus_genname (string);
EXTERNALS:  void ipb_start_recording (string, string);
EXTERNALS:  void printf (string, string);

/*--------------------------------------------------------------------------*\
|  several objects are defined here.  
|   1. a Form for the background of all of the objects
|   2. buttons to create a new component
|   3. buttons for Commands
\*--------------------------------------------------------------------------*/
  Variables:

    XmCommand_menu: 625;
    group_menu_x: 550;
    current_XmCommand;
    init_x;
    init_y;
    current_id: NULL;
    group_y_offset: 5;

   OBJECTS:

        main_background:     XmBulletinBoard
        {ATTRIBUTES:
           width:  765;
           height: 645;
           resizePolicy: 0;
/*           background: "white"; */
           selectedX: 0;
           selectedY: 0;

         Methods:

          select: {
            if (current_XmCommand = "create_instance") then
              current_id := create_sd_instance(
                "instance_sdd",
                "DM_BOX");


              put_component_value(
                current_id,
                "instance_name",
                cus_genname("CW"));


              put_component_value(
                current_id,
                "group_id",
                NULL);

              init_x := selectedX;
              init_y := selectedY;
             end if;

           }
         }   


      create_instance_object: XmPushButton
	{ATTRIBUTES:
           parent: main_background;
  
	   labelString: "Create Instance";

	   width: 120;
	   height: 30;

           x: XmCommand_menu;
           y: 5;

         Methods:

	  notify: {
            current_XmCommand := "create_instance";
          }
         }

	create_group_object: XmPushButton
	{ATTRIBUTES:
           parent: main_background;
  
	   labelString: "Create Group";

	   width: 120;
	   height: 30;

           x: XmCommand_menu;
           y: 45;

         METHODS:

          notify: {
            current_id := create_sd_instance(
              "group_sdd",
              "DM_BOX");

            put_component_value(
              current_id,
              "x_offset",
              0);

            put_component_value(
              current_id,
              "y_offset",
              0);

            put_component_value(
              current_id,
              "group_name",
              cus_genname("Group"));

            put_component_value(
              current_id,
              "group_color",
              "white");

            put_component_value(
              current_id,
              "exist",
              TRUE);

	}

      }

      delete_instance_object: XmPushButton
	{ATTRIBUTES:
           parent: main_background;
  
	   labelString: "Delete Instance";

	   width: 120;
	   height: 30;

           x: XmCommand_menu;
           y: 80;
 
         METHODS:

          notify: {
           current_XmCommand := "delete_instance";
          }
        }

      delete_group_object: XmPushButton
	{ATTRIBUTES:
           parent: main_background;
  
	   labelString: "Delete Group";

	   width: 120;
	   height: 30;

           x: XmCommand_menu;
           y: 115;

        Methods:

         notify: {
           current_XmCommand := "delete_group";
          }
         }

      add_instance_to_group_object: XmPushButton
	{ATTRIBUTES:
           parent: main_background;
  
	   labelString: "Add to Group";

	   width: 120;
	   height: 30;

           x: XmCommand_menu;
           y: 150;

        Methods:

         notify: {
          current_XmCommand := "add_instance_to_group";
          }
         }

      delete_instance_from_group_object: XmPushButton
	{ATTRIBUTES:
           parent: main_background;
  
	   labelString: "Delete From Group";

	   width: 120;
	   height: 30;

           x: XmCommand_menu;
           y: 185;

        Methods:

         notify: {
          current_XmCommand := "delete_from_group";
          }
         }

      Toggle_group_object: XmPushButton
	{ATTRIBUTES:
           parent: main_background;
  
	   labelString: "Toggle Group";

	   width: 120;
	   height: 30;

           x: XmCommand_menu;
           y: 220;

        Methods:

         notify: {
          current_XmCommand := "Toggle_group";
          }
         }

      exit_object: XmPushButton
	{ATTRIBUTES:
           parent: main_background;
  
	   labelString: "Quit";

	   width: 120;
	   height: 30;

           x: XmCommand_menu;
           y: 255;

        Methods:

         notify: {
            exit();
          }
         }


#include "instance.sl"

#include "group_ins.sl"





