#include "smo.ill"
#include "layout.ill"


||| 

/* This is a slang program to do the layout for an arbitrary tree.  It uses
the layout algorithm published by Sven Moen in the July 1990, IEEE Software.

This layout algorithm is implemented using several C routines that provide the
actual layout computations.

For testing and demonstration purposes, the program begins with a Form with
two items on the menu.

There is a view controller template which is instantiated for each Command
widget created to control each item of the tree.  This and the called C
routines actually do the layout.

*/



EXTERNALS:  void ipb_start_recording (string, string);
            integer add_new_leaf(id, integer, integer, integer, integer);
            void remove_leaf(id);
            string cus_genname (string);
            void layout_tree(id);
            void unlayout_tree(id);
            void dump_tree(id);
EXTERNALS:  void printf (string, id);

/*--------------------------------------------------------------------------*\
|  several objects are defined here.  
|   1. a XmForm for the background of all of the objects
|   2. button to create a new component
|   3. button to delete a component
\*--------------------------------------------------------------------------*/
  Variables:

    Command_menu: 705;
    current_command;
    current_sdd;
    parent_node;
    parent_distance: 30;
    border: 10;
    frame_height: 645;
    frame_width: 850;
    root_node;
    root_y: frame_height / 2;
    root_x: 0;

   OBJECTS:

        main_background:     XmBulletinBoard
        {ATTRIBUTES:
           width:  frame_width;
           height: frame_height;
           marginWidth: 3;
           resizePolicy: 0;
	}

      create_root_object: XmPushButton
	{ATTRIBUTES:
           parent: main_background;
           fontList: "9x15bold";  
	   labelString: "Create root";
	   width: 120;
	   height: 30;
           x: Command_menu;
           y: 5;

         Methods:

	  notify: {
            current_command := "create";
            current_sdd := create_sd_instance(
		"tree_element", "DM_BOX");
            put_sd_value(current_sdd, "parent", NULL);
            put_sd_value(current_sdd, "child", NULL);
            put_sd_value(current_sdd, "predecessor", NULL);
            put_sd_value(current_sdd, "sibling", NULL);

            put_sd_value(current_sdd,"x_absolute",0);
            put_sd_value(current_sdd,"y_absolute",0);
            put_sd_value(current_sdd,"x_offset",0);
            put_sd_value(current_sdd,"y_offset",0);

            put_sd_value(current_sdd,"create_flag",TRUE);

            parent_node := NULL;
            root_node := current_sdd;
          }
         }

      create_object: XmPushButton
	{ATTRIBUTES:
           parent: main_background;
           fontList: "9x15bold";  
	   labelString: "Create";
	   height: 30;
  	   width : { if (current_command = "create") then
                       width := 135;
                     else
                       width := 120;
                     end if;
                   }
           x: Command_menu;
           y: 45;

         Methods:

	  notify: {
            current_command := "create";
          }
         }

	delete_object: XmPushButton
	{ATTRIBUTES:
           parent: main_background;
           fontList: "9x15bold";    
	   labelString: "Delete";
  	   width : { if (current_command = "delete") then
                       width := 135;
                     else
                       width := 120;
                     end if;
                   }
	   height: 30;
           x: Command_menu;
           y: 80;

         METHODS:

          notify: {
            current_command := "delete";
	}
      }


      exit_object: XmPushButton
	{ATTRIBUTES:
           parent: main_background;
           fontList: "9x15bold";    
	   labelString: "Quit";
	   width: 120;
	   height: 30;
           x: Command_menu;
           y: 115;

        Methods:

         notify: {
            exit();
          }
         }
/*On Create :
{ 
  ipb_start_recording("layout.rec","roger");
}
*/

/*--------------------------------------------------------------------------*\
|  Name: element_vc
|
|  Description:
|    The element_vc instantiates on a new element in the tree to
|      be laid out
\*--------------------------------------------------------------------------*/
VC: element_vc

  Creation Condition: (new("tree_element") and tree_element.create_flag)

  Variables:

     abs_x: {if (local_current_sdd <> UNDEFINED) then
                if (tree_element.predecessor = NULL) then
                    put_sd_value(local_current_sdd,"x_absolute",0);

                 else

                 put_sd_value( local_current_sdd,"x_absolute",
                    get_sd_value(local_current_sdd,"x_offset") +
                    get_sd_value(tree_element.predecessor,"x_absolute"));
                 end if;
               end if;
              }

     abs_y: {if (local_current_sdd <> UNDEFINED) then
                if (tree_element.predecessor = NULL) then
                   tree_element.y_absolute := root_y;
                   tree_element.x_absolute := root_x;

                else

                 put_sd_value( local_current_sdd,"y_absolute",
                    get_sd_value(local_current_sdd,"y_offset") +
                    get_sd_value(tree_element.predecessor,"y_absolute"));

                 end if;
              end if;
             }

     local_current_sdd: UNDEFINED;
     temp;
     temp1;
     diff_x;
     diff_y;
     parent_line_flag: {if (tree_element.parent = NULL) then
                         parent_line_flag := FALSE;
                        else
                         parent_line_flag := TRUE;
                        end if;}
     child_flag: { if (tree_element.child = NULL) then
                       child_flag := FALSE;
                   else
                       child_flag := TRUE;
                   end if;}

   Objects:

    tree_instance: XmPushButton {
     Attributes: 
  
      parent: main_background;
      x: tree_element.x_absolute;
      y: tree_element.y_absolute;
      labelString: cus_genname("CW");
      height: 20;
      width: 120;
      allowUserMove: TRUE;
      fontList: "9x15bold";  
      borderWidth:0;
     Methods:

        move: {
          diff_x := tree_element.x_absolute - x;
          diff_y := tree_element.y_absolute - y;
          root_x := root_x - diff_x;
          root_y := root_y - diff_y;
       }

        notify: {
          if (current_command = "create") then
              current_sdd := create_sd_instance(
	  	"tree_element", "DM_BOX");

/*  save parent node for C code to put into tree*/

              parent_node := local_current_sdd;

/*  now initialize offsets to 0 */

              put_sd_value(current_sdd,"x_absolute",0);
              put_sd_value(current_sdd,"y_absolute",0);
              put_sd_value(current_sdd,"x_offset",0);
              put_sd_value(current_sdd,"y_offset",0);

             put_sd_value(current_sdd,"create_flag",TRUE);
          end if;

          if (current_command = "delete") then
             put_sd_value(local_current_sdd,"create_flag",FALSE);
          end if;
          }
     }

  On Create: {

     local_current_sdd := current_sdd;
          
     put_sd_value(current_sdd, "node_info",
        add_new_leaf(current_sdd, 
                   tree_instance.height, tree_instance.width,
                   border, parent_distance));

        unlayout_tree(root_node);

/*  node being created has no child*/
              put_sd_value(current_sdd, "child", NULL);

/*  get parents current eldest child - new node will be eldest child and
      old eldest child will be sibling of this node*/
              
              if (parent_node <> NULL) then
                 if (get_sd_value(parent_node,"child") = NULL) then
/*    if this is the first child, then want to save value as last child*/
                      put_sd_value(parent_node,"last_child",current_sdd);
                  end if;

                 temp := get_sd_value(parent_node, "child");
                 put_sd_value(current_sdd, "sibling", temp);
                 put_sd_value(parent_node, "child", current_sdd);
                 put_sd_value(current_sdd,"parent",parent_node);

               else
                 temp := NULL;
               end if;

/* if there was a child then new node is its predecessor*/
              if (temp <> NULL)  then
                put_sd_value(temp, "predecessor", current_sdd);
              end if;

/* predecessor of new node is parent*/
              put_sd_value(current_sdd,"predecessor", parent_node);

      layout_tree(root_node);

  }
  
  On Destroy: {

   if (root_node <> NULL) then
      unlayout_tree(root_node);
   end if;

   if (local_current_sdd = root_node) then
      root_node := NULL;
   end if;

   remove_leaf(local_current_sdd);

/* now we do the following to remove the subtree rooted here:

   1) delete all children
   2) if parent exists, then fix up links between parent, predecessor
       and sibling.
      if no parent then don't worry about links    */

   temp := get_sd_value(local_current_sdd, "child");
   WHILE (temp <> NULL) 
      put_sd_value(temp,"create_flag",FALSE);
      temp := get_sd_value(temp,"sibling");
   END WHILE;
    
   temp := get_sd_value(local_current_sdd,"parent");
   if (id_exists(temp)) then

/* first we worry about the last child.  if we are deleting the last child,
then we move predecessor into parent last child field.  this will work in
every case except where the last child is also the eldest child.  in this
case, nothing is done*/

     temp1 := get_sd_value(temp,"last_child");
     if (temp1 = local_current_sdd) then

/* we are, in fact, deleting last child*/

        if (temp1 <> get_sd_value(temp, "child")) then
            put_sd_value(temp,"last_child", 
                 get_sd_value(local_current_sdd,"predecessor"));
        end if;
     end if;

     temp1 := get_sd_value(local_current_sdd,"predecessor");
     if (temp1 = temp) then

/* the current node is the eldest child of its parent.  In this case, we move
current sibling into child spot of parent*/

        put_sd_value(temp,"child",get_sd_value(local_current_sdd,"sibling"));

/* we have adjusted the child pointer of the parent, now we need to adjust the
predecessor pointer of the sibling (if the sibling exists)*/

        if(id_exists(get_sd_value(local_current_sdd,"sibling"))) then
          put_sd_value(get_sd_value(local_current_sdd,"sibling"),
                "predecessor",temp1);
        end if;

     else

/*  the current node is not the eldest child of its parent.  In this case, we
move the current sibling pointer into the predecessor and the current
predecessor into the sibling (if it exists)*/


        put_sd_value(temp1,"sibling",
          get_sd_value(local_current_sdd,"sibling"));

        if (id_exists(get_sd_value(local_current_sdd,"sibling"))) then
           put_sd_value(get_sd_value(local_current_sdd,"sibling"),
            "predecessor",temp1);
        end if;
     end if;
   end if;

  destroy_sd_instance(local_current_sdd);

  if (root_node <> NULL) then
      layout_tree(root_node);
  end if;

}

/*--------------------------------------------------------------------------*\
|  Name: parent_line_vc
|
|  Description:
|    The vc controls the line out the front of a node (when it has a parent)
\*--------------------------------------------------------------------------*/
VC: parent_line_vc

  Creation Condition: (parent_line_flag)

  Variables:

   line_length: 8;

   Objects:

    parent_line: XmLabel {
     Attributes: 
      parent: main_background;
      height: 2;
      width: line_length;
      foreground: "black";
      background: "black"; 
      x: tree_instance.x - line_length;
      y: tree_instance.y + (tree_instance.height + height) / 2;
      borderWidth:0;
    }

   end VC parent_line_vc;  

/*--------------------------------------------------------------------------*\
|  Name: child_line_vc
|
|  Description:
|    The child_line_vc instantiates when the node described in element_vc
|    has a child.  It controls the line that extends from the node
|    the side of the children
\*--------------------------------------------------------------------------*/
VC: child_line_vc

  Creation Condition: (child_flag)

  Variables:

   line_length: parent_distance + border - 10;

   Objects:

    child_line: XmLabel {
     Attributes: 
      parent: main_background;
      height: 2;
      width: line_length;
      foreground: "black";
      background: "black"; 
      x: tree_instance.x + tree_instance.width;
      y: tree_instance.y + (tree_instance.height + height) / 2;
      borderWidth:0;      
     }

    across_line: XmLabel {
     Attributes: 
      parent: main_background;
      recomputeSize: FALSE;
      height: { if (tree_element.last_child <> NULL and 
                    tree_element.last_child <> UNDEFINED and
                    tree_element.child <> NULL and
                    tree_element.child <> UNDEFINED) then
                   if( get_sd_value(tree_element.last_child,"y_absolute") =
                         get_sd_value(tree_element.child,"y_absolute")) then
                       height := 2;
                   else
                       height := get_sd_value(tree_element.last_child,"y_absolute") -
                         get_sd_value(tree_element.child,"y_absolute");
                   end if;
               end if;}

      width: 2;
      foreground: "black";
      background: "black";  
      x: tree_instance.x + tree_instance.width +
                      parent_distance + border - 10;
      y: {if (tree_element.child <> NULL and
                          tree_element.child <> UNDEFINED) then
                        y := get_sd_value(tree_element.child,"y_absolute")+1 +
                               tree_instance.height/2;
                      end if;}
     }

    end VC child_line_vc;  

  end VC element_vc;







