/*--------------------------------------------------------------------------*\
|   
|  Name:  Arg
|
|  Description:  
|     This file contains the slang code which defines the actions
|      for input arguments
|
|  Component of:
|     vis_prog - visual programming demonstration.
|
|  Project:
|     Serpent
|     User Interface Project
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Len Bass
|     ljb@sei.cmu.edu  
|
|  Change History:
|     June 22, 1989 : Module created.
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define numb_font "vr-20"

/*--------------------------------------------------------------------------*\
|  Name: argument_vc
|
|  Description:
|    The argument_vc instantiates on a new arg_sdd in 
|    the shared data base. 
\*--------------------------------------------------------------------------*/
VC: argument_vc

  Creation Condition:  (new("arg_sdd") and arg_sdd.create_it)

  Variables:

     output_line_id: NULL;
     form_x;
     form_y;

     form_width: 100;
     max_value : 100;
     local_debug: TRUE;

     Objects:

       output_instance: push_button_widget {
         Attributes:
           parent: form_background;
           recompute_size: FALSE;
           font_list: numb_font;
           x: 133;
           y: {if (local_debug) then
                 y := 15;
               else
                 y := 15;
               end if;
               }

           width: 40;
           height: {if (local_debug) then
                 height := 20;
               else
                 height := 15;
               end if;
               }

           background_color: "black";
           foreground_color: {if (local_debug) then
                 foreground_color := "white";
               else
                 foreground_color := "black";
               end if;
               }
           label_string: { if (arg_sdd.value = UNDEFINED) then
                             label_string := "   ";
                         else
                             label_string := arg_sdd.value;
                         end if;
                        }
        Methods:

           notify: {
             if (current_command = "connect") then
                if (line_id <> NULL) then
                   message := "Have already selected source, hit connect to reset";
                else
                   line_id := create_sd_instance("line_sdd",
                                "DM_BOX");
                   set_line_values(line_id,0,0,0,0)
                   output_line_id := line_id;
                   value_id := get_creating_sd(argument_vc);
                   source_x := form_background.x +
                       output_instance.x + output_instance.width;
                   source_y := form_background.y +
                        output_instance.y + output_instance.height / 2;

                   message := "Select target for value";
                end if;
             elsif (current_command = "disconnect" and 
                id_exists (output_line_id)) then
                
                /* delete line output_line_id */

                destroy_sd_instance(output_line_id);
             elsif (current_command = "delete") then
                if (id_exists (output_line_id)) then
                  destroy_sd_instance(output_line_id);
                end if;
                destroy_sd_instance(get_creating_sd(argument_vc));
             end if;
          }
     }

       slider_background: bulletin_board_widget{
        Attributes:
          parent: form_background;
          height: 7;
          width: 125;
          border_width: 1;
          selected_x: 0;

        Methods:
           select:{
              arg_sdd.value := 
                  make_integer ( max_value * (selected_x / form_width));
              }
        }

       form_background: bulletin_board_widget{
         Attributes:
           parent: main_background;
           x: form_x+25;
           y: form_y;
           height: 7;
           width: 100;
           allow_user_move:  TRUE;
           selected_x: 0;

         Methods:

           move: {

             if ( id_exists (output_line_id)) then
                   source_x := form_background.x +
                       output_instance.x + output_instance.width;
                   source_y := form_background.y +
                        output_instance.y + output_instance.height / 2;
                   set_line_start_values(output_line_id,source_x,source_y)
              end if;
          }

             }


slider_length: label_widget {
  Attributes:

    parent: slider_background;
    height: 2;
    width:{  
           if (arg_sdd.value > max_value) then
             arg_sdd.value := max_value;
           end if;
 
           if (arg_sdd.value <> 0) then
             width := (arg_sdd.value / max_value) * form_width;
           else
              width := 1;
           end if;}
    x: 1;
    y: 5;

    foreground_color: "black";
    background_color: "black";
  }

decr_button: push_button_widget{
   Attributes:

     parent: slider_background;
     height: 18;
     width: 18;
     x: 1;
     y: 10;
     label_string: "<";

   Methods:
    notify: {
     arg_sdd.value := arg_sdd.value - 1;
     if (arg_sdd.value < 0) then
        arg_sdd.value := 0;
        end if;
    }
   }

incr_button: push_button_widget{
   Attributes:

     parent: slider_background;
     height: 18;
     width: 18;
     x: form_width - 5;
     y: 10;
     label_string: ">";

   Methods:
    notify: {
     arg_sdd.value := arg_sdd.value + 1;
     if (arg_sdd.value > max_value) then 
         arg_sdd.value := max_value; 
      end if;
    }
   }


  On Create: {
       message := "Move to desired position";
       form_x := init_x;
       form_y := init_y;
   }

  end VC argument_vc;

