/*  $Header: /u/04c/mnt/integ/source/dm/include/RCS/drc.h,v 1.16 91/04/01 17:27:34 dvk Exp $  */
/*  $Log:	drc.h,v $
 * Revision 1.16  91/04/01  17:27:34  dvk
 * No change
 * 
 * Revision 1.15  90/07/23  10:58:22  ejh
 * added drc_convert
 * 
 * Revision 1.14  90/07/05  13:11:50  ejh
 * switched most everything out to uc2
 * 
 * Revision 1.13  90/03/01  12:29:01  dvk
 * Added drc_free_atv to release space claimed by drc_get_atv.
 * Also, made drc_get_undefined static and deleted the extern
 * declaration for the non-existant routine "error"
 * 
 * Revision 1.12  90/02/22  14:56:01  dvk
 * Added drc_cvt_type to be used by dro_attr_assign_quick.
 * 
 * Revision 1.11  90/02/14  22:17:29  ejh
 * fixed include structure
 * 
 * Revision 1.10  90/01/30  20:07:36  clc
 * Eliminated static routines
 * 
 * Revision 1.9  90/01/26  16:18:01  rcs
 * changed debug print
 * 
 * Revision 1.8  89/12/05  18:13:49  ejh
 * qualified define of memoryPack
 * 
 * Revision 1.7  89/10/06  10:09:31  rcs
 * changes to support typeless buffesr
 * 
 * Revision 1.6  89/06/15  15:29:22  ljb
 * add conversion routines for buffer and boolean
 * 
 * Revision 1.5  89/02/09  16:51:09  ejh
 * added copyright notice
 * 
 * Revision 1.4  89/02/07  20:05:56  ejh
 * removed MAXSTR
 * 
 * Revision 1.3  88/10/11  16:51:22  ljb
 * add buffer to integer and vice versa
 * 
 * Revision 1.2  88/10/03  14:55:06  ljb
 * add routines strtobuff and bufftostr to allow conversion between buffers and strings
 *   */
/*--------------------------------------------------------------------------*\
|
|  Name:  Run Time Conversions Utility (drc)
|
|  Description:  
|     The run time conversions utility provides low level type conversions
|     that are either not avaialalbe or do not return the appropriate
|     parameters.
|
|  Component of:
|     Dialogue Manager Layer.
|
|  Related files:
|     drc.c - contains the implementation for the transactionmanager.
|     drc.h - contains the external definition for the transaction manager.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Robert C. Seacord
|     rcs@sei.cmu.edu  
|
|  Change History:
|     June 30, 1988 : Module created.
|     Oct 1, 1988: added strtobuff and bufftostr   ljb
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef	drc
#define	drc

/*--------------------------------------------------------------------------*\
|  Global Variable: drc_debug
|
|  Description:
|     Debug variable for the run time conversions utility.
|
\* Definition:                                                              */

#ifdef drc_TOP
  int drc_debug = 1;
#else
  extern int drc_debug;
#endif

/*--------------------------------------------------------------------------*\
|  Routine: drc_get_atv
| 
|  Description:
|     The dro get appropriately typed value routine accepts an operand and
|     a conversion type and returns a value of the conversion type or sets
|     status to invalid_conversion.
\* Syntax :								   */

	caddr_t drc_get_atv(
	  /* operand : dro_atomic_data_item */
	  /* data_type : idd_data_types */
	);
/*
|  Parameters:
|     value - dro_drohandle, idd_data_types
|
|  Returns:  
|     value in new idd_data_types
|  
|  Statuses:  
|     failed_conversion.
|  
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: drc_free_atv
| 
|  Description:
|    This routine frees the space used for the value acquired via drc_get_atv.
\* Syntax :								   */

	void drc_free_atv(
	/* atv : in caddr_t */
	/* atv_type : in idd_data_types */
	);
/*
|  Parameters:
|     atv - the addres of the data to release
|     atv_type - the type of atv we are freeing
|
|  Returns:  
|     nothing
|  
|  Statuses:  
|     none
|  
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: drc_convert
| 
|  Description:
|    This routine converts from one to another.
\* Syntax :								   */

	caddr_t drc_convert (
	/* from_type:	in	idd_data_types,
	   to_type:	in	idd_data_types,
	   data:	in	caddr_t */
	);
/*
|  Parameters:
|     from_type:	the type to be converted from
|     to_type:		the type to be converted to
|     data:		the data to convert
|
|  Returns:  
|     the value in the new type
|  
\*--------------------------------------------------------------------------*/

#endif
