/* $Header: /u/04c/mnt/integ/source/dm/include/RCS/dri.h,v 1.2 91/01/31 16:40:11 little Exp $ */

/*--------------------------------------------------------------------------*\
|
|  Name:  Run Time Interpreter (dri)
|
|  Description:  
|	The interpreter reads the binary files comprising the dialogue
|	executable and calls the appropriate routines to effect the
|	execution of the dialogue.
|
|  Component of:
|	Dialogue Manager
|
|  Related files:
|	dri.h - contains the definitions of the interpreter constants
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:  
|	Daniel Klein
|	dvk@sei.cmu.edu
|
|  Change History:
|*
|* $Log:	dri.h,v $
 * Revision 1.2  91/01/31  16:40:11  little
 * add new instructions: R_rec_on, R_rec_off, T_validate_sne, & c
 * 
 * Revision 1.1  91/01/02  17:51:26  dvk
 * Initial revision
 * 
|*
|*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef dri
#define dri

/*
 * If you make major changes to this file which will effect execution of
 * currently existing binaries, make sure that you also change DRI_VERSION.
 * This will cause dri_init to issue a warning.  Adding a new type to the
 * dri_token list (or changing the order) is one good reason.
 */
#define DRI_VERSION	1


void dri_init ( /* dlg_name */ );
    /*
     * char *dlg_name;
     */

/*
 * The following enumerated type defines the values emitted by the compiler
 * and interpreted by the routine dri_interpret.  The compiler outputs a
 * string of 16 bit integers (shorts), which are read by the interpreter.
 * The interpreter stack machine operates with a longword (i.e., at least
 * a 32-bit) valued stack.  To conserve disk space and increase speed, the
 * compiler generates a 16-bit instruction, since most constant values
 * generated by the compiler are small numbers.  When 32 bit numbers are
 * needed, there is a special prefix code that is used to indicate that
 * the following two shorts are to be combined into a single long.
 * 
 * The interpreter consists of the following files, where "abc" is the name
 * of the compiled dialogue:
 * 
 * 	.abc.z	String/constants table, an array of all the strings used
 * 		by the dialogue.  String values are output by the compiler
 * 		as indices into this array.  Routines which expect string
 * 		parameters have these indices converted to the string
 * 		addresses by the interpreter.  This file also contains
 * 		the list of integers to create integer adi's for, and
 * 		the list of doubles to create real adi's for.  See the
 * 		interpreter header structure for details.
 * 
 * 	.abc.t	Snippet table, an array of longword offsets into the
 * 		executable interpreter stream.  The interpreter is
 * 		called with a snippet number, and begins executing at
 * 		the corresponding snippet offset.
 * 
 * 	.abc.x	Snippet code, an array of short integers describing the
 * 		code to execute.  The following enumerated type describes
 * 		the code better.  In brief, it is a stack machine, where
 * 		most operations pop operands off the stack, and push their
 * 		result (if any) onto the stack.
 * 
 * 	.abc.c	Addresses of dialogue called C routines.  Only present if
 * 		there are any "EXTERNALS" in the dialogue.
 * 
 * 	.abc.h	Declarations for .abc.c.
 * 
 * The enumerated types are all negative numbers.  The C compiler guarantees
 * that all values in an enum are sequential following a specifically defined
 * enumerated value - hence the first value is listed as -32768 (the largest
 * negative number that fits in 16 bits), and all others *increment* towards
 * zero from there.  Any positive number which is scanned by the interpreter
 * is assumed to vbe a literal constant.  If a negative constant is present,
 * the special L_negative literal precedes it.  Because all values in the
 * on-disk interpreter executable file are shorts, special literal L_long
 * is used to precede a longword constant.
 * 
 * To aid in debugging, some attempt has been made to use a mnemonic naming
 * convention.  The R_ types cause a dialogue manager runtime routine to be
 * called with no interference by the interpreter.  The L_ types indicate a
 * special literal value follows in the instruction stream.  The C_ types
 * are used to flag special constants - these are values which have a constant
 * value which is known to the dialogue manager, but not necessarily the
 * compiler (but the compiler knows it's name).  The T_ types are used by the
 * interpreter for special functions which do not map to a single dialogue
 * manager routine call.
 */

typedef enum {
    L_negative = -32768,	/* Must all be negative! */
    L_long,

    R_get_parent_vc,
    R_get_creating_sd,
    R_get_object,
    R_get_component_value,
    R_put_component_value,
    R_get_vc_id,
    R_get_variable_value,
    R_put_variable_value,
    R_create_element,
    R_delete_element,
    R_get_name,
    R_get_type,
    R_exit,
    R_id_exists,
    R_rec_on,
    R_rec_off,

    R_and,
    R_or,
    R_dot,
    R_neq,
    R_leq,
    R_lss,
    R_geq,
    R_gtr,
    R_assign,
    R_eql,
    R_add,
    R_sub,
    R_neg,
    R_not,
    R_div,
    R_exp,
    R_mul,
    R_new,

    R_list_assign,
    R_validate_object,
    R_boolean,
    R_get_boolean_atv,
    R_get_integer_atv,
    R_get_real_atv,
    R_get_string_atv,
    R_get_record_atv,
    R_get_id_atv,
    R_get_buffer_atv,
    R_get_undefined_atv,
    R_make_boolean_constant_adi,
    R_make_buffer_constant_adi,
    R_make_integer_constant_adi,
    R_make_indir_string_constant_adi,
    R_make_immed_string_constant_adi,
    R_make_real_constant_adi,
    R_make_id_constant_adi,
    R_make_undefined_adi,
    R_get_sire_vc,
    R_get_OBJECT,
    R_put_OBJECT,
    R_get_variable_VALUE,
    R_put_variable_VALUE,
    R_get_component_VALUE,
    R_put_component_VALUE,
    R_bool_assign,
    R_int_assign,
    R_id_assign,
    R_string_assign,
    R_make_adi,

    T_and,
    T_if,
    T_goto,
    T_return,
    T_snip_end,
    T_compile_error,
    T_simple_parm_init,
    T_complex_parm_init,
    T_adi_from_parm,
    T_component_value_from_parm,
    T_vc_list_init,
    T_dpt_vc_exists,
    T_validate_sne,
    T_validate_sne_cc,
    T_not_dpt_rite_rule,
    T_vc_create,
    T_get_PARENT_vc,
    T_dmsg_setup,

    T_string_constant,
    T_slang_file,
    T_slang_line_num,
    T_reset_slang_line_num,
    T_method_verify,
    T_void_user_function,
    T_user_function,

    C_SELF,
    C_PARENT,
    C_vc_sym,
    C_object_sym,
    C_variable_sym,
    C_cus_ATTR,
    C_cus_CREATE,
    C_cus_DESTROY,
    C_cus_METH,
    C_cus_CCOND,
    C_cus_VAR,
    C_dob_id_NULL,
    C_dob_bool_TRUE,
    C_dob_bool_FALSE,
    C_dob_int_const,
    C_dob_real_const

    } dri_token;

#define DRI_MAGIC	412-268-7791	/* My office phone number */

typedef struct {
    long magic;			/* Magic number */
    long version;		/* Current (?) version number check */
    long string_size;		/* Size of string table (in bytes) */
    long int_const_size;	/* Size of integer constant table */
    long real_const_size;	/* Size of real constant table */
    long max_snippets;		/* Number of snippets in .dialogue.x */
    } dri_header;

typedef struct {
    short idx;			/* Integer constant array index */
    long val;			/* Value of constant */
    } dri_int;

typedef struct {
    short idx;			/* Real constant array index */
    double val;			/* Value of constant */
    } dri_real;

#endif
