static char rcsid [] = "$Header: /u/04c/mnt/integ/source/dm/src/RCS/din_st.c,v 1.12 91/03/26 11:17:16 ejh Exp $";

/*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*
$Log:	din_st.c,v $
 * Revision 1.12  91/03/26  11:17:16  ejh
 * changed MAX_SYMBOL_STRING_SIZE from 256 to 255 so that the test for the
 * length actually works
 * 
 * Revision 1.11  90/12/06  20:15:13  little
 * speed things up by keeping the original_name_prop, type_prop, and
 * template_prop in the dob_object record in addition to the sym tab
 * 
 * Revision 1.10  90/10/08  16:36:12  dvk
 * Added a level of context to the DM to speed up runtime lookups.  Also,
 * changed reading of xxxt.d file to reflect changes in compiler (will
 * run a bit faster, now).
 * 
 * Revision 1.9  90/05/23  15:36:20  clc
 * added use of SERPENT_DATA_PATH
 * 
 * Revision 1.8  90/04/10  18:29:22  ejh
 * changed strings.h to string.h
 * 
 * Revision 1.7  90/02/14  22:19:55  ejh
 * fixed include structure
 * 
 * Revision 1.6  90/01/19  16:25:26  dvk
 * Changed strlen calculations of constant strings to simple constants
 * 
 * Revision 1.5  90/01/09  13:55:24  ejh
 * took out fopen declaration
 * 
 * Revision 1.4  89/10/23  19:47:25  ejh
 * declared fopen
 * 
 * Revision 1.3  89/05/25  17:36:12  little
 * rearrange include order becasue of changes to d22_sne int d22.h
 * 
 * 
 * Revision 1.2  89/02/15  18:04:53  little
 * change var named buffer to a_buffer (do not ask me why)
 * add copyright notic
 * 
*/

#define	memoryPack              /* use memoryPack rather than mallocPack    */

#include	<stdio.h>
#include	"string.h"
#include	"global.h"
#include	"listPack.h"
#include	"cus.h"
#include	"ipc.h"
#include	"d22.h"
#include	"din.h"
#include	"dob.h"
#include	"ufa.h"
#include	"u22.h"

#define MAX_SYMBOL_STRING_SIZE 255
#define DATA_PATH_ENV_VAR_NAME "SERPENT_DATA_PATH"

static char *PrematureEOF = "premature EOF reading st data\n";


/*  -------------------------------------------------  din_get_st_data */
void din_get_st_data (file_name)
string file_name;
{  boolean cont;
   FILE *stream;
   string data_search_paths, full_data_path;

   if (!getenv (DATA_PATH_ENV_VAR_NAME)) {
      fprintf(stderr, "%s environment var not present\n",
              DATA_PATH_ENV_VAR_NAME);
      exit(1);
   }

   data_search_paths = u22_copy_string (getenv (DATA_PATH_ENV_VAR_NAME));

   if (!strlen (data_search_paths))
      {data_search_paths [0] = '.';
      data_search_paths [1] = '\0';
      }

   DEBUG3 (fprintf (stderr, "data_search_paths = %s\n", data_search_paths););

   full_data_path = ufa_full_file_name (data_search_paths, file_name);
   if (full_data_path == NULL)
      FATAL_ERROR (fprintf (stderr,
        "could not find symbol table descriptor file named %s\n", file_name));

   stream = fopen (full_data_path, "r");
   if (!stream)
      FATAL_ERROR (fprintf (stderr,
        "could not open symbol table descriptor file named %s\n",
        full_data_path));
   (void) cus_create_nu_context();	/* See comment in din_read_symbol */
   cont = din_get_st_entry (stream);
   while (cont)
      cont = din_get_st_entry (stream);
   return;
}

/*  -------------------------------------------------  din_get_st_entry */
boolean din_get_st_entry (stream)
FILE *stream;
{  cus_sym symbol;
   char a_buffer [10];
   boolean cont;
   char *name;
   data_object_record *the_object;

   fread (a_buffer, 1, ESCAPE_LENGTH, stream);
   if (feof (stream))
      return (FALSE);
   a_buffer [ESCAPE_LENGTH] = '\0';
   if (strcmp (a_buffer, BEGIN_SYMBOL_ENTRY_ESCAPE))
      FATAL_ERROR (fprintf (stderr,"bad start st entry escape = %s\n",
	a_buffer));
   symbol = din_get_symbol (stream);
   DEBUG3 (fprintf (stderr, "starting a symbol entry\n"));
   cont = din_get_prop (stream, symbol);
   while (cont)
      cont = din_get_prop (stream, symbol);
   if ((name = (char *) cus_get_prop (symbol, original_name_prop)) &&
       (the_object = (data_object_record *) cus_get_prop (symbol, id_prop)))
      the_object-> original_name = (caddr_t) name;
   return (TRUE);
}

/*  -------------------------------------------------  din_get_element */
char *din_get_element (stream)
FILE *stream;
{  char a_buffer [10];

   fread (a_buffer, 1, ESCAPE_LENGTH, stream);
   if (feof (stream))
      FATAL_ERROR (fprintf (stderr, PrematureEOF));
   a_buffer [ESCAPE_LENGTH] = '\0';
			/* ?? the case characters are hard wired here, it
			   should probably be fixed to somehow use the 
			   same mechanism as the xxx_ESCAPE
			   strings defined in the header file */
   switch (a_buffer [1])
      {
      case 'S':
         return ((char *) din_read_symbol (stream));
         break;
      case 'I':
         return ((char *) din_read_integer (stream));
         break;
      case '(':
         return ((char *) din_read_list (stream));
         break;
      case ')':
         DEBUG3 (fprintf (stderr, "end of a list\n"))
         return (NULL);
         break;
      case 'G':
         return ((char *) din_read_string (stream));
         break;
      default:
         FATAL_ERROR (fprintf (stderr, "bad escape switch = %s\n", a_buffer));
      }

			/* better not get here, but if do return something */
   return (NULL);
}

/*  -------------------------------------------------  din_get_symbol */
cus_sym din_get_symbol (stream)
FILE *stream;
{  char a_buffer [MAX_SYMBOL_STRING_SIZE + 1];

   fread (a_buffer, 1, ESCAPE_LENGTH, stream);
   if (feof (stream))
      FATAL_ERROR (fprintf (stderr, PrematureEOF));
   a_buffer [ESCAPE_LENGTH] = '\0';
   if (strcmp (a_buffer, SYMBOL_ESCAPE))
      FATAL_ERROR (fprintf (stderr, "bad symbol escape = %s\n", a_buffer));
   return (din_read_symbol (stream));
}

/*  -------------------------------------------------  din_read_symbol */
cus_sym din_read_symbol (stream)
FILE *stream;
{  char a_buffer [MAX_SYMBOL_STRING_SIZE + 1];
   int new;
   id_length len;
   symtype_length type;
   cus_sym entry;

   fread (&len, 1, sizeof (id_length), stream);
   if (feof (stream))
      FATAL_ERROR (fprintf (stderr, PrematureEOF));
   if (len > MAX_SYMBOL_STRING_SIZE)
      FATAL_ERROR (fprintf (stderr, "too long symbol string = %d\n", len));
   fread (a_buffer, 1, len, stream);
   if (feof (stream))
      FATAL_ERROR (fprintf (stderr, PrematureEOF));
   a_buffer [len] = '\0';
   fread (&type, 1, sizeof (symtype_length), stream);
   if (feof (stream))
      FATAL_ERROR (fprintf (stderr, PrematureEOF));
   /*
    * The only symbols which are looked up in the symbol table are SD
    * instances (which have the SD_QUALIFIER_STRING as a prefix), VC
    * templates (which have "VC" as a prefix), and the god_vc (which is
    * named by d22_god_text).  We don't need to add anything other than
    * these into the symbol table (we just need to create the associated
    * "cus_sym" structure).  However, this routine takes advantage of
    * the already entered syms when doing an add_sym.  To optimize later
    * lookups, we put symbols which are *not* looked up into the NO_CONTEXT,
    * and symbols which are into context 1 (created in din_get_st_data).
    * So, check the first few characters of a symbol, and act accordingly.
    */
   if ((strncmp (a_buffer, SD_QUALIFIER_STRING, SD_QUALIFIER_LENGTH) == 0) ||
       (strncmp (a_buffer, "VC", 2) == 0) ||
       (strcmp (a_buffer, d22_god_text) == 0))
       entry = cus_add_sym (a_buffer, (cus_sym_type) type, &new, WITH_CONTEXT);
   else
       entry = cus_add_sym (a_buffer, (cus_sym_type) type, &new, NO_CONTEXT);
   if (new)
      cus_put_prop (entry, id_prop, dob_create_symbol_table_entry(entry,type));

   return (entry);
}

/*  -------------------------------------------------  din_get_prop */
boolean din_get_prop (stream, symbol)
FILE *stream;
cus_sym symbol;
{  proptype_length prop;
   char a_buffer [10];
   char *prop_value;

   fread (a_buffer, 1, ESCAPE_LENGTH, stream);
   if (feof (stream))
      FATAL_ERROR (fprintf (stderr, PrematureEOF));
   a_buffer [ESCAPE_LENGTH] = '\0';
   if (!strcmp (a_buffer, END_SYMBOL_ENTRY_ESCAPE))
      return (FALSE);
   if (strcmp (a_buffer, PROPERTY_ESCAPE))
      FATAL_ERROR (fprintf (stderr, "bad property escape = %s\n", a_buffer));
   fread (&prop, sizeof (proptype_length), 1, stream);
   if (feof (stream))
      FATAL_ERROR (fprintf (stderr, PrematureEOF));
   prop_value = din_get_element (stream);

			/* some prop values need modifying before saving */
   switch ((cus_property_type)prop)
      {case mailbox_name_prop:	/* get the ipc_mailbox # from the string */
         cus_put_prop (symbol, (cus_property_type) prop, ipc_init (prop_value));
         break;
      default:			/* 'standard" prop value, no modification */
         cus_put_prop (symbol, (cus_property_type) prop, prop_value);
         break;
      }

   return (TRUE);
}

/*  -------------------------------------------------  din_read_integer */
char *din_read_integer (stream)
FILE *stream;
{  int the_int;

   fread (&the_int, sizeof (int), 1, stream);
   if (feof (stream))
      FATAL_ERROR (fprintf (stderr, PrematureEOF));
   return ((char *) the_int);
}

/*  -------------------------------------------------  din_read_list */
char *din_read_list (stream)
FILE *stream;
{  LIST the_list;
   char *cont;

   DEBUG3 (fprintf (stderr, "starting a list\n"));
   the_list = make_list ();
   cont = (char *) TRUE;
   while (cont)
      if (cont = din_get_element (stream))
         add_to_tail (the_list, cont);
   return ((char *) the_list);
}

/*  -------------------------------------------------  din_read_string */
char *din_read_string (stream)
FILE *stream;
{  string the_string;
   string_length len;

   fread (&len, 1, sizeof (string_length), stream);
   if (feof (stream))
      FATAL_ERROR (fprintf (stderr, PrematureEOF));
   the_string = make_node (len + 1);
   fread (the_string, 1, len, stream);
   if (feof (stream))
      FATAL_ERROR (fprintf (stderr, PrematureEOF));
   the_string [len] = '\0';
   return (the_string);
}
