static char rcsid [] =
        "$Header: /u/04c/mnt/integ/source/dm/src/RCS/dra.c,v 1.56 91/04/01 17:29:33 dvk Exp $";
/*--------------------------------------------------------------------------*\
|
|  Name:  Run Time Data Access Utility (dra)
|
|  Description:  
|     The run time data access utility provides run time access to various
|     data components within the object base.
|
|  Component of:
|     Dialogue Manager Layer.
|
|  Related files:
|     dra.c - contains the implementation for the run time data access 
|             utility.
|     dra.h - contains the external definition for the run time data
|             access utility.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Robert C. Seacord
|     rcs@sei.cmu.edu  
|
|  Change History:
|     July 16, 1988 : Module created.
$Log:	dra.c,v $
 * Revision 1.56  91/04/01  17:29:33  dvk
 * Added missing dra_delete_temp
 * 
 * Revision 1.55  91/02/20  18:53:32  little
 * take out DEBUGS so dbx works correctly
 * 
 * Revision 1.54  91/02/12  15:33:02  little
 * not much changed.  apparently i just removed a comment.
 * 
 * Revision 1.53  91/01/31  17:28:18  little
 * add new rec_on & rec_off functions
 * 
 * Revision 1.52  90/12/06  20:22:26  little
 * speed things up by keeping the original_name_prop, template_prop, and
 * e_prop in the dob_object in addition to the sym tab
 * 
 * Revision 1.51  90/07/25  16:45:01  dvk
 * Small architectural difference resolved.  Ooops.  Sorry.  (But Reed
 * checked it and he said he's sorry, too).
 * 
 * Revision 1.50  90/07/24  19:15:34  dvk
 * Added dra_get_sire_vc (gets N'th grandparent)
 * 
 * Revision 1.49  90/07/10  18:09:44  dvk
 * Added new routine dra_get_PARENT_vc
 * 
 * Revision 1.48  90/02/28  18:52:58  dvk
 * Changed only call of cus_get_sym_type_string => cus_type_print_val
 * 
 * Revision 1.47  90/02/15  11:23:45  ejh
 *  left debug on (oops)
 * 
 * Revision 1.46  90/02/14  22:22:02  ejh
 * fixed include structure
 * 
 * Revision 1.45  90/02/12  17:01:17  dvk
 * Added routines dra_get_OBJECT, dra_get_variable_VALUE, dra_put_variable_VALUE,
 * dra_get_component_NAME, and dra_put_component_NAME
 * 
 * Revision 1.44  90/01/19  16:34:57  dvk
 * Moved some of the guts of dra_put_variable value into a new routine
 * called dra_put_direct_value, in anticipation of subsequent changes
 * to the compiler.
 * 
 * Revision 1.43  90/01/12  15:06:50  dvk
 * Changed to use dob_boolean_TRUE and dob_boolean_FALSE
 * 
 * Revision 1.42  89/12/27  14:45:11  ejh
 * changed d22_ftl_err to d22_fatal
 * 
 * Revision 1.41  89/12/21  16:47:32  little
 * change d22_fatal_error.h to d22_ftl_err.h
 * 
 * Revision 1.40  89/12/21  11:55:39  little
 * add support for slang file name output during dm run-time error print
 * 
 * Revision 1.39  89/11/30  14:42:12  rcs
 * fixed notification of tap after isd_remove command problem
 * 
 * Revision 1.38  89/11/22  15:59:41  trm
 * no changes made.
 * 
 * Revision 1.37  89/11/02  16:16:33  little
 * no changes
 * 
 * Revision 1.36  89/10/05  16:57:56  rcs
 * various mods to support typed buffers
 * 
 * Revision 1.35  89/09/01  14:06:56  bmc
 * Changed dra_id_exists to handle null pointers and variables containing
 * null values.
 * 
 * Revision 1.34  89/08/31  19:00:08  bmc
 * Added dra_id_exists() function.
 * 
 * Revision 1.33  89/08/29  17:40:50  little
 * add dra_exit function
 * 
 * Revision 1.32  89/07/24  14:59:36  little
 * ops removal
 * 
 * Revision 1.31  89/05/22  11:11:15  bmc
 * Now explicitly include 'uer.h'.  Related to changes to 'dob.c', 'dob.h'
 * and 'uer.h'.
 * 
 * Revision 1.30  89/05/19  17:26:34  bmc
 * Changed to accomodate new definition of 'dro_atomic_data_item'.  (See
 * 'dro.h' for more information.)  Also, changed calls to dob routines
 * to be consistent with modifications in 'dob.h'.
 * 
 * Revision 1.29  89/03/28  11:26:33  serpent
 * cahnge ifdef 0 to if 0 so the stupid sun compiler can compile it
 * 
 * Revision 1.28  89/03/01  17:06:58  rcs
 * fixed dra_put_component_value to work with different typed data items
 * 
 * Revision 1.27  89/02/23  14:29:07  rcs
 * changed dra_get_name to not accept sd id's as valid parameters
 * 
 * Revision 1.26  89/02/21  23:19:07  rcs
 * fixed invocation of uer routines
 * 
 * Revision 1.25  89/02/09  17:41:29  ejh
 * added copyright notice
 * 
 * Revision 1.24  89/02/07  16:19:02  rcs
 * fixed error with dubug print
 * 
 * Revision 1.23  89/02/06  17:43:31  rcs
 * put undo before id validation
 * 
 * Revision 1.22  89/02/01  14:57:50  rcs
 * modified dra_new to return a boolean adi of TRUE
 * 
 * Revision 1.21  89/01/30  13:45:59  rcs
 * added run time error checking
 * added support for object_sym
 * added dra_get_type routine
 * added dra_get_name routine
 * 
 * Revision 1.20  89/01/04  15:38:33  little
 * propagate undefined parameters in all "user callable" routines
 * 
 * Revision 1.19  88/12/22  11:15:25  rcs
 * nothing in particular
 * 
 * Revision 1.18  88/12/09  17:50:47  little
 * no changes.
 * 
 * Revision 1.17  88/11/11  16:25:03  little
 * change dra_new to really check if something is new.
 * 
 * Revision 1.16  88/11/04  17:02:46  little
 * change DEBUG1 to print d22_slang_source
 * 
 * Revision 1.15  88/10/27  19:10:31  little
 * change dra_delete_temp to use dob_temp_release
 * 
 * Revision 1.14  88/10/27  17:53:10  little
 * add some additional error printout to dra_validate_thingie
 * 
 * Revision 1.13  88/10/26  17:21:40  little
 * add some better debug output to dra_find_thingie
 * 
 * Revision 1.12  88/10/07  15:41:37  little
 * not modified.
 * 
 * Revision 1.11  88/10/03  09:16:42  little
 * modify create_element to take 2 parameters and add delete_element
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	dra_TOP
#define	memoryPack

#include	<signal.h>
#include	<stdio.h>
#include	"string.h"
#include	"global.h"
#include	"listPack.h"
#include	"cus.h"
#include	"uer.h"
#include	"isc.h"
#include	"idd.h"
#include	"ipc.h"
#include	"ipb.h"
#include	"d22_debug.h"
#include	"d22_fatal.h"
#include	"dob.h"
/*
#include	"dob_make.h"
*/
#include	"dob_const.h"
#include	"drc.h"
#include	"dro.h"
#include	"dra.h"

#define MISC_BUF_LEN 	(256)		/* for error msgs, commands, etc. */


/*
??? LOOP_THROUGH_LIST definition should be in an h file
*/
#define LOOP_THROUGH_LIST( list, reg ) \
    for( reg = LISThead( list ); reg != NULLlist; reg = reg -> next )

#define DEBUG1(statement) DM_RT_DEBUG1(dra_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(dra_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(dra_debug, statement)

#define UER_GET_PARENT_VC "get_parent_vc"
#define UER_GET_CREATING_SD "get_bound_sd_instance"
#define UER_GET_NAME "get_name"
#define UER_GET_TYPE "get_declaration_type"
#define UER_GET_VC_ID "get_vc_id"
#define UER_GET_VARIABLE_VALUE "get_variable_value"
#define UER_PUT_VARIABLE_VALUE "put_variable_value"
#define UER_GET_OBJECT "get_object"
#define UER_GET_COMPONENT_VALUE "get_component_value"
#define UER_PUT_COMPONENT_VALUE "put_component_value"
#define UER_CREATE_ELEMENT "create_sd_instance"
#define UER_DESTROY_ELEMENT "destroy_sd_instance"
#define DRA_NEW "new"



/*--------------------------------------------------------------------------*\
|  Routine: dra_validate_thingie
| 
|  Description:  
|     ensures that thingie is of the required type, follows temp "links" to
|	find the "real" thingie
\*--------------------------------------------------------------------------*/
static dob_data_object dra_validate_thingie (in_thingie, required_type)
dob_data_object in_thingie;
cus_sym_type required_type;
{ cus_sym_type symbol_type;
  dob_data_object thingie;

   thingie = in_thingie;
   symbol_type = (cus_sym_type) thingie->obj_sym_type;

   while ((symbol_type == temp_sym) || (symbol_type == constant_sym)) {
					/* symbol type is a temp	    */

      thingie = (dob_data_object) dob_ask (thingie, NULL);
      if (!thingie)
         {fprintf(stderr, "dra_validate_thingie:bad status from while loop"); 
         fprintf(stderr, "; no thingie "); 
         fprintf(stderr, " for dob_get_symbol_type, required_type = %s",
	          cus_type_print_val (required_type));
         fprintf(stderr, " thingie = %d\n", thingie);
   	 return (dob_null_object);
         }
      symbol_type = (cus_sym_type) thingie-> obj_sym_type;
      if (!symbol_type)
         {fprintf(stderr, "dra_validate_thingie:bad status from while loop"); 
         fprintf(stderr, "; no symbol_type"); 
         fprintf(stderr, " for dob_get_symbol_type, required_type = %s",
	          cus_type_print_val (required_type));
         fprintf(stderr, " thingie = %d\n", thingie);
   	 return (dob_null_object);
         }
   }				/* end while symbol type is a temp	    */

   if (symbol_type != required_type)
      thingie = dob_null_object;

   return (thingie);
}

/*--------------------------------------------------------------------------*\
|  Routine: dra_find_thingie
| 
|  Description:  
|     finds a particular thingie (vc,var,object) from a thingie instances list
\*--------------------------------------------------------------------------*/
dob_data_object dra_find_thingie (parent, thingie_type, thingie_name)
dob_data_object parent;
cus_sym_type thingie_type;
string thingie_name;

{  dob_data_object found_thingie;
   LIST instances_list;
   register LIST instance;
   cus_sym parent_sym;
   string original_name;
   dob_data_object the_object, template;

   found_thingie = dob_null_object;

   parent_sym = parent-> ste;

   switch (thingie_type)
      {case variable_sym:
         instances_list = (LIST) cus_get_prop (parent_sym, variables_prop);
         rcheck_status ("dra_find_thingie cannot get variables_prop",
			dob_null_object);
         break;
      case object_sym:
         instances_list = (LIST) cus_get_prop (parent_sym, objects_prop);
         rcheck_status ("dra_find_thingie cannot get objects_prop",
			dob_null_object);
         break;
      case sd_sym:
         instances_list = (LIST) cus_get_prop (parent_sym, created_vcs_prop);
         rcheck_status ("dra_find_thingie cannot get created_vcs_prop",
			dob_null_object);
         break;
      case vc_sym:
         instances_list = (LIST) cus_get_prop (parent_sym, sub_vcs_prop);
         rcheck_status ("dra_find_thingie cannot get created_vcs_prop",
			dob_null_object);
         break;
      default:
         FATAL_ERROR (fprintf (stderr,
		"dra_find_thingie got an unknown type of %s\n",
		cus_type_print_val (thingie_type)));
         break;
      }			/* end switch (thingie_type) */

   if (!instances_list || (list_length (instances_list) == 0))
      goto FINISHED;

   LOOP_THROUGH_LIST (instances_list, instance)
      {the_object = (dob_data_object) LISTnode (instance);
      template = (dob_data_object) the_object-> template;
      original_name = (string) template-> original_name;

      if (!strcmp (original_name, thingie_name))
         {if (found_thingie == dob_null_object)
             found_thingie = (dob_data_object) (LISTnode (instance));
          else
            FATAL_ERROR (fprintf (stderr,"found_thingie found two thingies:");
              fprintf (stderr, "%d & %d\n", LISTnode (instance),
		       found_thingie));
         }
      }

FINISHED:

   return (found_thingie);
}

/*--------------------------------------------------------------------------*\
|  Routine: dra_delete_temp
| 
|  Description:  
|     deletes the parameter adi if it is a temp
\*--------------------------------------------------------------------------*/
void dra_delete_temp (candidate)
dro_atomic_data_item candidate;
{ cus_sym_type symbol_type;
				/* Get the symbol type of the candidate */

   symbol_type = (cus_sym_type) candidate.dob_handle-> obj_sym_type;

				/* Delete the candidate symbol if it is a
				   temp  */
   if (symbol_type == temp_sym)
      {dob_temp_release (candidate.dob_handle);
      check_status("dra_delete_temp: bad status from dob_temp_release");
      }
   return;
}					/* end dra_delete_temp          */


/*--------------------------------------------------------------------------*\
|  Routine: dra_recording_on
| 
|  Description:  
|     turns on transaction recording
\*--------------------------------------------------------------------------*/
void dra_recording_on (file_name, message)
dro_atomic_data_item file_name;
dro_atomic_data_item message;
{  string file_name_string, message_string;
   boolean message_undef, name_undef;

   set_status (ok);
			/* Check for undefined arguments */
   if (!dro_undefined (file_name) && !dro_undefined (message))
			/* arguments are defined, do the rest of the work */
      {file_name_string = (string) dro_ask2 (file_name);
      message_string = (string) dro_ask2 (message);
			/* now that we have the arguments in the correct
			   format, turn on recording */
      ipb_start_recording (file_name_string, message_string);
      }

			/* Delete temp parameters.  */
   dra_delete_temp (file_name);
   dra_delete_temp (message);
   return;
}					/* end dra_recording_on */


/*--------------------------------------------------------------------------*\
|  Routine: dra_recording_off
| 
|  Description:  
|     turns off transaction recording
\*--------------------------------------------------------------------------*/
void dra_recording_off ()
{
   ipb_stop_recording ();
   return;
}					/* end dra_recording_off */


/*--------------------------------------------------------------------------*\
|  Routine: dra_exit
| 
|  Description:  
|     exits a dialogue and notifies parent process
\*--------------------------------------------------------------------------*/
void dra_exit ()
{
				/* notify my parent */
   kill (getppid (), SIGINT);
				/* kill me */
   kill (getpid (), SIGINT);

   return;
}

/*--------------------------------------------------------------------------*\
|  Routine: dra_id_exists
| 
|  Description:  
|     Determines whether the an ID points to something that actually exists.
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_id_exists (id_parm)
dro_atomic_data_item id_parm;
{
  dob_data_object data_object;
  cus_sym_type symbol_type;
  boolean undefined, exists;

  /*
    id_parm represents a container.  What it contains (i.e., points to)
    is what we're ultimately checking for existence.
  */
  set_status(ok);

  exists = false;
					/*
					  Make sure what id_parm contains
					  is not undefined.  (dro_undefined
					  dereferences the id parameter).
					*/
  undefined = dro_undefined (id_parm);
  if (get_status () != ok)
    {
    					/*
					  dro_undefined barfed.  Kill the
					  dialogue.
					*/
    (void) sprintf (uer_error_msg,
      "dra_id_exists: can't tell if id is undefined or not.  Status = %s\n",
      isc_get_status_string ());
    uer_report_da_id_error (uer_error_msg, id_parm);
    }

  else if (! undefined)
    {
    data_object = id_parm.dob_handle;
    symbol_type = dob_get_symbol_type (data_object);
    if ( get_status () != ok )
      {
					/*
					  The container is invalid.  Report
					  error and kill the dialogue.
					*/
      (void) sprintf (uer_error_msg,
      "dra_id_exists: can't get symbol type for id parameter.  Status = %s\n",
	isc_get_status_string ());
      uer_report_da_id_error (uer_error_msg, id_parm);
      }

					/*
					  Keep dereferencing the container
					  through any intermediate temps
					  until we reach the real thingie
					  the container points to.
					*/
    while ((symbol_type == temp_sym) || (symbol_type == constant_sym))
      {
      data_object = (dob_data_object) dob_ask (data_object, NULL);
      if ( get_status () != ok )
	{
					/*
					  Couldn't dereference a container
					  that's supposed to have been okay.
					  Major error.  Kill the dialogue.
					*/
	(void) sprintf (uer_error_msg,
	  "dra_id_exists: can't dereference id.  Status = %s\n",
	  isc_get_status_string ());
	uer_report_da_id_error (uer_error_msg, id_parm);
	}

      else if (! dob_validate_object (data_object) )
	symbol_type = null_sym;

      else
	{
	symbol_type = dob_get_symbol_type (data_object);
	if ( get_status () != ok )
	  {
					/*
					  This should never happen.  Kill
					  the dialogue.
					*/
	  (void) sprintf (uer_error_msg,
	    "dra_id_exists: can't get symbol type for temp.  Status = %s\n",
	    isc_get_status_string ());
	  uer_report_da_id_error (uer_error_msg, id_parm);
	  }
	}
      } /* end while */

    exists = (boolean) (symbol_type != null_sym);
    } /* end else */

  dra_delete_temp (id_parm);
  return ( exists ? dob_bool_TRUE : dob_bool_FALSE );
}					/* end dra_id_exists	    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_get_sub_vc
| 
|  Description:  
|     Gets the id of the named sub-vc for the specified vc
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_sub_vc (parent_vc, vc_template_name)
dro_atomic_data_item parent_vc;
dro_atomic_data_item vc_template_name;

{  dro_atomic_data_item temp;
   dob_data_object real_id, vc_id;
   string vc_template_name_string;
   idd_data_types temp_type;

   set_status(ok);			/* begin			    */

				/* Create a temp to return the result in,
				   and set its initial value to undefined  */
   temp.component_name = NULL;
   temp.dob_handle = dob_temp_create();
   rcheck_status("dra_get_sub_vc: bad status from dob_temp_create", temp);
   dob_variable_tell (temp.dob_handle, idd_undefined, NULL);
   rcheck_status (
      "dra_get_sub_vc: bad status from dob_variable_tell of undefined", temp);

				/* initialize "returned" values */
   temp_type = idd_undefined;
   vc_id = dob_null_object;

				/* check arguments for undefinedness */
   if (dro_undefined (parent_vc) || dro_undefined (vc_template_name))
      goto FINISHED;

				/* Check for valid id type of vc */
   if ((real_id = dra_validate_thingie (parent_vc.dob_handle, vc_sym)) ==
	dob_null_object)
      {set_status(invalid_symbol);
      fprintf (stderr, "dra_get_sub_vc: status = %d\n", get_status ());
      goto FINISHED;
      }

   vc_template_name_string = (string)dro_ask2 (vc_template_name);
   vc_id = dra_find_thingie (real_id, vc_sym, vc_template_name_string);
   if (vc_id != dob_null_object)
      temp_type = idd_id;

FINISHED:
		/* Update the temp with the appropriate value and type */
   dob_variable_tell (temp.dob_handle, temp_type, (char *) vc_id);
   rcheck_status( "dra_get_sub_vc: bad status from dob_variable_tell", temp);
 
				/* delete parameters, if temp */
   dra_delete_temp (parent_vc);
   rcheck_status (
      "dra_get_sub_vc: bad status from dra_delete_temp for parent_vc", temp);
   dra_delete_temp (vc_template_name);
   rcheck_status (
      "dra_get_sub_vc: bad status from dra_delete_temp for vc_template_name",
      temp);

   return temp;
}					/* end dra_get_sub_vc	    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_wheres_poppa
| 
|  Description:  
|     Gets the id of the parent vc for the specified vc or object. 
|     Used internally only.
\*--------------------------------------------------------------------------*/
static dob_data_object dra_wheres_poppa (self)
dob_data_object self;
{
    cus_sym symbol_table_entry;
    dob_data_object real_id, parent_id;
    dro_atomic_data_item id_parm;
    dob_data_object poppa;

    /* 
     * Generate an error for any symbol that is not a vc, object
     * or variable.
     */
    if (((real_id=dra_validate_thingie(self, vc_sym)) == dob_null_object) &&
      ((real_id=dra_validate_thingie(self, object_sym)) == dob_null_object) &&
      ((real_id=dra_validate_thingie(self, variable_sym)) == dob_null_object)) {

	sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_GET_PARENT_VC);
	uer_report_da_id_error(uer_error_msg, id_parm);
	}
    /* 
     * Get handle for symbol table entry.  
     */
    symbol_table_entry = dob_get_symbol_entry(real_id);
    uer_rcheck_status_da_id(
	    "dra_wheres_poppa: bad status from dob_get_symbol_entry", 
	    UER_GET_PARENT_VC, self);
    /* 
     * Get the parent vc property for the object.
     */
    poppa = (dob_data_object)cus_get_prop(symbol_table_entry, parent_vc_prop);
    uer_rcheck_status_da_id(
	    "dra_wheres_poppa: bad status from cus_get_prop",
	    UER_GET_PARENT_VC, self);
    return poppa;
}

/*--------------------------------------------------------------------------*\
|  Routine: dra_get_parent_vc
| 
|  Description:  
|     Gets the id of the parent vc for the specified vc or object. 
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_parent_vc(id_parm)
dro_atomic_data_item id_parm;
{
    dro_atomic_data_item temp;
    cus_sym symbol_table_entry;
    dob_data_object real_id, parent_id;
    boolean id_parm_undef;

    set_status(ok);
    /* 
     * Create a temp to return the result in.
     */
    temp.component_name = NULL;

    temp.dob_handle = dob_temp_create();
    uer_rcheck_status_da_id(
	    "dro_get_parent_vc: bad status from dob_temp_create",
	    UER_GET_PARENT_VC, temp);
    /* 
     * Check for undefined argument.
     */
    id_parm_undef = dro_undefined(id_parm);
    uer_rcheck_status_da_id(
	    "dro_get_parent_vc: bad status from dro_undefined", 
	    UER_GET_PARENT_VC, temp);
    /*
     * If argument is undefined propogate undefined.
     */
    if (id_parm_undef) {
	/* 
	* Propogate undefined.
	*/
	dob_variable_tell(temp.dob_handle, idd_undefined, 0);
	uer_rcheck_status_da_id(
		"dra_get_parent_vc: bad status from dob_variable_tell",
		UER_GET_PARENT_VC, temp);
	}
    else {
	parent_id = dra_wheres_poppa (id_parm.dob_handle);
	/* 
	 * Update the temp with the appropriate value and type 
	 */
	dob_variable_tell(temp.dob_handle, idd_id, parent_id);
	uer_rcheck_status_da_id(
		"dra_get_parent_vc: bad status from dob_variable_tell",
		UER_GET_PARENT_VC, temp);
	}
    /* 
     * Delete id, if temp 
     */
    dra_delete_temp(id_parm);
    uer_rcheck_status_da_id(
	    "dra_get_parent_vc: bad status from dra_delete_temp",
	    UER_GET_PARENT_VC, temp);

    return temp;
}

/*--------------------------------------------------------------------------*\
|  Routine: dra_get_PARENT_vc
| 
|  Description:  
|     Gets the id of the parent vc for the specified vc or object. 
\*--------------------------------------------------------------------------*/
void dra_get_PARENT_vc(parent, self)
dro_atomic_data_item *parent;
dob_data_object self;
{
  cus_sym symbol_table_entry;
  dob_data_object real_id, parent_id;

  set_status(ok);
  /* 
   *  Use constant to return the result in (done once only per PARENT).
   */
  if (parent->dob_handle == NULL) {
      parent->dob_handle = dob_CONSTANT_create();
      check_status("dro_get_PARENT_vc: bad status from dob_CONSTANT_create");
      }
  parent_id = dra_wheres_poppa (self);
  /* 
   *  Update the *parent with the appropriate value and type 
   */
  dob_variable_tell(parent->dob_handle, idd_id, parent_id);
  check_status("dra_get_PARENT_vc: bad status from dob_variable_tell");
}


/*--------------------------------------------------------------------------*\
|  Routine: dra_get_sire_vc
| 
|  Description:  
|     Gets the id of the N'th parent vc for the specified vc or object. 
| 
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_sire_vc (id_parm, generation)
dro_atomic_data_item id_parm;
int generation;
{
    dro_atomic_data_item temp;
    cus_sym symbol_table_entry;
    dob_data_object real_id, parent_id;
    boolean id_parm_undef;

    set_status(ok);
    /* 
     * Create a temp to return the result in.
     */
    temp.component_name = NULL;

    temp.dob_handle = dob_temp_create();
    uer_rcheck_status_da_id(
	    "dro_get_sire_vc: bad status from dob_temp_create",
	    UER_GET_PARENT_VC, temp);
    /* 
     * Check for undefined argument.
     */
    id_parm_undef = dro_undefined(id_parm);
    uer_rcheck_status_da_id(
	    "dro_get_sire_vc: bad status from dro_undefined", 
	    UER_GET_PARENT_VC, temp);
    /*
     * If argument is undefined propogate undefined.
     */
    if (id_parm_undef) {
	/* 
	* Propogate undefined.
	*/
	dob_variable_tell(temp.dob_handle, idd_undefined, 0);
	uer_rcheck_status_da_id(
		"dra_get_sire_vc: bad status from dob_variable_tell",
		UER_GET_PARENT_VC, temp);
	}
    else {
	parent_id = dra_wheres_poppa (id_parm.dob_handle, TRUE);
	while (--generation)
	    parent_id = dra_wheres_poppa (parent_id, FALSE);
	/* 
	 * Update the temp with the appropriate value and type 
	 */
	dob_variable_tell(temp.dob_handle, idd_id, parent_id);
	uer_rcheck_status_da_id(
		"dra_get_sire_vc: bad status from dob_variable_tell",
		UER_GET_PARENT_VC, temp);
	}
    /* 
     * Delete id, if temp 
     */
    dra_delete_temp(id_parm);
    uer_rcheck_status_da_id(
	    "dra_get_sire_vc: bad status from dra_delete_temp",
	    UER_GET_PARENT_VC, temp);

    return temp;
}

/*--------------------------------------------------------------------------*\
|  Routine: dra_get_creating_sd
| 
|  Description:  
|     Gets the id of the shared data element which causes the specified
|     vc to be created.
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_creating_sd(id_parm)
dro_atomic_data_item id_parm;
{  
  dro_atomic_data_item temp;
  cus_sym symbol_table_entry;
  dob_data_object real_vc_id, sd_id;
  boolean id_parm_undef;

  set_status(ok);			/* begin			    */
/* 
   Create a temp to return the result in.
*/
  temp.component_name = NULL;
  temp.dob_handle = dob_temp_create();
  uer_rcheck_status_da_id(
	"dro_get_creating_sd: bad status from dob_temp_create", 
	UER_GET_CREATING_SD, temp);
/*
   Check for undefined argument.
*/
  id_parm_undef = dro_undefined(id_parm);
  uer_rcheck_status_da_id(
    "dro_get_creating_sd: bad status from dro_undefined", 
    UER_GET_CREATING_SD, temp);
/*
   If parameter is undefined then result is undefined.
*/  
  if (id_parm_undef) {             	 	/* id_parm undefined        */
/*
   Propogate undefined.
*/
    dob_variable_tell(temp.dob_handle, idd_undefined, 0);
    uer_rcheck_status_da_id(
      "dro_get_creating_sd: bad status from dob_variable_tell",
      UER_GET_CREATING_SD, temp);
  }                                     /* end if undefined                 */
  else {                                /* id defined                       */
/* 
   Generate an error for any symbol type other than a vc.
*/
    if ((real_vc_id = dra_validate_thingie(id_parm.dob_handle, vc_sym)) ==
	dob_null_object) { 

      fprintf(stderr, "dra_get_creating_sd: id has invalid symbol type.\n");

      sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_GET_CREATING_SD);
      uer_report_da_id_error(uer_error_msg, id_parm);
    }						/* end if invalid type	    */
/* 
   Get handle for symbol table entry.  
*/
    symbol_table_entry = dob_get_symbol_entry(real_vc_id);
    uer_rcheck_status_da_id(
      "dra_get_creating_sd: bad status from dob_get_symbol_entry", 
      UER_GET_CREATING_SD, temp);
/*
   Get the parent vc property for the object and update temp.
*/
    sd_id = (dob_data_object)cus_get_prop(symbol_table_entry, creating_sd_prop);
    uer_rcheck_status_da_id(
      "dra_get_creating_sd: bad status from cus_get_prop", 
      UER_GET_CREATING_SD, temp);
/* 
   Update the temp with the appropriate value and type.
*/
    dob_variable_tell(temp.dob_handle, idd_id, (char *)sd_id);
    uer_rcheck_status_da_id(
      "dra_get_creating_sd: bad status from dob_variable_tell",
      UER_GET_CREATING_SD, temp);
  }                                     /* end else id defined              */
/* 
   delete parameter, if temp 
*/
  dra_delete_temp(id_parm);
  uer_rcheck_status_da_id(
    "dra_get_creating_sd: bad status from dra_delete_temp", 
    UER_GET_CREATING_SD, temp);

  return temp;
}					/* end dra_get_creating_sd	    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_get_vc_id
| 
|  Description:  
|     Gets the id of a specific vc "tied" to an sd
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_vc_id(id_parm, name_parm)
dro_atomic_data_item id_parm;
dro_atomic_data_item name_parm;
{  
  dro_atomic_data_item temp;
  dob_data_object real_sd_id, created_vc_id;
  string vc_template_name_string;
  idd_data_types temp_type;
  boolean id_parm_undef, name_parm_undef;

  set_status(ok);			/* begin			    */
/* 
   Create a temp to return the result in.
*/
  temp.component_name = NULL;
  temp.dob_handle = dob_temp_create();
  uer_rcheck_status_da_id_str(
    "dro_get_vc_id: bad status from dob_temp_create", 
    UER_GET_VC_ID, temp);
/*
   Check for undefind argument.
*/
  id_parm_undef = dro_undefined(id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_vc_id: bad status from dro_undefined for id_parm", 
    UER_GET_VC_ID, temp);

  name_parm_undef = dro_undefined(name_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_vc_id: bad status from dro_undefined for name_parm", 
    UER_GET_VC_ID, temp);
/*
   If either argument is undefined then the result is undefined.
*/
  if (id_parm_undef || name_parm_undef) {	/* an argument is undefined */

    dob_variable_tell(temp.dob_handle, idd_undefined, NULL);
    uer_rcheck_status_da_id_str(
      "dra_get_vc_id: bad status from dob_variable_tell for undefined", 
      UER_GET_VC_ID, temp);
  }					/* end if an argument is undefined  */
  else {				/* defined arguments		    */
/*
   Generate an error for any symbol type other than sd.
*/
    if ((real_sd_id = dra_validate_thingie(id_parm.dob_handle, sd_sym)) ==
	dob_null_object) {

      fprintf(stderr, "dra_get_vc_id: id has invalid symbol type.\n");
 
      sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_GET_VC_ID);
      uer_report_da_id_error(uer_error_msg, id_parm);
    }						/* end if invalid type	    */
/*
   Get a pointer to the vc template name we're looking for.
*/
    vc_template_name_string = (string)dro_ask2(name_parm);
    uer_rcheck_status_da_id_str(
      "dra_get_vc_id: bad status from dro_ask2 for name_parm", 
      UER_GET_VC_ID, temp);
/* 
   Find the particular vc.
*/
    created_vc_id = dra_find_thingie(real_sd_id, sd_sym,
	vc_template_name_string);
    uer_rcheck_status_da_id_str(
      "dra_get_vc_id: bad status from dra_find_thingie for created_vc_id", 
      UER_GET_VC_ID, temp);
/*
   If vc found update the temp.
*/
    if (created_vc_id != dob_null_object) {	/* vc found		    */
/* 
   Update the temp with the appropriate value and type 
*/
      dob_variable_tell(temp.dob_handle, idd_id, (char *)created_vc_id);
      uer_rcheck_status_da_id_str(
        "dra_get_vc_id: bad status from dob_variable_tell",
        UER_GET_VC_ID, temp);
    }						/* end if vc found	    */
/*
   Else vc not found report error condition.
*/
    else {				/* vc not found			    */

      fprintf(stderr, "dra_get_vc_id: vc not found.\n");

      sprintf(uer_error_msg, UER_NOT_FOUND, UER_GET_VC_ID);
      uer_report_da_id_error(uer_error_msg, id_parm);
    }					/* end else vc not found	    */
  }                                     /* end else arguments defined       */
/* 
   Delete temp parameters.
*/
  dra_delete_temp(id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_vc_id: bad status from dra_delete_temp id_parm", 
    UER_GET_VC_ID, temp);

  dra_delete_temp (name_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_vc_id: bad status from dra_delete_temp name_parm", 
    UER_GET_VC_ID, temp);

  return temp;
}					/* end dra_get_vc_id		    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_get_name
| 
|  Description:  
|     Gets the name of a view controller template, shared data item or
|     object.
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_name(id_parm)
dro_atomic_data_item id_parm;
{					/* begin			    */
  dro_atomic_data_item temp;
  cus_sym symbol_table_entry;
  dob_data_object real_id, template_id;
  string name;
  cus_sym_type symbol_type;
  boolean id_parm_undef;

  set_status(ok);
/* 
   Create a temp to return the result in.
*/
  temp.component_name = NULL;

  temp.dob_handle = dob_temp_create();
  uer_rcheck_status_da_id("dra_get_name: bad status from dob_temp_create", 
    UER_GET_NAME, temp);
/* 
   Check for undefined argument.
*/
  id_parm_undef = dro_undefined(id_parm);
  uer_rcheck_status_da_id("dra_get_name: bad status from dro_undefined", 
    UER_GET_NAME, temp);

  if (id_parm_undef) {		/* id_parm undefined			    */
/* 
   Propogate undefined.
*/
    dob_variable_tell(temp.dob_handle, idd_undefined, 0);
    uer_rcheck_status_da_id("dra_get_name: bad status from dob_variable_tell",
      UER_GET_NAME, temp);
  }					/* end if undefined		    */
  else {				/* id_parm defined		    */
/* 
   Generate an error for a symbol type other than vc, object or variable.
*/
    if (((real_id = dra_validate_thingie(id_parm.dob_handle, vc_sym)) ==
	dob_null_object)
      && ((real_id = dra_validate_thingie(id_parm.dob_handle, object_sym)) ==
	dob_null_object)
      && ((real_id = dra_validate_thingie(id_parm.dob_handle, variable_sym)) ==
	dob_null_object))
    {

      fprintf(stderr, "dra_get_name: invalid symbol type.\n");

      sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_GET_NAME);
      uer_report_da_id_error(uer_error_msg, id_parm);
    }

/*
   Get the name.
*/
    name = dob_get_real_name(real_id);
    uer_rcheck_status_da_id("dra_get_name: bad status from dob_get_name", 
      UER_GET_NAME, temp);
/* 
   Update the temp with the appropriate value and type.
*/
    dob_variable_tell(temp.dob_handle, idd_string, name);
    uer_rcheck_status_da_id("dra_get_name: bad status from dob_variable_tell",
      UER_GET_NAME, temp);
  }					/* end else id_parm defined 	    */
/* 
   Delete id_parm, if temp 
*/
  dra_delete_temp(id_parm);
  uer_rcheck_status_da_id("dra_get_name: bad status from dra_delete_temp",
    UER_GET_NAME, temp);

  return temp;
}					/* end dra_get_name		    */


/*--------------------------------------------------------------------------*\
|  Routine: dra_get_type
| 
|  Description:  
|     Gets the declaration type of the id.  Valid types are vc, object, 
|     sd and variable.
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_type(id_parm)
dro_atomic_data_item id_parm;
{					/* begin			    */
  dro_atomic_data_item temp;
  cus_sym symbol_table_entry;
  cus_sym_type symbol_type;
  dob_data_object real_id, template_id;
  string original_name;
  boolean id_parm_undef;

  set_status(ok);
/* 
   Create a temp to return the result in.
*/
  temp.component_name = NULL;

  temp.dob_handle = dob_temp_create();
  uer_rcheck_status_da_id(
    "dro_get_parent_vc: bad status from dob_temp_create", 
    UER_GET_TYPE, temp);
/* 
   Check for undefined argument.
*/
  id_parm_undef = dro_undefined(id_parm);
  uer_rcheck_status_da_id("dro_get_parent_vc: bad status from dro_undefined", 
    UER_GET_TYPE, temp);

  if (id_parm_undef) {		/* id undefined			    */
/* 
   Propogate undefined.
*/
    dob_variable_tell(temp.dob_handle, idd_undefined, 0);
    uer_rcheck_status_da_id("dra_get_type: bad status from dob_variable_tell",
      UER_GET_TYPE, temp);
  }					/* end if undefined		    */
  else {				/* id_parm defined		    */
/* 
   Check for valid id type 
*/
    if (((real_id = dra_validate_thingie(id_parm.dob_handle, vc_sym)) ==
	dob_null_object)
      && ((real_id = dra_validate_thingie(id_parm.dob_handle, sd_sym)) ==
	dob_null_object)
      && ((real_id = dra_validate_thingie(id_parm.dob_handle, variable_sym)) ==
	dob_null_object)
      && ((real_id = dra_validate_thingie(id_parm.dob_handle, object_sym)) ==
	dob_null_object))
    {

      fprintf(stderr, "dra_get_type:  invalid symbol type.\n");

      sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_GET_TYPE);
      uer_report_da_id_error(uer_error_msg, id_parm);
  } 
/*
   Get the symbol type for the specified data item.
*/
    symbol_type = dob_get_symbol_type(real_id);
    uer_rcheck_status_da_id(
      "dra_get_type: bad status from dob_get_symbol_type",
      UER_GET_TYPE, temp);
/* 
   Update the temp with the appropriate value and type.
*/
    dob_variable_tell(temp.dob_handle, idd_string,
      cus_type_print_val(symbol_type));
    uer_rcheck_status_da_id("dra_get_type: bad status from dob_variable_tell",
      UER_GET_TYPE, temp);
  }					/* end else id_parm defined	    */
/* 
   Delete id_parm, if temp 
*/
  dra_delete_temp(id_parm);
  uer_rcheck_status_da_id("dra_get_type: bad status from dra_delete_temp",
    UER_GET_TYPE, temp);

  return temp;
}					/* end dra_get_type		    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_get_OBJECT
| 
|  Description:  
|     Gets the id of the named object within a vc instance.  Uses a constant
|     parameter for the name.
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_OBJECT(id_parm, name_parm)
dro_atomic_data_item id_parm;
string name_parm;
{
  dro_atomic_data_item temp;
  dob_data_object real_vc_id, object_id;
  string object_name_string;
  boolean id_parm_undef;

  set_status(ok);			/* begin			    */
/*  
   Create a temp to return the result in.
*/
  temp.component_name = NULL;

  temp.dob_handle = dob_temp_create();
  uer_rcheck_status_da_id_str(
    "dro_get_object: bad status from dob_temp_create", 
    UER_GET_OBJECT, temp);
/* 
   Check for undefined arguments.
*/
  id_parm_undef = dro_undefined(id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_OBJECT: bad status from dro_undefined for id_parm", 
    UER_GET_OBJECT, temp);

  if (id_parm_undef) {		/* undefined arguments	    */

    dob_variable_tell (temp.dob_handle, idd_undefined, NULL);
    uer_rcheck_status_da_id_str(
      "dra_get_OBJECT: bad status from dob_variable_tell for undefined", 
      UER_GET_OBJECT, temp);

  }					/* end if undefined arguments	    */
  else {				/* arguments are defined	    */
/* 
   Generate an error for anything other than an object symbol.  
*/
    if ((real_vc_id = dra_validate_thingie (id_parm.dob_handle, vc_sym)) ==
	dob_null_object) {

      fprintf(stderr,"dra_get_OBJECT: id has invalid symbol type.\n");

      sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_GET_OBJECT);
      uer_report_da_id_str_error(uer_error_msg, id_parm,
				dob_make_string_constant_adi(name_parm));
    }						/* end if invalid type	    */
/* 
   Get a pointer to the object name.  
*/
    object_id = dra_find_thingie(real_vc_id, object_sym, name_parm);
    uer_rcheck_status_da_id_str(
      "dra_get_OBJECT: bad status from dra_find_thingie for object_id", 
      UER_GET_OBJECT, temp);
/*
   Real object.
*/
    if (object_id != dob_null_object) {	/* object found			    */
/* 
   Update the temp with the appropriate value and type 
*/
      dob_variable_tell(temp.dob_handle, idd_id, (char *)object_id);
      uer_rcheck_status_da_id_str(
        "dra_get_OBJECT: bad status from dob_variable_tell",
        UER_GET_OBJECT, temp);
    }				/* end if object found			    */
/*
   Else object not found report error condition.
*/
    else {				/* object not found		    */

      fprintf(stderr, "dra_get_OBJECT: vc not found.\n");

      sprintf(uer_error_msg, UER_NOT_FOUND, UER_GET_OBJECT);
      uer_report_da_id_str_error(uer_error_msg, id_parm,
				dob_make_string_constant_adi(name_parm));
    }					/* end else object not found	    */
  }                                     /* end else arguments defined       */
/*
   Delete temp parameters.
*/
  dra_delete_temp (id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_OBJECT: bad status from delete_temp for id_parm",
    UER_GET_OBJECT, temp);

  return temp;
}					/* end dra_get_OBJECT	    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_get_object
| 
|  Description:  
|     Gets the id of the named object within a vc instance.
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_object(id_parm, name_parm)
dro_atomic_data_item id_parm;
dro_atomic_data_item name_parm;
{  
  dro_atomic_data_item temp;
  dob_data_object real_vc_id, object_id;
  string object_name_string;
  boolean id_parm_undef, name_parm_undef;

  set_status(ok);			/* begin			    */
/*  
   Create a temp to return the result in.
*/
  temp.component_name = NULL;

  temp.dob_handle = dob_temp_create();
  uer_rcheck_status_da_id_str(
    "dro_get_object: bad status from dob_temp_create", 
    UER_GET_OBJECT, temp);
/* 
   Check for undefined arguments.
*/
  id_parm_undef = dro_undefined(id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_object: bad status from dro_undefined for id_parm", 
    UER_GET_OBJECT, temp);

  name_parm_undef = dro_undefined(name_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_object: bad status from dro_undefined for name_parm", 
    UER_GET_OBJECT, temp);

  if (id_parm_undef || name_parm_undef) {	/* undefined arguments	    */

    dob_variable_tell (temp.dob_handle, idd_undefined, NULL);
    uer_rcheck_status_da_id_str(
      "dra_get_object: bad status from dob_variable_tell for undefined", 
      UER_GET_OBJECT, temp);
  }					/* end if undefined arguments	    */
  else {				/* arguments are defined	    */
/* 
   Generate an error for anything other than an object symbol.  
*/
    if ((real_vc_id = dra_validate_thingie (id_parm.dob_handle, vc_sym)) ==
	dob_null_object) {

      fprintf(stderr,"dra_get_object: id has invalid symbol type.\n");

      sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_GET_OBJECT);
      uer_report_da_id_str_error(uer_error_msg, id_parm, name_parm);
    }						/* end if invalid type	    */
/* 
   Get a pointer to the object name.  
*/
    object_name_string = (string)dro_ask2(name_parm);
    uer_rcheck_status_da_id_str(
      "dra_get_object: bad status from dro_ask2 for name_parm", 
      UER_GET_OBJECT, temp);

    object_id = dra_find_thingie(real_vc_id, object_sym, object_name_string);
    uer_rcheck_status_da_id_str(
      "dra_get_object: bad status from dra_find_thingie for object_id", 
      UER_GET_OBJECT, temp);
/*
   Real object.
*/
    if (object_id != dob_null_object) {	/* object found			    */
/* 
   Update the temp with the appropriate value and type 
*/
      dob_variable_tell(temp.dob_handle, idd_id, (char *)object_id);
      uer_rcheck_status_da_id_str(
        "dra_get_object: bad status from dob_variable_tell",
        UER_GET_OBJECT, temp);
    }				/* end if object found			    */
/*
   Else object not found report error condition.
*/
    else {				/* object not found		    */

      fprintf(stderr, "dra_get_object: vc not found.\n");

      sprintf(uer_error_msg, UER_NOT_FOUND, UER_GET_OBJECT);
      uer_report_da_id_str_error(uer_error_msg, id_parm, name_parm);
    }					/* end else object not found	    */
  }                                     /* end else arguments defined       */
/*
   Delete temp parameters.
*/
  dra_delete_temp (id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_object: bad status from delete_temp for id_parm",
    UER_GET_OBJECT, temp);

  dra_delete_temp (name_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_object: bad status from delete_temp for name_parm",
    UER_GET_OBJECT, temp);

  return temp;
}					/* end dra_get_object	    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_get_component_VALUE
| 
|  Description:  
|     Gets the value of the specified component. Uses a constant for the
|     name_parm.
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_component_VALUE(id_parm, name_parm)
dro_atomic_data_item id_parm;
string name_parm;
{					/* local type definition	    */
  dro_atomic_data_item temp;
  dob_data_object real_sd_id;
  idd_data_types temp_type;
  caddr_t temp_value;
  boolean id_parm_undef;

  set_status(ok);			/* begin			    */
/* 
   Create a temp to return the result in.
*/
  temp.component_name = NULL;

  temp.dob_handle = dob_temp_create();
  uer_rcheck_status_da_id_str(
    "dra_get_comp_VALUE: bad status from dob_temp_create", 
    UER_GET_COMPONENT_VALUE, temp);
/*
   Check for undefined arguments.
*/
  id_parm_undef = dro_undefined(id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_comp_VALUE: bad status from dro_undefined for id_parm", 
    UER_GET_COMPONENT_VALUE, temp);

  if (id_parm_undef) {
    dob_variable_tell (temp.dob_handle, idd_undefined, NULL);
    uer_rcheck_status_da_id_str(
     "dra_get_comp_VALUE: bad status from dob_variable_tell for undefined", 
      UER_GET_COMPONENT_VALUE, temp);
  }                                     /* end if undefined arguments       */
  else {                                /* arguments are defined            */
/* 
   Generate an error for any symbol type other than shared data or an 
   object.
*/
    if (((real_sd_id = dra_validate_thingie(id_parm.dob_handle, sd_sym)) ==
	dob_null_object)
      && ((real_sd_id = dra_validate_thingie(id_parm.dob_handle, object_sym)) ==
	dob_null_object))
    {
      fprintf(stderr, "dra_get_comp_VALUE: id has invalid symbol type.\n");
      sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_GET_COMPONENT_VALUE);
      uer_report_da_id_str_error(uer_error_msg, id_parm,
				dob_make_string_constant_adi(name_parm));
    }                                           /* end if invalid type      */
/* 
   Get the type and value of the specified component.  
*/
    temp_type = dob_get_base_data_type(real_sd_id, name_parm);
    uer_rcheck_status_da_id_str(
      "dra_get_comp_VALUE: bad status from get_data_type", 
      UER_GET_COMPONENT_VALUE, temp);

    temp_value = dob_ask(real_sd_id, name_parm);
     uer_rcheck_status_da_id_str(
      "dra_get_comp_VALUE: bad status from dob_ask", 
      UER_GET_COMPONENT_VALUE, temp);
/* 
   Update the temp with the appropriate value and type.  
*/
    dob_variable_tell (temp.dob_handle, temp_type, temp_value);
    uer_rcheck_status_da_id_str ( 
      "dra_get_comp_VALUE: bad status from dob_variable_tell", 
      UER_GET_COMPONENT_VALUE, temp);
  }                                     /* end else arguments defined       */
/*
   Delete temp parameters.
*/
  dra_delete_temp (id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_comp_VALUE: bad status from dra_delete_temp sd_id",
    UER_GET_COMPONENT_VALUE, temp);

  return temp;
}					/* end dra_get_comp_VALUE	    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_get_component_value
| 
|  Description:  
|     Gets the value of the specified component.
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_component_value(id_parm, name_parm)
dro_atomic_data_item id_parm;
dro_atomic_data_item name_parm;
{					/* local type definition	    */
  dro_atomic_data_item temp;
  dob_data_object real_sd_id;
  idd_data_types temp_type;
  string component_name_string;
  caddr_t temp_value;
  boolean id_parm_undef, name_parm_undef;

  set_status(ok);			/* begin			    */
/* 
   Create a temp to return the result in.
*/
  temp.component_name = NULL;

  temp.dob_handle = dob_temp_create();
  uer_rcheck_status_da_id_str(
    "dra_get_comp_value: bad status from dob_temp_create", 
    UER_GET_COMPONENT_VALUE, temp);
/*
   Check for undefined arguments.
*/
  id_parm_undef = dro_undefined(id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_comp_value: bad status from dro_undefined for id_parm", 
    UER_GET_COMPONENT_VALUE, temp);

  name_parm_undef = dro_undefined(name_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_comp_value: bad status from dro_undefined for name_parm", 
    UER_GET_COMPONENT_VALUE, temp);

  if (id_parm_undef || name_parm_undef) {

    dob_variable_tell (temp.dob_handle, idd_undefined, NULL);
    uer_rcheck_status_da_id_str(
     "dra_get_comp_value: bad status from dob_variable_tell for undefined", 
      UER_GET_COMPONENT_VALUE, temp);
  }                                     /* end if undefined arguments       */
  else {                                /* arguments are defined            */
/* 
   Generate an error for any symbol type other than shared data or an 
   object.
*/
    if (((real_sd_id = dra_validate_thingie(id_parm.dob_handle, sd_sym)) ==
	dob_null_object)
      && ((real_sd_id = dra_validate_thingie(id_parm.dob_handle, object_sym)) ==
	dob_null_object))
    {
      sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_GET_COMPONENT_VALUE);
      uer_report_da_id_str_error(uer_error_msg, id_parm, name_parm);
    }                                           /* end if invalid type      */
/* 
   Get the component name string.  
*/
    component_name_string = (string)dro_ask2(name_parm);
    uer_rcheck_status_da_id_str(
      "dra_get_comp_value: bad status from dro_ask2", 
      UER_GET_COMPONENT_VALUE, temp);
/* 
   Get the type and value of the specified component.  
*/
    temp_type = dob_get_base_data_type(real_sd_id, component_name_string);
    uer_rcheck_status_da_id_str(
      "dra_get_comp_value: bad status from get_data_type", 
      UER_GET_COMPONENT_VALUE, temp);

    temp_value = dob_ask(real_sd_id, component_name_string);
     uer_rcheck_status_da_id_str(
      "dra_get_comp_value: bad status from dob_ask", 
      UER_GET_COMPONENT_VALUE, temp);
/* 
   Update the temp with the appropriate value and type.  
*/
    dob_variable_tell (temp.dob_handle, temp_type, temp_value);
    uer_rcheck_status_da_id_str ( 
      "dra_get_comp_value: bad status from dob_variable_tell", 
      UER_GET_COMPONENT_VALUE, temp);
  }                                     /* end else arguments defined       */
/*
   Delete temp parameters.
*/
  dra_delete_temp (id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_comp_value: bad status from dra_delete_temp sd_id",
    UER_GET_COMPONENT_VALUE, temp);
  
  dra_delete_temp (name_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_comp_value: bad status from dra_delete_temp name_parm",
    UER_GET_COMPONENT_VALUE, temp);

  return temp;
}					/* end dra_get_comp_value	    */

/*
??? test setting an attribute to self
*/
/*--------------------------------------------------------------------------*\
|  Routine: dra_put_component_VALUE
| 
|  Description:  
|     Puts a specified value into a component of a shared data element.  Uses
|     a constant for the name_parm.
\*--------------------------------------------------------------------------*/
void dra_put_component_VALUE(id_parm, name_parm, value)
dro_atomic_data_item id_parm;
string name_parm;
dro_atomic_data_item value;
{					/* local type definition	    */
  dob_data_object real_sd_id;
  idd_data_types temp_type;
  caddr_t temp_value;
  boolean id_parm_undef;

  set_status(ok);			/* begin			    */
/*
   Generate an error for anything other than a shared data or object symbol.
*/
  if ( ((real_sd_id = dra_validate_thingie(id_parm.dob_handle, sd_sym)) ==
	dob_null_object)
    && ((real_sd_id = dra_validate_thingie(id_parm.dob_handle, object_sym)) ==
	dob_null_object))
  {sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_PUT_COMPONENT_VALUE);
    uer_report_da_id_str_val_error(uer_error_msg, id_parm,
			dob_make_string_constant_adi(name_parm), value);
  }                                           /* end if invalid type      */
/*
   Get the base to type to get the actual typed value of the stuff to be 
   stored.
*/
  temp_type = dob_get_base_data_type(real_sd_id, name_parm);
  uer_check_status_da_id_str_val(
    "dra_put_comp_VALUE: bad status from dob_get_base_data_type",
    UER_PUT_COMPONENT_VALUE);

  temp_value = drc_get_atv(value, temp_type);
  uer_check_status_da_id_str_val(
    "dra_put_comp_VALUE: bad status from drc_get_atv",
    UER_PUT_COMPONENT_VALUE);
/*
   Assign the value to the specified component.
*/
  dob_shared_data_tell(real_sd_id, name_parm, temp_value);
  uer_check_status_da_id_str_val(
    "dra_put_comp_VALUE: bad status from dob_shared_data_tell",
    UER_PUT_VARIABLE_VALUE);
/* 
   Delete the parameters if they are temps 
*/
  dra_delete_temp(id_parm);
  uer_check_status_da_id_str_val(
    "dra_put_comp_VALUE: bad status from dra_delete_temp id_parm",
    UER_PUT_COMPONENT_VALUE);

  dra_delete_temp (value);
  uer_check_status_da_id_str_val(
    "dra_put_comp_VALUE: bad status from dra_delete_temp",
    UER_PUT_COMPONENT_VALUE);

  return;
}					/* end dra_put_comp_VALUE	    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_put_component_value
| 
|  Description:  
|     Puts a specified value into a component of a shared data element.
\*--------------------------------------------------------------------------*/
void dra_put_component_value(id_parm, name_parm, value)
dro_atomic_data_item id_parm, name_parm, value;
{					/* local type definition	    */
  dob_data_object real_sd_id;
  string component_name_string;
  idd_data_types temp_type;
  caddr_t temp_value;
  boolean id_parm_undef, name_parm_undef;

  set_status(ok);			/* begin			    */
/*
   Generate an error for anything other than a shared data or object symbol.
*/
  if ( ((real_sd_id = dra_validate_thingie(id_parm.dob_handle, sd_sym)) ==
	dob_null_object)
    && ((real_sd_id = dra_validate_thingie(id_parm.dob_handle, object_sym)) ==
	dob_null_object))
  {sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_PUT_COMPONENT_VALUE);
    uer_report_da_id_str_val_error(uer_error_msg, id_parm, name_parm, value);
  }                                           /* end if invalid type      */
/* 
   Get the component name string.  
*/
  component_name_string = (string)dro_ask2(name_parm);
  uer_check_status_da_id_str_val(
    "dra_put_comp_value: bad status from dro_ask2", UER_PUT_COMPONENT_VALUE);
/*
   Get the base to type to get the actual typed value of the stuff to be 
   stored.
*/
  temp_type = dob_get_base_data_type(real_sd_id, component_name_string);
  uer_check_status_da_id_str_val(
    "dra_put_comp_value: bad status from dob_get_base_data_type",
    UER_PUT_COMPONENT_VALUE);

  temp_value = drc_get_atv(value, temp_type);
  uer_check_status_da_id_str_val(
    "dra_put_comp_value: bad status from drc_get_atv",
    UER_PUT_COMPONENT_VALUE);
/*
   Assign the value to the specified component.
*/
  dob_shared_data_tell(real_sd_id, component_name_string, temp_value);
  uer_check_status_da_id_str_val(
    "dra_put_comp_value: bad status from dob_shared_data_tell",
    UER_PUT_VARIABLE_VALUE);
/* 
   Delete the parameters if they are temps 
*/
  dra_delete_temp(id_parm);
  uer_check_status_da_id_str_val(
    "dra_put_comp_value: bad status from dra_delete_temp id_parm",
    UER_PUT_COMPONENT_VALUE);

  dra_delete_temp (name_parm);
  uer_check_status_da_id_str_val(
    "dra_put_comp_value: bad status from dra_delete_temp name_parm",
    UER_PUT_COMPONENT_VALUE);

  dra_delete_temp (value);
  uer_check_status_da_id_str_val(
    "dra_put_comp_value: bad status from dra_delete_temp",
    UER_PUT_COMPONENT_VALUE);

  return;
}					/* end dra_put_comp_value	    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_get_variable_VALUE
| 
|  Description:  
|     Gets the value of the specified variable.  Uses a constant as the
|     name parmm
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_variable_VALUE(id_parm, name_parm)
dro_atomic_data_item id_parm;
string name_parm;
{					/* local type definition	    */
  dro_atomic_data_item temp;
  dob_data_object real_vc_id, variable_id;
  idd_data_types temp_type;
  caddr_t temp_value;
  boolean id_parm_undef;

  set_status(ok);			/* begin			    */
/* 
   Create a temp to return the result in.
*/
  temp.component_name = NULL;
  temp.dob_handle = dob_temp_create();
  uer_rcheck_status_da_id_str(
    "dra_get_variable_VALUE: bad status from dob_temp_create", 
    UER_GET_VARIABLE_VALUE, temp);
/*
   Check for undefined arguments.
*/
  id_parm_undef = dro_undefined(id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_variable_VALUE: bad status from dro_undefined of id_parm_undef", 
    UER_GET_VARIABLE_VALUE, temp);
/*
   If argument is undefined then the result is undefined.
*/
  if (id_parm_undef) {		/* undefined arguments     */
    dob_variable_tell (temp.dob_handle, idd_undefined, NULL);
    uer_rcheck_status_da_id_str(
    "dra_get_variable_VALUE: bad status from dob_variable_tell for undefined",
      UER_GET_VARIABLE_VALUE, temp);
  }                                     /* end if undefined arguments       */
  else {                                /* defined arguments                */
/*
   Generate an error for any symbol type otehr than a vc.
*/
   if ((real_vc_id = dra_validate_thingie(id_parm.dob_handle, vc_sym))
	== dob_null_object)
  { sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_GET_VARIABLE_VALUE);
     uer_report_da_id_str_error(uer_error_msg, id_parm,
				dob_make_string_constant_adi(name_parm));
   }                                           /* end if invalid type      */
/* 
   get the correct variable, do not need to check for errors, 
   find_thingie craps out if there are 
*/
    variable_id = dra_find_thingie(real_vc_id, variable_sym, name_parm);
/*
    Check for valid id type of parent 
*/
    if (variable_id == dob_null_object) {/* variable not found		    */
      
      sprintf(uer_error_msg, UER_NOT_FOUND, UER_GET_VARIABLE_VALUE);
      uer_report_da_id_str_error(uer_error_msg, id_parm,
				dob_make_string_constant_adi(name_parm));
    }					/* end if variable not found	    */
/* 
   Get the type and value of the specified variable 
*/
    temp_type = dob_get_base_data_type(variable_id, NULL);
    uer_rcheck_status_da_id_str(
      "dra_get_variable_VALUE: bad status from dob_get_base_data_type",
      UER_GET_VARIABLE_VALUE, temp);

    temp_value = dob_ask(variable_id, NULL);
    uer_rcheck_status_da_id_str(
      "dra_get_variable_VALUE: bad status from dob_ask", 
      UER_GET_VARIABLE_VALUE, temp);
/* 
   put the value into a temp, along with the type 
*/
    dob_variable_tell (temp.dob_handle, temp_type, temp_value);
    uer_rcheck_status_da_id_str(
      "dra_get_variable_VALUE: bad status from dob_variable_tell", 
      UER_GET_VARIABLE_VALUE, temp);
  }                                     /* end else arguments defined       */
/* 
   delete the parameters if the are temporary
*/
  dra_delete_temp (id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_variable_VALUE: bad status from dra_delete_temp id_parm", 
    UER_GET_VARIABLE_VALUE, temp);

  return temp;
}					/* end dra_get_variable_VALUE	    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_get_variable_value
| 
|  Description:  
|     Gets the value of the specified variable.
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_get_variable_value(id_parm, name_parm)
dro_atomic_data_item id_parm;
dro_atomic_data_item name_parm;
{					/* local type definition	    */
  dro_atomic_data_item temp;
  dob_data_object real_vc_id, variable_id;
  idd_data_types temp_type;
  string variable_name_string;
  caddr_t temp_value;
  boolean id_parm_undef, name_parm_undef;

  set_status(ok);			/* begin			    */
/* 
   Create a temp to return the result in.
*/
  temp.component_name = NULL;
  temp.dob_handle = dob_temp_create();
  uer_rcheck_status_da_id_str(
    "dra_get_variable_value: bad status from dob_temp_create", 
    UER_GET_VARIABLE_VALUE, temp);
/*
   Check for undefined arguments.
*/
  id_parm_undef = dro_undefined(id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_variable_value: bad status from dro_undefined of id_parm_undef", 
    UER_GET_VARIABLE_VALUE, temp);

  name_parm_undef = dro_undefined(name_parm);
  uer_rcheck_status_da_id_str(
  "dra_get_variable_value: bad status from dro_undefined for name_parm_undef",
    UER_GET_VARIABLE_VALUE, temp);
/*
   If either argument is undefined then the result is undefined.
*/
  if (id_parm_undef || name_parm_undef) {	/* undefined arguments     */
    dob_variable_tell (temp.dob_handle, idd_undefined, NULL);
    uer_rcheck_status_da_id_str(
    "dra_get_variable_value: bad status from dob_variable_tell for undefined",
      UER_GET_VARIABLE_VALUE, temp);
  }                                     /* end if undefined arguments       */
  else {                                /* defined arguments                */
/*
   Generate an error for any symbol type otehr than a vc.
*/
   if ((real_vc_id = dra_validate_thingie(id_parm.dob_handle, vc_sym))
	== dob_null_object)
  { sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_GET_VARIABLE_VALUE);
     uer_report_da_id_str_error(uer_error_msg, id_parm, name_parm);
   }                                           /* end if invalid type      */
/* 
   Get the variable name string.  
*/
    variable_name_string = (string)dro_ask2(name_parm);
    uer_rcheck_status_da_id_str(
      "dra_get_variable_value: bad status from dro_ask2", 
      UER_GET_VARIABLE_VALUE, temp);
/* 
   get the correct variable, do not need to check for errors, 
   find_thingie craps out if there are 
*/
    variable_id = dra_find_thingie(real_vc_id, variable_sym,
      variable_name_string);
/*
    Check for valid id type of parent 
*/
    if (variable_id == dob_null_object) {/* variable not found		    */
      
      sprintf(uer_error_msg, UER_NOT_FOUND, UER_GET_VARIABLE_VALUE);
      uer_report_da_id_str_error(uer_error_msg, id_parm, name_parm);
    }					/* end if variable not found	    */
/* 
   Get the type and value of the specified variable 
*/
    temp_type = dob_get_base_data_type(variable_id, NULL);
    uer_rcheck_status_da_id_str(
      "dra_get_variable_value: bad status from dob_get_base_data_type",
      UER_GET_VARIABLE_VALUE, temp);

    temp_value = dob_ask(variable_id, NULL);
    uer_rcheck_status_da_id_str(
      "dra_get_variable_value: bad status from dob_ask", 
      UER_GET_VARIABLE_VALUE, temp);
/* 
   put the value into a temp, along with the type 
*/
    dob_variable_tell (temp.dob_handle, temp_type, temp_value);
    uer_rcheck_status_da_id_str(
      "dra_get_variable_value: bad status from dob_variable_tell", 
      UER_GET_VARIABLE_VALUE, temp);
  }                                     /* end else arguments defined       */
/* 
   delete the parameters if the are temporary
*/
  dra_delete_temp (id_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_variable_value: bad status from dra_delete_temp id_parm", 
    UER_GET_VARIABLE_VALUE, temp);

  dra_delete_temp (name_parm);
  uer_rcheck_status_da_id_str(
    "dra_get_variable_value: bad status from dra_delete_temp name_parm",
    UER_GET_VARIABLE_VALUE, temp);

  return temp;
}					/* end dra_get_variable_value	    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_put_direct_value
| 
|  Description:  
|     Puts a specified value into a variable whose id is directly specified
|	as a parameter.
\*--------------------------------------------------------------------------*/
void dra_put_direct_value (the_id, value)
dob_data_object the_id;
dro_atomic_data_item value;
{  idd_data_types temp_type;
   caddr_t temp_value;

   set_status (ok);
		/* Get the type and value of the stuff to be stored */

   temp_type = dob_get_base_data_type (value.dob_handle,value.component_name);
   if (get_status ())
      return;

   temp_value = dob_ask (value.dob_handle, value.component_name);
   if (get_status ())
      return;

	/* Update the temp with the appropriate value and type. */

   dob_variable_tell (the_id, temp_type, temp_value);
}					/* end dra_put_direct_value */

/*--------------------------------------------------------------------------*\
|  Routine: dra_put_variable_VALUE
| 
|  Description:  
|     Puts a specified value into a variable.  If the arguments (other
|     than the value) are undefined this is basically a no-op.  Uses a
|     constant as the name parameter.
\*--------------------------------------------------------------------------*/
void dra_put_variable_VALUE(id_parm, name_parm, value)
dro_atomic_data_item id_parm;
string name_parm;
dro_atomic_data_item value;
{					/* local type definition	    */
  dob_data_object real_vc_id, variable_id;
  boolean id_parm_undef;

  set_status(ok);			/* begin			    */
/*
   Check for undefined arguments.
*/
  id_parm_undef = dro_undefined(id_parm);
  uer_check_status_da_id_str_val(
    "dra_put_variable_VALUE: bad status from dro_undefined of id_parm_undef",
    UER_PUT_VARIABLE_VALUE);
/*
   If arguments are undefined return without doing anything.
*/
  if (id_parm_undef) {		/*  undefined arguments	    */

    return;
  }					/* end if undefined arguments	    */
  else {                                /* defined arguments                */
/* 
   Generate an error for any sympl that is not a vc.
*/
    if ((real_vc_id = dra_validate_thingie (id_parm.dob_handle, vc_sym))
	== dob_null_object)
    { sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_PUT_VARIABLE_VALUE);
      uer_report_da_id_str_val_error(uer_error_msg, id_parm,
			dob_make_string_constant_adi(name_parm), value);
    }                                           /* end if invalid type      */
/*
   Get the correct variable.
*/
    variable_id = dra_find_thingie(real_vc_id, variable_sym, name_parm);
    uer_check_status_da_id_str_val(
    "dra_put_variable_VALUE: bad status from dra_find_thingie of variable_id",
      UER_PUT_VARIABLE_VALUE);
/*
   If the varible is not found generate an error.
*/
    if (variable_id == dob_null_object)   /* variable not found		    */
    { sprintf(uer_error_msg, UER_NOT_FOUND, UER_PUT_VARIABLE_VALUE);
      uer_report_da_id_str_val_error(uer_error_msg, id_parm,
			dob_make_string_constant_adi(name_parm), value);
    }					/* end if variable not found	    */

    dra_put_direct_value (variable_id, value);
    uer_check_status_da_id_str_val (
      "dra_put_variable_VALUE: bad status from dra_put_direct_value",
      UER_PUT_VARIABLE_VALUE);
  }                                     /* end else arguments defined       */
/*
   Delete temp parameters.
*/
  dra_delete_temp (id_parm);
  uer_check_status_da_id_str_val(
    "dra_put_variable_VALUE: bad status from dra_delete_temp",
    UER_PUT_VARIABLE_VALUE);

  dra_delete_temp (value);
  uer_check_status_da_id_str_val(
    "dra_put_variable_VALUE: bad status from dra_delete_temp",
    UER_PUT_VARIABLE_VALUE);
}					/* end dra_put_variable_VALUE	    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_put_variable_value
| 
|  Description:  
|     Puts a specified value into a variable.  If the arguments (other
|     than the value) are undefined this is basically a no-op.
\*--------------------------------------------------------------------------*/
void dra_put_variable_value(id_parm, name_parm, value)
dro_atomic_data_item id_parm, name_parm, value;
{					/* local type definition	    */
  dob_data_object real_vc_id, variable_id;
  string variable_name_string;
  boolean id_parm_undef, name_parm_undef;

  set_status(ok);			/* begin			    */
/*
   Check for undefined arguments.
*/
  id_parm_undef = dro_undefined(id_parm);
  uer_check_status_da_id_str_val(
    "dra_put_var_value: bad status from dro_undefined of id_parm_undef",
    UER_PUT_VARIABLE_VALUE);

  name_parm_undef = dro_undefined(name_parm);
  uer_check_status_da_id_str_val(
    "dra_put_var_value: bad status from dro_undefined of name_parm_undef",
    UER_PUT_VARIABLE_VALUE);
/*
   If arguments are undefined return without doing anything.
*/
  if (id_parm_undef || name_parm_undef) {	/*  undefined arguments	    */

    return;
  }					/* end if undefined arguments	    */
  else {                                /* defined arguments                */
/* 
   Generate an error for any sympl that is not a vc.
*/
    if ((real_vc_id = dra_validate_thingie (id_parm.dob_handle, vc_sym))
	== dob_null_object)
    { sprintf(uer_error_msg, UER_INVALID_SYMBOL, UER_PUT_VARIABLE_VALUE);
      uer_report_da_id_str_val_error(uer_error_msg, id_parm,name_parm, value);
    }                                           /* end if invalid type      */
/* 
   Get the variable name string.  
*/
    variable_name_string = (string)dro_ask2(name_parm);
    uer_check_status_da_id_str_val(
      "dra_put_var_value: bad status from dro_ask2", UER_PUT_VARIABLE_VALUE);

/*
   Get the correct variable.
*/
    variable_id = dra_find_thingie(real_vc_id, variable_sym, 
      variable_name_string);
    uer_check_status_da_id_str_val(
      "dra_put_var_value: bad status from dra_find_thingie of variable_id",
      UER_PUT_VARIABLE_VALUE);
/*
   If the varible is not found generate an error.
*/
    if (variable_id == dob_null_object)   /* variable not found		    */
    { sprintf(uer_error_msg, UER_NOT_FOUND, UER_PUT_VARIABLE_VALUE);
      uer_report_da_id_str_val_error(uer_error_msg, id_parm, name_parm,value);
    }					/* end if variable not found	    */

    dra_put_direct_value (variable_id, value);
    uer_check_status_da_id_str_val (
      "dra_put_var_value: bad status from dra_put_direct_value",
      UER_PUT_VARIABLE_VALUE);
  }                                     /* end else arguments defined       */
/*
   Delete temp parameters.
*/
  dra_delete_temp (id_parm);
  uer_check_status_da_id_str_val(
    "dra_put_var_value: bad status from dra_delete_temp",
    UER_PUT_VARIABLE_VALUE);

  dra_delete_temp (name_parm);
  uer_check_status_da_id_str_val(
    "dra_put_var_value: bad status from dra_delete_temp",
    UER_PUT_VARIABLE_VALUE);

  dra_delete_temp (value);
  uer_check_status_da_id_str_val(
    "dra_put_var_value: bad status from dra_delete_temp",
    UER_PUT_VARIABLE_VALUE);
}					/* end dra_put_var_value	    */

/*--------------------------------------------------------------------------*\
|  Routine: dra_delete_element
| 
|  Description:  
|     Deletes a shared data element.
\*--------------------------------------------------------------------------*/
void dra_delete_element(element)
dro_atomic_data_item element;
{
  dob_data_object element_id;

/*
   Get the element id for the element.
*/
  element_id = (dob_data_object)dro_ask2(element);
  check_status("dra_delete_element: bad status from dob_ask for element");

/*
   Destroy the object.  Since this routine is called from Slang the tap
   will not know the data is being removed so notify is set to true.
*/
  dob_destroy_data_object(element_id, true);
  check_status("dra_delete_element: bad status from dob_destroy_data_obj");

  dra_delete_temp (element);
  check_status (
     "dra_delete_element: bad status from delete_temp for element_adi");
}

/*--------------------------------------------------------------------------*\
|  Routine: dra_create_element
| 
|  Description:  
|     Creates a shared data element.
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_create_element(element_adi, mailbox_adi)
dro_atomic_data_item element_adi;
dro_atomic_data_item mailbox_adi;
{  
  dro_atomic_data_item temp;
  dob_data_object object_id;
  string element_string, mailbox_string;
  ipc_mailbox mailbox;
  idd_data_types temp_type;

  set_status(ok);			/* begin			    */

                                /* Create a temp to return the result in,
                                   and set its initial value to undefined  */
  temp.component_name = NULL;
  temp.dob_handle = dob_temp_create ();
  rcheck_status ("dra_create_element: bad status from dob_temp_create", temp);
  dob_variable_tell (temp.dob_handle, idd_undefined, NULL);
  rcheck_status (
  "dra_create_element: bad status from dob_variable_tell for undefined",temp);

				/* initialize "returned" values */
   temp_type = idd_undefined;
   object_id = dob_null_object;

                                /* check arguments for undefinedness */
   if (dro_undefined (element_adi) || dro_undefined (mailbox_adi))
      goto FINISHED;
				/* find the element name */
   element_string = (string) dro_ask2 (element_adi);
   rcheck_status (
	"dra_create_element: bad status from dob_ask for element_adi", temp);

				/* find the mailbox name */
   mailbox_string = (string) dro_ask2 (mailbox_adi);
   rcheck_status (
	"dra_create_element: bad status from dob_ask for mailbox_adi", temp);

				/* get the mailbox object corresponding to the
				   mailbox string name */
   mailbox = ipc_init (mailbox_string);
				/* now make the object and register its
				   creation */
   object_id = dob_slang_obj_create (element_string, mailbox, NULL, NULL);
   if (object_id != dob_null_object)
      temp_type = idd_id;

FINISHED:
                /* Update the temp with the appropriate value and type */
   dob_variable_tell (temp.dob_handle, temp_type, (char *) object_id);
   rcheck_status( "dra_create_element: bad status from dob_variable_tell",
    temp);

                        /* delete parameters, if they are temps */
   dra_delete_temp (element_adi);
   rcheck_status (
     "dra_create_element: bad status from delete_temp for element_adi", temp);
   dra_delete_temp (mailbox_adi);
   rcheck_status (
     "dra_create_element: bad status from delete_temp for element_adi", temp);

   return temp;
}					/* end dra_create_element */

/*--------------------------------------------------------------------------*\
|  Routine: dra_new
|
|  Description:
|     Creates a shared data element.
\*--------------------------------------------------------------------------*/
dro_atomic_data_item dra_new (element)
dro_atomic_data_item element;
{
  dro_atomic_data_item temp;

  dra_delete_temp (element);

  temp = dob_bool_TRUE;

  return temp;
}

#if 0

/*--------------------------------------------------------------------------*\
|  Routine: dra_new
| 
|  Description:  
|     Determines if a view controller should be instantiated
|     for the given combination of shared data instance, vc template,
|     and parent vc.
\*--------------------------------------------------------------------------*/
boolean dra_new(element, vc_template_name, parent_vc)
dro_atomic_data_item element, vc_template_name, parent_vc;
{  					/* begin			    */
  dob_data_object real_element, real_parent_vc;
  cus_sym element_ste, parent_vc_ste;
  boolean ret_val = false;

  set_status(ok);
/*
   Check to make sure that the element is a real shared data instance.
*/
  if ((real_element = dra_validate_thingie(element.dob_handle, sd_sym))
	== dob_null_object)
  { sprintf(uer_error_msg, UER_INVALID_SYMBOL, DRA_NEW);
    uer_report_new_error(uer_error_msg, element, vc_template_name, parent_vc);
  }                                             /* end if invalid type      */
/*
   Make sure that the parent_vc is a real view controller instance.
*/
  if ((real_parent_vc = dra_validate_thingie(element.dob_handle, vc_sym))
	== dob_null_object)
  { sprintf(uer_error_msg, UER_INVALID_SYMBOL, DRA_NEW);
    uer_report_new_error(uer_error_msg, element, vc_template_name, parent_vc);
  }                                             /* end if invalid type      */
/* 
   ??? can any of the parameters be undefined?  if so it should
   probably be treated as a run time error.
*/
/*
  Get symbol table entry for the element.
*/
  element_ste = dob_get_symbol_entry(real_element);
  uer_rcheck_status_new("dra_new: bad status from dob_get_symbol_entry");
/*
   Get the list of vcs instantiated for this property.
*/
  vc_list = (LIST)cus_get_prop(element_ste, created_vcs_prop);
  uer_rcheck_status_new(
    "dra_new: bad status from cus_get_prop of created_vcs_prop");
/*
   Get the actual template name.
*/
  vc_template_name_string = (string)dro_ask2(vc_template_name);
  uer_rcheck_status_new(
    "dra_new: bad status from dro_ask2 for vc_template_name");
/*
   Examine list vc list to determine if this vc has already been
   created or if it needs to be created.
*/
  loop_through_list(vc_list, vc_id, dob_data_object) {/* while list not empty
							*/
/*
   Get the real name for the vc
*/
    real_name = dob_get_real_name(vc_id);
    uer_rcheck_status_new(
      "dra_new: bad status from cus_get_prop of created_vcs_prop");
/*
   Determine if a new vc should be created.
*/
    if (strcmp(vc_template_name_string, real_name) == 0) { /* same name	    */

      symbol_table_entry =  dob_get_symbol_entry(vc_id);
      uer_rcheck_status_new(
        "dra_new: bad status from cus_get_prop of created_vcs_prop");

      temp_parent_vc = (dob_data_object)cus_get_prop(symbol_table_entry, 
        template_prop);
      uer_rcheck_status_new(
        "dra_new: bad status from cus_get_prop of created_vcs_prop");
/*
   If the parent vc is the same then return true.
*/
      if (temp_parent_vc == parent_vc)
        ret_val = true;
    }				/* end if names compare equal		    */
  }				/* end loop through list		    */

  return ret_val;
}					/* end dra_new */

#endif
