static char rcsid [] =
        "$Header: /u/04c/mnt/integ/source/dm/src/RCS/drc.c,v 1.47 91/04/02 16:04:08 ejh Exp $";
/*
???? propagate undefined in all run-time functions
*/
/*--------------------------------------------------------------------------*\
|
|  Name:  Run Time Conversion Utility (drc)
|
|  Description:  
|     The run time operations utility provides various run time arithmetic
|     operations for the run time portion of the dialogue manager.  The
|     run time operations utility works on data objects stored in the
|     object base.  These may be either symbols or shared data elements.
|
|  Component of:
|     Dialogue Manager Layer.
|
|  Related files:
|     dro.c - contains the implementation for the transactionmanager.
|     dro.h - contains the external definition for the transaction manager.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Stephen T. Chou
|     stc@sei.cmu.edu
|
|     Robert C. Seacord
|     rcs@sei.cmu.edu  
|
|  Change History:
$Log:	drc.c,v $
 * Revision 1.47  91/04/02  16:04:08  ejh
 * fixed FATAL_ERROR call; took out all refs. to result
 * 
 * Revision 1.46  91/04/01  17:27:16  dvk
 * No change
 * 
 * Revision 1.45  91/01/09  18:23:39  dvk
 * Fixed return of drc_convert for undefined id
 * 
 * Revision 1.44  90/07/24  18:17:50  ejh
 * drc_convert now converts all "id"s coming in from objects to ids, then
 * converts all outgoing ids from ids to objects
 * 
 * Revision 1.43  90/07/24  16:01:51  ejh
 * in drc_convert, set up to use the_arg, then never used it 8=(
 * 
 * Revision 1.42  90/07/23  11:03:44  ejh
 * created drc_convert, which checks for undefines before calling
 * uc2_convert
 * 
 * Revision 1.41  90/07/16  11:53:41  ejh
 * no change
 * 
 * Revision 1.40  90/07/05  13:12:29  ejh
 * switched most everything out to uc2
 * 
 * Revision 1.39  90/03/09  14:53:15  ljb
 * take out ifdef around is_real that had allowed for
 * bug in sunos3.5
 * 
 * Revision 1.38  90/03/01  12:27:55  dvk
 * Added drc_free_atv to release space claimed by drc_get_atv.
 * Also, made drc_get_undefined static and deleted the unused and
 * static checkchar
 * 
 * Revision 1.37  90/02/22  14:55:22  dvk
 * Added drc_cvt_type to be used by dro_attr_assign_quick.
 * 
 * Revision 1.36  90/02/14  22:22:32  ejh
 * f/7]ixed include structure
 * 
 * Revision 1.35  90/01/30  20:08:11  clc
 * Rearranged routines, and added a type declaration for funcall().
 * 
 * Revision 1.34  90/01/26  16:20:45  rcs
 * changed debug print
 * 
 * Revision 1.33  90/01/05  16:38:25  little
 * nevermind
 * 
 * Revision 1.32  89/12/27  14:46:30  ejh
 * changed d22_ftl_err to d22_fatal
 * 
 * Revision 1.31  89/12/23  20:47:16  ejh
 * fixed the ifdef for different arches
 * 
 * Revision 1.30  89/12/22  08:59:07  dvk
 * Changed hardwired 33 and 34 to be EDOM and ERANGE, respectively
 * 
 * Revision 1.29  89/12/21  16:48:02  little
 * change d22_fatal_error.h to d22_ftl_err.h
 * 
 * Revision 1.28  89/12/21  11:56:13  little
 * add support for slang file name output during dm run-time error print
 * 
 * Revision 1.27  89/12/08  10:53:59  dvk
 * Modified funcall to use a macro (from_to) and to use idd_types to
 * get rid of that afwul hardcoded-with-constants switch statement.
 * (functionality unchanged, maintainability greatly enhanced)
 * 
 * Revision 1.26  89/12/08  09:41:47  rcs
 * fixed buffer -> string conversion
 * 
 * Revision 1.25  89/11/10  15:18:01  rcs
 * made changes to fix id-> buffer conversion
 * 
 * Revision 1.24  89/11/02  16:17:01  little
 * 
 * no changes
 * 
 * Revision 1.23  89/10/23  19:42:21  ejh
 * switched over to u22_copy_string
 * 
 * Revision 1.22  89/10/05  17:06:22  rcs
 * checked in wrong file last time
 * 
 * Revision 1.20  89/08/29  10:56:41  bmc
 * Conditionally compiled code no longer defaults to VAX architecture if
 * '#ifdef mc68000' test fails.  Instead, explicit tests are included for
 * each supported architecture.  Unsupported architectures will cause
 * compilation errors.  (SPR #109)
 * 
 * Revision 1.19  89/08/14  16:35:46  ljb
 * try again to fix up isreal.  added test for special case of single
 * + or single -.  Also ifdefed depending upon architecture to test
 * for Sun bug in returning one past last character scanned.
 * 
 * Revision 1.18  89/08/08  12:01:58  ljb
 * change isinteger and isreal to use unix functions strtol and strtod rather
 * than roll our own.
 * 
 * 
 * Revision 1.17  89/06/15  15:26:21  ljb
 * add conversion routines for boolean to other things and
 * virtually anything to buffer
 * make buffer universal type which holds values in binary rather
 * than ascii
 * 
 * Revision 1.16  89/05/19  17:21:46  bmc
 * Adjusted to accomodate new definition of 'dro_atomic_data_item'.
 * See 'dro.h' for more information.
 * 
 * Revision 1.15  89/04/25  18:11:05  rnk
 * fixed isinteger so that it doesn't barf on trailing blanks
 * 
 * Revision 1.14  89/04/11  14:41:43  rcs
 * added string to id and id to string conversions
 * 
 * Revision 1.13  89/02/09  17:42:50  ejh
 * added copyright notice
 * 
 * Revision 1.12  89/02/07  20:29:11  ejh
 * rebuilt rtoi and blstr; general portability fixes
 * 
 * Revision 1.11  89/02/07  19:02:07  ejh
 * fixed rtoi
 * 
 * Revision 1.10  89/01/30  13:48:59  rcs
 * modified strtobuff and bufftostr to use and support
 *   NULL body buffers
 * added debug print for calls, parameters, exit and return values
 * 
 * Revision 1.9  89/01/12  14:37:36  little
 * add #ifndef mc68000 so things work correctly on sun
 * 
 * Revision 1.8  89/01/04  15:39:12  little
 * add printing of d22_slang_source to error printout
 * 
 * Revision 1.7  88/12/21  19:16:27  rcs
 * modified rtoa and itoa
 * 
 * Revision 1.6  88/12/21  14:24:22  little
 * add drc_get_undefined function
 * add debugs print to funcall & itoa
 * 
 * Revision 1.5  88/11/04  17:03:24  little
 * change DEBUG1 to print d22_slang_source
 * 
|     June 14, 1988 : Module created.
|     Oct 3, 1988: add conversion string <--> buffer       ljb
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	drc_TOP
#define	memoryPack

#include	<stdio.h>
#include	<errno.h>
#include	"global.h"
#include	"memoryPack.h"
#include	"uc2.h"
#include	"isc.h"
#include	"idd.h"
#include	"d22_fatal.h"
#include	"dob.h"
#include	"dro.h"
#include	"drc.h"

#define	DEBUG1(statement) SERPENT_DEBUG1(drc_debug, statement)
#define	DEBUG2(statement) SERPENT_DEBUG2(drc_debug, statement)
#define	DEBUG3(statement) SERPENT_DEBUG3(drc_debug, statement)

/*--------------------------------------------------------------------------*\
|  Routine: drc_get_atv
| 
|  Description:
|     The drc get appropriately typed value routine accepts an operand and
|     a conversion type and returns a value of the conversion type or sets
|     status to invalid_conversion.
\*--------------------------------------------------------------------------*/
caddr_t	drc_get_atv (operand, data_type)
dro_atomic_data_item	operand;
idd_data_types		data_type;
    {
    idd_data_types	op_type;
    caddr_t		op_data,
			ret_val;

    set_status(ok);
/*
   If the data type is undefined force a fatal error.
*/
    if (data_type == idd_undefined)
	FATAL_ERROR (fprintf (stderr,
	    "drc_get_atv: attempt to convert to the undefined type\n"));
/*
   Get the type and value of the operand.
*/
    op_type = dob_get_base_data_type (operand.dob_handle,
	operand.component_name);
    rcheck_status ("drc_get_atv:  bad status from dob_get_base_data_type",
	NULL);

    op_data = (caddr_t) dob_ask (operand.dob_handle, operand.component_name);
    rcheck_status ("drc_get_atv:bad status from dob_ask", NULL);
/*
   Get the actual value.
*/
    ret_val = drc_convert (op_type, data_type, op_data);

    return (ret_val);
}

/*--------------------------------------------------------------------------*\
|  Routine: drc_free_atv
| 
|  Description:
|    This routine frees the space used for the value acquired via drc_get_atv.
\*--------------------------------------------------------------------------*/
void	drc_free_atv (atv, atv_type)
caddr_t		atv;
idd_data_types	atv_type;
{
    switch (atv_type)
	{
	case idd_string:
	    free_node (atv);
	    break;
	case idd_real:
	    free_node (atv);
	    break;
	case idd_buffer:
	    free_node (((idd_buffer_type *) atv)->body);
	    free_node (atv);
	    break;
	}
    }

/*--------------------------------------------------------------------------*\
|  Routine: drc_convert
| 
|  Description:
|    This routine converts from one to another.
\*--------------------------------------------------------------------------*/
caddr_t drc_convert (from_type, to_type, data)
idd_data_types	from_type,
		to_type;
caddr_t		data;
    {
    caddr_t	the_arg,
		value;

    if (dob_undefined (from_type, data))
	{
	return uc2_get_undefined (to_type);
	}
    else if (from_type == idd_id)
	if (((iid_id_type) data == iid_null_id) ||
	    ((dob_data_object) data == dob_null_object))
	    {
	    return uc2_get_undefined (to_type);
	    }
	else
	    the_arg = (caddr_t) dob_get_object_id (data);
    else
	the_arg = data;
    value = (caddr_t) uc2_convert (to_type, from_type, the_arg);
    if (isc_status == failed_conversion)
	FATAL_ERROR (fprintf (stderr, "failed conversion in drc_convert\n"));
    if (to_type == idd_id)
	value = (caddr_t) dob_get_object_from_id (value);
    set_status (ok);
    return (value);
    }
