static char rcsid [] = "$Header: /u/04c/mnt/integ/source/dm/src/RCS/dri.c,v 1.9 91/04/01 17:27:48 dvk Exp $";

/*--------------------------------------------------------------------------*\
|
|  Name:  Run Time Interpreter (dri)
|
|  Description:  
|	The interpreter reads the binary files comprising the dialogue
|	executable and calls the appropriate routines to effect the
|	execution of the dialogue.
|
|  Component of:
|	Dialogue Manager
|
|  Related files:
|	dri.h - contains the definitions of the interpreter constants
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:  
|	Daniel Klein
|	dvk@sei.cmu.edu
|
|*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*
 * $Log:	dri.c,v $
 * Revision 1.9  91/04/01  17:27:48  dvk
 * Added missing dra_delete_temp in T_complex_parm_init and in various
 * calls to get_atv (partial fix to memory leak + performance improvement)
 * 
 * Revision 1.8  91/02/20  18:54:17  little
 * set the sen component of ME when everything else is set
 * 
 * Revision 1.7  91/02/12  15:33:54  little
 * removed the concept of parm_list cause the added vc_table took its place.
 * this also measn we need not pass the parm_list into dob_create_vc
 * to use as a vce.  added the vc_table hash and supporting match & hash
 * routines.  the vc_table contains all necessary info to match vc instances
 * with their corresponding parent instance and optional creating sd
 * instance.  this can be used to see if we have a vc instance or not
 * when processing the cc in T_vc_create.  it is also used by T_validate_sne_cc
 * to determine if a vc instance which sould be deleted due to the the
 * parent instance or creating sd instance being gone exists.
 * 
 * Revision 1.6  91/01/31  17:29:10  little
 * add new rec_on & off and the sne_validate instructions
 * 
 * Revision 1.5  91/01/09  13:25:22  dvk
 * Now allows dialogues to be executed from directories other than the
 * current directory.
 * 
 * Revision 1.4  91/01/08  17:06:30  dvk
 * Included code to implement real constants
 * 
 * Revision 1.3  91/01/08  13:32:36  ejh
 * bad call to dob_get_symbol_entry was fixed
 * 
 * Revision 1.2  91/01/07  17:54:38  dvk
 * Deleted superfluous extra parameter in call in T_component_value_from_parm
 * 
 * Revision 1.1  91/01/02  17:50:50  dvk
 * Initial revision
 * 
 */

#define	memoryPack

#include	<listPack.h>
#include	<string.h>
#include	<dri.h>
#include	<dra.h>
#include	<dro.h>
#include	<dob_make.h>
#include	<d22.h>
#include	<dpt.h>
#include	<iid.h>
#include	<isc.h>
#include	<hashPack.h>
#include	<cus.h>
#include	<global.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<ufa.h>

/*
 * These are a collection of constants that are generated once and
 * (hopefully) reused many times.
 */
dro_atomic_data_item dob_bool_TRUE;
dro_atomic_data_item dob_bool_FALSE;
dro_atomic_data_item dob_id_NULL;
dro_atomic_data_item *dob_int_const;
dro_atomic_data_item *dob_real_const;
dro_atomic_data_item *PARENT;

#define INTERPRETER_STACK_DEPTH	128	/* Depth of interpreter stack */
#define ADI_STACK_DEPTH		128	/* Maximum depth of ADI's */
#define NUM_PARMS		30	/* Maximum number of VC parms */
#define VC_HASH_SIZE		257	/* size of the vc hash table */

extern caddr_t (*dri_user_functs []) ();
static caddr_t user_retval;
static int dri_max_snippets;

static HASH vc_table;
typedef struct {
   iid_id_type parent_vc;
   iid_id_type sd;
   int routine;
   dob_data_object the_vc;
   iid_id_type the_vc_id;
   } dri_vc_table_entry;

/*
 * The dialogue manager interpreter creates a stack machine, where operators
 * pop operands off the stack and (when necessary) push results onto the
 * stack.  Unfortunately, the dialogue manager uses the dro_atomic_data_item
 * type extensively, and this is a structure which cannot be cast into a
 * caddr_t (the primary type of the stack).  Consequently, it is necessary
 * to use a pair of stacks - one for the adi's, and one for the rest of the
 * stack.  To consolidate things, the main stack points at the structure
 * elements in the adi stack, and the routine dri_pop will remove elements
 * from both stacks as necessary.  In the interpreter, items will call either
 * dri_adi_push (if an adi needs to be pushed on the stack), or dri_push (if
 * a simple element needs to be pushed on the stack), but *never* both.  The
 * routine dri_adi_push takes care of calling dri_push for you.  Hopefully,
 * this keeps things simpler.
 */

static caddr_t dri_stack[INTERPRETER_STACK_DEPTH];
static caddr_t *dri_tos = dri_stack;

static dro_atomic_data_item dri_adi_stack[ADI_STACK_DEPTH];
static dro_atomic_data_item *dri_adi_tos = dri_adi_stack;

/*
 * The following tables are initialized by dri_initialize, and contain the
 * interpreter executable code, string space, integer and real constant
 * adi's, and snippet offset addresses.
 */
static dro_atomic_data_item *dri_int_const;
static dro_atomic_data_item *dri_real_const;
static short *dri_code;
static char *dri_string_table;
static long *dri_snip_table;

#ifdef DRI_STAT
static int max_depth = 0;
static int max_adi_depth = 0;
static int MAX_depth = 0;
static int MAX_adi_depth = 0;
#endif
/*
 * This constant is used in the rare instances where multiple operations
 * need to be performed on the stack (and one of them is a push), but
 * the stack needs to be unchanged (before the push).
 */
static caddr_t dri_atv;


/*--------------------------------------------------------------------------*\
|  Routine: dri_match_vc
|
|  Description:
|     compares sne information w/elements in vc table
\*--------------------------------------------------------------------------*/
static int dri_match_vc (candidate_sne, a_vc_entry)
dgr_sne *candidate_sne;
dri_vc_table_entry *a_vc_entry;
{
   return ((candidate_sne->self_vc_id == a_vc_entry->parent_vc) &&
           (candidate_sne->sd_handle_id == a_vc_entry->sd) &&
           (candidate_sne->routine == a_vc_entry->routine));
}


/*--------------------------------------------------------------------------*\
|  Routine: dri_hash_vc
|
|  Description:
|     converts sne information into a vc table index
\*--------------------------------------------------------------------------*/
static int dri_hash_vc (a_sne)
dgr_sne *a_sne;
{  int the_hash;

   the_hash = (((((int)a_sne->routine * (int) 0x10000) 
			+
                  ((int)a_sne->self_vc_id * (int) 0x1000) 
			+
	   (int)a_sne->sd_handle_id) & 0x7fffffff)
			% VC_HASH_SIZE);
   return (the_hash);
}



/*
 * Push an argument onto the stack, checking for overflow
 */
static void dri_push (arg)
caddr_t arg;
{
    *dri_tos++ = arg;
    if (dri_tos == &dri_stack[INTERPRETER_STACK_DEPTH-1])
	FATAL_ERROR ( fprintf (stderr, "?Interpreter stack overflow!\n"); )
#ifdef DRI_STAT
    if ((dri_tos - dri_stack) > max_depth)
	max_depth = dri_tos - dri_stack;
    if (max_depth > MAX_depth)
	MAX_depth = max_depth;
#endif
}

/*
 * Pop "count" arguments off the stack (leaving them there to be used later)
 */
static void dri_pop (count, adi_count)
int count, adi_count;
{
    dri_tos -= count;
    if (dri_tos < dri_stack)
	FATAL_ERROR ( fprintf (stderr, "?Interpreter stack underflow!\n"); )
    dri_adi_tos -= adi_count;
    if (dri_adi_tos < dri_adi_stack)
	FATAL_ERROR ( fprintf (stderr, "?ADI stack underflow!\n"); )
}

/*
 * Push an argument onto the adi stack, checking for overflow
 */
static void dri_adi_push (arg)
dro_atomic_data_item arg;
{
    *dri_adi_tos = arg;
    if (dri_adi_tos == &dri_adi_stack[ADI_STACK_DEPTH-1])
	FATAL_ERROR ( fprintf (stderr, "?ADI stack overflow!\n"); )
    dri_push ((caddr_t)(dri_adi_tos++));
#ifdef DRI_STAT
    if ((dri_adi_tos - dri_adi_stack) > max_adi_depth)
	max_adi_depth = dri_adi_tos - dri_adi_stack;
    if (max_adi_depth > MAX_adi_depth)
	MAX_adi_depth = max_adi_depth;
#endif
}

/*
 * Pop "count" arguments off the adi stack (leaving them there to be used later)
 */
static void dri_adi_pop (count)
int count;
{
    dri_adi_tos -= count;
    if (dri_adi_tos < dri_adi_stack)
	FATAL_ERROR ( fprintf (stderr, "?ADI stack underflow!\n"); )
}


static boolean validate_sne (the_sne)
dgr_sne *the_sne;
{
   if (((the_sne->self_vc_id == iid_null_id) ||
        dob_get_object_from_id (the_sne->self_vc_id))
		&&
       ((the_sne->sd_handle_id == iid_null_id) ||
         dob_get_object_from_id (the_sne->sd_handle_id)))
      return (TRUE);
   else
      return (FALSE);
}


#ifdef DRI_STAT
static void dri_stat (routine)
int routine;
{
    fprintf (stdout, "%4d: max = %-2d(%2d), adi = %-2d(%2d), cur = %d(%d)\n",
	routine, max_depth, MAX_depth, max_adi_depth, MAX_adi_depth,
	dri_tos - dri_stack, dri_adi_tos - dri_adi_stack);
    max_depth = 0;
    max_adi_depth = 0;
}
#endif

void dri_interpret (sne)
dgr_sne *sne;
{
    int i;
    dri_token token;
    caddr_t (*user_funct) ();
    int num_user_parms;
    dob_data_object SELF;
    dob_data_object PARM[NUM_PARMS];
    dob_data_object the_vc;
    d22_header ME;
    long long_temp;
    long dri_pc;
    dri_vc_table_entry *look;

    SELF = (dob_data_object)sne->self_vc_object;

    dri_pc = dri_snip_table[sne->routine];
    while (1) {
	switch (token = (dri_token)dri_code[dri_pc++]) {
	    case L_negative:
		dri_push ((caddr_t)dri_code[dri_pc++]);
		break;
	    /*
	     * Grab the next two words out of the instruction stream, and
	     * construct a longword.  These are the only places where the
	     * issue of big-endian vs. little-endian arises in the
	     * interpreter.  Most code compiled on one platform "should"
	     * be executable on another, but this checks for it.
	     */
	    case L_long:
#ifndef ENDIAN_SWAP
		long_temp = ((unsigned short)dri_code[dri_pc++] & 0xffff);
		long_temp |= (unsigned short)dri_code[dri_pc++] << 16;
#else
		long_temp = (unsigned short)dri_code[dri_pc++] << 16;
		long_temp |= ((unsigned short)dri_code[dri_pc++] & 0xffff);
#endif
		dri_push ((caddr_t)long_temp);
		break;
	    case R_get_parent_vc:
		dri_pop (1, 1);
		dri_adi_push (
		    dra_get_parent_vc (
			*((dro_atomic_data_item *)dri_tos[0])));
		break;
	    case R_get_creating_sd:
		dri_pop (1, 1);
		dri_adi_push (
		    dra_get_creating_sd (
			*((dro_atomic_data_item *)dri_tos[0])));
		break;
	    case R_get_object:
		dri_pop (2, 2);
		dri_adi_push (
		    dra_get_object (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_get_component_value:
		dri_pop (2, 2);
		dri_adi_push (
		    dra_get_component_value (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_put_component_value:
		dri_pop (3, 3);
		dra_put_component_value (
		    *((dro_atomic_data_item *)dri_tos[0]),
		    *((dro_atomic_data_item *)dri_tos[1]),
		    *((dro_atomic_data_item *)dri_tos[2]));
		break;
	    case R_get_vc_id:
		dri_pop (2, 2);
		dri_adi_push (
		    dra_get_vc_id (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_get_variable_value:
		dri_pop (2, 2);
		dri_adi_push (
		    dra_get_variable_value (
			*((dro_atomic_data_item *)dri_tos[0]),
		        *((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_put_variable_value:
		dri_pop (3, 3);
		dra_put_variable_value (
		    *((dro_atomic_data_item *)dri_tos[0]),
		    *((dro_atomic_data_item *)dri_tos[1]),
		    *((dro_atomic_data_item *)dri_tos[2]));
		break;
	    case R_create_element:
		dri_pop (2, 2);
		dri_adi_push (
		    dra_create_element (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_delete_element:
		dri_pop (1, 1);
		dra_delete_element (
		    *((dro_atomic_data_item *)dri_tos[0]));
		break;
	    case R_get_name:
		dri_pop (1, 1);
		dri_adi_push (
		    dra_get_name (
			*((dro_atomic_data_item *)dri_tos[0])));
		break;
	    case R_get_type:
		dri_pop (1, 1);
		dri_adi_push (
		    dra_get_type (
			*((dro_atomic_data_item *)dri_tos[0])));
		break;
	    /*
	     * The only easy way out of here (other than T_return)
	     */
	    case R_exit:
#ifdef DRI_STAT
		dri_stat (sne->routine);
#endif
		dra_exit ();
		break;
	    case R_id_exists:
		dri_pop (1, 1);
		dri_adi_push (
		    dra_id_exists (
			*((dro_atomic_data_item *)dri_tos[0])));
		break;
	    case R_rec_on:
		dri_pop (2, 2);
		dra_recording_on (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1]));
		break;
	    case R_rec_off:
		dra_recording_off ();
		break;

	    case R_and:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_and (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_or:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_or (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_dot:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_dot (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_neq:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_not_equal (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_leq:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_less_than_or_equal (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_lss:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_less (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_geq:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_greater_than_or_equal (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_gtr:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_greater (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_assign:
		dri_pop (2, 2);
		dro_assign (
		    *((dro_atomic_data_item *)dri_tos[0]),
		    *((dro_atomic_data_item *)dri_tos[1]));
		break;
	    case R_eql:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_equal (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_add:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_add (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_sub:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_subtract (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_neg:
		dri_pop (1, 1);
		dri_adi_push (
		    dro_negate (
			*((dro_atomic_data_item *)dri_tos[0])));
		break;
	    case R_not:
		dri_pop (1, 1);
		dri_adi_push (
		    dro_not (
			*((dro_atomic_data_item *)dri_tos[0])));
		break;
	    case R_div:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_divide (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_exp:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_exponentiate (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_mul:
		dri_pop (2, 2);
		dri_adi_push (
		    dro_multiply (
			*((dro_atomic_data_item *)dri_tos[0]),
			*((dro_atomic_data_item *)dri_tos[1])));
		break;
	    case R_new:
		dri_pop (1, 1);
		dri_adi_push (
		    dra_new (
			*((dro_atomic_data_item *)dri_tos[0])));
		break;

	    case R_list_assign:
		dri_pop (4, 0);
		/* 293, C_idd_string, "label", 50, R_list_assign */
		fprintf (stderr, "Can't do list_assign\n");
		break;
	    case R_validate_object:
		dri_pop (1, 0);
		dri_push (
		    (caddr_t)dob_validate_object (
			PARM[(int)dri_tos[0]]));
		break;
	    case R_boolean:
		dri_pop (1, 1);
		dri_push (
		    (caddr_t)dro_boolean (
			*((dro_atomic_data_item *)dri_tos[0])));
		break;
	    case R_get_boolean_atv:
		dri_pop (1, 1);
		dri_atv = (caddr_t)drc_get_atv (
			*((dro_atomic_data_item *)dri_tos[0]),
			idd_boolean);
		dra_delete_temp(*((dro_atomic_data_item *)dri_tos[0]));
		dri_push (dri_atv);
		break;
	    case R_get_integer_atv:
		dri_pop (1, 1);
		dri_atv = (caddr_t)drc_get_atv (
			*((dro_atomic_data_item *)dri_tos[0]),
			idd_integer);
		dra_delete_temp(*((dro_atomic_data_item *)dri_tos[0]));
		dri_push (dri_atv);
		break;
	    case R_get_real_atv:
		dri_pop (1, 1);
		dri_atv = (caddr_t)drc_get_atv (
			*((dro_atomic_data_item *)dri_tos[0]),
			idd_real);
		/*
		 * There is a memory leak here - if we delete the temp,
		 * then we also delete the value (which is probably used
		 * later on).  If we don't delete the temp, we have a
		 * memory leak.  Fixing the real problem is too hard right
		 * now, so we live with the leak.
		 *
		 * dra_delete_temp(*((dro_atomic_data_item *)dri_tos[0]));
		 */
		dri_push (dri_atv);
		break;
	    case R_get_string_atv:
		dri_pop (1, 1);
		dri_atv = (caddr_t)drc_get_atv (
			*((dro_atomic_data_item *)dri_tos[0]),
			idd_string);
		/*
		 * There is a memory leak here - if we delete the temp,
		 * then we also delete the value (which is probably used
		 * later on).  If we don't delete the temp, we have a
		 * memory leak.  Fixing the real problem is too hard right
		 * now, so we live with the leak.
		 *
		 * dra_delete_temp(*((dro_atomic_data_item *)dri_tos[0]));
		 */
		dri_push (dri_atv);
		break;
	    case R_get_record_atv:
		dri_pop (1, 1);
		dri_atv = (caddr_t)drc_get_atv (
			*((dro_atomic_data_item *)dri_tos[0]),
			idd_record);
		dra_delete_temp(*((dro_atomic_data_item *)dri_tos[0]));
		dri_push (dri_atv);
		break;
	    case R_get_id_atv:
		dri_pop (1, 1);
		dri_atv = (caddr_t)drc_get_atv (
			*((dro_atomic_data_item *)dri_tos[0]),
			idd_id);
		dra_delete_temp(*((dro_atomic_data_item *)dri_tos[0]));
		dri_push (dri_atv);
		break;
	    case R_get_buffer_atv:
		dri_pop (1, 1);
		dri_atv = (caddr_t)drc_get_atv (
			*((dro_atomic_data_item *)dri_tos[0]),
			idd_buffer);
		/*
		 * There is a memory leak here - if we delete the temp,
		 * then we also delete the value (which is probably used
		 * later on).  If we don't delete the temp, we have a
		 * memory leak.  Fixing the real problem is too hard right
		 * now, so we live with the leak.
		 *
		 * dra_delete_temp(*((dro_atomic_data_item *)dri_tos[0]));
		 */
		dri_push (dri_atv);
		break;
	    case R_get_undefined_atv:
		dri_pop (1, 1);
		dri_atv = (caddr_t)drc_get_atv (
			*((dro_atomic_data_item *)dri_tos[0]),
			idd_undefined);
		dra_delete_temp(*((dro_atomic_data_item *)dri_tos[0]));
		dri_push (dri_atv);
		break;
	    case R_make_boolean_constant_adi:
		dri_pop (1, 0);
		dri_adi_push (
		    dob_make_boolean_constant_adi (
			(boolean)dri_tos[0]));
		break;
	    case R_make_buffer_constant_adi:
		dri_pop (1, 0);
		dri_adi_push (
		    dob_make_buffer_constant_adi (
			(caddr_t)dri_tos[0]));
		break;
	    case R_make_integer_constant_adi:
		dri_pop (1, 0);
		dri_adi_push (
		    dob_make_integer_constant_adi (
			(int)dri_tos[0]));
		break;
	    /*
	     * There are two routines to convert strings to ADIs.
	     * The first assumes that the integer on the stack is
	     * an address in the string table; the second assumes
	     * that the integer is the address of a string.
	     */
	    case R_make_indir_string_constant_adi:
		dri_pop (1, 0);
		dri_adi_push (
		    dob_make_string_constant_adi (
			&dri_string_table[(int)dri_tos[0]]));
		break;
	    case R_make_immed_string_constant_adi:
		dri_pop (1, 0);
		dri_adi_push (
		    dob_make_string_constant_adi (
			(char *)dri_tos[0]));
		break;
#if 0
	    case R_make_real_constant_adi:
		dri_pop (1, 0);
		dri_adi_push (
		    dob_make_real_constant_adi (
			(double)dri_tos[0]));
		break;
#endif
	    case R_make_id_constant_adi:
		dri_pop (1, 0);
		dri_adi_push (
		    dob_make_id_constant_adi (
			(iid_id_type)dri_tos[0]));
		break;
	    case R_make_undefined_adi:
		dri_adi_push (
		    dob_make_undefined_adi ());
		break;
	    case R_get_sire_vc:
		dri_pop (2, 1);
		dri_adi_push (
		    dra_get_sire_vc (
			*((dro_atomic_data_item *)dri_tos[0]),
			(int)dri_tos[1]));
		break;
	    case R_get_OBJECT:
		dri_pop (2, 1);
		dri_adi_push (
		    dra_get_OBJECT (
			*((dro_atomic_data_item *)dri_tos[0]),
			&dri_string_table[(int)dri_tos[1]]));
		break;
#if 0
	    case R_put_OBJECT:
		break;
#endif
	    case R_get_variable_VALUE:
		dri_pop (2, 1);
		dri_adi_push (
		    dra_get_variable_VALUE (
			*((dro_atomic_data_item *)dri_tos[0]),
			&dri_string_table[(int)dri_tos[1]]));
		break;
	    case R_put_variable_VALUE:
		dri_pop (3, 2);
		dra_put_variable_VALUE (
		    *((dro_atomic_data_item *)dri_tos[0]),
		    &dri_string_table[(int)dri_tos[1]],
		    *((dro_atomic_data_item *)dri_tos[2]));
		break;
	    case R_get_component_VALUE:
		dri_pop (2, 1);
		dri_adi_push (
		    dra_get_component_VALUE (
			*((dro_atomic_data_item *)dri_tos[0]),
			&dri_string_table[(int)dri_tos[1]]));
		break;
	    /*
	     * Due to a quirk in the compiler, this is the *only*
	     * place where the stack is used out of order.
	     */
	    case R_put_component_VALUE:
		dri_pop (3, 2);
		dra_put_component_VALUE (
		    *((dro_atomic_data_item *)dri_tos[1]),
		    &dri_string_table[(int)dri_tos[2]],
		    *((dro_atomic_data_item *)dri_tos[0]));
		break;
	    case R_bool_assign:
		dri_pop (2, 0);
		dro_attr_assign_quick (
		    SELF,
		    &dri_string_table[(int)dri_tos[0]],
		    idd_boolean,
		    (caddr_t)dri_tos[1]);
		break;
	    case R_int_assign:
		dri_pop (2, 0);
		dro_attr_assign_quick (
		    SELF,
		    &dri_string_table[(int)dri_tos[0]],
		    idd_integer,
		    (caddr_t)dri_tos[1]);
		break;
	    case R_id_assign:
		dri_pop (2, 0);
		dro_attr_assign_quick (
		    SELF,
		    &dri_string_table[(int)dri_tos[0]],
		    idd_id,
		    (caddr_t)dri_tos[1]);
		break;
	    case R_string_assign:
		dri_pop (2, 0);
		dro_attr_assign_quick (
		    SELF,
		    &dri_string_table[(int)dri_tos[0]],
		    idd_string,
		    &dri_string_table[(int)dri_tos[1]]);
		break;
	    case R_make_adi:
		dri_pop (1, 0);
		dri_adi_push (
		    dob_make_adi (
			SELF,
			&dri_string_table[(int)dri_tos[0]]));
		break;

	    case T_and:
		dri_pop (2, 0);
		dri_push (
		    (int)dri_tos[0] && (int)dri_tos[1]);
		break;
	    /*
	     * Conditional execution.  The stack looks like:
	     *		test_condition if_false_target T_if
	     */
	    case T_if:
		dri_pop (2, 0);
		if ( ! (int)dri_tos[0])
		    dri_pc = (long)dri_tos[1];
		break;
	    case T_goto:
		dri_pop (1, 0);
		dri_pc = (long)dri_tos[0];
		break;
	    /*
	     * The only easy way out of here (other than R_exit)
	     */
	    case T_return:
#ifdef DRI_STAT
		dri_stat (sne->routine);
#endif
		return;
	    case T_snip_end:
		FATAL_ERROR (
		    fprintf (stderr, "Fell of end of snip %d\n", sne->routine)
		    )
	    case T_compile_error:
		FATAL_ERROR (
		    fputs ("Cannot execute code with compile errors\n", stderr)
		    )
	    /*
	     * The following tokens take care of VC creation and initialization
	     */
	    case T_simple_parm_init:
		dri_pop (1, 0);
		if ((int)dri_tos[0] >= NUM_PARMS)
		    FATAL_ERROR ( fprintf (stderr, "VC CC parm overflow\n") )
		PARM[(int)dri_tos[0]] = (dob_data_object)sne->sd_handle_object;
		break;
	    case T_complex_parm_init:
		dri_pop (4, 1);
		if ((int)dri_tos[0] >= NUM_PARMS)
		    FATAL_ERROR ( fprintf (stderr, "VC CC parm overflow\n") )
		PARM[(int)dri_tos[0]] = dra_find_thingie (
		    dro_ask2 (*((dro_atomic_data_item *)dri_tos[1])),
		    (cus_sym_type)dri_tos[2],
		    &dri_string_table[(int)dri_tos[3]]);
		dra_delete_temp(*((dro_atomic_data_item *)dri_tos[1]));
		break;
	    case T_adi_from_parm:
		dri_pop (1, 0);
		dri_adi_push (
		    dob_make_adi (
			PARM[(int)dri_tos[0]], ""));
		break;
	    case T_component_value_from_parm:
		dri_pop (2, 0);
		dri_adi_push (
		    dra_get_component_VALUE (
			dob_make_id_constant_adi (PARM[(int)dri_tos[0]]),
			&dri_string_table[(int)dri_tos[1]]));
		break;
		break;
	    case T_validate_sne:
		if (validate_sne (sne))
                   dri_push (1);
                else
		   dri_push (0);
		break;
	    case T_validate_sne_cc:
		if (validate_sne (sne))
                   dri_push (1);
                else {
		   dri_push (0);
                   if (look = (dri_vc_table_entry *)
				    get_from_hashtable (vc_table, sne)) {
		      if (dob_get_object_from_id (look->the_vc_id))
                         dob_destroy_data_object (look->the_vc, TRUE);
                      delete_from_hashtable (vc_table, sne);
                      free_node (look);
                      }
		   }
		break;
	    case T_not_dpt_rite_rule:
		dri_pop (1, 0);
		dri_push (
		    (caddr_t) ! dpt_rite_rule (
			SELF,
			PARM[(int)dri_tos[0]]));
		break;
	    case T_vc_create:
		dri_pop (3, 0);
                look = (dri_vc_table_entry *)
			   get_from_hashtable (vc_table, sne);
		/*
		 * Check to see if the creation condition is true
		 */
		if ((int)dri_tos[0]) {
		    /*
		     * If we do not have a vc instance, create one
		     */
                    if (!look) {
                        look = (dri_vc_table_entry *) make_node
                                                (sizeof (dri_vc_table_entry));
                        look->the_vc = 
                          dob_vc_create (
                            SELF,
			    cus_get_prop (
				cus_look_up (
				    &dri_string_table[(int)dri_tos[1]]),
				    id_prop),
			    (int) dri_tos[2] == 999 ?
				(dob_data_object) cus_get_prop (
				    dob_get_symbol_entry (SELF),
				    creating_sd_prop) :
				PARM[(int)dri_tos[2]]);
                        look->the_vc_id = look->the_vc->id;
                        look->parent_vc = sne->self_vc_id;
                        look->sd = sne->sd_handle_id;
                        look->routine = sne->routine;
                        add_to_hashtable (vc_table, look, sne);
			}
		    }
		else {
		    /*
		     * If there is a vc instance, destroy it, if it has not
		     * been destroyed already
		     */
                    if (look) {
		       if (dob_get_object_from_id (look->the_vc_id))
                          dob_destroy_data_object (look->the_vc, TRUE);
                       delete_from_hashtable (vc_table, sne);
                       free_node (look);
                       }
		    }
		break;
	    case T_get_PARENT_vc:
		dra_get_PARENT_vc (&PARENT[sne->routine], SELF);
		break;
	    case T_dmsg_setup:
		dri_pop (7, 0);
		ME.slang_line_num = (int)dri_tos[0];
		ME.slang_file = &dri_string_table[(int)dri_tos[1]];
		ME.cur_type = (cus_code_type)dri_tos[2];
		ME.str1 = &dri_string_table[(int)dri_tos[3]];
		ME.str2 = &dri_string_table[(int)dri_tos[4]];
		ME.cur_vc = &dri_string_table[(int)dri_tos[5]];
                ME.sne = sne;
		dmsg_setup (&ME, (int)dri_tos[6] ? sne : (dgr_sne *)0);
		break;

	    case T_string_constant:
		dri_pop (1, 0);
		dri_push (&dri_string_table[(int)dri_tos[0]]);
		break;
	    case T_slang_file:
		dri_pop (1, 0);
		d22_slang_file = &dri_string_table[(int)dri_tos[0]];
		break;
	    case T_slang_line_num:
		dri_pop (1, 0);
		d22_slang_line_num = (int)dri_tos[0];
		break;
	    case T_reset_slang_line_num:
		d22_slang_line_num = -1;
		break;
	    case T_method_verify:
		dri_pop (1, 0);
		if (strcmp (&dri_string_table[(int)dri_tos[0]],
		    cus_sym_representation (
			dob_get_symbol_entry (
			    cus_get_prop (
				dob_get_symbol_entry (SELF), template_prop)))))
		    dri_push ((caddr_t)1);
		else
		    dri_push ((caddr_t)0);
		break;
	    /*
	     * Call a non-valued (i.e., void) user function.  The stack
	     * looks like:
	     *     arg_0 arg_1 ... num_args function_idx T_void_user_function
	     */
	    case T_void_user_function:
		dri_pop (2, 0);
		num_user_parms = (int)dri_tos[0];
		user_funct = dri_user_functs[(int)dri_tos[1]];
		dri_pop (num_user_parms, 0);
		switch (num_user_parms) {
		    case 0:
			(void)(*user_funct) ();
			break;
		    case 1:
			(void)(*user_funct) (
			    (caddr_t)dri_tos[0]);
			break;
		    case 2:
			(void)(*user_funct) (
			    (caddr_t)dri_tos[0],
			    (caddr_t)dri_tos[1]);
			break;
		    case 3:
			(void)(*user_funct) (
			    (caddr_t)dri_tos[0],
			    (caddr_t)dri_tos[1],
			    (caddr_t)dri_tos[2]);
			break;
		    case 4:
			(void)(*user_funct) (
			    (caddr_t)dri_tos[0],
			    (caddr_t)dri_tos[1],
			    (caddr_t)dri_tos[2],
			    (caddr_t)dri_tos[3]);
			break;
		    case 5:
			(void)(*user_funct) (
			    (caddr_t)dri_tos[0],
			    (caddr_t)dri_tos[1],
			    (caddr_t)dri_tos[2],
			    (caddr_t)dri_tos[3],
			    (caddr_t)dri_tos[4]);
			break;
		    case 6:
			(void)(*user_funct) (
			    (caddr_t)dri_tos[0],
			    (caddr_t)dri_tos[1],
			    (caddr_t)dri_tos[2],
			    (caddr_t)dri_tos[3],
			    (caddr_t)dri_tos[4],
			    (caddr_t)dri_tos[5]);
			break;
		    case 7:
			(void)(*user_funct) (
			    (caddr_t)dri_tos[0],
			    (caddr_t)dri_tos[1],
			    (caddr_t)dri_tos[2],
			    (caddr_t)dri_tos[3],
			    (caddr_t)dri_tos[4],
			    (caddr_t)dri_tos[5],
			    (caddr_t)dri_tos[6]);
			break;
		    case 8:
			(void)(*user_funct) (
			    (caddr_t)dri_tos[0],
			    (caddr_t)dri_tos[1],
			    (caddr_t)dri_tos[2],
			    (caddr_t)dri_tos[3],
			    (caddr_t)dri_tos[4],
			    (caddr_t)dri_tos[5],
			    (caddr_t)dri_tos[6],
			    (caddr_t)dri_tos[7]);
			break;
		    case 9:
			(void)(*user_funct) (
			    (caddr_t)dri_tos[0],
			    (caddr_t)dri_tos[1],
			    (caddr_t)dri_tos[2],
			    (caddr_t)dri_tos[3],
			    (caddr_t)dri_tos[4],
			    (caddr_t)dri_tos[5],
			    (caddr_t)dri_tos[6],
			    (caddr_t)dri_tos[7],
			    (caddr_t)dri_tos[8]);
			break;
		    case 10:
			(void)(*user_funct) (
			    (caddr_t)dri_tos[0],
			    (caddr_t)dri_tos[1],
			    (caddr_t)dri_tos[2],
			    (caddr_t)dri_tos[3],
			    (caddr_t)dri_tos[4],
			    (caddr_t)dri_tos[5],
			    (caddr_t)dri_tos[6],
			    (caddr_t)dri_tos[7],
			    (caddr_t)dri_tos[8],
			    (caddr_t)dri_tos[9]);
			break;
		    default:
			FATAL_ERROR (
			    fprintf (stderr, "Too many parameters to user function!\n");
			    );
		    }
		break;
	    /*
	     * Call a valued (i.e., non-void) user function.  The stack
	     * looks like:
	     *     function_idx arg_0 arg_1 ... num_args T_user_function
	     */
	    case T_user_function:
		dri_pop (2, 0);
		num_user_parms = (int)dri_tos[0];
		user_funct = dri_user_functs[(int)dri_tos[1]];
		dri_pop (num_user_parms, 0);
		switch (num_user_parms) {
		    case 0:
			dri_push (
			    (*user_funct) ());
			break;
		    case 1:
			dri_push (
			    (*user_funct) (
				(caddr_t)dri_tos[0]));
			break;
		    case 2:
			dri_push (
			    (*user_funct) (
				(caddr_t)dri_tos[0],
				(caddr_t)dri_tos[1]));
			break;
		    case 3:
			dri_push (
			    (*user_funct) (
				(caddr_t)dri_tos[0],
				(caddr_t)dri_tos[1],
				(caddr_t)dri_tos[2]));
			break;
		    case 4:
			dri_push (
			    (*user_funct) (
				(caddr_t)dri_tos[0],
				(caddr_t)dri_tos[1],
				(caddr_t)dri_tos[2],
				(caddr_t)dri_tos[3]));
			break;
		    case 5:
			dri_push (
			    (*user_funct) (
				(caddr_t)dri_tos[0],
				(caddr_t)dri_tos[1],
				(caddr_t)dri_tos[2],
				(caddr_t)dri_tos[3],
				(caddr_t)dri_tos[4]));
			break;
		    case 6:
			dri_push (
			    (*user_funct) (
				(caddr_t)dri_tos[0],
				(caddr_t)dri_tos[1],
				(caddr_t)dri_tos[2],
				(caddr_t)dri_tos[3],
				(caddr_t)dri_tos[4],
				(caddr_t)dri_tos[5]));
			break;
		    case 7:
			dri_push (
			    (*user_funct) (
				(caddr_t)dri_tos[0],
				(caddr_t)dri_tos[1],
				(caddr_t)dri_tos[2],
				(caddr_t)dri_tos[3],
				(caddr_t)dri_tos[4],
				(caddr_t)dri_tos[5],
				(caddr_t)dri_tos[6]));
			break;
		    case 8:
			dri_push (
			    (*user_funct) (
				(caddr_t)dri_tos[0],
				(caddr_t)dri_tos[1],
				(caddr_t)dri_tos[2],
				(caddr_t)dri_tos[3],
				(caddr_t)dri_tos[4],
				(caddr_t)dri_tos[5],
				(caddr_t)dri_tos[6],
				(caddr_t)dri_tos[7]));
			break;
		    case 9:
			dri_push (
			    (*user_funct) (
				(caddr_t)dri_tos[0],
				(caddr_t)dri_tos[1],
				(caddr_t)dri_tos[2],
				(caddr_t)dri_tos[3],
				(caddr_t)dri_tos[4],
				(caddr_t)dri_tos[5],
				(caddr_t)dri_tos[6],
				(caddr_t)dri_tos[7],
				(caddr_t)dri_tos[8]));
			break;
		    case 10:
			dri_push (
			    (*user_funct) (
				(caddr_t)dri_tos[0],
				(caddr_t)dri_tos[1],
				(caddr_t)dri_tos[2],
				(caddr_t)dri_tos[3],
				(caddr_t)dri_tos[4],
				(caddr_t)dri_tos[5],
				(caddr_t)dri_tos[6],
				(caddr_t)dri_tos[7],
				(caddr_t)dri_tos[8],
				(caddr_t)dri_tos[9]));
			break;
		    default:
			FATAL_ERROR (
			    fprintf (stderr, "Too many parameters to user function!\n");
			    );
		    }
		break;

	    /*
	     * The following are literal tokens - they correspond to
	     * a simple literal constant, so push them onto the stack.
	     */
	    case C_SELF:
		dri_push ((caddr_t)SELF);
		break;
	    case C_PARENT:
		dri_adi_push (PARENT[sne->routine]);
		break;
	    case C_vc_sym:
		dri_push ((caddr_t)vc_sym);
		break;
	    case C_object_sym:
		dri_push ((caddr_t)object_sym);
		break;
	    case C_variable_sym:
		dri_push ((caddr_t)variable_sym);
		break;
	    case C_cus_ATTR:
		dri_push ((caddr_t)cus_ATTR);
		break;
	    case C_cus_CREATE:
		dri_push ((caddr_t)cus_CREATE);
		break;
	    case C_cus_DESTROY:
		dri_push ((caddr_t)cus_DESTROY);
		break;
	    case C_cus_METH:
		dri_push ((caddr_t)cus_METH);
		break;
	    case C_cus_CCOND:
		dri_push ((caddr_t)cus_CCOND);
		break;
	    case C_cus_VAR:
		dri_push ((caddr_t)cus_VAR);
		break;
	    case C_dob_id_NULL:
		dri_adi_push (dob_id_NULL);
		break; 
	    case C_dob_bool_TRUE:
		dri_adi_push (dob_bool_TRUE);
		break; 
	    case C_dob_bool_FALSE:
		dri_adi_push (dob_bool_FALSE);
		break; 
	    case C_dob_int_const:
		dri_pop (1, 0);
		dri_adi_push (dob_int_const[(int)dri_tos[0]]);
		break; 
	    case C_dob_real_const:
		dri_pop (1, 0);
		dri_adi_push (dob_real_const[(int)dri_tos[0]]);
		break; 

	    default:
		/*
		 * If the token is negative, we are lost - all negative
		 * numbers are command tokens, and we missed this one.
		 *
		 * If the token is positive, it is a literal value, so
		 * just push it into the stack.
		 */
		if ((int)token < 0)
		    FATAL_ERROR ( fprintf (stderr, "Unknown token: %d!\n", (int)token) )
		else
		    dri_push ((caddr_t)token);
	    }
	}

}					/* end dri_interpret */



void dgr_run_snippet (sne)
dgr_sne *sne;
{
    push_astack (d22_save_pfl, d22_pfl);
    d22_pfl = make_list ();
    d22_pot_fl_time_stamp++;
    push_astack (d22_save_sne, d22_sne);
    if ((sne->routine > NO_GENERIC_ROUTINE) &&
	    (sne->routine < dri_max_snippets) &&
	    dob_validate_object (sne->self_vc_object))
	dri_interpret (sne);

    dpg_process_pfl ();
    d22_pfl = (LIST) pop_astack (d22_save_pfl);
    d22_pot_fl_time_stamp--;
    d22_sne = (dgr_sne *) pop_astack (d22_save_sne);
}

static char *dri_dialogue_name = "";

char *din_get_dm_name ()
{
    return dri_dialogue_name;
}

void dri_init (dlg_name)
char *dlg_name;
{
    int i, fd, pathlen;
    char *env, *name, *fullname, *path;
    dri_header file_header;
    struct stat statbuf;
    dri_int iconst;
    dri_real rconst;
    char *getenv();

    vc_table = make_hashtable (VC_HASH_SIZE, dri_hash_vc, dri_match_vc);
    if (vc_table)
       set_status (ok);
    else {
       set_status (out_of_memory);
       return;
       }

    dri_dialogue_name = strcpy (make_node (strlen (dlg_name) + 1), dlg_name);

    /*
     * Fabricate the string/constants file name, and remember where you
     * located the file.  This is to ensure that we get all of the data
     * files from the same place.
     */
    name = make_node (1 /*.*/ + strlen (dlg_name) + 2 /*.z*/ + 1 /*\0*/ );
    env = getenv ("SERPENT_DATA_PATH");
    if (env == NULL || strlen(env) == 0) {
	fputs ("dri_init: WARNING - Could not find SERPENT_DATA_PATH in environment\n", stderr);
	env = ".";
	}
    (void)strcat (strcat (strcpy (name, "."), dlg_name), ".z");
    fullname = ufa_full_file_name (env, name);
    if (fullname == NULL) {
	fprintf (stderr, "dri_init: Cannot locate string/constants table %s\n", name);
	exit (1);
	}
    pathlen = strrchr(fullname, '/') - fullname;
    path = make_node (pathlen + 1);
    (void) strncpy (path, fullname, pathlen);
    /*
     * Read in the string/constants table.  This entails reading in the
     * string table, then reading in the integer constants (and making
     * the associated adi's), and repeating with the reals.
     */
    if ((fd = open (fullname, 0)) == -1) {
	perror ("dri_init: Cannot open string/constants table");
	exit (1);
	}
    fstat (fd, &statbuf);
    if (statbuf.st_size < sizeof (dri_header)) {
	fputs ("dri_init: Cannot read string/constants table header!\n", stderr);
	exit (1);
	}
    read (fd, &file_header, sizeof (dri_header));
    if (file_header.magic != DRI_MAGIC) {
	fprintf ("dri_init: %s is not a string/constants file!\n", fullname);
	exit (1);
	}
    if (file_header.version != DRI_VERSION) {
	fprintf (stderr, "dri_init: WARNING - Version number in %s (%d) is different\n",
	    dlg_name, file_header.version);
	fprintf (stderr, "\tfrom that in the DM (%d).\n\n", DRI_VERSION);
	}
    dri_max_snippets = file_header.max_snippets;
    if (statbuf.st_size != sizeof (dri_header) +
			    file_header.string_size +
			    (file_header.int_const_size * sizeof (dri_int)) +
			    (file_header.real_const_size * sizeof (dri_real))) {
	fputs ("dri_init: String/constant declared header size != file size!\n", stderr);
	exit (1);
	}
    if ((dri_string_table = make_node (file_header.string_size)) == NULL) {
	fputs ("dri_init: Cannot allocate string table!\n", stderr);
	exit (1);
	}
    read (fd, dri_string_table, file_header.string_size);

    dob_bool_TRUE = dob_make_boolean_CONSTANT_adi (1);
    dob_bool_FALSE = dob_make_boolean_CONSTANT_adi (0);
    dob_id_NULL = dob_make_id_CONSTANT_adi (-1);

    if (file_header.int_const_size > 0) {
	dob_int_const = (dro_atomic_data_item *) make_node (
			    file_header.int_const_size *
			    sizeof (dro_atomic_data_item));
	if (dob_int_const == NULL) {
	    fputs ("dri_init: Cannot allocate integer constants table!\n", stderr);
	    exit (1);
	    }
	}
    for (i = 0; i < file_header.int_const_size; i++) {
	read (fd, &iconst, sizeof (dri_int));
	dob_int_const[iconst.idx] = dob_make_integer_CONSTANT_adi (iconst.val);
	}

    if (file_header.real_const_size > 0) {
	dob_real_const = (dro_atomic_data_item *) make_node (
			    file_header.real_const_size *
			    sizeof (dro_atomic_data_item));
	if (dob_real_const == NULL) {
	    fputs ("dri_init: Cannot allocate real constants table!\n", stderr);
	    exit (1);
	    }
	}
    for (i = 0; i < file_header.real_const_size; i++) {
	read (fd, &rconst, sizeof (dri_real));
	dob_real_const[rconst.idx] = dob_make_real_CONSTANT_adi (rconst.val);
	}
    close (fd);

    /*
     * Read in the snippet pointer table, and create the PARENT array
     * and initialize it.
     */
    fullname = make_node (pathlen + 2 /*/.*/ + strlen (dlg_name) + 2 /*.t*/ + 1 /*\0*/ );
    strcat (strcat (strcat (strcpy (fullname, path), "/."), dlg_name), ".t");
    if ((fd = open (fullname, 0)) == -1) {
	perror ("dri_init: Cannot open snippet index table");
	fprintf (stderr, "\tWas looking for %s\n", fullname);
	exit (1);
	}
    fstat (fd, &statbuf);
    if ((statbuf.st_size / sizeof (long)) != dri_max_snippets) {
	fputs ("dri_init: Snippet count != snippet indirect table size!\n", stderr);
	exit (1);
	}
    if ((dri_snip_table = (long *)make_node (statbuf.st_size)) == NULL) {
	fputs ("dri_init: Cannot Cannot allocate snippet indirect table!\n", stderr);
	exit (1);
	}
    read (fd, dri_snip_table, statbuf.st_size);
    if ((PARENT = (dro_atomic_data_item *)make_node (
			dri_max_snippets *
			sizeof (dro_atomic_data_item))) == NULL) {
	fputs ("dri_init: Cannot allocate snippet indirect table!\n", stderr);
	exit (1);
	}
    for (i = 0; i < dri_max_snippets; i++)
	PARENT[i].dob_handle = NULL;
    close (fd);

    /*
     * Read in the snippet code table, and initialize it.
     */
    strcat (strcat (strcat (strcpy (fullname, path), "/."), dlg_name), ".x");
    if ((fd = open (fullname, 0)) == -1) {
	perror ("dri_init: Cannot open snippet code table");
	fprintf (stderr, "\tWas looking for %s\n", fullname);
	exit (1);
	}
    fstat (fd, &statbuf);
    if ((dri_code = (short *)make_node (statbuf.st_size)) == NULL) {
	fputs ("dri_init: Cannot allocate snippet code table!\n", stderr);
	exit (1);
	}
    read (fd, dri_code, statbuf.st_size);
    close (fd);
}

