	static char rcsid[]= "$Header: /u/04c/mnt/integ/source/externs/src/RCS/arith.c,v 1.6 1991/06/28 18:02:44 dvk Exp $";

/*--------------------------------------------------------------------------*\
|   
|  Name:  arith.c
|
|  Description:  
|     This file defines extended arithmetic functions for Slang programs.
|
|  Component of:
|     Function libraries.
|
|  Related files:
|     arith.ext - Slang declarations of the functions.
|
|  Project:
|     Serpent
|     User Interface Project
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Brian M. Clapper
|     bmc@sei.cmu.edu  
|
$Log: arith.c,v $
 * Revision 1.6  1991/06/28  18:02:44  dvk
 * Added round_to_nearest function
 *
 * Revision 1.5  1991/03/21  13:27:50  mpottier
 * confusion broke out...made a few fixes to itoo again...
 *
 * Revision 1.4  91/03/21  12:48:25  mpottier
 * Changed itoo to return an octal string. Tested it also.
 * 
 * Revision 1.3  91/03/08  16:17:50  mpottier
 * *** empty log message ***
 * 
 * Revision 1.2  91/03/08  16:01:40  mpottier
 * cd ../include
 * checkingooops...added itoo function
 * 
 * Revision 1.1  89/10/08  23:15:35  serpent
 * Initial revision
 * 
 * Revision 1.1  89/09/12  18:40:11  bmc
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|
\*--------------------------------------------------------------------------*/

#include "global.h"

/*--------------------------------------------------------------------------*\
|  Routine: make_integer
|
|  Description:
|     Converts a Slang expression to an integer, forcing truncation if
|     necessary.
|
|  Parameters:
|     operand - The expression or variable to be converted to integer.
|               While the above syntax implies that only integers are
|               accepted, in reality, any type of expression may be passed.
|               Conversion is handled by the Slang run-time system.
|
|  Result:
|     An integer version of the operand is passed back.  Real data is
|     truncated, not rounded.
|
|  Notes:
|     This function is basically just a no-op.  By the time it is called,
|     the generate Slang code has already converted the argument integer.
|     The function is only necessary because there has to be *something*
|     around to call.
\*                                                                          */

int make_integer (the_integer)
int the_integer;
{
    return the_integer;
}

/*--------------------------------------------------------------------------*\
|  Routine: itoo
|
|  Description:
|     Converts an integer value to its octal equivelant. This value is then
|     returned in integer form.
|
|  Parameters:
|     operand - The integer to be converted.
|
|  Result:
|     The octal equivelant in a string
|
|  Notes:
|     This function is intended to be used with the cursor edit attribute
|     editor. It allows an integer to be converted and used to access the
|     low level ascii codes of the cursor font. It the fonts are not being
|     displayed correctly, this may be the place to check.. not all fonts
|     use the same octal addresses (or so I've been told).
\*                                                                          */

string itoo(number)
  int number;
{
static
string octals[] = {"\000","\001","\002","\003","\004","\005","\006","\007",
                   "\010","\011","\012","\013","\014","\015","\016","\017",
                   "\020","\021","\022","\023","\024","\025","\026","\027",
                   "\030","\031","\032","\033","\034","\035","\036","\037",
                   "\040","\041","\042","\043","\044","\045","\046","\047",
                   "\050","\051","\052","\053","\054","\055","\056","\057",
                   "\060","\061","\062","\063","\064","\065","\066","\067",
                   "\070","\071","\072","\073","\074","\075","\076","\077",
                   "\100","\101","\102","\103","\104","\105","\106","\107",
		   "\110","\111","\112","\113","\114","\115","\116","\117",
                   "\120","\121","\122","\123","\124","\125","\126","\127",
                   "\130","\131","\132","\133","\134","\135","\136","\137",
                   "\140","\141","\142","\143","\144","\145","\146","\147",
                   "\150","\151","\152","\153","\154","\155","\156","\157",
                   "\160","\161","\162","\163","\164","\165","\166","\167",
                   "\170","\171","\172","\173","\174","\175","\176","\177"};

    return(octals[number]);
}

/*--------------------------------------------------------------------------*\
|  Routine: round_to_nearest
| 
|  Description:  
|     This routine simply rounds its first parameter to the nearest multiple
|     of the second parameter, and returns it.
\*--------------------------------------------------------------------------*/
int round_to_nearest (what, round_to)
int what, round_to;
{
    int v1 = (what / round_to) * round_to;
    int v2 = ((what + round_to) / round_to) * round_to;

    if ((what - v1) <= (v2 - what))
	return v1;
    else
	return v2;
}

