/* $Header: /u/04c/mnt/integ/source/glue/include/RCS/g22.h,v 1.6 91/03/18 18:48:28 dvk Exp $ */

/*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*
$Log:	g22.h,v $
 * Revision 1.6  91/03/18  18:48:28  dvk
 * Added "priority" attribute component.
 * 
 * Revision 1.5  90/10/03  18:00:59  rnk
 * added conv_rtne to WIDGET structure
 * 
 * Revision 1.4  90/08/12  14:36:33  rnk
 * changed glue_common.h to gcm.h
 * 
 * Revision 1.3  90/08/09  18:14:50  rnk
 * modified the WIDGET structure to support widget_type
 * 
 * Revision 1.2  90/06/06  16:11:59  dvk
 * Added support for routine-valued parameters
 * 
 * Revision 1.1  90/05/17  16:12:43  dvk
 * Initial revision
 * 
 */


#ifndef g22
#define g22

#include "alistPack.h"
#include "idd.h"
#include "gt_type.h"
#include "gcm.h"		/* Defines AXS_* and ValueType */

/*
 * The union which lists all the possible types a value may be
 */
typedef union {
    char *s;
    float r;
    int i;
    } XYZZY;

/*
 * Internal storage representation of a widget
 */
typedef struct {
    char *name;		/* The name of the widget exported to Slang */
    char *description;	/* A textual description of the widget */
    char *userdef;	/* Anything the glue designer wants it to be */
    char *class;	/* The X class of the widget */
    char *include_name;	/* The include file containing the widget guts */
    ALIST attrs;	/* A list of widget attributes (struct ATTR) */
    ALIST methods;	/* A list of widget methods (struct METHOD) */
    char *check_rtne;	/* The check routine to verify attribute values */
    char *conv_rtne;	/* The convenience routine for this widget */
    WidgetType widget_type;	/* The type of this widget */
    } WIDGET;

/*
 * The data structure to store an attribute
 */
typedef struct {
    char *xname;	/* The name of the attribute in the X domain */
    char *sname;	/* The name of the attribute in the Serpent domain */
    GT_TYPE xtype;	/* The type of the attribute in the X domain */
    idd_data_types stype;/* The type of the attribute in the Slang domain */
    int range;		/* For strings etc., the range of the string */
    short is_proc;	/* Does this have a procedure name default */
    ValueType val_type;	/* Is there a default value there? */
    idd_data_types dtype;/* The type of the default value */
    XYZZY val;		/* The actual default value (if any) */
    char *description;	/* The help description string for the attribute */
    char *userdef;	/* Anything the glue designer wants it to be */
    short access;	/* The access rules allowed for this attribute */
    short priority;	/* Display priority for editor */
    WIDGET *widget;	/* The parent of this attribute */
    } ATTR;

/*
 * The data structure to store a methods
 */
typedef struct {
    char *name;		/* The name of the method */
    char *rtne;		/* The name of the C routine to call */
    ALIST parms;	/* The list of parameters for the method (ATTR *) */
    char *description;	/* The help description string for the method */
    char *userdef;	/* Anything the glue designer wants it to be */
    WIDGET *widget;	/* The parent of this method */
    } METH;

/*
 * The data structure to store a method's parameter
 */
typedef struct {
    char *name;		/* The given name of the parameter */
    ATTR *attr;		/* The actual attribute that corresponds to this name */
    } PARM;

#endif
