/* $Header: /u/04c/mnt/integ/source/glue/include/RCS/ggc.h,v 1.3 90/10/03 17:58:19 rnk Exp $ */

/*
 * This file describes the interface to the GLUE data extraction routines
 */

/*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*
$Log:	ggc.h,v $
 * Revision 1.3  90/10/03  17:58:19  rnk
 * added glue_get_conv_alist
 * 
 * Revision 1.2  90/06/06  16:14:46  dvk
 * Added glue_get_proc_alist routine
 * 
 * Revision 1.1  90/05/18  17:12:43  dvk
 * Initial revision
 * 
 */

#ifndef ggc
#define ggc

#include "alistPack.h"


/*==========================================================================*\
 * Routine:
 *	glue_get_class_alist
 *
 * Description:
 *	Given a technology name, opens the appropriate technology data file,
 * reads in the list of classes, and returns an ALIST of the classes.  This
 * routine ignores the remainder of the technology file.
 *	The name of the technology will typically be an unqualified file
 * name.  If an extension is given, it will be used - otherwise the file
 * extension is assumed to be ".tx".  If the filename is given as an
 * absolute path name, that file will be used, otherwise, the file will be
 * looked for in the directories specified in the environment variable
 * SERPENT_DATA_PATH
 *
 * Syntax:
 */

	ALIST glue_get_class_alist (
		/* char *name ; */
	    );

/*
 * Parameters:
 *	name : name of technology to be accessed
 *
 * Returns:
 *	Pointer to ALIST if open is successful
 *
 * Statuses:
 *	Returns (ALIST)NULL if open fails
 *
\*--------------------------------------------------------------------------*/


/*==========================================================================*\
 * Routine:
 *	glue_get_proc_alist
 *
 * Description:
 *	Given a technology name, opens the appropriate technology data file,
 * reads in the list of procedures, and returns an ALIST of the procedures.
 * This routine ignores the remainder of the technology file.
 *	The name of the technology will typically be an unqualified file
 * name.  If an extension is given, it will be used - otherwise the file
 * extension is assumed to be ".tx".  If the filename is given as an
 * absolute path name, that file will be used, otherwise, the file will be
 * looked for in the directories specified in the environment variable
 * SERPENT_DATA_PATH
 *
 * Syntax:
 */

	ALIST glue_get_proc_alist (
		/* char *name ; */
	    );

/*
 * Parameters:
 *	name : name of technology to be accessed
 *
 * Returns:
 *	Pointer to ALIST if open is successful
 *
 * Statuses:
 *	Returns (ALIST)NULL if open fails
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================*\
 * Routine:
 *	glue_get_conv_alist
 *
 * Description:
 *	Given a technology name, opens the appropriate technology data file,
 * reads in the list of convenience routines, and returns an ALIST of the 
 * convenience routines.  This routine ignores the remainder of the technology
 * file.
 *	The name of the technology will typically be an unqualified file
 * name.  If an extension is given, it will be used - otherwise the file
 * extension is assumed to be ".tx".  If the filename is given as an
 * absolute path name, that file will be used, otherwise, the file will be
 * looked for in the directories specified in the environment variable
 * SERPENT_DATA_PATH
 *
 * Syntax:
 */

	ALIST glue_get_conv_alist (
		/* char *name ; */
	    );

/*
 * Parameters:
 *	name : name of technology to be accessed
 *
 * Returns:
 *	Pointer to ALIST if open is successful
 *
 * Statuses:
 *	Returns (ALIST)NULL if open fails
 *
\*--------------------------------------------------------------------------*/

#endif
