/* $Header: /u/04c/mnt/integ/source/glue/include/RCS/glue_inst.h,v 1.1 90/05/17 16:21:52 dvk Exp $ */

/*
 * This file describes the routines for dealing with instances of
 * widgets and attributes for a given technology.
 */

/*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*
$Log:	glue_inst.h,v $
 * Revision 1.1  90/05/17  16:21:52  dvk
 * Initial revision
 * 
 */

#ifndef glue_inst
#define glue_inst

#undef True			/* This shuts up a cpp warning */
#undef False			/* This shuts up a cpp warning */
#include <X11/Intrinsic.h>
#include "alistPack.h"
#include "gt_type.h"
#include "glue_intf.h"
#include "iid.h"

typedef enum {
    NOT_STRS,
    AND_STRS
    } StrDestroy;

/*==========================================================================
 * The INST structure describes an instance of a widget.  It is used to
 * keep track of the individual incarnations of each widget type from
 * a given technology.
 */
typedef struct {
    Widget widget;	/* The X widget structure for this instance */
    iid_id_type id;	/* The Serpent ID of this instance */
    ALIST child;	/* An (iid_id_type)ALIST of children of this instance */
    ALIST attributes;	/* An (ATTR *)ALIST of attributes */
    WIDG *template;	/* A pointer back to the original widget structure */
    } INST;

/*==========================================================================*\
 * Routine:
 *	glue_create_instance
 *
 * Description:
 *	Given a technology and a widget, create an instance of that
 * widget.  Note that this does not create the necessary X image or
 * structure - what it does is create the Glue structure.  Both the
 * X Widget pointer and the Serpent ID pointer must be filled in with
 * a call to glue_register_instance()
 *
 * Syntax:
 */

	INST * glue_create_instance (
		/* WIDG * widget ; */
	    );

/*
 * Parameters:
 *	widget - a pointer to the widget for which an instance is to
 *			be created.
 *
 * Returns:
 *	A pointer to an instance of the widget
 *
 * Statuses:
 *	Returns (INST *)NULL if the widget is null or if out of memory
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================*\
 * Routine:
 *	glue_register_instance
 *
 * Description:
 *	Once an instance has been created, the caller must create the widget
 * in both the X and Serpent domains.  Once this has been done, the caller
 * then calls glue_register_instance to register the instance into the
 * recall tables used by glue_find_by_id and glue_find_by_widget.
 *	NOTE: This routine assumes that the values of instance->widget and
 * instance->id have been filled in prior to being called!
 *
 * Syntax:
 */

	boolean glue_register_instance (
		/* INST * instance ; */
	    );

/*
 * Parameters:
 *	instance - the instance returned by glue_create_instance
 *
 * Returns:
 *	true if the instance is registered correctly
 *
 * Statuses:
 *	Returns false if either the instance->widget or instance->id fields
 * are not unique, or if the instance is (INST *)NULL
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================*\
 * Routine:
 *	glue_get_by_id
 *
 * Description:
 *	Recovers an instance of a widget using the (unique) Serpent ID
 * as a recovery handle.
 *
 * Syntax:
 */

	INST * glue_get_by_id (
		/* iid_id_type id ; */
	    );

/*
 * Parameters:
 *	id - the ID used by the Saddle interface, which serves to identify
 *		the particular instance
 *
 * Returns:
 *	A pointer to the particular instance tagged by the ID
 *
 * Statuses:
 *	Returns (INST *)NULL if the ID tag is not found
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================*\
 * Routine:
 *	glue_get_by_widget
 *
 * Description:
 *	Recovers an instance of a widget using the (unique) X Widget
 * address as a recovery handle.
 *
 * Syntax:
 */

	INST * glue_get_by_widget (
		/* Widget widget ; */
	    );

/*
 * Parameters:
 *	widget - the Widget address used by X, which serves to identify
 *		the particular instance
 *
 * Returns:
 *	A pointer to the particular instance tagged by the Widget structure
 *
 * Statuses:
 *	Returns (INST *)NULL if the Widget tag is not found
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================*\
 * Routine:
 *	glue_get_attr_from_instance
 *
 * Description:
 *	Grab the attribute structure associated with a given (technology
 * or Serpent) name from an instance of a widget.
 *
 * Syntax:
 */

	ATTR * glue_get_attr_from_instance (
		/* INST *instance ;
		/* char *attr_name ; */
		/* NameType t_or_s ; */
	    );

/*
 * Parameters:
 *	instance - a pointer to the instance from which the attribute
 *			is to be retrieved
 *	attr_name - a pointer to a string of characters naming the attribute
 *	t_or_s - a switch which says to use technology names or Serpent names
 *
 * Returns:
 *	A pointer to the ATTR structure requested
 *
 * Statuses:
 *	Returns (ATTR *)NULL if the attribute is not found, or if the
 * instance pointer is NULL.
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================*\
 * Routine:
 *	glue_destroy_instance
 *
 * Description:
 *	Destroy an instance, removing the retrieval tags from the various
 * has lists, and freeing up the space associated with the instance.
 * 	NOTE: data space taken up by the values of attributes is NOT freed
 * unless the nodes flag is set to AND_STRS.  This means that if an attribute
 * has a value of type string or type buffer, glue_destroy_instance will do
 * a free_node on the data pointer if AND_STRS is set.  If NOT_STRS is
 * passed in, only the instance structure and the attributes ALIST is freed.
 *
 * Syntax:
 */

	void glue_destroy_instance (
		/* INST *instance ;
		/* StrDestroy do_strs ; */
	    );

/*
 * Parameters:
 *	instance - a pointer to the instance to be destroyed
 *	do_strs - a switch which says to free string space (or not)
 *		Can be either AND_STRS or NOT_STRS
 *
 * Returns:
 *	none
 *
 * Statuses:
 *	none
 *
\*--------------------------------------------------------------------------*/

#endif
