/* $Header: /u/04c/mnt/integ/source/glue/include/RCS/glue_intf.h,v 1.16 91/03/22 14:43:35 rcs Exp $ */

/*
 * This file describes the interface to the GLUE data extraction routines
 */

/*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/


/*
$Log:	glue_intf.h,v $
 * Revision 1.16  91/03/22  14:43:35  rcs
 * nnew access routine
 * 
 * Revision 1.15  91/03/18  18:46:59  dvk
 * Added "priority" attribute component.
 * 
 * Revision 1.14  90/11/30  10:51:52  rcs
 * *** empty log message ***
 * 
 * Revision 1.13  90/11/19  15:21:42  rcs
 * adadded headers for new access routines
 * 
 * Revision 1.12  90/10/12  08:37:04  little
 * add methods_got & attributes_got components to WIDG structure
 * 
 * Revision 1.11  90/10/03  17:59:48  rnk
 * added declarations for convenience routines 
 * 
 * Revision 1.10  90/08/17  11:52:35  rcs
 * added debug variable
 * 
 * Revision 1.9  90/08/14  12:15:55  rnk
 * added check_rtn to WIDG structure
 * 
 * Revision 1.8  90/08/12  14:36:09  rnk
 * changed glue_common.h to gcm.h
 * 
 * Revision 1.7  90/08/10  17:33:18  rnk
 * changed glue_get_attr_default to return a buffer
 * 
 * Revision 1.6  90/08/09  18:05:16  rnk
 * modified the WIDGET structure to support widget_type
 * 
 * Revision 1.5  90/06/06  16:09:24  dvk
 * Added declarations for glueSpecialProcList and glueSpecialProcCount.
 * 
 * Revision 1.4  90/05/30  14:42:53  dvk
 * Added an idx field to the ATTRs to correct a bug in glue_inst.c
 * 
 * Revision 1.3  90/05/21  12:13:56  dvk
 * Added ill_file and mailbox to TECH structure
 * 
 * Revision 1.2  90/05/18  17:20:33  dvk
 * Oops - no changes
 * 
 * Revision 1.1  90/05/17  16:22:01  dvk
 * Initial revision
 * 
 */

#ifndef glue_intf_h
#define glue_intf_h

#undef True			/* This shuts up a cpp warning */
#undef False			/* This shuts up a cpp warning */
#include <X11/Intrinsic.h>
#include "alistPack.h"
#include "gt_type.h"
#include "gcm.h"		/* Defines AXS_* and ValueType */
#include "idd.h"		/* Defines idd_boolean, etc. */

/*--------------------------------------------------------------------------*\
|  Global Variable: gif_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef gif_TOP
  int gif_debug = 1;
#else
  extern int gif_debug;
#endif

/*
\*--------------------------------------------------------------------------*/

/*==========================================================================
 * The routine glue_get_technology expects to find an array named
 * glueClassList.  This array lists the classes associated with the
 * various widgets in a technology, and the variable glueClassCount
 * contains the number of elements in the array.  For the technology
 * driver (e.g., SAW, SMOT, etc.), a module called <technology>_bind.c
 * will be created by Glue that contains the real glueClassList array,
 * and a routine called glueClassFiller, used to fill the array.  For
 * an application (i.e., the visual editor) gluelib.a contains a dummy
 * dummy module with dummy glueClassFiller routine, an empty glueClassList
 * and a glueClassCount of zero.
 *
 * The same commentary holds true for glueSpecialProcList/glueSpecialProcCount
 * and glueConvenienceRtnList/glueConvenienceRtnCount.
 */

extern WidgetClass *glueClassList[];
extern int glueClassCount;

typedef void (*PFV)();
extern PFV glueSpecialProcList[];
extern int glueSpecialProcCount;

typedef Widget (*PFW)();
extern PFW glueConvenienceRtnList[];
extern int glueConvenienceRtnCount;


/*==========================================================================
 * The NameType enumeration provides a switch to distinguish whether to use
 * technology names or Serpent names in the routines glue_get_attribute(),
 * glue_alist_all_attributes(), and glue_alist_defaulted_attributes()
 */
typedef enum {
    TECH_NAMES	= 0,
    SERP_NAMES	= 1
    } NameType;

/*==========================================================================
 * The TECH structure is the main mechanism to access information on
 * the various technologies in the Serpent suite.  NOTE: The caller
 * may read the WIDG structure here with impunity - however, it is
 * expected that the caller will use the faster routines provided below
 * rather than using a linear search algorithm.  WARNING: Do not fuck
 * with the hook structure!  It is for internal use only.
 */
typedef struct {
    char *description;	/* The description of the technology */
    char *mailbox;	/* The name of the mailbox for the technology */
    char *ill_file;	/* The name of the ILL file for the technology */
    GT_TYPE gt_last;	/* The last entry when this file was written */
    ALIST widgets;	/* WIDG* ALIST of all widgets in this technology */
    struct hook *hook;	/* Hook used by the internals of this package */
    } TECH;

/*==========================================================================
 * The WIDG structure defines the way in which widgets are stored
 * by glue access routines.  NOTE: The caller may read the ATTR and
 * METH structures here with impunity - however, it is expected that
 * the caller will use the faster routines provided below rather than
 * using a linear search algorithm.
 */
typedef struct {
    char *name;			/* The name of the widget */
    WidgetClass class;		/* Widget class for X access */
    ALIST attributes;		/* (ATTR*) ALIST of attributes for widget */
    ALIST methods;		/* (METH*) ALIST of methods for widget */
    char *description;		/* Description of widget */
    char *userdef;		/* Any string the glue designer wants  */
    PFV check_rtn;		/* Check routine for the widget */
    PFW conv_rtn;		/* Convenience routine for the widget */
    WidgetType widget_type;	/* The type of a widget */
    TECH *technology;		/* A pointer back to the technology */
    short idx;			/* Internal use only */
    short attributes_got;	/* editor app internal use */
    short methods_got;		/* editor app internal use */
    } WIDG;

/*==========================================================================
 * The ATTR structure defines the way in which attributes are stored
 * by glue access routines.
 */
typedef struct {
    string t_name;		/* Name of attribute in technology domain */
    string s_name;		/* Name of attribute in the Serpent domain */
    GT_TYPE x_type;		/* Type of attribute in the X domain */
    idd_data_types s_type;	/* Type in Serpent domain */
    ValueType val_type;		/* Does this attribute have a value? */
    union {
	int i; double r; string s; caddr_t c;
	} value;		/* Actual value (if any) */
    short access;		/* Access allowed on attribute */
    short priority;		/* Display priority for editor */
    string description;		/* Description of attribute */
    string userdef;		/* Anything the glue designer wants it 2 be */
    short idx;		/* Internal use only (index of ATTR in ALIST)*/
    } ATTR;

/*==========================================================================
 * The METH structure defines the way in which methods are stored
 * by glue access routines.
 */
typedef struct {
    char *s_name;		/* Name of method in Serpent domain */
    char *c_name;		/* Name of routine in C domain */
    int c_index;		/* Index of routine in XtActionsRec */
    ALIST parms;		/* (ATTR*) ALIST of parameters */
    char *description;		/* Description of method */
    char *userdef;		/* Anything the glue designer wants it to be */
    } METH;


/*==========================================================================*\
 * Routine:
 *	glue_get_technology
 *
 * Description:
 *	Given a technology name, opens the appropriate technology data file,
 * initializes a TECH structure, and returns a pointer to the structure
 * (similar to the way fopen returns a FILE *).
 *	The name of the technology will typically be an unqualified file
 * name.  If an extension is given, it will be used - otherwise the file
 * extension is assumed to be ".tx".  If the filename is given as an
 * absolute path name, that file will be used, otherwise, the file will be
 * looked for in the directories specified in the environment variable
 * SERPENT_DATA_PATH
 *
 * Syntax:
 */

	TECH * glue_get_technology (
		/* char *name ; */
	    );

/*
 * Parameters:
 *	name : name of technology to be accessed
 *
 * Returns:
 *	Pointer to TECH structure if open is successful
 *
 * Statuses:
 *	Returns (TECH *)NULL if open fails
 *
\*--------------------------------------------------------------------------*/


/*==========================================================================*\
 * Routine: 
 *	glue_alist_widgets
 *
 * Description:
 *	Given a technology, this routine returns a list of all of the
 * names of widgets associated with it.  The widgets themselves are
 * stored using the WIDG structure (described above), and can be gotten
 * using other routines in this package.
 *
 * Syntax:
 */

	ALIST glue_alist_widgets (
		/* TECH *technology ; */
	    );

/*
 * Parameters:
 *	technology : a pointer to the TECH structure corresponding to
 *		the technology to be listed.
 *
 * Returns:
 *	an ALIST of strings containing the widget names
 *
 * Statuses:
 *	Returns (ALIST)NULL if technology not opened.
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================*\
 * Routine: 
 *	glue_get_mailbox
 *
 * Description:
 *	The glue_get_mailbox returns the mailbox associated with the 
 *      specified technology.
 *
 * Syntax:
 */

	string glue_get_mailbox (
		/* TECH *technology ; */
	    );

/*
 * Parameters:
 *	technology : a pointer to the TECH structure corresponding to
 *		the technology to be listed.
 *
 * Returns:
 *	the mailbox
 *
 * Statuses:
 *	Returns (ALIST)NULL if technology not opened.
 *
\*--------------------------------------------------------------------------*/


/*==========================================================================*\
 * Routine: 
 *	glue_get_widget
 *
 * Description:
 *	Given a technology and a widget name, return a pointer to the WIDG
 * structure associaated with it.  The caller should use this routine to
 * access the class and description ccomponents, and should use other
 * routines to inspect the contents of the METH and ATTR ALISTs.
 *
 * Syntax:
 */

	WIDG * glue_get_widget (
		/* TECH *technology ; */
		/* char *widget_name ; */
	    );

/*
 * Parameters:
 *	technology : a pointer to the TECH structure corresponding to
 *		the technology to be accessed
 *	widget_name : a pointer to the name of the widget to be accessed
 *
 * Returns:
 *	a pointer to a widget structure associated with the widget name
 *
 * Statuses:
 *	Returns (WIDG *)NULL if the technology not opened or if the widget
 * name is not valid.
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================
 * Routine:
 *      glue_get_attribute_alist
 *
 * Description:
 *      This routine takes a technology and a widget name and returns
 * the attribute alist.
 *
 * Syntax:
 */

	ALIST glue_get_attribute_alist(
	  /* TECH *technology */
	  /* char *widget_name */
	);

/*
 * Parameters:
 *      technology : a pointer to the TECH structure corresponding to
 *              the technology to be accessed
 *      widget_name : a pointer to the name of the widget to be accessed
 *
 * Returns:
 *      The attribute list.
 *
 * Statuses:
 *      Returns (WIDG *)NULL if the technology not opened or if the widget
 * name is not valid.
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================*\
 * Routine: 
 *	glue_alist_all_attributes
 *
 * Description:
 *	Given a technology and a widget name, this routine returns a list
 * of all of the names of attributes associated with the widget.  The
 * attributes themselves are stored using the ATTR structure (described
 * above), and can be gotten using other routines in this package.
 *
 * Syntax:
 */

	ALIST glue_alist_all_attributes (
		/* TECH *technology ; */
		/* char *widget_name ; */
		/* NameType t_or_s; */
	    );

/*
 * Parameters:
 *	technology : a pointer to the TECH structure corresponding to
 *		the technology to be listed.
 *	widget_name : a pointer to the name of the widget to be accessed
 *	t_or_s : a switch which says to use technology names or Serpent names
 *
 * Returns:
 *	an ALIST of strings containing the attribute names.
 *
 * Statuses:
 *	Returns (ALIST)NULL if technology not opened or if the widget
 * name is invalid.
 *
\*--------------------------------------------------------------------------*/


/*==========================================================================*\
 * Routine: 
 *	glue_alist_defaulted_attributes
 *
 * Description:
 *	Given a technology and a widget name, this routine returns a list
 * of all of the names of attributes associated with the widget that have a
 * Serpent specified default value.
 *
 * Syntax:
 */

	ALIST glue_alist_defaulted_attributes (
		/* TECH *technology ; */
		/* char *widget_name ; */
		/* NameType t_or_s; */
	    );

/*
 * Parameters:
 *	technology : a pointer to the TECH structure corresponding to
 *		the technology to be listed.
 *	widget_name : a pointer to the name of the widget to be accessed
 *	t_or_s : a switch which says to use technology names or Serpent names
 *
 * Returns:
 *	an ALIST of strings containing the attribute names. If there are
 * no Serpent specified default values, the ALIST will be empty.
 *
 * Statuses:
 *	Returns (ALIST)NULL if technology not opened or if the widget
 * name is invalid.
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================
 * Routine: glue_get_widget_type
 *
 * Description:
 *      This routine takes a pointer to an widget structure and
 * returns the type of the widget.
 *
 * Syntax:
 */

	WidgetType glue_get_widget_type(
	  /* WIDG *widget */
	);

/*
 * Parameters:
 *      widget : a pointer to the widget structure to be accessed
 *
 * Returns:
 *      The widget type.
 *
 * Statuses:
 *      none.
\*--------------------------------------------------------------------------*/

/*==========================================================================
 * Routine: glue_get_userdef
 *
 * Description:
 *      This routine takes a pointer to an widget structure and
 * returns a pointer to the user def string.
 *
 * Syntax:
 */

	string glue_get_userdef(
	  /* WIDG *widget */
	);

/*
 * Parameters:
 *      widget : a pointer to the widget structure to be accessed
 *
 * Returns:
 *      The user defined string.
 *
 * Statuses:
 *      none.
\*--------------------------------------------------------------------------*/

/*==========================================================================
 * Routine:
 *      glue_get_attribute_name
 *
 * Description:
 *      This routine takes a pointer to an attribute structure and
 * returns the name of the attribute.
 *
 * Syntax:
 */

	string glue_get_attribute_name(
	  /* ATTR *attribute */
	);

/*
 * Parameters:
 *      attribute : a pointer to the attribute structure to be accessed
 *
 * Returns:
 *      The Serpent name for the attribute.
 *
 * Statuses:
 *      none.
\*--------------------------------------------------------------------------*/

/*==========================================================================
 * Routine:
 *      glue_get_attribute_xtype
 *
 * Description:
 *      This routine takes a pointer to an attribute structure and
 * returns the x type of the attribute.
 *
 * Syntax:
 */

	int glue_get_attribute_xtype(
	  /* ATTR *attribute */
	);

/*
 * Parameters:
 *      attribute : a pointer to the attribute structure to be accessed
 *
 * Returns:
 *      The x type of the attribute.
 *
 * Statuses:
 *      none.
\*--------------------------------------------------------------------------*/

/*==========================================================================
 * Routine:
 *      glue_get_attribute_priority
 *
 * Description:
 *      This routine takes a pointer to an attribute structure and
 * returns the priority of the attribute.
 *
 * Syntax:
 */

	int glue_get_attribute_priority(
	  /* ATTR *attribute */
	);

/*
 * Parameters:
 *      attribute : a pointer to the attribute structure to be accessed
 *
 * Returns:
 *      The priority of the attribute.
 *
 * Statuses:
 *      none.
\*--------------------------------------------------------------------------*/

/*==========================================================================*\
 * Routine:  glue_get_attr_default
 *
 * Description:
 *	Given a widget and an attribute name, this routine returns a buffer
 * containing the default for that attribute.
 *
 * Syntax:
 */

	idd_buffer_type * glue_get_attr_default (
		/* WIDG *widget ; */
		/* char *attr_name ; */
		/* NameType t_or_s; */
	    );

/*
 * Parameters:
 *	widget : a pointer to the widget structure to be accessed
 *	attr_name : a pointer to the name of the attribute to be gotten
 *	t_or_s : a switch which says to use technology names or Serpent names
 *
 * Returns:
 *	A pointer to a buffer containing the attribute's default.
 *
 * Statuses:
 *	Returns NULL if the technology is not open, if the WIDG
 * structure is invalid, or if the attribute name is invalid.
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================*\
 * Routine:  glue_get_attr_description
 *
 * Description:
 *	Given a widget and an attribute name, this routine returns a buffer
 * containing the description field for that attribute.
 *
 * Syntax:
 */

	idd_buffer_type *glue_get_attr_description(
		/* WIDG *widget ; */
		/* char *attr_name ; */
		/* NameType t_or_s; */
	    );

/*
 * Parameters:
 *	widget : a pointer to the widget structure to be accessed
 *	attr_name : a pointer to the name of the attribute to be gotten
 *	t_or_s : a switch which says to use technology names or Serpent names
 *
 * Returns:
 *	A pointer to a buffer containing the attribute's default.
 *
 * Statuses:
 *	Returns NULL if the technology is not open, if the WIDG
 * structure is invalid, or if the attribute name is invalid.
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================*\
 * Routine:  glue_get_attr_access
 *
 * Description:
 *      Given a widget and an attribute name, this routine returns the
 * access type.
 *
 * Syntax:
 */

	int glue_get_attr_access(
 	  /* WIDG *widget ; */
	  /* char *attr_name ; */
	  /* NameType t_or_s; */
        );

/*
 * Parameters:
 *	widget : a pointer to the widget structure to be accessed
 *	attr_name : a pointer to the name of the attribute to be gotten
 *	t_or_s : a switch which says to use technology names or Serpent names
 *
 * Returns:
 *	The access type for the attribute.
 *
 * Statuses:
 *	Returns -1 if the technology is not open, if the WIDG
 * structure is invalid, or if the attribute name is invalid.
 *
\*--------------------------------------------------------------------------*/

/*==========================================================================*\
 * Routine:  glue_get_attribute
 *
 * Description:
 *	Given a widget and an attribute name, this routine returns a pointer
 * to the ATTR structure that corresponds to to the named attribute in the
 * specified widget.
 *
 * Syntax:
 */

	ATTR * glue_get_attribute (
		/* WIDG *widget ; */
		/* char *attr_name ; */
		/* NameType t_or_s; */
	    );

/*
 * Parameters:
 *	widget : a pointer to the widget structure to be accessed
 *	attr_name : a pointer to the name of the attribute to be gotten
 *	t_or_s : a switch which says to use technology names or Serpent names
 *
 * Returns:
 *	A pointer to the ATTR structure corresponding to that requested
 *
 * Statuses:
 *	Returns (ATTR *)NULL if the technology is not open, if the WIDG
 * structure is invalid, or if the attribute name is invalid.
 *
\*--------------------------------------------------------------------------*/


/*==========================================================================*\
 * Routine: 
 *	glue_alist_methods
 *
 * Description:
 *	Given a technology and a widget name, this routine returns a ALIST
 * of all of the names of methods associated with the widget.  The methods
 * themselves are stored using the METH structure (described above), and
 * can be gotten using other routines in this package.
 *
 * Syntax:
 */

	ALIST glue_alist_methods (
		/* TECH *technology ; */
		/* char *widget_name ; */
	    );

/*
 * Parameters:
 *	technology : a pointer to the TECH structure corresponding to
 *		the technology to be listed.
 *	widget_name : a pointer to the name of the widget to be accessed
 *
 * Returns:
 *	an ALIST of strings containing the methods names
 *
 * Statuses:
 *	Returns (ALIST)NULL if technology not opened or if the widget
 * name is invalid.
 *
\*--------------------------------------------------------------------------*/


/*==========================================================================*\
 * Routine: 
 *	glue_get_method
 *
 * Description:
 *	Given a widget and a method name, this routine returns a pointer
 * to the METH structure that corresponds to to the named method in the
 * specified widget.
 *
 * Syntax:
 */

	METH * glue_get_method (
		/* WIDG *widget ; */
		/* char *meth_name ; */
	    );

/*
 * Parameters:
 *	widget : a pointer to the widget structure to be accessed
 *	meth_name : a pointer to the name of the method to be gotten
 *
 * Returns:
 *	A pointer to the ATTR structure corresponding to that requested
 *
 * Statuses:
 *	Returns (ATTR *)NULL if the technology is not open, if the WIDG
 * structure is invalid, or if the method name is invalid.
 *
\*--------------------------------------------------------------------------*/

#endif
