        static char rcsid [] = "$Header: /usr/users/rnk/serpent/source/glue/src/RCS/check_intf.c,v 1.13 1991/06/14 02:31:04 rnk Exp $";

/*--------------------------------------------------------------------------
|
|  Name:	check_intf.c
|
|  Description:
|	This is the test module to check the functioning of glue_intf.c
|
|  Component of:
|	UNIT test for GLUE
|
|  Related files:
|	glue_intf.h
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:  
|	Daniel V. Klein
|	dvk@sei.cmu.edu
|
|  Change History:
$Log: check_intf.c,v $
 * Revision 1.13  1991/06/14  02:31:04  rnk
 * changed char type to KeySym, added XmNavigationType & removed XmStringList
 *
 * Revision 1.12  1991/03/18  18:47:51  dvk
 * Added "priority" attribute component.
 *
 * Revision 1.11  90/10/07  16:58:09  rnk
 * added "top" and "bottom" types
 * 
 * Revision 1.10  90/10/03  18:01:37  rnk
 * added support for convenience routines
 * 
 * Revision 1.9  90/09/01  20:04:32  ejh
 * took out include of strings.h
 * 
 * Revision 1.8  90/08/14  22:48:57  rnk
 * allowed for a widget_type of "none"
 * 
 * Revision 1.7  90/08/13  19:25:12  rnk
 * no changes
 * 
 * Revision 1.6  90/08/09  17:59:25  rnk
 * added code to support new types in gt_type.h, new serpent_default field
 * and the widget_type field.
 * 
 * Revision 1.5  90/06/06  16:10:24  dvk
 * Added code for new types in gt_type.h.  Also added code to print
 * out routine-valued attributes.
 * 
 * Revision 1.4  90/05/30  15:10:05  dvk
 * Changed include of <string.h> to <strings.h> (stupid Sun system...)
 * 
 * Revision 1.3  90/05/21  12:49:39  dvk
 * Added list of classes, mailbox, and ill_file to printout
 * 
 * Revision 1.2  90/05/18  17:22:45  dvk
 * Added printout of class list
 * 
 * Revision 1.1  90/05/17  16:08:19  dvk
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include "alistPack.h"
#include "hashPack.h"
#include "idd.h"
#include "glue_intf.h"
#include "ggc.h"
#include "gt_type.h"			/* see note below about GT_LAST_V? */

main (argc, argv)
int argc;
char **argv;
{
    TECH *t;
    ALIST widgets, attributes, methods, classes, procs, convs ;
    WIDG *w;
    ATTR *a;
    METH *m;
    char *cp;
    int w_idx, m_idx, a_idx, p_idx, c_idx, co_idx;
    int w_max, m_max, a_max, p_max, c_max, co_max;

    if (argc != 2) {
	fprintf (stderr, "Usage: %s <technology>[.tx]\n", *argv);
	exit (1);
	}

/*===========================================================================*/

    if ((classes=glue_get_class_alist (argv[1])) == (ALIST)NULL) {
	fprintf (stderr, "%s: glue_get_class_alist() returned NULL\n", *argv);
	exit (1);
	}
    c_max = alist_length (classes);
    fputs ("/*\n * Classes in use:\n", stdout);
    for (c_idx = 1; c_idx <= c_max; c_idx++)
	fprintf (stdout, " * \t%s\n", (char *) get_alist_pos (classes, c_idx));
    fputs (" */\n\n", stdout);
    

    if ((procs=glue_get_proc_alist (argv[1])) == (ALIST)NULL) {
	fprintf (stderr, "%s: glue_get_proc_alist() returned NULL\n", *argv);
	exit (1);
	}
    p_max = alist_length (procs);
    fputs ("/*\n * Procedures in use:\n", stdout);
    for (p_idx = 1; p_idx <= p_max; p_idx++)
	fprintf (stdout, " * \t%s\n", (char *) get_alist_pos (procs, p_idx));
    fputs (" */\n\n", stdout);
    
    if ((convs=glue_get_conv_alist(argv[1])) != (ALIST)NULL) 
      {
	co_max = alist_length (convs);
	fputs ("/*\n * Convenience routines in use:\n", stdout);
	for (co_idx = 1; co_idx <= co_max; co_idx++)
	  fprintf (stdout, " * \t%s\n", (char *) get_alist_pos(convs, co_idx));
	fputs (" */\n\n", stdout);
      }    

/*===========================================================================*/

    if ((t=glue_get_technology (argv[1])) == (TECH *)NULL) {
	fprintf (stderr, "%s: glue_get_technology() returned NULL\n", *argv);
	exit (1);
	}

    /*
     * When you update this program to include newer versions of gt_type,
     * make sure you update this test to reflect the new GT_LAST_V? number.
     */
    if (t->gt_last != GT_LAST_V6)
	fprintf (stderr, "!WARNING! Technology newer than %s!\nYou need to modify check_intf.c\n", *argv);

    fputs ("#include \"glue_equiv.gl\"\n\n", stdout);
    if (t->description != NULL)
	fprintf (stdout, "description = \"%s\";\n", t->description);
    if (t->mailbox != NULL)
	fprintf (stdout, "/* mailbox = \"%s\"; */\n", t->mailbox);
    if (t->ill_file != NULL)
	fprintf (stdout, "/* ill_file = \"%s\"; */\n\n", t->ill_file);
    w_max = alist_length (t->widgets);
    widgets = glue_alist_widgets (t);
    if (w_max != alist_length (widgets)) {
	fputs ("?Widget list length inconsistency!\n", stderr);
	exit (1);
	}
    for (w_idx = 1; w_idx <= w_max; w_idx++) {
	w = glue_get_widget (t, get_alist_pos (widgets, w_idx));
	if (w == (WIDG *)NULL) {
	    fputs ("?Number of widgets doesn't match list!\n", stderr);
	    exit (1);
	    }
	fprintf (stdout, "%s {\n", w->name);
	if (w->description != NULL)
	    fprintf (stdout, "    description = \"%s\";\n", w->description);
	if (w->userdef != NULL)
	    fprintf (stdout, "    userdef = \"%s\";\n", w->userdef);
	fprintf (stdout, "    /* class = %#X */\n", w->class);
	/*
	 * Iterate through the list of attributes, filling them in
	 */
	fputs ("    widget_type = ", stdout);
	switch (w->widget_type)
	  {
	  case WT_WIDGET:
	    fputs ("widget;\n", stdout);
	    break;
	  case WT_NONE:
	    fputs ("none;\n", stdout);
	    break;
	  case WT_WM_SHELL:
	    fputs ("shell;\n", stdout);
	    break;
	  case WT_OVERRIDE_SHELL:
	    fputs ("override shell;\n", stdout);
	    break;
	  }
	fprintf (stdout, "    attributes = {\n");
	attributes = glue_alist_all_attributes (t, w->name, TECH_NAMES);
	if (attributes == (ALIST)NULL) {
	    fputs ("?No attribute list!\n", stderr);
	    exit (1);
	    }
	a_max = alist_length (attributes);
	for (a_idx = 1; a_idx <= a_max; a_idx++) {
	    a = glue_get_attribute (w,
				get_alist_pos (attributes, a_idx), TECH_NAMES);
	    if (a == (ATTR *)NULL) {
		fputs ("?Number of attributes doesn't match list!\n", stderr);
		exit (1);
		}
	    fprintf (stdout, "\t\t\"%s\" : {\n", a->t_name);
	    if (strcmp (a->t_name, a->s_name))
		fprintf (stdout, "\t\t    serpent_name = \"%s\";\n",a->s_name);
	    fputs ("\t\t    x_type = ", stdout);
	    switch (a->x_type) {
		case GT_Accelerators:
		    fputs ("Accelerators;\n", stdout);
		    break;
		case GT_Boolean:
		    fputs ("Boolean;\n", stdout);
		    break;
		case GT_bottom:
		    fputs ("bottom;\n", stdout);
		    break;
		case GT_caddr_t:
		    fputs ("caddr_t;\n", stdout);
		    break;
		case GT_Cardinal:
		    fputs ("Cardinal;\n", stdout);
		    break;
		case GT_KeySym:
		    fputs ("KeySym;\n", stdout);
		    break;
		case GT_char_star:
		    fputs ("char_star;\n", stdout);
		    break;
		case GT_Colormap:
		    fputs ("Colormap;\n", stdout);
		    break;
		case GT_Cursor:
		    fputs ("Cursor;\n", stdout);
		    break;
		case GT_Dimension:
		    fputs ("Dimension;\n", stdout);
		    break;
		case GT_float:
		    fputs ("float;\n", stdout);
		    break;
		case GT_focus:
		    fputs ("focus;\n", stdout);
		    break;
		case GT_int:
		    fputs ("int;\n", stdout);
		    break;
		case GT_Pixel:
		    fputs ("Pixel;\n", stdout);
		    break;
		case GT_Pixmap:
		    fputs ("Pixmap;\n", stdout);
		    break;
		case GT_Pointer:
		    fputs ("Pointer;\n", stdout);
		    break;
		case GT_Position:
		    fputs ("Position;\n", stdout);
		    break;
		case GT_Screen:
		    fputs ("Screen;\n", stdout);
		    break;
		case GT_short:
		    fputs ("short;\n", stdout);
		    break;
		case GT_String:
		    fputs ("String;\n", stdout);
		    break;
		case GT_top:
		    fputs ("top;\n", stdout);
		    break;
		case GT_unsigned_char:
		    fputs ("unsigned_char;\n", stdout);
		    break;
		case GT_Widget:
		    fputs ("Widget;\n", stdout);
		    break;
		case GT_WidgetList:
		    fputs ("WidgetList;\n", stdout);
		    break;
		case GT_Window:
		    fputs ("Window;\n", stdout);
		    break;
		case GT_XawTextSelectType_star:
		    fputs ("XawTextSelectType_star;\n", stdout);
		    break;
		case GT_XFontStruct:
		    fputs ("XFontStruct;\n", stdout);
		    break;
		case GT_XID:
		    fputs ("XID;\n", stdout);
		    break;
		case GT_XmCreatePopupChildProc:
		    fputs ("XmCreatePopupChildProc;\n", stdout);
		    break;
		case GT_XmFontList:
		    fputs ("XmFontList;\n", stdout);
		    break;
		case GT_XmString:
		    fputs ("XmString;\n", stdout);
		    break;
		case GT_XmStringDirection:
		    fputs ("XmStringDirection;\n", stdout);
		    break;
		case GT_XmTextPosition:
		    fputs ("XmTextPosition;\n", stdout);
		    break;
		case GT_XtCallbackList:
		    fputs ("XtCallbackList;\n", stdout);
		    break;
		case GT_XtOrderProc:
		    fputs ("XtOrderProc;\n", stdout);
		    break;
		case GT_XtTranslations:
		    fputs ("XtTranslations;\n", stdout);
		    break;
		case GT_XmStringTable:
		    fputs ("XmStringTable;\n", stdout);
		    break;
		case GT_XmNavigationType:
		    fputs ("XmNavigationType;\n", stdout);
		    break;
		case GT_XtProc:
		    fputs ("XtProc;\n", stdout);
		    break;
		case GT_WidgetClass:
		    fputs ("WidgetClass;\n", stdout);
		    break;
		case GT_unsigned_int:
		    fputs ("unsigned_int;\n", stdout);
		    break;
		case GT_MethodName:
		    fputs ("MethodName;\n", stdout);
		    break;
		default:
		    fputs ("?Unknown technology type!\n", stdout);
		    exit (1);
		}
	    fputs ("\t\t    /* serpent_type = ", stdout);
	    switch (a->s_type) {
		case idd_integer:
		    fputs ("integer; */\n", stdout);
		    break;
		case idd_boolean:
		    fputs ("boolean; */\n", stdout);
		    break;
		case idd_real:
		    fputs ("real; */\n", stdout);
		    break;
		case idd_string:
		    fputs ("string; */\n", stdout);
		    break;
		case idd_buffer:
		    fputs ("buffer; */\n", stdout);
		    break;
		case idd_id:
		    fputs ("id; */\n", stdout);
		    break;
		default:
		    fputs ("?Unknown serpent type!\n", stdout);
		    exit (1);
		}
	    fprintf (stdout, "\t\t    access = ");
	    if (a->access == AXS_ALL)
		fputs ("all;\n", stdout);
	    else {
		if ( ! (a->access & AXS_CREATE))
		    fputc ('!', stdout);
		fputs ("create, ", stdout);
		if ( ! (a->access & AXS_GET))
		    fputc ('!', stdout);
		fputs ("get, ", stdout);
		if ( ! (a->access & AXS_SET))
		    fputc ('!', stdout);
		fputs ("set, ", stdout);
		if ( ! (a->access & AXS_TECH))
		    fputc ('!', stdout);
		fputs ("technology, ", stdout);
		if ( ! (a->access & AXS_SERPENT))
		    fputc ('!', stdout);
		fputs ("serpent;\n", stdout);
		}
	    if (a->val_type == VAL_TECH_DEFAULT)
	      switch (a->s_type) {
	        case idd_id:
		case idd_integer:
		    fprintf (stdout, "\t\t    technology_default = %d;\n",
						a->value.i);
		    break;
		case idd_boolean:
		    fprintf (stdout, "\t\t    technology_default = %s;\n",
				    a->value.i ? "true" : "false");
		    break;
		case idd_real:
		    fprintf (stdout, "\t\t    technology_default = %f;\n",
						a->value.r);
		    break;
		case idd_buffer:
		case idd_string:
		    fprintf (stdout, "\t\t    technology_default = \"%s\";\n",
						a->value.s);
		    break;
		}
	    if (a->val_type == VAL_SERP_DEFAULT)
	      switch (a->s_type) {
	        case idd_id:
		case idd_integer:
		    fprintf (stdout, "\t\t    serpent_default = %d;\n",
						a->value.i);
		    break;
		case idd_boolean:
		    fprintf (stdout, "\t\t    serpent_default = %s;\n",
				    a->value.i ? "true" : "false");
		    break;
		case idd_real:
		    fprintf (stdout, "\t\t    serpent_default = %f;\n",
						a->value.r);
		    break;
		case idd_buffer:
		case idd_string:
		    fprintf (stdout, "\t\t    serpent_default = \"%s\";\n",
						a->value.s);
		    break;
		}
	    fprintf (stdout, "\t\t    priority = %d;\n", a->priority);
	    if (a->description != NULL)
		fprintf (stdout, "\t\t    description = \"%s\";\n", a->description);
	    if (a->userdef != NULL)
		fprintf (stdout, "\t\t    userdef = \"%s\";\n", a->userdef);
	    fputs ("\t\t    }\n", stdout);
	    }
	fputs ("\t\t};\n", stdout);
	/*
	 * Iterate through the list of methods, filling them in
	 */
	methods = glue_alist_methods (t, w->name);
	if (methods == (ALIST)NULL) {
	    fputs ("?No method list!\n", stderr);
	    exit (1);
	    }
	m_max = alist_length (methods);
	if (m_max > 0) {
	    fprintf (stdout, "    methods = {\n");
	    for (m_idx = 1; m_idx <= m_max; m_idx++) {
		m = glue_get_method (w, get_alist_pos (methods, m_idx));
		if (m == (METH *)NULL) {
		    fputs ("?Number of methods doesn't match list!\n", stderr);
		    exit (1);
		    }
		fprintf (stdout, "\t\t%s : {\n", m->s_name);
		if ((p_max=alist_length (m->parms)) != 0) {
		    fputs ("\t\t    parameters = ", stdout);
		    for (p_idx = 1; p_idx <= p_max; p_idx++) {
			a = (ATTR *) get_alist_pos (m->parms, p_idx);
			fprintf (stdout, "%s\"%s\"",
				    p_idx > 1 ? ", " : "",
				    a->s_name);
			}
		    fputs (";\n", stdout);
		    }
		if (m->description != NULL)
		    fprintf (stdout, "\t\t    description = \"%s\";\n", m->description);
		if (m->userdef != NULL)
		    fprintf (stdout, "\t\t    userdef = \"%s\";\n", m->userdef);
		fputs ("\t\t     }\n", stdout);
		}
	    fputs ("\t\t};\n", stdout);
	    }
	fputs ("    };\n\n", stdout);
	}
}
