%{
        static char rcsid [] = "$Header: /u/04c/mnt/integ/source/glue/src/RCS/gal.l,v 1.11 1991/06/18 19:47:15 dvk Exp $";

/*--------------------------------------------------------------------------*\
|
|  Name:	gal.l
|
|  Description:
|	This is the lexical analyzer for GLUE.
|
|  Component of:
|	GLUE processor
|
|  Related files:
|	many yacc, lex, and c files
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:  
|	Daniel V. Klein
|	dvk@sei.cmu.edu
|
|  Change History:
$Log: gal.l,v $
 * Revision 1.11  1991/06/18  19:47:15  dvk
 * Fixed problem whereby lexer got confused on "# line file" rule when file
 * name was mot present.
 *
 * Revision 1.10  1991/06/14  02:28:56  rnk
 * increased YYLMAX to allow for longer strings (translation tables)
 *
 * Revision 1.9  1991/04/17  21:35:46  dvk
 * Added support to lexer to alow negative numbers in parser.
 *
 * Revision 1.8  1991/03/18  18:46:47  dvk
 * Added "priority" attribute component.
 *
 * Revision 1.7  91/02/07  08:46:32  rnk
 * no changes
 * 
 * Revision 1.6  90/10/03  18:04:55  rnk
 * added support for convenience routines
 * 
 * Revision 1.5  90/08/09  18:00:41  rnk
 * added new tokens for widget_type and technology_default
 * 
 * Revision 1.4  90/06/21  11:41:34  dvk
 * Rick needed big strings ( > 1024 chars), so I tripled the size
 * 
 * Revision 1.3  90/06/08  16:17:11  dvk
 * Added smarter string parsing - recognizes full set of C escape
 * characters now (check it out - neat use of %Start codes!)
 * 
 * Revision 1.2  90/06/06  16:12:13  dvk
 * Added new token '+' to support new glue_equiv.gl and routine-valued
 * attributes.  Installed smarter string parsing routines (allows a
 * string to cross multiple lines if each line ends with a '\', and
 * allows double quotes to be in strings if they are marked with \").
 * Also added FindTokensOrValues to help support routine-valued attrs.
 * 
 * Revision 1.1  90/05/17  16:09:28  dvk
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	memoryPack

#include	<stdio.h>	/* for I/O */
#include	"u22.h"		/* String copying and converting */
#include	"g22.h"		/* Data structure declarations */
#include	"gay.h"		/* yacc token declarations */

	/* make the max input buffer size something reasonable */

#undef YYLMAX
#define YYLMAX 4*BUFSIZ

char	lextext [YYLMAX];	/* line buffer; YYLMAX is plenty big enough */
char *	lextp = lextext;	/* a pointer to it */
char *	yyfile;
double  atof(); 
int	i, j;

#define	get_line() \
	(*lextp == '\0'? (fgets (lextext, YYLMAX - 1, yyin), \
		lextp = lextext, *lextp++): *lextp++)

#undef	input
#define	input() \
	(((yytchar = yysptr > yysbuf?U (*--yysptr):get_line ()) == \
		'\n'?yylineno++:0), feof (yyin)?0:yytchar)

#undef	unput
#define	unput(c) \
	{yytchar = (c); if (yytchar == '\n') yylineno--; *yysptr++=yytchar;}

/*
 * These two routines allow the parser to communicate with the lexer to
 * indicate when it should detect keywords.  Since it is possible for
 * Slang or C tokens to have the same value as Glue "keywords", we only
 * recognize tokens as keywords when they are expected.  Otherwise, all
 * words are recognized as tokens, which are passwd through intact.  As
 * a side effect, keywords are all converted to lower case.
 */

static short doKeyLookup = 0;
static short doOnlyValues = 0;

void FindTokens()
{
    doKeyLookup = 0;
}

void FindTokensOrValues()
{
    doKeyLookup = 1;
    doOnlyValues = 1;
}

void FindKeywords()
{
    doKeyLookup = 1;
    doOnlyValues = 0;
}

/*
 *	Why, you may ask, do we have this routine here when lex can do all
 * of this silly stuff faster, and with a simple description?  The answer
 * is that it makes for a smaller, and faster lexer.  Doing a binary tree
 * search is pretty fast stuff, and while lex is pretty fast, it has to do
 * a lot more work than that if it has to decode each keyword.
 */
#define Key(str, tok)		{str, tok, tok}
#define Val(str, tok, lexval)	{str, tok, lexval}
#define END(v)  (v-1 + sizeof v / sizeof v[0])
 
struct keytable {
    char *keyword;
    int token;
    int lexval;
    };

static struct keytable keytable[] = {
    /*
     * Alphabetical order is a MUST!!
     */
    Key( "access",		ACCESS ),
    Key( "attributes",		ATTRIBUTES ),
    Key( "c_routine",		C_ROUTINE ),
    Key( "check_routine",	CHECK_ROUTINE ),
    Key( "class",		CLASS ),
    Key( "convenience_routine",	CONVENIENCE_ROUTINE ),
    Key( "description",		DESCRIPTION ),
    Val( "false",		BOOLEAN, 0 ),
    Key( "include_file",	INCLUDE_FILE ),
    Key( "methods",		METHODS ),
    Key( "parameters",		PARAMETERS ),
    Key( "priority",		PRIORITY ),
    Key( "serpent_default",	SERPENT_DEFAULT ),
    Key( "serpent_name",	SERPENT_NAME ),
    Key( "technology_default",	TECHNOLOGY_DEFAULT ),
    Val( "true",		BOOLEAN, 1 ),
    Key( "userdef",		USERDEF ),
    Key( "widget_type",		WIDGET_TYPE ),
    Key( "x_type",		X_TYPE ),
    };

static int keylookup()
{
    struct keytable *low = keytable,
		    *high = END(keytable),
		    *mid;
    char *cp;
    int c;
 
    cp = u22_convert_string (yytext, LOWERCASE);
    while (low <= high) {
	mid = low + (high-low)/2;
	if ((c=strcmp(mid->keyword,cp)) == 0) {
	    (void) free_node (cp);
	    if (doOnlyValues && mid->lexval == mid->token)
		break;
	    else {
		yylval.ival = mid->lexval;
		return mid->token;
		}
	    }
	else if (c < 0)
	    low = mid+1;
	else
	    high = mid-1;
	}
    /*
     *	If we got here, it is not a keyword, so we call it an ordinary token.
     */
    yylval.cptr = cp;
    return (C_TOKEN);
}

/*
 *	This routine converts hexadecimal number strings into decimal values.
 */
static int xatoi(str)
char *str;
{
    register int num = 0;
    register char ch;

    while (ch = *str++) {
	if (ch >= '0' && ch <= '9')
	    num = num*0x10 + (ch-'0');
	else if (ch >= 'a' && ch <= 'z')
	    num = num*0x10 + (ch-'a'+0xa);
	else
	    num = num*0x10 + (ch-'A'+0xa);
	}
    return (num);
}

/*
 * This routine and variables check for a string overrunning past the
 * end of a file.
 */

static int in_string = 0;
static int too_long = 0;
static int str_start_line;

yywrap()
{
    if (in_string) {
	yyerror ("EOF encountered in string begun on line %d", str_start_line);
	in_string = 0;
	}
    return 1;
}

%}

%Start tok str special octal2 octal3

%%

<tok>\"			{
			BEGIN str;
			str_start_line = yylineno;
			in_string = 1;
			too_long = 0;
			}

<str>[^"\n\\]*["\n\\]	{
			if (yytext[yyleng-1] == '\\') {
			    yymore();			/* More to come */
			    if (yyleng >= YYLMAX-132 && too_long == 0) {
				too_long = 1;
				yyerror ("Warning: String too long");
				}
			    BEGIN special;		/* Special processing */
			    yytext[--yyleng] = '\0';	/* Erase the '\' */
			    }
			else {
			    BEGIN tok;
			    in_string = 0;
			    if (yytext[yyleng-1] == '\n')
				yyerror("Missing close quote");
			    yytext[--yyleng] = '\0';	/* erase '"' or '\n' */
			    yylval.cptr = u22_copy_string (yytext);
			    return(C_STRING);
			    }
			}

<octal2>[0-7]		{
			yymore();
			if (yyleng >= YYLMAX-132 && too_long == 0) {
			    too_long = 1;
			    yyerror ("Warning: String too long");
			    }
			yytext[yyleng-2] *= 8;
			yytext[yyleng-2] += yytext[yyleng-1] - '0';
			yytext[--yyleng] = '\0';
			BEGIN octal3;
			}

<octal3>[0-7]		{
			yymore();
			if (yyleng >= YYLMAX-132 && too_long == 0) {
			    too_long = 1;
			    yyerror ("Warning: String too long");
			    }
			yytext[yyleng-2] *= 8;
			yytext[yyleng-2] += yytext[yyleng-1] - '0';
			yytext[--yyleng] = '\0';
			BEGIN str;
			}

<octal2,octal3>.	|
<octal2,octal3>\n	{
			yymore();
			if (yyleng >= YYLMAX-132 && too_long == 0) {
			    too_long = 1;
			    yyerror ("Warning: String too long");
			    }
			if (yytext[yyleng-2] == '\0')
			    yyerror ("Warning: String contains the null char");
			unput(yytext[yyleng-1]);
			yytext[--yyleng] = '\0';
			BEGIN str;
			}

<special>[0-7]		{
			yymore();
			if (yyleng >= YYLMAX-132 && too_long == 0) {
			    too_long = 1;
			    yyerror ("Warning: String too long");
			    }
			yytext[yyleng-1] -= '0';
			BEGIN octal2;
			}

<special>.		|
<special>\n		{
			yymore();
			if (yyleng >= YYLMAX-132 && too_long == 0) {
			    too_long = 1;
			    yyerror ("Warning: String too long");
			    }
			/*
			 * Replace the digraph with the special character
			 */
			switch (yytext[yyleng-1]) {
			    case 'n':
				yytext[yyleng-1] = '\n';
				break;
			    case 'b':
				yytext[yyleng-1] = '\b';
				break;
			    case 'f':
				yytext[yyleng-1] = '\f';
				break;
			    case 'r':
				yytext[yyleng-1] = '\r';
				break;
			    case 't':
				yytext[yyleng-1] = '\t';
				break;
			    case 'v':
				yytext[yyleng-1] = '\v';
				break;
			    case '\\':
				yytext[yyleng-1] = '\\';
				break;
			    case '"':
				yytext[yyleng-1] = '"';
				break;
			    case '\'':
				yytext[yyleng-1] = '\'';
				break;
			    case '\n':	/* This one means "ignore newline" */
				yytext[--yyleng] = '\0';
				break;
			    default:	/* Ignore everything else */
				yyerror ("Unknown escape character ignored");
				break;
			    }
			BEGIN str;	/* Continue string processing */
			}

<tok>[a-zA-Z][a-zA-Z0-9_]*	{
	if (yyleng > 32)
		{
		yytext [32] = '\0';
		yyleng = 32;
		}
	if ( doKeyLookup ) {
	    return (keylookup(yytext));
	    }
	else {
	    yylval.cptr = u22_copy_string(yytext);
	    return(C_TOKEN);
	    }
	}

<tok>[0-9]+"."[0-9]*	|
<tok>"."[0-9]+		{
			yylval.rval = atof (yytext);
			return (REAL);
			}

<tok>[0-9]+		{
			yylval.ival = atoi (yytext);
			return (INTEGER);
			}

<tok>0[Xx][0-9a-fA-F]+	{
			yylval.ival = xatoi (yytext);
			return (INTEGER);
			}

<tok>[-;:,\[\]{}()=!+]	return (yylval.ival = yytext[0]);

^#[^\n]*	{

	/*
	 * This says: if you find a pound sign at the beginning of a line,
	 * read the entire line, then pick off the line number and file name
	 * Some preprocesors only include the file name if it has changed
	 * since the last use, so also handle the case of just a line number.
	 */
	i = 1;
	while (!isdigit (yytext [i]))
	    i++;
	yylineno = 0;
	while (isdigit (yytext [i]))
	    yylineno = 10 * yylineno + yytext [i++] - '0';
	while (yytext [i] != '"' && yytext[i] != '\0')
	    i++;
	if (yytext[i] != '\0') {
	    i++;
	    j = i;
	    while (yytext [j] != '"')
		j++;
	    yytext [j] = '\0';
	    yyfile = u22_copy_string (yytext + i);
	    }
	}

<tok>[ \t\n\f]+		{ /* Skip whitespace */ }

<tok>.	{
	fprintf (stderr, "Illegal character in text:\n%s\n", yytext);
	fprintf (stderr, "At or near line %d in file %s\n", yylineno, yyfile);
	}

.	|
\n	{
	/*
	 * This rule is here because 1) Any Lex rule not preceded by a start
	 * condition (i.e., <tok>) will match for any start condition, and
	 * 2) Lex always tries to match the most it can for any start
	 * condition.  This means that, for example, if you are looking for
	 * special character (start condition <special>), the rule which
	 * matches tokens will succeed over the rule to match a single
	 * special character *unless* the former rule is in a start condition
	 * which is excluded.  Thus we have the start condition <tok> which
	 * covers everything which is not otherwise covered.  However, we
	 * have to be able to get into that start condition, so this rule
	 * matches only one thing, and immediately shifts start conditions.
	 *
	 * Because the two rules immediately preceding this one will always
	 * match anything which this rule would match, even though this rule
	 * remains active, it is ignored after the first character.
	 */
	BEGIN tok;
	unput (yytext[0]);
	}

%%
