        static char rcsid [] = "$Header: /usr/users/little/serpent/src/glue/src/RCS/gen.c,v 1.22 1991/05/14 22:43:59 little Exp $";

/*--------------------------------------------------------------------------*\
|
|  Name:	gen.c
|
|  Description:
|	This is the output file generator module for GLUE.
|
|  Component of:
|	GLUE processor
|
|  Related files:
|	many yacc, lex, and c files
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:  
|	Daniel V. Klein
|	dvk@sei.cmu.edu
|
|  Change History:
$Log: gen.c,v $
 * Revision 1.22  1991/05/14  22:43:59  little
 * add -m parameter and its workings
 *
 * Revision 1.21  1991/05/11  04:04:04  rcs
 * who knows
 *
 * Revision 1.20  91/03/18  18:48:13  dvk
 * Added "priority" attribute component.
 * 
 * Revision 1.19  91/02/16  22:59:13  ejh
 * changed PFW to PFV on the line declaring glueSpecialProcList when it's
 * of zero length
 * 
 * Revision 1.18  91/02/14  17:21:47  rnk
 * fixed problem with empty SpecialProc lists
 * 
 * Revision 1.17  90/10/03  18:05:50  rnk
 * added support for convenience routines
 * 
 * Revision 1.16  90/09/01  20:02:27  ejh
 * took out include of strings.h; put in include of X11/Xos.h to get rindex,
 * which is *strongly* suggested by the X folks
 * 
 * Revision 1.15  90/08/13  19:24:08  rnk
 * added code to generate check routines
 * 
 * Revision 1.14  90/08/10  17:34:41  rnk
 * changed method code generation to use the c_routine name for the include
 * file name
 * 
 * Revision 1.13  90/08/09  18:02:07  rnk
 * added code to support widget_type and serpent_default fields
 * 
 * Revision 1.12  90/08/02  16:08:37  rnk
 * removed generated code for methods and replaced it with a #include
 * 
 * Revision 1.11  90/07/19  15:17:36  dvk
 * No change
 * 
 * Revision 1.10  90/07/16  14:51:36  dvk
 * Rick requested that class and include file be optional - they now are
 * 
 * Revision 1.9  90/07/13  15:02:57  rnk
 * changed sdd startup command to <<technology technology>>.  This is
 * necessary while we have two versions of Six (Sat and Smo).
 * 
 * Revision 1.8  90/06/07  15:19:39  dvk
 * Fixed a small bug associated with procedure attributes
 * 
 * Revision 1.7  90/06/06  16:06:22  dvk
 * Changed so that file.sdd now uses <<six file>> instead of just
 * <<file>.  Also only generates file_bind with -M switch.  Also
 * added all support for routine-valued attributes.
 * 
 * Revision 1.6  90/05/30  15:15:01  dvk
 * Changed include of <string.h> to <strings.h> (stupid Sun system...)
 * 
 * Revision 1.5  90/05/22  14:38:53  dvk
 * Neatened up the printing aa little
 * 
 * Revision 1.4  90/05/22  11:20:47  dvk
 * No changes.  The bug was in add_unique_to_hashtable (!)
 * 
 * Revision 1.3  90/05/21  12:14:15  dvk
 * Added ill_file and mailbox to TECH structure
 * 
 * Revision 1.2  90/05/18  17:17:50  dvk
 * Added internal class consistency checking.  Also no longer
 * generates <file>.h, and renamed generated file <file>_methods.c
 * to a shorter <file>_meth.c
 * 
 * Revision 1.1  90/05/17  16:10:26  dvk
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <X11/Xos.h>
#include "alistPack.h"
#include "hashPack.h"
#include "u22.h"
#include "idd.h"
#include "gt_type.h"
#include "g22.h"
#include "ggc.h"
#include "tx_fmt.h"

/*
 * Global variables declared in this package and exported to others
 */
ALIST widget_list;		/* The main storage bin for widgets */
char *tech_description;		/* A description of this technology */
char *saddle_command;		/* The command description for <file>.sdd */

/* global variables used by this package */
extern char *mail_box_name_override;   /* command line optional input */


/*
 * Local variables for use in this package only
 */
static FILE *fd;		/* Output file descriptor */
static char *ill_file_name;	/* The name of the ILL file this tech uses */
static char *mailbox_name;	/* The name of the mailbox this tech uses */

/*
 * Simple hash table functions for cross checking methods
 */
#define METH_HASH_SIZE	30

static int method_hash (ident)
char *ident;
{
    if (ident == NULL)
	return 0;
    else
	return (ident[0]*strlen(ident) + ident[1]) % METH_HASH_SIZE;
}

static int method_match (ident, meth)
char *ident;
METH *meth;
{
    return (strcmp (ident, meth->rtne) == 0);
}

/*
 * Simple hash table functions and structures for building a string table
 */
#define STRING_HASH_SIZE 75

typedef struct {
    char *str;
    str_idx idx;
    } STRING;

static str_idx zero = 0;

static int string_hash (ident)
char *ident;
{
    int ln = strlen(ident);

    if (ident == NULL)
	return 0;
    else
	return (ident[0]*26*ln + ident[1]*ln + ident[ln-1]) % STRING_HASH_SIZE;
}

static int string_match (ident, s)
char *ident;
STRING *s;
{
    return (strcmp (ident, s->str) == 0);
}

/*
 * Routine to generate list of include files required by this glue spec
 */
static void gen_includes ()
{
    int w_index, w_len;
    WIDGET *w;

    w_len = alist_length (widget_list);
    for (w_index = 1; w_index <= w_len; w_index++) {
	w = (WIDGET *) get_alist_pos (widget_list, w_index);
	if (w->include_name != NULL)
	    fprintf (fd, "#include <%s>\n", w->include_name);
	}
}

/*
 * Routine to generate saddle file from this glue spec
 */
static void gen_saddle (technology)
char *technology;
{
    int w_index, w_len;
    int a_index, a_len;
    WIDGET *w;
    ATTR *a;

    fprintf (fd, "<<%s %s>>\n\n", technology,
		saddle_command != NULL ? saddle_command : technology);
    fprintf (fd, "%s : shared data\n\n", technology);

    w_len = alist_length (widget_list);
    for (w_index = 1; w_index <= w_len; w_index++) {
	w = (WIDGET *) get_alist_pos (widget_list, w_index);
	fprintf (fd, "  %s : record\n", w->name);
	a_len = alist_length (w->attrs);
	for (a_index = 1; a_index <= a_len; a_index++) {
	    a = (ATTR *) get_alist_pos (w->attrs, a_index);
	    fprintf (fd, "    %s %.*s: ", a->sname, 22-strlen(a->sname),
			 "                                        ");
	    switch (a->stype) {
		case idd_boolean:
		    fputs ("boolean", fd);
		    break;
		case idd_string:
		    fputs ("string", fd);
		    break;
		case idd_integer:
		    fputs ("integer", fd);
		    break;
		case idd_real:	
		    fputs ("real", fd);
		    break;
		case idd_buffer:
		    fputs ("buffer", fd);
		    break;
		case idd_id:
		    fputs ("id", fd);
		    break;
		case idd_no_data_type:
		    /* Do nothing - prevents double error messages */
		    break;
		default:
		    fprintf (stderr, "?Unknown idd_type in gen_saddle()\n");
		    break;
		}
	    if (a->range >= 0)
		fprintf (fd, "[%d];\n", a->range);
	    else
		fputs (";\n", fd);
	    }
	fputs ("  end record;\n\n", fd);
	}
    fputs ("end shared data;\n", fd);
}

/*
 * Routine to consistency check the contents of the various methods and
 * to build the hash table used to access it later.
 */
static HASH check_methods ()
{
    int w_index, w_len;
    int m_index, m_len;
    int p_index, p_len;
    int a_index, a_len;
    int c_index, c_len;
    int found;
    WIDGET *w;
    METH *m, *prev;
    ATTR *a;
    PARM *p, *cmp;
    HASH methods = make_hashtable (METH_HASH_SIZE, method_hash, method_match);

    w_len = alist_length (widget_list);
    for (w_index = 1; w_index <= w_len; w_index++) {
	w = (WIDGET *) get_alist_pos (widget_list, w_index);
	m_len = alist_length (w->methods);
	a_len = alist_length (w->attrs);
	for (m_index = 1; m_index <= m_len; m_index++) {
	    m = (METH *) get_alist_pos (w->methods, m_index);
	    if (m->rtne == NULL) {
		m->rtne = (char *)make_node (strlen(m->name)+
						1+
						strlen("_method"));
		(void) strcat (strcpy (m->rtne, m->name), "_method");
		}
	    /*
	     * Consistency check the parameters - make sure that all the
	     * parameter names are real serpent names for this widget
	     */
	    if ((p_len=alist_length (m->parms)) > 0) {
		for (p_index = 1; p_index <= p_len; p_index++) {
		    p = (PARM *) get_alist_pos (m->parms, p_index);
		    for (found = 0, a_index = 1; a_index <= a_len; a_index++) {
			a = (ATTR *) get_alist_pos (w->attrs, a_index);
			if (strcmp (a->sname, p->name) == 0) {
			    found = 1;
			    p->attr = a;
			    break;
			    }
			if (strcmp (a->xname, p->name) == 0) {
			    found = -1;
			    p->attr = a;
			    }
			}
		    if (found <= 0) {
			fprintf (stderr, "ERROR: Parameter %d ('%s') of method '%s' in\n", p_index, p->name, m->name);
			fprintf (stderr, "\twidget '%s' is not the name of any attribute in the widget.\n", w->name);
			}
		    if (found == -1)
			fprintf (stderr, "\tHINT: Use the Serpent attribute name ('%s'), not the X name.\n", p->attr->sname);
		    }
		}
	    /*
	     * Stick the method into the hash table
	     */
	    prev = (METH *) add_unique_to_hashtable (methods, m, m->rtne);
	    /*
	     * If we found another entry in the hashtable, make sure that
	     * the number and names of parameters match
	     */
	    if (prev == NULL) {
		prev = (METH *) get_from_hashtable (methods, m->rtne);
		/*
		 * First the easy test - the number of parameters
		 */
		c_len = alist_length(prev->parms);
		if (p_len != c_len) {
		    fprintf (stderr, "ERROR: Number of parameters for routine '%s' for method '%s' in\n", m->rtne, m->name);
		    fprintf (stderr, "\twidget '%s' differs from previous uses of that method name\n", w->name);
		    fprintf (stderr, "\t(Initially defined in method '%s' of widget '%s').\n", prev->name, prev->widget->name);
		    }
		/*
		 * Now the N^2 algorithm to check that the names all match
		 */
		for (p_index = 1; p_index <= p_len; p_index++) {
		    p = (PARM *) get_alist_pos (m->parms, p_index);
		    for (found = 0, c_index = 1; c_index <= c_len; c_index++) {
			cmp = (PARM *) get_alist_pos (prev->parms, c_index);
			if (strcmp (cmp->name, p->name) == 0) {
			    found = 1;
			    break;
			    }
			}
		    if (found == 0) {
			fprintf (stderr, "ERROR: Parameter %d ('%s') for routine '%s' for method '%s' in\n", p_index, p->name, m->rtne, m->name);
			fprintf (stderr, "\twidget '%s' is not used in previous use of that routine\n", w->name);
			fprintf (stderr, "\t(Initially defined in method '%s' of widget '%s').\n", prev->name, prev->widget->name);
			}
		    }
		}
	    }
	}
    return (methods);
}

/*
 * Internal routine used by gen_methods() and gen_bindings()
 */
#define	RTNE_BODY	0
#define	RTNE_EXTERN	1
#define	RTNE_TABLE	2

static int print_method_names (key, m)
int key;
METH *m;
{
    int p_index, p_len;
    PARM *p;

    switch (key) {
	case RTNE_BODY:
	    fputs ("/*************************************************\n", fd);
	    fprintf (fd, " * Method name: %s\n", m->name);
	    fprintf (fd, " * Routine name: %s\n", m->rtne);
	    if ((p_len=alist_length (m->parms)) > 0) {
		fputs ( " * Parameters:\n", fd);
		for (p_index = 1; p_index <= p_len; p_index++) {
		    p = (PARM *) get_alist_pos (m->parms, p_index);
		    fprintf (fd, " *\t%s\t: ", p->name);
		    switch (p->attr->stype) {
			case idd_boolean:
			    fputs ("boolean\n", fd);
			    break;
			case idd_string:
			    fputs ("string\n", fd);
			    break;
			case idd_buffer:
			    fputs ("buffer\n", fd);
			    break;
			case idd_integer:
			    fputs ("integer\n", fd);
			    break;
			case idd_real:
			    fputs ("real\n", fd);
			    break;
			default:
			    fputs ("?unknown?\n", fd);
			    fprintf (stderr, "?Unknown idd_type in print_method_names()\n");
			    break;
			}
		    }
		}
	    fputs ( " */\n", fd);
	    fprintf (fd, "void %s (w, event, params, num_params)\n", m->rtne);
	    fputs ("Widget w;\n", fd);
	    fputs ("XEvent *event;\n", fd);
	    fputs ("String *params;\n", fd);
	    fputs ("Cardinal *num_params;\n", fd);
	    fputs ("{\n", fd);

	    /* The method's code is kept in a separate file included here */ 
	    fprintf (fd, "#include \"%s.c\"\n", m->rtne);
	    fputs ("}\n\n", fd);
	    break;
	case RTNE_EXTERN:
	    fprintf (fd, "extern void %s();\n", m->rtne);
	    break;
	case RTNE_TABLE:
	    fprintf (fd, "\t{ \"%s\",  %s },\n", m->name, m->rtne);
	    break;
	}
}

/*
 * Routine to generate list of method routines specified by this glue spec
 */
static void gen_methods (file, method_table)
char *file;
HASH method_table;
{
    fputs ("#include <X11/Intrinsic.h>\n", fd);
    fputs ("#include <X11/StringDefs.h>\n\n", fd);
    fprintf (fd, "/* Include files generated by Glue from %s.gl */\n", file);
    gen_includes();
    fputs ("\n/* Serpent and Glue include files */\n", fd);
    fputs ("#include \"isd.h\"\n", fd);
    fputs ("#include \"six_meth.h\"\n", fd);
    fputs ("#include \"glue_intf.h\"\n", fd);
    fputs ("#include \"glue_inst.h\"\n\n", fd);
    for_all_hashtable (method_table, RTNE_BODY, print_method_names);
}

/*
 * Routine to generate list of method bindings.
 */
static void gen_bindings (file, method_table, class_list, proc_list, conv_list)
char *file;
HASH method_table;
ALIST class_list;
ALIST proc_list;
ALIST conv_list;
{
    int c_index, c_len;
    int p_index, p_len;
    char *cp;

    fputs ("#include <X11/Intrinsic.h>\n\n", fd);
    fprintf (fd, "/* Include files generated by Glue from %s.gl */\n", file);
    gen_includes();
    putc ('\n', fd);
    for_all_hashtable (method_table, RTNE_EXTERN, print_method_names);
    fprintf (fd, "\nint glue_actions_count = %d;\n\n",
					size_of_hashtable (method_table));
    fputs ("XtActionsRec glue_actions[] = {\n", fd);
    for_all_hashtable (method_table, RTNE_TABLE, print_method_names);
    fputs ("};\n\n", fd);
    c_len = alist_length (class_list);
    fprintf (fd, "int glueClassCount = %d;\n\n", c_len);
    fprintf (fd, "WidgetClass *glueClassList[%d] = {\n", c_len);
    for (c_index = 1; c_index <= c_len; c_index++) {
	cp = (char *) get_alist_pos (class_list, c_index);
	if (strcmp(cp,""))
	    fprintf (fd, "    &%s,\n", cp);
	else
	    fputs ("    (WidgetClass *)NULL,\n", fd);
	}
    fputs ("};\n", fd);
    p_len = alist_length (proc_list);
    fprintf (fd, "int glueSpecialProcCount = %d;\n\n", p_len);

    c_len = alist_length (conv_list);
    fprintf (fd, "int glueConvenienceRtnCount = %d;\n\n", c_len);

    for (p_index = 1; p_index <= p_len; p_index++) {
	cp = (char *) get_alist_pos (proc_list, p_index);
	if (cp != NULL)
	    fprintf (fd, "extern void %s();\n", cp);
	}
    fputs ("\n", fd);
    fputs ("typedef void (*PFV)();\n", fd);
    if (p_len == 0)
      {
	fputs ("PFV glueSpecialProcList[1] = {\n    NULL\n", fd);
      }
    else
      {
	fprintf (fd, "PFV glueSpecialProcList[%d] = {\n", p_len);
	for (p_index = 1; p_index <= p_len; p_index++) {
	    cp = (char *) get_alist_pos (proc_list, p_index);
	    if (cp != NULL)
	      fprintf (fd, "    %s,\n", cp);
	    else
	      fputs ("    (PFV)NULL,\n", fd);
	  }
      }
    
    fputs ("};\n", fd);

    fputs ("\n", fd);
    fputs ("typedef Widget (*PFW)();\n", fd);
    if (c_len == 0)
      {
	fputs ("PFW glueConvenienceRtnList[1] = {\n    NULL\n", fd);
      }
    else
      {
	fprintf (fd, "PFW glueConvenienceRtnList[%d] = {\n", c_len);
	for (p_index = 1; p_index <= c_len; p_index++) 
	  {
	    cp = (char *) get_alist_pos (conv_list, p_index);
	    if (cp != NULL)
	      fprintf (fd, "    %s,\n", cp);
	    else
	      fputs ("    NULL,\n", fd);
	  }
      }
    fputs ("};\n", fd);
}

/*
 * Three routines to build string table used in technology data file
 */
static void stick_it (tab, cp)
HASH tab;
char *cp;
{
    STRING *s;

    if (cp == NULL)
	return;

    s = (STRING *) make_node (sizeof (STRING));
    s->str = cp;
    if ((STRING *)add_unique_to_hashtable (tab, s, cp) == NULL)
	free_node (s);
}

static HASH build_strings ()
{
    int w_index, w_len;
    int m_index, m_len;
    int p_index, p_len;
    int a_index, a_len;
    WIDGET *w;
    METH *m;
    ATTR *a;
    HASH strings = make_hashtable(STRING_HASH_SIZE, string_hash, string_match);

    stick_it (strings, tech_description);
    stick_it (strings, ill_file_name);
    stick_it (strings, mailbox_name);
    w_len = alist_length (widget_list);
    for (w_index = 1; w_index <= w_len; w_index++) {
	w = (WIDGET *) get_alist_pos (widget_list, w_index);
	stick_it (strings, w->name);
	stick_it (strings, w->description);
	stick_it (strings, w->userdef);
	stick_it (strings, w->class);
	stick_it (strings, w->check_rtne);
	stick_it (strings, w->conv_rtne);
	m_len = alist_length (w->methods);
	for (m_index = 1; m_index <= m_len; m_index++) {
	    m = (METH *) get_alist_pos (w->methods, m_index);
	    stick_it (strings, m->name);
	    stick_it (strings, m->description);
	    stick_it (strings, m->userdef);
	    }
	a_len = alist_length (w->attrs);
	for (a_index = 1; a_index <= a_len; a_index++) {
	    a = (ATTR *) get_alist_pos (w->attrs, a_index);
	    stick_it (strings, a->xname);
	    stick_it (strings, a->sname);
	    stick_it (strings, a->description);
	    stick_it (strings, a->userdef);
	    if (a->val_type == VAL_SERP_DEFAULT  ||
		a->val_type == VAL_TECH_DEFAULT)
	      if (a->stype==idd_string || a->stype==idd_buffer || a->is_proc)
		stick_it (strings, a->val.s);
	    }
	}
    return (strings);
}

static void dump_strings (file_idx, s)
long *file_idx;
STRING *s;
{
    int len;

    s->idx = *file_idx;
    len = strlen(s->str)+1;
    (void) fwrite (s->str, sizeof (char), len, fd);
    *file_idx += len;
}

/*
 * These two routines build the class list of the current technology file,
 * and compares it against the existing technology file, issuing a warning
 * message if if detects any phase errors
 */
static int in_order (s1, s2)
char *s1, *s2;
{
    return (strcmp (s1, s2) <= 0);
}

static ALIST check_classes (file)
char *file;
{
    count w_index, w_len;
    count c_index, c_len;
    WIDGET *w;
    char *cp, *xp;
    ALIST old_classes = glue_get_class_alist (file);
    ALIST new_classes = make_alist ();

    /*
     * For each widget, add its class to the list
     */
    w_len = alist_length (widget_list);
    for (w_index = 1; w_index <= w_len; w_index++) {
	w = (WIDGET *) get_alist_pos (widget_list, w_index);
	(void) add_to_atail (new_classes, w->class);
	}
    /*
     * Sort the list (alphabetically), and delete any duplicates
     */
    (void) sort_alist_func (new_classes, in_order);
    c_len = alist_length (new_classes);
    cp = (char *) get_alist_pos (new_classes, c_len);
    for (c_index = c_len-1; c_index >= 1; c_index--) {
	xp = (char *) get_alist_pos (new_classes, c_index);
	if (strcmp (cp, xp) == 0)
	    (void) delete_alist_pos (new_classes, c_index);
	else
	    cp = xp;
	}
    c_len = alist_length (new_classes);

    /*
     * Compare the new list against the existing technology (if any)
     */
    if (old_classes == (ALIST) NULL)
	return (new_classes);

    if (c_len != alist_length (old_classes)) {
	fprintf (stderr, "WARNING! The number of X classes used in %s.gl is\n", file);
	fprintf (stderr, "different from that in %s.tx.  You need to relink\n", file);
	fputs ("the technology driver or you will have erroneous behavior.\n\n", stderr);
	return (new_classes);
	}

    for (c_index = 1; c_index <= c_len; c_index++) {
	cp = get_alist_pos (old_classes, c_index);
	xp = get_alist_pos (new_classes, c_index);
	if (strcmp (cp, xp) != 0) {
	    fprintf (stderr, "WARNING! The names of X classes used in %s.gl are\n", file);
	    fprintf (stderr, "different from that in %s.tx.  You need to relink\n", file);
	    fputs ("the technology driver or you will have erroneous behavior.\n\n", stderr);
	    return (new_classes);
	    }
	}
	    
    return (new_classes);
}

/*
 * This routine builds the routine list for the current technology file
 * and compares it against the existing technology file, issuing a warning
 * message if if detects any phase errors
 */
static ALIST check_procs (file)
char *file;
{
    count w_index, w_len;
    count a_index, a_len;
    count p_index, p_len;
    WIDGET *w;
    ATTR *a;
    char *cp, *xp;
    ALIST old_procs = glue_get_proc_alist (file);
    ALIST new_procs = make_alist ();

    /*
     * For each widget, add routine attributes, convenience routines
     * and check_routines to the list
     */
    w_len = alist_length (widget_list);
    for (w_index = 1; w_index <= w_len; w_index++) {
	w = (WIDGET *) get_alist_pos (widget_list, w_index);
	if (w->check_rtne != NULL)
	  (void) add_to_atail (new_procs, w->check_rtne);

	a_len = alist_length (w->attrs);
	for (a_index = 1; a_index <= a_len; a_index++) {
	    a = (ATTR *) get_alist_pos (w->attrs, a_index);
	    if (a->is_proc && a->val_type == VAL_SERP_DEFAULT)
		(void) add_to_atail (new_procs, a->val.s);
	    }
	}
    /*
     * Sort the list (alphabetically), and delete any duplicates
     */
    (void) sort_alist_func (new_procs, in_order);
    p_len = alist_length (new_procs);
    cp = (char *) get_alist_pos (new_procs, p_len);
    for (p_index = p_len-1; p_index >= 1; p_index--) {
	xp = (char *) get_alist_pos (new_procs, p_index);
	if (strcmp (cp, xp) == 0)
	    (void) delete_alist_pos (new_procs, p_index);
	else
	    cp = xp;
	}
    p_len = alist_length (new_procs);

    /*
     * Compare the new list against the existing technology (if any)
     */
    if (old_procs == (ALIST) NULL)
	return (new_procs);

    if (p_len != alist_length (old_procs)) {
	fprintf (stderr, "WARNING! The number of X procedures used in %s.gl is\n", file);
	fprintf (stderr, "different from that in %s.tx.  You need to relink the\n", file);
	fputs ("technology driver or you will have erroneous behavior.\n\n", stderr);
	return (new_procs);
	}

    for (p_index = 1; p_index <= p_len; p_index++) {
	cp = get_alist_pos (old_procs, p_index);
	xp = get_alist_pos (new_procs, p_index);
	if (strcmp (cp, xp) != 0) {
	    fprintf (stderr, "WARNING! The names of X procedures used in %s.gl are\n", file);
	    fprintf (stderr, "different from that in %s.tx.  You need to relink\n", file);
	    fputs ("the technology driver or you will have erroneous behavior.\n\n", stderr);
	    return (new_procs);
	    }
	}
	    
    return (new_procs);
}

/*
 * This routine builds the convenience routine list for the current technology
 * file and compares it against the existing technology file, issuing a 
 * warning message if if detects any phase errors
 */
static ALIST check_convs (file)
char *file;
{
    count w_index, w_len;
    count a_index, a_len;
    count c_index, c_len;
    WIDGET *w;
    ATTR *a;
    char *cp, *xp;
    ALIST old_convs = glue_get_conv_alist (file);
    ALIST new_convs = make_alist ();

    /*
     * For each widget, add routine attributes, convenience routines
     * and check_routines to the list
     */
    w_len = alist_length (widget_list);
    for (w_index = 1; w_index <= w_len; w_index++) 
      {
	w = (WIDGET *) get_alist_pos (widget_list, w_index);
	if (w->conv_rtne != NULL)
	  (void) add_to_atail (new_convs, w->conv_rtne);
      }

    /*
     * Sort the list (alphabetically), and delete any duplicates
     */
    (void) sort_alist_func (new_convs, in_order);
    c_len = alist_length (new_convs);
    cp = (char *) get_alist_pos (new_convs, c_len);
    for (c_index = c_len-1; c_index >= 1; c_index--) {
	xp = (char *) get_alist_pos (new_convs, c_index);
	if (strcmp (cp, xp) == 0)
	    (void) delete_alist_pos (new_convs, c_index);
	else
	    cp = xp;
	}
    c_len = alist_length (new_convs);

    /*
     * Compare the new list against the existing technology (if any)
     */
    if (old_convs == (ALIST) NULL)
	return (new_convs);

    if (c_len != alist_length (old_convs)) {
	fprintf (stderr, "WARNING! The number of convenience routines used in %s.gl is\n", file);
	fprintf (stderr, "different from that in %s.tx.  You need to relink the\n", file);
	fputs ("technology driver or you will have erroneous behavior.\n\n", stderr);
	return (new_convs);
	}

    for (c_index = 1; c_index <= c_len; c_index++) {
	cp = get_alist_pos (old_convs, c_index);
	xp = get_alist_pos (new_convs, c_index);
	if (strcmp (cp, xp) != 0) {
	    fprintf (stderr, "WARNING! The names of convenience routines used in %s.gl are\n", file);
	    fprintf (stderr, "different from that in %s.tx.  You need to relink\n", file);
	    fputs ("the technology driver or you will have erroneous behavior.\n\n", stderr);
	    return (new_convs);
	    }
	}
	    
    return (new_convs);
}

static void gen_technology (file, string_table, class_list, proc_list, 
			    conv_list)
char *file;
HASH string_table;
ALIST class_list;
ALIST proc_list;
ALIST conv_list;
{
    TX tx_header;
    count w_index, w_len;
    count a_index, a_len;
    count m_index, m_len;
    count p_index, p_len;
    count c_index, c_len;
    count s_index, s_len;
    count co_index, co_len;
    GT_TYPE last;
    WIDGET *w;
    METH *m;
    ATTR *a;
    STRING *s;
    PARM *p;
    char *cp;


    tx_header.magic = TX_MAGIC;
    /*
     * Start the string table at the end of the header, and collect the
     * total size of the string table.  The first byte of the string table
     * is an empty string.  Index 0 into the table is special - it is
     * interpreted not as an empty string, but as the NULL string.  If there
     * is really an empty string used in the technology, it will be entered
     * into the hash table and processed as any other string.
     */
    fseek (fd, sizeof (TX), 0);
    (void) fwrite ("", sizeof (char), 1, fd);
    tx_header.string_size = 1;
    for_all_hashtable (string_table, &tx_header.string_size, dump_strings);
    /*
     * Write out the list of classes
     */
    c_len = alist_length (class_list);
    tx_header.num_classes = c_len;
    for (c_index = 1; c_index <= c_len; c_index++) {
	cp = (char *) get_alist_pos (class_list, c_index);
	s = (STRING *) get_from_hashtable(string_table, cp);
	(void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
	}
    /*
     * Write out the list of special procedure names
     */
    s_len = alist_length (proc_list);
    tx_header.num_procs = s_len;
    for (s_index = 1; s_index <= s_len; s_index++) {
	cp = (char *) get_alist_pos (proc_list, s_index);
	s = (STRING *) get_from_hashtable(string_table, cp);
	(void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
	}
    /*
     * Write out the list of convenience routines
     */
    co_len = alist_length (conv_list);
    tx_header.num_convs = co_len;
    for (co_index = 1; co_index <= co_len; co_index++) {
	cp = (char *) get_alist_pos (conv_list, co_index);
	s = (STRING *) get_from_hashtable(string_table, cp);
	(void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
	}
    /*
     * Write out the description string (if any) and the mailbox name
     * and the ILL file name
     */
    if (tech_description != NULL) {
	s = (STRING *) get_from_hashtable(string_table, tech_description);
	(void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
	}
    else
	(void) fwrite (&zero, sizeof (zero), 1, fd);
    s = (STRING *) get_from_hashtable(string_table, mailbox_name);
    (void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
    s = (STRING *) get_from_hashtable(string_table, ill_file_name);
    (void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
    /*
     * Sanity check - write out the GT_TYPE "version" number by writing
     * the last element in the list (less one, which then equates to the
     * most recent GT_LAST_V?)
     */
    last = GT_LAST-1;
    fwrite (&last, sizeof (last), 1, fd);
    /*
     * Now comes the real work - write all the attributes, and all of their
     * included methods.  This is done by writing out fixed length records
     * for each of the components (the variable parts - the strings - are
     * simply referenced by their byte index into the string array).
     */
    w_len = alist_length (widget_list);
    tx_header.num_widgets = w_len;
    for (w_index = 1; w_index <= w_len; w_index++) {
	w = (WIDGET *) get_alist_pos (widget_list, w_index);
	/*
	 * Write out the widget name, description, userdef, widget type
	 * and number of attributes
	 */
	s = (STRING *) get_from_hashtable (string_table, w->name);
	(void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
	if (w->description != NULL) {
	    s = (STRING *) get_from_hashtable(string_table, w->description);
	    (void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
	    }
	else
	    (void) fwrite (&zero, sizeof (zero), 1, fd);
	if (w->userdef != NULL) {
	    s = (STRING *) get_from_hashtable(string_table, w->userdef);
	    (void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
	    }
	else
	    (void) fwrite (&zero, sizeof (zero), 1, fd);

	/* Write out the widget type */
	(void) fwrite (&w->widget_type, sizeof (w->widget_type), 1, fd);

	/* Write out the index for this widget's check_routine. */
	if (w->check_rtne != NULL) 
	  {
	    for (s_index = 1; s_index <= s_len; s_index++) {
	      cp = (char *) get_alist_pos (proc_list, s_index);
	      if (strcmp(w->check_rtne, cp) == 0) {
		(void) fwrite (&s_index, sizeof (s_index), 1, fd);
		break;
	      }
	    }
	    if (s_index > s_len) {
	      fputs ("Consistency error in gen_technology()!\n", stderr);
	      fputs ("s_index > s_len\n", stderr);
	      exit (1);
	    }
	  }
	else
	  {
	    s_index = 0 ;
	    (void) fwrite (&s_index, sizeof (s_index), 1, fd);
	  }

	/* Write out the index for this widget's convenience_routine. */
	if (w->conv_rtne != NULL) 
	  {
	    for (co_index = 1; co_index <= co_len; co_index++) {
	      cp = (char *) get_alist_pos (conv_list, co_index);
	      if (strcmp(w->conv_rtne, cp) == 0) {
		(void) fwrite (&co_index, sizeof (co_index), 1, fd);
		break;
	      }
	    }
	    if (co_index > co_len) {
	      fputs ("Consistency error in gen_technology()!\n", stderr);
	      fputs ("co_index > co_len\n", stderr);
	      exit (1);
	    }
	  }
	else
	  {
	    co_index = 0 ;
	    (void) fwrite (&co_index, sizeof (co_index), 1, fd);
	  }

	/*
	 * Write out the index into the class array of this widget's class
	 */
	for (c_index = 1; c_index <= c_len; c_index++) {
	    cp = (char *) get_alist_pos (class_list, c_index);
	    if (strcmp (w->class, cp) == 0) {
		(void) fwrite (&c_index, sizeof (c_index), 1, fd);
		break;
		}
	    }
	if (c_index > c_len) {
	    fputs ("Consistency error in gen_technology()!\n", stderr);
	    fputs ("c_index > c_len\n", stderr);
	    exit (1);
	    }
	/*
	 * Write out the number of attributes
	 */
	a_len = alist_length (w->attrs);
	(void) fwrite (&a_len, sizeof (a_len), 1, fd);
	/*
	 * Now, for each attribute...
	 */
	for (a_index = 1; a_index <= a_len; a_index++) {
	    a = (ATTR *) get_alist_pos (w->attrs, a_index);
	    /*
	     * Write out the indices of the xname, sname, description,
	     * and userdef for the attribute.
	     */
	    s = (STRING *) get_from_hashtable (string_table, a->xname);
	    (void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
	    s = (STRING *) get_from_hashtable (string_table, a->sname);
	    (void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
	    if (a->description != NULL) {
		s = (STRING *)get_from_hashtable(string_table, a->description);
		(void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
		}
	    else
		(void) fwrite (&zero, sizeof (zero), 1, fd);
	    if (a->userdef != NULL) {
		s = (STRING *) get_from_hashtable(string_table, a->userdef);
		(void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
		}
	    else
		(void) fwrite (&zero, sizeof (zero), 1, fd);
	    /*
	     * Write out the X and Serpent types of the attribute, the access
	     * type, prioprity, and the default value, if any (leave a hole,
	     * if not).
	     */
	    (void) fwrite (&a->xtype, sizeof (a->xtype), 1, fd);
	    (void) fwrite (&a->stype, sizeof (a->stype), 1, fd);
	    (void) fwrite (&a->access, sizeof (a->access), 1, fd);
	    (void) fwrite (&a->priority, sizeof (a->priority), 1, fd);
	    (void) fwrite (&a->is_proc, sizeof (a->is_proc), 1, fd);
	    (void) fwrite (&a->val_type, sizeof (a->val_type), 1, fd);
	    if (a->val_type == VAL_SERP_DEFAULT ||
		a->val_type == VAL_TECH_DEFAULT)
	      switch (a->stype) {
		case idd_buffer:
		case idd_string:
		    s = (STRING *) get_from_hashtable (string_table, a->val.s);
		    (void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
		    break;
		case idd_boolean:
		    (void) fwrite (&a->val.i, sizeof (a->val.i), 1, fd);
		    break;
		case idd_integer:
		    if (a->is_proc) {
			/*
			 * Write out the index into the procedure array of
			 * this widgets procedure, not the value itself.
			 */
			for (s_index = 1; s_index <= s_len; s_index++) {
			    cp = (char *) get_alist_pos (proc_list, s_index);
			    if (strcmp (a->val.s, cp) == 0) {
				(void) fwrite (&s_index, sizeof (s_index), 1, fd);
				break;
				}
			    }
			if (s_index > s_len) {
			    fputs ("Consistency error in gen_technology()!\n", stderr);
			    fputs ("s_index > s_len\n", stderr);
			    exit (1);
			    }
			}
		    else
			(void) fwrite (&a->val.i, sizeof (a->val.i), 1, fd);
		    break;
		case idd_real:
		    (void) fwrite (&a->val.r, sizeof (a->val.r), 1, fd);
		    break;
		case idd_id:
		    /* Do nothing */
		    break;
		case idd_no_data_type:
		default:
		    fprintf(stderr, "?Unknown idd_type in gen_technology()\n");
		    break;
		}
	    }

	/*
	 * Write out the number of methods
	 */
	m_len = alist_length (w->methods);
	(void) fwrite (&m_len, sizeof (m_len), 1, fd);
	/*
	 * Now, for each method...
	 */
	for (m_index = 1; m_index <= m_len; m_index++) {
	    m = (METH *) get_alist_pos (w->methods, m_index);
	    /*
	     * Write out the method's name, description, and parameter count
	     */
	    s = (STRING *) get_from_hashtable (string_table, m->name);
	    (void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
	    if (m->description != NULL) {
		s = (STRING *)get_from_hashtable(string_table, m->description);
		(void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
		}
	    else
		(void) fwrite (&zero, sizeof (zero), 1, fd);
	    if (m->userdef != NULL) {
		s = (STRING *) get_from_hashtable(string_table, m->userdef);
		(void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
		}
	    else
		(void) fwrite (&zero, sizeof (zero), 1, fd);
	    p_len = alist_length (m->parms);
	    (void) fwrite (&p_len, sizeof (p_len), 1, fd);
	    /*
	     * Now write out the indices of all the parameters
	     */
	    for (p_index = 1; p_index <= p_len; p_index++) {
		p = (PARM *) get_alist_pos (m->parms, p_index);
		s = (STRING *) get_from_hashtable (string_table, p->name);
		(void) fwrite (&s->idx, sizeof (s->idx), 1, fd);
		}
	    }
	}

    /*
     * Now that we're all counted up, write out the header at the beginning
     */
    fseek (fd, 0, 0);
    (void) fwrite ((char *)&tx_header, sizeof (TX), 1, fd);
}

void genfiles (infile, make_methskel, verbose)
char *infile;
short make_methskel, verbose;
{
    char *loc, *rootfile;
    char outfile[256];	/* Bigger than we'll ever need */
    HASH method_table, string_table;
    ALIST class_list;
    ALIST proc_list;
    ALIST conv_list;
    
    /*
     * Eliminate any extension to the file, and make sure that we don't
     * trash the input file as an output file.
     */
    if ((loc=rindex (infile, '.')) != NULL) {
	if (strcmp (loc, ".tx") == 0 ||
	    strcmp (loc, ".sdd") == 0) {
	    fprintf (stderr, "ERROR: Input file will be overwritten by output file\n");
	    exit (1);
	    }
	*loc = '\0';
	}
    /*
     * Eliminate any leading directory components (so that the output files
     * always appear in the current directory)
     */
    if ((rootfile=rindex(infile, '/')) == NULL)
	rootfile = infile;
    else
	rootfile++;

    if (verbose) {
	fprintf (stdout, "Generating: ");
	fflush (stdout);
	}

    (void) strcat( strcpy (outfile, rootfile), ".sdd");
    if ((fd=fopen(outfile, "w")) == NULL) {
	fprintf (stderr, "Cannot open output file '%s'.\n", outfile);
	exit (1);
	}
    else {
	if (verbose) {
	    fprintf (stdout, "%s ", outfile);
	    fflush (stdout);
	    }
	gen_saddle (rootfile);
	fclose (fd);
	}

    /*
     * Build the method table (used by routines following this point)
     */
    method_table = check_methods();
    /*
     * Build the class list (and check that the new class list matches the
     * existing class table, if any).  Do the same with the procedure list.
     */
    class_list = check_classes (rootfile);
    proc_list = check_procs (rootfile);
    conv_list = check_convs (rootfile);
    /*
     * Build the string table.
     */
    ill_file_name = make_node (strlen (rootfile) + strlen (".ill") + 1);
    (void) strcat (strcpy (ill_file_name, rootfile), ".ill");
    if (mail_box_name_override)
       {mailbox_name = make_node (strlen (mail_box_name_override) +
                                  strlen ("_BOX") + 1);
       strcat (strcpy (mailbox_name, mail_box_name_override), "_BOX");
       }
    else
       {mailbox_name = make_node (strlen (rootfile) + strlen ("_BOX") + 1);
       strcat (strcpy (mailbox_name, rootfile), "_BOX");
       }
    u22_convert_string_inplace (mailbox_name, UPPERCASE);
    string_table = build_strings();

    if (make_methskel) {
	(void) strcat( strcpy (outfile, rootfile), "_meth.c");
	if ((fd=fopen(outfile, "w")) == NULL) {
	    fprintf (stderr, "Cannot open output file '%s'.\n", outfile);
	    exit (1);
	    }
	else {
	    if (verbose) {
		fprintf (stdout, "%s ", outfile);
		fflush (stdout);
		}
	    gen_methods (rootfile, method_table);
	    fclose (fd);
	    }

	(void) strcat( strcpy (outfile, rootfile), "_bind.c");
	if ((fd=fopen(outfile, "w")) == NULL) {
	    fprintf (stderr, "Cannot open output file '%s'.\n", outfile);
	    exit (1);
	    }
	else {
	    if (verbose) {
		fprintf (stdout, "%s ", outfile);
		fflush (stdout);
		}
	    gen_bindings (rootfile, method_table, class_list, proc_list, 
			  conv_list);
	    fclose (fd);
	    }
	}

    (void) strcat( strcpy (outfile, rootfile), ".tx");
    if ((fd=fopen(outfile, "w")) == NULL) {
	fprintf (stderr, "Cannot open output file '%s'.\n", outfile);
	exit (1);
	}
    else {
	if (verbose) {
	    fprintf (stdout, "%s ", outfile);
	    fflush (stdout);
	    }
	gen_technology (rootfile, string_table, class_list, proc_list, 
			conv_list);
	fclose (fd);
	}

    if (verbose) {
	putc ('\n', stdout);
	fflush (stdout);
	}
}
