        static char rcsid [] = "$Header: /u/04c/mnt/integ/source/glue/src/RCS/ggc.c,v 1.3 90/10/03 18:06:18 rnk Exp $";

/*--------------------------------------------------------------------------
|
|  Name:	ggc.c
|
|  Description:
|	This is the class list reader module for GLUE.
|
|  Component of:
|	GLUE interface
|
|  Related files:
|	ggc.h
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:  
|	Daniel V. Klein
|	dvk@sei.cmu.edu
|
|  Change History:
$Log:	ggc.c,v $
 * Revision 1.3  90/10/03  18:06:18  rnk
 * added support for convenience routines
 * 
 * Revision 1.2  90/06/06  16:14:21  dvk
 * Added glue_get_proc_alist routine
 * 
 * Revision 1.1  90/05/18  17:12:26  dvk
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/
#include <stdio.h>
#include "tx_fmt.h"
#include "alistPack.h"
#include "gft.h"


/*==========================================================================*\
 * Routine:
 *	glue_get_class_alist
 *
 * Description:
 *	Given a technology name, opens the appropriate technology data file,
 * reads in the list of classes, and returns an ALIST of the classes.  This
 * routine ignores the remainder of the technology file.
 *	The name of the technology will typically be an unqualified file
 * name.  If an extension is given, it will be used - otherwise the file
 * extension is assumed to be ".tx".  If the filename is given as an
 * absolute path name, that file will be used, otherwise, the file will be
 * looked for in the directories specified in the environment variable
 * SERPENT_DATA_PATH
 */

ALIST glue_get_class_alist (name)
char *name;
{
    FILE *fd;
    TX tx;
    ALIST classes;
    char *strings;
    str_idx i;

    /*
     * Find the file from the search path of SERPENT_DATA_PATH
     */
    fd = glue_find_tech (name);
    if (fd == NULL)
	return (ALIST)NULL;
    /*
     * Read the header and make sure it's legit
     */
    if (fread (&tx, sizeof (TX), 1, fd) != 1) {
	fclose (fd);
	return (ALIST)NULL;
	}
    if (tx.magic != TX_MAGIC) {
	fclose (fd);
	return (ALIST)NULL;
	}
    /*
     * Start by creating an alist and the string space
     */
    classes = make_alist ();
    strings = (char *) make_node (tx.string_size);
    (void) fread (strings, sizeof (char), tx.string_size, fd);
    /*
     * Read in the classes, adding each one to the tail of the ALIST
     */
    while (tx.num_classes--) {
	(void) fread (&i, sizeof (i), 1, fd);
	(void) add_to_atail (classes, &strings[i]);
	}

    (void) fclose (fd);			/* Close the .tx file */
    return (classes);
}


/*==========================================================================*\
 * Routine:
 *	glue_get_proc_alist
 *
 * Description:
 *	Given a technology name, opens the appropriate technology data file,
 * reads in the list of procedures, and returns an ALIST of the procedures.
 * This routine ignores the remainder of the technology file.
 *	The name of the technology will typically be an unqualified file
 * name.  If an extension is given, it will be used - otherwise the file
 * extension is assumed to be ".tx".  If the filename is given as an
 * absolute path name, that file will be used, otherwise, the file will be
 * looked for in the directories specified in the environment variable
 * SERPENT_DATA_PATH
 */
ALIST glue_get_proc_alist (name)
char *name;
{
    FILE *fd;
    TX tx;
    ALIST procs;
    char *strings;
    str_idx i;

    /*
     * Find the file from the search path of SERPENT_DATA_PATH
     */
    fd = glue_find_tech (name);
    if (fd == NULL)
	return (ALIST)NULL;
    /*
     * Read the header and make sure it's legit
     */
    if (fread (&tx, sizeof (TX), 1, fd) != 1) {
	fclose (fd);
	return (ALIST)NULL;
	}
    if (tx.magic != TX_MAGIC) {
	fclose (fd);
	return (ALIST)NULL;
	}
    /*
     * Start by creating an alist and the string space
     */
    procs = make_alist ();
    strings = (char *) make_node (tx.string_size);
    (void) fread (strings, sizeof (char), tx.string_size, fd);
    /*
     * Skip over the class list
     */
    (void) fseek (fd, tx.num_classes*sizeof(i), 1);
    /*
     * Read in the procs, adding each one to the tail of the ALIST
     */
    while (tx.num_procs--) {
	(void) fread (&i, sizeof (i), 1, fd);
	(void) add_to_atail (procs, &strings[i]);
	}

    (void) fclose (fd);			/* Close the .tx file */
    return (procs);
}

/*==========================================================================*\
 * Routine:
 *	glue_get_conv_alist
 *
 * Description:
 *	Given a technology name, opens the appropriate technology data file,
 * reads in the list of convenience routines, and returns an ALIST of the 
 * convenience routines.  This routine ignores the remainder of the technology
 * file.
 *	The name of the technology will typically be an unqualified file
 * name.  If an extension is given, it will be used - otherwise the file
 * extension is assumed to be ".tx".  If the filename is given as an
 * absolute path name, that file will be used, otherwise, the file will be
 * looked for in the directories specified in the environment variable
 * SERPENT_DATA_PATH
 */
ALIST glue_get_conv_alist (name)
char *name;
{
    FILE *fd;
    TX tx;
    ALIST convs;
    char *strings;
    str_idx i;

    /*
     * Find the file from the search path of SERPENT_DATA_PATH
     */
    fd = glue_find_tech (name);
    if (fd == NULL)
	return (ALIST)NULL;
    /*
     * Read the header and make sure it's legit
     */
    if (fread (&tx, sizeof (TX), 1, fd) != 1) {
	fclose (fd);
	return (ALIST)NULL;
	}
    if (tx.magic != TX_MAGIC) {
	fclose (fd);
	return (ALIST)NULL;
	}
    /*
     * Start by creating an alist and the string space
     */
    convs = make_alist ();
    strings = (char *) make_node (tx.string_size);
    (void) fread (strings, sizeof (char), tx.string_size, fd);
    /*
     * Skip over the class list
     */
    (void) fseek (fd, tx.num_classes*sizeof(i), 1);
    /*
     * Skip over the proc list
     */
    (void) fseek (fd, tx.num_procs*sizeof(i), 1);
    /*
     * Read in the conv routines, adding each one to the tail of the ALIST
     */
    while (tx.num_convs--) {
	(void) fread (&i, sizeof (i), 1, fd);
	(void) add_to_atail (convs, &strings[i]);
	}

    (void) fclose (fd);			/* Close the .tx file */
    return (convs);
}
