        static char rcsid [] = "$Header: /usr/users/little/serpent/mips/glue/src/RCS/glue.c,v 1.10 1991/05/14 22:26:34 little Exp $";

/*--------------------------------------------------------------------------*\
|
|  Name:	glue.c
|
|  Description:
|	This is the main module for GLUE.
|
|  Component of:
|	GLUE processor
|
|  Related files:
|	many yacc, lex, and c files
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:  
|	Daniel V. Klein
|	dvk@sei.cmu.edu
|
|  Change History:
$Log: glue.c,v $
 * Revision 1.10  1991/05/14  22:26:34  little
 * add the -m parameter
 *
 * Revision 1.9  1991/03/26  08:58:49  ejh
 * put in a stderr where it was needed (fprintf)
 *
 * Revision 1.8  90/10/07  16:59:16  rnk
 * removed code which indicates suppressed "no default" warnings
 * 
 * Revision 1.7  90/08/15  16:09:00  rnk
 * no changes
 * 
 * Revision 1.6  90/07/19  15:09:22  dvk
 * Cosmetic changes - moved announcement print to after initial error checks
 * 
 * Revision 1.5  90/07/16  14:51:11  dvk
 * Rick requested that class and include file be optional - they now are
 * 
 * Revision 1.4  90/07/02  18:03:39  dvk
 * Added "-w" switch to turn off "missing default value" warning
 * messages from the parser.
 * 
 * Revision 1.3  90/06/21  12:03:49  dvk
 * Now allows you to say "glue foo" and mean "glue foo.gl".  Also
 * beefed up error checking a little.
 * 
 * Revision 1.2  90/05/22  14:39:11  dvk
 * Forgot that when doing an exec, you need to also specify
 * a value for argv[0].  I remedied that, and now cpp behaves
 * better and doesn't ignore it's first argument.
 * 
 * Revision 1.1  90/05/17  16:10:48  dvk
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/
#include "memoryPack.h"
#include <stdio.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include "gen.h"

#define GLUE_VERSION	"1.0"
#define INCL_SIZE	50	/* Number of -I arguments possible */

extern char *getenv();
extern char *strtok();
extern char *mktemp();

short no_warnings = 0;
short num_class_warn = 0;
short num_include_warn = 0;
char *mail_box_name_override = NULL;

main(argc, argv, envp)
int argc;
char **argv;
char **envp;
{
    char **ap, **cp;
    char *cpp_args[INCL_SIZE+3];	/* +3 for filenames */
    char *env, *str, *temp;
    char *file = NULL;
    int ac;
    short keeptemp = 0;
    short cpp_only = 0;
    short verbose = 0;
    short make_methskel = 0;
    struct stat statbuf;

    ap = argv;
    ac = argc;
    for (cp = cpp_args; cp < &cpp_args[INCL_SIZE]; cp++)
	*cp = 0;

    cp = cpp_args;
    *cp++ = "cpp";
    while (--ac) {
	++ap;
	if (ap[0][0] == '-') {
	    switch (ap[0][1]) {
		case 'C':
		case 'D':
		case 'I':
		case 'U':
		    *cp++ = *ap;
		    if (cp >= &cpp_args[INCL_SIZE]) {
			fprintf (stderr, "Arg list too long\n");
			exit(1);
			}
		    break;
		case 'E':
		    cpp_only = 1;
		    break;
		case 'v':
		    verbose = 1;
		    break;;
		case 'k':
		    keeptemp = 1;
		    break;
		case 'M':
		    make_methskel = 1;
		    break;
		case 'w':
		    no_warnings = 1;
		    break;
		case 'm':
				/* get past the -m */
		    mail_box_name_override = *ap + 2;
		    break;
		default:
		    fprintf (stderr, "Unknown flag %s\n", *ap);
		    goto usage;
		    break;
		}
	    }
	else {
	    file = *ap;
	    if (ac > 1) {
		fprintf (stderr, "Extra args after filename \"%s\"\n", file);
		goto usage;
		}
	    }
	}
    if (file == NULL) {
	fprintf (stderr, "Missing glue filename\n");
	goto usage;
	}
    else {
	if (stat (file, &statbuf) == -1) {
	    temp = strcat(strcpy (make_node (strlen (file)+1+3), file), ".gl");
	    if (stat (temp, &statbuf) == -1) {
		fprintf (stderr, "Cannot find source file \"%s\"\n", file);
		goto usage;
		}
	    else
		file = temp;
	    }
	}
    if ((env = getenv ("SERPENT_DATA_PATH")) != NULL) {
	(void) strtok (env, " ");
	*cp = make_node (strlen (env) + 3);	/* 2 for "-I", 1 for '\0' */
	(void) strcpy (*cp, "-I");
	(void) strcat (*cp++, env);
	if (cp >= &cpp_args[INCL_SIZE]) {
	    goto toomuch;
	    }
	while (str = strtok (NULL, " ")) {
	    *cp = make_node (strlen (str) + 3);
	    (void) strcpy (*cp, "-I");
	    (void) strcat (*cp++, str);
	    if (cp >= &cpp_args[INCL_SIZE]) {
toomuch:
		fprintf (stderr, "Too many includes in SERPENT_DATA_PATH\n");
		exit(1);
		}
	    }
	}

    fprintf(stdout, "\n\tYou're rubber, I'm GLUE (V%s)\n\n", GLUE_VERSION);

    *cp++ = file;
    if ( ! cpp_only)
	*cp++ = temp = mktemp("/tmp/glueXXXXXX");

    if (verbose) {
	fprintf (stdout, "/lib/cpp");
	for (cp = cpp_args+1; *cp; cp++)	/* Skip argv[0] */
	    fprintf (stdout, " %s", *cp);
	putc ('\n', stdout);
	}

    switch (fork()) {
	case -1:	/* Error */
	    fprintf (stderr, "?fork() for /lib/cpp failed\n");
	    exit(1);
	    break;
	case 0:		/* Child process */
	    if (execve ("/lib/cpp", cpp_args, envp)) {
		fprintf (stderr, "?execve of /lib/cpp failed\n");
		exit(1);
		}
	    break;
	default:	/* Parent process */
	    (void) wait((union wait *)0);
	    break;
	}
    if (cpp_only)
	exit(0);

    if ( freopen (temp, "r", stdin) == NULL ) {
	fprintf (stderr, "?Cannot reopen temporary file \"%s\"!\n", temp);
	exit(1);
	}
    if (keeptemp) {
	fprintf (stdout, "Keeping temporary file \"%s\"\n", temp);
	}
    else {
	if (unlink (temp) != 0) {
	    fprintf (stderr, "?Could not delete temp file \"%s\"!\n", temp);
	    exit(1);
	    }
	}

    if (verbose)
	fprintf (stdout, "Parsing %s\n", temp);
    if (yyparse() == 0) {
	if (no_warnings) {
	    if (num_class_warn > 0)
		fprintf (stderr, "Note: Supressed %d 'no class value' warning message%s\n",
				num_class_warn, num_class_warn > 1 ? "s" : "");
	    if (num_include_warn > 0)
		fprintf (stderr, "Note: Supressed %d 'no include file value' warning message%s\n",
				num_include_warn, num_include_warn > 1 ? "s" : "");
	    }
	genfiles (file, make_methskel, verbose);
	exit (0);
	}
    else
	exit (1);

usage:
    fprintf (stderr,
	"Usage: %s {<cpp args>} -E -M -mmailbox_name -k -v <file>\n", *argv);
    exit(1);
}

