        static char rcsid [] = "$Header: /u/04c/mnt/integ/source/glue/src/RCS/glue_inst.c,v 1.4 90/06/08 17:11:27 dvk Exp $";

/*--------------------------------------------------------------------------
|
|  Name:	glue_inst.c
|
|  Description:
|	This module creates and maintains instances of widgets for Glue
|
|  Component of:
|	GLUE interface
|
|  Related files:
|	glue_inst.h
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:  
|	Daniel V. Klein
|	dvk@sei.cmu.edu
|
|  Change History:
$Log:	glue_inst.c,v $
 * Revision 1.4  90/06/08  17:11:27  dvk
 * It is possible to do a glue_get_by_id before the hashtables are
 * created, and then the routine dumped.  Fixed it so glue_get_by_id
 * and glue_get_by_widget check for hashtable creation, and return
 * a NULL pointer if not.
 * 
 * Revision 1.3  90/06/04  12:28:41  dvk
 * Fixed glue_register_instance, which checked for setting of the
 * variable "initialized", but never set it.
 * 
 * Revision 1.2  90/05/30  14:43:10  dvk
 * Corrected glue_get_attr_from_instance to use the new ATTR->idx
 * field (instead of the incorrect use of WIDG->idx)
 * 
 * Revision 1.1  90/05/17  16:11:16  dvk
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/
#include <stdio.h>
#include "alistPack.h"
#include "hashPack.h"
#include "tx_fmt.h"
#include "glue_intf.h"
#include "glue_inst.h"
#include "iid.h"

/*==========================================================================*\
 * Various variables and routines used internally by this module
 */
#define HASH_SIZE	100

static short HashInit = 0;
    
HASH id_hash;
HASH wg_hash;

static int w_hash_fn (ident)
Widget ident;
{
    return (int)ident % HASH_SIZE;
}

static int w_match_fn (ident, i)
Widget ident;
INST *i;
{
    return (ident == i->widget);
}

static int i_hash_fn (ident)
iid_id_type ident;
{
    return (int)ident % HASH_SIZE;
}

static int i_match_fn (ident, i)
iid_id_type ident;
INST *i;
{
    return (ident == i->id);
}


/*==========================================================================*\
 * Routine:
 *	glue_create_instance
 *
 * Description:
 *	Given a technology and a widget, create an instance of that
 * widget.  Note that this does not create the necessary X image or
 * structure - what it does is create the Glue structure.  Both the
 * X Widget pointer and the Serpent ID pointer must be filled in with
 * a call to glue_register_instance()
 */
INST * glue_create_instance (widget)
WIDG * widget;
{
    INST *i;

    if (widget == (WIDG *)NULL)
	return (INST *)NULL;
    i = (INST *) make_node (sizeof (INST));
    if (i == (INST *)NULL) {
	fputs ("Out of memory in glue_create_instance\n", stderr);
	return (INST *)NULL;
	}
    i->widget = (Widget)NULL;
    i->id = (iid_id_type)NULL;
    i->child = make_alist();
    i->attributes = copy_alist (widget->attributes, AND_NODES);
    i->template = widget;
    return (i);
}

/*==========================================================================*\
 * Routine:
 *	glue_register_instance
 *
 * Description:
 *	Once an instance has been created, the caller must create the widget
 * in both the X and Serpent domains.  Once this has been done, the caller
 * then calls glue_register_instance to register the instance into the
 * recall tables used by glue_find_by_id and glue_find_by_widget.
 *	NOTE: This routine assumes that the values of instance->widget and
 * instance->id have been filled in prior to being called!
 */
boolean glue_register_instance (instance)
INST * instance;
{
    if (HashInit == 0) {
	id_hash = make_hashtable(HASH_SIZE, i_hash_fn, i_match_fn);
	wg_hash = make_hashtable(HASH_SIZE, w_hash_fn, w_match_fn);
	HashInit++;
	}
    if (instance == (INST *)NULL)
	return (false);
    if (instance->widget == (Widget)NULL || instance->id == (iid_id_type)NULL)
	return (false);
    if (add_unique_to_hashtable (id_hash, instance, instance->id) == NULL)
	return (false);
    if (add_unique_to_hashtable (wg_hash, instance, instance->widget) == NULL) {
	(void) delete_from_hashtable (id_hash, instance->id);
	return (false);
	}
    return (true);
}

/*==========================================================================*\
 * Routine:
 *	glue_get_by_id
 *
 * Description:
 *	Recovers an instance of a widget using the (unique) Serpent ID
 * as a recovery handle.
 */
INST * glue_get_by_id (id)
iid_id_type id;
{
    if (HashInit)
	return ((INST *) get_from_hashtable (id_hash, id));
    else
	return (INST *) NULL;
}

/*==========================================================================*\
 * Routine:
 *	glue_get_by_widget
 *
 * Description:
 *	Recovers an instance of a widget using the (unique) X Widget
 * address as a recovery handle.
 */
INST * glue_get_by_widget (widget)
Widget widget;
{
    if (HashInit)
	return ((INST *) get_from_hashtable (wg_hash, widget));
    else
	return (INST *) NULL;
}

/*==========================================================================*\
 * Routine:
 *	glue_get_attr_from_instance
 *
 * Description:
 *	Grab the attribute structure associated with a given (technology
 * or Serpent) name from an instance of a widget.
 */
ATTR * glue_get_attr_from_instance (instance, attr_name, t_or_s)
INST *instance;
char *attr_name;
NameType t_or_s;
{
    ATTR *a;

    if (instance == (INST *)NULL)
	return (ATTR *)NULL;
    a = (ATTR *) glue_get_attribute (instance->template, attr_name, t_or_s);
    if (a == (ATTR *)NULL)
	return (ATTR *)NULL;
    /*
     * ALISTs are indexed starting at 1, but C arrays and a->idx are
     * indexed starting at 0.  Increment by 1 to get the right one.
     */
    return ((ATTR *) get_alist_pos (instance->attributes, a->idx+1));
}

/*==========================================================================*\
 * Routine:
 *	free_attrs
 *
 * Description:
 *	Local routine used to free elements from the attribute list.
 */
static int free_strs (dummy, attr)
int dummy;
ATTR *attr;
{
    if (attr->s_type == idd_string && attr->val_type == VAL_CURRENT)
	free_node (attr->value.s);
}

/*==========================================================================*\
 * Routine:
 *	glue_destroy_instance
 *
 * Description:
 *	Destroy an instance, removing the retrieval tags from the various
 * has lists, and freeing up the space associated with the instance.
 * 	NOTE: data space taken up by the values of attributes is NOT freed
 * unless the nodes flag is set to AND_STRS.  This means that if an attribute
 * has a value of type string or type buffer, glue_destroy_instance will do
 * a free_node on the data pointer if AND_STRS is set.  If NOT_STRS is
 * passed in, only the instance structure and the attributes ALIST is freed.
 */
void glue_destroy_instance (instance, do_strs)
INST *instance;
StrDestroy do_strs;
{
    if (instance == (INST *)NULL)
	return;
    if (do_strs == AND_STRS)
	for_all_alist (instance->attributes, (NODE)NULL, free_strs);
    free_alist (instance->attributes, AND_NODES);
    delete_from_hashtable (id_hash, instance->id);
    delete_from_hashtable (wg_hash, instance->widget);
    free_node (instance);
}
