        static char rcsid [] = "$Header: /u/04c/mnt/integ/source/glue/src/RCS/glue_intf.c,v 1.23 91/03/22 13:46:11 rcs Exp $";

/*--------------------------------------------------------------------------
|
|  Name:	glue_intf.c
|
|  Description:
|	This is the technology file reader module for GLUE.
|
|  Component of:
|	GLUE interface
|
|  Related files:
|	glue_intf.h
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:  
|	Daniel V. Klein
|	dvk@sei.cmu.edu
|
|  Change History:
$Log:	glue_intf.c,v $
 * Revision 1.23  91/03/22  13:46:11  rcs
 * added routine to access priorities from slang
 * 
 * Revision 1.22  91/03/18  18:47:28  dvk
 * Added "priority" attribute component.
 * 
 * Revision 1.21  90/11/30  14:19:55  rcs
 * added some new routines
 * 
 * Revision 1.20  90/11/19  15:19:59  rcs
 * added some data access routines
 * 
 * Revision 1.19  90/10/12  08:36:23  little
 * initialize the methods and attributes got components
 * 
 * Revision 1.18  90/10/03  18:06:42  rnk
 * added support for convenience routines
 * 
 * Revision 1.17  90/09/07  15:54:47  rnk
 * changed glue_get_attr_default to return an undefined buffer if the
 * attribute is not found or has no value
 * 
 * Revision 1.16  90/09/01  19:52:30  ejh
 * took out include of strings.h
 * 
 * Revision 1.15  90/08/17  12:19:23  rcs
 * removed extraneous debug print
 * 
 * Revision 1.14  90/08/16  21:32:20  rcs
 * no changes
 * 
 * Revision 1.13  90/08/14  13:13:52  rnk
 * changed logic to ensure that check_routine is always set
 * 
 * Revision 1.12  90/08/13  19:24:34  rnk
 * added code to read in check_routines
 * 
 * Revision 1.11  90/08/10  17:32:34  rnk
 * changed glue_get_attr_default to return a buffer
 * 
 * Revision 1.10  90/08/09  18:02:57  rnk
 * added the routine glue_get_attr_default, which returns the default of
 * an attribute.
 * 
 * Revision 1.9  90/06/07  15:20:10  dvk
 * Fixed a small bug associated with procedure attributes
 * 
 * Revision 1.8  90/06/07  14:53:15  rnk
 * No changes
 * 
 * Revision 1.7  90/06/07  10:39:16  rnk
 * changed type of glueSpecialProcList to caddr_t
 * 
 * Revision 1.6  90/06/06  16:07:55  dvk
 * Added checks for routine-valued procedures, plus interface to
 * glueSpecialProcList and glueSpecialProcCount.
 * 
 * Revision 1.5  90/05/30  15:15:26  dvk
 * Changed include of <string.h> to <strings.h> (stupid Sun system...)
 * 
 * Revision 1.4  90/05/30  14:42:02  dvk
 * Added an idx field to the ATTRs to correct a bug in glue_inst.c
 * 
 * Revision 1.3  90/05/21  12:13:12  dvk
 * Added ill_file and mailbox to TECH structure
 * 
 * Revision 1.2  90/05/18  17:19:02  dvk
 * Added fseek around new class list
 * 
 * Revision 1.1  90/05/17  16:11:34  dvk
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define gif_TOP

#include <stdio.h>
#include <string.h>
#include "alistPack.h"
#include "hashPack.h"
#include "isc.h"
#include "idd.h"
#include "tx_fmt.h"
#include "gt_type.h"
#include "glue_intf.h"
#include "gft.h"
#include "uc2.h"

#define W_HASH_SIZE	29
#define A_HASH_SIZE	31

static int w_hash_fn (ident)
char *ident;
{
    return (ident[0]*strlen(ident) + ident[1]) % W_HASH_SIZE;
}

static int w_match_fn (ident, w)
char *ident;
WIDG *w;
{
    return (strcmp (ident, w->name) == 0);
}

static int a_hash_fn (ident)
char *ident;
{
    return (ident[0]*strlen(ident) + ident[1]) % A_HASH_SIZE;
}

static int at_match_fn (ident, a)
char *ident;
ATTR *a;
{
    return (strcmp (ident, a->t_name) == 0);
}

static int as_match_fn (ident, a)
char *ident;
ATTR *a;
{
    return (strcmp (ident, a->s_name) == 0);
}



/*==========================================================================
 * The HOOK structure is used to quickly access all of the goodies in the
 * TECH structure.  The philosophy is that the glue_get_technology() routine
 * should do a little extra work at startup so that the glue_get_*() routines
 * can work quickly and efficiently by reading precompiled tables.
 */
typedef struct {
    char *strings;	/* All the strings associated with this technology */
    HASH w_table;	/* Hash table of widgets */
    ALIST w_names;	/* ALIST of widget names */
    HASH *at_tables;	/* Array of HASH tables of technology attributes */
    HASH *as_tables;	/* Array of HASH tables of serpent attributes */
    ALIST *at_names;	/* Array of ALISTs of all attribute tech names */
    ALIST *as_names;	/* Array of ALISTs of all attribute serp names */
    ALIST *dt_names;	/* Array of ALISTs of defaulted attribute tech names */
    ALIST *ds_names;	/* Array of ALISTs of defaulted attribute serp names */
    ALIST *m_names;	/* Array of ALISTs of method names */
    } HOOK;


/*==========================================================================
 * Routine:
 *	glue_get_technology
 *
 * Description:
 *	Given a technology name, opens the appropriate technology data file,
 * initializes a TECH structure, and returns a pointer to the structure
 * (similar to the way fopen returns a FILE *).
 *	The name of the technology will typically be an unqualified file
 * name.  If an extension is given, it will be used - otherwise the file
 * extension is assumed to be ".tx".  If the filename is given as an
 * absolute path name, that file will be used, otherwise, the file will be
 * looked for in the directories specified in the environment variable
 * SERPENT_DATA_PATH
 */
TECH *glue_get_technology (name)
char *name ;
{
    FILE *fd;
    TX tx;
    TECH *tech;
    WIDG *w;
    ATTR *a;
    METH *m;
    HOOK *h;
    str_idx i;
    char *cp;
    count wdx, mdx, adx, pdx, cdx;
    count mmax, amax, pmax;
    short is_proc;

    /*
     * Find the file from the search path of SERPENT_DATA_PATH
     */
    fd = glue_find_tech (name);
    if (fd == NULL)
	return (TECH *)NULL;
    /*
     * Read the header and make sure it's legit
     */
    if (fread (&tx, sizeof (TX), 1, fd) != 1) {
	fclose (fd);
	return (TECH *)NULL;
	}
    if (tx.magic != TX_MAGIC) {
	fclose (fd);
	return (TECH *)NULL;
	}
    /*
     * Start by allocating a TECH (and the associated HOOK) structure and
     * the string space
     */
    tech = (TECH *) make_node (sizeof (TECH));
    tech->widgets = make_alist ();
    h = (HOOK *) make_node (sizeof (HOOK));
    h->w_table = make_hashtable (W_HASH_SIZE, w_hash_fn, w_match_fn);
    h->w_names = make_alist ();
    h->at_tables = (HASH *) make_node (tx.num_widgets * sizeof (HASH));
    h->as_tables = (HASH *) make_node (tx.num_widgets * sizeof (HASH));
    h->at_names = (ALIST *) make_node (tx.num_widgets * sizeof (ALIST));
    h->as_names = (ALIST *) make_node (tx.num_widgets * sizeof (ALIST));
    h->dt_names = (ALIST *) make_node (tx.num_widgets * sizeof (ALIST));
    h->ds_names = (ALIST *) make_node (tx.num_widgets * sizeof (ALIST));
    h->m_names = (ALIST *) make_node (tx.num_widgets * sizeof (ALIST));
    for (wdx = 0; wdx < tx.num_widgets; wdx++) {
	h->at_tables[wdx] = make_hashtable(A_HASH_SIZE,a_hash_fn, at_match_fn);
	h->as_tables[wdx] = make_hashtable(A_HASH_SIZE,a_hash_fn, as_match_fn);
	h->at_names[wdx] = make_alist ();
	h->as_names[wdx] = make_alist ();
	h->dt_names[wdx] = make_alist ();
	h->ds_names[wdx] = make_alist ();
	h->m_names[wdx] = make_alist ();
	}
    h->strings = (char *) make_node (tx.string_size);
    (void) fread (h->strings, sizeof (char), tx.string_size, fd);
    /*
     * Skip over the class, proc and convenience rtn lists
     */
    (void) fseek (fd, (tx.num_classes+tx.num_procs+tx.num_convs)*sizeof(i), 1);
    /*
     * Get the description of the technology, the mailbox name and ILL file
     */
    (void) fread (&i, sizeof (i), 1, fd);
    if (i != 0)
	tech->description = &h->strings[i];
    else
	tech->description = NULL;
    (void) fread (&i, sizeof (i), 1, fd);
    if (i != 0)
	tech->mailbox = &h->strings[i];
    else
	tech->mailbox = NULL;
    (void) fread (&i, sizeof (i), 1, fd);
    if (i != 0)
	tech->ill_file = &h->strings[i];
    else
	tech->ill_file = NULL;
    /*
     * Read the GT_LAST_V? "version" number
     */
    (void) fread (&tech->gt_last, sizeof (tech->gt_last), 1, fd);
    /*
     * Iterate through the list of widgets, grabbing the attributes and
     * methods for each one.
     */
    for (wdx = 0; wdx < tx.num_widgets; wdx++) {
	w = (WIDG *) make_node (sizeof (WIDG));
	w->technology = tech;
	w->attributes = make_alist();
	w->methods = make_alist();
	w->idx = wdx;
        w->attributes_got = 0;
        w->methods_got = 0;

	/*
	 * Get the name, description, userdef, and "is_shell" of the widget
	 */
	(void) fread (&i, sizeof (i), 1, fd);
	w->name = &h->strings[i];

	(void) fread (&i, sizeof (i), 1, fd);
	if (i != 0)
	    w->description = &h->strings[i];
	else
	    w->description = NULL;

	(void) fread (&i, sizeof (i), 1, fd);
	if (i != 0)
	    w->userdef = &h->strings[i];
	else
	    w->userdef = NULL;

	(void) fread (&w->widget_type, sizeof (w->widget_type), 1, fd);

	/* Get the check_routine for the widget */
	(void) fread (&cdx, sizeof (cdx), 1, fd);
	if (cdx != 0 && glueSpecialProcCount != 0)
	  w->check_rtn = glueSpecialProcList[cdx-1];
	else
	  w->check_rtn = ((PFV)NULL);

	/* Get the convenience_routine for the widget */
	(void) fread (&cdx, sizeof (cdx), 1, fd);
	if (cdx != 0 && glueConvenienceRtnCount != 0)
	  w->conv_rtn = glueConvenienceRtnList[cdx-1];
	else
	  w->conv_rtn = ((PFW)NULL);

	/*
	 * Get the class of the widget
	 */
	(void) fread (&cdx, sizeof (cdx), 1, fd);
	if (glueClassCount == 0 || cdx == 1)
	    w->class = (WidgetClass)NULL;
	else
	    w->class = *glueClassList[cdx-1];

	/*
	 * Iterate through the list of attributes, filling them in
	 */
	(void) fread (&amax, sizeof (amax), 1, fd);
	for (adx = 0; adx < amax; adx++) {
	    a = (ATTR *) make_node (sizeof (ATTR));
	    a->idx = adx;
	    (void) fread (&i, sizeof (i), 1, fd);
	    a->t_name = &h->strings[i];
	    (void) fread (&i, sizeof (i), 1, fd);
	    a->s_name = &h->strings[i];
	    (void) fread (&i, sizeof (i), 1, fd);
	    if (i != 0)
		a->description = &h->strings[i];
	    else
		a->description = NULL;
	    (void) fread (&i, sizeof (i), 1, fd);
	    if (i != 0)
		a->userdef = &h->strings[i];
	    else
		a->userdef = NULL;
	    (void) fread (&a->x_type, sizeof (a->x_type), 1, fd);
	    if (a->x_type == GT_ILLEGAL) {
		fprintf (stderr, "glue_get_technology: Attribute '%s' has no X type\n", a->t_name);
		exit (1);
		}
	    (void) fread (&a->s_type, sizeof (a->s_type), 1, fd);
	    (void) fread (&a->access, sizeof (a->access), 1, fd);
	    (void) fread (&a->priority, sizeof (a->priority), 1, fd);
	    (void) fread (&is_proc, sizeof (is_proc), 1, fd);
	    (void) fread (&a->val_type, sizeof (a->val_type), 1, fd);
	    if (a->val_type == VAL_TECH_DEFAULT || 
		a->val_type == VAL_SERP_DEFAULT) {
		switch (a->s_type) {
		    case idd_buffer:
		    case idd_string:
			(void) fread (&i, sizeof (i), 1, fd);
			a->value.s = &h->strings[i];
			break;
		    case idd_id:
		    case idd_no_data_type:
			/* Do nothing */
			break;
		    case idd_boolean:
		    case idd_integer:
			if (is_proc) {
			  (void)fread(&cdx, sizeof(cdx), 1, fd);
			  if (glueSpecialProcCount != 0)
			    if (cdx != 0)
			      a->value.c = (caddr_t)glueSpecialProcList[cdx-1];
			    else
			      a->value.c = (caddr_t) ((PFV)NULL);
			}
			else
			  (void)fread(&a->value.i, sizeof(a->value.i), 1,fd);
			break;
		    case idd_real:
			(void) fread (&a->value.r, sizeof (a->value.r), 1, fd);
			break;
		    default:
			fprintf (stderr, "glue_get_technology: Bad default data type in widget '%s', attribute '%s'\n", w->name, a->t_name);
			exit (1);
		    }
		}
	    (void) add_to_hashtable (h->at_tables[wdx], a, a->t_name);
	    (void) add_to_hashtable (h->as_tables[wdx], a, a->s_name);
	    if (a->val_type == VAL_SERP_DEFAULT) {
		(void) add_to_atail (h->dt_names[wdx], a->t_name);
		(void) add_to_atail (h->ds_names[wdx], a->s_name);
		}
	    (void) add_to_atail (h->at_names[wdx], a->t_name);
	    (void) add_to_atail (h->as_names[wdx], a->s_name);
	    (void) add_to_atail (w->attributes, a);
	    }
	/*
	 * Iterate through the list of methods, filling them in
	 */
	(void) fread (&mmax, sizeof (mmax), 1, fd);
	for (mdx = 0; mdx < mmax; mdx++) {
	    m = (METH *) make_node (sizeof (METH));
	    (void) fread (&i, sizeof (i), 1, fd);
	    m->s_name = &h->strings[i];
	    (void) fread (&i, sizeof (i), 1, fd);
	    if (i != 0)
		m->description = &h->strings[i];
	    else
		m->description = NULL;
	    (void) fread (&i, sizeof (i), 1, fd);
	    if (i != 0)
		m->userdef = &h->strings[i];
	    else
		m->userdef = NULL;
	    (void) fread (&pmax, sizeof (pmax), 1, fd);
	    m->parms = make_alist ();
	    for (pdx = 0; pdx < pmax; pdx++) {
		(void) fread (&i, sizeof (i), 1, fd);
		a = (ATTR *) get_from_hashtable (h->as_tables[wdx], &h->strings[i]);
		if (a == (ATTR *) NULL) {
		    fprintf (stderr, "glue_get_technology: Method '%s' references unknown attribute '%s'\n", m->s_name, &h->strings[i]);
		    exit (1);
		    }
		(void) add_to_atail (m->parms, a);
		}
	    (void) add_to_atail (h->m_names[wdx], m->s_name);
	    (void) add_to_atail (w->methods, m);
	    }
	/*
	 * Finally, stick the widget into the list of widgets for this tech
	 */
	(void) add_to_hashtable (h->w_table, w, w->name);
	(void) add_to_atail (h->w_names, w->name);
	(void) add_to_atail (tech->widgets, w);
	}
    (void) fclose (fd);			/* Close the .tx file */
    tech->hook = (struct hook *)h;	/* Fake the cast to satisfy cc */
    return (tech);
}


/*==========================================================================
 * Routine: 
 *	glue_alist_widgets
 *
 * Description:
 *	Given a technology, this routine returns a list of all of the
 * names of widgets associated with it.  The widgets themselves are
 * stored using the WIDG structure (described above), and can be gotten
 * using other routines in this package.
 */

ALIST glue_alist_widgets (technology)
TECH *technology;
{
    return ((HOOK *)technology->hook)->w_names;
}


/*==========================================================================
 * Routine: 
 *	glue_get_mailbox
 *
 * Description:
 *      The glue_get_mailbox returns the mailbox associated with the
 *      specified technology.  
 * ????  is it ok that this routine doesn't allocate memory?
 */

string glue_get_mailbox(technology)
TECH *technology;
{
  return technology->mailbox;
}

/*==========================================================================
 * Routine: 
 *	glue_get_widget
 *
 * Description:
 *	Given a technology and a widget name, return a pointer to the WIDG
 * structure associaated with it.  The caller should use this routine to
 * access the class and description components, and should use other
 * routines to inspect the contents of the METH and ATTR arrays.
 */
WIDG *glue_get_widget (technology, widget_name)
TECH *technology;
char *widget_name;
{
    return (WIDG *) get_from_hashtable (
				    ((HOOK *)technology->hook)->w_table,
				    widget_name
				    );
}


/*==========================================================================
 * Routine: 
 *	glue_get_attribute_alist
 *
 * Description:
 *	This routine takes a technology and a widget name and returns
 * the attribute alist.
 */
ALIST glue_get_attribute_alist(technology, widget_name)
TECH *technology;
char *widget_name;
{
  WIDG *w;

  w = (WIDG *)get_from_hashtable(
    ((HOOK *)technology->hook)->w_table, widget_name
  );
    
  return w->attributes;
}

/*==========================================================================
 * Routine: 
 *	glue_alist_all_attributes
 *
 * Description:
 *	Given a technology and a widget name, this routine returns a list
 * of all of the names of attributes associated with the widget.  The
 * attributes themselves are stored using the ATTR structure (described
 * above), and can be gotten using other routines in this package.
 */
ALIST glue_alist_all_attributes (technology, widget_name, t_or_s)
TECH *technology;
char *widget_name;
NameType t_or_s;
{
    WIDG *w;

    w = (WIDG *) get_from_hashtable (
				((HOOK *)technology->hook)->w_table,
				widget_name
				);
    if (t_or_s == TECH_NAMES)
	return ((HOOK *)(technology->hook))->at_names[w->idx];
    else
	return ((HOOK *)(technology->hook))->as_names[w->idx];
}


/*==========================================================================
 * Routine: glue_alist_defaulted_attributes
 *
 * Description:
 *	Given a technology and a widget name, this routine returns a list
 * of all of the names of attributes associated with the widget that have a
 * Serpent specified default value.
 */
ALIST glue_alist_defaulted_attributes (technology, widget_name, t_or_s)
TECH *technology;
char *widget_name;
NameType t_or_s;
{
    WIDG *w;

    w = (WIDG *) get_from_hashtable (
				((HOOK *)technology->hook)->w_table,
				widget_name
				);
    if (t_or_s == TECH_NAMES)
	return ((HOOK *)(technology->hook))->dt_names[w->idx];
    else
	return ((HOOK *)(technology->hook))->ds_names[w->idx];
}


/*==========================================================================
 * Routine: 
 *	glue_get_attribute
 *
 * Description:
 *	Given a widget and an attribute name, this routine returns a pointer
 * to the ATTR structure that corresponds to to the named attribute in the
 * specified widget.
 */
ATTR * glue_get_attribute (widget, attr_name, t_or_s)
WIDG *widget;
char *attr_name;
NameType t_or_s;
{
    if (t_or_s == TECH_NAMES)
	return (ATTR *) get_from_hashtable (
		    ((HOOK *)widget->technology->hook)->at_tables[widget->idx],
		    attr_name
		    );
    else
	return (ATTR *) get_from_hashtable (
		    ((HOOK *)widget->technology->hook)->as_tables[widget->idx],
		    attr_name
		    );
}


/*==========================================================================
 * Routine: 
 *	glue_get_widget_type
 *
 * Description:
 *	This routine takes a pointer to an widget structure and 
 * returns the type of the widget.
 */
WidgetType glue_get_widget_type(widget)
WIDG *widget;
{
  return widget->widget_type;
}

/*==========================================================================
 * Routine:  glue_get_userdef 
 *
 * Description:
 *	This routine takes a pointer to an widget structure and 
 * returns a pointer to the user def string.
 */
string glue_get_userdef(widget)
WIDG *widget;
{
  return widget->userdef;
}

/*==========================================================================
 * Routine: 
 *	glue_get_attribute_name
 *
 * Description:
 *	This routine takes a pointer to an attribute structure and 
 * returns the name of the attribute.
 */
string glue_get_attribute_name(attribute)
ATTR *attribute;
{
  return attribute->s_name;
}

/*==========================================================================
 * Routine:  glue_get_attribute_xtype
 *
 * Description:
 *	This routine takes a pointer to an attribute structure and 
 * returns the xtype of the attribute.
 */
int glue_get_attribute_xtype(attribute)
ATTR *attribute;
{
  return attribute->x_type;
}

/*==========================================================================
 * Routine:  glue_get_attribute_priority
 *
 * Description:
 *	This routine takes a pointer to an attribute structure and 
 * returns the priority of the attribute.
 */
int glue_get_attribute_priority(attribute)
ATTR *attribute;
{
  return attribute->priority;
}

/*==========================================================================
 * Routine: 
 *	glue_get_attr_default
 *
 * Description:
 *	Given a widget and an attribute name, this routine returns a string
 * containing the default for that attribute.
 */
idd_buffer_type *glue_get_attr_default (widget, attr_name, t_or_s)
WIDG *widget;
char *attr_name;
NameType t_or_s;
{
    ATTR *temp_attr ;
    idd_buffer_type *buffer ;

    if (t_or_s == TECH_NAMES) {
      temp_attr = (ATTR *) get_from_hashtable 
	(((HOOK *)widget->technology->hook)->at_tables[widget->idx],
	 attr_name);
    }
    else {
      temp_attr = (ATTR *) get_from_hashtable
	(((HOOK *)widget->technology->hook)->as_tables[widget->idx],
	 attr_name);
    }
/* 
**  If the attribute is not listed or has no value, return UNDEFINED 
*/
    if ((temp_attr == NULL) || (temp_attr->val_type == VAL_NONE)) {
	buffer = (idd_buffer_type *)make_node (sizeof (idd_buffer_type));
	buffer-> length = UNDEFINED_BUFFER_LENGTH;
	return(buffer) ;
    }
/* 
**  If the attribute has a default, return it as a buffer 
*/
    if (temp_attr->val_type == VAL_SERP_DEFAULT || 
	temp_attr->val_type == VAL_TECH_DEFAULT) 
      {
	if (temp_attr->s_type == idd_buffer) {
	    buffer = (idd_buffer_type *)
 	    uc2_convert(idd_buffer, idd_string, temp_attr->value.c);
	}
	else {
	    buffer = (idd_buffer_type *)
	      uc2_convert(idd_buffer, temp_attr->s_type, temp_attr->value.c);
	}
      }

    return(buffer);

}


/*==========================================================================
 * Routine: glue_get_attr_description
 *
 * Description:
 *	Given a widget and an attribute name, this routine returns a string
 * containing the description of that attribute.
 */

idd_buffer_type *glue_get_attr_description(widget, attr_name, t_or_s)
WIDG *widget;
string attr_name;
NameType t_or_s;
{
    ATTR *temp_attr ;
    idd_buffer_type *buffer ;

/*
**  Determine if attribute name is a technology name or serpent name.
*/
    if (t_or_s == TECH_NAMES) {
      temp_attr = (ATTR *) get_from_hashtable 
	(((HOOK *)widget->technology->hook)->at_tables[widget->idx],
	 attr_name);
    }
    else {
      temp_attr = (ATTR *) get_from_hashtable
	(((HOOK *)widget->technology->hook)->as_tables[widget->idx],
	 attr_name);
    }
/*
** If the attribute is not listed or has no value, return UNDEFINED 
*/
    if ((temp_attr == NULL) || (temp_attr->description == NULL)) {
	buffer = (idd_buffer_type *)make_node(sizeof(idd_buffer_type));
	buffer->length = UNDEFINED_BUFFER_LENGTH;
	return(buffer) ;
    }
/* 
**  Convert the description to a buffer type.
*/
    buffer = (idd_buffer_type *)uc2_convert(
      idd_buffer, idd_string, temp_attr->description
    );

    return(buffer);

}

/*==========================================================================
 * Routine: glue_get_attr_access
 *
 * Description:
 *	Given a widget and an attribute name, this routine returns the
 * access type.
 */

int glue_get_attr_access(widget, attr_name, t_or_s)
WIDG *widget;
string attr_name;
NameType t_or_s;
{
    ATTR *temp_attr ;
    int *buffer ;

/*
**  Determine if attribute name is a technology name or serpent name.
*/
    if (t_or_s == TECH_NAMES) {
      temp_attr = (ATTR *)get_from_hashtable 
	(((HOOK *)widget->technology->hook)->at_tables[widget->idx],attr_name
      );
    }
    else {
      temp_attr = (ATTR *) get_from_hashtable
	(((HOOK *)widget->technology->hook)->as_tables[widget->idx],attr_name
      );
    }
/*
** If the attribute is not listed or has no value, return UNDEFINED 
*/
    if (temp_attr == NULL) {
	return(-1) ;
    }

    return(temp_attr->access);
}

/*==========================================================================
 * Routine: 
 *	glue_alist_methods
 *
 * Description:
 *	Given a technology and a widget name, this routine returns a list
 * of all of the names of methods associated with the widget.  The methods
 * themselves are stored using the METH structure (described above), and
 * can be gotten using other routines in this package.
 */
ALIST glue_alist_methods (technology, widget_name)
TECH *technology;
char *widget_name;
{
    WIDG *w;

    w = (WIDG *) get_from_hashtable (
				((HOOK *)technology->hook)->w_table,
				widget_name
				);
    return ((HOOK *)(technology->hook))->m_names[w->idx];
}


/*==========================================================================
 * Routine: 
 *	glue_get_method
 *
 * Description:
 *	Given a widget and a method name, this routine returns a pointer
 * to the METH structure that corresponds to to the named method in the
 * specified widget.
 */
METH * glue_get_method (widget, meth_name)
WIDG *widget;
char *meth_name;
{
    register int idx, max;
    register METH *m;

    /*
     * We expect the number of methods per widget to be small (i.e., less
     * than 4), so a linear search is faster than a hash table lookup.
     */
    max = alist_length (widget->methods);
    for (idx = 1; idx <= max; idx++) {
	m = (METH *) get_alist_pos (widget->methods, idx);
	if (strcmp (m->s_name, meth_name) == 0)
	    return (m);
	}
    return (METH *) NULL;
}
