--+
--|
--| Package spec:	C_SERP
--|
--| Defines the interface between Ada and the C Serpent interface
--|
--+
with system;

with ca_conv;
  use ca_conv;

with s_types;
  use s_types;

package c_serp is

  procedure isd_init (
    mailbox : in c_string;
    ill_file : in c_string
  );

  function isd_start_transaction return c_handle;

  function isd_add_shared_data (
    transaction : in c_handle;
    element_name, component_name : in c_string;
    data : in system.address
  ) return int;


  procedure isd_put_shared_data(
    transaction    	: in c_handle;
    id             	: in int; 
    element_name   	: in c_string;
    component_name 	: in c_string; 
    data           	: in system.address
  );

  procedure isd_remove_shared_data(
    transaction  : in c_handle; 
    element_name : in c_string; 
    id           : in int
  );

  procedure isd_rollback_transaction(
    transaction : in c_handle
  );

  procedure isd_commit_transaction(
    transaction : in c_handle 
  );

  function isd_get_transaction return c_handle;

  function isd_get_transaction_no_wait return c_handle;

  function isd_get_first_changed_element(
    transaction : in c_handle
  ) return int;

  function isd_get_next_changed_element(
    transaction : in c_handle
  ) return int;

  function isd_get_element_name(
    transaction : in c_handle;
    id : in int 
  ) return c_string;

  function isd_get_change_type(
    transaction : in c_handle;
    id : int  
  ) return change_type;

  function isd_create_changed_component_list(
    transaction : in c_handle;
    id : in int 
  ) return c_handle;

  function isd_get_next_component_name(
    component_list : in LIST;
    component_node : in NODE 
  ) return NODE;

  procedure isd_destroy_changed_component_list(
    changed_component_list : in c_handle
  );

  procedure isd_incorporate_changes(
    transaction : in c_handle;
    id   : in int;
    data : in system.address     
  );


  procedure isd_purge_transaction(
    transaction : in c_handle
  );

  procedure ipb_start_recording(
    transaction : in c_handle; 
    file_name   : in c_string 
  );

  procedure ipb_stop_recording;

 
--  this not implemented yet ?????
--  procedure ipb_playback(
--    file_name  : in c_string 
--  );
                                                                
  procedure isd_cleanup;

  function isd_get_shared_data_type (
    element_name    : in c_string;
    component_name  : in c_string
  ) return shared_data_types;

  function isd_get_shared_data (
    transaction : in c_handle;
    id		: in int;
    component_name : in c_string
  ) return system.address;
    
  function isd_get_length (
    element_name    : in c_string
  ) return int;

  procedure isd_get_component_lop (
    element_name	: in c_string;
    component_name	: in c_string;
    length 		: in int_ptr;
    offset 		: in int_ptr;
    position 		: in int_ptr
  );

  function isc_fget_status return status_codes;

  procedure isc_print_status(
    error_msg : in c_string
  );

private

--
--  Putting information into the interface.
--
  pragma INTERFACE (C, isd_init);
  pragma INTERFACE (C, isd_start_transaction);
  pragma INTERFACE (C, isd_add_shared_data);
  pragma INTERFACE (C, isd_put_shared_data);
  pragma INTERFACE (C, isd_remove_shared_data);
  pragma INTERFACE (C, isd_rollback_transaction);
  pragma INTERFACE (C, isd_commit_transaction);
--
--  Getting information out of the interface.
--
  pragma INTERFACE (C, isd_get_transaction);
  pragma INTERFACE (C, isd_get_transaction_no_wait);
  pragma INTERFACE (C, isd_get_first_changed_element);
  pragma INTERFACE (C, isd_get_next_changed_element);
  pragma INTERFACE (C, isd_get_element_name);
  pragma INTERFACE (C, isd_get_change_type);
  pragma INTERFACE (C, isd_create_changed_component_list);
  pragma INTERFACE (C, isd_destroy_changed_component_list);
  pragma INTERFACE (C, isd_get_next_component_name);
  pragma INTERFACE (C, isd_incorporate_changes);
  pragma INTERFACE (C, isd_get_shared_data);
  pragma INTERFACE (C, isd_purge_transaction);
  pragma INTERFACE (C, ipb_start_recording);
  pragma INTERFACE (C, ipb_stop_recording);
 
-- NOT IMPLEMENTED YET ???????
--  pragma INTERFACE (C, ipb_playback);
 

  pragma INTERFACE (C, isd_cleanup);
--
--  Non-transaction specific queries.
--
  pragma INTERFACE (C, isd_get_shared_data_type);
  pragma INTERFACE (C, isd_get_length);
  pragma INTERFACE (C, isd_get_component_lop);
--
--  Status routines.
--
  pragma INTERFACE (C, isc_print_status);
  pragma INTERFACE (C, isc_fget_status);

end c_serp;
