--+
--|
--| Package spec:	SERPENT
--|
--| Defines the Ada wrappers for the Serpent interface routines
--|
--+
with system;
 
with text_io;
use text_io;

with ca_conv;
  use ca_conv;

with c_serp;
  use c_serp;

with s_types;
  use s_types;

package serpent is

  procedure serpent_init (mailbox, ill_file : in string);

  function start_transaction return transaction_type;

  function add_shared_data (
    transaction : in transaction_type;
    element_name, component_name : in string;
    data : in system.address
  ) return id_type;

  procedure put_shared_data(
    transaction    : in transaction_type;
    id             : in id_type; 
    element_name   : in string;
    component_name : in string; 
    data           : in system.address 
  );

  procedure remove_shared_data(
    transaction  : in transaction_type; 
    element_name : in string; 
    id           : in id_type
  );
 
  procedure rollback_transaction(
    transaction : in transaction_type
  );

  procedure commit_transaction(
    transaction : in transaction_type 
  );

  function get_transaction return transaction_type;

  function get_transaction_no_wait return transaction_type;

  function get_first_changed_element(
    transaction : in transaction_type
  ) return id_type;

  function get_next_changed_element(
    transaction : in transaction_type
  ) return id_type;

  procedure get_element_name(
    transaction : in transaction_type;
    id : in id_type;
    ada_element_name : out string
  );
 


  function get_change_type(
    transaction : in transaction_type;
    id : in id_type 
  ) return change_type;

  function create_changed_component_list(
    transaction : in transaction_type;
    id : in id_type 
  ) return LIST;

  function get_first_node(
    list_arg : in LIST
  ) return NODE;

  function get_next_node(
    list_arg : in LIST;
    node_arg : in NODE
  ) return NODE;

  procedure get_component_name(
    node_arg : in NODE;
    component_name : out string
  );

  procedure destroy_changed_component_list(
    changed_component_list : in LIST 
  );

  procedure incorporate_changes(
    transaction : in transaction_type;
    id   : in id_type;
    data : in system.address     
  );

  function get_shared_data (
    transaction : in transaction_type;
    id		: in id_type;
    component_name : in string
  ) return system.address;

  procedure get_shared_data_string (
    transaction : in transaction_type;
    id		: in id_type;
    component_name : in string;
    value       : out string
  );

  procedure get_shared_data_real (
    transaction : in transaction_type;
    id		: in id_type;
    component_name : in string;
    value       : out long_float
  );

  procedure get_shared_data_integer (
    transaction : in transaction_type;
    id		: in id_type;
    component_name : in string;
    value       : out integer
  );
    

  function get_shared_data_type (
    element_name    : in string;
    component_name  : in string
  ) return shared_data_types;


  procedure purge_transaction(
    transaction : in transaction_type
  );

  procedure start_recording(
    transaction : in transaction_type; 
    file_name   : in string 
  );

  procedure stop_recording;

 
  function get_status return status_codes;

  procedure print_status(
    error_msg : in string
  );
 
-- NOT IMPLEMENTED YET ??????
--  procedure playback(
--    file_name   : in string 
--  );

  procedure serpent_cleanup;
    
end serpent;
