--+
--|
--| Package body:	SERPENT
--|
--| Defines the Ada wrappers for the Serpent interface routines
--|
--+


with unchecked_conversion;

package body serpent is

  function addr_to_cstr is new unchecked_conversion (system.address,c_string);

  function addr_to_integer is new unchecked_conversion (
                                 system.address,integer_pointer);

  function addr_to_real is new unchecked_conversion (
                                 system.address,real_pointer);

  procedure serpent_init (mailbox, ill_file : in string) is
    c_mailbox : string(1..mailbox'last+1);
    c_ill_file : string(1..ill_file'last+1);
  begin
    ada_to_c_string(mailbox, c_mailbox);
    ada_to_c_string(ill_file, c_ill_file);
    isd_init (  addr_to_cstr(c_mailbox'address), 
                addr_to_cstr(c_ill_file'address));
  end serpent_init;

  function start_transaction return transaction_type is
  begin
    return transaction_type (isd_start_transaction);
  end start_transaction;

  function add_shared_data (
    transaction : in transaction_type;
    element_name, component_name : in string;
    data : in system.address
  ) return id_type is

    c_element_name :  string(1..element_name'last+1);
    c_component_name : string(1..component_name'last+1);
    c_element_name_address : c_string;
    c_component_name_address : c_string;

  begin
--
--  Convert ada string parameters.
--
    ada_to_c_string(element_name, c_element_name);
    ada_to_c_string(component_name, c_component_name);
 
    c_element_name_address := addr_to_cstr(c_element_name'address);
    c_component_name_address := addr_to_cstr(c_component_name'address);

    if element_name = "" then
       c_element_name_address := null;
    end if;

    if component_name = "" then
       c_component_name_address := null;
    end if;

--
--  Call corresponding c add shared data routine.
--    
    return id_type (isd_add_shared_data (
                      c_handle (transaction),
                      c_element_name_address,
                      c_component_name_address,
                      data
                    )
           );
  end;

  procedure put_shared_data(
    transaction    : in transaction_type;
    id             : in id_type; 
    element_name   : in string;
    component_name : in string; 
    data           : in system.address 
  ) is
    c_element_name :  string(1..element_name'last+1);
    c_component_name :  string(1..component_name'last+1);
    c_element_name_address : c_string;
    c_component_name_address : c_string;

  begin
    ada_to_c_string(element_name, c_element_name);
    ada_to_c_string(component_name, c_component_name);

    c_element_name_address := addr_to_cstr(c_element_name'address);
    c_component_name_address := addr_to_cstr(c_component_name'address);

    if element_name = "" then
       c_element_name_address := null;
    end if;

    if component_name = "" then
       c_component_name_address := null;
    end if;

    isd_put_shared_data(
      c_handle (transaction), 
      int (id),
      c_element_name_address,
      c_component_name_address,
      data
    );
  end;


  procedure remove_shared_data(
    transaction  : in transaction_type; 
    element_name : in string; 
    id           : in id_type
  ) is
    c_element_name :  string(1..element_name'last+1);
  begin
    ada_to_c_string(element_name, c_element_name);
    isd_remove_shared_data (
      c_handle (transaction),
      addr_to_cstr(c_element_name'address),
      int (id)
    );
  end;

  procedure rollback_transaction(
    transaction : in transaction_type
  ) is
  begin
    isd_rollback_transaction (c_handle (transaction));
  end;

  procedure commit_transaction(
    transaction : in transaction_type 
  ) is
  begin
    isd_commit_transaction (c_handle (transaction));
  end;

  function get_transaction return transaction_type is
  begin
    return transaction_type (isd_get_transaction);    
  end;

  function get_transaction_no_wait return transaction_type is
  begin
        return transaction_type (isd_get_transaction_no_wait);    
  end;

  function get_first_changed_element(
    transaction : in transaction_type
  ) return id_type is
  begin
    return id_type (isd_get_first_changed_element (c_handle (transaction)));
  end;

  function get_next_changed_element(
    transaction : in transaction_type
  ) return id_type is
  begin
    return id_type (isd_get_next_changed_element (c_handle (transaction)));
  end;

  procedure get_element_name(
    transaction : in transaction_type;
    id : in id_type;
    ada_element_name : out string
  ) is
    c_element_name : c_string;
--    n : natural;
  begin
    c_element_name := isd_get_element_name( c_handle (transaction), int (id));
    c_to_ada_string (c_element_name,ada_element_name);
    return;
--    n := 1;
--    while (n <= c_element_name'length) and (c_element_name(n) /= ascii.nul) 
--      loop
--        n := n+1;
--      end loop;
--    n := n-1;
--    return c_element_name(1..n);
  end;

  function get_change_type(
    transaction : in transaction_type;
    id : in id_type 
  ) return change_type is
    change : change_type;
  begin
    change := isd_get_change_type (c_handle (transaction), int (id));
    return change;
  end;

  function create_changed_component_list(
    transaction : in transaction_type;
    id : in id_type 
  ) return LIST is
  begin
    return LIST(isd_create_changed_component_list (c_handle (transaction),
		int (id)));
  end;

  function get_first_node(
    list_arg : in LIST
  ) return NODE is
  begin
    return NODE(isd_get_next_component_name (list_arg, NULL));
  end;

  function get_next_node(
    list_arg : in LIST;
    node_arg : in NODE
  ) return NODE is
  begin
    return NODE(isd_get_next_component_name (list_arg, node_arg));
  end;

  procedure get_component_name(
    node_arg : in NODE;
    component_name : out string
  ) is
  begin
    c_to_ada_string (node_arg,component_name);
    return;
  end;

  procedure destroy_changed_component_list(
    changed_component_list : in LIST 
  ) is
  begin
    isd_destroy_changed_component_list (c_handle (changed_component_list));
  end;

  procedure incorporate_changes(
    transaction : in transaction_type;
    id   : in id_type;
    data : in system.address     
  ) is
  begin
    isd_incorporate_changes (c_handle (transaction), int (id), data);
  end;

  function get_shared_data_type (
    element_name    : in string;
    component_name  : in string
  ) return shared_data_types is
    c_element_name :  string(1..element_name'last+1);
    c_component_name :  string(1..component_name'last+1);
    c_element_name_address : c_string;
    c_component_name_address : c_string;
  begin
    ada_to_c_string(component_name, c_component_name);
    ada_to_c_string(element_name, c_element_name);

    c_element_name_address := addr_to_cstr(c_element_name'address);
    c_component_name_address := addr_to_cstr(c_component_name'address);

    if element_name = "" then
       c_element_name_address := null;
    end if;

    if component_name = "" then
       c_component_name_address := null;
    end if;

    return isd_get_shared_data_type (c_element_name_address, 
                                     c_component_name_address);
  end;



function get_shared_data(
    transaction : in transaction_type;
    id             : in id_type;
    component_name : in string 
  ) return system.address is
    c_component_name :  string(1..component_name'last+1);
    c_component_name_address : c_string;
  begin
    ada_to_c_string(component_name, c_component_name);

    c_component_name_address := addr_to_cstr(c_component_name'address);

    if component_name = "" then
       c_component_name_address := null;
    end if;

    return isd_get_shared_data (c_handle (transaction),
				int (id),
				c_component_name_address);
  end;

    
procedure get_shared_data_string(
    transaction : in transaction_type;
    id             : in id_type;
    component_name : in string;
    value          : out string
  )  is
    c_component_name :  string(1..component_name'last+1);
    c_component_name_address : c_string;
  begin
    ada_to_c_string(component_name, c_component_name);

    c_component_name_address := addr_to_cstr(c_component_name'address);

    if component_name = "" then
       c_component_name_address := null;
    end if;

     c_to_ada_string (addr_to_cstr (isd_get_shared_data (
                                c_handle (transaction),
				int (id),
				c_component_name_address)),
                      value);
     return;
  end;

    
procedure get_shared_data_real(
    transaction : in transaction_type;
    id             : in id_type;
    component_name : in string;
    value          : out long_float
  ) is
    c_component_name :  string(1..component_name'last+1);
    c_component_name_address : c_string;
  begin
    ada_to_c_string(component_name, c_component_name);

    c_component_name_address := addr_to_cstr(c_component_name'address);

    if component_name = "" then
       c_component_name_address := null;
    end if;

    value := addr_to_real (isd_get_shared_data (c_handle (transaction),
				int (id),
				c_component_name_address)).all;
    return;
  end;

    
procedure get_shared_data_integer(
    transaction : in transaction_type;
    id             : in id_type;
    component_name : in string;
    value          : out integer
  ) is
    c_component_name :  string(1..component_name'last+1);
    c_component_name_address : c_string;
  begin
    ada_to_c_string(component_name, c_component_name);

    c_component_name_address := addr_to_cstr(c_component_name'address);

    if component_name = "" then
       c_component_name_address := null;
    end if;

    value := addr_to_integer (isd_get_shared_data (c_handle (transaction),
				int (id),
				c_component_name_address)).all;
    return;
  end;

    
  procedure purge_transaction(
    transaction : in transaction_type
  ) is
  begin
    isd_purge_transaction (c_handle (transaction));
  end;

  procedure start_recording(
    transaction : in transaction_type; 
    file_name   : in string
  ) is
    c_file_name :  string(1..file_name'last+1);
  begin
    ada_to_c_string(file_name, c_file_name);
    ipb_start_recording(c_handle (transaction), 
                        addr_to_cstr(c_file_name'address));
  end;

  procedure stop_recording
  is
  begin
    ipb_stop_recording;
  end;

 
--  MSUT CHECK OUT THIS IMPLEMENTATION !!!!!
--  procedure playback(
--    file_name   : in string 
--  ) is
--    c_file_name : c_string;
--  begin
--    c_file_name := ada_to_c_string(file_name, c_file_name);
--    ipb_playback(c_file_name);
--  end;
 
  function get_status return status_codes is
  begin
    return isc_fget_status;
  end get_status;


  procedure print_status(error_msg : in string) is
    c_error_msg :  string(1..error_msg'last+1);
  begin
    ada_to_c_string(error_msg, c_error_msg);
    isc_print_status( addr_to_cstr(c_error_msg'address));
  end;


  procedure serpent_cleanup is
  begin
    isd_cleanup;
  end;

end serpent;
