/*      $Header: /u/04c/mnt/integ/source/int/include/RCS/isc.h,v 1.25 91/03/12 15:54:41 rcs Exp $ */

/*--------------------------------------------------------------------------*\
| 
|  Name:  Status Code Abstraction (isc)
|
|  Description:  
|     The status code abstraction provides a repository for status code
|     information.
|
|  Component of:
|     Interface Layer.
|
|  Related files:
|     isc.h - contains the external definition for the status code 
|	      abstraction.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Robert C. Seacord
|     rcs@sei.cmu.edu  
|
|  Change History:
|     Oct. 19, 1987 : Module created.
|     $Log:	isc.h,v $
 * Revision 1.25  91/03/12  15:54:41  rcs
 * removed aborts because they were duplicated with debug1 stuff
 * 
 * Revision 1.24  91/03/07  18:40:29  rcs
 * modified rcheck routines to dump core on DEBUG1
 * 
 * Revision 1.23  90/09/11  20:33:58  rcs
 * replaced gcore with abort
 * 
 * Revision 1.22  90/08/29  18:25:06  rcs
 * modified debug1 to core dump
 * 
 * Revision 1.21  90/08/14  10:55:12  rcs
 * moved status codes to ist.h
 * 
 * Revision 1.20  90/06/27  08:42:38  little
 * no changes, sorry.
 * 
 * Revision 1.19  90/02/14  21:53:31  ejh
 * fixed include structure
 * 
 * Revision 1.18  89/12/14  20:11:45  rcs
 * added de status codes
 * 
 * Revision 1.17  89/10/05  17:21:10  rcs
 * new error type
 * 
 * Revision 1.16  89/02/13  12:10:18  ejh
 * added copyright notice
 * 
 * Revision 1.15  89/02/10  11:00:43  rcs
 * added FATAL_ERROR macro but then commented it out
 * since reed imbeds it in each of his files
 * 
 * Revision 1.14  89/01/04  17:01:08  rcs
 * added removed_data_object status code
 * 
 * Revision 1.13  88/12/21  14:16:27  rcs
 * Modified status codes to have contiguous numbers
 * 
 * Revision 1.12  88/12/19  17:38:03  rcs
 * added check_status_nr macro
 * added isc_get_status_string external definition
 * 
 * Revision 1.11  88/11/02  14:16:22  rcs
 * moved debug variable from c file - rCs
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef	ISC_H
#define	ISC_H

#include <stdio.h>              /* C Standard I/O Package                   */
#include "global.h"             /* Global type defs and constants           */
#include "ist.h"		/* defines for status messages		    */

/*--------------------------------------------------------------------------*\
|  Global Variable: isc_debug
|
|  Description:
|     Debug variable for the abstraction.
|
\* Definition:                                                              */

#ifdef isc_TOP
  int isc_debug = 1;
#else
  extern int isc_debug;
#endif

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Name: set_status
|
|  Description:
|     The set status routine sets status to a specified value.
|
\* Syntax:                                                                  */

#define set_status(val) isc_status=val

/*
|  Parameters:
|     val - New status value.
|
|  Returns:
|     Void.
|
|  Statuses:
|     None.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Name: get_status
|
|  Description:
|     Returns the current system status.
|
\* Syntax:                                                                  */

#define get_status() isc_status

/*
|  Parameters:
|     None.
|
|  Returns:
|     The current status.
|
|  Statuses:
|     none.
|
\*--------------------------------------------------------------------------*/

#define error_val -1

/*--------------------------------------------------------------------------*\
|  Name: Serpent Debug
|
|  Description:
|     The Serpent debug macro is used as part of a debug/trace facility.
|     The macros check to see if a certain level of debug is enabled.
|     If so, they execute a single statement or sequence of statements
|     supplied by the caller and then flush stderr.
|
\* Syntax:                                                                  */

#ifndef NDEBUG

#define SERPENT_DEBUG1(flag, statement)\
  if(flag > 0) {\
    statement;\
    fprintf (stderr, "      file %s, line %d\n", __FILE__, __LINE__);\
    fflush(stderr);\
    abort();\
  }

#define SERPENT_DEBUG2(flag, statement)\
  if(flag > 1) {\
    statement;\
    fflush(stderr);\
  }

#define SERPENT_DEBUG3(flag, statement)\
  if(flag > 2) {\
    statement;\
    fprintf (stderr, "      file %s, line %d\n", __FILE__, __LINE__);\
    fflush(stderr);\
  }

#else

#define SERPENT_DEBUG1(flag, statement)
#define SERPENT_DEBUG2(flag, statement)
#define SERPENT_DEBUG3(flag, statement)

#endif

/*
|  Parameters:
|     flag - The specific debug variable to examine to determine if debug
|            is on at a given level.
|     statement - Statement or sequence of statements to execute.
|
|  Returns:
|     Void.
|
|  Statuses:
|     None.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Name:  FATAL_ERROR
|
|  Description:
|     The fatal error macro causes debug information to be printed 
|     to stderr, a core dump and an exit.
|
\* Syntax:                                                                  */

/*
|  Parameters:
|     statement - Statement or sequence of statements to execute.
|
|  Returns:
|     Void.
|
|  Statuses:
|     None.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Name: Check Status
|
|  Description:
|     The check status macros are used to check the status of a returning
|     call, and if debug1 is enabled to print out the status.
|
\* Syntax:                                                                  */

#define check_status(string)\
  if (get_status()) {\
    DEBUG1(\
      isc_print_status(string);\
    );\
    return;\
  }

#define check_null(result, string, status)\
  if (result == NULL) {\
    DEBUG1(\
      fprintf(stderr, string);\
    );\
    set_status(status);\
    return;\
  }

#define check_error(result, string, status)\
  if (result == -1) {\
    DEBUG1(\
      perror(string);\
    );\
    set_status(status);\
    return;\
  }

#define rcheck_status(string, return_value)\
  if (get_status()) {\
    DEBUG1(\
      isc_print_status(string);\
    );\
    return(return_value);\
  }

#define rcheck_null(result, string, status, return_value)\
  if (result == NULL) {\
    DEBUG1(\
      fprintf(stderr, string);\
    );\
    set_status(status);\
    return(return_value);\
  }

#define rcheck_error(result, string, status, return_value)\
  if (result == -1) {\
    DEBUG1(\
      perror(string);\
    );\
    set_status(status);\
    return(return_value);\
  }

/*
|  Parameters:
|     result - Resultant value to check for error condition.
|     string - String to print on error condition.
|     status - Status to be set.
|     return_value - Value to return to calling routine.
|
|  Returns:
|     Void.
|
|  Statuses:
|     None.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Type: isc_status_codes
|
|  Description:
|     The isc status codes type enumerates the possible returned status
|     values from the interface layer.
|
\* Definition:                                                              */

        typedef int isc_status_codes; 

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Type: isc_validation_stamp
|
|  Description:
|     The isc validation stamps type enumerates the validation stamps for
|     the various private types defined within interface layer.
|
\* Definition:                                                              */

	typedef enum validation_stamp {
	  isc_process_record_vs		=1,
	  isc_transaction_record_vs	=2,
	  isc_rcvd_tran_vs		=3
	} isc_validation_stamp;

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Variable: isc_status
|
|  Description:
|     The isc status variable contains the status for the interface layer.
|
\* Definition:                                                              */

extern isc_status_codes isc_status;

/*
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Name: isc_get_status_string
|
|  Description:
|     The isc_get_status_string function returns the string corresponding
|     to the current status.
|
\* Syntax:                                                                  */

        string isc_get_status_string();

/*
|  Parameters:
|     None.
|
|  Returns:
|     the string corresponding to the current status.
|
|  Statuses:
|     none.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Name: isc_print_status
|
|  Description:
|     Prints out a user defined error message annd the current status.
|
\* Syntax:                                                                  */

        void isc_print_status(
	  /* error_msg : in string */
	);

/*
|  Parameters:
|     error_msg - User defined error message.
|
|  Returns:
|     Void.
|
|  Statuses:
|     none.
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Name: isc_validate
|
|  Description:
|     Validates a handle.
|
\* Syntax:                                                                  */

        boolean isc_validate(
	  /* handle : in char * 			*/
	  /* validation : in isc_validation_stamp 	*/
	);

/*
|  Parameters:
|     handle - Handle to be validated.
|     validation - Validation stamp to examine for.
|
|  Returns:
|     True if handle is valid, false if handle is invalid.
|
|  Statuses:
|     none.
|
\*--------------------------------------------------------------------------*/
#endif

