static char rcsid [] =
	"$Header: /usr/users/dvk/serpent/sun3/int/src/RCS/idd.c,v 1.20 1991/04/23 21:00:24 dvk Exp $";

/*--------------------------------------------------------------------------*\
|
|  Name:  Interface Data Definition abstraction (idd)
|
|  Description:  
|     The interface data definition abstraction provides basic operations for
|     creating data definitions to be used by an object base.
|
|  Component of:
|     Interface Layer.
|
|  Related files:
|     idd.c - contains the implementation for the interface data definition
|	      abstraction.
|     idd.h - contains the external definition for the interface data 
|	      definition abstraction.
|     iddtest.c - test file.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Robert C. Seacord
|     rcs@sei.cmu.edu  
|
|  Change History:
|     17 may 1988: reed -- added definitions for:
|        idd_walk_data_definition, idd_get_name, and
|        idd_get_element_number_of_components
|
|     Oct. 19, 1987 : Module created.
|     $Log: idd.c,v $
 * Revision 1.20  1991/04/23  21:00:24  dvk
 * Changed MAX_HASH back to 67 (from V1.15, 'cuz I botched it up in V1.16)
 *
 * Revision 1.19  1991/04/23  01:57:20  rcs
 * modified element/component not found to print out element/component name
 *
 * Revision 1.18  1991/04/19  19:18:50  ejh
 * went back to revision 1.16
 *
 * Revision 1.16  1991/03/11  16:47:14  dvk
 * Changed idd_create_component_list to idd_create_component_alist, added
 * some more hash cacheing.
 *
 * Revision 1.15  91/02/18  16:25:49  rcs
 * added the set undefined routine
 * 
 * Revision 1.14  90/10/10  15:44:38  dvk
 * Added small cacheing function in idd_get_shared_data_type and
 * idd_get_component_name.
 * 
 * Revision 1.13  90/07/19  13:25:09  little
 * change hash table size to a prime number
 * 
 * Revision 1.12  90/06/25  16:35:50  rcs
 * no changes
 * 
 * Revision 1.11  90/02/14  21:59:34  ejh
 * fixed include structure
 * 
 * Revision 1.10  89/11/30  14:45:24  rcs
 * added idd_undefined routine
 * 
 * Revision 1.9  89/09/20  12:51:17  little
 * add idd_validate_component
 * 
 * Revision 1.8  89/04/05  15:54:47  trm
 * just checking out checkin. no changes made.  trm
 * 
 * Revision 1.7  89/02/09  19:00:36  ejh
 * added copyright notice
 * 
 * Revision 1.6  89/01/30  14:40:24  rcs
 * modified order of type name strings in idd_data_type_strings
 * 
 * Revision 1.5  88/12/19  17:35:46  rcs
 * add idd_get_data_string utility
 * 
 * Revision 1.4  88/11/03  18:45:53  little
 * add idd_validate_element routine
 * 
 * Revision 1.3  88/11/02  15:41:54  rcs
 * *** empty log message ***
 * 
 * Revision 1.2  88/11/02  14:18:09  rcs
 * moved debug variable to interface file - rCs
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	idd_TOP
#define	memoryPac

#include	<stdio.h>
#include	"listPack.h"
#include	"hashPack.h"
#include	"alistPack.h"
#include	"global.h"
#include	"string.h"
#include	"ubo.h"
#include	"u22.h"
#include	"iid.h"
#include	"isc.h"
#include	"idd.h"

#define DEBUG1(statement) SERPENT_DEBUG1(idd_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(idd_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(idd_debug, statement)

#ifdef EXPERIMENTAL_HASH	/* It doesn't help here */
/*
 * This is intentionally a value that is 2^n.  See the routine
 * hash_string for details.
 */
#define MAX_HASH 64
#else
#define MAX_HASH 67
#endif

typedef struct {
  string name;
  idd_data_types type;
  int offset, length, position;
  char *parent;
} component_record;

typedef struct {
  string name;
  idd_data_types type;
  int components;
  int length;
  ALIST component_names;
  HASH component_table;
} element_record;

static char *idd_data_type_strings[] = {
  "boolean",
  "integer",
  "real",
  "string",
  "record",
  "id",
  "buffer",
  "undefined",
  "no data type"
};

/*--------------------------------------------------------------------------*\
|  Routine: match_element_name
| 
|  Description:  
|     Routine to compare the identifiers with user elements.
|   
\*--------------------------------------------------------------------------*/
int match_element_name(identifier, element)
string identifier;
element_record *element;
{
  return(strcmp(identifier, element->name) == 0);
}

/*--------------------------------------------------------------------------*\
|  Routine: match_component_name
| 
|  Description:  
|     Routine to compare the identifiers with user components.
|   
\*--------------------------------------------------------------------------*/
int match_component_name(identifier, component)
string identifier;
component_record *component;
{
  return(strcmp(identifier, component->name) == 0);
}

/*--------------------------------------------------------------------------*\
|  Routine: hash_string
| 
|  Description:  
|     Internal function which will convert a string identifiers into indicies
|     in the array of hash lists.
|
\*--------------------------------------------------------------------------*/
int hash_string(name)
string name;
{
  register char h;
  register char *p;

#ifdef EXPERIMENTAL_HASH	/* It doesn't help here */
/*
 * This routine is adapted from one described in "Fast Hashing of Variable
 * Length Text Strings" by Peter K. Pearson, CACM 33(6):677-680 June 1990
 */   
  static char T[256] = {
    1, 87, 49, 12, 176, 178, 102, 16, 121, 193, 6, 84, 249, 230, 44, 163,
    14, 197, 213, 181, 161, 85, 218, 80, 64, 239, 24, 226, 236, 142, 38, 200,
    110,177, 104, 103, 141, 253, 255, 50, 77, 101, 81, 18, 45, 96, 31, 222,
    25, 107, 190, 70, 86, 237, 240, 34, 72, 242, 20, 214, 244, 227, 149, 235,
    97, 234, 57, 22, 60, 250, 82, 175, 208, 5, 127, 199, 111, 62, 135, 248,
    174, 169, 211, 58, 66, 154, 106, 195, 245, 171, 17, 187, 182, 179, 0, 243,
    132, 56, 148, 75, 128, 133, 158, 100, 130, 126, 91, 13, 153, 246, 216, 219,
    119, 68, 223, 78, 83, 88, 201, 99, 122, 11, 92, 32, 136, 114, 52, 10,
    138, 30, 48, 183, 156, 35, 61, 26, 143, 74, 251, 94, 129, 162, 63, 152,
    170, 7, 115, 167, 241, 206, 3, 150, 55, 59, 151, 220, 90, 53, 23, 131,
    125, 173, 15, 238, 79, 95, 89, 16, 105, 137, 225, 224, 217, 160, 37, 123,
    118, 73, 2, 157, 46, 116, 9, 145, 134, 228, 207, 212, 202, 215, 69, 229,
    27, 188, 67, 124, 168, 252, 42, 4, 29, 108, 21, 247, 19, 205, 39, 203,
    23, 40, 186, 147, 198, 192, 155, 33, 164, 191, 98, 204, 165, 180, 117, 76,
    140, 36, 210, 172, 41, 54, 159, 8, 185, 232, 113, 196, 231, 47, 146, 120,
    51, 65, 28, 144, 254, 221, 93, 189, 194, 139, 112, 43, 71, 109, 184, 209
    };

/*
 * We assume that the hash table size is a power of two (doesn't need to be
 * prime, like most Knuthian algorithms, because the hash uses a cryptographic
 * checksum with a bounded sum and a very good hash separation).  This means
 * that we don't need to do a modulo operation, but can instead just do an AND
 * of the table size - 1.
 */
  for (p = name, h = 0; *p; p++)
      h = T[h ^ *p];
  return (h & (MAX_HASH-1));
#else
  for (p = name, h = 0; *p; p++)
      h += *p;
  return ((h & 077777) % MAX_HASH);
#endif
}

/*--------------------------------------------------------------------------*\
|  Routine: match_position
| 
|  Description:  
|     Routine to compare the identifiers with user elements.
|   
\*--------------------------------------------------------------------------*/
int match_position(identifier, component)
int identifier;
component_record *component;
{
  return(identifier == component->position);
}

/*--------------------------------------------------------------------------*\
|  Routine: idd_create_data_definition
| 
|  Description:  
|     The create data definition routine is used to create a data definition.
\*--------------------------------------------------------------------------*/
idd_data_definition idd_create_data_definition()
{
  HASH data_definition;

  data_definition = make_hashtable(MAX_HASH, hash_string,
	match_element_name);
  return((idd_data_definition)data_definition);
}

/*--------------------------------------------------------------------------*\
|  Routine: idd_create_element
| 
|  Description:  
|     The create element routine is used to create an element for a given
|     data definition.
\*--------------------------------------------------------------------------*/
idd_element idd_create_element(data_definition, name, type, length)
idd_data_definition data_definition;
string name;
idd_data_types type;
int length;
{ 					/* local type definitions	    */
  element_record *element;

  set_status(ok);			/* begin 			    */
  DEBUG2(fprintf(stderr, "idd_create_element: call.\n");
    fprintf(stderr, "data_definition = %x.\n", data_definition);
    fprintf(stderr, "name = %s.\n", name);
    fprintf(stderr, "type = %d.\n", type);
    fprintf(stderr, "length = %d.\n", length));
/*
   Create Element.
*/
  element = (element_record *)make_node(sizeof(element_record));
  rcheck_null(element,
	"idd_create_element:  out of memory during make_node.\n",
    out_of_memory, NULL);
/*
   Initialize element fields.
*/
  element->name = u22_copy_string (name);

  element->length = length;
  element->components = -1;
  element->type = type;
/*
   If element is a record create two hash tables for the components,
   one that is indexed by the component name and another which is
   indexed by the position of the component in the element.
*/
  if (type == idd_record) {		/* record type			    */

    element->component_table = make_hashtable(MAX_HASH, hash_string, 
      match_component_name);

    element->component_names = make_alist();
  }					/* end record type		    */
/*
   Add the element to the data definition hash table using the element
   name as the hashing index.
*/
  add_to_hashtable(data_definition, element, name);

  DEBUG3(fprintf(stderr, "element_name = %s.\n", (idd_element)element->name));

  DEBUG2(fprintf(stderr, "element = %x.\n", (idd_element)element);
    fprintf(stderr, "idd_create_element:  exiting.\n"));
  return((idd_element)element);
} 					/* end idd_create_element   */

/*--------------------------------------------------------------------------*\
|  Routine: idd_create_component
| 
|  Description:  
|     The create component routine is used to create a component of a 
|     particular element.
\*--------------------------------------------------------------------------*/
idd_component idd_create_component(element, name, type, offset, length)
idd_element element;
string name;
idd_data_types type;
int offset, length;
{ 					/* local type definitions	    */
  component_record *component;

  set_status(ok);			/* begin 			    */
/*
 * Allocate storage for component record.
 */
  component = (component_record *)make_node(sizeof(component_record));
  rcheck_null(component, 
    "idd_create_component:  out of memory during make_node.\n",
    out_of_memory, NULL);
/*
 * Initialize component fields.
 */
  component->name = u22_copy_string (name);

  component->type = type;
  component->offset = offset;
  component->length = length;
  component->parent = (char *)element;		/* back pointer	*/
/*
   Add component to list of components referenced by element name.
 */

  add_to_hashtable(((element_record *)element)->component_table, 
							component, name);
/*
 * Increment component number in element record and set position field
 * in component record accordingly.  Add component to hash referenced by 
 * component number.
 */
  component->position = ++(((element_record *)element)->components);

  add_to_atail(((element_record *)element)->component_names, component);

  return((idd_component)component);
}

/*--------------------------------------------------------------------------*\
|  Routine: idd_get_shared_data_type
|
|  Description:
|     The get shared data type routine is used to get the type associated with
|     a shared data element or component if not nullstring.
\*--------------------------------------------------------------------------*/
idd_data_types idd_get_shared_data_type(data_definition,
				element_name, component_name)
idd_data_definition data_definition;
string element_name;
string component_name;
{                                       /* local variable definitions       */
  idd_data_types data_type;
  element_record *element;
  component_record *component;

  /*
   * It turns out that we hit the hashtables with the same first level (and
   * sometimes the same second level) values.  We cache the last entries
   * to be looked up in the table, and save some speed.
   */
  static char cache_element_name[256] = {0};
  static element_record *cache_element;
  static char cache_component_name[256] = {0};
  static component_record *cache_component;

  set_status(ok);                       /* begin                            */
/*
 * Find element name in hash table.
 */
  if (strcmp (element_name, cache_element_name) == 0) {
      element = cache_element;
      }
  else {
      element = (element_record *)get_from_hashtable(data_definition, 
	element_name);

      if (element == NULL) {
        DEBUG1(
          fprintf(
            stderr,
            "idd_get_shared_data_type: element `%s` not found.\n",
            element_name
          );
        );
        set_status(not_found);
        return(idd_no_data_type);
      }

      (void)strcpy(cache_element_name, element_name);
      cache_element = element;
      (void)strcpy (cache_component_name, "");
      }
/* 
 * If component name not specified return element type.
 */
  if (component_name == NULL)
    data_type = element->type;
/* 
 *  Else if component name specified return component type.
 */
  else {				/* component specified		    */
/*
 *  Find component name in component table.
 */
    if (strcmp (component_name, cache_component_name) == 0) {
      component = cache_component;
      }
    else {
      component = (component_record *)get_from_hashtable(
	  element->component_table, component_name);

      if (component == NULL) {
        DEBUG1(
          fprintf(stderr,
            "idd_get_shared_data_type: component `%s` not found.\n",
            component_name
          );
        );
        set_status(not_found);
        return(idd_no_data_type);
      }

      (void)strcpy (cache_component_name, component_name);
      cache_component = component;
      }

    data_type = component->type;  

  }					/* end else component specified	    */

  return data_type;
}

/*--------------------------------------------------------------------------*\
|  Routine: build_component_alist
|
|  Description:
|     The build component alist routine adds a component name to the tail
|     of a specified alist.
\*--------------------------------------------------------------------------*/
static void build_component_alist(component_alist, component)
ALIST component_alist;
component_record *component;
{
  (void)add_to_atail(component_alist, component->name);
}

/*--------------------------------------------------------------------------*\
|  Routine: idd_create_component_alist
|
|  Description:
|     The get component name alist routine accepts an element name as a
|     parameter and returns a list of component names.
\*--------------------------------------------------------------------------*/
ALIST idd_create_component_alist(data_definition, element_name)
idd_data_definition data_definition;
string element_name;
{                                       /* local variable definitions       */
  ALIST component_alist;
  element_record *element;

  set_status(ok);
/*
  Find element name in hash table.
*/
  element = (element_record *)get_from_hashtable(data_definition, 
    element_name
  );
  if (element == NULL) {
    DEBUG1(
      fprintf(
        stderr,
        "idd_create_component_alist: element `%s` not found.\n",
        element_name
      );
    );
    set_status(not_found);
    return(NULL);
  }
/* 
   Validate that the element is a record.
*/
  if (element->components == -1) {	/* not a record			    */
    set_status(element_not_a_record);
    DEBUG1(isc_print_status("idd_create_component_alist"));
    return NULL;
  }					/* end if no components		    */
/*
   Create a alist in process memory to return the list of component names.
*/
  component_alist = make_alist();
  rcheck_null(component_alist,
    "idd_create_component_alist:  make_alist failed.\n", out_of_memory, NULL);
/*
   Convert alist structure of hash elements by applying build alist function
   to each element in the change table.
*/
  for_all_hashtable(element->component_table, component_alist,
    build_component_alist);

  return component_alist;
}

/*--------------------------------------------------------------------------*\
|  Routine: idd_destroy_component_alist
|
|  Description:
|     The destroy component alist routine frees all storage associated
|     with the specified component alist.
\*--------------------------------------------------------------------------*/
void idd_destroy_component_alist(component_alist)
ALIST component_alist;
{
  set_status(ok);
/*
   Destroy the list not including the nodes (which are actually pointers
   to data in the data definition.
*/
  free_alist(component_alist, NOT_NODES);
}

/*--------------------------------------------------------------------------*\
|  Routine: idd_get_component_name
| 
|  Description:  
|     The get component name routine returns the nth component in a 
|     shared data element record.
\*--------------------------------------------------------------------------*/
string idd_get_component_name(data_definition, element_name, position)
idd_data_definition data_definition;
string element_name;
int position;
{ 					/* local type definitions	    */
  element_record *element;
  component_record *component;
  string component_name;

  /*
   * It turns out that we hit the hashtables with the same first level (and
   * sometimes the same second level) values.  We cache the last entries
   * to be looked up in the table, and save some speed.  This routine differs
   * a bit from idd_get_shared_data_type, because alist lookups are fast
   * enough not to need to be cached.
   */
  static string cache_name[256] = {0};
  static element_record *cache_element;
  static int cache_position = -1;
  static component_record *cache_component;

  set_status(ok);			/* begin 			    */
/*
 * Find element name in hash table.
 */
  if (strcmp (element_name, cache_name) == 0) {
      element = cache_element;
      }
  else {
      element = (element_record *)get_from_hashtable(
	data_definition, element_name
      );
      if (element == NULL) {
        DEBUG1(
          fprintf(
            stderr,
            "idd_get_component_name: element `%s` not found.\n",
            element_name
          );
        );
        set_status(not_found);
        return(NULL);
      }

      (void)strcpy(cache_name, element_name);
      cache_element = element;
      cache_position = -1;
      }
/* 
 *  Validate that the element is a record.
 */
  if (element->components == -1) {	/* not a record			    */
    set_status(element_not_a_record);
    return NULL;
  }					/* end if no components		    */
/*
 *  Find component name in component name hashtable.
 */
  component = (component_record *)get_alist_pos(element->component_names,
							position+1);
  if (component == NULL) {
    DEBUG1(
      fprintf(stderr,
        "idd_get_component_name: component `%s` not found.\n",
        component_name
      );
    );
    set_status(not_found);
    return(NULL);
  }

  return component->name;
} 					/* end idd_get_component_name	    */

/*--------------------------------------------------------------------------*\
|  Routine: idd_get_length
| 
|  Description:  
|     The get length routine is used to returns the length of a specified 
|     shared data element.
\*--------------------------------------------------------------------------*/
int idd_get_length(data_definition, element_name)
idd_data_definition data_definition;
string element_name;
{ 					/* local type definitions	    */
  element_record *element;

  set_status(ok);			/* begin 			    */
  DEBUG2(fprintf(stderr, "idd_get_length: call.\n"));

/*
  Find element name in hash table.
*/
  element = (element_record *)get_from_hashtable(data_definition, 
    element_name);
  if (element == NULL) {
    DEBUG1(
      fprintf(
        stderr,
        "idd_get_length: element `%s` not found.\n",
        element_name
      );
    );
    set_status(not_found);
    return(-1);
  }


  DEBUG2(fprintf(stderr, "idd_get_length: length = %d.\n", element->length);
    fprintf(stderr, "idd_get_length: exiting.\n"));
  return element->length;
} 					/* end idd_get_length	    */

/*--------------------------------------------------------------------------*\
|  Routine: idd_validate_element
| 
|  Description:  
|     determines whethere a shared data element is in a data definition
\*--------------------------------------------------------------------------*/
boolean idd_validate_element (data_definition, element_name)
idd_data_definition data_definition;
string element_name;
{  element_record *element;

   set_status(ok);			/* begin 			    */
   DEBUG2 (fprintf(stderr, "idd_validate_element: call.\n"));
 
			/* Find element name in hash table.  */
   element = (element_record *) get_from_hashtable (data_definition,
	element_name);

   if (element)
      return (TRUE);
    else
      return (FALSE);
} 					/* end idd_validate_element */
 
/*--------------------------------------------------------------------------*\
|  Routine: idd_validate_component
| 
|  Description:  
|     determines whethere a shared data component is in an element in a data
|	definition
\*--------------------------------------------------------------------------*/
boolean idd_validate_component (data_definition, element_name, component_name)
idd_data_definition data_definition;
string element_name;
string component_name;
{  element_record *element;
   component_record *component;

   set_status(ok);			/* begin 			    */
   DEBUG2 (fprintf (stderr, "idd_validate_component: call.\n"));
 
 					/* Find element name in hash table  */
   element = (element_record *) get_from_hashtable (data_definition,
	element_name);
   if (element)
				/* Find component name in component table */
      {component = (component_record *) get_from_hashtable
	(element->component_table, component_name);
      if (component)
         return (TRUE);
        else
         return (FALSE);
      }
    else
      return (FALSE);
} 					/* end idd_validate_component */

/*--------------------------------------------------------------------------*\
|  Routine: idd_get_component_lop
| 
|  Description:  
|     The get offset routine returns the size and offset of a component 
|     within a shared data element.
\*--------------------------------------------------------------------------*/
idd_data_types idd_get_component_lop(data_definition, element_name,
				component_name, length, offset, position)
idd_data_definition data_definition;
string element_name, component_name;
int *length;
int *offset;
int *position;
{ 					/* local type definitions	    */
  element_record *element;
  component_record *component;
  /*
   * It turns out that we hit the hashtables with the same first level (and
   * sometimes the same second level) values.  We cache the last entries
   * to be looked up in the table, and save some speed.
   */
  static char cache_element_name[256] = {0};
  static element_record *cache_element;
  static char cache_component_name[256] = {0};
  static component_record *cache_component;

  set_status(ok);			/* begin 			    */
/*
 * Find element name in hash table.
 */
  if (strcmp (element_name, cache_element_name) == 0) {
      element = cache_element;
      }
  else {
    element = (element_record *)get_from_hashtable(data_definition, 
      element_name
    );
    if (element == NULL) {
      DEBUG1(
        fprintf(
          stderr,
          "idd_get_component_lop: element `%s` not found.\n",
          element_name
        );
      );
      set_status(not_found);
      return;
    }
    /* 
     * Validate that the element is a record.
     */
      if (element->components == -1) {	/* not a record			    */
	set_status(element_not_a_record);
	DEBUG1(isc_print_status("idd_create_component_list"));
	return;
	}
      (void)strcpy(cache_element_name, element_name);
      cache_element = element;
      (void)strcpy (cache_component_name, "");
      }
/*
 * Find component name in component table.
 */
    if (strcmp (component_name, cache_component_name) == 0) {
      component = cache_component;
    }
    else {
      component = (component_record *)get_from_hashtable
		(element->component_table, component_name);
      if (component == NULL) {
        DEBUG1(
          fprintf(stderr,
            "idd_get_component_lop: component `%s` not found.\n",
            component_name
          );
        );
        set_status(not_found);
      }


      (void)strcpy (cache_component_name, component_name);
      cache_component = component;
    }
/*
 * Set indirect return values and return component type
 */
  *length = component->length;
  *offset = component->offset;
  *position = component->position;

  return component->type;
}

/*--------------------------------------------------------------------------*\
|  Routine: idd_get_number_of_components
| 
|  Description:  
|     The get number of components returns the number of components in 
|     a shared data record.
\*--------------------------------------------------------------------------*/
int idd_get_number_of_components(data_definition, element_name)
idd_data_definition data_definition;
string element_name;
{ 					/* local type definitions	    */
  element_record *element;

  set_status(ok);			/* begin 			    */
  DEBUG2(fprintf(stderr, "idd_get_number_of_components: call.\n"));
/*
  Find element name in hash table.
*/
  element = (element_record *)get_from_hashtable(
    data_definition,element_name
  );
  if (element == NULL) {
    DEBUG1(
      fprintf(
        stderr,
        "idd_get_number_of_components: element `%s` not found.\n",
        element_name
      );
    );
    set_status(not_found);
    return(-1);
  }
/*
   Return number of components.
*/
  DEBUG2(fprintf(stderr, 
      "idd_get_number_of_components:  # of components = %d.\n",
      element->components);
    fprintf(stderr, "idd_get_number_of_components:  exiting.\n"));
  return element->components;
} 					/* end idd_get_number_of_components */

/*--------------------------------------------------------------------------*\
|  Routine: idd_destroy_component
| 
|  Description:  
|     The destroy component is used to destroy an existing component.
\*--------------------------------------------------------------------------*/
void idd_destroy_component(component)
idd_component component;
{
  set_status(ok);			/* begin 			    */
  DEBUG2(fprintf(stderr, "idd_destroy_component: call.\n"));
/*
   Free associated storage.
*/  
  free_node(((component_record *)component)->name);  
  free_node(component);

  DEBUG2(fprintf(stderr, "idd_destroy_component: exiting.\n"));
  return;
} 					/* end idd_destroy_component	    */

/*--------------------------------------------------------------------------*\
|  Routine: destroy_components
| 
|  Description:  
|     Prints the element name field of an element record.
\*--------------------------------------------------------------------------*/
static void destroy_components(nada, component)
component_record *nada;			/* dummy parameter		    */
component_record *component;		/* component to destroy 	    */
{
  idd_destroy_component(component);  
}

/*--------------------------------------------------------------------------*\
|  Routine: idd_destroy_element
| 
|  Description:  
|     The destroy element is used to destroy an existing element.
\*--------------------------------------------------------------------------*/
void idd_destroy_element(element)
idd_element element;
{

  set_status(ok);
/*
   If element is a record free component structures.
*/
  if (((element_record *)element)->type == idd_record) {  /* record type    */
/*
    Destroy each component in the record.
*/
    for_all_hashtable(((element_record *)element)->component_table, NULL,
      destroy_components);
/*
   Free all storage associated with both component hash tables.
*/
    free_hashtable(((element_record *)element)->component_table, NOT_NODES);
    free_alist(((element_record *)element)->component_names, NOT_NODES);

  }					/* end if record type		    */
/*
   Free storage associated with element name.
*/
  free_node(((element_record *)element)->name);
/*
   Free storage associated with element record.
*/
  free_node(element);
}

/*--------------------------------------------------------------------------*\
|  Routine: print_element_name
| 
|  Description:  
|     Prints the element name field of an element record.

\*--------------------------------------------------------------------------*/
static void print_element_name(nada, element)
element_record *nada;			/* dummy parameter		    */
element_record *element;		/* element to print name of	    */
{
  fprintf(stderr, "element_name = %s.\n", element->name);
}

/*--------------------------------------------------------------------------*\
|  Routine: idd_print_data_definition
| 
|  Description:  
|     The print data definition is used to print the element name of
|     each element in the specified data definition.  This routine is
|     mainly for debug purposes.
\*--------------------------------------------------------------------------*/
void idd_print_data_definition(data_definition)
idd_data_definition data_definition;
{

  set_status(ok);			/* begin 			    */
  DEBUG2(fprintf(stderr, "idd_print_data_definition: call.\n"));
/*
   Print the element name for each element in the hash table.
*/  
  for_all_hashtable(data_definition, NULL, print_element_name);

  DEBUG2(fprintf(stderr, "idd_print_data_definition: exiting.\n"));
  return;
} 					/* end idd_print_data_definition  */

/*--------------------------------------------------------------------------*\
|  Routine: destroy_elements
| 
|  Description:  
|     Destroys an element in the hash table.
\*--------------------------------------------------------------------------*/
static void destroy_elements(nada, element)
element_record *nada;			/* dummy parameter		    */
element_record *element;		/* element to destroy	 	    */
{
  idd_destroy_element(element);  
}

/*--------------------------------------------------------------------------*\
|  Routine: idd_destroy_data_definition
| 
|  Description:  
|     The destroy data definition is used to destroy an existing data 
|     definition including all user specified elements within the data
|     definition.
\*--------------------------------------------------------------------------*/
void idd_destroy_data_definition(data_definition)
idd_data_definition data_definition;
{

  set_status(ok);
/*
   Destroy all elements in the hash table.
*/
  for_all_hashtable(data_definition, NULL, destroy_elements);
/*
   Free storage associated with hash table structure.
*/
  free_hashtable((HASH)data_definition, NOT_NODES);
}

/*--------------------------------------------------------------------------*\
|  Routine: idd_get_name
| 
|  Description:  
|     return the name of the input element
\*--------------------------------------------------------------------------*/
string idd_get_name (element)
element_record *element;		/* element in question */
{

  set_status(ok);			/* begin 			    */
/*
  return the element name
*/
  return (element->name);
} 					/* end idd_get_name */

/*--------------------------------------------------------------------------*\
|  Routine: idd_walk_data_definition
| 
|  Description:  
|     call the passed though call-back function for every element of the
|	data definition
\*--------------------------------------------------------------------------*/
void idd_walk_data_definition (data_definition, the_funct, the_thing)
idd_data_definition data_definition;
int (*the_funct) ();
char *the_thing;
{

  set_status(ok);			/* begin 			    */
  DEBUG2(fprintf(stderr, "idd_walk_data_definition: call.\n"));

  for_all_hashtable (data_definition, the_thing, the_funct);

  DEBUG2(fprintf(stderr, "idd_walk_data_definition:  exiting.\n"));

  return;
} 					/* end idd_walk_data_definition */

/*--------------------------------------------------------------------------*\
|  Routine: idd_get_element_number_of_components
| 
|  Description:  
|     return the number of components in an element record
\*--------------------------------------------------------------------------*/
int idd_get_element_number_of_components (element)
element_record *element;		/* element in question */
{

  set_status(ok);			/* begin 			    */
  DEBUG2(fprintf(stderr, "idd_get_element_number_of_components: call.\n"));

/*
  return the element's number of components
*/
  DEBUG2(fprintf(stderr, 
      "idd_get_element_number_of_components:  number of components = %d.\n",
      element->components);
    fprintf(stderr, "idd_get_element_number_of_components:  exiting.\n"));
  return (element->components + 1);
}				/* end idd_get_element_number_of_components */

/*--------------------------------------------------------------------------*\
|  Routine: idd_get_type_string
| 
|  Description:  
|     The idd_get_type_string returns the string corresponding to the
|     specified type.
\*--------------------------------------------------------------------------*/
string idd_get_type_string(data_type)
idd_data_types data_type;
{

  set_status(ok);			/* begin 			    */
  DEBUG2(fprintf(stderr, "idd_get_type_string: call.\n"));
/*
   Return the appropriate string.
*/
  DEBUG2(fprintf(stderr, "idd_get_type_string:  data type = %s.\n",
      idd_data_type_strings[(int)data_type]);
    fprintf(stderr, "idd_get_type_string:  exiting.\n"));
  return(idd_data_type_strings[(int)data_type]);
}				/* end idd_get_type_string */

/*--------------------------------------------------------------------------*\
|  Routine: idd_is_undefined
| 
|  Description:  
|     Returns true if undefined.
\*--------------------------------------------------------------------------*/
boolean idd_is_undefined(op1_type, op1_data)
idd_data_types op1_type;
caddr_t op1_data;
{					/* local type definition	    */
  boolean value;

  set_status (ok);
  DEBUG2(fprintf(stderr, "idd_is_undefined: call.\n");
    fprintf(stderr, "idd_is_undefined: op1_type = %s.\n", 
      idd_get_type_string(op1_type));
    fprintf(stderr, "idd_is_undefined:  op1_data = %x.\n", op1_data));
/* 
   Type determines which version of undefined to compare with.
*/
  switch (op1_type) {			
    case idd_boolean:
      value = (*((boolean *)op1_data) == UNDEFINED_BOOLEAN);
      break;
    case idd_integer:
      value = (*((int*)op1_data) == UNDEFINED_INTEGER);
      break;
    case idd_real:
      value = (*((double *)op1_data) == UNDEFINED_REAL);
      break;
    case idd_string:
      value = (!strcmp((string)op1_data, UNDEFINED_STRING));
      break;
    case idd_record:
      set_status(operation_undefined_types);
      return(value);
    case idd_id:
      value = (*((iid_id_type *)op1_data) == UNDEFINED_ID);
      break;
    case idd_buffer:
      value = ubo_undefined((idd_buffer_type *)op1_data);
      break;
    case idd_undefined:
      value = true;
      break;
    			/* ???? what about default case */
  }

  DEBUG2(fprintf(stderr, "idd_is_undefined: value = %d.\n", value);
    fprintf(stderr, "idd_is_undefined: exiting.\n"));

  return value;
}					/* end idd_is_undefined		    */

/*--------------------------------------------------------------------------*\
|  Routine: idd_set_undefined
| 
|  Description:  
|     Sets the value of op1_data to undefined based on op1_type.
\*--------------------------------------------------------------------------*/
void idd_set_undefined(op1_type, op1_data)
idd_data_types op1_type;
caddr_t op1_data;
{					/* local type definition	    */
  set_status(ok);
  DEBUG2(
    fprintf(stderr, "idd_set_undefined: call.\n");
    fprintf(stderr, "idd_set_undefined: op1_type = %s.\n", 
      idd_get_type_string(op1_type));
    fprintf(stderr, "idd_set_undefined:  op1_data = %x.\n", op1_data);
  );
/* 
**  Type determines which version of undefined to compare with.
*/
  switch (op1_type) {			
    case idd_boolean:
      (*((boolean *)op1_data) = UNDEFINED_BOOLEAN);
      break;
    case idd_integer:
      (*((int*)op1_data) = UNDEFINED_INTEGER);
      break;
    case idd_real:
      (*((double *)op1_data) = UNDEFINED_REAL);
      break;
    case idd_string:
      strcpy((string)op1_data, UNDEFINED_STRING);
      break;
    case idd_record:
      set_status(operation_undefined_types);
      break;
    case idd_id:
      (*((iid_id_type *)op1_data) = UNDEFINED_ID);
      break;
    case idd_buffer:
      ubo_undefined((idd_buffer_type *)op1_data);
      break;
    case idd_undefined:
      break;
    			/* ???? what about default case */
  }

  DEBUG2(
    fprintf(stderr, "idd_set_undefined:  op1_data = %x.\n", op1_data);
    fprintf(stderr, "idd_set_undefined: exiting.\n")
  );

  return;
}				/* end idd_set_undefined		    */
