static char rcsid [] = "$Header: /u/04c/mnt/integ/source/int/src/RCS/ids.c,v 1.11 91/01/02 18:07:51 dvk Exp $";

/*--------------------------------------------------------------------------*
| 
|  Name:  Data Synchronization (ids)
|
|  Description:  
|     The data syncrhonization provides data synchronization 
|     using monitors.
|
|  Component of:
|     Interface Layer.
|
|  Related files:
|     ids.c - contains the implementation for the data synch abstraction.
|     ids.h - contains the external definition for the data synch abstraction.
|     idstest.c - test file.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Robert C. Seacord
|     rcs@sei.cmu.edu  
|
|  Change History:
|     Oct. 19, 1987 : Module created.
|     $Log:	ids.c,v $
 * Revision 1.11  91/01/02  18:07:51  dvk
 * Added error check before atoi of result of getenv
 * 
 * Revision 1.10  90/02/14  21:59:57  ejh
 * fixed include structure
 * 
 * Revision 1.9  90/01/26  16:09:05  rcs
 * nothing much
 * 
 * Revision 1.8  90/01/24  18:40:16  rcs
 * modified semaphore algorithm
 * 
 * Revision 1.7  89/12/27  17:16:48  ejh
 * took out architecture dependencies for sembuf
 * 
 * Revision 1.6  89/02/09  19:01:30  ejh
 * added copyright notice
 * 
 * Revision 1.5  89/01/23  11:32:39  rcs
 * added code to support architectural dependecies on short and int
 * 
 * Revision 1.4  88/11/02  17:41:23  rcs
 * changed short back to int since this did not work on microvax
 * 
 * Revision 1.3  88/11/02  14:23:18  rcs
 * moved debug variable to include file - rCs
 * 
 * Revision 1.2  88/10/29  18:33:49  rcs
 * replace int with short for sun compatability - rCs
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	ids_TOP

#include	<sys/types.h>
#include	<sys/ipc.h>
#include	<sys/sem.h>
#include	<stdio.h>
#include	"global.h"
#include	"isc.h"
#include	"ids.h"

#define DEBUG1(statement) SERPENT_DEBUG1(ids_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(ids_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(ids_debug, statement)

#define SEMAPHORE_KEY IPC_PRIVATE
#define NUMBER_OF_SEMAPHORES 2
#define SEMAPHORE_FLAG 0777

struct sembuf
#define RD_INIT_ELEMENTS 2
    rd_init[RD_INIT_ELEMENTS] = {       /* read initialization semop        */
      { 1, 0, 0},
      { 0, 1, SEM_UNDO}
    },

#define RD_DONE_ELEMENTS 1
    rd_done[RD_DONE_ELEMENTS] = {       /* read done semop                  */
      { 0, -1, SEM_UNDO}
    },

#define WRT_INIT_ELEMENTS 3
    wrt_init[WRT_INIT_ELEMENTS] = {     /* write initialization semop       */
      { 0,  0, 0},
      { 1,  0, 0},
      { 1,  1, SEM_UNDO},
    },

#define WRT_DONE_ELEMENTS 1
    wrt_done[WRT_DONE_ELEMENTS] = {     /* write done semop                 */
      { 1, -1, SEM_UNDO}
    };

static int semaphore_id;

/*--------------------------------------------------------------------------*\
|  Name: ids_create
| 
|  Description:  
|     Create a semaphore set and return the id.
\*--------------------------------------------------------------------------*/
int ids_create()
{

  set_status(ok);

  /*
   *  Create semaphore.
   */
  semaphore_id = semget(SEMAPHORE_KEY, NUMBER_OF_SEMAPHORES, SEMAPHORE_FLAG);
  check_error(semaphore_id,"ids_create error",system_operation_failed);

  return semaphore_id;
}

/*--------------------------------------------------------------------------*\
|  Name: ids_init
| 
|  Description:  
|     Initializes semaphore id for ids instances of child processes.
\*--------------------------------------------------------------------------*/
void ids_init()
{
  char *sem_id;
  extern char *getenv();

  set_status(ok);  		          /* begin                          */
  /*
   *  Initialize semaphore id from environment variable.
   */
  sem_id = getenv("SEM_ID");
  if (sem_id == NULL || strlen (sem_id) == 0) {
    fprintf (stderr, "ids_init: SEM_ID environment variable not found\n\n");
    semaphore_id = 0;
    }
  else
    semaphore_id = atoi(sem_id);
}

/*--------------------------------------------------------------------------*\
|  Name: ids_start_read
| 
|  Description:  
|     Start of read transaction.
\*--------------------------------------------------------------------------*/
void ids_start_read()
{
  int status;

  set_status(ok);  		          /* begin                          */
/*
 *  Perform read init sempahore operation.
 */
  status = semop(semaphore_id, rd_init, RD_INIT_ELEMENTS);
  check_error(status,"ids_start_read error",system_operation_failed);
}

/*--------------------------------------------------------------------------*\
|  Name: ids_end_read
| 
|  Description:  
|     End of read transaction.
\*--------------------------------------------------------------------------*/
void ids_end_read()
{                             		/* local type definitions         */
  int status;

  set_status(ok); 			/* begin                          */
/*
   Perform read done semaphore operation.
*/
  status = semop(semaphore_id, rd_done, RD_DONE_ELEMENTS);
  check_error(status,"ids_end_read error",system_operation_failed);
}

/*--------------------------------------------------------------------------*\
|  Name: ids_start_write
| 
|  Description:  
|     Start of write transaction.
\*--------------------------------------------------------------------------*/
void ids_start_write()
{                                         /* local type definitions         */
  int status;

  set_status(ok);  		          /* begin                          */
/*
   Perform write init semaphore operation.
*/
  status = semop(semaphore_id, wrt_init, WRT_INIT_ELEMENTS);
  check_error(status,"ids_start_write error",system_operation_failed);
}

/*--------------------------------------------------------------------------*\
|  Name: ids_end_write
| 
|  Description:  
|     End of write transaction.
\*--------------------------------------------------------------------------*/
void ids_end_write()
{                                         /* local type definitions         */
  int status;

  set_status(ok);  		          /* begin                          */
/*
   Perform write done semaphore operation.
*/
  status = semop(semaphore_id, wrt_done, WRT_DONE_ELEMENTS);
  check_error(status,"ids_end_write error",system_operation_failed);
}

/*--------------------------------------------------------------------------*\
|  Name: ids_cleanup
| 
|  Description:  
|     Perform necessary initialization for the data synch abstraction.
\*--------------------------------------------------------------------------*/
void ids_cleanup()
{                                         /* local type definitions         */
  set_status(ok);
}

/*--------------------------------------------------------------------------*\
|  Name: ids_destroy
| 
|  Description:  
|    Destroys semaphore id set.
\*--------------------------------------------------------------------------*/
void ids_destroy()
{                                         /* local type definitions         */
  int status;

  set_status(ok);  		          /* begin                          */
/*
   Remove semaphore id.
*/
  status = semctl(semaphore_id, 0, IPC_RMID, 0);
  check_error(status,"ids_destroy error",system_operation_failed);
}
