static char rcsid [] = "$Header: /u/04c/mnt/integ/source/int/src/RCS/isd1.c,v 1.34 1991/04/19 19:19:45 ejh Exp $";

/*--------------------------------------------------------------------------*\
| 
|  Name:  Shared data abstraction (isd)
|
|  Description:  
|     The shared data abstraction provides a means of communicating data
|     between the various processes of the SERPENT system. 
|
|  Component of:
|     Interface Layer.
|
|  Related files:
|     isd.c - contains the implementation for the shared data abstraction.
|     isd.h - contains the external definition for the shared data abstr.
|     isdtest.c - test file.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Robert C. Seacord
|     rcs@sei.cmu.edu  
|
|  Change History:
|     Nov. 19, 1987 : Module created.
|     Apr. 13, 1988 : Module modified to have separate 
|                     init_process_record procedure and call journaling  ljb
|     Sep. 26, 1988 : Fixed bug in free_element_change_record when deleting
|		      buffers of zero length. - rCs
|     $Log: isd1.c,v $
 * Revision 1.34  1991/04/19  19:19:45  ejh
 * went back to revision 1.32
 *
 * Revision 1.32  1991/03/13  11:11:47  dvk
 * More speedups by eliminating redundant assignment in add_data_to_ecr
 *
 * Revision 1.31  91/03/12  11:26:49  dvk
 * Fixed bug in new scan routine that was tickled by scpp (which converts
 * strings of tabs and spaces into a single space).
 * 
 * Revision 1.30  91/03/11  16:49:08  dvk
 * Changed idd_create_component_list to idd_create_component_alist
 * 
 * Revision 1.29  91/02/14  11:45:12  rcs
 * removed some unused variables
 * 
 * Revision 1.28  91/02/13  19:30:07  rcs
 * fixed over zealous intialization of change bits
 * 
 * Revision 1.27  91/02/12  15:17:20  little
 * change isd_init_sd_element to take a fourth parameter which is the
 * element_change info.  if it is non-null, isd_init_sd_element will set
 * the changed bit for all components in the element is is setting to
 * undefined.  cahnges all calls to isd_init_sd_element to take the new
 * parameter into account.
 * 
 * Revision 1.26  91/01/02  18:15:51  dvk
 * Un-hardwired "DM_BOX" to global #defined constant.  Cleaned up considerably
 * after some types were no longer declared private.
 * 
 * Revision 1.25  90/06/25  16:36:19  rcs
 * changed open ill file routine to use record type as indicator 
 * of new element rather than element name being equal to 
 * the component name
 * 
 * Revision 1.24  90/03/01  15:13:53  rcs
 * modified put_shared_data to ignore things that have been
 * removed
 * 
 * Revision 1.23  90/02/14  22:01:51  ejh
 * fixed include structure
 * 
 * Revision 1.22  89/11/22  13:00:21  rcs
 * no change i guess
 * 
 * Revision 1.21  89/11/10  15:16:54  rcs
 * made changes for brian and maybe something else
 * 
 * Revision 1.20  89/11/06  15:05:27  bmc
 * Added isd_read_ill_prologue routine and its necessary minions
 * 
 * Revision 1.19  89/10/13  10:51:00  rcs
 * fixed memory leak
 * 
 * Revision 1.18  89/09/29  16:38:56  ljb
 * make new version of add_shared_data which takes id as input
 * 
 * Revision 1.17  89/09/29  14:48:22  rcs
 * added support for typed buffers
 * 
 * Revision 1.16  89/09/20  16:38:35  ljb
 * record transaction prior to notifying other process of it
 * this is because other process was changing hash table which
 * recording was recording
 * 
 * Revision 1.15  89/02/26  15:10:55  little
 * change SERPENT_ILL_PATH to be SERPENT_DATA_PATH
 * 
 * Revision 1.14  89/02/16  23:47:51  ejh
 * empty transactions are no longer recorded
 * 
 * Revision 1.13  89/02/15  14:48:56  rcs
 * modified delete buffer to not free memory for null body pointers,
 * as opposed to buffers of length zero
 * 
 * Revision 1.12  89/02/09  19:07:42  ejh
 * added copyright notice
 * 
 * Revision 1.11  89/01/26  16:31:05  ejh
 * added code to handle SERPENT_ILL_PATH to search for ill files
 * 
 * Revision 1.10  88/12/08  15:11:27  rcs
 * fixed problem with everything being undefined
 * 
 * Revision 1.9  88/12/06  17:48:54  rcs
 * fixed bug setting strings undefined
 * 
 * Revision 1.8  88/12/06  15:43:43  rcs
 * added isd_init_sd_element routine
 * 
 * Revision 1.7  88/11/02  16:50:54  rcs
 * *** empty log message ***
 * 
 * Revision 1.6  88/11/02  14:45:53  rcs
 * moved debug variable to interface file - rCs
 * 
 * Revision 1.5  88/10/13  15:06:29  rcs
 * added 1 to number of components for call to ubs_create_bitstring
 * 
 * 
 * Revision 1.4  88/10/03  08:09:11  little
 * add isd_dm_init_sdd routine and change isd_dm_init to only initialize
 * the interface and not create a process structure for the dm. now
 * the isd_dm_int_sdd creates the process structure and, if a dm sdd is
 * present, adds the sdd to the process structure
 * 
 * Revision 1.3  88/09/29  14:47:01  rcs
 * Sep. 29, 1988 : Fixed isd_put_shared data to handle 
 *                 buffers and scalars.
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	isd_TOP
#define	memoryPack

#include	<sys/types.h>
#include	<stdio.h>
#include	<string.h>
#include	<global.h>
#include	<hashPack.h>
#include	<ubo.h>
#include	<ubs.h>
#include	<ufa.h>
#include	<ush.h>
#include	<u22.h>
#include	<isc.h>
#include	<ipc.h>
#include	<iid.h>
#include	<ids.h>
#include	<ism.h>
#include	<idd.h>
#include	<isd.h>
#include	<isd_priv.h>
#include	<ipb.h>

#define DEBUG1(statement) SERPENT_DEBUG1(isd_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(isd_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(isd_debug, statement)

#define PROCESS_RECORD_HASH_SIZE  17  

process_record in_box;		/* attached process mailbox & data def	    */
isd_process out_box;		/* dm process mailbox & attached	    */

HASH isd_process_table;		/* list of process records indexed by mbx   */
typedef struct {
  ipc_mailbox hash_mbx_number;
  process_record *hash_process_record;
} process_table_entry;

/*--------------------------------------------------------------------------*\
|  Routine: isd_addto_process_table
| 
|  Description:  
|     The add to process table routine adds a new entry to the process
|     table
\*-------------------------------------------------------------------------*/ 
void isd_addto_process_table (process_box)
process_record *process_box;
{
process_table_entry *temp;

   temp = (process_table_entry *)make_node(sizeof(process_table_entry));
   temp->hash_mbx_number = process_box->mailbox;
   temp->hash_process_record = process_box;
   add_to_hashtable (isd_process_table, temp, process_box->mailbox);
   return;
}

/*--------------------------------------------------------------------------*\
|  Routine: isd_getfrom_process_table
| 
|  Description:  
|     The get from process table routine retrieves an entry to the process
|     table or NULL
\*-------------------------------------------------------------------------*/ 
isd_process isd_getfrom_process_table (mbx)
ipc_mailbox mbx;
{
process_table_entry *process_entry;

   process_entry = (process_table_entry *)get_from_hashtable
	(isd_process_table, mbx);
   if (process_entry == NULL)
	return NULL;
   return (isd_process)(process_entry->hash_process_record);
}

/*--------------------------------------------------------------------------*\
|  Routine: isd_deletefrom_process_table
| 
|  Description:  
|     The delete from process table routine deletes an entry from the process
|     table
\*-------------------------------------------------------------------------*/ 
void isd_deletefrom_process_table (process)
process_record *process;
{
   delete_from_hashtable (isd_process_table, process->mailbox);
   return;
}

/*--------------------------------------------------------------------------*\
|  Routine: isd_cleanup_process_table
| 
|  Description:  
|     The clean up process table routine deletes the process table
\*-------------------------------------------------------------------------*/ 
void isd_cleanup_process_table ()
{
   free_hashtable (isd_process_table, AND_NODES);
}
/*--------------------------------------------------------------------------*\
|  Routine: isd_process_cmp
| 
|  Description:  
|     The init process compare routine returns true or false depending upon
|     whether the entry matches the comparision value
\*-------------------------------------------------------------------------*/ 
boolean isd_process_cmp (test_mailbox, process_entry)
ipc_mailbox test_mailbox;
process_table_entry *process_entry;
{
   return test_mailbox == process_entry->hash_mbx_number;
}
/*--------------------------------------------------------------------------*\
|  Routine: isd_process_hash
| 
|  Description:  
|     The process hash routine returns the hash number for the specified arg
\*-------------------------------------------------------------------------*/ 
int isd_process_hash(mbx_number)
ipc_mailbox mbx_number;
{
   return ((((int) mbx_number) & 077777) % PROCESS_RECORD_HASH_SIZE);
}
/*--------------------------------------------------------------------------*\
|  Routine: isd_init_process_table
| 
|  Description:  
|     The init process table routine initializes the hash table used
|     to maintain the mailbox number, process record pairs
\*-------------------------------------------------------------------------*/ 
void isd_init_process_table()
{
   isd_process_table = make_hashtable (PROCESS_RECORD_HASH_SIZE,
        isd_process_hash, isd_process_cmp);
}
/*--------------------------------------------------------------------------*\
|  Routine: isd_init_once
| 
|  Description:  
|     The init once routine initializes all the stuff that only needs to be
|     initialized once.  This is called directly by the dialogue manager
|     and indirectly by the taps.
\*-------------------------------------------------------------------------*/ 
void isd_init_once()
{
  set_status(ok);                       /* begin                            */
/*
   Initialize id counter.
*/
  iid_init();
  check_status("isd_init_once: status from idd_init.");
/*
   Initialize semaphore id.
*/
  ids_init();
  check_status("isd_init_once: status from ids_init.");
/*
   Initialize shared memory.
*/
  ism_init();
  check_status("isd_init_once: status from ism_init.");
/*
  initialize process table
*/
   isd_init_process_table();
}                                       /* end isd_init_once		    */

/*--------------------------------------------------------------------------*\
|  Routine: isd_init_process_record
| 
|  Description:  
|     The initialize process record routine fills in a structure which defines
|      all of the elements of a process record EXCEPT the pointer to the
|      data definition.  It is the responsibility of the calling routine
|      to fill in the data definition field
|     
|     This routine is called directly by from within the isd data abstraction
\*-------------------------------------------------------------------------*/
void isd_init_process_record(process, mailbox, ill_file)
process_record *process;
string mailbox;
string ill_file;
{
/*
   Validate parameters.
*/
  check_null(mailbox, "isd_init_process_record: null parameter 'mailbox'.\n",
    null_mailbox_name);

  check_null(ill_file,"isd_init_process_record: null parameter 'ill_file'.\n",
    null_ill_file_name);
/*
   set validation stamp.
*/
  process->validation_stamp = isc_process_record_vs;
/*
   Allocate storage for ill file and mailbox names and copy into process
   record.
*/
  process->ill_file_name = u22_copy_string (ill_file);
  process->mailbox_name = u22_copy_string (mailbox);
/*
   Initialize record_fp and recording fields.
*/
  process->record_fp = NULL;
  process->recording = false;
/*
   Create the mailbox.
*/
  process->mailbox = ipc_init(mailbox);
  check_status("isd_init_process_record: status from ipc_init.");
/*
  add to process table
*/
  isd_addto_process_table(process);
}

/*--------------------------------------------------------------------------*\
|  Local Routine: isd_fix_ill_string
| 
|  Description:  
|     Fixes a string read from the ILL file.
\*-------------------------------------------------------------------------*/ 
static void isd_fix_ill_string (the_string)
char *the_string;
{
   for (;
       ((*the_string != '\n') &&
        (*the_string != '\0') &&
        (*the_string != '\f'));
        the_string++)
      ;

   if (*the_string != '\0')
     *the_string = '\0';
}

/*--------------------------------------------------------------------------*\
|  Local Routine: isd_read_ill_line
| 
|  Description:  
|     Reads the a single line from the ILL file prologue into an allocated
|     buffer.
\*-------------------------------------------------------------------------*/ 
static char *isd_read_ill_line (file_ptr)
FILE *file_ptr;
{
#  define ILL_LINE_LENGTH	(127)	/* arbitrary max line length */

   char *line_in;

   if ( (line_in = make_node (ILL_LINE_LENGTH)) == NULL)
      {
      set_status (out_of_memory);
      DEBUG1(isc_print_status("isd_get_data_definition:"));
      }

   else
      {
      (void) fgets (line_in, ILL_LINE_LENGTH, file_ptr);
      isd_fix_ill_string (line_in);
      }

   return (line_in);
}
/*--------------------------------------------------------------------------*\
|  Routine: isd_read_ill_prologue
| 
|  Description:  
|     Reads the prologue portion of an ILL file and returns the decoded
|     contents.
\*-------------------------------------------------------------------------*/ 
isd_ill_prologue isd_read_ill_prologue	(file_ptr)
FILE *file_ptr;		/* pointer to a currently open ill file */
{
   isd_ill_prologue prologue;

   set_status (ok);

   prologue.generic_magic_id  = NULL;
   prologue.specific_magic_id = NULL;
   prologue.tap_name          = NULL;
   prologue.ill_file_name     = NULL;
   prologue.mailbox_name      = NULL;
   prologue.execution_name    = NULL;
   prologue.execution_command = NULL;

   prologue.generic_magic_id = isd_read_ill_line (file_ptr);
   prologue.specific_magic_id = isd_read_ill_line (file_ptr);
   prologue.tap_name = isd_read_ill_line (file_ptr);
   prologue.ill_file_name = isd_read_ill_line (file_ptr);
   prologue.mailbox_name = isd_read_ill_line (file_ptr);
   prologue.execution_name = isd_read_ill_line (file_ptr);
   prologue.execution_command = isd_read_ill_line (file_ptr);

   return (prologue);
}

/*--------------------------------------------------------------------------*\
|  Routine: isd_input_open_ill_file
| 
|  Description:  
|     The input open ill file routine takes as input an empty data definition
|     and a fill open to an ill fill, reads the ill file in and creates the
|     data definition from it.
\*-------------------------------------------------------------------------*/ 
void isd_input_open_ill_file(data_definition, fp)
idd_data_definition data_definition;
FILE *fp;
{
#define MAX_STR 256
idd_element element;
char *element_name, *component_name, dummy[MAX_STR];
int offset, length;
idd_data_types type;
char *cp;
/*
   While file not empty create the appropriate data definitions.
*/
  while (1) {
    fgets (dummy, MAX_STR, fp);
    if (dummy[0] == '\n')
	continue;
    if (strcmp (dummy, ">>\n") == 0)
	break;
    element_name = strtok (dummy, " \t");
    component_name = strtok (NULL, " \t");
    offset = atoi (strtok (NULL, " \t"));
    length = atoi (strtok (NULL, " \t"));
    type = (idd_data_types)atoi (strtok (NULL, "\n"));

    if (type == idd_record) {	/* element */
      element = idd_create_element(data_definition, element_name, type,
        length);
      check_status("isd_open_input_file: status from idd_create_element.");
      }
    else {			/* component */
      idd_create_component(element, component_name, type, offset, length);
      check_status(
        "isd_create_process_def: status from idd_create_component.");
      }
  }
  ungetc ('\n', fp);
}
/*--------------------------------------------------------------------------*\
|  Routine: isd_input_ill_file
| 
|  Description:  
|     The input ill file routine takes as input an empty data definition
|     and an ill file name, reads the ill file in and creates the
|     data definition from it.
\*-------------------------------------------------------------------------*/ 
void isd_input_ill_file(data_definition, ill_file)
idd_data_definition data_definition;
string ill_file;
{
FILE *fp;
char check_string[255];
string	full_file_name,
	env_value,
	getenv ();
boolean	found_env = TRUE;

/*
   Open the ill file.
*/

  env_value = u22_copy_string (getenv (DATA_PATH_ENV_VAR));
  if (!strlen (env_value))
	{
	found_env = FALSE;
	env_value [0] = '.';
	env_value [1] = '\0';
	}
  full_file_name = ufa_full_file_name (env_value, ill_file);
/*
  check_null (full_file_name,
	("no %s in SERPENT_DATA_PATH or current directory", ill_file),
	null_ill_file_name);
*/
  if (!full_file_name)
	{
	DEBUG1 (if (found_env)
		  fprintf (stderr, "no %s in SERPENT_DATA_PATH\n", ill_file);
		else
		  fprintf (stderr, "no %s in current directory\n", ill_file));
	set_status (null_ill_file_name);
	return;
	}
  fp = fopen (full_file_name, "r");
  free_node (full_file_name);
  free_node (env_value);
  check_null(fp, "isd_input_ill_file: could not open ill file.\n",
    null_ill_file_name);
/*
  skip to correct position on ill file
*/
  fscanf(fp, "%s", check_string);
  while ( strcmp(check_string,"|||") != 0)   fscanf(fp, "%s", check_string);

  isd_input_open_ill_file(data_definition, fp);

/*
   Close the ill file.
*/
  fclose(fp);
}
/*--------------------------------------------------------------------------*\
|  Routine: isd_create_process_definition
| 
|  Description:  
|     The create process definition routine creates a structure which defines
|      both the mailbox id and the shared data definition for a process. 
|     
|     This routine is called directly by the dialogue manager and indirectly
|     by the application.
\*-------------------------------------------------------------------------*/ 
isd_process isd_create_process_definition(mailbox, ill_file)
string mailbox;
string ill_file;
{                                       /* local variable definitions       */
process_record *process;

  set_status(ok);                       /* begin                            */
/*
   Validate parameters.
*/
  rcheck_null(mailbox, 
    "isd_create_process_definition: null parameter 'mailbox'.\n",
    null_mailbox_name, NULL);

  rcheck_null(ill_file, 
    "isd_create_process_definition: null parameter 'ill_file'.\n",
    null_ill_file_name, NULL);
/*
   Create a process record and set validation stamp.
*/
  process = (process_record *)make_node(sizeof(process_record));
  rcheck_null(process, "isd_create_process_definition: out_of_memory", 
    out_of_memory, NULL);
/*
   Fill in the process record
*/
  isd_init_process_record(process, mailbox, ill_file);
  rcheck_status(
    "isd_create_process_definition: status from isd_init_process_record.",
    NULL);
/*
   Create the data definition.
*/
  process->data_definition = idd_create_data_definition();
  rcheck_status(
    "isd_create_process_definition: status from idd_create_data_def.", NULL);

  isd_input_ill_file(process->data_definition, ill_file);
  rcheck_status("isd_create_process_definition: status from isd_input_ill_file",
  NULL);

  return (isd_process)process;
}                                       /* end isd_create_process_def       */

/*--------------------------------------------------------------------------*\
|  Routine: isd_get_mailbox
| 
|  Description:  
|     The get mailbox routine returns the mailbox associated with a given
|     process.
\*-------------------------------------------------------------------------*/ 
ipc_mailbox isd_get_mailbox(process)
isd_process process;
{
  set_status(ok);                       /* begin                            */
/*
   Validate parameter.
*/
  if (!isc_validate(process, isc_process_record_vs)) { 
					/* invalid transaction handle	    */
    set_status(invalid_process_handle);
    DEBUG1(isc_print_status("isd_get_mailbox:"));
    return NULL;
  }					/* end if invalid transaction hand  */

  return process->mailbox;
}                                       /* end isd_get_mailbox		    */

/*--------------------------------------------------------------------------*\
|  Routine: isd_get_data_definition
| 
|  Description:  
|     The get data definition routine returns the data definition associated
|     with a given process.
\*-------------------------------------------------------------------------*/ 
idd_data_definition isd_get_data_definition(process)
isd_process process;
{
  set_status(ok);                       /* begin                            */
/*
   Validate parameter.
*/
  if (!isc_validate(process, isc_process_record_vs)) { 
					/* invalid transaction handle	    */
    set_status(invalid_process_handle);
    DEBUG1(isc_print_status("isd_get_data_definition:"));
    return NULL;
  }					/* end if invalid transaction hand  */

  return process->data_definition;
}                                       /* end isd_get_data_definition	    */

/*--------------------------------------------------------------------------*\
|  Routine: isd_destroy_process_definition
| 
|  Description:  
|     The destroy process definition frees all storage associated 
|     with a process definition record.
\*-------------------------------------------------------------------------*/ 
void isd_destroy_process_definition(process)
isd_process process;
{
  set_status(ok);                       /* begin                            */
/*
   Validate parameter.
*/
  if (!isc_validate(process, isc_process_record_vs)) { 
					/* invalid transaction handle	    */
    set_status(invalid_process_handle);
    DEBUG1(isc_print_status("isd_destroy_process_definition:"));
    return;
  }					/* end if invalid transaction hand  */

   isd_deletefrom_process_table (process);
/*
   Destroy the data definition.
*/
  idd_destroy_data_definition(process->data_definition);
  check_status("isd_destroy_proc_def: status from idd_destry_data_def.");
/*
   Destroy the process record.
*/
  free_node(process);
}                                       /* end isd_destroy_process_def      */
 
/*--------------------------------------------------------------------------*\
|  Routine: isd_init
| 
|  Description:  
|     The init routine performs necessary initialization of the interface
|     layer.  
\*-------------------------------------------------------------------------*/ 
void isd_init(mailbox, ill_file)
string mailbox;
string ill_file;
{

  set_status(ok);                       /* begin                            */
/*
   Validate parameters.
*/
  check_null(mailbox, 
    "isd_create_process_definition: null parameter 'mailbox'.\n",
    null_mailbox_name);

  check_null(ill_file, 
    "isd_create_process_definition: null parameter 'ill_file'.\n",
    null_ill_file_name);
/*
   Peform "one time" initialization code.
*/
  isd_init_once();
  check_status("ids_init: status from isd_init.");
/*
   Define an "out_box" for the tap which contains the mailbox of the 
   dialogue manager and the shared data definition for the tap.
*/
  out_box = isd_create_process_definition(DM_BOX, ill_file);

  check_status("ids_init: status from isd_create_process_definition.");
/*
   Fill in the process record for in_box
*/
  isd_init_process_record (&in_box, mailbox, ill_file);
  check_status( "isd_init: status from isd_init_process_record.");

  in_box.data_definition = out_box->data_definition;
}                                       /* end isd_init			    */

/*--------------------------------------------------------------------------*\
|  Routine: isd_dm_init
| 
|  Description:  
|     The init routine performs necessary initialization of the interface
|     layer for the dialogue manager.
\*-------------------------------------------------------------------------*/ 
void isd_dm_init ()
{
   set_status(ok);                       /* begin                           */

				/* Peform "one time" initialization code.  */
   isd_init_once ();
   check_status ("isd_dm_init: status from isd_init_once.");
}

/*--------------------------------------------------------------------------*\
|  Routine: isd_dm_init_sdd
| 
|  Description:
|     This routine performs necessary initialization of the shared data
|       data description structure from an ill for the dialogue manager.
\*-------------------------------------------------------------------------*/ 
void isd_dm_init_sdd (ill_file)
string ill_file;        
{
  set_status(ok);                       /* begin                            */

   if (ill_file)		/* there was a dm ill file present, define the
				   in_box and the shared data definition for
				   the dialogue manager.  */
      {isd_init_process_record (&in_box, DM_BOX, ill_file);
      check_status (
       "isd_dm_init_sdd: bad status from isd_init_process_record w/ill file");
				/* Create the data definition.  */
      in_box.data_definition = idd_create_data_definition ();
      check_status ("isd_dm_init_sdd: bad status from idd_create_data_def.");
				/* read the ill file and place in internal
				   structure */
      isd_input_ill_file (in_box.data_definition, ill_file);
      check_status ("isd_dm_init_sdd: bad status from isd_input_ill_file");
      }
    else			/* there is no a dm ill file, define the in_box
				   and a dummy shared data definition for the
				   dialogue manager.  */
      {isd_init_process_record (&in_box, DM_BOX, "");
      check_status (
	"isd_dm_init_sdd: status from isd_init_process_record w/o ill file");
      in_box.data_definition = NULL;
      }
}                                       /* end isd_dm_init_sdd		    */

/*--------------------------------------------------------------------------*\
|  Routine: match_id
| 
|  Description:  
|     Routine to compare the identifiers with user elements.
|   
\*--------------------------------------------------------------------------*/
int match_id(id, element_change)
iid_id_type id;
element_change_record *element_change;
{ 					/* local type definitions	    */
  set_status(ok);			/* begin 			    */

  return (element_change->id == id);
} 					/* end match_id			    */

/*--------------------------------------------------------------------------*\
|  Routine: hash_id
| 
|  Description:  
|     Internal function which will convert an id into indices
|     in the array of hash lists.
|   
\*--------------------------------------------------------------------------*/
int hash_id(id)
iid_id_type id;
{
  set_status(ok);			/* begin 			    */
/*
   Return a value in the right range.
*/
  return((int)id % MAX_HASH);
} 					/* end hash_id			    */

/*--------------------------------------------------------------------------*\
|  Routine: isd_start_dm_transaction
| 
|  Description:  
|     The start dm transaction routine is used to define the start of a series
|     of data modifications that must be performed as a group.
\*--------------------------------------------------------------------------*/
isd_trans isd_start_dm_transaction(receiver)
isd_process receiver;
{                                       /* local variable definitions       */
transaction_record *transaction;

  set_status(ok);                       /* begin                            */
/*
   Validate parameter.
*/
  if (!isc_validate(receiver, isc_process_record_vs)) { 
					/* invalid transaction handle	    */
    set_status(invalid_process_handle);
    DEBUG1(isc_print_status("isd_start_dm_transaction:"));
    return NULL;
  }					/* end if invalid transaction hand  */
/*
   Create a transaction record in shared memory and set validation stamp.
*/
  transaction = (transaction_record *)make_node(sizeof(transaction_record));
  rcheck_null(transaction, "isd_start_dm_transaction: out of memory.\n",
    out_of_memory, NULL);
  transaction->validation_stamp = isc_transaction_record_vs;
/*
   Set up intended receiver.  The receiver record contians both the mailbox
   id and the handle to the associated data definition.
*/
  transaction->receiver = receiver;
/*
   Set up change table.
*/
  transaction->change_table = ush_make_shm_hashtable(MAX_HASH, hash_id,
    match_id);
  rcheck_null(transaction, 
    "isd_start_dm_transaction:  ush_make_shm_hashtable failed.\n",
    out_of_memory, NULL);

  return (isd_trans)transaction;
}                                       /* end isd_start_dm_transaction     */

/*--------------------------------------------------------------------------*\
|  Routine: isd_start_transaction
| 
|  Description:  
|     The start transaction routine is used to define the start of a series
|     of data modifications that must be performed as a group.
\*--------------------------------------------------------------------------*/
isd_trans isd_start_transaction()
{                                       /* local variable definitions       */
isd_trans transaction;

  set_status(ok);                       /* begin                            */
/*
   Call start transaction to create transaction record etc.
*/
  transaction = isd_start_dm_transaction(out_box);
  rcheck_status("isd_start_trans:  status from isd_start_dm_trans.", NULL);

  return transaction;
}                                       /* end isd_start_transaction	    */

/*--------------------------------------------------------------------------*\
|  Routine: create_element_change_record
| 
|  Description:  
|     Internal routine to create an element change record set the element
|     name field.
\*--------------------------------------------------------------------------*/
element_change_record *create_element_change_record(transaction, element_name)
isd_trans transaction;
string element_name;
{                                       /* local variable definitions       */
element_change_record *element_change;

/*
   Validate parameter.
*/
  if (!isc_validate(transaction, isc_transaction_record_vs)) { 
					/* invalid transaction handle	    */
    set_status(invalid_transaction_handle);
    DEBUG1(isc_print_status("isd_create_element_change_record:"));
    return NULL;
  }					/* end if invalid transaction hand  */
/*
   Allocate storage for change record in shared memory.
*/
  element_change = (element_change_record *)ism_get_shared_memory(
    sizeof(element_change_record));
  rcheck_status("create_element_change_record: status from ism_get_shm.",
	NULL);
/*
   Allocate shared memory storage for element name and copy in.
*/
  element_change->element_name = ism_get_shared_memory
	(strlen(element_name)+1);

  strcpy(element_change->element_name, element_name);

  return element_change;
}                                       /* end create_element_change_record */

/*--------------------------------------------------------------------------*\
|  Routine: set_undefined
| 
|  Description:  
|     The set_undefined routine sets a shared data component or scalar
|     element to undefined.
\*--------------------------------------------------------------------------*/
void set_undefined(data_type, data, offset)
idd_data_types data_type;
caddr_t data;
int offset;
{                                       /* local variable definitions       */
  int undef;
  double undef_real;

/*
   Set scalar element or component to UNDEFINED.
*/
  switch (data_type) {			/* switch on data type		    */

    case idd_boolean:
      undef = UNDEFINED_BOOLEAN;
      ism_copy_into_shared_memory(data + offset, sizeof(UNDEFINED_BOOLEAN), 
        &undef);
      check_status("set_undefined: status from ism_copy_into_shm.");
      break;

    case idd_integer:
      undef = UNDEFINED_INTEGER;
      ism_copy_into_shared_memory(data + offset, sizeof(UNDEFINED_INTEGER), 
        &undef);
      check_status("set_undefined: status from ism_copy_into_shm.");
      break;

    case idd_real:
      undef_real = UNDEFINED_REAL;
      ism_copy_into_shared_memory(data + offset, sizeof(UNDEFINED_REAL), 
        &undef_real);
      check_status("set_undefined: status from ism_copy_into_shm.");
      break;

    case idd_id:
      undef = UNDEFINED_ID;
      ism_copy_into_shared_memory(data + offset, sizeof(UNDEFINED_ID), 
        &undef);
      check_status("set_undefined: status from ism_copy_into_shm.");
      break;

    case idd_string:
      ism_copy_into_shared_memory(data + offset, sizeof(UNDEFINED_STRING), 
        UNDEFINED_STRING);
      check_status("set_undefined: status from ism_copy_into_shm.");
      break;

    case idd_buffer:
      ubo_init_buffer(*((idd_buffer_type *)(data + offset)));
      break;

    default:
      DEBUG1(fprintf(stderr, "set_undefined: unknown component type.\n"));
      set_status(invalid_data_type);
      return;
      
  }
}					/* end set_undefined		    */

/*--------------------------------------------------------------------------*\
|  Routine: isd_init_sd_element
| 
|  Description:  
|     The isd_init_sd_element routine is used to set an existing shared
|     data element to UNDEFINED.
\*--------------------------------------------------------------------------*/
void isd_init_sd_element(data_definition, element_name, data,
			        element_change, set_changed_bits)
idd_data_definition data_definition;
string element_name;
caddr_t data;
element_change_record *element_change;
boolean set_changed_bits;
{                                       /* local variable definitions       */
  idd_data_types data_type;
  int length, offset, position, components;
  ALIST component_alist;
  string component_i_name;

/*
   Get the number of components in the shared data element.
*/
  components = idd_get_number_of_components(data_definition, element_name);
  check_status("isd_init_sd_elem: status from idd_get_#_components.");
/*
   Initialize the element to undefined.
*/
/*
   If element is a scalar set the offset to 0, get the data type of the 
   element and call set_undefined to set the value to undefined.
*/
  if (components == -1) {		/* element is a scalar		    */

    offset = 0;

    data_type = idd_get_shared_data_type(data_definition, element_name, NULL);
    check_status("isd_init_sd_elem: status from idd_get_shrd_data_type.");

    set_undefined(data_type, data, offset);
    check_status("isd_init_sd_elem: status from set_undefined.");
/*
**  If the set changed bits flag is set go ahead and set them.
*/
    if (set_changed_bits) {
      ubs_set_bit(element_change->changed_components, position);
      check_status("isd_init_sd_elem: bad status from ubs_set_bit.");
    }

  }					/* end if element is a scalar	    */
/*
   Else if element is a recoord loop through list of components and
   assign each to UNDEFINED.
*/
  else {				/* element is a record		    */

    component_alist = idd_create_component_alist(data_definition, element_name);
    check_status("isd_init_sd_elem: status from idd_create_comp_alist.");

    loop_through_alist(component_alist, component_i_name, string) {
/*
**  Get the length, offset and position and data type of the component.
*/
      data_type = idd_get_component_lop(data_definition, element_name,
	component_i_name, &length, &offset, &position);
      check_status("isd_init_sd_elem: status from idd_get_lngthNoff.");

      set_undefined(data_type, data, offset);
      check_status("isd_init_sd_elem: status from set_undefined.");
 
    }					/* end loop_through_alist	    */

    idd_destroy_component_alist(component_alist);
    check_status("isd_init_sd_elem: status from idd_destroy_comp_alist.");
  }					/* end if component specified	    */
}                                       /* end isd_init_sd_elem 	    */

/*--------------------------------------------------------------------------*\
|  Routine: copy_component
| 
|  Description:  
|     The copy component routine examines a component to determine if it
|     is a buffer.  If so, shared memory space is allocated of buffer 
|     length and user data copied into the buffer.
\*--------------------------------------------------------------------------*/
static void copy_component(data_definition, element_change, data, component_name)
idd_data_definition data_definition;
caddr_t data;
element_change_record *element_change;
string component_name;
{                                       /* local variable definitions       */
  idd_data_types data_type;
  int length, offset, position;
  idd_buffer_type *user_buffer;
  idd_buffer_type *shm_buffer;

/*
   Get the length, offset and position and data type of the component.
*/
  data_type = idd_get_component_lop(data_definition,
    element_change->element_name, component_name, &length, &offset, &position);
  check_status("copy_component: status from idd_get_lngthNoff.");
/*
   Set appropriate change bit in changed component bitstring.
*/
  ubs_set_bit(element_change->changed_components, position);
  check_status("copy_component: status from ubs_set_bit.");
/*
   If buffer handle special case.
*/
  if (data_type == idd_buffer) {	/* buffer type			    */

/*
   Free any existing buffer body regardless of if there is new data or not.
*/
    shm_buffer = (idd_buffer_type *)((char *)element_change->data + offset);

    if (shm_buffer->body != NULL) {	/* if existing buffer body	    */
      ism_free_shared_memory(shm_buffer->body);
      check_status("copy_component: status from ism_free_shared_memory.");
    }					/* end if existing buffer body	    */
/*
   Copy buffer into shared memory if buffer length is non zero.
*/
    user_buffer = (idd_buffer_type *)((char *)data + offset);

    shm_buffer->length = user_buffer->length;
    shm_buffer->type = user_buffer->type;
    shm_buffer->body = NULL;
/*
   If buffer defined and of non-zero length allocate storage for buff1 body.
*/
    if ((user_buffer->length != UNDEFINED_BUFFER_LENGTH) && /* buff defined */
        (user_buffer->length != 0)) {           /* and of non-zero length   */

      shm_buffer->body = ism_get_shared_memory(user_buffer->length);
      check_status("copy_component: status from ism_get_shared_memory.");

      ism_copy_into_shared_memory(shm_buffer->body, user_buffer->length, 
        user_buffer->body);
      check_status("copy_component: status from ism_copy_into_shm.");
    }	 		/* end if buffer defined and of of non-zero length  */

  }					/* end if buffer type		    */
  else {				/* not buffer type		    */
/*
   Copy user data into shared memory.
*/
    ism_copy_into_shared_memory((element_change->data)+offset, length, 
      data + offset);
    check_status("copy_component: status from ism_copy_into_shm.");
  }
}					/* end copy component		    */

/*--------------------------------------------------------------------------*\
|  Routine: add_data_to_ecr
| 
|  Description:  
|     The add data to ecr routine adds data to the element change record.
|     If existing_ecr is false an ecr and associated structures are created.
\*--------------------------------------------------------------------------*/
void add_data_to_ecr(transaction, element_change, component_name, 
                     existing_ecr, data, set_changed_bits)
isd_trans transaction;
element_change_record *element_change;
string component_name;
boolean existing_ecr, set_changed_bits;
caddr_t data;
{                                       /* local variable definitions       */
  idd_data_types data_type;
  idd_buffer_type *user_buffer;
  idd_buffer_type *shm_buffer;
  int length, offset, position, components;
  ALIST component_alist;
  string component_i_name;

/* 
   Determine size of shared memory required to maintain an instance of
   the shared data element and allocate the memory if ecr doesn't already
   exist.
*/
  length = idd_get_length(
    transaction->receiver->data_definition, element_change->element_name);
  check_status("add_data_to_ecr: status from idd_get_length.");

  if (!existing_ecr) {		/* ecr does not exist			    */
    element_change->data = ism_get_shared_memory(length);
    check_status("add_data_to_ecr: status from ism_get_shm.");
  }				/* end if ecr does not exist		    */
/*
   Get the number of components in the shared data element.
*/
  components = idd_get_number_of_components(
    transaction->receiver->data_definition, element_change->element_name);
  check_status("add_data_to_ecr: status from idd_get_#_components.");
/*
   If the ecr doesn't already exists, create the changed component bit 
   string and initialize the element_change->data to UNDEFINED.
*/
  if (!existing_ecr) {          /* new ecr	                            */

    element_change->changed_components = ubs_create_bitstring(components+1);

    /*
     * If we are not going to immediately overwrite the element with some
     * initializers, explicitly initialize the element with UNDEFINED
     */
    if (data != NULL && component_name == NULL)
	isd_init_sd_element(transaction->receiver->data_definition,
			 element_change->element_name,
			 element_change->data,
			 element_change, set_changed_bits);

  }				/* end if new ecr			    */
/*
   If no data specified return without doing anything.
*/
  if (data == NULL)
    return;
/*
   If element specified copy element data into shared memory otherwise
   copy component data into shared memory.
*/
  if (component_name == NULL) {		/* element specified		    */
/*
   Set appropriate change bit in changed component bitstring.
*/
    ubs_set_bitstring(element_change->changed_components, components);
    check_status("add_data_to_ecr: status from ubs_set_bit.");
/*
   If element is a scalar copy user data into scalar shared data instance.
*/
    if (components == -1) {	/* if element is a scalar		    */
/*
   Get component data type.
*/
      data_type = idd_get_shared_data_type(
        transaction->receiver->data_definition,
        element_change->element_name, NULL);
      check_status("add_data_to_ecr: status from idd_get_shared_data_type.");
/*
   If element is of buffer type create shared memory buffer and copy in 
   user data.
*/
      if (data_type == idd_buffer) {	/* buffer type			    */
/*
   Free any existing buffer body regardless of if there is new user data.
*/
        shm_buffer = (idd_buffer_type *)element_change->data;

        if (shm_buffer->body != NULL) {	/* if existing buffer body	    */
          ism_free_shared_memory(shm_buffer->body);
          check_status("add_data_to_ecr: status from ism_free_shared_mem.");
        }				/* end if existing buffer body	    */

        user_buffer = (idd_buffer_type *)data;

        if (user_buffer->length != 0) {    /* length is non-zero	    */

          shm_buffer->length = user_buffer->length;
          shm_buffer->type = user_buffer->type;

          shm_buffer->body = ism_get_shared_memory(user_buffer->length);
          check_status("add_data_to_ecr: status from ism_get_shared_memory.");

          ism_copy_into_shared_memory(shm_buffer->body, user_buffer->length, 
            user_buffer->body);
          check_status("add_data_to_ecr: status from ism_copy_into_shm.");

	}			/* end if non-zero buffer length	    */

      }				/* end if buffer type			    */
/*
   Copy user data into shared memory.
*/
      else {			/* else if not buffer type		    */

        ism_copy_into_shared_memory(element_change->data, length, data);
        check_status("add_data_to_ecr: status from ism_copy_into_shm.");

      }				/* end else not buffer type		    */

    }				/* end if element is a scalar		    */
    else {			/* element is a record			    */
      component_alist = idd_create_component_alist(
        transaction->receiver->data_definition, element_change->element_name);
      check_status("add_data_to_ecr: status from idd_create_component_alist.");

      for_all_3alist(component_alist,
        transaction->receiver->data_definition,
	element_change, data, copy_component);

      idd_destroy_component_alist(component_alist);
      check_status("add_data_to_ecr: status from idd_destroy_comp_alist.");
    }					/* end if element is a record	    */

  }					/* end if element specified	    */
  else {				/* component specified		    */
/*
   Get the length, offset and position and data type of the component.
*/
    data_type = idd_get_component_lop(transaction->receiver->data_definition,
      element_change->element_name, component_name, &length, &offset,
      &position);
    check_status("isd_add_data_to_ecr: status from idd_get_component_lop.");
/*
   Set appropriate change bit in changed component bitstring.
*/
    ubs_set_bit(element_change->changed_components, position);
    check_status("isd_add_shrd_data: status from ubs_set_bit.");
/*
   If buffer handle special case.
*/
    if (data_type == idd_buffer) {	/* buffer type			    */

      user_buffer = (idd_buffer_type *)data;

      if (user_buffer->length != 0) {    /* length is non-zero	    */

        shm_buffer = (idd_buffer_type *)
	  ((char *)element_change->data + offset);
        shm_buffer->length = user_buffer->length;
        shm_buffer->type = user_buffer->type;

        shm_buffer->body = ism_get_shared_memory(user_buffer->length);
        check_status("add_data_to_ecr: status from ism_get_shared_memory.");

        ism_copy_into_shared_memory(shm_buffer->body, user_buffer->length, 
          user_buffer->body);
        check_status("add_data_to_ecr: status from ism_copy_into_shm.");

      }					/* end if buffer length is nonzero  */

    }					/* end if buffer type		    */
    else {				/* not buffer type		    */
/*
   Copy user data into shared memory.
*/
      ism_copy_into_shared_memory((element_change->data)+offset, length, 
        data);
      check_status("add_data_to_ecr: status from ism_copy_into_shm.");
    }					/* end else not buffer type	    */
  }					/* end else component specified	    */

}                                       /* end add_data_to_ecr 	    */

/*--------------------------------------------------------------------------*\
|  Routine: isd_add_shared_data_with_id
| 
|  Description:  
|     The add shared data w/ id routine creates an instance for the specified 
|     shared data element.  
|     The data may be either initialized or uninitialized. 
\*--------------------------------------------------------------------------*/
void isd_add_shared_data_with_id(transaction, element_name, component_name, 
				data, id)
isd_trans transaction;
string element_name, component_name;
caddr_t data;
iid_id_type id;
{                                       /* local variable definitions       */

  element_change_record *element_change;

/*
**  Validate parameters.
*/
  if (!isc_validate(transaction, isc_transaction_record_vs)) { 
					/* invalid transaction handle	    */
    set_status(invalid_transaction_handle);
    DEBUG1(isc_print_status("isd_add_shared_data:"));
    return;
  }					/* end if invalid transaction hand  */

  if (element_name == NULL) {		/* null element string		    */
    set_status(null_element_name);
    DEBUG1(isc_print_status("isd_add_shared_data:"));
    return;
  }					/* end if null element string	    */
/*
**  Create an element change record.
*/
  element_change = create_element_change_record(transaction, element_name);
  check_status(
    "isd_add_shared_data:  status from create_element_change_record."
  );
/*
**  Add data to element change record.
*/
  add_data_to_ecr(
    transaction, element_change, component_name, false, data, true
  );
  check_status("isd_add_shared_data:  status from add_data_to_ecr.");
/*
   Set change type in element change record to create.
*/
  element_change->change_type = isd_create;
/*
  add id to element change record.
*/
  element_change->id = id;
/*
   Add change record to change table associated with transaction.
*/
  ush_add_to_shm_hashtable(transaction->change_table, element_change, id);

}                                       /* end isd_add_shared_data 	    */

/*--------------------------------------------------------------------------*\
|  Routine: isd_add_shared_data
| 
|  Description:  
|     The add shared data routine creates an instance for the specified 
|     shared data element and returns a unique ID.  
|     The data may be either initialized or uninitialized. 
\*--------------------------------------------------------------------------*/
iid_id_type isd_add_shared_data(transaction, element_name, component_name, 
				data)
isd_trans transaction;
string element_name, component_name;
caddr_t data;
{
iid_id_type id;

/*
   Create ID 
*/
  id = iid_create_id();
  rcheck_status("isd_add_shared_data:  status from idd_create_id.", NULL);
  isd_add_shared_data_with_id(transaction, element_name, component_name,
                              data, id);

  return id;
}

/*--------------------------------------------------------------------------*\
|  Routine: isd_put_shared_data
| 
|  Description:  
|     The put shared data call is used to put information into shared data.
\*--------------------------------------------------------------------------*/
void isd_put_shared_data(transaction, id, element_name, component_name, data)
isd_trans transaction;
iid_id_type id;
string element_name, component_name;
caddr_t data;
{                                       /* local variable definitions       */
element_change_record *element_change;

  set_status(ok);                       /* begin                            */
/*
   Validate required parameters.
*/
  if (!isc_validate(transaction, isc_transaction_record_vs)) { 
					/* invalid transaction handle	    */
    set_status(invalid_transaction_handle);
    DEBUG1(isc_print_status("isd_put_shared_data:"));
    return;
  }					/* end if invalid transaction hand  */

  if (element_name == NULL) {		/* null element string		    */
    set_status(null_element_name);
    DEBUG1(isc_print_status("isd_put_shared_data:"));
    return;
  }					/* end if null element string	    */
/*
   Find the change record associated with the given id on the list of 
   changes associated with the transaction.
*/
  element_change = (element_change_record *)ush_get_from_shm_hashtable(
    transaction->change_table, id);
/*
   If id is not found in transaction create a new element change record
   and add it to the transaction.
*/
  if (element_change == NULL) {		/* assume id not in transaction     */
/*
   Create an element change record.
*/
    element_change = create_element_change_record(transaction, element_name);
    check_status("isd_put_shared_data:  status from crt_elem_chg_rcrd.");
/*
   Add data to element change record.
*/
    add_data_to_ecr(
      transaction, element_change, component_name, false, data, false
    );
    check_status("isd_put_shared_data:  status from add_data_to_ecr.");
/*
   Set id to specified id.
*/
    element_change->id = id;
/*
   Set change type in element change record to modify.
*/
    element_change->change_type = isd_modify;
/*
   Add change record to change table associated with transaction.
*/
    ush_add_to_shm_hashtable
	(transaction->change_table, element_change, id);
  }					/* end if id not in transaction     */
/*
   Else modify existing (create) element change record with new data.
*/
  else {				/* element change record found	    */
/*
   If change type not isd_remove, modify the data in ecr.
*/
    if (element_change->change_type != isd_remove) { /* remove		*/

      add_data_to_ecr(
        transaction, element_change, component_name, true, data, false
      );
      check_status("isd_put_shared_data: status from add_data_to_ecr.");

    }				/* end if change type of remove		*/
  }					/* end else ecr found		    */

}                                       /* end isd_put_shared_data 	    */

/*--------------------------------------------------------------------------*\
|  Routine: delete_buffer
| 
|  Description:  
|     The delete buffer routine examines a component to determine if it
|     is a buffer.  If the body of the buffer is not null it is deleted.
\*--------------------------------------------------------------------------*/
static void delete_buffer(data_definition, element_change, component_name)
idd_data_definition data_definition;
element_change_record *element_change;
string component_name;
{                                       /* local variable definitions       */
  idd_data_types data_type;
  int length, offset, position;
  idd_buffer_type *shm_buffer;

/*
   Get the length, offset and position and data type of the component.
*/
  data_type = idd_get_component_lop(data_definition,
    element_change->element_name, component_name, &length, &offset, &position);
  check_status("delete_buffer: status from idd_get_lngthNoff.");
/*
   Test appropriate change bit in changed component bitstring.
*/
  if (ubs_test_bit(element_change->changed_components, position)) {
/*
   If component is of buffer type free buffer storage.
*/
    if (data_type == idd_buffer) {	/* buffer type			    */
      shm_buffer = (idd_buffer_type *)((char *)element_change->data + offset);
      if (shm_buffer->body != NULL) {	/* body is not null		    */
        ism_free_shared_memory(shm_buffer->body);
        check_status("delete_buffer: status from ism_free_shared_memory.");
      }					/* end if non-zero length buffer    */

    }					/* end if buffer type		    */
  }					/* end if component changed	    */
}					/* end delete_buffer		    */

/*--------------------------------------------------------------------------*\
|  Routine: free_element_change_record
| 
|  Description:  
|     Internal routine to free storage associated with an element change 
|     record.
\*--------------------------------------------------------------------------*/
void free_element_change_record(data_definition, element_change)
idd_data_definition data_definition;
element_change_record *element_change;
{                                       /* local variable definitions       */
  idd_data_types data_type;
  ALIST component_alist;

  set_status(ok);                       /* begin                            */
/*
   If the change type is not remove free all allocated storage.
*/
  if (element_change->change_type != isd_remove) { /* chg type = remove	    */

/*
   If element is a record create a list of changed components and 
   for each buffer component free associated buffer.
*/
    data_type = idd_get_shared_data_type(data_definition,
      element_change->element_name, NULL);
    check_status("free_elem_chg_rcrd: status from idd_get_shared_data_type.");

    if (data_type == idd_record) {	/* element is a record		    */

      component_alist = idd_create_component_alist(data_definition,
        element_change->element_name);
      check_status("f.e.c.r.: status from idd_create_component_alist.");

      for_all_2alist(component_alist, data_definition, element_change,
        delete_buffer);

      idd_destroy_component_alist(component_alist);
      check_status("f.e.c.r.: status from idd_destroy_comp_alist.");
    }					/* element is a record		    */
    else if (data_type == idd_buffer) {	/* element is a scalar and buffer   */
      ism_free_shared_memory(((idd_buffer_type *)element_change->data)->body);
    }					/* element is a scalar and buffer   */

    ubs_destroy_bitstring(element_change->changed_components);
    check_status("free_elem_chg_rec: status from ubs_dstry_bitstring.");

    ism_free_shared_memory(element_change->data); 
    check_status("free_element_change_rcrd: status from ism_free_shm.");
  }					/* end if change type is remove     */
/*
   Free storage associated with element name.
*/
  ism_free_shared_memory(element_change->element_name);
  check_status("free_element_change_record: status from ism_free_shm.");

}                                       /* end free_element_chnge_record    */

/*--------------------------------------------------------------------------*\
|  Routine: isd_remove_shared_data
| 
|  Description:  
|     The remove routine is used to remove a specified shared data element.
\*--------------------------------------------------------------------------*/
void isd_remove_shared_data(transaction, element_name, id)
string element_name;
isd_trans transaction;
iid_id_type id;          
{                                       /* local variable definitions       */
  element_change_record *element_change;

  set_status(ok);                       /* begin                            */
/*
   Validate parameters.
*/
  if (!isc_validate(transaction, isc_transaction_record_vs)) { 
					/* invalid transaction handle	    */
    set_status(invalid_transaction_handle);
    DEBUG1(isc_print_status("isd_remove:"));
    return;
  }					/* end if invalid transaction hand  */

  if (element_name == NULL) {		/* null element string		    */
    set_status(null_element_name);
    DEBUG1(isc_print_status("isd_remove:"));
    return;
  }					/* end if null element string	    */
/*
   Determine if a change record associated with the given id is on the list 
   of changes associated with the transaction.
*/
  element_change = (element_change_record *)ush_get_from_shm_hashtable(
    transaction->change_table, id);

/*
   If id is not found in transaction create a new element change record,
   set the change type to remove and add it to the transaction.
*/
  if (element_change == NULL) {		/* assume id not in transaction     */
/*
   Create an element change record.
*/
    element_change = create_element_change_record(transaction, 
      element_name);
    check_status("isd_add_shared_data:  status from crte_elem_chg_rec.");
/*
   Set change type to remove.
*/
    element_change->change_type = isd_remove;    
/*
   Set id to specified id.
*/
    element_change->id = id;    
/*
   Add change record to change table associated with transaction.
*/
    ush_add_to_shm_hashtable (transaction->change_table, element_change, id);
  }					/* end if id not in transaction     */
/*
   Else if (id is in transaction) change type is create then delete
   element change record.
*/
  else if (element_change->change_type == isd_create) {	/* create	    */
/*
   Delete the change element corresponding to the given id from the
   change table associated with the transaction.
*/
    element_change = (element_change_record *)ush_delete_from_shm_hashtable(
      transaction->change_table, id);
    check_null(element_change, 
      "isd_put_shared_data:  Could not delete element from shm hashtable.\n",
      not_found);
/*
   Free all storage associated with element change record.
*/
    free_element_change_record(
      transaction->receiver->data_definition, element_change);
    check_status("isd_remove_shared_data: status from free_elem_chnge.");
  }				/* end if change type is create		    */
/* 
   Else if change type is modify change change type from modify to remove,
   delete the data assoicated with the change and nullify the pointer.
*/
  else if (element_change->change_type == isd_modify) { /* modify	    */

    element_change->change_type = isd_remove;    

    ubs_destroy_bitstring(element_change->changed_components);
    check_status("isd_remove_sd: status from ubs_dstry_bitstring.");

    ism_free_shared_memory(element_change->data);
    check_status("isd_remove_shared_data: status from ism_free_shm.");

    element_change->data = NULL;
  }				/* end if change type is create		    */

}                                       /* end isd_remove_shared_data       */

/*--------------------------------------------------------------------------*\
|  Routine: isd_rollback_transaction
| 
|  Description:  
|     The rollback transaction routine is used to abort a given transaction 
|     and delete the associated transaction buffer.
\*--------------------------------------------------------------------------*/
void isd_rollback_transaction(transaction)
isd_trans transaction;
{
  set_status(ok);                       /* begin                            */
/*
   Validate parameters.
*/
  if (!isc_validate(transaction, isc_transaction_record_vs)) { 
					/* invalid transaction handle	    */
    set_status(invalid_transaction_handle);
    DEBUG1(isc_print_status("isd_rollback_transaction:"));
    return;
  }					/* end if invalid transaction hand  */
/*
   Free storage associated with all element change records in transaction.
*/
  ush_for_all_shm_hashtable(
    transaction->change_table, 
    transaction->receiver->data_definition,
    free_element_change_record);
/*
   Delete the change_table hash table structure including all attached nodes.
*/
  ush_free_shm_hashtable(transaction->change_table, AND_NODES);
/*
   Free receiver record. 
*/
  free_node(transaction->receiver);
/*
   Free transaction record. 
*/
  free_node(transaction);
}                                       /* end isd_rollback_transaction     */

/*--------------------------------------------------------------------------*\
|  Routine: isd_commit_transaction
| 
|  Description:  
|     The commit transaction routine is used to commit a transaction to the
|     data base and notify the reciever.
\*--------------------------------------------------------------------------*/
void isd_commit_transaction(transaction)
isd_trans transaction;
{
  set_status(ok);                       /* begin                            */
/*
   Validate parameters.
*/
  if (!isc_validate(transaction, isc_transaction_record_vs)) { 
					/* invalid transaction handle	    */
    set_status(invalid_transaction_handle);
    DEBUG1(isc_print_status("isd_commit_transaction:"));
    return;
  }					/* end if invalid transaction hand  */
/*
   Send the pointer to the changes to the reciever specified in the receiver
   record.
*/

/*
   Commit transaction if the hashtable is not empty, and check to
   allow recording.
*/
  if (!hashtable_empty(transaction->change_table)) {

    ipb_record_transaction (
      (char *) transaction->change_table,
      in_box.mailbox, transaction->receiver->mailbox);

    check_status("isd_commit_transaction: status from ipb_record_trans.");

    ipc_send(transaction->receiver->mailbox, in_box.mailbox,
      (char *) transaction->change_table);
    check_status("isd_commit_transaction: status from ipc_send.");
  }
/*
   If empty, destroy empty hash table.
*/
  else {                                        /* empty table		*/

    ush_free_shm_hashtable(transaction->change_table, AND_NODES);

  }					/* end else empty hash		*/

/*
   Free transaction record.
*/
  free_node(transaction);
}                                       /* end isd_commit_transaction       */
/*--------------------------------------------------------------------------*\
|  Routine: isd_get_dialogue_name
| 
|  Description:  
|     The get dialogue name routine returns the name of the current dialogue
\*--------------------------------------------------------------------------*/
char *isd_get_dialogue_name()
{
   return "";
}
