/* $Header: /u/04c/mnt/integ/source/six/src/RCS/omove.c,v 1.1 91/02/12 14:02:55 rm Exp $ */

/*--------------------------------------------------------------------------*\
|
|  Name:
|	mmove.c
|
|  Description:
|	Motif move method code
|
|  Component of:
|	Six
|
|  Related files:
|	resize.c notify.c etc.
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version:
|	1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:
|	Rick Kazman
|	rnk@sei.cmu.edu
|
|  Change History:
 $Log:	omove.c,v $
 * Revision 1.1  91/02/12  14:02:55  rm
 * Initial revision
 * 
 * Revision 1.3  90/10/12  15:08:42  rnk
 * changed attribute names to new style (no underscores)
 * 
 * Revision 1.2  90/09/07  15:31:26  rnk
 * added code to check that a widget has the allow_user_move attribute
 * and fixed bug which caused the move method to be returned to the
 * wrong widget
 * 
 * Revision 1.1  90/08/22  18:25:52  rnk
 * Initial revision
 * 
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

    isd_trans trans;
    INST *inst = glue_get_by_widget (w);
    ATTR *x, *y, *move ;
    Widget parent ;
    Window window, parent_win ;
    Display *display ;
    Arg local_args[50] ;
    int status, i ;
    register int prev_x;                /* Previous event window X location.*/
    register int prev_y;                /* Previous event window Y location.*/
    XWindowAttributes window_info;      /* Event window information.    */
    int cur_x;                          /* Current event window X location. */
    int cur_y;                          /* Current event window Y location. */
    int root_x;                         /* Root window X location. */
    int root_y;                         /* Root window Y location. */
    int ulx, uly;                       /* Event window upper left X and Y. */
    int lrx, lry;                       /* Event window lower right X and Y.*/
    int num_vectors;                    /* Number of vectors in box. */
    int ptrmask;                        /* state of ptr when queried */
    Window sub_window;                  /* Query mouse event sub-window. */
    Window root;                        /* Query mouse event root. */
    XEvent button_event;                /* Button event packet. */
    XSegment box[MAX_BOX_VECTORS];      /* Box vertex buffer. */
    boolean found ;			/* flag */

    /* Loop upwards from child to parent, searching for the first widget */
    /* in the hierarchy which allows moving.				 */
    move = glue_get_attr_from_instance (inst, "allowusermove", SERP_NAMES);
    found = FALSE ;
    while (!found)
      {
	if (move->value.i == TRUE) found = TRUE ;
	else
	  {
	    w = XtParent(w) ;
	    if (w == NULL) return ;
	    if ((inst = glue_get_by_widget(w)) == NULL) return ;
	    move = glue_get_attr_from_instance(inst, "allowusermove",
					       SERP_NAMES) ;
	    if (move == NULL) return ;
	  }
      }

    /* Get the appropriate instances */
    x = glue_get_attr_from_instance (inst, "x", SERP_NAMES);
    y = glue_get_attr_from_instance (inst, "y", SERP_NAMES);

    /* Find the display and the window id of the widget and its parent */
    display = XtDisplay(w) ;
    window = XtWindow(w) ;
    parent = XtParent(w) ;
    parent_win = XtWindow(parent) ;

    /* Change the cursor. */
    XChangeActivePointerGrab(display,EVENTMASK,ArrowCrossCursor,CurrentTime);

    /* Clear the vector buffers. */
    bzero(box, sizeof(box));

    /* Gather info on the event window. */
    if (!XGetWindowAttributes(display, window, &window_info))
        fprintf(stderr, "m22_child_move:  Can't get window attributes\n") ;

    /* Initialize movement variables. */
    ulx = window_info.x;
    uly = window_info.y;
    lrx = window_info.x + window_info.width +
                     (window_info.border_width << 1) - 1;
    lry = window_info.y + window_info.height +
                     (window_info.border_width << 1) - 1;

    /* Store the box. */
    num_vectors = StoreBox(box, ulx, uly, lrx, lry);

    XQueryPointer(display, window, &root,  &sub_window,
                      &root_x, &root_y, &cur_x, &cur_y, &ptrmask) ;
    prev_x = root_x ;
    prev_y = root_y ;

    /* Process any pending exposure events before drawing the box. */
    while (QLength(display) > 0) {
        XPeekEvent(display, &button_event);
        if (((XAnyEvent *)&button_event)->window == window &&
                (button_event.type == ButtonPress ||
                        button_event.type == ButtonRelease))
            break;
        XNextEvent(display, &button_event);
    }

    /* Freeze the server so we get a solid box instead of a flickering one */
    XGrabServer(display);

    /* Now draw the box.  */
    DrawBox();

    /* Main loop. */
    while (TRUE)
    {
        /* Check to see if we have a change in mouse button status.
         * This is how we get out of this "while" loop.         */
        XNextEvent(display, &button_event);

        /* Process the pending events, this sequence is the only
         * way out of the loop and the routine.    */

        if ((button_event.type != ButtonPress) &&
            (button_event.type != ButtonRelease) &&
            (button_event.type != MotionNotify))
        {
            continue; /* spurious menu event... */
	}

        DrawBox();
        XUngrabServer(display);

        if (button_event.type == ButtonRelease)
        {
            /* The button was released, so move the window.      */
            XtMoveWidget(w, ulx, uly); 
	    x->value.i = ulx ;
	    y->value.i = uly ;

	    /* Send the transactions to the interface */
	    trans = isd_start_transaction ();

	    /* Send the method name */
	    isd_put_shared_data (trans, inst->id, inst->template->name, 
				 "method", "move");

	    /* Send the changed attributes */
	    isd_put_shared_data (trans, inst->id, inst->template->name, 
				 "x", &x->value.i);
	    isd_put_shared_data (trans, inst->id, inst->template->name, 
				 "y", &y->value.i);

	    /* Commit the transation */
	    isd_commit_transaction (trans);
            return;
	  }

        /* Take care of all the little things that have changed. */
        XQueryPointer(display, window, &root,  &sub_window,
                      &root_x, &root_y, &cur_x, &cur_y, &ptrmask);
        if ((root_x != prev_x) || (root_y != prev_y))
        {

            /* Box position has changed. */
            ulx += root_x - prev_x;
            uly += root_y - prev_y;
            lrx += root_x - prev_x;
            lry += root_y - prev_y;

            /* Box needs to be restored. */
            num_vectors = StoreBox(box, ulx, uly, lrx, lry);
	}

        /* Save old box position. */
        prev_x = root_x;
        prev_y = root_y;
        DrawBox();
    }
