/* $Header: /u/04c/mnt/integ/source/six/src/RCS/resize.c,v 1.1 90/08/02 19:41:21 rnk Exp $ */

/*--------------------------------------------------------------------------*\
|
|  Name:
|	resize.c
|
|  Description:
|	Resize method code
|
|  Component of:
|	Six
|
|  Related files:
|	move.c resize.c notify.c etc.
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version:
|	1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:
|	Rick Kazman
|	rnk@sei.cmu.edu
|
|  Change History:
 $Log:	resize.c,v $
 * Revision 1.1  90/08/02  19:41:21  rnk
 * Initial revision
 *  
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

    isd_trans trans;
    INST *inst = glue_get_by_widget (w);
    INST *parent_inst ;
    ATTR *x = glue_get_attr_from_instance (inst, "x", SERP_NAMES);
    ATTR *y = glue_get_attr_from_instance (inst, "y", SERP_NAMES);
    ATTR *width = glue_get_attr_from_instance (inst, "width", SERP_NAMES);
    ATTR *height = glue_get_attr_from_instance (inst, "height", SERP_NAMES);
    ATTR *resize = glue_get_attr_from_instance (inst, "allow_user_resize", 
						SERP_NAMES);
    Window window, parent_win ;
    Widget parent ;
    XWindowAttributes window_info;      /* Event window info. */
    register int prev_x;                /* Previous event window X location.*/
    register int prev_y;                /* Previous event window Y location.*/
    int mid_x, mid_y;                   /* mid-point of the widget */
    int cur_x, cur_y;                   /* current pointer position */
    int i_height, i_width ;		/* widget dimensions (int version) */
    int ulx, uly;                       /* Event window upper left X and Y. */
    int lrx, lry;                       /* Event window lower right X and Y.*/
    int root_x, root_y;                 /* root window coordinates */
    int corner ;                        /* corner which controls resize */
    int ptrmask;                        /* pointer status word */
    int num_vectors;                    /* Number of vectors to XDraw. */
    Window sub_window;                  /* Mouse query sub window. */
    Window root;                        /* Root query window. */
    XEvent button_event;                /* Button event packet. */
    XSegment box[MAX_BOX_VECTORS];      /* Box drawing vertex buffer. */
    boolean found ;			/* flag */

    /* Loop upwards from child to parent, searching for the first widget */
    /* in the hierarchy which allows resizing.				 */
    found = FALSE ;
    while (!found)
      {
	if (resize->value.i == TRUE) found = TRUE ;
	else
	  {
	    w = XtParent(w) ;
	    if (w == NULL) return ;
	    if ((parent_inst = glue_get_by_widget(w)) == NULL) return ;
	    resize = glue_get_attr_from_instance(parent_inst, 
						 "allow_user_resize",
						 SERP_NAMES) ;
	  }
      }

    /* Find the display and the window id of the widget and its parent */
    window = XtWindow(w) ;
    display = XtDisplay(w) ;
    parent = XtParent(w) ;
    parent_win = XtWindow(parent) ;

    /* Change the cursor. */
    XChangeActivePointerGrab(display,EVENTMASK,ArrowCrossCursor,CurrentTime);

    /* Clear the vector buffers. */
    bzero(box, sizeof(box));

    /* Gather info on the event window. */
    if (!XGetWindowAttributes(display, window, &window_info))
        fprintf(stderr, "resize_method:  Can't get window attributes\n");
 
    /* Initialize movement variables. */
    ulx = window_info.x;
    uly = window_info.y;
    lrx = window_info.x + window_info.width +
                     (window_info.border_width << 1) - 1;
    lry = window_info.y + window_info.height +
                     (window_info.border_width << 1) - 1;

    num_vectors = StoreBox(box, ulx, uly, lrx, lry) ;

    /* Find and save the initial position of the pointer */
    XQueryPointer(display, window, &root, &sub_window,
                &root_x, &root_y, &cur_x, &cur_y, &ptrmask);

    prev_x = root_x ;
    prev_y = root_y ;

    /* Locate the middle of the widget */
    mid_x = window_info.width/2 ;
    mid_y = window_info.height/2 ;

    if (cur_x < mid_x)
    {
        if (cur_y < mid_y)
        {   /* Pointer is in the upper left quadrant of the widget */
            corner = UL ;
	} else
        {   /* Pointer is in the lower left quadrant of the widget */
            corner = LL ;
	}
    } else {
        if (cur_y < mid_y)
        {   /* Pointer is in the upper right quadrant of the widget */
            corner = UR ;
	} else
        {   /* Pointer is in the lower right quadrant of the widget */
            corner = LR ;
	}
    }

    /* Process any pending exposure events before drawing the box. */
    while (QLength(display) > 0)
    {
        XPeekEvent(display, &button_event);
        if (((XAnyEvent *)&button_event)->window == window
           && (button_event.type == ButtonPress ||
                button_event.type == ButtonRelease))
            break;
        XNextEvent(display, &button_event);
    }

    /* Freeze the server so we get a solid box instead of a flickering one */
    XGrabServer(display);

    /* Now draw the box.  */
    DrawBox();

    /* Main loop. */
    while (TRUE)
    {
        /* Check to see if we have a change in mouse button status.
         * This is how we get out of this "while" loop.         */
        XNextEvent(display, &button_event);

        if ((button_event.type != ButtonPress) &&
            (button_event.type != ButtonRelease) &&
            (button_event.type != MotionNotify))
        {
            continue; /* spurious menu event... */
	}

        DrawBox();
        XUngrabServer(display);

        if (button_event.type == ButtonRelease)
	  {
            /* The button was released, so move and resize the window. */

            x->value.i = ulx ;
            y->value.i = uly ;
            i_width = lrx - ulx ;
            i_height = lry - uly ;
            if (i_width < 2) width->value.i = 2 ;
	    else width->value.i = i_width ;
            if (i_height < 2) height->value.i = 2 ;
	    else height->value.i = i_height ;

	    XtConfigureWidget(w, x->value.i, y->value.i, width->value.i, 
			      height->value.i, window_info.border_width) ;

            /* Redraw the affected widget */
            XClearArea (display, window, 0, 0, 0, 0, TRUE) ;

	    /* Send the transactions to the interface */
	    trans = isd_start_transaction ();

	    /* Send the method name */

	    isd_put_shared_data (trans, inst->id, inst->template->name, 
				 "method", "resize");

	    /* Send the changed attributes */

	    isd_put_shared_data (trans, inst->id, inst->template->name, 
				 "x", &x->value.i);
	    isd_put_shared_data (trans, inst->id, inst->template->name, 
				 "y", &y->value.i);
	    isd_put_shared_data (trans, inst->id, inst->template->name, 
				 "width", &width->value.i);
	    isd_put_shared_data (trans, inst->id, inst->template->name, 
				 "height", &height->value.i);

	    /* Commit the transation */

	    isd_commit_transaction (trans);
            return;
	  }

        /* Take care of all the little things that have changed. */
        XQueryPointer(display, window, &root,  &sub_window,
                      &root_x, &root_y, &cur_x, &cur_y, &ptrmask);

        if ((root_x != prev_x) || (root_y != prev_y))
        {
            /* Box position has changed. */
            switch(corner)
            {
                case UL:

                    ulx += root_x - prev_x;
                    uly += root_y - prev_y;
                    break ;

                case UR:

                    uly += root_y - prev_y;
                    lrx += root_x - prev_x;
                    break ;

                case LL:

                    ulx += root_x - prev_x;
                    lry += root_y - prev_y;
                    break ;

                case LR:

                    lrx += root_x - prev_x;
                    lry += root_y - prev_y;
                    break ;
	    }

            /* Box needs to be restored. */
            num_vectors = StoreBox(box, ulx, uly, lrx, lry);
	}

        /* Save old box position. */
        prev_x = root_x;
        prev_y = root_y;

        DrawBox();
    }
