static char rcsid[] = "$Header: /u/04c/mnt/integ/source/six/src/RCS/sbm.c,v 1.4 91/03/26 09:20:00 ejh Exp $" ;

/*--------------------------------------------------------------------------*\
|
|  Name:
|	sbm.c
|
|  Description:
|	This module handles parsing and loading bitmaps from files into
|	the appropriate internal form (an array of unsigned chars).
|
|  Component of:
|	Six
|
|  Related files:
|	sbm.h
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version:
|	1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:
|	Rick Kazman
|	rnk@sei.cmu.edu
|
|  Change History:
 $Log:	sbm.c,v $
 * Revision 1.4  91/03/26  09:20:00  ejh
 * voided some unused assignments; took out break after return
 * 
 * Revision 1.3  90/12/10  10:23:50  rnk
 * sbm_load_bitmap complains if it can't read the bitmap file, and returns
 * a NULL (rather than bombing out).
 * 
 * Revision 1.2  90/08/10  17:12:09  rnk
 * fixed problem with files not being closed
 * 
 * Revision 1.1  90/06/21  13:48:32  rnk
 * Initial revision
 * 
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	memoryPack

#include <stdio.h>
#include "hashPack.h"
#include "u22.h"
#include "ufa.h"
#include "six.h"

#define ENV_DATA_PATH   "SERPENT_DATA_PATH"

#define OTHER		0
#define WORD		1
#define HEX		2
#define NUMBER		3
#define OPEN		4
#define CLOSE		5
#define DEFINE		6
#define ERROR		7

/*=========================================================================*\
 * sbm_load_bitmap opens a file, parses its input (assumed in bitmap form)
 * allocates space for the bitmap and loads it into the allocated array.
\*=========================================================================*/
unsigned char *sbm_load_bitmap (bitmap_name, width, height, array_size)
     
     char *bitmap_name ;
     int *width, *height, *array_size ;
{ 
  FILE *fp ;
  int i, type, length, node_size ;
  char token[100], *data_search_paths, *full_data_path ;
  unsigned char hex_val, temp, *bitmap ;
  
  if (!getenv (ENV_DATA_PATH)) 
    {
      fprintf(stderr, "sbm_load_bitmap:  %s environment var not present\n", 
	      ENV_DATA_PATH);
      return(NULL);
    }

  data_search_paths = u22_copy_string(getenv (ENV_DATA_PATH));

  if (!strlen (data_search_paths))
    {
      data_search_paths [0] = '.';
      data_search_paths [1] = '\0';
    }

  SIX_DEBUG3(fprintf (stderr, "sbm_load_bitmap:  data_search_paths=%s\n", 
		      data_search_paths));

  full_data_path = ufa_full_file_name (data_search_paths, bitmap_name);
  if (full_data_path == NULL)
    {
      fprintf (stderr, "sbm_load_bitmap:  could not find bitmap file %s\n\n", 
	       bitmap_name);
      return(NULL);
    }

  fp = fopen(full_data_path, "r") ;
  if (fp == NULL) 
    {
      fprintf(stderr, "sbm_load_bitmap:  Unable to open bitmap file %s\n", 
	      bitmap_name) ;
      return(NULL) ;
    }
  
  *array_size = *width = *height = 0 ;
  
  while ((type = Six_get_token(token, &length, fp)) != EOF)
    {
      switch (type) {

	/* Get the width and height */
      case DEFINE:
	(void) Six_get_token(token, &length, fp) ;
	if (!strcmp("width", token+length-5))
	  {
	    type = Six_get_token(token, &length, fp) ;
	    if (type != NUMBER) printf("no value found for width\n") ;
	    *width = atoi(token) ;
	  } else if (!strcmp("height", token+length-6))
	    {
	      type = Six_get_token(token, &length, fp) ;
	      if (type != NUMBER) printf("no value found for height\n");
	      *height = atoi(token) ;
	    } else
	      {		/* Swallow the number */
		(void) Six_get_token(token, &length, fp) ;
	      }
	break ;

	/* Add to the array */
      case HEX:
	hex_val = 0 ;
	for (i=2; i<length; i++) 
	  {
	    if (token[i] >= '0' && token[i] <= '9') temp = token[i] - '0' ;
	    else if (token[i] >= 'a' && token[i] <= 'f') 
	      temp = (token[i] - 'a') + 10 ;
	    hex_val = hex_val * 16 + temp ;
	  }
	bitmap[*array_size] = hex_val ;
	(*array_size)++ ;
	if (*array_size == node_size)
	  {
	    fprintf(stderr, "sbm_load_bitmap: exceeding alloc'ed storage\n") ;
	    fprintf(stderr, "width=%d  height=%d size=%d\n",
		    *width, *height, node_size) ;
	    return(NULL) ;
	  }
	break ;
	
	/* Allocate the array */
      case OPEN:
	/* Figure out how many bytes = width * height bits */
	node_size = (*width/8+1) * (*height/8+1) * 8 ;
	bitmap = (unsigned char *) make_node(node_size) ;
	if (bitmap == NULL)
	  {
	    fprintf(stderr,
		    "sbm_load_bitmap: couldn't alloc enough storage\n") ;
	    return(NULL) ;
	  }
	
	break ;
	
	/* Do nothing */
      case CLOSE: 
	break ;
	
      case OTHER:
	break ;

      case ERROR:
	fprintf(stderr, "sbm_load_bitmap: problem with file %s\n", 
		bitmap_name);
	return(NULL) ;
      }
    }
  
  fclose(fp) ;
  return(bitmap) ;
}


/*=========================================================================*\
 * Six_get_token gets a token from input and returns it, along with 
 * its type classification.
\*=========================================================================*/
Six_get_token(token, length, fpin)

int *length ;
char *token ;
FILE *fpin ;
{
    int type, i, temp ;
    register int in ;

    while ((in = getc(fpin)) == ' ' || in == '\n') ;
    *length = 0 ;
    if (in == EOF) return(EOF) ;  /* Check for end of file */
    token[0] = in ;
    
    switch (in) {
	                       /* Accept a word */
        case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
        case 'g': case 'h': case 'i': case 'j': case 'k': case 'l':
        case 'm': case 'n': case 'o': case 'p': case 'q': case 'r':
        case 's': case 't': case 'u': case 'v': case 'w': case 'x':
        case 'y': case 'z':
        case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
        case 'G': case 'H': case 'I': case 'J': case 'K': case 'L':
        case 'M': case 'N': case 'O': case 'P': case 'Q': case 'R':
        case 'S': case 'T': case 'U': case 'V': case 'W': case 'X':
        case 'Y': case 'Z':
            type = WORD ;
            *length = 0 ;
	    while (   (token[*length] >= 'A' && token[*length] <= 'Z') 
	           || (token[*length] >= 'a' && token[*length] <= 'z') 
	           || (token[*length] >= '0' && token[*length] <= '9') 
		   || token[*length] == '_'  || token[*length] == '-')
            {
		(*length)++ ;
		token[*length] = getc(fpin) ;
            }
	    ungetc(token[*length],fpin) ;
	    break ;
	                       /* Accept a #define */
	case '#': 
            type = DEFINE ;
            *length = 1 ;
	    while ((token[*length] = getc(fpin)) != ' ') (*length)++ ;
	    token[*length] = '\0' ;
	    if (strcmp(token, "#define"))
	    {
		fprintf(stderr,"Six_get_token: illegal define '%s'\n", token) ;
		return(ERROR) ;
	    }
	    break ;
	                       /* Accept a left curly */
        case '{':
            type = OPEN ;
            *length = 1 ;
	    break ;
	                       /* Accept a right curly */
        case '}':
            type = CLOSE ;
            *length = 1 ;
	    break ;
	                       /* Accept other chars */
        case '=': case '[': case ']': case ';': case ',':
            type = OTHER ;
            *length = 1 ;
	    break ;
	                       /* Accept a number */
        case '0': case '1': case '2': case '3': case '4':
        case '5': case '6': case '7': case '8': case '9': case '-':
            type = NUMBER ;
            *length = 0 ;
	    while((token[*length] >= '0' && token[*length] <= '9')
		 || token[*length] == 'x' || token[*length] == '-' 
		 || (token[*length] >= 'a' && token[*length] <= 'f' 
		     && type == HEX) )
	    {
		if (token[*length] == 'x') type = HEX ;
		(*length)++ ;
		token[*length] = getc(fpin) ;
            }
	    ungetc(token[*length],fpin) ;
	    break ;
	                       /* Catch anything else */
        default:
	    fprintf(stderr, "Six_get_token:  can't handle char '%c' (%d)\n", 
		    in, in);
	    fprintf (stderr, "Next 50 chars: ") ;
	    for (i=0; i<50; i++) fputc(stderr,getc(fpin)) ;
	    fputs ("\n", stderr) ;
	    return(ERROR) ;
    }
    token[*length] = '\0' ;

    return(type) ;
}   

