static char rcsid[] = "$Header: /u/04c/mnt/integ/source/six/src/RCS/sct.c,v 1.1 90/06/21 13:49:13 rnk Exp $" ;

/*--------------------------------------------------------------------------*\
|
|  Name:
|	sct.c
|
|  Description:
|	The implementation of the color lookup table for Six; the main
|	routine transforms a string color name into the appropriate X Pixel
|	value.
|
|  Component of:
|	Six
|
|  Related files:
|	sct.h
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version:
|	1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:
|	Rick Kazman
|	rnk@sei.cmu.edu
|
|  Change History:
 $Log:	sct.c,v $
 * Revision 1.1  90/06/21  13:49:13  rnk
 * Initial revision
 * 
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define memoryPack

#include	<X11/Xlib.h>
#include	<X11/Intrinsic.h>

#include	<stdio.h>
#include	"string.h"
#include	"hashPack.h"
#include        "six.h"

static	char	*color_names[] =
{
  "aquamarine", "black", "blue", "blue violet", "brown", "cadet blue",
  "coral", "cornflower blue", "cyan", "dark green", "dark olive green", 
  "dark orchid", "dark slate blue", "dark slate gray", "dark slate grey", 
  "dark turquoise",  "dim gray", "dim grey", "firebrick", "forest green",
  "gold", "goldenrod", "gray", "green", "green yellow", "grey", 
  "indian red", "khaki", "light blue", "light grey", "light gray", 
  "light steel blue", "lime green", "magenta", "maroon", 
  "medium aquamarine", "medium blue", "medium forest green", 
  "medium goldenrod", "medium orchid", "medium sea green", 
  "medium slate blue", "medium spring green", "medium turquoise", 
  "medium violet red", "midnight blue", "navy", "navy blue",
  "orange", "orange red", "orchid", "pale green", "pink", "plum", "red",
  "salmon", "sea green", "sienna", "sky blue", "slate blue", 
  "spring green", "steel blue", "tan", "thistle", "turquoise", 
  "violet", "violet red", "wheat", "white", "yellow", "yellow green"
};

#define	NUM_COLORS	71

#define	HASH_SIZE	101
HASH	x_color_table;

typedef	struct	
{
  char	*name;
  XColor	color;
}    sct_color_node;


/*--------------------------------------------------------------------------*\
|  Name: sct_hash
|
|  Description:
|	sct_hash is the hashing function for the Zimmerman hash table
|	package when using a color string as the lookup key. This function is
|	called by the Zimmerman package.
|
|  Parameters:
|	name	the string used to identify a particular node in a hash table;
|		converted by this function into an integer between 0 and
|		(HASH_SIZE - 1).
|
|  Returns: the sum of the individual characters in name modulo HASH_SIZE.
\*--------------------------------------------------------------------------*/

int	sct_hash (name)
     char	*name;
{
  register	int	hash_value = 0;
  register	char	*pointer;
  
  /* loop through each character in the string 'name' and add  */
  /* the character value to the accumulator 'hash_value'.      */
  
  for (pointer = name; *pointer != '\0'; pointer++)
    hash_value += *pointer;
  
  /* return a value in the right range. */
  
  return (hash_value % HASH_SIZE);
}


/*--------------------------------------------------------------------------*\
|  Name: sct_match
|
|  Description:
|	sct_match_component is called by the Zimmerman hash table 
|	package in order to match queries to nodes in a table.
|
|  Parameters:
|	id	the string used to look up a particular node in a hash table
|		keyed by a string.
|	color	the node from the hash table whose hash value matched the hash
|		value of id and is possibly the node being looked up.
|
|  Returns: true if a match occurs and false if there is not a match.
\*--------------------------------------------------------------------------*/

int	sct_match (id, color)
     char		*id;
     sct_color_node	*color;
{
  SIX_DEBUG2 (fprintf (stderr, "sct_match: Entering.\n");
	  fprintf (stderr, "sct_match: id =%s  color=%s\n", 
 		   id, color->name);
	  fprintf (stderr, "sct_match: Leaving.\n"));
  
  return (strcmp (id, color->name) == 0) ;
}


/*--------------------------------------------------------------------------*\
|  Name: sct_make_color_table
|
|  Description:
|	sct_make_color_table constructs a hash table of XColor structures
|	which match the string color names in the array color_names; the table
|	key is the string itself; the hash table is called x_color_table; this
|	is a global variable which is changed by this function.
|
|  Parameters: None.
|
|  Returns: Nothing.
\*--------------------------------------------------------------------------*/

void	sct_make_color_table (display)
     Display	*display;
{
  sct_color_node	*temp;
  int			i;
  XColor		dummy;
  
  SIX_DEBUG2 (fprintf (stderr, "sct_make_color_table: Entering.\n"));
  
  x_color_table = make_hashtable (HASH_SIZE, sct_hash, sct_match);
  
  for (i = 0; i < NUM_COLORS; i++)
    {
      temp = (sct_color_node *) make_node (sizeof (sct_color_node));
      temp->name = color_names [i];
      
      XAllocNamedColor (display, DefaultColormap (display,
	  DefaultScreen (display)), color_names [i], &(temp->color),&dummy);
      
      add_to_hashtable (x_color_table, temp, temp->name);
    }
  
  SIX_DEBUG2 (fprintf (stderr, "sct_make_color_table: Leaving.\n"));
}


/*--------------------------------------------------------------------------*
|  Name: sct_get_pixel
|
|  Description:
|	sct_get_pixel retrieves the pixel value for a color from
|	x_color_table. If the color does not exist, it returns NULL.
|
|  Parameters:
|	color_name	the color for which the user needs the pixel value.
|
|  Returns: XColor value of the color or NULL
\*-------------------------------------------------------------------------*/

Pixel	sct_get_pixel (color_name)
     char	*color_name ;
{
  sct_color_node	*temp ;
  
  SIX_DEBUG2 (fprintf (stderr, "sct_get_pixel: Entering.\n") ;
	  fprintf (stderr, "sct_get_pixel: color_name = %s.\n", 
		   color_name)) ;
  
  temp = (sct_color_node *) get_from_hashtable (x_color_table, color_name) ;
  
  if (temp == NULL)
    {
      SIX_DEBUG2(fprintf(stderr, "sct_get_pixel:  Color %s not found.\n",
		     color_name);
	     fprintf(stderr, "sct_get_pixel: Leaving.\n"));
      return(BAD_COLOR);
    }
  else
    {
      SIX_DEBUG3 (fprintf (stderr, "sct_get_pixel: pixel is %d.\n",
		       temp->color.pixel);
	      fprintf (stderr, "sct_get_pixel: Leaving.\n"));
      return (temp->color.pixel);
    }
}
