static char rcsid[] = "$Header: /usr/users/rnk/serpent/source/six/src/RCS/six.c,v 1.31 1991/08/23 15:02:53 rnk Exp $" ;

/*--------------------------------------------------------------------------*\
|
|  Name:
|	Six.c
|
|  Description:
|	This is the main program for Six (Serpent Interface to Xt)
|
|  Component of:
|	Six
|
|  Related files:
|	lots
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version:
|	1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:
|	Rick Kazman
|	rnk@sei.cmu.edu
|
|  Change History:
  $Log: six.c,v $
 * Revision 1.31  1991/08/23  15:02:53  rnk
 * no changes
 *
 * Revision 1.30  1991/06/19  21:19:40  ejh
 * changed XmSTRING_DEFAULT_CHARSET to charset
 * 
 *
 * Revision 1.29  1991/06/14  02:34:49  rnk
 * changed char type to KeySym and made Six_string_table return the number
 * of items in the table.
 *
 * Revision 1.28  1991/04/25  22:00:14  rnk
 * changed delete algorithm to speed it up
 *
 * Revision 1.27  1991/04/22  19:47:11  rnk
 * changed default shell to use applicationShellWidgetClass and changed
 * calling sequence for check routines
 *
 * Revision 1.26  91/03/26  09:13:36  ejh
 * fixed a fprintf: added stderr
 * 
 * Revision 1.25  91/03/14  13:45:12  rnk
 * changed the parameters to check routines to include the widget's
 * arglist.
 * 
 * Revision 1.24  91/01/03  21:23:44  rnk
 * changed Six_manage and Six_process to look at an object's "manage" flag
 * to decide whether it should be managed or not.
 * 
 * Revision 1.23  90/12/20  15:01:15  rnk
 * added support for XmStringTable attribute
 * 
 * Revision 1.22  90/12/18  17:32:09  rnk
 * changed sorting algorithm for removes, changed bitmaps to that they
 * would work on color displays, etc.
 * 
 * Revision 1.21  90/11/08  20:50:30  rnk
 * changed warning messages in Six_set
 * 
 * Revision 1.20  90/11/06  22:33:50  rnk
 * created a single transaction which is sent from six back to the dm after
 * widgets are created/modified (the old way sent a separate transaction for
 * each creation/modification).
 * 
 * Revision 1.19  90/10/22  17:09:39  rnk
 * fixed problem with return value from convenience routines not being saved
 * 
 * Revision 1.18  90/10/12  15:06:54  rnk
 * added ifdefs for users who don't have Motif
 * 
 * Revision 1.17  90/10/10  18:09:56  rnk
 * added logic to check the managedwhencreated flag for widgets being created
 * with default shells.
 * 
 * Revision 1.16  90/10/07  17:01:41  rnk
 * fixed bug with GT_cursor type, added code to support GT_top and GT_bottom
 * types and added code to handle an attribute which tells whether to create
 * the widget using XtCreateWidget or XtCreateManagedWidget
 * 
 * Revision 1.15  90/10/03  17:50:53  rnk
 * added convenience routine support, better debugging info for infinite
 * loop detection.
 * 
 * Revision 1.14  90/09/19  15:12:20  rnk
 * changed the way that widgets are realized and managed (added a new routine
 * Six_manage) so that all widgets in a transaction are created before any
 * are managed.  Also, changed the check for a parent in Six_process so that
 * even if a parent attribute exists, if its value is null, it is ignored.
 * Finally, added support for parenting a popup shell.
 * 
 * Revision 1.13  90/09/01  20:16:08  ejh
 * put in tests for hp; sigvec is called sigvector on that machine
 * 
 * Revision 1.12  90/08/30  12:08:52  rnk
 * added code to check for undefined attributes to Six_set
 * 
 * Revision 1.11  90/08/27  18:46:06  rnk
 * fixed usage message and remove code
 * 
 * Revision 1.10  90/08/23  20:40:18  rnk
 * added support for widget attribute
 * 
 * Revision 1.9  90/08/20  19:04:35  rnk
 * added code to check for infinite dependency loops in Six_dispatch and
 * to check for empty arg lists in Six_process
 * 
 * Revision 1.8  90/08/17  17:40:58  rnk
 * added code to support Screen types, as well as random bug fixes
 * 
 * Revision 1.6  90/08/13  19:23:27  rnk
 * added code to call check_routines and removed kluge code
 * 
 * Revision 1.5  90/08/09  18:24:15  rnk
 * added code to call XtMoveWidget and XtResizeWidget (for sat only),
 * and to support the creation of PopupShells.  Also added code for
 * focus, and other assorted bug fixes.
 * 
 * Revision 1.2  90/07/12  16:27:34  rnk
 * simplified transaction handling mechanism, added smarter freeing code,
 * added code to support Accelerators, Cursors, floats and FontStructs,
 * and various bug fixes
 * 
 * Revision 1.1  90/06/21  13:47:11  rnk
 * Initial revision
 * 
/*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	memoryPack
#define SIX_H

#include 	<sys/time.h>
#include	<signal.h>
#include	<stdio.h>
#include	<X11/Xlib.h>
#include	<X11/Intrinsic.h>
#include	<X11/Shell.h>
#include	<X11/cursorfont.h>

#ifdef HasMotif
#include	<Xm/Xm.h>
#endif

#include	"glue_intf.h"
#include	"glue_inst.h"
#include	"u22.h"
#include	"uc2.h"
#include	"iid.h"
#include	"isd.h"
#include	"isc.h" 
#include	"sct.h"
#include	"sbm.h"
#include	"hashPack.h"	/* Hash table includes */
#include	"six.h"
#include	"serpent.h"

#define HASH_SIZE	257
#define Screen_id	(Widget)1
#define StringTableSize	50
#define TempStringSize	128
#define ListSeparator	'/'

extern XtActionsRec glue_actions[] ;
extern int glue_actions_count ;

static  char    *event_names[] =
{"Protocol Error", "Protocol Error", "KeyPress", "KeyRelease",
   "ButtonPress", "ButtonRelease", "MotionNotify", "EnterNotify",
   "LeaveNotify", "FocusIn", "FocusOut", "KeymapNotify", "Expose",
   "GraphicExpose", "NoExpose", "VisibilityNotify", "CreateNotify",
   "DestroyNotify", "UnmapNotify", "MapNotify", "MapRequest",
   "ReparentNotify", "ConfigureNotify", "ConfigureRequest",
   "GravityNotify", "ResizeRequest", "CirculateNotify",
   "CirculateRequest", "PropertyNotify", "SelectionClear",
   "SelectionRequest", "SelectionNotify", "ColormapNotify",
   "ClientMessage", "MappingNotify", "LASTEvent"};

/* Definitions for Graphics Context */
#define DRAW_WIDTH              0       /* use fastest hardware draw */
#define DRAW_VALUE              0xfd
#define DRAW_FUNC               GXxor
XGCValues xgc ;                         /* to create font GCs */
GC      DrawGC ;                        /* graphics context for drawing */
Cursor  ArrowCrossCursor ;

/* Definitions for handling different displays */
int screen ;
boolean	is_color ;
int screen_height, screen_width, depth ;
Display *display ;
XtAppContext app_context ;
     
/* Random definitions and declarations */
#ifdef HasMotif
static  XmStringCharSet charset = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
#endif

boolean	serpent_inited = FALSE;
boolean transaction_started ;
isd_trans transaction ;
TECH *technology ;
isd_trans return_trans ;
/*extern process_record in_box;   /* The mailbox for this process */


#ifdef hpux
#undef sigvec
#endif

/* Definitions for sleep routine */
#define	mask(s)	(1<<((s)-1))
#define	setvec(vec, a) \
  vec.sv_handler = a; vec.sv_mask = vec.sv_onstack = 0
  static int ringring;
void sleepx() { ringring = 1; }
     

/*=========================================================================*\
 * Modified version of the system sleep function to use smaller time 
 * slices (system granularity is 1 sec, this routine is 1/100 sec)	
\*=========================================================================*/
sleep_10ms(n)
     unsigned n;
{
  int omask;
  struct itimerval itv, oitv;
  register struct itimerval *itp = &itv;
  struct sigvec vec, ovec;
  
  if (n == 0) return;
  timerclear(&itp->it_interval);
  timerclear(&itp->it_value);
  if (setitimer(ITIMER_REAL, itp, &oitv) < 0) return;
  setvec(ovec, SIG_DFL);
  omask = sigblock(0);
  /* Multiply millisecond time by 10,000 to get 10 ms chunks */
  itp->it_value.tv_usec = n * 10000 ;
  if (timerisset(&oitv.it_value))    
    {
      if (timercmp(&oitv.it_value, &itp->it_value, >))
	oitv.it_value.tv_sec -= itp->it_value.tv_sec;
      else 
    	{
	  itp->it_value = oitv.it_value;
	  /*
	   * This is a hack, but we must have time to
	   * return from the setitimer after the alarm
	   * or else it'll be restarted.  And, anyway,
	   * sleep never did anything more than this before.
	   */
	  oitv.it_value.tv_sec = 1;
	  oitv.it_value.tv_usec = 0;
    	}
    }
  setvec(vec, sleepx);
#ifdef hpux
  (void) sigvector(SIGALRM, &vec, &ovec);
#else
  (void) sigvec(SIGALRM, &vec, &ovec);
#endif
  ringring = 0;
  (void) setitimer(ITIMER_REAL, itp, (struct itimerval *)0);
  while (!ringring) sigpause(omask &~ mask(SIGALRM));
#ifdef hpux
  (void) sigvector(SIGALRM, &ovec, (struct sigvec *)0);
#else
  (void) sigvec(SIGALRM, &ovec, (struct sigvec *)0);
#endif
  (void) setitimer(ITIMER_REAL, &oitv, (struct itimerval *)0);
}


/*=========================================================================*\
 *  Six_initialize_toolkit initializes the toolkit, creates an application
 *  context and opens the display.
\*=========================================================================*/
void Six_initialize_toolkit (argc, argv)
     int argc;
     char **argv;
{
  Widget	toplevel ;
  Arg		args [5] ;
  int		nargs ;
  isd_trans 	trans ;
  
  SIX_DEBUG1(fprintf (stderr, "Six_initialize_toolkit: entering\n")) ;
  XtToolkitInitialize() ;
  app_context = XtCreateApplicationContext();
  XtAppAddActions(app_context, glue_actions, glue_actions_count) ;
  
  display = XtOpenDisplay(app_context, NULL, "Six", "Six", NULL, 
			  0, &argc, argv);
  if (!display)
    {
      fprintf (stderr, "\nSix:  can't open display--exiting\n");
      exit(1) ;
    }
  screen = DefaultScreen(display);
  depth = DisplayPlanes(display, screen);
  screen_height = DisplayHeight(display, screen) ;
  screen_width = DisplayWidth(display, screen) ;

  if (depth == 1) is_color = FALSE ;
  else is_color = TRUE ;
  
  SIX_DEBUG3(fprintf
	     (stderr, "Six_initialize_toolkit: display=%d  screen=%d\n", 
	      display, screen));
  SIX_DEBUG1(fprintf (stderr, "Six_initialize_toolkit: leaving\n")) ;
}


/*=========================================================================*\
 * Six_callback is the generic callback procedure.  It simply sends a 
 * method ("notify") to the dm.
\*=========================================================================*/
void	Six_callback (widget, client_data, call_data)
Widget	widget;
caddr_t	client_data;
caddr_t	call_data;

{
  isd_trans trans;
  INST *inst = glue_get_by_widget(widget);

  trans = isd_start_transaction ();

  /* Send the method name */
  isd_put_shared_data (trans, inst->id, inst->template->name, 
		       "method", "notify");

  isd_commit_transaction (trans);
}


/*=========================================================================*\
 * Six_interrupt_handler exits the program when an interrupt is caught
\*=========================================================================*/
void Six_interrupt_handler()
{
  if (serpent_inited) serpent_cleanup();
  fprintf (stderr, "\nSix exiting\n");
  exit(0) ;
}


/**********/
/*  Main  */
/**********/

main (argc, argv)
     int argc ;
     char **argv ;
{
  isc_status_codes	status = ok ;
  XEvent		my_event ;
  void 			Six_sift(), Six_manage() ;
  LIST 			creates, removes, modifies, order_list, Six_dispatch();
  
  if (argc < 2 || argc > 3) 
    {
      fprintf (stderr, "Usage:  %s technology [1-3]\n", argv[0]);
      exit(1);
    }

  six_debug = 0 ;

  /* Make sure that we have the right version of includes */
  if (GT_LAST_V6 + 1 < GT_LAST) 
    {
      fprintf(stderr, "Six: Bad value for GT_LAST--you need to modify Six\n") ;
      exit(1) ;
    }
  
  fprintf(stderr, "Six/%s (version 1.29)\n\n", argv[1]);

  /* Register the interrupt handler */
  signal(SIGINT, Six_interrupt_handler);
  signal(SIGTERM, Six_interrupt_handler);

  /* Get the technology description */  
  technology = glue_get_technology(argv[1]) ;
  if (!technology)
    {
      fprintf (stderr, "Six: get_technology failed\n");
      exit(1);
    }

  if (argc == 3) six_debug = atoi(argv[2]) ;

  /* Open the mail box for communication with the interface */
  serpent_init(technology->mailbox, technology->ill_file);
  if (status != ok)
    {
      fprintf (stderr, "Six: serpent_init failed\n");
      exit(1);
    }

  /* Initialize the X toolkit */
  Six_initialize_toolkit(argc, argv); 
  
  /* make the color table */
  sct_make_color_table(display) ;
  
  /* Create the cursor and the graphics context for move/resize */
  ArrowCrossCursor = XCreateFontCursor(display, XC_fleur);
  
  xgc.line_width = DRAW_WIDTH ;
  xgc.foreground = DRAW_VALUE;
  xgc.function = DRAW_FUNC;
  xgc.subwindow_mode = IncludeInferiors;
  DrawGC = XCreateGC(display, RootWindow(display,screen),
	    (GCLineWidth | GCForeground | GCFunction | GCSubwindowMode), &xgc);
  
  /* Initialize lists for keeping track of Six objects */
  removes = make_list() ;
  creates = make_list() ;
  modifies = make_list() ;
  
  transaction_started = FALSE ;
  fprintf (stderr, "Six/%s:  Ready.\n", argv[1]) ;
  
  /* Now loop forever--the interrupt handler will take care of exiting */
  while (TRUE)
    {
      status = not_available;
      while (status == not_available)
	{
	  while (XtAppPending(app_context))
	    {
	      XtAppNextEvent(app_context, &my_event);
	      XtDispatchEvent(&my_event);
	    } 
	  
	  transaction = isd_get_transaction_no_wait();
	  status = get_status ();
	  if (status == not_available && !XtAppPending(app_context))
	    sleep_10ms(10) ;
	}
      
      if (status != ok)
	{
	  fprintf (stderr, "Six: isd_get_transaction of %d.\n", status) ;
	  serpent_cleanup() ;
	  fprintf(stderr, "Six: Exiting.\n") ;
	  fflush(stderr) ;
	  exit(1) ;
	}
      
      SIX_DEBUG3(fprintf (stderr, "Six: Got a transaction.\n"));
      
      /* Dispatch the transaction elements in the order: */
      /* removes, creates and then modifies.		 */
      Six_sift(removes, creates, modifies) ;

      /* Handle removes */
      Six_dispatch(isd_remove, removes) ;

      order_list = Six_dispatch(isd_create, creates) ;
      /* If some creates were done, manage and realize them */
      if (order_list != NULL)
	{
	   if (!list_empty(order_list)) Six_manage(order_list) ;
	   free_list(order_list, AND_NODES) ; 
	}
      
      /* Handle modifies */
      Six_dispatch(isd_modify, modifies) ;

      /* Purge transaction. */
      SIX_DEBUG3(fprintf(stderr, "Six: Purging transaction.\n"));
      isd_purge_transaction(transaction) ;

      /* Clear the id lists for the next transaction */
      flush_list(creates, AND_NODES) ;
      flush_list(removes, AND_NODES) ;
      flush_list(modifies, AND_NODES) ;

      if (transaction_started)
	{
	  isd_commit_transaction(return_trans);
	  transaction_started = FALSE;
	  SIX_DEBUG3 (fprintf (stderr, "Six: sent transaction\n"));
	}
    }	/* end while (TRUE) */
}


/*=========================================================================*\
 * Six_process gets a transaction element from the main program, and
 * creates, modifies or destroys widgets (and their associated INST 
 * structures) to reflect the information in the element.
\*=========================================================================*/
void Six_process(change, id, to_dm) 
     isd_trans to_dm ;
     iid_id_type id ;
     isd_change_type change;
{
  LIST		change_list;
  char		*data, *data_ptr ;
  int           record_size, length, offset, pos ;
  Cardinal	num_args ;
  Widget	widget, default_shell ;
  Arg		args[50], shell_args[1] ;
  string	element_name, component_name ;
  iid_id_type   *temp_id ;
  WIDG		*def_widget ;
  INST		*widget_inst, *parent_inst ;
  ATTR		*attr_def, *parent_attr, *create_attr, *manage_attr ;
  
  SIX_DEBUG1(fprintf (stderr, "Six_process: entering\n")) ;

  /* Get the element name from the transaction */
  element_name = isd_get_element_name(transaction, id) ;
  SIX_DEBUG3(fprintf (stderr, "Six_process: element_name=%s  id=%d\n", 
		      element_name, id));

  switch(change)
    {
    case isd_create:
      SIX_DEBUG1(fprintf (stderr, "Six_process: change=create\n")) ;

      /* Get the widget definition */
      def_widget = glue_get_widget(technology, element_name) ;
      
      data = isd_get_shared_data(transaction, id, NULL) ;
      change_list = isd_create_changed_component_list(transaction, id) ;
      
      /* Create a widget definition instance (for storing current values) */
      widget_inst = glue_create_instance(def_widget) ;
      widget_inst->id = id ;
      
      /* Load defaults into the arg list */
      num_args = 0;
      Six_do_defaults(widget_inst, def_widget, args, &num_args) ;
      
      /* Load attributes from the change_list into the widget's arg list */
      component_name = get_list_next(change_list, NULL);

      while (component_name != NULL)
	{
	  attr_def = glue_get_attr_from_instance(widget_inst, component_name,
						 SERP_NAMES) ;
	  if (attr_def == NULL)
	    {
	      fprintf(stderr,
		      "Six_process:  attribute not found for %s--ignoring\n",
		      component_name) ;
	    }
	  else
	    {
	      isd_get_component_lop(element_name, attr_def->s_name, 
				    &length, &offset, &pos) ;
	      data_ptr = data + offset;
	      Six_set(change, widget_inst, component_name, data_ptr, 
		      args, &num_args) ;
	    }
	  component_name = get_list_next(change_list, component_name);
	}
      
      /* Call the widget's check routine, if one exists */
      if (def_widget->check_rtn != NULL) 
	(*(def_widget->check_rtn))(widget_inst, change, change_list) ;

      /* Check widget type here for widget creation */
      switch (def_widget->widget_type)
	{

	/* Create a widget */
	case WT_WIDGET:
	  /* Check to see if this widget has a parent.	*/
	  parent_attr = glue_get_attr_from_instance(widget_inst,
						    "parent", SERP_NAMES) ;
	  if (parent_attr->val_type == VAL_CURRENT &&
	      parent_attr->value.i != iid_null_id)
	    {
	      /* Retrieve the parent instance */
	      parent_inst = 
		glue_get_by_id((iid_id_type) parent_attr->value.i) ;
	      if (parent_inst == NULL) 
		{
		  fprintf(stderr, 
			  "Six_process: parent not found for id %d\n", id) ;
		  exit(1) ;
		}

	      /* If there is a convenience routine defined for this */
	      /* widget, use it, otherwise use XtCreateWidget	    */
	      if (def_widget->conv_rtn == NULL)
		{
		   /* If the "managedWhenCreated" or "manage" flag is false */
		   /* use XtCreateWidget, else use XtCreateManagedWidget    */
		    create_attr = glue_get_attr_from_instance(widget_inst,
					  "managedwhencreated", SERP_NAMES) ;
		    manage_attr = glue_get_attr_from_instance(widget_inst,
					  "manage", SERP_NAMES) ;
		    if ((create_attr != NULL && create_attr->value.i == FALSE)
		     || (manage_attr != NULL && manage_attr->value.i == FALSE))
		      widget = XtCreateWidget(element_name, def_widget->class,
					  parent_inst->widget, args, num_args);
		    else      
		      widget = XtCreateManagedWidget(element_name,
			         def_widget->class, parent_inst->widget, 
			         args, num_args);
		}
	      else
		widget = (*(def_widget->conv_rtn))(parent_inst->widget, 
					  element_name, args, num_args);

	      /* Add this widget to its parent's list of children */
	      temp_id = (iid_id_type *) make_node(sizeof(iid_id_type)) ;
	      *temp_id = id ;
	      add_to_atail(parent_inst->child, temp_id) ;

	      /* Put the widget id in a transaction returned to the dialogue */
	      put_shared_data(to_dm, id, element_name, "widget", &widget) ;
	    }
	  else
	    {
	       /* This widget does not have a parent, so create	*/
	       /* a default application shell.			*/
	       default_shell = XtAppCreateShell("Six", NULL, 
					       applicationShellWidgetClass, 
					       display, shell_args, 0);

	       /* If there is a convenience routine defined for this */
	       /* widget, use it, otherwise use XtCreateWidget	    */
	       if (def_widget->conv_rtn == NULL)
		 {
		   /* If the "managedWhenCreated" or "manage" flag is false */
		   /* use XtCreateWidget, else use XtCreateManagedWidget    */
		    create_attr = glue_get_attr_from_instance(widget_inst,
					  "managedwhencreated", SERP_NAMES) ;
		    manage_attr = glue_get_attr_from_instance(widget_inst,
					  "manage", SERP_NAMES) ;
		    if ((create_attr != NULL && create_attr->value.i == FALSE)
		     || (manage_attr != NULL && manage_attr->value.i == FALSE))
		      widget = XtCreateWidget(element_name, def_widget->class,
				   default_shell, args, num_args);
		    else      
		      widget = XtCreateManagedWidget(element_name,
				   def_widget->class, default_shell, 
			           args, num_args);
		 }     
	      else
		widget = (*(def_widget->conv_rtn))(default_shell, element_name,
						   args, num_args);
	   }
	  break;

	/* Create a normal shell */
	case WT_WM_SHELL:
	  widget = XtAppCreateShell("Six", NULL, def_widget->class,
				    display, args, num_args);
	  break;

	/* Create a popup shell */
	case WT_OVERRIDE_SHELL:
	  parent_attr = glue_get_attr_from_instance(widget_inst,
						    "parent", SERP_NAMES) ;
	  if (parent_attr->val_type != VAL_NONE &&
	      parent_attr->value.i != NULL)
	    {
	      parent_inst = 
		glue_get_by_id((iid_id_type) parent_attr->value.i) ;
	      if (parent_inst == NULL) 
		{
		  fprintf(stderr, 
			  "Six_process: parent not found for id %d\n", id) ;
		  exit(1) ;
		}
	      widget = XtCreatePopupShell(element_name, def_widget->class,
					  parent_inst->widget, args, num_args);
	    }
	  else
	    fprintf(stderr, 
		    "Six_process: Can't create an unparented PopupShell\n") ;
	  break;

	/* If it isn't a widget, give it a fake widget id and then ignore it */
	case WT_NONE:
	  widget = Screen_id ;
	  break;
	}

      /* Store the widget in widget_inst */
      widget_inst->widget = widget ;

      /* Call the widget's check routine again, if one exists */
      if (def_widget->check_rtn != NULL) 
	(*(def_widget->check_rtn))(widget_inst, change, change_list) ;

      /* Register the widget instance */
      SIX_DEBUG3(fprintf(stderr,
			 "Six_process: Registering id %d (%s widget %d)\n", 
			 id, element_name, widget));
      if (!glue_register_instance(widget_inst)) 
	{
	  fprintf(stderr, "Six: failed glue_register_instance\n") ;
	  exit(1) ;
	}

      free_node(data) ;
      
      break ;
      
    case isd_modify:
      SIX_DEBUG1(fprintf (stderr, "Six_process: change=modify\n")) ;

      /* Get the widget's instance record */
      widget_inst = glue_get_by_id(id) ;
      if (widget_inst == NULL) 
	{
	  fprintf(stderr,"Six_process: widget_inst not found for id %d\n", id);
	  exit(1) ;
	}
      
      /* Get the widget definition */
      def_widget = glue_get_widget(technology, element_name) ;
      
      data = get_shared_data(transaction, id, NULL) ;
      change_list = create_changed_component_list(transaction, id) ;
      
      /* Load attributes from the change_list into the widget's arg list */
      component_name = get_list_next(change_list, NULL);
      
      num_args = 0;
      while (component_name != NULL)
	{
	  attr_def = glue_get_attr_from_instance(widget_inst, component_name,
						 SERP_NAMES) ;
	  if (attr_def == NULL)
	    {
	      fprintf(stderr,
		      "Six_process:  attribute not found for %s--ignoring\n",
		      component_name) ;
	    }
	  else
	    {
	      isd_get_component_lop(element_name, attr_def->s_name, 
				&length, &offset, &pos) ;
	      data_ptr = data + offset;
	      Six_set(change, widget_inst, component_name, data_ptr, 
		      args, &num_args) ;
	    }
	  component_name = get_list_next(change_list, component_name);
	}
      
      /* call the widget's check routine, if one exists */
      if (def_widget->check_rtn != NULL) 
	(*(def_widget->check_rtn))(widget_inst, change, change_list) ;

      if (num_args) XtSetValues (widget_inst->widget, args, num_args);
      free_node(data) ;
      
      break ;
      
    case isd_remove:
      SIX_DEBUG1(fprintf (stderr, "Six_process: change=remove\n")) ;

      /* Get the widget's instance record */
      widget_inst = glue_get_by_id(id) ;

      /* Recursively destroy the widget tree rooted at widget_inst */
      Six_destroy(widget_inst) ;
      
      break ;
      
    default:
      fprintf(stderr, "Six_process: unknown change type %d\n", change) ;
      break ;
    }

  SIX_DEBUG1(fprintf (stderr, "Six_process: leaving\n")) ;
}


/*=========================================================================*\
 * Six_destroy descends down a widget tree and destroys it.
\*=========================================================================*/
Six_destroy(widget_inst)

INST *widget_inst ;
{
  iid_id_type   *cur_child ;

  /* If this widget has already been deleted, just return */
  if (widget_inst == NULL) return ;
      
      
  /* Ask X to destroy the widget, and then destroy the instance structure. */
  XtDestroyWidget(widget_inst->widget) ;

  /* If the child list is not empty, destroy the child widgets	*/
  /* before deleting the widget's own instance record.		*/
  if (alist_length(widget_inst->child) > 0)
    {
      cur_child = (iid_id_type *) get_alist_next(widget_inst->child, NULL) ;
      while (cur_child != NULL)
	{
	  Six_destroy(glue_get_by_id(*cur_child)) ;
	  cur_child = 
	    (iid_id_type *) get_alist_next(widget_inst->child, cur_child);
	}
    }
  glue_destroy_instance (widget_inst, AND_STRS) ;

  return ;
}

      
/*=========================================================================*\
 * Six_set gets an attribute and its current value, and attempts to
 * change the attribute in X and in the attribute's instance structure
 * to reflect this value.  It might not change the attribute if it has 
 * the wrong access type.
\*=========================================================================*/
Six_set(change, widget_inst, attr_name, data_ptr, args, num_args) 
     
     isd_change_type change;
     INST *widget_inst ;
     char *attr_name ;
     char *data_ptr ;
     ArgList args ;
     Cardinal *num_args ;
{
#ifdef HasMotif
  XmFontList fontlist ;
  XmString ListItems[StringTableSize] ;
  int Six_string_table() ;
#endif
  XFontStruct *font ;
  static XtCallbackRec callbacks[2] ;
  Pixel color ;
  INST *w_inst ;
  ATTR *cur_attr, *temp_attr ;
  unsigned char *bitmap ;
  int bm_width, bm_height, bm_size, length, items ;
  char *ret_string ;
  isd_trans to_dm ;
  Cursor cursor ;
  boolean ret_val = FALSE ;
  Position temp_x, temp_y ;
  Dimension temp_w, temp_h, temp_bw ;
  

  SIX_DEBUG1(fprintf (stderr, "Six_set: entering\n")) ;
  SIX_DEBUG3(fprintf (stderr, "Six_set: INST=%d  attr_name=%s  num_args=%d\n",
		      widget_inst, attr_name, *num_args)) ;

  /* Get the attribute information */
  cur_attr = glue_get_attr_from_instance(widget_inst, attr_name, SERP_NAMES) ;
  if (cur_attr == NULL)
    {
      fprintf(stderr, "Six_set:  attribute not found for %s--ignoring\n",
	      attr_name) ;
      return ;
    }
  
  /* If the attribute is undefined, return */
  if (cur_attr->s_type == idd_string || cur_attr->s_type == idd_buffer
      || cur_attr->s_type == idd_real) 
    {
      if (uc2_undefined(cur_attr->s_type, (caddr_t) data_ptr))
         return;
    }
  else
    if (uc2_undefined(cur_attr->s_type, (* (int *) data_ptr)))
         return;
  
  /* Make sure that this attribute has the right access type */
  if (  (change == isd_create && !(cur_attr->access & AXS_CREATE))
      || (change == isd_modify && !(cur_attr->access & AXS_SET))  ) 
    {
      fprintf(stderr, "Six_set:  attr=%s  change=%d  access=%d\n", 
	      attr_name, change, cur_attr->access) ;
      fprintf(stderr, 
	      "Six_set:  Incompatible access types--returning\n") ;
      fflush(stderr) ;
      return ;
    }
  
  /* Process the attribute according to its x type */
  switch (cur_attr->x_type) 
    {
    case GT_Accelerators:
      ret_string = (char *)
	uc2_convert(idd_string, idd_buffer, (idd_buffer_type *) data_ptr) ;
      SIX_DEBUG3(fprintf (stderr, "Six_set: Accelerators=`%s'  t_name=%s\n", 
		      ret_string, cur_attr->t_name)); 

      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, 
		   XtParseAcceleratorTable(ret_string));
	  (*num_args)++; 
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  /* If a string had already been stored, free its space */
	  if (cur_attr->val_type == VAL_CURRENT) free_node(cur_attr->value.s);
	  else cur_attr->val_type = VAL_CURRENT ;
	  
	  /* Store the current string */
	  cur_attr->value.s = ret_string ;
	}
      else
	{
	  free_node(ret_string) ;  
	} 
      break ;

    case GT_Boolean:
      SIX_DEBUG3 (fprintf (stderr, "Six_set: Boolean=%d  t_name=%s\n", 
		       *((boolean *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, *((boolean *) data_ptr));
	  (*num_args)++;
	} 
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((boolean *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;
      
    case GT_bottom:
      SIX_DEBUG3(fprintf (stderr, "Six_set: bottom=%d  t_name=%s\n", 
			  *((boolean *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	    if (*((boolean *) data_ptr) && widget_inst->widget)
	      XLowerWindow(display, XtWindow(widget_inst->widget)) ;

	      /* Re-set the attribute to false and send to the dm */
	      to_dm = start_transaction() ;
	      put_shared_data(to_dm, widget_inst->id, 
				  widget_inst->template->name, 
				  cur_attr->s_name, ret_val) ;
	      commit_transaction(to_dm) ;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((boolean *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;
      
    case GT_char_star:
      ret_string = (char *)
	uc2_convert(idd_string, idd_buffer, (idd_buffer_type *) data_ptr) ;
      SIX_DEBUG3 (fprintf (stderr, "Six_set: char_star=%s  t_name=%s\n", 
		      ret_string, cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, ret_string) ;
	  (*num_args)++;
	} 
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  /* If a string had already been stored, free its space */
	  if (cur_attr->val_type == VAL_CURRENT) free_node(cur_attr->value.s);
	  else cur_attr->val_type = VAL_CURRENT ;
	  
	  /* Store the current string */
	  cur_attr->value.s = ret_string ;
	}
      else
	{
	  free_node(ret_string) ; 
	} 
      break ;
      
    case GT_Cursor:
      SIX_DEBUG3(fprintf (stderr, "Six_set: cursor=%d  t_name=%s\n", 
			  *((int *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  cursor = XCreateFontCursor(display, *((int *) data_ptr));
	  XtSetArg(args[*num_args], cur_attr->t_name, cursor);
	  (*num_args)++;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((int *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;
      
    case GT_Dimension:
      SIX_DEBUG3 (fprintf (stderr, "Six_set: Dimension=%d  t_name=%s\n", 
		       *((int *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, *((int *) data_ptr));
	  (*num_args)++;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((int *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;
      
    case GT_float:
      SIX_DEBUG3 (fprintf (stderr, "Six_set: float=%f  t_name=%s\n", 
		       *((float *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, *((float *) data_ptr));
	  (*num_args)++;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	cur_attr->value.r = *((double *) data_ptr) ;
      break ;
      
    case GT_focus:
      SIX_DEBUG3(fprintf (stderr, "Six_set: focus=%d  t_name=%s\n",
			  *((iid_id_type *) data_ptr), cur_attr->t_name)) ;

      /* Look up this widget instance, by its id */
      if (*((iid_id_type *) data_ptr) != iid_null_id &&
	  *((iid_id_type *) data_ptr) != UNDEFINED_ID)
	{
	  w_inst = glue_get_by_id(*((iid_id_type *) data_ptr)) ;
	  if (w_inst == NULL) 
	    {
	      fprintf(stderr,"Six_set: Null widget instance found for id %d\n",
		      *((iid_id_type *) data_ptr)) ;
	      return ;
	    }
      
	  /* Set the keyboard focus to this widget */
	  if (cur_attr->access & AXS_TECH)
	      XtSetKeyboardFocus(XtParent(w_inst->widget), w_inst->widget) ;
	}

      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((iid_id_type *) data_ptr) ;
	}
      
      break ;
      
    case GT_int:
      SIX_DEBUG3(fprintf (stderr, "Six_set: int=%d  t_name=%s\n", 
			  *((int *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, *((int *) data_ptr));
	  (*num_args)++;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((int *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;
      
    case GT_KeySym:
      SIX_DEBUG3 (fprintf (stderr, "Six_set: KeySym=%s  t_name=%s\n", 
			   data_ptr, cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, data_ptr) ;
	  (*num_args)++;
	} 
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.c = (caddr_t) make_node(sizeof(caddr_t)) ;
	  strcpy(cur_attr->value.c, data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;
      
    case GT_MethodName:
      if (cur_attr->access & AXS_SERPENT)
	{
	  /* If a string had already been stored, free its space */
	  if (cur_attr->val_type == VAL_CURRENT) free_node(cur_attr->value.s);
	  else cur_attr->val_type = VAL_CURRENT ;
	  
	  /* Store the current string */
	  cur_attr->value.s = u22_copy_string(data_ptr) ;
	}
      break ;
      

    case GT_Pixel:
      SIX_DEBUG3 (fprintf (stderr, "Six_set: Pixel=%s  t_name=%s\n", 
		      data_ptr, cur_attr->t_name));

      if (cur_attr->access & AXS_TECH)
	{
	  color = sct_get_pixel(data_ptr) ;
	  if (color != BAD_COLOR) 
	    {
	      XtSetArg(args[*num_args], cur_attr->t_name, color) ;
	      (*num_args)++;
	    }
	  else
	    {
	      fprintf(stderr, "Six_set:  Color %s ignored--using default\n",
		      data_ptr) ;
	    }
	}

      if (cur_attr->access & AXS_SERPENT)
	{
	  /* If a string had already been stored, free its space */
	  if (cur_attr->val_type == VAL_CURRENT) free_node(cur_attr->value.s);
	  else cur_attr->val_type = VAL_CURRENT ;
	  
	  /* Store the current string */
	  cur_attr->value.s = u22_copy_string(data_ptr) ;
	}
      break ;

    case GT_Pixmap:
      ret_string = (char *)
	uc2_convert(idd_string, idd_buffer, (idd_buffer_type *) data_ptr) ;
      SIX_DEBUG3 (fprintf (stderr, "Six_set: Pixmap=%s  t_name=%s\n", 
			   ret_string, cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  bitmap = sbm_load_bitmap(ret_string, &bm_width, &bm_height,&bm_size);
	  if (bitmap) 
	    {
	       XtSetArg(args[*num_args], cur_attr->t_name, 
		 XCreatePixmapFromBitmapData(display,
					     RootWindow(display,screen),
					     bitmap, bm_width, bm_height,
					     BlackPixel(display, screen), 
					     WhitePixel(display, screen), 
					     depth)) ;  
	      (*num_args)++;
	    }

	  free_node(bitmap) ;
        }
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  /* If a string had already been stored, free its space */
	  if (cur_attr->val_type == VAL_CURRENT) free_node(cur_attr->value.s);
	  else cur_attr->val_type = VAL_CURRENT ;
	  
	  /* Store the current string */
	  cur_attr->value.s = ret_string ;
	}
      else
	{
	  free_node(ret_string) ; 
	} 
      
      break ;
      
    case GT_Position:
      SIX_DEBUG3 (fprintf (stderr, "Six_set: Position=%d  t_name=%s\n", 
			     *((int *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
        {
          XtSetArg(args[*num_args], cur_attr->t_name, *((int *) data_ptr));
          (*num_args)++;
        }
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((int *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;

    case GT_Screen:
      SIX_DEBUG3(fprintf (stderr, "Six_set: Screen=%d  t_name=%s\n", 
			  *((int *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, *((int *) data_ptr));
	  (*num_args)++;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((int *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;
      
    case GT_short:
      SIX_DEBUG3(fprintf (stderr, "Six_set: short=%d  t_name=%s\n", 
			  *((int *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, *((int *) data_ptr));
	  (*num_args)++;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((int *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;
      
    case GT_String:
      ret_string = (char *)
	uc2_convert(idd_string, idd_buffer, (idd_buffer_type *) data_ptr) ; 
      SIX_DEBUG3 (fprintf (stderr, "Six_set: string=%s  t_name=%s\n", 
		      ret_string, cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, ret_string) ;
	  (*num_args)++;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  /* If a string had already been stored, free its space */
	  if (cur_attr->val_type == VAL_CURRENT) free_node(cur_attr->value.s);
	  else cur_attr->val_type = VAL_CURRENT ;
	  
	  /* Store the current string */
	  cur_attr->value.s = ret_string ;
	}
      else
	{
	  free_node(ret_string) ; 
	} 

      break ;
      
    case GT_top:
      SIX_DEBUG3(fprintf (stderr, "Six_set: top=%d  t_name=%s\n", 
			  *((boolean *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	    if (*((boolean *) data_ptr) && widget_inst->widget)
	      XRaiseWindow(display, XtWindow(widget_inst->widget)) ;

	      /* Re-set the attribute to false and send to the dm */
	      to_dm = start_transaction() ;
	      put_shared_data(to_dm, widget_inst->id, 
				  widget_inst->template->name, 
				  cur_attr->s_name, ret_val) ;
	      commit_transaction(to_dm) ;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((boolean *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;
      
    case GT_unsigned_char:
      SIX_DEBUG3 (fprintf (stderr, "Six_set: unsigned char=%d  t_name=%s\n", 
		       *((int *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, *((int *) data_ptr));
	  (*num_args)++;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((int *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;
      
    case GT_Widget:
      SIX_DEBUG3(fprintf (stderr, "Six_set: Widget=%d  t_name=%s\n",
			  *((iid_id_type *) data_ptr), cur_attr->t_name)) ;

      /* Look up this widget instance, by its id */
      if (*((iid_id_type *) data_ptr) != iid_null_id &&
	  *((iid_id_type *) data_ptr) != UNDEFINED_ID)
	{
	  w_inst = glue_get_by_id(*((iid_id_type *) data_ptr)) ;
	  if (w_inst == NULL) 
	    {
	      fprintf(stderr,"Six_set: Null widget instance found: id=%d\n",
		      *((iid_id_type *) data_ptr)) ;
	      return ;
	    }
      
	  if (cur_attr->access & AXS_TECH)
	    {
	      XtSetArg(args[*num_args], cur_attr->t_name, w_inst->widget) ;
	      (*num_args)++;
	    }
	}

      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((iid_id_type *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      
      break ;
      
    case GT_XFontStruct:
      ret_string = (char *)
	uc2_convert(idd_string, idd_buffer, (idd_buffer_type *) data_ptr) ; 
      SIX_DEBUG3 (fprintf (stderr, "Six_set: fontlist=%s  t_name=%s\n", 
		      ret_string, cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  if ((font = XLoadQueryFont(display, ret_string)) != NULL)
	    {
	      XtSetArg(args[*num_args], cur_attr->t_name, font) ;
	      (*num_args)++;
	    }
	  else
	    fprintf(stderr, "Six_set: font not found--using default\n") ;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  /* If a string had already been stored, free its space */
	  if (cur_attr->val_type == VAL_CURRENT) free_node(cur_attr->value.s);
	  else cur_attr->val_type = VAL_CURRENT ;
	  
	  /* Store the current string */
	  cur_attr->value.s = ret_string ;
	}
      else
	{
	  free_node(ret_string) ; 
	} 
      break ;
      
    case GT_XID:
      SIX_DEBUG3(fprintf (stderr, "Six_set: XID=%d  t_name=%s\n", 
			  *((int *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, *((int *) data_ptr));
	  (*num_args)++;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((int *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;
      
#ifdef HasMotif
    case GT_XmFontList:
      ret_string = (char *)
	uc2_convert(idd_string, idd_buffer, (idd_buffer_type *) data_ptr) ;
      SIX_DEBUG3 (fprintf (stderr, "Six_set: fontlist=%s  t_name=%s\n", 
		      ret_string, cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  if ((font = XLoadQueryFont(display, ret_string)) != NULL)
	    {
	      fontlist = XmFontListCreate(font, charset) ;
	      XtSetArg(args[*num_args], cur_attr->t_name, fontlist) ;
	      (*num_args)++;
	    }
	  else
	    fprintf(stderr, "Six_set: font not found--using default\n") ;
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  /* If a string had already been stored, free its space */
	  if (cur_attr->val_type == VAL_CURRENT) free_node(cur_attr->value.s);
	  else cur_attr->val_type = VAL_CURRENT ;
	  
	  /* Store the current string */
	  cur_attr->value.s = ret_string ;
	}
      else
	{
	  free_node(ret_string) ; 
	} 
      break ;
      
    case GT_XmString:
      ret_string = (char *)
	uc2_convert(idd_string, idd_buffer, (idd_buffer_type *) data_ptr) ;
      SIX_DEBUG3(fprintf (stderr, "Six_set: XmString=%s  t_name=%s\n", 
		      ret_string, cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, 
		   XmStringCreateLtoR(ret_string, charset)) ;
	  (*num_args)++;
	} 
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  /* If a string had already been stored, free its space */
	  if (cur_attr->val_type == VAL_CURRENT) free_node(cur_attr->value.s);
	  else cur_attr->val_type = VAL_CURRENT ;
	  
	  /* Store the current string */
	  cur_attr->value.s = ret_string ;
	}
      else
	{
	  free_node(ret_string) ; 
	} 
      break ;
      
    case GT_XmStringTable:
      ret_string = (char *)
	uc2_convert(idd_string, idd_buffer, (idd_buffer_type *) data_ptr) ;
      SIX_DEBUG3(fprintf (stderr, "Six_set: XmStringTable=%s  t_name=%s\n", 
		      ret_string, cur_attr->t_name));
      
      /* Create the string table */
      items = Six_string_table(ret_string, ListItems) ; 
     
      if (cur_attr->access & AXS_TECH)
	{
	   XtSetArg(args[*num_args], cur_attr->t_name, ListItems) ; 
	  (*num_args)++;
	} 
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  /* If a string had already been stored, free its space */
	  if (cur_attr->val_type == VAL_CURRENT) free_node(cur_attr->value.s);
	  else cur_attr->val_type = VAL_CURRENT ;
	  
	  /* Store the current string */
	  cur_attr->value.s = ret_string ;
	}
      else
	{
	  free_node(ret_string) ; 
	} 
      break ;
      
    case GT_XmStringDirection:
      SIX_DEBUG3(fprintf (stderr, "Six_set: XmStringDirection=%d  t_name=%s\n",
			  *((int *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, *((int *) data_ptr));
	  (*num_args)++;
	}

      if (cur_attr->access & AXS_SERPENT)
	{
	  cur_attr->value.i = *((int *) data_ptr) ;
	  cur_attr->val_type = VAL_CURRENT ;
	}
      break ;
#endif
      
    case GT_XtCallbackList:
      SIX_DEBUG3(fprintf (stderr, "Six_set: XtCallbackList=%d  t_name=%s\n", 
		      *((int *) data_ptr), cur_attr->t_name));
      
      if (cur_attr->access & AXS_TECH)
	{
	  callbacks[0].callback = *((XtCallbackProc *) data_ptr) ;
	  callbacks[0].closure = NULL ;
	  callbacks[1].callback = NULL ;
	  callbacks[1].closure = NULL ;
	  XtSetArg(args[*num_args], cur_attr->t_name, callbacks);
	  (*num_args)++;
	} 

      if (cur_attr->access & AXS_SERPENT) 
	cur_attr->value.c = *((caddr_t *) data_ptr) ;
      break ;

    case GT_XtTranslations:
      ret_string = (char *)
	uc2_convert(idd_string, idd_buffer, (idd_buffer_type *) data_ptr) ;
      SIX_DEBUG3(fprintf (stderr, "Six_set: XtTranslations=`%s'  t_name=%s\n", 
		      ret_string, cur_attr->t_name)); 

      if (cur_attr->access & AXS_TECH)
	{
	  XtSetArg(args[*num_args], cur_attr->t_name, 
		   XtParseTranslationTable(ret_string));
	  (*num_args)++; 
	}
      
      if (cur_attr->access & AXS_SERPENT)
	{
	  /* If a string had already been stored, free its space */
	  if (cur_attr->val_type == VAL_CURRENT) free_node(cur_attr->value.s);
	  else cur_attr->val_type = VAL_CURRENT ;
	  
	  /* Store the current string */
	  cur_attr->value.s = ret_string ;
	}
      else
	{
	  free_node(ret_string) ;  
	} 
      break ;

    default:
      fprintf(stderr,"Six_set:  type %d for `%s' not found--ignored\n", 
	      cur_attr->x_type, cur_attr->t_name) ;
      break ;
      
    }
  
  SIX_DEBUG1(fprintf (stderr, "Six_set: leaving\n")) ;
}


/*=========================================================================*\
 * Six_do_defaults gets the list of defaulted attributes from the 
 * widget definition and calls Six_set for each of them.
\*=========================================================================*/

Six_do_defaults(widget_inst, default_widget, args, num_args) 
     
     INST *widget_inst ;
     WIDG *default_widget ;
     ArgList args ;
     Cardinal *num_args ;
{
  ALIST attr_list ;
  ATTR *cur_attr ;
  char *cur_attr_name ;
  int i ;
  isd_change_type change = isd_create ;
  idd_buffer_type *temp_buffer ;
  
  SIX_DEBUG1(fprintf (stderr, "Six_do_defaults: entering\n")) ;
  SIX_DEBUG3(fprintf (stderr, "Six_do_defaults: INST=%d  WIDG=%d\n",
		      widget_inst, default_widget)) ;

  attr_list = glue_alist_defaulted_attributes(technology, 
					      default_widget->name, 
					      SERP_NAMES) ;
  
  cur_attr_name = get_alist_next(attr_list, NULL) ;
  SIX_DEBUG3(fprintf (stderr, "Six_do_defaults: attr_name=%s\n", 
		      cur_attr_name)) ;

  /* Look through the alist of attribute names, get the ATTR */
  /* structure for each and call Six_set to set the arg list */
  while (cur_attr_name != NULL)
    {
      cur_attr = glue_get_attribute(default_widget, cur_attr_name, 
				    SERP_NAMES) ;
      /* Check if the attribute really exists */
      if (cur_attr == NULL)
	{
	  fprintf(stderr, 
		  "Six_do_defaults:  ATTR not found for %s--ignoring\n",
		  cur_attr_name) ;
	}
      else
	{
	  /* If the attribute is expecting a buffer, then create  */
	  /* one and copy the defaulted attribute into it.	  */
	  if (cur_attr->s_type == idd_buffer)
	    {
	      temp_buffer = 
		(idd_buffer_type *)make_node(sizeof(idd_buffer_type));
	      temp_buffer->length = strlen(cur_attr->value.s);
	      temp_buffer->type = idd_string;
	      temp_buffer->body = make_node(temp_buffer->length);
	      strcpy(temp_buffer->body, cur_attr->value.s);

	      Six_set(change, widget_inst, cur_attr_name, temp_buffer, 
		      args, num_args) ;

	      /* Free the buffer here, but not the storage for the  */
	      /* string (since Six_set will do that)		    */
	      free_node(temp_buffer) ;
	    }
	  else 
	    Six_set(change, widget_inst, cur_attr_name, &(cur_attr->value.c), 
		    args, num_args) ;
	  cur_attr_name = get_alist_next(attr_list, cur_attr_name) ;
	  SIX_DEBUG3(fprintf (stderr, "Six_do_defaults: attr_name=%s\n", 
			      cur_attr_name)) ;
	}
    }

  SIX_DEBUG1(fprintf (stderr, "Six_do_defaults: leaving\n")) ;
}


/*=========================================================================*\
 * Six_sift looks at a transaction and returns three (possibly empty)
 * lists: removes, creates and modifies.  
\*=========================================================================*/

void Six_sift(removes, creates, modifies) 
   LIST removes, creates, modifies ;
{
   iid_id_type id, *temp_id ;
   isd_change_type change ;
  
   SIX_DEBUG1(fprintf (stderr, "Six_sift: entering\n")) ;

   /* Get the element name from the transaction */
   id = get_first_changed_element (transaction);
   SIX_DEBUG3(fprintf (stderr, "Six_sift: id=%d\n", id));
  
  if (id != iid_null_id)
    {
      while (id != iid_null_id)
	{
	  change = get_change_type(transaction, id);

	  switch(change)
	    {
	    case isd_create:
	      temp_id = (iid_id_type *) make_node(sizeof(iid_id_type)) ;
	      *temp_id = id ;
	      add_to_tail(creates, temp_id) ;

	      break ;

	    case isd_modify:
	      temp_id = (iid_id_type *) make_node(sizeof(iid_id_type)) ;
	      *temp_id = id ;
	      add_to_tail(modifies, temp_id) ;

	      break ;

	    case isd_remove:
	      temp_id = (iid_id_type *) make_node(sizeof(iid_id_type)) ;
	      *temp_id = id ;
	      add_to_tail(removes, temp_id) ;

	      break ;

	    default:
	      fprintf(stderr, "Six_sift: unknown change type %d\n", change) ;
	      
	      break ;
	    }	  

	  id = get_next_changed_element(transaction) ;
	  SIX_DEBUG3(fprintf (stderr, "Six_sift: id=%d\n", id));
	}
    }
  SIX_DEBUG1(fprintf (stderr, "Six_sift: leaving\n")) ;
}


/*=========================================================================*\
 * Six_manage manages widgets which were created in the current transaction.
\*=========================================================================*/
void Six_manage(order_list) 
     LIST order_list ;
{
  INST *cur_inst ;
  iid_id_type *id ;
  ATTR *manage_attr ;

  SIX_DEBUG1(fprintf (stderr, "Six_manage: entering\n")) ;
  
  id = (iid_id_type *) get_list_next(order_list, NULL) ; 
  while (id != NULL)
    {
      if (*id != iid_null_id && *id != UNDEFINED_ID)
	{
	  if ((cur_inst = glue_get_by_id(*id)) == NULL) 
	    {
	      fprintf(stderr, "Six_manage: Problem with id_list\n") ;
	      exit(1) ;
	    }
	  else
	    {
	      if (cur_inst->widget != Screen_id &&
		  cur_inst->template->widget_type != WT_WM_SHELL &&
		  cur_inst->template->widget_type != WT_OVERRIDE_SHELL)
		{
		  /* Make sure the widget is managed and realized */
		  SIX_DEBUG3(fprintf (stderr, "Six_manage: id=%d widget=%d\n",
				      *id, cur_inst->widget)) ;
		  
		  manage_attr = glue_get_attr_from_instance(cur_inst,
					  "manage", SERP_NAMES) ;
		  if (!XtIsManaged(cur_inst->widget) &&
		      (manage_attr == NULL || 
		       (manage_attr != NULL && manage_attr->value.i == TRUE)))
		    XtManageChild(cur_inst->widget) ; 
		  if (!XtIsRealized(XtParent(cur_inst->widget)))
		    XtRealizeWidget(XtParent(cur_inst->widget)) ; 
	       } 
	   } 
       }
      id = (iid_id_type *) get_list_next(order_list, id) ; 
   }
    
  SIX_DEBUG1(fprintf (stderr, "Six_manage: leaving\n")) ;
}



/*=========================================================================*\
 * print_ids prints the contents of an id list (called from Six_dispatch)
\*=========================================================================*/
static print_ids(dummy, id)
     int dummy ;
     iid_id_type *id ;
{
  fprintf(stderr, "    id=%d\n", *id) ;
}


/*=========================================================================*\
 * Six_dispatch looks at a list of ids from a transaction and if they're
 * removes or modifies, dispatches them (to Six_process).  If the ids are
 * creates, Six_dispatch checks to see that there are no unsatisfied 
 * dependencies before dispatching an element.  If nothing can be removed
 * from the list, the process reports an error and dies.
\*=========================================================================*/
LIST Six_dispatch(change, id_list) 
     LIST id_list ;
     isd_change_type change ;
{
  LIST components, order_list ;
  string el_name, comp_name ;
  idd_data_types type ;
  boolean OK ;
  int checked = 0 ;
  int position = 1 ;
  char *data ;
  iid_id_type *id, *test_id, *id_temp ;
  int list_len = list_length(id_list) ;
  isd_trans to_dm ;
  boolean Six_sort_removes() ;

  SIX_DEBUG1(fprintf (stderr, "Six_dispatch: entering\n")) ;
  SIX_DEBUG3(fprintf (stderr, "Six_dispatch: list_len=%d  change=%d\n",
		      list_len, change)) ;

  if (list_len == 0) return(NULL) ;
  
  if (change == isd_modify)
    {
      id = (iid_id_type *) get_list_next(id_list, NULL) ;
      while (id != NULL)
	{
	  Six_process(change, *id, to_dm) ;
	  id = (iid_id_type *) get_list_next(id_list, id) ;
	}
      return(NULL) ;
    }

  if (change == isd_remove)
    {
       if (list_len > 1) sort_list_func(id_list, Six_sort_removes) ;
       id = (iid_id_type *) get_list_next(id_list, NULL) ;
       while (id != NULL)
	 {
	    Six_process(change, *id, to_dm) ;
	    id = (iid_id_type *) get_list_next(id_list, id) ;
	 }
       return(NULL) ;
    }
  
  if (change == isd_create)
    {
       /* Create a list to record the order in which elements are */
       /* serviced from the transaction				 */
       order_list = make_list() ;

       /* Start a transaction for info going back to the dialogue */
       to_dm = start_transaction() ;

       /* Grab ids from the list, one at a time */
       id = (iid_id_type *) get_list_pos(id_list, position) ;
       while (id != NULL)
	 {
	    /* Get the list of changed components */
	    components = create_changed_component_list(transaction, *id) ;
	    if (components == NULL)
	      {
		 fprintf(stderr, 
		      "Six_dispatch: can't create component list--exiting\n") ;
		 exit(1) ;
	      }

	    el_name = get_element_name(transaction, *id) ;

	    /* Look at each changed component to see if it is of type "id" */
	    comp_name = get_list_next(components, NULL) ;
	    OK = TRUE ;
	    while (comp_name != NULL && OK)
	      {
		 type = get_shared_data_type(el_name, comp_name) ;
		 
		 /* If the type is an id, check that the object for this */
		 /* id already exists--if not, skip to the next id.      */
		 if (type == idd_id)
		   {
		      test_id = (iid_id_type *)
			get_shared_data(transaction, *id, comp_name);
		      if (*test_id != iid_null_id && *test_id != UNDEFINED_ID)
			if (glue_get_by_id(*test_id) == NULL) OK = FALSE ;
		      free_node(test_id) ;
		   }
		 comp_name = get_list_next(components, comp_name) ;
	      }
	    
	    /* Free the component list */
	    destroy_changed_component_list(components) ;
	    
	    /* If no unsatisfied dependencies, send this id to be processed */
	    if (OK) 
	      {
		 Six_process(change, *id, to_dm) ;
		 checked = 0 ;
		 id_temp = (iid_id_type *) make_node(sizeof(iid_id_type)) ;
		 *id_temp = *id ;
		 add_to_tail(order_list, id_temp) ;
		 
		 if (delete_list_pos(id_list, position) == NULL) 
		   {
		      fprintf(stderr, 
			      "Six_dispatch: id_list is corrupted--exiting\n");
		      exit(1) ;
		   }
		 list_len-- ;
	      }
	    else
	      {
		 checked++ ;
		 position++ ;
	      }
	    
	    if (position > list_len) position = 1 ;
	    
	    if (checked > list_len)
	      {
		 /* If we are here, there is an infinite dependency loop. */
		 fprintf(stderr, 
			 "Six_dispatch: Infinite loop detected--exiting\n") ;
		 for_all_list(id_list, NULL, print_ids) ;
		 exit(1) ; 
	      }

	    id = (iid_id_type *) get_list_pos(id_list, position) ;
	 }
       
       /* Commit the transaction back to the dm */
       commit_transaction(to_dm) ;
       return(order_list) ;
    }

  SIX_DEBUG1(fprintf (stderr, "Six_dispatch: leaving\n")) ;
}


/*=========================================================================*\
 * Six_sort_removes is a sort function called by sort_list_func (for removes).
 * It compares two shared data elements and checks to see if id2 is the parent
 * of id1.  If so, it returns FALSE, otherwise it returns TRUE.
\*=========================================================================*/
boolean Six_sort_removes(id1, id2) 
   iid_id_type *id1, *id2 ;

{
   INST *cur_inst ;
   ATTR *parent_attr ;

   SIX_DEBUG1(fprintf (stderr, "Six_sort_removes: entering\n")) ;
   SIX_DEBUG3(fprintf (stderr, "Six_sort_removes: id1=%d  id2=%d\n", 
		       *id1, *id2)) ;

   /* Check to see if id1 has a parent.	*/
   if ((cur_inst = glue_get_by_id(*id1)) == NULL) 
     {
	fprintf(stderr, "Six_sort_removes: No INST for id1 (%d)\n", *id1) ;
	exit(1) ;
     }
   
   /* Get the ATTR structure for the parent */
   parent_attr = glue_get_attr_from_instance(cur_inst, "parent", SERP_NAMES) ;

   if (parent_attr == NULL) return(TRUE) ;
   
   /* Compare the ids */
   if (parent_attr->val_type == VAL_CURRENT &&
       parent_attr->value.i != iid_null_id)
     {
	if (parent_attr->value.i == *id2) return(FALSE) ;
	else
	  return(TRUE) ;
     }
   else
     return(TRUE) ;
}


#ifdef HasMotif
int Six_string_table(in_string, items) 
   char *in_string ;
   XmString items[] ;
{
   int i, str_pos, temp_pos, last, item_count, str_len;
   static char temp[TempStringSize] ;

   str_len = strlen(in_string) ;
   last = 0 ;
   item_count = 0 ;   

   for (i=0; i<=str_len; i++)
     {
	if (in_string[i] == '\0' || 
	    (in_string[i] == ListSeparator && in_string[i+1] == ListSeparator))
	  {
	     /* Copy the string into a temporary string */
	     for (str_pos=last, temp_pos=0;
		  str_pos<i && temp_pos<TempStringSize-1;
		  str_pos++, temp_pos++)
	       temp[temp_pos] = in_string[str_pos]; 
	     temp[temp_pos] = '\0' ;

	     /* Create a compound string from the temporary string */
	     items[item_count] = 
	       XmStringCreateLtoR(temp, charset); 

	     last = i+2 ;
	     item_count++ ;
	     if (item_count > StringTableSize)
	       {
		  fprintf(stderr, "Six_string_table: Too many items\n") ;
		  return ;
	       }
	  }
     }
   return(item_count) ;
}
#endif
