	static char rcsid [] = "$Header: /u/04c/mnt/integ/source/tools/src/RCS/dm_dump.c,v 1.4 1991/07/12 12:38:59 ejh Exp $";

/*--------------------------------------------------------------------------*\
|
|  Name:  Dialogue Manager Interpreter Dump (dm_dump)
|
|  Description:  
|	This program reads a dialogue binary and prints it in a readable
|	fashion.  It takes the name of the dialogue as an argument, followed
|	by an optional sniippet number.  Without the optional parameter,
|	it will print the entire binary.  With the parameter, it will print
|	only that snippet.  Example:
|
|		dm_dump dialogue [snip-num]
|
|	It looks a lot like dri.c because it is modelled after dri.c
|
|  Component of:
|	Dialogue Manager
|
|  Related files:
|	dri.h - contains the definitions of the interpreter constants
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:  
|	Daniel Klein
|	dvk@sei.cmu.edu
|
|*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

/*
 * $Log: dm_dump.c,v $
 * Revision 1.4  1991/07/12  12:38:59  ejh
 * put in a missing stderr on an fprintf
 *
 * Revision 1.3  1991/02/13  15:08:41  dvk
 * Added facility so that when C_dob_int_const and C_dob_real_const is
 * printed, dm_dump also prints the value of the constant (not just the
 * index in the constant array).
 *
 * Revision 1.2  91/01/31  17:37:51  little
 * add knowledge of the rec on &off and the sne_validate instructions
 * 
 * Revision 1.1  91/01/03  19:37:27  dvk
 * Initial revision
 * 
 */

#include	<string.h>
#include	<string.h>
#include	<dri.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<sys/stat.h>

extern char *malloc ();

static int dri_max_snippets;
static int dri_string_size;

static short *dri_code;
static char *dri_string_table;
static long *dri_snip_table;
static long snip_code_size;

static long *iconst;
static double *rconst;

void dri_interpret (snip_num)
int snip_num;
{
    int i;
    dri_token token;
    int num_user_parms;
    long long_temp;
    long dri_pc;
    int snip_idx;
    long last_const;

    snip_idx = (snip_num == -1 ? 0 : snip_num);
    dri_pc = dri_snip_table[snip_idx];
    while (dri_pc < snip_code_size) {
	if (dri_snip_table[snip_idx] == dri_pc) {
	    fprintf (stdout, "======== Begin Snippet %d ========\n", snip_idx);
	    fprintf (stdout, "%5d:\t", dri_pc);
	    snip_idx++;
	    }
	switch (token = (dri_token)dri_code[dri_pc++]) {
	    case L_negative:
		fprintf (stdout, "%d ", last_const = dri_code[dri_pc++]);
		break;
	    /*
	     * Grab the next two words out of the instruction stream, and
	     * construct a longword.  These are the only places where the
	     * issue of big-endian vs. little-endian arises in the
	     * interpreter.  Most code compiled on one platform "should"
	     * be executable on another, but this checks for it.
	     */
	    case L_long:
#ifndef ENDIAN_SWAP
		long_temp = ((unsigned short)dri_code[dri_pc++] &0xffff);
		long_temp |= (unsigned short)dri_code[dri_pc++] << 16;
#else
		long_temp = (unsigned short)dri_code[dri_pc++] << 16;
		long_temp |= ((unsigned short)dri_code[dri_pc++] & 0xffff);
#endif
		fprintf (stdout, "%ld", last_const = long_temp);
		if ((long_temp < dri_string_size) &&
		    (dri_string_table[long_temp] != '\0') &&
		    (dri_string_table[long_temp-1] == '\0'))
		    fprintf (stdout, "<%s>", &dri_string_table[long_temp]);
		fputc (' ', stdout);
		break;
	    case R_get_parent_vc:
		fputs ("R_get_parent_vc\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_creating_sd:
		fputs ("R_get_creating_sd\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_object:
		fputs ("R_get_object\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_component_value:
		fputs ("R_get_component_value\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_put_component_value:
		fputs ("R_put_component_value\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_vc_id:
		fputs ("R_get_vc_id\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_variable_value:
		fputs ("R_get_variable_value\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_put_variable_value:
		fputs ("R_put_variable_value\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_create_element:
		fputs ("R_create_element\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_delete_element:
		fputs ("R_delete_element\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_name:
		fputs ("R_get_name\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_type:
		fputs ("R_get_type\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    /*
	     * The only easy way out of here (other than T_return)
	     */
	    case R_exit:
		fputs ("R_exit\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_id_exists:
		fputs ("R_id_exists\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_rec_on:
		fputs ("R_rec_on\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_rec_off:
		fputs ("R_rec_off\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;

	    case R_and:
		fputs ("R_and\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_or:
		fputs ("R_or\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_dot:
		fputs ("R_dot\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_neq:
		fputs ("R_neq\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_leq:
		fputs ("R_leq\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_lss:
		fputs ("R_lss\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_geq:
		fputs ("R_geq\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_gtr:
		fputs ("R_gtr\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_assign:
		fputs ("R_assign\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_eql:
		fputs ("R_eql\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_add:
		fputs ("R_add\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_sub:
		fputs ("R_sub\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_neg:
		fputs ("R_neg\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_not:
		fputs ("R_not\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_div:
		fputs ("R_div\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_exp:
		fputs ("R_exp\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_mul:
		fputs ("R_mul\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_new:
		fputs ("R_new\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;

	    case R_list_assign:
		fputs ("R_list_assign\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_validate_object:
		fputs ("R_validate_object\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_boolean:
		fputs ("R_boolean\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_boolean_atv:
		fputs ("R_get_boolean_atv\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_integer_atv:
		fputs ("R_get_integer_atv\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_real_atv:
		fputs ("R_get_real_atv\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_string_atv:
		fputs ("R_get_string_atv\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_record_atv:
		fputs ("R_get_record_atv\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_id_atv:
		fputs ("R_get_id_atv\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_buffer_atv:
		fputs ("R_get_buffer_atv\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_undefined_atv:
		fputs ("R_get_undefined_atv\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_make_boolean_constant_adi:
		fputs ("R_make_boolean_constant_adi\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_make_buffer_constant_adi:
		fputs ("R_make_buffer_constant_adi\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_make_integer_constant_adi:
		fputs ("R_make_integer_constant_adi\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_make_indir_string_constant_adi:
		fputs ("R_make_indir_string_constant_adi\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_make_immed_string_constant_adi:
		fputs ("R_make_immed_string_constant_adi\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_make_real_constant_adi:
		fputs ("R_make_real_constant_adi\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_make_id_constant_adi:
		fputs ("R_make_id_constant_adi\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_make_undefined_adi:
		fputs ("R_make_undefined_adi\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_sire_vc:
		fputs ("R_get_sire_vc\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_OBJECT:
		fputs ("R_get_OBJECT\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_put_OBJECT:
		fputs ("R_put_OBJECT\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_variable_VALUE:
		fputs ("R_get_variable_VALUE\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_put_variable_VALUE:
		fputs ("R_put_variable_VALUE\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_get_component_VALUE:
		fputs ("R_get_component_VALUE\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_put_component_VALUE:
		fputs ("R_put_component_VALUE\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_bool_assign:
		fputs ("R_bool_assign\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_int_assign:
		fputs ("R_int_assign\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_id_assign:
		fputs ("R_id_assign\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_string_assign:
		fputs ("R_string_assign\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case R_make_adi:
		fputs ("R_make_adi\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;

	    case T_and:
		fputs ("T_and\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_if:
		fputs ("T_if\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_goto:
		fputs ("T_goto\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_return:
		fputs ("T_return\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_snip_end:
		fputs ("T_snip_end\n", stdout);
		if (snip_num != -1)
		    return;
		break;
	    case T_compile_error:
		fputs ("*** T_compile_error ***\n", stdout);
		fputc ('\t', stdout);
		break;

	    case T_simple_parm_init:
		fputs ("T_simple_parm_init\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_complex_parm_init:
		fputs ("T_complex_parm_init\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_adi_from_parm:
		fputs ("T_adi_from_parm\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_component_value_from_parm:
		fputs ("T_component_value_from_parm\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_vc_list_init:
		fputs ("T_vc_list_init\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_dpt_vc_exists:
		fputs ("T_dpt_vc_exists\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_validate_sne:
		fputs ("T_validate_sne\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_validate_sne_cc:
		fputs ("T_validate_sne_cc\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_not_dpt_rite_rule:
		fputs ("T_not_dpt_rite_rule\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_vc_create:
		fputs ("T_vc_create\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_get_PARENT_vc:
		fputs ("T_get_PARENT_vc\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_dmsg_setup:
		fputs ("T_dmsg_setup\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;

	    case T_string_constant:
		fputs ("T_string_constant ", stdout);
		break;
	    case T_slang_file:
		fputs ("T_slang_file\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_slang_line_num:
		fputs ("T_slang_line_num\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_reset_slang_line_num:
		fputs ("T_reset_slang_line_num\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_method_verify:
		fputs ("T_method_verify\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_void_user_function:
		fputs ("T_void_user_function\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;
	    case T_user_function:
		fputs ("T_user_function\n", stdout);
		fprintf (stdout, "%5d:\t", dri_pc);
		break;

	    case C_SELF:
		fputs ("C_SELF ", stdout);
		break;
	    case C_PARENT:
		fputs ("C_PARENT ", stdout);
		break;
	    case C_vc_sym:
		fputs ("C_vc_sym ", stdout);
		break;
	    case C_object_sym:
		fputs ("C_object_sym ", stdout);
		break;
	    case C_variable_sym:
		fputs ("C_variable_sym ", stdout);
		break;
	    case C_cus_ATTR:
		fputs ("C_cus_ATTR\n     :\t", stdout);
		break;
	    case C_cus_CREATE:
		fputs ("C_cus_CREATE\n     :\t", stdout);
		break;
	    case C_cus_DESTROY:
		fputs ("C_cus_DESTROY\n     :\t", stdout);
		break;
	    case C_cus_METH:
		fputs ("C_cus_METH\n     :\t", stdout);
		break;
	    case C_cus_CCOND:
		fputs ("C_cus_CCOND\n     :\t", stdout);
		break;
	    case C_cus_VAR:
		fputs ("C_cus_VAR\n     :\t", stdout);
		break;
	    case C_dob_id_NULL:
		fputs ("C_dob_id_NULL ", stdout);
		break; 
	    case C_dob_bool_TRUE:
		fputs ("C_dob_bool_TRUE ", stdout);
		break; 
	    case C_dob_bool_FALSE:
		fputs ("C_dob_bool_FALSE ", stdout);
		break; 
	    case C_dob_int_const:
		fprintf (stdout, "C_dob_int_const(%d) ", iconst[last_const]);
		break; 
	    case C_dob_real_const:
		fprintf (stdout, "C_dob_real_const(%f) ", rconst[last_const]);
		break; 

	    default:
		/*
		 * If the token is negative, we are lost - all negative
		 * numbers are command tokens, and we missed this one.
		 *
		 * If the token is positive, it is a literal value, so
		 * just push it into the stack.
		 */
		if ((int)token < 0)
		     fprintf (stderr, "Unknown token! %d\n", (int)token);
		else {
		    fprintf (stdout, "%d", last_const = (int)token);
		    if (((int)token < dri_string_size) &&
			(dri_string_table[(int)token] != '\0') &&
			(dri_string_table[(int)token-1] == '\0'))
			fprintf (stdout, "<%s>", &dri_string_table[(int)token]);
		    fputc (' ', stdout);
		    }
	    }
	}
}

void dri_init (dlg_name)
char *dlg_name;
{
    int i;
    int fd;
    char *fname;
    dri_header file_header;
    dri_int itmp;
    dri_real rtmp;
    struct stat statbuf;

    fname = malloc (1 /*.*/ + strlen (dlg_name) + 2 /*.z*/ + 1 /*\0*/ );
    /*
     * Read in the string/constants table.  This entails reading in the
     * string table, then reading in the integer constants (and making
     * the associated adi's), and repeating with the reals.
     */
    (void)strcat (strcat (strcpy (fname, "."), dlg_name), ".z");
    if ((fd = open (fname, 0)) == -1) {
	perror ("Cannot open string/constants table");
	exit (1);
	}
    fstat (fd, &statbuf);
    if (statbuf.st_size < sizeof (dri_header)) {
	fputs ("Cannot read string/constants table header!\n", stderr);
	exit (1);
	}
    read (fd, &file_header, sizeof (dri_header));
    if (file_header.magic != DRI_MAGIC) {
	fprintf (stderr, "%s is not a string/constants file!\n", fname);
	exit (1);
	}
    dri_max_snippets = file_header.max_snippets;
    dri_string_size = file_header.string_size;
    fprintf (stdout, "=+==+==+==+==+==+==+==+==+==+==+==+==+==+==+=\n");
    fprintf (stdout, "Created with DRI_VERSION = %ld\n", file_header.version);
    fprintf (stdout, "%ld snippets\n", file_header.max_snippets);
    fprintf (stdout, "%ld bytes in string table\n", file_header.string_size);
    fprintf (stdout, "%ld integer constants\n", file_header.int_const_size);
    fprintf (stdout, "%ld real constants\n", file_header.real_const_size);
    if (statbuf.st_size != sizeof (dri_header) +
			    file_header.string_size +
			    (file_header.int_const_size * sizeof (dri_int)) +
			    (file_header.real_const_size * sizeof (dri_real))) {
	fputs ("String/constant declared header size != file size!\n", stderr);
	exit (1);
	}
    if ((dri_string_table = malloc (file_header.string_size)) == NULL) {
	fputs ("Cannot allocate string table!\n", stderr);
	exit (1);
	}
    read (fd, dri_string_table, file_header.string_size);
    if (file_header.int_const_size > 0) {
	iconst = (long *) make_node (
			    file_header.int_const_size * sizeof (long));
	if (iconst == NULL) {
	    fputs ("Cannot allocate integer constants table!\n", stderr);
	    exit (1);
	    }
	}
    for (i = 0; i < file_header.int_const_size; i++) {
	read (fd, &itmp, sizeof (dri_int));
	iconst[itmp.idx] = itmp.val;
	}

    if (file_header.real_const_size > 0) {
	rconst = (double *) make_node (
			    file_header.real_const_size * sizeof (double));
	if (rconst == NULL) {
	    fputs ("Cannot allocate real constants table!\n", stderr);
	    exit (1);
	    }
	}
    for (i = 0; i < file_header.real_const_size; i++) {
	read (fd, &rtmp, sizeof (dri_real));
	rconst[rtmp.idx] = rtmp.val;
	}
    close (fd);

    /*
     * Read in the snippet pointer table, and create the PARENT array
     * and initialize it.
     */
    (void)strcat (strcat (strcpy (fname, "."), dlg_name), ".t");
    if ((fd = open (fname, 0)) == -1) {
	perror ("Cannot open snippet index table");
	exit (1);
	}
    fstat (fd, &statbuf);
    if ((statbuf.st_size / sizeof (long)) != dri_max_snippets) {
	fputs ("Snippet count != snippet indirect table size!\n", stderr);
	exit (1);
	}
    if ((dri_snip_table = (long *)malloc (statbuf.st_size)) == NULL) {
	fputs ("Cannot Cannot allocate snippet indirect table!\n", stderr);
	exit (1);
	}
    read (fd, dri_snip_table, statbuf.st_size);
    close (fd);

    /*
     * Read in the snippet table, and initialize it.
     */
    (void)strcat (strcat (strcpy (fname, "."), dlg_name), ".x");
    if ((fd = open (fname, 0)) == -1) {
	perror ("Cannot open snippet table");
	exit (1);
	}
    fstat (fd, &statbuf);
    if ((dri_code = (short *)malloc (statbuf.st_size)) == NULL) {
	fputs ("Cannot Cannot allocate snippet code table!\n", stderr);
	exit (1);
	}
    read (fd, dri_code, statbuf.st_size);
    snip_code_size = statbuf.st_size / sizeof (short);
    close (fd);
}

main(argc, argv, envp)
int argc;
char **argv, **envp;
{
    if (argc < 2 || argc > 3) {
	fprintf (stderr, "Usage: %s dialogue [snippet-number]\n", *argv);
	exit(1);
	}
    dri_init (argv[1]);
    dri_interpret (argc == 3 ? atoi(argv[2]) : -1);
    exit (0);
}
