        static char rcsid [] = "$Header: /u/04c/mnt/integ/source/tools/src/RCS/serp_exec.c,v 1.2 91/01/07 00:17:41 ejh Exp $";

/*--------------------------------------------------------------------------*\
|
|  Name:	serp_exec.c
|
|  Description:
|	This module handles execution of subprograms
|
|  Component of:
|
|  Related files:
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|	Len Bass
|	ljb@sei.cmu.edu
|
|  Author:  
|	Daniel V. Klein
|	dvk@sei.cmu.edu
|
|  Change History:
$Log:	serp_exec.c,v $
 * Revision 1.2  91/01/07  00:17:41  ejh
 * no change
 * 
 * Revision 1.1  91/01/02  17:59:31  dvk
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/
#include "memoryPack.h"
#include "alistPack.h"
#include <stdio.h>
#include <sys/wait.h>
#include <string.h>

extern char *getenv ();

void fork_and_path_exec (cmd, args, envs)
char *cmd;
char **args, **envs;
{
    static ALIST exe_alist = NULL;
    NODE cur;
    char *env, *str;
    char fname[BUFSIZ];
    union wait child_status;

    /*
     * Initialize the execute path, if necessary
     */
    if (exe_alist == NULL) {
	exe_alist = make_alist();
	if ((env = getenv ("PATH")) != NULL) {
	    str = strcpy (make_node (strlen (env)+1), env);
	    (void) strtok (str, ":");
	    (void) add_to_atail (exe_alist, str);
	    while (str = strtok (NULL, ":"))
		(void) add_to_atail (exe_alist, str);
	    }
	else {
	    fputs ("Warning: PATH not found!\n", stderr);
	    add_to_atail (exe_alist, ".");
	    add_to_atail (exe_alist, "/bin");
	    add_to_atail (exe_alist, "/usr/bin");
	    }
	}

    switch (fork()) {
	case -1:	/* Error */
	    fprintf (stderr, "?fork() for %s failed\n", cmd);
	    exit(1);
	    break;
	case 0:		/* Child process */
	    /*
	     * Traverse the execution path
	     */
	    cur = NULL;
	    while ((cur = get_alist_next (exe_alist, cur)) != NULL) {
		(void) strcat (strcat (strcpy (fname, cur), "/"), cmd);
		if (strlen (fname) + strlen (cmd) + 1 >= BUFSIZ)
		    fprintf (stderr, "Warning: Command %s/%s too big!\n", fname, cmd);
		execve (fname, args, envs);
		}
	    /*
	     * If we get here, we couldn't exec the file - error!
	     */
	    fprintf (stderr, "?Internal error - execve of %s ", cmd);
	    perror ("failed");
	    exit(1);
	default:	/* Parent process */
	    (void) wait(&child_status);
	    if (WIFSIGNALED (child_status))
		fprintf (stderr, "?%s terminated with a signal%s\n", cmd,
		    child_status.w_coredump ? " and dumped core" : "");
	    if (child_status.w_retcode || WIFSIGNALED (child_status))
		exit (1);
	    break;
	}
}


void serp_exec (cmd, args, envs)
char *cmd;
char **args, **envs;
{
    static ALIST exe_alist = NULL;
    NODE cur;
    char *env, *str;
    char fname[BUFSIZ];

    /*
     * Initialize the execute path, if necessary
     */
    if (exe_alist == NULL) {
	exe_alist = make_alist();
	if ((env = getenv ("SERPENT_EXE_PATH")) != NULL) {
	    str = strcpy (make_node (strlen (env)+1), env);
	    (void) strtok (str, " ");
	    (void) add_to_atail (exe_alist, str);
	    while (str = strtok (NULL, " "))
		(void) add_to_atail (exe_alist, str);
	    }
	else {
	    fputs ("Warning: SERPENT_EXE_PATH not found!\n", stderr);
	    add_to_atail (exe_alist, ".");
	    }
	}
    /*
     * Traverse the execution path
     */
    cur = NULL;
    while ((cur = get_alist_next (exe_alist, cur)) != NULL) {
	(void) strcat (strcat (strcpy (fname, cur), "/"), cmd);
	if (strlen (fname) + strlen (cmd) + 1 >= BUFSIZ)
	    fprintf (stderr, "Warning: Command %s/%s too big!\n", fname, cmd);
	execve (fname, args, envs);
	}
    /*
     * If we get here, we couldn't exec the file - error!
     */
    fprintf (stderr, "?Internal error - execve of %s ", cmd);
    perror ("failed");
}

