	static char rcsid [] = "$Header: /u/04c/mnt/integ/source/tools/src/RCS/serp_init.c,v 1.3 91/02/18 16:41:49 dvk Exp $";
/*--------------------------------------------------------------------------*\
|
|  Name:  Initialization and Cleanup (serp_init)
|
|  Description:  
|     Peforms necessary initialization and cleanup for the interface layer.
|
|  Component of:
|     Serpent run command
|
|  Related files:
|     serp.c
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Author:  
|     Daniel V. Klein
|     dvk@sei.cmu.edu  
|
|  Change History:
$Log:	serp_init.c,v $
 * Revision 1.3  91/02/18  16:41:49  dvk
 * Squashed a bug where the DM message queue was not being deleted
 * 
 * Revision 1.2  91/01/07  00:21:48  ejh
 * fixed includes to work under sysv, esp. hpux
 * 
 * Revision 1.1  91/01/02  18:00:02  dvk
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#include <stdio.h>
#include <ctype.h>
#include <alistPack.h>
#include <X11/Xos.h>

#include <global.h>		/* Global type defs and constants	    */

#include <ism.h>		/* shared memory abstraction		    */
#include <iid.h>		/* id abstraction			    */
#include <ids.h>		/* data synchronization abstraction	    */
#include <ipc.h>		/* process communication abstraction	    */
#include <isc.h>
#include <ipb.h>

extern char *getenv ();
extern int getppid ();
extern FILE *serp_open ();
extern void serp_clean ();
extern short verbose;

static ALIST env_alist = NULL;

/*--------------------------------------------------------------------------*\
|  Name:  match_env
| 
|  Description:  
|     Tells if an environment variable is found in our internal alist
\*--------------------------------------------------------------------------*/
static int match_env (s1, s2)
char *s1, *s2;
{
    register len = strlen (s1);

    return ((strncmp (s1, s2, len) == 0) && (s2[len] == '='));
}

/*--------------------------------------------------------------------------*\
|  Name:  Serpent Init
| 
|  Description:  
|     Peforms necessary initializations for the interface abstraction.
\*--------------------------------------------------------------------------*/


char **serp_init (dialogue, envp, fake_init)
char *dialogue;
char **envp;
short fake_init;
{
    FILE *env_fd, *init_fd;
    NODE cur;
    char **new_env;
    char *user_name;
    char foo [BUFSIZ];
    int i;
    ipc_mailbox mailbox;
    char mailbox_name[33];
    char *init_file_name;
    char env_file_name[50];	/* Enough for /tmp/<user>.<ppid> */
    char *dm_box;
    int did_dm_box = 0;
    int semaphore_id;
    int *shared_memory_id;
    iid_counter counter;
  
    /*
     * Open file .dialoguei.d to read.
     */
    init_file_name = make_node (strlen (dialogue) + 1 + 3 + 1);
    (void) strcat (strcat (strcpy (init_file_name, "."), dialogue), "i.d");
    init_fd = serp_open (init_file_name, "r");

    /*
     * Create/open file the environment file for writing.
     */

    if (fake_init) {
	/*
	** Don't use cuserid() to get user name.  The calling shell will refer
	** to environment var USER to get the user name, and cuserid doesn't
	** always return the same thing USER is set to (particularly after an
	** su(1)).  Instead, get the user name by querying the USER environment
	** variable.
	*/
	if ((user_name = getenv ("USER")) == NULL || strlen (user_name) == 0) {
	    fputs ("serp_init: USER environment variable is empty or missing\n", stderr);
	    exit (1);
	    }

	else
	    (void) sprintf (env_file_name, "/tmp/%s.%d", user_name, getppid ());

	if ((env_fd = fopen (env_file_name, "w")) == NULL) {
	    fprintf (stderr, "serp_init:  error opening %s\n", env_file_name);
	    exit (1);
	    }
	else
	    fprintf (stdout, "Created %s\n", env_file_name);
	}
    else
	env_alist = make_alist ();
    /*
     * Read mailbox names from  .dialoguei.d file.
     * Create mailboxes, assign to environment variables, and (optionally)
     * write to serpent_environment file.  Create a dummy name +DM_BOX which
     * is the same as the dialogue name.  If there is not _BOX for the
     * environment name, then cobble up a +DM_BOX by itself.
     */
    dm_box = make_node (strlen (dialogue) + 4 /* _BOX */ + 1 /* \0 */ );
    (void) sprintf (dm_box, "%s_BOX", dialogue);
    for (i = 0; dm_box[i]; i++)
	if (islower (dm_box[i]))
	    dm_box[i] = toupper (dm_box[i]);
    while (fscanf (init_fd, "%s %*s", mailbox_name) != EOF) {
	mailbox = ipc_create();
	if (get_status ()) {
	    fprintf (stderr, "?Error %d in ipc_create\n", get_status ());
	    exit (1);
	    }

	if (fake_init) {
	    fprintf (env_fd, "setenv %s %d\n", mailbox_name, mailbox);
	    if (strcmp (mailbox_name, dm_box) == 0) {
		fprintf (env_fd, "setenv %s %d\n", DM_BOX, mailbox);
		did_dm_box++;
		}
	    }
	else {
	    (void) sprintf (foo, "%s=%d", mailbox_name, mailbox);
	    add_to_atail (env_alist, strcpy(make_node(strlen(foo)+1), foo));
	    if (verbose)
		fprintf (stdout, "%s\n", foo);
	    if (strcmp (mailbox_name, dm_box) == 0) {
		(void) sprintf (foo, "%s=%d", DM_BOX, mailbox);
		add_to_atail (env_alist, strcpy(make_node(strlen(foo)+1), foo));
		if (verbose)
		    fprintf (stdout, "%s\n", foo);
		did_dm_box++;
		}
	    }
	}
    if (did_dm_box == 0) {
	mailbox = ipc_create();
	if (get_status ()) {
	    fprintf (stderr, "?Error %d in ipc_create\n", get_status ());
	    exit (1);
	    }

	if (fake_init)
	    fprintf (env_fd, "setenv %s %d\n", DM_BOX, mailbox);
	else {
	    (void) sprintf (foo, "%s=%d", DM_BOX, mailbox);
	    add_to_atail (env_alist, strcpy(make_node(strlen(foo)+1), foo));
	    if (verbose)
		fprintf (stdout, "%s\n", foo);
	    }
	}
    /*
     * Create semaphore id, assign to environment variable.
     */
    semaphore_id = ids_create();
    if (get_status ()) {
	fprintf (stderr, "Error %d in ipc_create.\n", get_status ());
	exit (1);
	}

    if (fake_init)
	fprintf (env_fd, "setenv SEM_ID %d\n", semaphore_id);
    else {
	(void) sprintf (foo, "SEM_ID=%d", semaphore_id);
	add_to_atail (env_alist, strcpy(make_node(strlen(foo)+1), foo));
	if (verbose)
	    fprintf (stdout, "%s\n", foo);
	}
    /*
     *  Create shared memory segments.
     */
    shared_memory_id = ism_create();
    if (get_status ()) {
	fprintf(stderr, "Error %d during ism_create.\n", get_status ());
	exit (1);
	}
    /*
     *  Create an environmental variable for each segment.
     */
    for (i = 0; (i < SHM_SEGMENTS) && shared_memory_id[i] != -1; i++) 
	if (fake_init)
	    fprintf (env_fd, "setenv SHM_ID%d %d\n", i, shared_memory_id[i]);  
	else {
	    (void) sprintf (foo, "SHM_ID%d=%d", i, shared_memory_id[i]);
	    add_to_atail (env_alist, strcpy(make_node(strlen(foo)+1), foo));
	    if (verbose)
		fprintf (stdout, "%s\n", foo);
	    }
    /*
     * Create id counter, assign to environment variable, and write to file.
     */
    counter = iid_create_counter();
    if (get_status ()) {
	fprintf (stderr, "Error %d in iid_create_counter.\n", get_status ());
	exit (1);
	}

    if (fake_init)
	fprintf(env_fd, "setenv ID_HAND %d\n", counter);
    else {
	(void) sprintf (foo, "ID_HAND=%d", counter);
	add_to_atail (env_alist, strcpy(make_node(strlen(foo)+1), foo));
	if (verbose)
	    fprintf (stdout, "%s\n", foo);
	}

    ism_cleanup();
    if (get_status ()) {
	fprintf (stderr, "Error %d in ism_cleanup.\n", get_status ());
	exit (1);
	}

    /*
     * Close opened files, and either quit (fake_init), or return the
     * new environment to the caller.
     */ 
    fclose(init_fd);
    if (fake_init) {
	fclose(env_fd);
	exit (0);
	}
    else {
	/*
	 * Add the passed-in environment variables, and then build an array
	 */
	for ( ; *envp; envp++)
	    add_to_atail (env_alist, *envp);
	new_env = (char **) make_node(alist_length(env_alist)+1 * sizeof (char *));
	envp = new_env;
	cur = NULL;
	while ((cur = get_alist_next (env_alist, cur)) != NULL)
	    *envp++ = cur;
	*envp = 0;
	return (new_env);
	}
}

/*--------------------------------------------------------------------------*\
|  Name:  Serpent Cleanup
| 
|  Description:  
|     Peforms necessary cleanup for the Serpent system.
\*--------------------------------------------------------------------------*/
void serp_clean (dialogue, fake_clean)
char *dialogue;
short fake_clean;
{
    FILE *init_fd;
    char *mailbox;
    char *id_hand;
    char *dm_box;
    int i;
    int did_dm_box = 0;
    string init_file_name;
    char mailbox_name[33];

    /*
     * Create fake +DM_BOX name if needed (see serp_init for details)
     */
    dm_box = make_node (strlen (dialogue) + 4 /* _BOX */ + 1 /* \0 */ );
    (void) sprintf (dm_box, "%s_BOX", dialogue);
    for (i = 0; dm_box[i]; i++)
	if (islower (dm_box[i]))
	    dm_box[i] = toupper (dm_box[i]);
    /*
     * Terminate the interface in order to determine system resource ids.
     */
    if (fake_clean) {
	id_hand = getenv ("ID_HAND");
	if (id_hand == NULL || strlen (id_hand) == 0) {
	    fputs ("Error: environment variable ID_HAND not found\n", stderr);
	    exit (1);
	    }
	isd_init_once ();
	}
    else {
	id_hand = search_alist (env_alist, "ID_HAND", match_env);
	if (id_hand)
	    id_hand = index (id_hand, '=') + 1;
	if (id_hand == NULL || strlen (id_hand) == 0) {
	    fputs ("Internal consistancy error: ID_HAND not found\n", stderr);
	    exit (1);
	    }
	}

    /*
     * Open file .dialoguei.d to read.
     */
    init_file_name = make_node (strlen (dialogue) + 1 + 3 + 1);
    (void) strcat (strcat (strcpy (init_file_name, "."), dialogue), "i.d");
    init_fd = serp_open (init_file_name, "r");

    /*
     * Read mailbox names from file.  Get mailbox ids
     * from environment variables and destroy each mailbox.
     */
    while (fscanf (init_fd, "%s %*s\n", mailbox_name) != EOF) {
	if (fake_clean)
	    mailbox = getenv(mailbox_name);
	else {
	    mailbox = search_alist (env_alist, mailbox_name, match_env);
	    if (mailbox)
		mailbox = index (mailbox, '=') + 1;
	    }
	if (mailbox == NULL || strlen (mailbox) == 0)
	    fprintf (stderr, "Warning: No environment variable %s\n", mailbox);
	else {
	    ipc_destroy((ipc_mailbox) atoi(mailbox));
	    if (get_status ()) {
	       fputs ("serp_clean: bad status from ipc_destroy.\n", stderr);
	       exit (1);
	       }
	    if (strcmp (mailbox_name, dm_box) == 0)
		did_dm_box++;
	    }
	}
    if (did_dm_box == 0) {
	if (fake_clean)
	    mailbox = getenv(DM_BOX);
	else {
	    mailbox = search_alist (env_alist, DM_BOX, match_env);
	    if (mailbox)
		mailbox = index (mailbox, '=') + 1;
	    }
	if (mailbox == NULL || strlen (mailbox) == 0)
	    fprintf (stderr, "Warning: No environment variable %s\n", mailbox);
	else {
	    ipc_destroy((ipc_mailbox) atoi(mailbox));
	    if (get_status ()) {
	       fputs ("serp_clean: bad status from ipc_destroy.\n", stderr);
	       exit (1);
	       }
	    }
	}
#if 0
    /*
     * Destroy ID counter in shared memory.
     */
    iid_destroy_counter();
    if (get_status ()) {
	fputs ("serp_clean: bad status from iid_destroy_counter.\n", stderr);
	exit (1);
	}
#endif
    /*
     * Destroy shared memory ID.
     */
    ism_destroy();
    if (get_status ()) {
	fputs ("serp_clean: bad status from ism_destroy.\n", stderr);
	exit (1);
	}
    /*
     * Destroy semaphore ID.
     */
    ids_destroy();
    if (get_status ()) {
	fputs ("serp_clean: bad status from ids_destroy.\n", stderr);
	exit (1);
	}
    /*
     * Close opened file.
     */ 
    fclose(init_fd);

    exit (0);
}
