	static char rcsid [] = "$Header: /u/04c/mnt/integ/source/tools/src/RCS/serp_run.c,v 1.2 91/01/07 00:22:29 ejh Exp $";
/*--------------------------------------------------------------------------*\
|
|  Name:  serp_run.c
|
|  Description:  
|     Executes dialogue
|
|  Component of:
|     Serpent run command
|
|  Related files:
|     serp.c
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Author:  
|     Daniel V. Klein
|     dvk@sei.cmu.edu  
|
|  Change History:
$Log:	serp_run.c,v $
 * Revision 1.2  91/01/07  00:22:29  ejh
 * fixed includes to work under sysv, esp. hpux
 * 
 * Revision 1.1  91/01/02  18:00:21  dvk
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#include <stdio.h>		/* C Standard I/O Package		    */
#include <sys/wait.h>
#include <signal.h>
#include <errno.h>
#include <alistPack.h>
#include <X11/Xos.h>

#include <global.h>		/* Global type defs and constants	    */

#include <u22.h>
#include <ufa.h>
#include <isc.h>

extern FILE *serp_open ();
extern void serp_exe ();
extern char *getenv ();
extern short verbose;

#define IN_BUF_SIZE 256

static ALIST kids_alist;
static char *dialogue_name;


/* ----------------------------------------------------------    get_args */
static char **get_args (command, parameters)
string command, parameters;
{  ALIST build_alist;
   int next_string, i, num_args, parm_string_length;
   char **ret_val;
   char *modified_cmd;

   build_alist = make_alist ();
   parm_string_length = strlen (parameters);
   
   /*
    * If this is the dialogue name, add an extra parameter '-'.  This way,
    * the dialogue will know it has been called from Serpent, and not by the
    * user typing in the dialogue name to the shell.  In the latter case, the
    * dialogue will call serpent to do the proper set-up and shut-down
    * (which is done here).  Make sure we strip off any leading pathname.
    */
    add_to_atail (build_alist, command);
    if ((modified_cmd = rindex (command, '/')) != NULL)
	modified_cmd++;
    else
	modified_cmd = command;
    if (strcmp (modified_cmd, dialogue_name) != 0)
	num_args = 1;
    else {
	add_to_atail (build_alist, "-");
	num_args = 2;
	}
    i = 0;
    next_string = 0;

    while (i < parm_string_length) {
	if (parameters [i] == '"') {
	    while (++i < parm_string_length && parameters [i] != '"')
		/* Empty loop */ ;
	    if (i == parm_string_length) {
		fprintf (stderr, "runit: unmatched \".");
		exit(1);
		}
	    next_string++;
	    parameters[i] = ' ';
	    }
	while (i < parm_string_length && parameters [i] != ' ')
	    i++;
	parameters [i] = '\0';
	num_args++;
	add_to_atail (build_alist, parameters + next_string);
	while (++i < parm_string_length && parameters [i] == ' ');
	next_string = i;
	}

    ret_val = (char **) make_node ((num_args + 1) * sizeof (char *));

    for (i = 0; i < num_args; i++)
	ret_val [i] = get_alist_pos (build_alist, i + 1);
    ret_val [i] = 0;
    return (ret_val);
}


/* ----------------------------------------------------------    kill_kids */
static void kill_kids (sig)
int sig;
{
    int kid, pid;
    char *name;
    union wait status;

    fputc ('\n', stdout);
    loop_through_alist (kids_alist, kid, int)
	kill (kid, sig);
    while ((pid = wait (&status)) != -1) {
	name = (char *) get_anode_val (kids_alist, pid);
	if (name)
	    fprintf (stdout, "Terminated Serpent process named %s\n", name);
	}
}

/* ----------------------------------------------------------    fire_up */
static void fire_up (parm_alist, envp)
ALIST parm_alist;
char **envp;
{
    int pid;
    char **argv, **ap;
    NODE cur;

    /*
     * Convert the alist to a vector of char *'s
     */
    ap = argv = (char **) make_node (
			sizeof (char *) * (alist_length (parm_alist)+1));
    cur = NULL;
    while ((cur = get_alist_next (parm_alist, cur)) != NULL)
	*ap++ = cur;
    *ap = (char *)0;
    /*
     * Run that sucker
     */
    if (verbose) {
	fputs ("Firing up:", stderr);
	for (ap = argv; *ap; ap++) {
	    fputc (' ', stderr);
	    fputs (*ap, stderr);
	    }
	fputc ('\n', stderr);
	}
    switch (pid = fork ()) {
        case 0:		 /* child */
	    serp_exec (argv[0], argv, envp);
	    kill (getppid(), SIGTERM);		/* Make poppa shut down */
	    exit (1);
    	default:		 /* parent */
	    add_to_atail_val (kids_alist, pid, argv[0]);
	    break;
	case -1:
	    fprintf (stderr, "Ran out of processes firing up %s\n", argv[0]);
	    kill_kids (SIGTERM);
	    exit (1);
	}
}

/* ----------------------------------------------------------    quit_things */
static void quit_things ()
{ 
    (void) kill_kids (SIGINT);
    serp_clean (dialogue_name, 0);
    exit (0);
}

/* -------------------------------------------------    abnormal_terminate */
static void abnormal_terminate ()
{
    (void) kill_kids (SIGTERM);
    serp_clean (dialogue_name, 0);
    fprintf (stderr, "\nAbnormal termination of Serpent\n");
    exit (1);
}

/*--------------------------------------------------------------------------*\
|  Name:  serpent_run
| 
|  Description:  
|     Actually executes the dialogue
\*--------------------------------------------------------------------------*/
run_dialogue (dialogue, envp)
char *dialogue;
char **envp;
#if 0
run_dialogue (dialogue, tap_args, playback_args, ep)
char *dialogue;
char **tap_args, **playback_args, **ep;
#endif
{
    FILE *exec_f;
    int pid;
    union wait status;
    char in_buf [IN_BUF_SIZE];
    char *tmp_buf;
    ALIST exec_alist;
    ALIST arg_alist;
    char *name, *arg, *env, *env_var, *exec_file;

    dialogue_name = dialogue;

    /*
     * Open file containing listing taps to execute.
     */
    exec_file = make_node (strlen (dialogue) + 1 + 3 + 1);
    strcat (strcat (strcpy (exec_file, "."), dialogue), "e.d");
    exec_f = serp_open (exec_file, "r");

    kids_alist = make_alist ();

    /*
     * Set up signals.
     */
    signal(SIGINT, quit_things);
    signal(SIGTERM, abnormal_terminate);
    signal(SIGHUP, abnormal_terminate);
    signal(SIGQUIT, abnormal_terminate);
    /*
     * Read taps from "e.d" file.
     */
    while (fgets (in_buf, IN_BUF_SIZE, exec_f) != NULL) {
	/*
	 * Get env_var and command from the input string and then copy 
	 * the rest of the line as parameters.
	 */
	arg_alist = make_alist ();
	tmp_buf = strcpy (make_node (strlen (in_buf)+1), in_buf);
	env_var = strtok (tmp_buf, " ");
	/*
	 * Test to see if command and parameters are to overwritten
	 * by an environment variable.  If so, grab the first component
	 * as the command name, and then proceed to cut out the arguments.
	 */
	if (env = getenv (env_var)) {
	    free_node (tmp_buf);
	    tmp_buf = strcpy (make_node (strlen (env)+1), env);
	    arg = strtok (tmp_buf, " ");
	    }
	else
	    arg = strtok (NULL, " \n");
	do {
	    add_to_atail (arg_alist, arg);
	    } while (arg = strtok (NULL, " \n"));
	/*
	 * Fire up this command
	 */
	fire_up (arg_alist, envp);
	}

    /*
     * Wait for all of the commands to terminate (normally or otherwise)
     */
    while ((pid = wait (&status)) != -1) {
	name = (string) get_anode_val (kids_alist, pid);
	if (name) {
	    fprintf (stdout, "\nSerpent process \"%s\" exited ", name);
	    if (WIFSIGNALED (status))
		fprintf (stdout, "with signal %d%s\n", status.w_termsig,
		    status.w_coredump ? " and dumped core" : "");
	    else if (status.w_retcode)
		fprintf (stdout, "with status %d\n", status.w_retcode);
	    else
		fputs ("normally\n", stdout);
	    }
	}
}
