        static char rcsid [] = "$Header: /usr/users/serpent/integ/src/tools/src/RCS/serpent.c,v 1.13 1991/08/30 14:41:48 serpent Exp $";

/*--------------------------------------------------------------------------*\
|
|  Name:	serpent.c
|
|  Description:
|	This is the main driver module for Serpent
|
|  Component of:
|
|  Related files:
|
|  Project:
|	SERPENT System
|	User Interface Prototyping
|	Software Engineering Institute
|	Carnegie Mellon University
|	Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Author:  
|	Daniel V. Klein
|	dvk@sei.cmu.edu
|
|  Change History:
$Log: serpent.c,v $
 * Revision 1.13  1991/08/30  14:41:48  serpent
 * new version number
 *
 * Revision 1.12  1991/05/16  14:33:56  ejh
 * fflush stdout after welcome message
 *
 * Revision 1.11  1991/05/14  15:09:45  little
 * change version name
 *
 * Revision 1.10  1991/04/18  04:06:52  ejh
 * got it to work this time
 *
 * Revision 1.9  1991/04/18  03:38:07  ejh
 * added CFLAGS to link phase
 *
 * Revision 1.8  1991/04/02  17:14:19  ejh
 * now gets CFLAGS passed in from the Makefile
 *
 * Revision 1.7  91/02/13  18:39:35  ejh
 * took out stupid Done message
 * 
 * Revision 1.6  91/01/10  14:20:12  dvk
 * Fixed certain cases where cpp temporary file was not deleted
 * 
 * Revision 1.5  91/01/08  15:19:33  little
 * use scpp instead of cpp
 * 
 * Revision 1.4  91/01/07  00:07:22  ejh
 * fixed includes to handle sysv (esp. hpux)
 * 
 * Revision 1.3  91/01/04  17:22:46  dvk
 * Added hack to allow Makefile to pass in list of libraries instead of
 * having them hardwired into the serpent command.  Also reinserted -O.
 * 
 * Revision 1.2  91/01/03  01:44:02  dvk
 * Forgot to end with an exit(0) at the end of main()
 * 
 * Revision 1.1  91/01/02  17:58:58  dvk
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/
#include "memoryPack.h"
#include "alistPack.h"
#include <stdio.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <X11/Xos.h>

#define SERPENT_VERSION	"1.0 beta.2"
#define GEN_SIZE	50	/* Maximum outgoing arguments possible */
#define TAP_SIZE	10	/* Number of -a arguments possible */

extern char *getenv ();
extern char *strtok ();
extern char *mktemp ();
extern char **serp_init ();
extern void serp_clean ();

short verbose = 0;
short do_optimize = 0;
char *argv0;
char *out_args[GEN_SIZE];

static char *cpp_temp_file;

static void delete_cpp_temp_file ()
{
    (void) unlink (cpp_temp_file);
}

static void usage (prog, quick)
char *prog;
int quick;
{
    fprintf (stderr, "Usage: %s {<cpp args>} [-cdlgmOikv] [-L user_defined_lib] [-a command arg_string] [-r filename message] <dialogue_name>\n", prog);
    if (! quick) {
	fputs ("where:\t-c => compile a dialogue,\n", stderr);
	fputs ("\t-l => link a dialogue and\n", stderr);
	fputs ("\t-g => execute dialogue [default if no -c, -d, or -l]\n", stderr);
	fputs ("\t-m => start serpent monitor with the dialogue\n", stderr);
	fputs ("\t-L => link a user defined library\n", stderr);
	fputs ("\t-O => when compiling/linking, optimize\n", stderr);
	fputs ("\t-a => execute command with parameters arg_string\n", stderr);
	fputs ("\t-r => turn on recording with output to filename\n", stderr);
	fputs ("\t-i => equivalent to old serp_init\n", stderr);
	fputs ("\t-k => equivalent to old serp_cleanup\n", stderr);
	fputs ("\t-v => verbose processing\n", stderr);
	fputs ("\n", stderr);
    }
    fputs ("\t-m and -r CANNOT be specified together.\n", stderr);
    fputs ("\t-i and -k CANNOT be specified with any other option.\n", stderr);
    fputs ("\t-r can only be specified once.\n", stderr);
    exit(1);
}

FILE *serp_open (file, how)
char *file, *how;
{
    static ALIST open_alist = NULL;
    NODE cur;
    FILE *fd;
    char *env, *str;
    char fname[BUFSIZ];

    /*
     * Initialize the open path, if necessary
     */
    if (open_alist == NULL) {
	open_alist = make_alist();
	if ((env = getenv ("SERPENT_DATA_PATH")) != NULL) {
	    str = strcpy (make_node (strlen (env)+1), env);
	    (void) strtok (str, " ");
	    (void) add_to_atail (open_alist, str);
	    while (str = strtok (NULL, " "))
		(void) add_to_atail (open_alist, str);
	    }
	else {
	    fputs ("Warning: SERPENT_DATA_PATH not found!\n", stderr);
	    add_to_atail (open_alist, ".");
	    }
	}
    /*
     * Traverse the open path
     */
    cur = NULL;
    while ((cur = get_alist_next (open_alist, cur)) != NULL) {
	(void) strcat (strcat (strcpy (fname, cur), "/"), file);
	if (strlen (fname) + strlen (file) + 1 >= BUFSIZ)
	    fprintf (stderr, "Warning: Filename %s/%s too big!\n", fname, file);
	if ((fd = fopen (fname, how)) != NULL)
	    break;
	}
    /*
     * If we get here, check to see if we could open the file
     */
    if (fd == NULL) {
	fprintf (stderr, "?Internal error - open of %s failed\n", file);
	exit (1);
	}
    return fd;
}

int cpp_and_compile (dialogue, envp, cpp_only, cpp_alist)
char *dialogue;
char **envp;
short cpp_only;
ALIST cpp_alist;
{
    char *env, *str, *file;
    NODE cur;
    char **ap;
    struct stat statbuf;
    union wait child_status;

    file = strcat(strcpy (make_node (strlen (dialogue)+1+3), dialogue), ".sl");
    if (stat (file, &statbuf) == -1) {
	fprintf (stderr, "Cannot find dialogue file \"%s\"\n", file);
	usage (argv0, 1);
	}
    /*
     * Build cpp arguments.  Arg[0] is the file name.  Then copy the
     * list of arguments from the original argument list.  Then, tack
     * on the -I's from SERPENT_DATA_PATH.  Finally, add the name of
     * the file to preprocess (optionally followed by the output file).
     */
    ap = out_args;
    *ap++ = "scpp";
    cur = NULL;
    while ((cur = get_alist_next (cpp_alist, cur)) != NULL) {
	*ap++ = cur;
	if (ap >= &out_args[GEN_SIZE])
	    goto toomuch;
	}
    if ((env = getenv ("SERPENT_DATA_PATH")) != NULL) {
	str = strcpy (make_node (strlen (env) + 1), env);
	(void) strtok (str, " ");
	*ap = make_node (strlen (str) + 3);	/* 2 for "-I", 1 for '\0' */
	(void) strcpy (*ap, "-I");
	(void) strcat (*ap++, str);
	if (ap >= &out_args[GEN_SIZE])
	    goto toomuch;
	while (str = strtok (NULL, " ")) {
	    *ap = make_node (strlen (str) + 3);
	    (void) strcpy (*ap, "-I");
	    (void) strcat (*ap++, str);
	    if (ap >= &out_args[GEN_SIZE]) {
toomuch:
		fputs ("Too many includes in SERPENT_DATA_PATH\n", stderr);
		exit(1);
		}
	    }
	}

    *ap++ = file;
    if ( ! cpp_only) {
	*ap++ = cpp_temp_file = mktemp("/tmp/serpXXXXXX");
	/*
	 * Set up signals.
	 */
	signal(SIGINT, delete_cpp_temp_file);
	signal(SIGTERM, delete_cpp_temp_file);
	signal(SIGHUP, delete_cpp_temp_file);
	signal(SIGQUIT, delete_cpp_temp_file);
	}
    *ap++ = NULL;

    if (verbose) {
	fprintf (stdout, "scpp");
	for (ap = out_args+1; *ap; ap++)	/* Skip argv[0] */
	    fprintf (stdout, " %s", *ap);
	putc ('\n', stdout);
	}

    /*
     * Execute cpp
     */
    switch (fork()) {
	case -1:	/* Error */
	    fputs ("?fork() for scpp failed\n", stderr);
	    exit(1);
	    break;
	case 0:		/* Child process */
	    serp_exec ("scpp", out_args, envp);
	    exit(1);
	    break;
	default:	/* Parent process */
	    (void) wait(&child_status);
	    if (WIFSIGNALED (child_status))
		fprintf (stderr, "?scpp terminated with a signal%s\n",
		    child_status.w_coredump ? " and dumped core" : "");
	    if (child_status.w_retcode || WIFSIGNALED (child_status)) {
		fputs ("Error during scpp - aborting\n", stderr);
		(void) unlink (cpp_temp_file);
		exit (1);
		}
	    break;
	}
    if (cpp_only)
	exit(0);

    /*
     * Build slang_comp arguments
     */
    ap = out_args;
    *ap++ = "slang_comp";
    *ap++ = cpp_temp_file;
    *ap++ = dialogue;
    *ap++ = NULL;

    if (verbose) {
	for (ap = out_args; *ap; ap++)
	    fprintf (stdout, "%s ", *ap);
	putc ('\n', stdout);
	}

    /*
     * Execute slang_comp
     */
    switch (fork()) {
	case -1:	/* Error */
	    fprintf (stderr, "?fork() for slang_comp failed\n");
	    (void) unlink (cpp_temp_file);
	    exit(1);
	    break;
	case 0:		/* Child process */
	    serp_exec ("slang_comp", out_args, envp);
	    exit(1);
	default:	/* Parent process */
	    (void) wait(&child_status);
	    (void) unlink (cpp_temp_file);
	    if (WIFSIGNALED (child_status))
		fprintf (stderr, "?slang_comp terminated with a signal%s\n",
		    child_status.w_coredump ? " and dumped core" : "");
	    if (child_status.w_retcode || WIFSIGNALED (child_status))
		return (-1);
	    break;
	}
    
    return (0);
}

void build_alists (dialogue, delete_alist)
char *dialogue;
ALIST delete_alist;
{
#   define EXTRA 5	/* 1 leading '.' + 3 trailing '?.?' + 1 '\0' */
    static char *delete_suf[] = { ".h", ".x", ".z", ".t", ".c",
				  "e.d", "i.d", "t.d", 0 };
    int len = strlen (dialogue) + EXTRA;
    char *str;
    char **cp;

    for (cp = delete_suf; *cp; cp++) {
	str = strcat (strcat (strcpy (make_node (len), "."), dialogue), *cp);
	(void) add_to_atail (delete_alist, str);
	}
}

void compile_and_link (dialogue, lib_alist, envp)
char *dialogue;
ALIST lib_alist;
char **envp;
{
#   undef EXTRA
#   define EXTRA 5	/* 1 leading '.' + 3 trailing '?.?' + 1 '\0' */
#   define LIB "/lib/"
    int len;
    int have_c_file, have_obj_file, fake_c_file;
    struct stat target_statbuf, c_statbuf, obj_statbuf;
    char *c_file, *obj_file;
    char *cp, *base, *str, *env;
    char target[256];
    char **lp, **ap;
    NODE cur;
    /*
     * The following three strings are passed in via the Makefile.  They
     * all are space separated lists of libraries to link in with the
     * generated dialogues.
     */
    static char *libs = SERP_SLANG_LIBS;
    static char *other_libs[] = { OTHERLIBS, EXTRA_LIBRARIES, 0};
    static char *cflags = CFLAGS;

    /*
     * Build the list of libraries to tack on the end of the link command
     */
    if ((base = getenv ("SERPENT_BASE_PATH")) == (char *)0) {
	fputs ("Cannot find environment variable SERPENT_BASE_PATH\n", stderr);
	exit (1);
	}
    /*
     * Start with SERP_SLANG_LIBS, which get SERPENT_BASE_PATH/lib/ prepended
     */
    len = strlen (base) + strlen(LIB);
    cp = strtok (libs, " ");
    do {
	str = (char *) make_node (len + strlen (cp) + 1);
	(void) strcat (strcat (strcpy (str, base), LIB), cp);
	(void) add_to_atail (lib_alist, str);
	} while ((cp = strtok (NULL, " ")) != NULL);
    /*
     * Then do the other libraries, which don't get anything tacked on.
     */
    for (lp = other_libs; *lp; lp++) {
	cp = strtok (*lp, " ");
	do {
	    (void) add_to_atail (lib_alist, cp);
	    } while ((cp = strtok (NULL, " ")) != NULL);
	}
    /*
     * Compile the generated C file as needed
     */
    len = strlen (dialogue) + EXTRA;
    c_file = strcat (strcat (strcpy (make_node (len), "."), dialogue), ".c");
    obj_file = strcat (strcat (strcpy (make_node (len), "."), dialogue), ".o");
    have_c_file = (stat (c_file, &c_statbuf) != -1);
    have_obj_file = (stat (obj_file, &obj_statbuf) != -1);
    if (have_c_file &&
	(!have_obj_file ||
	    (c_statbuf.st_mtime > obj_statbuf.st_mtime))) {
	if (unlink (obj_file) != -1 && verbose)
	    fprintf (stderr, "rm %s\n", obj_file);
	ap = out_args;
	*ap++ = "cc";

	/* get the cflags */
	cp = strtok (cflags, " ");
	do
	    {
	    *ap++ = strcpy (make_node (strlen (cp) + 1), cp);
	    } while ((cp = strtok (NULL, " ")) != NULL);
	*ap++ = "-c";
	*ap++ = "-I.";
	*ap++ = c_file;
	*ap = (char *)0;
	for (ap = out_args; *ap; ap++)
	    fprintf (stdout, "%s ", *ap);
	putc ('\n', stdout);
	fork_and_path_exec ("cc", out_args, envp);
	}
    /*
     * Delete the old copy of the dialogue
     */
    if (unlink (dialogue) != -1 && verbose)
	fprintf (stderr, "rm %s\n", dialogue);
    /*
     * Set up to do the link of the dialogue.  If we have a C file, really
     * link, otherwise just make a symlink to the DM.
     */
    fake_c_file = 0;
    if (!have_c_file) {
	if ((env = getenv ("SERPENT_EXE_PATH")) != NULL) {
	    str = strcpy (make_node (strlen (env)+1), env);
	    (void) strtok (str, " ");
	    do {
		(void) strcat (strcpy (target, str), "/DM");
		if (stat (target, &target_statbuf) != -1) {
		    if (symlink (target, dialogue) == -1) {
			perror ("Couldn't symlink to generic dialogue manager");
			fputs ("Trying again the \"hard\" way\n", stderr);
			fake_c_file++;
			}
		    else {
			if (verbose)
			    fprintf (stdout, "ln -s %s %s\n", target, dialogue);
			}
		    break;
		    }
	        } while (str = strtok (NULL, " "));
	    if (str == NULL) {
		fputs ("Couldn't find generic dialogue manager in SERPENT_EXE_PATH\n", stderr);
		fputs ("Trying again the \"hard\" way\n", stderr);
		fake_c_file++;
		}
	    }
	else {
	    fputs ("Warning: SERPENT_EXE_PATH not found!\n", stderr);
	    (void) strcpy (target, "./DM");
	    }
	}
    if (have_c_file || fake_c_file) {
	ap = out_args;
	*ap++ = "cc";

	/* get the cflags */
	cflags = CFLAGS;
	cp = strtok (cflags, " ");
	do
	    {
	    *ap++ = strcpy (make_node (strlen (cp) + 1), cp);
	    } while ((cp = strtok (NULL, " ")) != NULL);
	*ap++ = "-o";
	*ap++ = dialogue;
	/*
	 * Add the object files (if present)
	 */
	if (have_c_file)
	    *ap++ = obj_file;
	/*
	 * Add the list of libraries
	 */
	cur = NULL;
	while ((cur = get_alist_next (lib_alist, cur)) != NULL)
	    *ap++ = (char *) cur;
	*ap++ = "-lm";
	*ap = (char *)0;
	for (ap = out_args; *ap; ap++)
	    fprintf (stdout, "%s ", *ap);
	putc ('\n', stdout);
	fork_and_path_exec ("cc", out_args, envp);
	}
}


main(argc, argv, envp)
int argc;
char **argv;
char **envp;
{
    char **ap, **ep;
    ALIST cpp_alist, lib_alist, compile_alist;
    ALIST delete_alist;
    NODE cur;
    union wait child_status;
    char *tap_args[TAP_SIZE][2];
    short tap_idx;
    char *env, *str, *file, *temp;
    char *dialogue = NULL;
    int ac, i;
    short do_compile = 0;
    enum {NO_TRACE, MONITOR, RECORD} do_trace = NO_TRACE;
    char *record_args[2];
    enum {NO_GO, YES_GO, MAYBE_GO, FAKE_GO} do_go = MAYBE_GO;
    short do_serp_init = 0;
    short do_serp_clean = 0;
    short do_link = 0;
    short do_playback = 0;
    short cpp_only = 0;
    char *playback_args[3];

    argv0 = *argv;
    ap = argv;
    ac = argc;
    cpp_alist = make_alist ();
    lib_alist = make_alist ();
    for (tap_idx = 0; tap_idx < TAP_SIZE; tap_idx++) {
	tap_args[tap_idx][0] = NULL;
	tap_args[tap_idx][1] = NULL;
	}
    tap_idx = 0;

    while (--ac) {
	++ap;
	if (ap[0][0] == '-') {
	    for (i = 1; ap[0][i]; i++) {
		switch (ap[0][i]) {
		    case 'C':
		    case 'D':
		    case 'I':
		    case 'U':
			add_to_atail (cpp_alist, *ap);
			goto next_arg;
		    case 'E':
			cpp_only = 1;
			break;
		    case 'v':
			verbose = 1;
			break;
		    case 'i':
			if (do_serp_clean) {
			    fputs ("Cannot specify both -i and -k\n", stderr);
			    exit (1);
			    }
			do_serp_init = 1;
			break;
		    case 'k':
			if (do_serp_init) {
			    fputs ("Cannot specify both -i and -k\n", stderr);
			    exit (1);
			    }
			do_serp_clean = 1;
			break;
		    case 'O':
			do_optimize = 1;
			break;
		    case 'c':
			do_compile = 1;
			if (do_go == MAYBE_GO)
			    do_go = NO_GO;
			break;
		    case 'l':
			do_link = 1;
			if (do_go == MAYBE_GO)
			    do_go = NO_GO;
			break;
		    case 'g':
			do_go = YES_GO;
			break;
		    case 'a':
			tap_args[tap_idx][0] = *++ap;
			tap_args[tap_idx][1] = *++ap;
			tap_idx++;
			ac -= 2;
			goto next_arg;
		    case 'p':
			if (do_playback) {
			    fputs ("Can only specify one -p flag\n", stderr);
			    exit(1);
			    }
			else if (ac <= 3) {
			    fputs ("The -p flag requires 3 parameters\n", stderr);
			    exit(1);
			    }
			do_playback = 1;
			playback_args[0] = *++ap;
			playback_args[1] = *++ap;
			playback_args[2] = *++ap;
			ac -= 3;
			goto next_arg;
		    case 'r':
			if (do_trace == MONITOR) {
			    fputs ("Cannot specify both -m and -r\n", stderr);
			    exit(1);
			    }
			else if (do_trace == RECORD) {
			    fputs ("Can only specify a single -r\n", stderr);
			    exit(1);
			    }
			else if (ac <= 2) {
			    fputs ("The -r flag needs 2 parameters\n", stderr);
			    exit(1);
			    }
			do_trace = RECORD;
			record_args[0] = *++ap;
			record_args[1] = *++ap;
			ac -= 2;
			goto next_arg;
		    case 'm':
			if (do_trace == RECORD) {
			    fputs ("Cannot specify both -m and -r\n", stderr);
			    exit(1);
			    }
			do_trace = MONITOR;
			break;
		    case 'L':
			if (ap[0][i+1])
			    (void) add_to_atail (lib_alist, &ap[0][2]);
			else {
			    (void) add_to_atail (lib_alist, *++ap);
			    ac--;
			    }
			goto next_arg;
		    case '-':
			dialogue = *++ap;
			if (--ac > 1) {
			    fputs ("Extra args after dialogue name\n", stderr);
			    usage (argv0, 1);
			    }
			goto done_with_flags;
		    default:
			fprintf (stderr, "Unknown flag %s\n", *ap);
			usage (argv0, 0);
			break;
		    }
		}
	    next_arg: ;
	    }
	else {
	    dialogue = *ap;
	    if (ac > 1) {
		fputs ("Extra args after dialogue name\n", stderr);
		usage (argv0, 0);
		}
	    }
	}

done_with_flags:

    if (do_go == MAYBE_GO)
	do_go = YES_GO;
    if (do_serp_init || do_serp_clean) {
	do_compile = 0;
	do_link = 0;
	do_go = FAKE_GO;
	}
    if (dialogue == NULL) {
	fprintf (stderr, "Missing Serpent dialogue name\n");
	usage (argv0, 1);
	}
    /*
     * Trim the dialogue name down, removing leading path and trailing extent
     */
    if (temp = rindex (dialogue, '/'))
	dialogue = temp+1;
    if (temp = rindex (dialogue, '.'))
	*temp = '\0';

    fprintf(stdout, "Serpent Version %s\n", SERPENT_VERSION);
    fflush (stdout);

    /*
     * Generate file lists
     */
    delete_alist = make_alist ();
    build_alists (dialogue, delete_alist);

    /*
     * Compile
     *
     * Delete the target files, and run the compiler.
     */
    if (do_compile) {
	cur = NULL;
	while ((cur = get_alist_next (delete_alist, cur)) != NULL)
	    if (unlink (cur) != -1 && verbose)
		fprintf (stderr, "rm %s\n", cur);
	if (cpp_and_compile (dialogue, envp, cpp_only, cpp_alist)) {
	    fputs ("Error during slang_comp - aborting\n", stderr);
	    exit (1);
	    }
	}

    /*
     * Link the dialogue.  This means if there was a generated C file,
     * compile and link a new DM.  Otherwise, generate a dummy executable.
     */
    if (do_link)
	compile_and_link (dialogue, lib_alist, envp);
    
    /*
     * Run the bloody dialogue
     */
    if (do_go == YES_GO || do_serp_init) {
	ep = serp_init (dialogue, envp, do_serp_init);
	if (do_serp_init)
	    exit (0);
	}
    if (do_go == YES_GO)
	run_dialogue (dialogue, ep);
    if (do_go == YES_GO || do_serp_clean) {
	serp_clean (dialogue, do_serp_clean);
	if (do_serp_clean)
	    exit (0);
	}

    /*
     * All done
     */
    exit (0);
}
