/* $Header: /u/04c/mnt/integ/source/utilities/include/RCS/u22.h,v 1.10 91/03/25 21:58:44 ejh Exp $ */
/*--------------------------------------------------------------------------*\
|
|  Name:  u22.h
|
|  Description:  
|     Global declarations of general interest to Serpent source modules.
|     Specifically, this file contains useful macros and definitions that
|     are useful across the Serpent system.
|
|  Component of:
|     Utilities Layer.
|
|  Related files:
|     None.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Brian M. Clapper
|     bmc@sei.cmu.edu  
|
|  Change History:
|     30 Aug 1989 : Module created.
$Log:	u22.h,v $
 * Revision 1.10  91/03/25  21:58:44  ejh
 * took out comment at end of #endif at end of file
 * 
 * Revision 1.9  90/01/19  16:29:42  dvk
 * Added u22_convert_string_inplace
 * 
 * Revision 1.8  89/11/17  13:44:44  bmc
 * Added u22_concat_string routine
 * 
 * Revision 1.7  89/11/16  18:06:47  bmc
 * Added u22_read_line routine
 * 
 * Revision 1.6  89/11/16  14:46:43  bmc
 * Added definition for u22_get_token routine.
 * 
 * Revision 1.5  89/11/10  15:16:59  bmc
 * Added constant definitions for SERPENT_DATA_PATH and SERPENT_EXE_PATH
 * 
 * Revision 1.4  89/10/18  10:13:46  ejh
 * created u22_convert_string (came from a22); defined constants UPPERCASE
 * and LOWERCASE associated with it
 * 
 * Revision 1.3  89/10/02  14:20:53  serpent
 * added include of global.h so we get the definition of string
 * 
 * Revision 1.2  89/09/27  23:02:01  bmc
 * Added u22_copy_string routine.
 * 
 * Revision 1.1  89/08/30  14:51:48  bmc
 * Initial revision
 * 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef _u22_h_
#define _u22_h_

#include <ctype.h>
#include "global.h"

/*--------------------------------------------------------------------------*\
|  Macros: Character Conversion
|
|  Description:
|     The standard tolower() and toupper() functions (defined in <ctype.h>)
|     are handled inconsistently across UNIX systems.  These macros provide
|     a portable front-end to the functions.
|
|     ToLower(a)	If the argument is an upper-case alphabetic, the
|                       macro returns its lower-case equivalent.  Otherwise,
|                       the argument is returned unchanged.
|     ToUpper(a)	If the argument is an lower-case alphabetic, the
|                       macro returns its lower-case equivalent.  Otherwise,
|                       the argument is returned unchanged.
|
|  Note: The inclusion of <ctype.h> preceding this comment is REQUIRED!
|
\* Definition:                                                              */

#define ToLower(a) 	( isupper ((a)) ? tolower ((a)) : (a) );
#define ToUpper(a) 	( islower ((a)) ? toupper ((a)) : (a) );

/*--------------------------------------------------------------------------*\
|  Constants:	UPPERCASE, LOWERCASE
| 
|  Description:
|	used to tell u22_convert_string which case to convert to
| 
\* Definition:                                                              */

#define	UPPERCASE	1
#define	LOWERCASE	2

/*                                                                          *\
|
|  Components:
|	UPPERCASE	used to tell convert_string to convert to upper case
|	LOWERCASE	used to tell convert_string to convert to lower case
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Constants:	Environment variable names
| 
|  Description:
|	defines the names of Serpent environment variables that are queried
|	in various parts of Serpent 
| 
\* Definition:                                                              */

#define DATA_PATH_ENV_VAR	"SERPENT_DATA_PATH"
#define	EXECUTION_PATH_ENV_VAR	"SERPENT_EXE_PATH"

/*                                                                          *\
|
|  Components:
|	DATA_PATH_ENV_VAR	specifies locations for various data files
|				required by Serpent and some of its utilities
|	EXECUTION_PATH_ENV_VAR	specifies locations for Serpent executables
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: u22_copy_string (or just copy_string)
| 
|  Description:  
|     Allocates storage for a string and copies the parameter into it.
| 
\* Syntax:                                                                  */

        string u22_copy_string
	    (
	/*  old_string : string */
	    );


/* 
| Macro Synonym
\*                                                                          */

#define	copy_string(old_string) \
	u22_copy_string(old_string)

/* 
|  Parameters:
|     old_string - The string to duplicate.
|
|  Returns:  
|     Exact copy of old_string in an allocated buffer, or NULL in case of
|     memory allocation error.
|  
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine:	u22_convert_string
| 
|  Description:
|	converts a name from upper case to lower, or vice versa, allocating
|       a new string to hold the converted string.
| 
\* Definition:                                                              */

	string	u22_convert_string (
	/*	the_name:	in	string,
		the_case:	in	int
	*/
	);

/*                                                                          *\
|
|  Parameters:
|	the_name	the name to be converted
|	the_case	what case it is to be converted to
|
|  Returns:
|	the converted name in a new string
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine:	u22_convert_string_inplace
| 
|  Description:
|	converts a name from upper case to lower, or vice versa
| 
\* Definition:                                                              */

	string	u22_convert_string_inplace (
	/*	the_name:	in	string,
		the_case:	in	int
	*/
	);

/*                                                                          *\
|
|  Parameters:
|	the_name	the name to be converted
|	the_case	what case it is to be converted to
|
|  Returns:
|	the converted name (in the same place as the original)
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine:	u22_get_token
| 
|  Description:
|	Parses a token from a string, without modifying the string.
|       (Provides an alternative to strtok(3).)
| 
\* Definition:                                                              */

	string u22_get_token
	    (
	/*  the_string    : in the_string,	*/
	/*  delimiters    : in delimiters	*/
	/*  chars_to_skip : out *chars_to_skip  */
	    );

/*                                                                          *\
|
|  Parameters:
|	the_string    - The string to be parsed.  The contents of this string
|			are not modified.
|	delimiters    - The set of characters delimiting tokens.  When locating
|			the token, u22_get_token skips any character in this
|			string.
|       chars_to_skip - The number of characters to skip in the string to
|                       get past the parsed token.  This value should be used
|                       to advance the line pointer before parsing the next
|                       token.
|
|  Returns:
|	An allocated string containing the parsed token, or NULL if the line
|	was empty or contained only token delimiters.
|
| Example of use:
|
|	char *line;
|	int chars_to_skip;
|	char *token;
|
|	[ code to read line from somewhere ]
|
|	while ( (token = u22_get_token (line, "\t ", &chars_to_skip)) != NULL )
|	    {
|
|	    [ code to process token ]
|
|	    line += chars_to_skip;
|	    }
|
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine:	u22_read_line
|
|  Description: Reads a line of text from a file, stopping when:
|      1. end of file is encountered;
|      2. a newline character is encountered;
|      3. a system error aborts the read.
| 
\* Definition:                                                              */

	string u22_read_line
	    (
	/*  f : in FILE *	*/
	    );

/*                                                                          *\
|  Parameters:
|    file - Open stream descriptor from which to read.
| 
|  Externals:
|    errno - Examines and/or sets this variable.  (See <errno.h> and intro(2))
| 
|  Returns:
|    Upon a successful read, a pointer to a buffer containing the line of
|    input is returned, minus the trailing newline.  The buffer is allocated
|    using make_node(3C) package and is guaranteed to contain the entire
|    line without any truncation.  Freeing the buffer is the calling routine's
|    responsibility.
| 
|    NULL is returned when end-of-file is encountered or if an error occurs.
|    For end-of-file, the external errno will be 0; otherwise errno will be set
|    to indicate the error.
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
| Routine:	u22_concat_string
|
| Description:  Concatenates any number of null-terminated strings, returning
|   a pointer to single string.
|
\* Definition:                                                              */

	string u22_concat_string
	    (
	/*  s1	: in string	*/
	/*     ...		*/
	/*  NULL		*/
	    );

/*                                                                          *\
| Parameters:
|   This routine accepts any number of string (i.e., character pointer)
|   arguments, terminated by a NULL pointer.  The strings can refer to
|   static or variable data, since this routine does not modify its input.
|   The final NULL pointer is required.  See the examples, below.
|
| Returns:
|   This routine returns a NULL pointer if it cannot allocate a buffer 
|   large enough to hold the final string.  Otherwise, it returns a pointer
|   to a buffer containing a concatenated version of the argument list. This
|   pointer refers to an area of memory allocated with the make_node
|   routine; it is the caller's responsibility to free the memory when it is
|   no longer needed.
|
| Examples:
|   1. The following code fragment just concatenates two static strings,
|      printing out the result if u22_concat_string works.
|
|          char *buffer;
|
|             .
|             .
|
|          buffer = u22_concat_string ("Can't open file \"",
|                                      CONFIG_FILE,
|                                      "\"",
|                                      NULL);
|          if (buffer == NULL)
|             (void) fprintf (stderr, "unable to allocate memory\n");
|          else
|             {
|             (void) printf ("%s\n", buffer);
|             free_node (buffer);
|             }
|
|   2. The following code fragment builds an argument list to the vi(1)
|      command.  Command-line argument 1 represents the line number where
|      editing is to start.  Command-line argument 2 specifies the file to
|      edit.
|
|          main (argc, argv)
|          int argc;
|          char *argv [];
|
|             .
|             .
|             .
|
|             char *buffer;
|
|             buffer = u22_concat_string ("/bin/vi +",
|                                         argv [1],
|                                         " ",
|                                         argv [2],
|                                         NULL);
|             if (buffer == NULL)
|                (void) fprintf (stderr, "unable to allocate memory\n");
|             else
|                {
|                (void) system (buffer);
|                free_node (buffer);
|                }
\*--------------------------------------------------------------------------*/

#endif
