/*  $Header: /u/04c/mnt/integ/source/utilities/include/RCS/uc2.h,v 1.4 90/07/23 10:56:17 ejh Exp $ */

/*--------------------------------------------------------------------------*\
|
|  Name:  universal conversion utilities
|
|  Description:  
|     The run time conversions utility provides low level type conversions
|     that are either not avaialalbe or do not return the appropriate
|     parameters.
|
|  Component of:
|     Utilities
|
|  Related files:
|     uc2.c - implementation of these
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Robert C. Seacord
|     rcs@sei.cmu.edu  
|
|     Erik J. Hardy
|     erik@sei.cmu.edu
|
|  Change History:
$Log:	uc2.h,v $
 * Revision 1.4  90/07/23  10:56:17  ejh
 * added uc2_undefined and uc2_get_undefined: part of the effort to get rid
 * of dm dependencies, of which there are now none
 * 
 * Revision 1.3  90/07/05  14:47:45  ejh
 * changed isinteger and isreal to int
 * 
 * Revision 1.2  90/07/05  14:44:19  ejh
 * exposed isinteger and isreal
 * 
 * Revision 1.1  90/07/05  13:10:15  ejh
 * Initial revision
 * 
|     July 1, 1990 : Module created.
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#ifndef	UC2_H
#define	UC2_H

#ifdef uc2_TOP
int	uc2_debug = 1;
#else
extern	int	uc2_debug;
#endif

#define issign(ch) ((ch == '+') || (ch == '-'))
#define isexp(ch)  ((ch == 'E') || (ch == 'e'))
#define isperiod(ch) (ch == '.')
#define UNDEF 0	

/*--------------------------------------------------------------------------*\
|  Routine: isinteger
| 
|  Description:
|     check if an ascii string is an integer
\* Syntax :								   */

	int	isinteger(
	/* str:	in	*char
	*/
	);
/*
|  Parameters:
|     str:	the string to be checked
|
|  Returns:  
|     whether or not it's true
|  
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: isreal
| 
|  Description:
|     check if an ascii string is a real
\* Syntax :								   */

	int	isreal(
	/* str:	in	*char
	*/
	);
/*
|  Parameters:
|     str:	the string to be checked
|
|  Returns:  
|     whether or not it's true
|  
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: uc2_undefined
| 
|  Description:
|     determines whether something is undefined or not
\* Syntax :								   */

	boolean uc2_undefined (
	/* type:	in	idd_data_types */
	/* data:	in	caddr_t */
	);
/*
|  Parameters:
|     type - the type to convert to
|     data - the data to check
|
|  Returns:  
|     whether or not it's undefined
|  
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: uc2_convert
| 
|  Description:
|     converts type
\* Syntax :								   */

	caddr_t uc2_convert(
	/* to_type : in idd_data_types */
	/* from_type : in idd_data_types */
	/* from_value : in caddr_t */
	);
/*
|  Parameters:
|     to_type - the type to convert to
|     from_type - the type to convert from
|     from_value - the value to convert into the new type
|
|  Returns:  
|     value as new idd_data_types
|  
|  Statuses:  
|     failed_conversion.
|  
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
|  Routine: uc2_get_undefined
| 
|  Description:
|     returns undefined in the appropriate type
\* Syntax :								   */

	caddr_t uc2_get_undefined (
	/* desired_type:	in	idd_data_types */
	);
/*
|  Parameters:
|     desired_type - the type of undefined
|
|  Returns:  
|     the appropriate type of undefined
|  
\*--------------------------------------------------------------------------*/

#endif
