static char rcsid [] =
	"$Header: /u/04c/mnt/integ/source/utilities/src/RCS/cus.c,v 1.28 91/03/25 22:23:26 ejh Exp $";


/*--------------------------------------------------------------------------*\
|
|  Name:  Compiler symbol table package (cus)
|
|  Version: 1.0
|
|
|  Description:  
|     Provides the routines necessary to manage and use the symbol table
|
|  Change History:
|
$Log:	cus.c,v $
 * Revision 1.28  91/03/25  22:23:26  ejh
 * defined a couple of stream's; case the last i in the file as a
 * cus_property_type
 * 
 * Revision 1.27  90/11/12  12:18:39  dvk
 * Added an external seed parameter to cus_gensym and cus_gensym_no_add
 * 
 * Revision 1.26  90/10/02  15:17:33  dvk
 * No changes
 * 
 * Revision 1.25  90/09/17  19:12:27  dvk
 * Changed context lookup code to a comb structure.  Now uses ALISTS instead
 * of LISTS for properties, and only creates 'em when needed.  Added routines
 * cus_create_sym and cus_gensym_no_add (which create symbols but don't add
 * 'em into the symbol table).
 * 
 * Revision 1.24  90/04/26  15:31:13  little
 * take out dependency on and inclusion of c22.h
 * 
 * Revision 1.23  90/04/20  16:19:40  ejh
 * #define'd cus_c for c22.h
 * 
 * Revision 1.22  90/04/20  15:32:21  ejh
 * added #define c22_TOP before #include c22.h
 * 
 * Revision 1.21  90/04/10  18:39:03  ejh
 * changed strings.h to string.h
 * 
 * Revision 1.20  90/02/28  18:48:19  dvk
 * Filled in cus_delete_sym (may have problems with redefined, but
 * the compiler never calls it, so...).  Also deleted now unused
 * and superfluous cus_get_sym_type_string.
 * 
 * Revision 1.19  90/02/23  16:43:00  rcs
 * nothing changed
 * 
 * Revision 1.18  90/02/14  21:48:01  ejh
 * fixed include structure
 * 
 * Revision 1.17  90/01/30  15:34:15  clc
 * Moved cus_int_is_sym to just before it's first reference.
 * 
 * Revision 1.16  90/01/30  14:30:13  ejh
 * fixed a missing \t (was \sym)
 * 
 * Revision 1.15  89/10/08  00:16:22  serpent
 * short names: cus_private.h => cus_priv.h
 * 
 * Revision 1.14  89/09/26  18:10:34  bmc
 * Change cus_create_sym_tab so that it doesn't create symbol table if it
 * already exists.
 * 
 * Revision 1.13  89/06/19  11:41:05  little
 * fix some errors in the reinstallation of redefined processing
 * 
 * Revision 1.12  89/06/14  10:59:01  little
 * add redefined processing back and make its use selectable via a parameter
 *   to the sym tba init routine
 * 
 * Revision 1.11  89/06/09  13:40:54  little
 * take out use of redefined_prop and associated mechanism
 * reorder case statements for props which are kept in the "symbol" itself
 *   to take into account frequency of use (most frequent first).
 * add prop usage count mechanism
 * add template,original_name, current_value, & current_type props to
 *   the symbol structure (for speed of access)
 * 
 * Revision 1.10  89/05/19  09:49:14  little
 * fix up some comments
 * 
 * Revision 1.9  89/05/15  11:41:14  little
 * add some comments to make len happy
 * 
 * Revision 1.8  89/05/11  15:41:44  little
 * take the redefined_prop out of the property list and put into the
 * symbol structure.  this is to speed-up get prop calls.
 * 
 * Revision 1.7  89/02/10  14:47:53  ejh
 * added copyright notice
 * 
 * Revision 1.6  89/02/01  21:44:18  rcs
 * fixed bug in cus_sym_representation of incorrectly setting bad status
 * 
 * Revision 1.5  89/02/01  16:20:23  little
 * change cus_sym_representation to work correctly with the
 * new property named alternate_print_string_prop, if the property
 * is present, it's value is the sym_representation
 * 
 * Revision 1.4  89/01/17  16:05:26  rcs
 * added cus_get_sym_type_string routine and supporting data structure
 * 
 * Revision 1.3  89/01/09  15:51:45  little
 * add the concept of the redefined_prop.  when the prop is set for a symbol,
 * use the value of the prop (redefined_prop) as a symbol and get the
 * requested prop from that "new" symbol.
 * 
 * Revision 1.2  88/10/06  13:37:57  little
 * add log comments
 * 
|
|	9 jun 88 - change property "name" print to use the name array instead
|		of a print routine
|	8 jun 88 - add the cus_sym_delete routine (it is a do nothing for now)
|	10 may 1988 - module created

|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	memoryPack

#include	<stdio.h>
#include	"hashPack.h"
#include	"alistPack.h"
#include	"global.h"
#include	"string.h"
#include	"idd.h"
#include	"isc.h"
#include	"u22.h"
#include	"cus.h"
#include	"cus_priv.h"

			/* do not need no stinking DEBUG */
#define DEBUG2(statement)
#define DEBUG3(statement)
#define FATAL_ERROR(statement)\
   {  long pid;\
      char command [256];\
   fprintf (stderr, "!!!!!     FATAL ERROR\n");\
   fprintf (stderr, "\tin Serpent system file %s, line %d\n", __FILE__, __LINE__);\
   statement;\
   fprintf (stderr, "\n");\
   fflush (stderr);\
   pid = getpid ();\
   sprintf (command, "gcore %d", pid);\
   system (command);\
   sprintf (command, "mv core.%d core", pid);\
   system (command);\
   exit (1);\
   }


static char cus_no_sym_message [] = "NOT IN SYMBOL TABLE";

	/* used to determined whether redefine processing is to be peformed */

static boolean cus_do_redefine;

/*--------------------------------------------------------------------------*\
|  Description:
|     array containing counts of number of times each property has been set
|
\* Definition:                                                              */

				/* array is initailized to 0 */
static int cus_prop_sets [last_prop_DO_NOT_USE + 1] =  {0};

/*--------------------------------------------------------------------------*\
|  Description:
|     array containing counts of number of times each property has been fetched
|
\* Definition:                                                              */

				/* array is initailized to 0 */
static int cus_prop_gets [last_prop_DO_NOT_USE + 1]  = {0};

/*
 * The symbol table is created as a list of hash tables.  The base context
 * table is larger to allow for a non-hierarchical symbol table.  For the
 * compiler, where contexts are required, the table is stored as a list of
 * hash tables.  To look for a symbol, the hash tables must be searched in
 * turn, from inner most context (tail of list) to outermost (head of list).
 *
 * All contexts are also stored in the ALIST ctx_list, so they can be
 * retrieved by the compiler during a second code generation pass.
 */

static int		cus_gensym_seed = 0;
static ALIST		cus_sym_tab;
static ALIST		ctx_list;
static HASH		name_table;

/*--------------------------------------------------------------------------*\
|  Routine: cus_int_get_prop
|
|  Description:
|     the internal version of the get property routine, this is the guy
|	that does most of the work.   it is seperate because there are time
|	when we do not want to go through the redefined prop (which is not
|	being used at this time to speed things up)
| 
\*--------------------------------------------------------------------------*/
char *cus_int_get_prop (sym, property_name)
cus_sym sym;
cus_property_type property_name;

{
   register int i;

   set_status (ok);
   cus_prop_gets [(int) property_name] ++;
   
   if (sym)
      switch (property_name) {
	 case type_prop:
            return (char *) sym->type;
         case template_prop:
            return (char *) sym->template;
         case original_name_prop:
            return (char *) sym->original_name;
         case current_value_prop:
            return (char *) sym->current_value;
         case current_type_prop:
            return (char *) sym->current_type;
         case redefined_prop:
            if (cus_do_redefine)
               return (char *) sym->redefined;
            else
               FATAL_ERROR (
		    fprintf (stderr, "cus_int_get_prop got a redefined_prop\n")
		    );
            break;
         default:
	    if (sym->properties == (ALIST)NULL) {
	        set_status (cus_no_property);
		return (cus_priv_property)NULL;
		}
	    else {
		/*
		 * I would really like to use get_alist_val here, but I can't
		 * because we need to distinguish between a property whose
		 * value is zero (i.e., DEFAULT_ALIST_VAL), and a property
		 * which does not exists in the list.  So, we manually scan
		 * through the property list to effectively do:
		 *     return get_alist_val (sym->properties, property_name);
		 */
		for (i = alist_length (sym->properties); i > 0; i--)
		    if (get_alist_pos_val (sym->properties, i) == (int)property_name)
			return get_alist_pos (sym->properties, i);
	        /*
		 * We get here if there was no prop, so set status and return
		 */
	        set_status (cus_no_property);
	        return NULL;
	        }
         }
    else {
      set_status (cus_null_symbol);
      return NULL;
      }
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_get_redefined
|
|  Description:
|     this routine "runs down" all redefined links, until a "gounded" symbol
|     is found.  the redefined prop allows for a user to temporarily
|     (or permentaly if desired) redefine one symbol to be another.  this way
|     one user routine can "fool" other user routines as to what a symbol
|     really is.
\*--------------------------------------------------------------------------*/
static cus_sym cus_get_redefined (sym)
cus_sym sym;

{  cus_sym test, redefined_sym;

   set_status (ok);
   if (!cus_do_redefine)
      return (sym);
   if (sym && sym->ident == SYM_IDENTIFIER)
      {
      test = sym;
      redefined_sym = test;
      while (test = (cus_sym) test->redefined)
         redefined_sym = test;
      set_status (ok);
      return (redefined_sym);
      }
    else
      return (NULL);

}

/*--------------------------------------------------------------------------*\
|  Routine: cus_is_sym
|
|  Description:
|     returns whether the passed in thing is a symbol or not.
|     
\*--------------------------------------------------------------------------*/
int cus_is_sym (sym)
cus_sym sym;

{
   if (sym = (cus_sym) cus_get_redefined (sym))
      return ((sym->ident == SYM_IDENTIFIER));
    else
      return (NULL);
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_genname
|
|  Description:
|     generate a unique string
|     
\*--------------------------------------------------------------------------*/
char *cus_genname (prefix)
string prefix;
{  char buf [256];			/* WARNING: assumes generated strings
					   are no longer than 255 characters,
					   and does no testing for same
					 * WARNING: cus_gensym also diddles
					   cus_gensym_seed
					 */

   sprintf (buf, "%s_%d", prefix, cus_gensym_seed++);
   return u22_copy_string (buf);
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_activate_context
|
|  Description:
|     user visible routine to "place" the sym tab into the desired context
| 
\*--------------------------------------------------------------------------*/
void cus_activate_context (context)
cus_context_type context;
{
   (void) add_to_atail_val (cus_sym_tab,
			    get_alist_pos (ctx_list, context),
			    get_alist_pos_val (ctx_list, context));
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_current_context
|
|  Description:
|     returns the context in which the sym tab is currently
|     
\*--------------------------------------------------------------------------*/
cus_context_type cus_current_context ()
{
   return (cus_context_type) (get_atail_val (cus_sym_tab));
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_deactivate_context
|
|  Description:
|     user visible routine to place the sym tab into a parent context
\*--------------------------------------------------------------------------*/
cus_context_type cus_deactivate_context ()

{
    if (alist_length (cus_sym_tab) == BASE_CONTEXT)
      FATAL_ERROR (fprintf (stderr,
	  "cus_deactivate_context: trying to deactivate base context\n"));
    (void) get_from_atail (cus_sym_tab);
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_hash_fcn
|
|  Description:
|     internal sym tab hash function
|     
\*--------------------------------------------------------------------------*/
static int cus_hash_fcn (name)
char* name;

{  register char *cp;
   register int i;

   cp = name;
   i = 0;
   while (*cp)
      i = i*2 + *cp++;
   return (i & 077777) % HASH_SIZE;
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_match_fcn
|
|  Description:
|     internal sym tab match function
|     
\*--------------------------------------------------------------------------*/
static int cus_match_fcn (candidate_name, sym)
char *candidate_name;
cus_sym sym;
{
   return (strcmp (candidate_name, sym->name) == 0);
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_name_hash_fcn
|
|  Description:
|     internal sym tab name_hash function
|     
\*--------------------------------------------------------------------------*/
static int cus_name_hash_fcn (name)
char* name;

{  register char *cp;
   register int i;

   cp = name;
   i = 0;
   while (*cp)
      i = i*2 + *cp++;
   return (i & 077777) % HASH_SIZE;
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_name_match_fcn
|
|  Description:
|     internal sym tab name_match function
|     
\*--------------------------------------------------------------------------*/
static int cus_name_match_fcn (candidate_name, name)
char *candidate_name, *name;
{
   return (strcmp (candidate_name, name) == 0);
}					/* end cus_name_match_fcn */

/*--------------------------------------------------------------------------*\
|  Routine: cus_create_nu_context
|
|  Description:
|     user visible routine to create a nu context and go to same
|     
\*--------------------------------------------------------------------------*/
cus_context_type cus_create_nu_context ()
{
    HASH new_ctx;
    int new_val = alist_length (ctx_list) + 1;

    new_ctx = make_hashtable (HASH_SIZE, cus_hash_fcn, cus_match_fcn);
    add_to_atail_val (cus_sym_tab, new_ctx, new_val);
    add_to_atail_val (ctx_list, new_ctx, new_val);

    return new_val;
}

#if 0
/*--------------------------------------------------------------------------*\
|  Routine: cus_internal_walk
|
|  Description:
|     internal routine to actually do the sym tab walk
|     
\*--------------------------------------------------------------------------*/
static void cus_internal_walk (their_funct, their_thing, the_bucket)
int (*their_funct) ();
char *their_thing;
cus_name_bucket_ptr the_bucket;

{
   DEBUG2 (fprintf (stderr, "entering cus_internal_walk\n"));

   if (the_bucket->sym)
      their_funct (their_thing, the_bucket->sym);
   for_all_list (the_bucket->inner_contexts, their_thing, their_funct);
   for_all_list (the_bucket->outer_contexts, their_thing, their_funct);
   return;
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_walk_sym_tab
|
|  Description:
|     user visible routine to do the symbol table walk
|     
\*--------------------------------------------------------------------------*/
void cus_walk_sym_tab (the_funct, the_thing)
int (*the_funct) ();
char *the_thing;

{
   for_all_2hashtable (cus_sym_tab->the_table, the_funct, the_thing,
		       cus_internal_walk);
   return;
}
#endif

/*--------------------------------------------------------------------------*\
|  Routine: cus_create_sym_tab
|
|  Description:
|     sym tab structure creater and initializer.  if it ever turns out more
|	than one sym tab is needed, this routine and most others will need
|	to have an additional parameter, which is a sym tab instead of using
|	the global cus_sym_tab.
|     
\*--------------------------------------------------------------------------*/
void cus_create_sym_tab (do_redefine_flag)
boolean do_redefine_flag;
{

    HASH table;

    if (cus_sym_tab == (ALIST)0)
	{
	cus_sym_tab = make_alist();
	ctx_list = make_alist();
	table = make_hashtable (HASH_SIZE, cus_hash_fcn, cus_match_fcn);
	add_to_atail_val (cus_sym_tab, table, HIDDEN_CONTEXT);
	add_to_atail_val (ctx_list, table, HIDDEN_CONTEXT);
	table = make_hashtable (BASE_HASH_SIZE, cus_hash_fcn, cus_match_fcn);
	add_to_atail_val (cus_sym_tab, table, BASE_CONTEXT);
	add_to_atail_val (ctx_list, table, BASE_CONTEXT);
	name_table = make_hashtable (NAME_HASH_SIZE, cus_name_hash_fcn, cus_name_match_fcn);
	cus_do_redefine = do_redefine_flag;
	}
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_gensym
|
|  Description:
|     generate and enter a symbol with a unique name
|     
\*--------------------------------------------------------------------------*/
cus_sym cus_gensym (prefix, type, seed)
string prefix;
cus_sym_type type;
int *seed;
{
   int new;
   char buf [256];			/* WARNING: assumes generated strings
					   are no longer than 255 characters,
					   and does no testing for same
					 * WARNING: cus_genname also diddles
					   cus_gensym_seed
					 */
    if (seed == (int *)0)
       sprintf (buf, "%s_%d", prefix, cus_gensym_seed++);
    else
       sprintf (buf, "%s_%d", prefix, (*seed)++);
   return cus_add_sym (buf, type, &new, NO_CONTEXT);
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_gensym_no_add
|
|  Description:
|     generate a symbol with a unique name but don't add to symtab
|     
\*--------------------------------------------------------------------------*/
cus_sym cus_gensym_no_add (prefix, type, seed)
string prefix;
cus_sym_type type;
int *seed;
{
   char buf [256];			/* WARNING: assumes generated strings
					   are no longer than 255 characters,
					   and does no testing for same
					 * WARNING: cus_genname also diddles
					   cus_gensym_seed
					 */
    if (seed == (int *)0)
       sprintf (buf, "%s_%d", prefix, cus_gensym_seed++);
    else
       sprintf (buf, "%s_%d", prefix, (*seed)++);
   return cus_create_sym (buf, type);
}

#if 0
/*--------------------------------------------------------------------------*\
|  Routine: cus_sym_context
|
|  Description:
|     return the context of the passed in symbol
|     
\*--------------------------------------------------------------------------*/
cus_context_type cus_sym_context (sym)
cus_sym sym;

{
   if (sym = cus_get_redefined (sym))
      return (sym->context);
    else
      return (-1);
}
#endif

/*--------------------------------------------------------------------------*\
|  Routine: cus_sym_representation
|
|  Description:
|     return the print representation of a symbol
|     
\*--------------------------------------------------------------------------*/
char *cus_sym_representation (sym)
cus_sym sym;

{

   set_status (ok);
   if (sym = cus_get_redefined (sym))
     {
     set_status (ok);			     /* mask any possible error	    */
     if ((sym->name) == NULL)
	return (cus_no_sym_message);
      else
	return (sym->name);
     }
    else
      return (cus_no_sym_message);
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_prop_print_val
|
|  Description:
|     returns the print string corresponding to the passed in prop
|     
\*--------------------------------------------------------------------------*/
string cus_prop_print_val (property)
cus_property_type property;
{
   return (cus_property_type_print_value [(int) property]);
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_type_print_val
|
|  Description:
|     returns the print string corresponding to the passed in type
|     
\*--------------------------------------------------------------------------*/
string cus_type_print_val (type)
cus_sym_type type;

{ 
   return (cus_symbol_type_print_value [(int) type]);
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_look_up
|
|  Description:
|     user visible routine to check and see if a string has been entered
|	in the sym tab
|     
\*--------------------------------------------------------------------------*/
cus_sym cus_look_up (name)
char *name;
{
    cus_sym sym;
    int i;

    if (cus_sym_tab == (ALIST)NULL)
	FATAL_ERROR (fprintf (stderr, "no symbol table about\n"));

    sym = (cus_sym) NULL;
    for (i = alist_length (cus_sym_tab); i >= 1; i--)
	if (sym = (cus_sym) get_from_hashtable (get_alist_pos (cus_sym_tab, i), name))
	    break;
    return cus_get_redefined (sym);
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_create_sym
|
|  Description:
|     enter a string in the tab, creating new symbol if necessary
|     
\*--------------------------------------------------------------------------*/
cus_sym cus_create_sym (name, type)
char* name;
cus_sym_type type;
{
    char *str;
    cus_sym sym;

    sym = (cus_sym) make_node (sizeof (cus_sym_entry));
    sym->ident = SYM_IDENTIFIER;
    sym->name = u22_copy_string(name);
    sym->type = type;
    sym->redefined = (caddr_t) NULL;
    sym->template = (caddr_t) NULL;
    sym->current_type = (idd_data_types) NULL;
    sym->current_value = (caddr_t) NULL;
    sym->original_name = (caddr_t) NULL;
    sym->properties = (ALIST) NULL;
    return sym;
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_add_sym
|
|  Description:
|     enter a string in the tab, creating new symbol if necessary
|     
\*--------------------------------------------------------------------------*/
cus_sym cus_add_sym (name, type, new, worry_about_context)
char* name;
cus_sym_type type;
int* new;
int worry_about_context;
{
    char *str;
    cus_sym sym;

    if (cus_sym_tab == (ALIST)NULL)
	FATAL_ERROR (fprintf (stderr, "no symbol table about\n"));

    /*
     * If we are worried about context, look in the current context
     * level.  Otherwise, look in the "hidden" context level.
     */
    if (worry_about_context)
	sym = (cus_sym) get_from_hashtable (get_alist_tail (cus_sym_tab), name);
    else
	sym = (cus_sym) get_from_hashtable (get_alist_head (cus_sym_tab), name);

    if (!sym) {
	*new = 1;
	sym = (cus_sym) make_node (sizeof (cus_sym_entry));
	sym->ident = SYM_IDENTIFIER;
	str = get_from_hashtable (name_table, name);
	if (!str)
	    str = add_to_hashtable (name_table, u22_copy_string(name), name);
	sym->name = str;
	sym->type = type;
	sym->redefined = (caddr_t) NULL;
	sym->template = (caddr_t) NULL;
	sym->current_type = (idd_data_types) NULL;
	sym->current_value = (caddr_t) NULL;
	sym->original_name = (caddr_t) NULL;
	sym->properties = (ALIST) NULL;
	/*
	 * If we are worried about context, add it to the current context
	 * level.  Otherwise, add it to the "hidden" context level.
	 */
	if (worry_about_context)
	    add_to_hashtable (get_alist_tail (cus_sym_tab), sym, name);
	else
	    add_to_hashtable (get_alist_head (cus_sym_tab), sym, name);
        }
    else
	*new = 0;

    return sym;
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_put_prop
|
|  Description:
|     user visible routine to place a new property value into a prop for a sym
|     
\*--------------------------------------------------------------------------*/
void cus_put_prop (sym, property_name, property_value)
cus_sym sym;
cus_property_type property_name;
char *property_value;

{  cus_priv_property prop;

   cus_prop_sets [(int) property_name] ++;

   sym = cus_get_redefined (sym);
   if (sym) {
      switch (property_name) {
	 case type_prop:
            sym->type = (cus_sym_type) property_value;
            break;
         case template_prop:
            sym->template = (caddr_t) property_value;
            break;
         case original_name_prop:
            sym->original_name = (caddr_t) property_value;
            break;
         case current_value_prop:
            sym->current_value = (caddr_t) property_value;
            break;
         case current_type_prop:
            sym->current_type = (idd_data_types) property_value;
            break;
         case redefined_prop:
            if (cus_do_redefine)
               sym->redefined = (caddr_t) property_value;
              else
					/* if someone asks for redefined_prop
					   and not enabled it is an error */
               FATAL_ERROR (fprintf (stderr,
				  "cus_put_prop got a redefined_prop\n"));
            break;
         default:
	    if (sym->properties == (ALIST)NULL) {
		sym->properties = make_alist();
                add_to_atail_val (sym->properties, property_value, property_name);
		}
	    else {
		/*
		 * It would be nice if there was a "update_anode_val", which
		 * would change the node associated with a given val, and add
		 * that node if the corresponding val was not in the alist.
		 * There ain't, so we do it this way.  The delete is to get
		 * rid of any existing instance (and this will be a no-op if
		 * nothing is in the list with that val currently).
		 */
		delete_alist_val (sym->properties, property_name);
		add_to_atail_val (sym->properties, property_value, property_name);
		}
	    break;
         }
      }
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_get_prop
|
|  Description:
|     user visible routine to get the current value for a prop for a sym
|     
\*--------------------------------------------------------------------------*/
char *cus_get_prop (sym, property_name)
cus_sym sym;
cus_property_type property_name;

{  cus_sym test;

   return (cus_int_get_prop (cus_get_redefined (sym), property_name));

}

/*--------------------------------------------------------------------------*\
|  Routine: cus_print_property_list
|
|  Description:
|     user visible routine to print the property list for the specified
|	sym.  if the user passed in a prop value print routine, use it,
|	otherwise just the print the prop names of all the props which the
|	sym has.
|     
\*--------------------------------------------------------------------------*/
void cus_print_property_list (stream, sym, indent, their_funct)
FILE *stream;
cus_sym sym;
string indent;
int (*their_funct) ();

{  LIST prop_list;

   DEBUG2 (fprintf (stderr, "entering cus_print_property_list\n"));

   sym = cus_get_redefined (sym);
   if (sym)
      prop_list =  LISThead (sym->properties);
      {if (their_funct)
         {their_funct (stream, indent, sym, type_prop);
         if (sym->redefined)
            their_funct (stream, indent, sym, redefined_prop);
         if (sym->template)
            their_funct (stream, indent, sym, template_prop);
         if (sym->current_type)
            their_funct (stream, indent, sym, current_type_prop);
         if (sym->current_value)
            their_funct (stream, indent, sym, current_value_prop);
         if (sym->original_name)
            their_funct (stream, indent, sym, original_name_prop);
         while (prop_list != NULL)
            {their_funct (stream, indent, sym, (int) LISTvalue (prop_list));
            prop_list = LISThead (prop_list);
            }
         }
       else
         {fprintf (stream, "%s%s = %s\n", indent,
   		   cus_property_type_print_value [(int) type_prop],
   	 	   cus_type_print_val (sym->type));
         if (sym->redefined)
            fprintf (stream, "%s%s = %s\n", indent,
   		   cus_property_type_print_value [(int) redefined_prop],
   	 	   cus_type_print_val (sym->redefined));
         if (sym->template)
            fprintf (stream, "%s%s = %s\n", indent,
   		   cus_property_type_print_value [(int) template_prop],
   	 	   cus_type_print_val (sym->template));
         if (sym->current_type)
            fprintf (stream, "%s%s = %s\n", indent,
   		   cus_property_type_print_value [(int) current_type_prop],
   	 	   cus_type_print_val (sym->current_type));
         if (sym->current_value)
            fprintf (stream, "%s%s = %s\n", indent,
   		   cus_property_type_print_value [(int) current_value_prop],
   	 	   cus_type_print_val (sym->current_value));
         if (sym->original_name)
            fprintf (stream, "%s%s = %s\n", indent,
   		   cus_property_type_print_value [(int) original_name_prop],
   	 	   cus_type_print_val (sym->original_name));
         while (prop_list != NULL)
            {fprintf (stream, "%s%s\n", indent,
                      cus_property_type_print_value
				[(int)LISTvalue (prop_list)]);
            prop_list = LISThead (prop_list);
            }
         }
      }
   return;
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_delete_sym
|
|  Description:
|     delete the passed in sym from the tab
|     
\*--------------------------------------------------------------------------*/
boolean cus_delete_sym (sym)
cus_sym sym;

{
    if (!sym)
	return false;
    /*
     * Delete the property alist associated with the symbol, then delete
     * the symbol itself and its name.
     */
    free_alist(sym->properties, NOT_NODES);
    delete_from_hashtable ( get_alist_tail (cus_sym_tab), sym->name);
    free_node (sym->name);
    /*
     * We deleted the symbol (and possibly more), so return true.
     */
    return true;
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_string_constant
|
|  Description:
|     enter passed in string and make a string constant, perhaps this should
|     not be in the sym tab set of routines, but so it so goes.
|     
\*--------------------------------------------------------------------------*/
cus_sym cus_string_constant (the_string)
string the_string;

{  cus_sym sym;

   sym = cus_gensym_no_add ("string", constant_sym, 0);
   cus_put_prop (sym, is_constant_prop, 1);
   cus_put_prop (sym, current_type_prop, idd_string);
   cus_put_prop (sym, current_value_prop, the_string);
   return (sym);
}

/*--------------------------------------------------------------------------*\
|  Routine: cus_print_prop_nums
|
|  Description:
|     prints the property set/use info
|     
\*--------------------------------------------------------------------------*/
void cus_print_prop_nums (stream, indent)
FILE *stream;
{  int i;

   fprintf (stream, "\n\nPROPERTY USAGE\n");
   fprintf (stream, "\tSET\tUSE\tTOTAL\n");
   for (i = 0; i < (int) last_prop_DO_NOT_USE; i++)
      {fprintf (stream, "%d\t%d\t%d\t%s\n",
		cus_prop_sets [i], cus_prop_gets [i],
		cus_prop_sets [i] + cus_prop_gets [i],
		cus_prop_print_val ((cus_property_type) i));
      }
   fprintf (stream, "\n\n");
}
