static char rcsid [] =	"$Header: /u/04c/mnt/integ/source/utilities/src/RCS/ipb2.c,v 1.18 91/03/25 23:01:26 ejh Exp $";

/*--------------------------------------------------------------------------*\
| 
|  Name:  Playback formatting (ipb2)
|
|  Description:  
|     The playback formatting reads and produces formatted output for
|     the journalling file
|
|  Component of:
|     Interface Layer.
|
|  Related files:
|     ipb2.c - contains the implementation for the playback formater
|     ipb.h - contains the external definition for the playback abstr.
|     ipb2test.c - test file.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Owner:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Len Bass
|     ljb@sei.cmu.edu  
|
|  Change History:
$Log:	ipb2.c,v $
 * Revision 1.18  91/03/25  23:01:26  ejh
 * removed two routines: cleanup_mbx and get_data_def
 * 
 * Revision 1.17  90/08/31  02:06:30  rcs
 * tried to fix up buffer print
 * 
 * Revision 1.16  90/08/22  16:34:08  rcs
 * modified format to print buffers based on the buffer type
 * 
 * Revision 1.15  90/06/06  17:39:44  clc
 * On exec_monitor_call as file_name, use stdin for journaling file
 * 
 * Revision 1.14  90/02/16  12:59:41  clc
 * Declared internal routines to be static
 * 
 * Revision 1.13  90/02/14  21:48:25  ejh
 * fixed include structure
 * 
 * Revision 1.12  90/01/30  16:37:38  clc
 * Added missing " mark.
 * 
 * Revision 1.11  90/01/26  19:10:21  clc
 * Flagged buffers being passed to the usr_subroutine.
 * 
 * Revision 1.10  89/12/16  04:18:12  clc
 * Set up the structure to pass the output to shared data through a user's
 * subroutine.  Also made the output easier to read.
 * 
 * Revision 1.9  89/11/22  12:59:03  rcs
 * allow for read of undefined buffers
 * modified formatted output
 * 
 * Revision 1.8  89/06/15  20:17:11  ljb
 * fix trailing null problem for component_name
 * 
 * Revision 1.7  89/06/06  15:04:33  little
 * add boolean type to format_component (ljb)
 * 
 * Revision 1.6  89/06/05  09:02:55  ljb
 * make work with "remove" and fix minor bugs
 * 
 * Revision 1.5  89/06/01  15:37:05  ejh
 * fixed buffer record/playback; turned status tests into io_test
 * 
 * Revision 1.4  89/02/10  14:48:51  ejh
 * added copyright notice
 * 
 * Revision 1.3  88/10/14  16:08:07  little
 * fix to work with dm sd
 * 
|     April 28, 1988 : Module created.
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	memoryPack

#include	<stdio.h>
#include	<errno.h>
#include	<sys/time.h>
#include	<values.h>
#include	<varargs.h>

#include	"hashPack.h"

#include	"global.h"
#include	"string.h"
#include	"isc.h"
#include	"ubo.h"
#include	"u22.h"
#include	"isd.h"
#include	"iid.h"
#include	"idd.h"
#include	"ipb.h"
#include	"ipb2.h"

#define DEBUG1(statement)  SERPENT_DEBUG1(ipb_debug, statement)
#define DEBUG2(statement)  SERPENT_DEBUG2(ipb_debug, statement)
#define DEBUG3(statement)  SERPENT_DEBUG3(ipb_debug, statement)

int	ipb_debug;
static	FILE	*fp;
static	HASH	mbx_table;
typedef	struct {
    string		mbx_name;
    idd_data_definition	data_definition;
} mbx_table_type;

void (*gbl_usr_subroutine)();
enum {
    TO_STDOUT,
    TO_SUBROUTINE
} gbl_submit_output;

/*--------------------------------------------------------------------------*\
|  Routine: ipb_mbx_hash
|
|  Description:
|	This routine calculates the hash function for a mailbox
\*--------------------------------------------------------------------------*/

static	int	ipb_mbx_hash (mbx_name)
string	mbx_name;

    {
    int		hash_value = 0;
    string	pointer;

    for (pointer = mbx_name; *pointer != '\0'; pointer ++)
	hash_value += *pointer;
    return ((((int) hash_value) & 077777) % HASH_TABLE_SIZE);
    }

/*--------------------------------------------------------------------------*\
|  Routine: ipb_mbx_cmp
|
|  Description:
|	This routine compares test value to see if have found actual value
\*--------------------------------------------------------------------------*/

static	boolean	ipb_mbx_cmp (test_mbx, mbx_table_struct)
string		test_mbx;
mbx_table_type	*mbx_table_struct;

    {
    return (!strcmp (test_mbx, mbx_table_struct->mbx_name));
    }

/*--------------------------------------------------------------------------*\
|  Routine: ipb_init_mbx
|
|  Description:
|	This routine initializes the hash table for mailbox vs data def
\*--------------------------------------------------------------------------*/

static	void	ipb_init_mbx ()

    {
    mbx_table = make_hashtable (HASH_TABLE_SIZE, ipb_mbx_hash, ipb_mbx_cmp);
    return;
    }

/*--------------------------------------------------------------------------*\
|  Routine: ipb_add_new_mbx
|
|  Description:
|	This routine adda a new mbx, data def pair to hash table
\*--------------------------------------------------------------------------*/
static	void	ipb_add_new_mbx (mbx, data_def)
string			mbx;
idd_data_definition	data_def;

    {
    mbx_table_type	*temp;
/*  
   this routine creates a node with mailbox name, data definition in it
*/
    temp = (mbx_table_type *) make_node (sizeof (mbx_table_type));
    temp->data_definition = data_def;
    temp->mbx_name = u22_copy_string (mbx);
    (void) add_to_hashtable (mbx_table, temp, mbx);
    return;
    }

/*--------------------------------------------------------------------------*\
|  Routine: ipb_submit
|
|  Description:
|	This is a switch on gbl_submit_output that determines if the
| 	information goes to stdout, or gets sent to the user-written
|	usr_subroutine.  This gives the user a chance to copy the info
| 	and place it in shared memory.  This routine uses variable
|	argument lists, which are passed on to either vprintf or the
|	user's usr_subroutine.
|  Global variables:
|	gbl_submit_output
|	gbl_usr_subroutine
\*--------------------------------------------------------------------------*/
static	void	ipb_submit (va_alist)
va_dcl

    {
    va_list args;
    ipb_submit_type info_type;
    char   *format;

    va_start (args);
    info_type = va_arg (args, ipb_submit_type);
    format = va_arg (args, char *);

    switch (gbl_submit_output) {

      case TO_STDOUT: 
	vfprintf (stdout, format, args);
	break;

      case TO_SUBROUTINE: 
	gbl_usr_subroutine (info_type, format, args);
	break;

      default: 
	fprintf (stderr, "Bad output parameter in ipb_submit");
	break;
    }

    va_end (args);
    return;
}
/*--------------------------------------------------------------------------*\
|  Routine: ipb_format_component
|
|  Description:
|	The format element routine causes the component dependent portion of
|	the journaling file to be output to stdout
\*--------------------------------------------------------------------------*/
static	void	ipb_format_component (data_def, element_name)
idd_data_definition	data_def;
string			element_name;

    {
    int			len = -1,
			offset,
			position,
			int_temp;
    boolean		boolean_temp;
    char		component_name [100],
			*string_temp;
    double		real_temp;
    idd_data_types	data_type;
    iid_id_type		id_temp;
    idd_buffer_type	buffer_temp;

  while (len) {
    (void) fread (&len, sizeof (len), 1, fp);
    if (len) {
      (void) fread (component_name, len, 1, fp);
      component_name[len] = '\0';
      idd_get_component_lop (data_def, element_name, component_name,
        &len, &offset, &position
      );
      data_type = idd_get_shared_data_type (data_def, element_name,
        component_name
      );
/*
   Print out component name and type.
*/
      ipb_submit(COMP_NAME_TYPE,
        "      %-22s %-9s\t", 
        component_name, 
        idd_get_type_string(data_type)
      );

/*
   If the data is going into shared data, it need not be forced into a form
   that printf can accept.  In fact, it is much easier to send the data in
   a buffer, whose type indicates the data type.
*/
      if (gbl_submit_output == TO_SUBROUTINE) {

/*
   We are passing the data in buffer_temp.  If the data is a buffer, force
   buffer_temp to be the buffer given by the data.
*/
	if (data_type == idd_buffer) {
	  (void) fread (&buffer_temp, sizeof (buffer_temp), 1, fp);
	  string_temp = "Buffer being passed";
	}

/*
   Otherwise, set the type and the length of the data, so that Slang
   can interpret the data as the variable type it represents.
*/
	else {
	  buffer_temp.type = data_type;
	  buffer_temp.length = len;
	  string_temp = "Normal data being passed";
	}

/*
   Read in either the variable or the body of the buffer variable.
*/
	buffer_temp.body = make_node (buffer_temp.length);
	(void) fread (buffer_temp.body, buffer_temp.length, 1, fp);

	ipb_submit (COMPONENT_DATA, string_temp, &buffer_temp);

	free_node (buffer_temp.body);

      }
      else
/*
   Since gbl_submit_output != TO_SUBROUTINE, we must send the data
   in a form that printf can handle.
*/
      {
/*
   Print out body of component.
*/
      switch (data_type) {

        case idd_integer: 
          (void) fread(&int_temp, sizeof(int_temp), 1, fp);
          if (int_temp == UNDEFINED_INTEGER)
            ipb_submit(PRINT_COMPONENT, "UNDEFINED_INTEGER\n");
          else
            ipb_submit(PRINT_COMPONENT, "%d\n", int_temp);
          break;

        case idd_real: 
          (void) fread(&real_temp, sizeof(real_temp), 1, fp);
          if (real_temp == UNDEFINED_REAL)
            ipb_submit(PRINT_COMPONENT, "UNDEFINED_REAL\n");
          else
            ipb_submit(PRINT_COMPONENT, "%f\n", real_temp);
          break;

        case idd_boolean: 
          (void) fread(&int_temp, sizeof(int_temp), 1, fp);
          if (((boolean) int_temp) == UNDEFINED_BOOLEAN)
            ipb_submit(PRINT_COMPONENT, "UNDEFINED_BOOLEAN\n");
          else
	    if (int_temp)
              ipb_submit(PRINT_COMPONENT, "True\n");
	    else
	      ipb_submit(PRINT_COMPONENT, "False\n");
          break;

        case idd_string: 
          string_temp = make_node(len + 1);
          (void) fread(string_temp, len, 1, fp);
          if (! strcmp(string_temp, UNDEFINED_STRING))
            ipb_submit(PRINT_COMPONENT, "UNDEFINED_STRING\n");
          else
            ipb_submit(PRINT_COMPONENT, "%s\n", string_temp);
          free_node(string_temp);
          break;

        case idd_id: 
          (void) fread(&id_temp, len, 1, fp);
          if (id_temp == UNDEFINED_ID)
            ipb_submit(PRINT_COMPONENT, "UNDEFINED_ID\n");
          else
            ipb_submit(PRINT_COMPONENT, "%d\n", id_temp);
          break;

        case idd_buffer: 
          (void) fread(&buffer_temp, len, 1, fp);
/*
   If buffer undefined print "undefined".
*/
          if (ubo_undefined(&buffer_temp)) {
            ipb_submit(PRINT_COMPONENT, "UNDEFINED_BUFFER");
	  }
/*
   If buffer not undefined print contents.
*/
          else {
            ipb_submit(PRINT_COMPONENT, "length = %d\t", buffer_temp.length);
            ipb_submit(PRINT_COMPONENT, "type = %s\n",
              idd_get_type_string(buffer_temp.type));
/*
   Only print out the buffer body if it is not empty.
*/
            if (ubo_not_empty(&buffer_temp)) {

              buffer_temp.body = make_node(buffer_temp.length);
              (void)fread(buffer_temp.body, buffer_temp.length, 1, fp);

              ipb_submit(PRINT_COMPONENT, "  body = ");
/*  
   Use a different print format for the buffer according to the type.
*/
              switch (buffer_temp.type) {
                case idd_boolean:
                  if ((boolean)buffer_temp.body == true) {
		    ipb_submit(PRINT_COMPONENT, "True\n");
		  }
                  else {
                    ipb_submit(PRINT_COMPONENT, "False\n");
		  }
                  break;
                case idd_id:
                case idd_integer:
                  ipb_submit(PRINT_COMPONENT, "%d\n", (int) buffer_temp.body);
                  break;
                case idd_real:
                  ipb_submit(
		    PRINT_COMPONENT, "%f\n", *((double *)buffer_temp.body)
                  );
                  break;
                case idd_string:
                  ipb_submit(PRINT_COMPONENT, "%s\n", buffer_temp.body);
                  break;
                case idd_buffer:
/*
   Print out the first 10 bytes of data in the buffer body.
*/
                  for (
                    offset = 0; 
                    (offset < 10) && (offset < buffer_temp.length);
	      	    ++offset
                  ) {
                    ipb_submit (PRINT_COMPONENT, "0x%X ",
                      (*(buffer_temp.body + offset)) &
                      ((1 << BITS (char)) -1)
                    );
		  }

               default:
                 DEBUG1(
                   fprintf(stderr, "ipb_format_component: unkown type.\n");
                 );

              }			/* end switch on type			*/

	      free_node (buffer_temp.body);

            }			/* end if buffer not empty		*/
	  }			/* end else buffer defined.		*/
/*
   Print final return.
*/
          ipb_submit (PRINT_COMPONENT, "\n");

          break;

        default: 
          fprintf (stderr, "Strange data type in Journal file.\n");
          fprintf (stderr, "Ignoring data field.\n");
          ipb_submit (PRINT_COMPONENT, "%s\n", "Unknown type of data.");
          string_temp = make_node (len);
          io_test (fread (string_temp, len, 1, fp));
          free_node (string_temp);
          break;
      }				/* end switch (data_type)		  */
      }				/* end if (submit_output==TO_SUBROUTINE) */ 
    }				/* end if (len)				  */
  }				/* end while (len)			  */

  return;
}


/*--------------------------------------------------------------------------*\
|  Routine: ipb_format_element
|
|  Description:
|  	This routine causes the element dependent portion of the journaling
|	file to be output to stdout
\*--------------------------------------------------------------------------*/

static	void	ipb_format_element (data_def)
idd_data_definition	data_def;

    {
    int			len = -1;
    char		element_name [100];
    isd_change_type	change_type;
    iid_id_type		id;

    while (len)
	{
	(void) fread (&len, sizeof (len), 1, fp);
	if (len)
	    {
	    (void) fread (element_name, len, 1, fp);
	    ipb_submit (ELEMENT_NAME, "   Element name: %s", element_name);
	    (void) fread (&change_type, sizeof (change_type), 1, fp);
	    switch (change_type) {
	    case isd_create:
		ipb_submit (CHANGE_TYPE_STR, "  Change type: create");
		break;
	    case isd_modify:
		ipb_submit (CHANGE_TYPE_STR, "  Change type: modify");
		break;
	    case isd_remove:
		ipb_submit (CHANGE_TYPE_STR, "  Change type: remove");
		break;
	    case isd_get:
		ipb_submit (CHANGE_TYPE_STR, "  Change type: get   ");
		break;
	    default:
		ipb_submit (CHANGE_TYPE_STR, " Change type: Unknown");
		break;
	    }
	    (void) fread (&id, sizeof (id), 1, fp);
	    ipb_submit (ELEMENT_ID, "  ID: %d\n", id);

	    if (change_type != isd_remove) ipb_format_component (data_def, element_name);
	    }
	}
    return;
    }

/*--------------------------------------------------------------------------*\
|  Routine: ipb_format_transaction
|
|  Description:
|	This routine causes the transaction dependent portion of the
|	journaling file to be output to stdout
\*--------------------------------------------------------------------------*/

static	void	ipb_format_transaction ()

    {
    struct	timeval	tod;
    string		char_time;
    int			len;
    char		sender [100],
			receiver [100];
    mbx_table_type	*hash_entry;

    ipb_submit (ANNOUNCE_TRANS, "TRANSACTION:\n");
    (void) fread (&tod, sizeof (tod), 1, fp);
    char_time = ctime (&tod.tv_sec);
/*
   ctime returns a 26 char string of the form dow mmm dd hh:mm:ss yyyy\n\0;
   we insert a \0 between the ss and the yyyy.  This allows printing the
   microseconds before the year
*/
    char_time [19] = '\0';
    ipb_submit (TIMESTAMP, "  %s:%d  %s", char_time, tod.tv_usec/1000,
	&char_time [20]);

    (void) fread (&len, sizeof (len), 1, fp);
    (void) fread (sender, len, 1, fp);
    sender [len] = '\0';

    ipb_submit (SENDER, "   Sender: %s\n", sender);

    (void) fread (&len, sizeof (len), 1, fp);
    (void) fread (receiver, len, 1, fp);
    receiver [len] = '\0';

    ipb_submit (RECEIVER, "   Receiver: %s\n", receiver);
/*
   locate data definition from mailbox name, only one of sender receiver
   should be in the hash table
*/
    hash_entry = (mbx_table_type *) get_from_hashtable (mbx_table, sender);
    if (!(hash_entry) || (!strcmp (hash_entry->mbx_name, "DM_BOX")))
	hash_entry = (mbx_table_type *) get_from_hashtable (mbx_table,
	    receiver);

    ipb_format_element (hash_entry->data_definition);
    ipb_submit (ANNOUNCE_END_OF_TRANS, "\n");
    return;
    }

/*--------------------------------------------------------------------------*\
|  Routine: ipb_format_participant
|
|  Description:
|	This routine causes a new participant information to be written to
|	stdout, the ill file to be read in and the data_definition information
|	to be added to the participant hash table
\*--------------------------------------------------------------------------*/

static	void	ipb_format_participant	(message)
string	message;

    {
    int			len, fval;
    char		ill_file_name [100],
			mbx_name [100];
    idd_data_definition	data_def;

    ipb_submit (ANNOUNCE_PARTN, "%s\n", message);
    io_test ((fval = fread (&len, sizeof (len), 1, fp)));
    io_test (fread (ill_file_name, len, 1, fp));
    ill_file_name [len] = '\0';
    ipb_submit (ILL_FILE_NAME, "   ill file name: %s\n", ill_file_name);

    io_test (fread (&len, sizeof (len), 1, fp));
    io_test (fread (mbx_name, len, 1, fp));

    mbx_name [len] = '\0';
    ipb_submit (MAILBOX_NAME, "   mailbox name: %s\n\n", mbx_name);

    if (strcmp (ill_file_name, ""))
	{
	data_def = idd_create_data_definition();
	isd_input_ill_file (data_def, ill_file_name);
	ipb_add_new_mbx (mbx_name, data_def);
	}
    return;
    }

/*--------------------------------------------------------------------------*\
|  Routine: ipb_format_file
|
|  Description:
|	This routine causes the playback file recorded earlier to be output to
|	either stdout or the usr_subroutine.
\*--------------------------------------------------------------------------*/

static	void	ipb_format_file (file_name)
string	file_name;

    {
    char	dial_name [100],
		message [100],
		check_string [9];
    int		len;

    DEBUG2 (fprintf (stderr, "ipb_format_file: call.\n"));
    DEBUG3 (fprintf (stderr, "ipb_format_file: file name = %s.\n",
	file_name));

    set_status (ok);
/*
   Create/open file for playback.
*/
    if (!strcmp (file_name, "exec_monitor_call"))
      fp = stdin;
    else
      fp = fopen (file_name, "r");

    check_null (fp, "ipb_format_file: error opening specified file.\n",
	bad_parameter);

    (void) fread (check_string, 8, 1, fp);
    check_string [8] = '\0';
    if (strcmp (check_string, "JOURNAL:"))
	{
	set_status (invalid_journal_file);
        if (fp != stdin) fclose (fp);
	return;
	}
    ipb_submit (JRNL_FILE_NAME, "FORMATTING JOURNAL FILE: %s\n\n", file_name);
    ipb_submit (ANNOUNCE_HEADER, "HEADER: \n");
/*
  read in dialogue name, message and ill file name
*/
    io_test (fread (&len, sizeof (len), 1, fp));
    io_test (fread (dial_name, len, 1, fp));
    dial_name [len] = '\0';
    ipb_submit (DIALOGUE_NAME, "   dialogue name: %s\n", dial_name);

    io_test (fread (&len, sizeof (len), 1, fp));
    io_test (fread (message, len, 1, fp));
    message [len] = '\0';
    ipb_submit (JRNL_MESSAGE, "   message: %s\n", message);

    ipb_init_mbx ();
    ipb_format_participant ("OWNER:");

    while (fread (check_string, 6, 1, fp))
	{
        check_string [6] = '\0';
	if (!strcmp (check_string, "PARTN:"))
	    ipb_format_participant ("PARTICIPANT:");
	else if (!strcmp (check_string, "TRANS:"))
	    ipb_format_transaction ();
	}

    DEBUG2 (fprintf (stderr, "ipb_format_file: exit.\n"));

    return;
    }

void ipb_format_playback (file_name)
string file_name;
    {
    gbl_submit_output = TO_STDOUT;

    ipb_format_file (file_name);
    }

void ipb_format_playback_to_subroutine (file_name, usr_subroutine)
string file_name;
void (*usr_subroutine)();
    {
    gbl_submit_output = TO_SUBROUTINE;
    gbl_usr_subroutine = usr_subroutine;

    ipb_format_file (file_name);
    }
