/*
$Header: /u/04c/mnt/integ/source/utilities/src/RCS/u22.c,v 1.8 91/03/25 21:54:46 ejh Exp $
*/

/*--------------------------------------------------------------------------*\
|
|  Name:  u22.c
|
|  Description:  
|     Global declarations of general interest to Serpent source modules.
|     Specifically, this file contains useful macros and definitions that
|     are useful across the Serpent system.
|
|  Component of:
|     Utilities Layer.
|
|  Related files:
|     None.
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:  
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Brian M. Clapper
|     bmc@sei.cmu.edu  
|

$Log:	u22.c,v $
 * Revision 1.8  91/03/25  21:54:46  ejh
 * took out unused line (buf_size = 0 just before buf_size = BUFFER_INC)
 * 
 * Revision 1.7  90/01/19  16:28:51  dvk
 * Added u22_convert_string_inplace and moved a strlen calculation out
 * of the loop in u22_convert_string
 * 
 * Revision 1.6  89/11/17  13:44:19  bmc
 * Added u22_concat_string routine
 * 
 * Revision 1.5  89/11/16  18:06:24  bmc
 * Added u22_read_line routine
 * 
 * Revision 1.4  89/11/16  14:47:16  bmc
 * Added u22_get_token routine.
 * 
 * Revision 1.3  89/10/18  10:15:26  ejh
 * created u22_convert_string (came from a22)
 * 
 * Revision 1.2  89/09/28  10:52:00  bmc
 * No important changes.  (Testing CM.)
 * 
 
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <varargs.h>
#include "global.h"
#include "u22.h"

#define memoryPack
#include "memoryPack.h"

/*--------------------------------------------------------------------------*\
|  Routine: u22_copy_string
\*--------------------------------------------------------------------------*/

string u22_copy_string (old_string)
string old_string;
{
    char *new_string = NULL;


    if ( (old_string != NULL) &&
	 ((new_string = make_node (strlen (old_string) + 1)) != NULL) )
	(void) strcpy (new_string, old_string);

    return (new_string);
}


/*--------------------------------------------------------------------------*\
|  Routine: u22_convert_string
\*--------------------------------------------------------------------------*/

string	u22_convert_string (the_name, the_case)
string	the_name;
int	the_case;

    {
    int		i = 0;
    int		len = strlen (the_name);
    string	new_name;

/*	make a copy of the name and convert each character to new case */

    new_name = u22_copy_string (the_name);
    switch (the_case)
	{
	case UPPERCASE:
	    while (i < len)
		{
		new_name [i] = ToUpper (new_name [i]);
		i++;
		}
	    break;
	case LOWERCASE:
	    while (i < len)
		{
		new_name [i] = ToLower (new_name [i]);
		i++;
		}
	    break;
	}
    return (new_name);
    }

/*--------------------------------------------------------------------------*\
|  Routine: u22_convert_string_inplace
\*--------------------------------------------------------------------------*/

string	u22_convert_string_inplace (the_name, the_case)
string	the_name;
int	the_case;

    {
    int		i = 0;
    int		len = strlen (the_name);

/*	convert each character to new case */

    switch (the_case)
	{
	case UPPERCASE:
	    while (i < len)
		{
		the_name [i] = ToUpper (the_name [i]);
		i++;
		}
	    break;
	case LOWERCASE:
	    while (i < len)
		{
		the_name [i] = ToLower (the_name [i]);
		i++;
		}
	    break;
	}
    return (the_name);
    }

/*--------------------------------------------------------------------------*\
|  Routine:     u22_get_token
\*--------------------------------------------------------------------------*/

string u22_get_token (line, token_delims, chars_to_skip)
string	line;
string	token_delims;
int	*chars_to_skip;
{
    int token_start;
    char temp;
    string token = NULL;

    *chars_to_skip = 0;
    token_start = strspn (line, token_delims);
    line += token_start;
    if (*line != '\0')
	{
	int token_len;
	char *after_token;

	token_len = strcspn (line, token_delims);

	/*
	** temporarily replace the character past the token with an
	** end-of-string delimiter so u22_copy_string will work.
	*/
	after_token = line + token_len;
	temp = *after_token;
	*after_token = '\0';

	token = u22_copy_string (line);

	/* Now restore the character we overwrote. */

	*after_token = temp;

	*chars_to_skip = token_start + token_len;
	}

    return (token);
}

/*--------------------------------------------------------------------------*\
|  Routine:     u22_read_line
\*--------------------------------------------------------------------------*/

string u22_read_line (file)
FILE  *file;
{
#  define BUFFER_INC  (132)  /* allocate buffer in increments of this size */

   extern int errno;               /* UNIX error number */

   boolean done_read;              /* true -> read complete */
   boolean free_buffer;            /* true -> free the buffer */
   short buf_size;                 /* size of input buffer */
   short next;                     /* next available location in buffer */

   int c;                          /* next character from file */
   char *buffer;                   /* pointer to allocated line */


   errno       = 0;
   next        = 0;
   done_read   = false;
   free_buffer = false;
   buf_size = BUFFER_INC;

   /* Allocate initial buffer.  Allocating it here makes handling
   ** an empty string easier.
   */
   if ( (buffer = make_node (BUFFER_INC)) == NULL )
      errno = ENOMEM;

   else
      {
      while ( (errno == 0) && (! done_read) )
         {
         /* 
         ** NOTE:  Errno is undefined upon return from getc().  (In fact,
         **   it's often set to some absurd value.)  Since this routine uses
         **   errno to provide additional error information, we have to be
         **   sure to set it to something reasonable after getc() returns.
         **   Further, when getc() returns EOF, there's no reliable way to
         **   determine when it encountered a true end-of-file or an error.
         */
         c = getc (file);
         errno = 0;
   
         if (c == EOF)
            {
            done_read = true;
   
            /* 
            ** If the buffer contains some data, return it as a full line.
            ** The next call to this function will return EOF.  Otherwise,
            ** if the buffer is empty, we have a true EOF.  This scheme
            ** prevents the caller from ignoring the last line of a file if
            ** the line doesn't end with a newline character.
            */
            if (next == 0)
               free_buffer = true;
            }
   
         else if ( (char) c == '\n')
            done_read = true;

         else
            {
            /*
	    ** Valid character was read.  Is there room left in buffer?
            ** (buf_size - 1) is used to allow room for trailing '\0'
	    */
            if (next >= (buf_size - 1)) 
               {
               /* At end of buffer; enlarge it. */

               buffer = change_node_size (buffer, buf_size + BUFFER_INC);
               if (buffer == NULL)
                  {
                  free_buffer = false;
                  errno = ENOMEM;
                  }
               else
                  buf_size += BUFFER_INC;
               }

            if (errno == 0)
               buffer [next++] = (char) c;
            }
         }
   
      buffer [next] = '\0';
      }

   if (free_buffer)
      {
      free_node (buffer);
      buffer = NULL;
      }

   return (buffer);
}

/*--------------------------------------------------------------------------*\
|  Routine:     u22_concat_string
|
|  This routine uses the varargs package supplied with the C library.
|  The function declaration, as weird as it may look, is correct.
|  Consult varargs(3) for more information.
\*--------------------------------------------------------------------------*/

string u22_concat_string (va_alist)
va_dcl
{
   va_list args;                  /* the argument list */
   int length;                    /* length of allocated string */
   string s;                      /* a string */
   string result;


   /*
   ** Traverse the argument list once, counting the size of each string,
   ** to determine how large a buffer to allocate.
   */
   va_start (args);

      length = 0;
      while ( (s = va_arg (args, char *)) != NULL )
         length += strlen (s);

      /* Add one more byte for the trailing '\0' character. */
      length++;

   va_end (args);

   /*
   ** Attempt to allocate a buffer large enough to hold all the strings
   ** plus the string terminator.
   */
   if ( (result = make_node (length)) != NULL )
      {
      /* Traverse the argument list again, concatenating all strings into
      ** the allocated buffer.
      */
      va_start (args);

         *result = '\0';
         while ( (s = va_arg (args, char *)) != NULL )
            (void) strcat (result, s);

      va_end (args);
      }

   return (result);
}
