	static char rcsid [] = "$Header: /u/04c/mnt/integ/source/utilities/src/RCS/uc2.c,v 1.11 91/03/27 20:33:28 ejh Exp $";

/*--------------------------------------------------------------------------*\
|
|  Name:  Conversion Utilities
|
|  Description:  
|     Converts values
|
|  Component of:
|     utilities
|
|  Related files:
|     uc2.h - external definition
|
|  Project:
|     SERPENT System
|     User Interface Prototyping
|     Software Engineering Institute
|     Carnegie Mellon University
|     Pittsburgh, PA  15213
|
|  Version: 1.0
|
|  Project Leader:
|     Len Bass
|     ljb@sei.cmu.edu
|
|  Author:  
|     Stephen T. Chou
|     stc@sei.cmu.edu
|
|     Robert C. Seacord
|     rcs@sei.cmu.edu
|
|     Erik J. Hardy
|     erik@sei.cmu.edu
|
|  Change History:
$Log:	uc2.c,v $
 * Revision 1.11  91/03/27  20:33:28  ejh
 * took out some assignments that are never used
 * 
 * Revision 1.10  91/02/20  18:36:09  little
 * no changes, oh boy am i sorry.
 * 
 * Revision 1.9  91/01/25  16:11:16  little
 * no changes
 * 
 * Revision 1.8  91/01/09  18:24:15  dvk
 * Fixed return in uc2_get_undefined for strings
 * 
 * Revision 1.7  90/09/07  18:25:05  rcs
 * changed isinteger and is real to not barf on trailing white space
 * 
 * Revision 1.6  90/08/29  16:16:59  little
 * add a default case to the switch in uc2_undefined
 * 
 * Revision 1.5  90/07/23  10:57:13  ejh
 * got rid of dm dependencies
 * 
 * Revision 1.4  90/07/10  14:27:34  rnk
 * fixed bug in realtobuff
 * 
 * Revision 1.3  90/07/10  12:16:07  ejh
 * made better error message for failed conversion in uc2_convert
 * 
 * Revision 1.2  90/07/05  14:45:52  ejh
 * exposed isinteger and isreal
 * 
 * Revision 1.1  90/07/05  13:11:19  ejh
 * Initial revision
 * 
|     July 1, 1990 : Module created.
|
\*--------------------------------------------------------------------------*\
|
|   Distribution: Approved for public release; distribution is unlimited.
|
|   Copyright (c) 1989  by  Carnegie  Mellon  University, Pittsburgh, PA.  The
|  Software Engineering  Institute  (SEI) is a federally  funded  research and
|  development center established  and  operated by Carnegie Mellon University
|  (CMU).  Sponsored  by  the  U.S.  Department  of   Defense  under  contract
|  F19628-85-C-0003,  the  SEI  is  supported  by  the  services  and  defense
|  agencies, with the U.S. Air Force as the executive contracting agent.
|
|    Permission to use,  copy,  modify, or  distribute  this  software and its
|  documentation for  any purpose and without fee  is hereby granted, provided
|  that  the above copyright notice appear  in  all copies and that both  that
|  copyright  notice  and   this  permission  notice  appear   in   supporting
|  documentation.   Further,  the  names  Software  Engineering  Institute  or
|  Carnegie  Mellon  University  may  not be used in  advertising or publicity
|  pertaining to distribution of the software without specific, written  prior
|  permission.  CMU makes no claims  or representations  about the suitability
|  of this software for any purpose.  This software is provided "as is" and no
|  warranty,  express  or  implied,  is  made  by  the  SEI  or CMU, as to the
|  accuracy  and  functioning of the program and related program material, nor
|  shall   the  fact of  distribution   constitute  any   such  warranty.   No
|  responsibility is assumed by the SEI or CMU in connection herewith.
|
\*--------------------------------------------------------------------------*/

#define	uc2_TOP
#define	memoryPack

#include	<math.h>
#include	<stdio.h>
#include	<errno.h>
#include 	<ctype.h>
#include	"string.h"
#include	"global.h"
#include	"memoryPack.h"
#include	"u22.h"
#include	"uc2.h"
#include	"ubo.h"
#include	"isc.h"
#include	"iid.h"
#include	"idd.h"

#define DEBUG1(statement) SERPENT_DEBUG1(uc2_debug, statement)
#define DEBUG2(statement) SERPENT_DEBUG2(uc2_debug, statement)
#define DEBUG3(statement) SERPENT_DEBUG3(uc2_debug, statement)

#define NO_OF_TYPES 8

/*--------------------------------------------------------------------------*\
|  This define is used in the routine funcall to implement the
| type-to-type conversion checks.  The integer number of the types
| comes from idd_data_types in idd.h
\*--------------------------------------------------------------------------*/
#define from_to(fromtype,totype) ((int)NO_OF_TYPES*(int)totype+(int)fromtype)
/*--------------------------------------------------------------------------*\
|  Routine: isinteger
| 
|  Description:  
|     Check if an ascii string is a integer string
\*--------------------------------------------------------------------------*/
int	isinteger (str)
char	*str;
    {
    long	strtol ();
    char	*dummy;
    int		return_val;

    DEBUG2 (fprintf (stderr, "isinteger: call.\n"));

    (void) strtol(str, &dummy, 10);

    if (*dummy == '\0' || isspace(*dummy))
	return_val = 1;
    else
	return_val = NULL;

    DEBUG2 (fprintf (stderr, "isinteger: exit.\n"));
    return (return_val);
    }

/*--------------------------------------------------------------------------*\
|  Routine: is real
| 
|  Description:  
|     check if an ascii string is a real string.
\*--------------------------------------------------------------------------*/
int	isreal (str)
char	*str;
    {
    double	strtod ();
    char	*dummy;
    int		return_val;

    DEBUG2 (fprintf (stderr, "isreal: call.\n"));

    (void) strtod (str, &dummy);

    if (*dummy == '\0' || isspace(*dummy))
	return_val = 1;
    else
	return_val = 0;

    if (strcmp (str,"+") == 0)
	return_val = 0;

    if (strcmp (str,"-") == 0)
	return_val = 0;

    DEBUG2 (fprintf (stderr, "isreal: exit.\n"));
    return (return_val);
    }

/*--------------------------------------------------------------------------*\
|  Routine: astoi
| 
|  Description:  
|     Takes in ascii and return integer.
\*--------------------------------------------------------------------------*/
static	caddr_t	astoi (num)
char	*num;
    {
    int	result;

    DEBUG2 (fprintf (stderr, "astoi: call.\n"));

    if (isinteger (num) == NULL)
	{
	DEBUG1 (fprintf(stderr, "astoi: failed conversion.\n"));

	set_status (failed_conversion);
	return (NULL);
	}

    result = atoi (num);

    DEBUG2 (fprintf (stderr, "astoi: exit.\n"));
    return ((caddr_t) result);
    }

/*--------------------------------------------------------------------------*\
|  Routine: astor
| 
|  Description:  
|     Takes in ascii and return real.
\*--------------------------------------------------------------------------*/
static	caddr_t	astor (num)
char	*num;
    {
    double	*result;

    DEBUG2 (fprintf (stderr, "astor: call.\n"));

    result = (double *) make_node (sizeof (double));
    if (isreal (num) == NULL)
	{
	DEBUG1 (fprintf (stderr, "astor: failed conversion.\n"));

	set_status (failed_conversion);
	return (NULL);
	}
    *result = atof (num);

    DEBUG2 (fprintf (stderr, "astor: %e, at %d\n", *result, (int) result));

    /* If *result = HUGE or 0, error! */

    if ((*result == HUGE) || (errno == ERANGE))
	{
	DEBUG1 (fprintf (stderr, "astor: failed conversion.\n"));

	set_status (failed_conversion);
	return (NULL); 
	}
    else
	{
	DEBUG2 (fprintf (stderr, "astor: exit.\n"));
	return ((caddr_t) result);
	}
    }

/*--------------------------------------------------------------------------*\
|  Routine: rtoi
| 
|  Description:  
|     Takes in address of real and return integer (rounded real).
\*--------------------------------------------------------------------------*/
static	caddr_t	rtoi (addr)
caddr_t	addr;
    {
    int		result;
    double	f;

    DEBUG2 (fprintf (stderr, "rtoi: call.\n");
	fprintf (stderr, "rtoi: value to be converted: %e\n",
	    *((double *) addr)));

    f = *((double *) addr);
    result = f;
    if ((result - f) >= 0.5)
	result++;
    DEBUG2 (fprintf (stderr, "rtoi: result: %d\n", result);
	fprintf(stderr, "rtoi: exit.\n"));

    return ((caddr_t) result);
    }

/*--------------------------------------------------------------------------*\
|  Routine: itor
| 
|  Description:  
|     Converts an integer to a real number.
\*--------------------------------------------------------------------------*/
/* takes in an integer and return real */
static	caddr_t	itor (num)
int	num;
    {
    double	*result;

    DEBUG2 (fprintf (stderr, "itor: call.\n"));

    result = (double *) make_node (sizeof (double));
    *result = num;

    DEBUG2 (fprintf (stderr, "itor: %d to %f\n", result, *result);
	fprintf(stderr, "itor: exit.\n"));

    return ((caddr_t) result);
    }

/*--------------------------------------------------------------------------*\
|  Routine: rtoa
| 
|  Description:  
|     converts a real number to an ascii string
\*--------------------------------------------------------------------------*/
static	caddr_t	rtoa (num1)
caddr_t	num1;
    {
    char	s [BUFSIZ];
    char	*str;

    DEBUG2 (fprintf (stderr, "rtoa: called.\n");
	fprintf (stderr, "rtoa: real number = %f.\n", num1));

    sprintf (s, "%f", *((double *) num1));
    str = u22_copy_string (s);

    DEBUG2 (fprintf (stderr, "rtoa: string = %s.\n", str);
	fprintf(stderr, "rtoa: exiting.\n"));

    return ((caddr_t) str);
    }

/*--------------------------------------------------------------------------*\
|  Routine: itoa
| 
|  Description:  
|     converts an integer to an ascii string
\*--------------------------------------------------------------------------*/
static	caddr_t	itoa (num)
int	num;
    {
    char	s [BUFSIZ];
    char	*str;

    DEBUG2 (fprintf (stderr, "itoa: called.\n");
	fprintf (stderr, "itoa: integer number = %d.\n", num));

    sprintf (s, "%d", num);
    str = u22_copy_string (s);

    DEBUG2 (fprintf (stderr, "itoa: string = %s.\n", str);
	fprintf (stderr, "itoa: exiting.\n"));

    return ((caddr_t) str);
    }
	
/*--------------------------------------------------------------------------*\
|  Routine: booltostr
| 
|  Description:  
|     converts a boolean to an ascii string of "true" or "false"
\*--------------------------------------------------------------------------*/
static	caddr_t	booltostr (bool)
boolean	bool;
    {
    char	*result;

    if (bool != 0)
	result = u22_copy_string ("true");
    else
	result = u22_copy_string ("false");

    return ((caddr_t) result);
    }

/*------------------------------------------------------------*\
|  Routine: strtobuff
|
|  Description: 
|   This routine takes as input a string and returns a buffer with
|   the same length and body
\*------------------------------------------------------------*/
static	caddr_t	strtobuff (arg)
string	arg;
    {
    idd_buffer_type	*result;

    DEBUG2 (fprintf (stderr, "strtobuff: call.\n");
	fprintf(stderr, "strtobuff: arg = %s.\n", arg));

/* The buffer is the same size as the strlen and does not include a null
   termination character.  In case of zero length strings no memory is
   allocated. */

    result = (idd_buffer_type *) make_node (sizeof (idd_buffer_type));
    result->type = idd_string;

/* If the string is not NULL allocate memory and copy the contents of the
   string including the null termination char. */

    if (arg != NULL)
	{
	result->length = strlen (arg) + 1;
	result->body = u22_copy_string (arg);
	}
    else
	{
	result->length = 0;
	result->body = NULL;
	}

    return ((caddr_t) result);
    }

/*------------------------------------------------------------*\
|  Routine: inttobuff
|
|  Description: 
|   This routine takes as input an integer and returns a buffer with
|   the buffer string representation
\*------------------------------------------------------------*/
static	caddr_t	inttobuff (arg)
int	arg;
    {
    idd_buffer_type	*result;

    DEBUG2 (fprintf (stderr, "inttobuff: call.\n");
	fprintf(stderr, "inttobuff: arg = %d.\n", arg));

    result = (idd_buffer_type *) make_node (sizeof (idd_buffer_type));
    result->length = sizeof (arg);
    result->type = idd_integer;
    result->body = make_node (result->length);
    memcpy (result->body, &arg, result->length);

    DEBUG2 (fprintf (stderr, "inttobuff: result = %x.\n", result);
	fprintf (stderr, "inttobuff: result->length = %d.\n", result->length);
	fprintf (stderr, "inttobuff:  result->type = %s.\n",
	    idd_get_type_string (result->type));
	fprintf (stderr, "inttobuff: result->body = %x.\n", result->body);
	fprintf (stderr, "inttobuff: exit.\n"));

    return ((caddr_t) result);
    }

/*------------------------------------------------------------*\
|  Routine: idtobuff
|
|  Description: 
|   This routine takes as input an id and returns a buffer with
|   the buffer string representation
\*------------------------------------------------------------*/
static	caddr_t	idtobuff (arg)
iid_id_type	arg;
    {
    idd_buffer_type	*result;

    DEBUG2 (fprintf (stderr, "idtobuff: call.\n");
	fprintf (stderr, "idtobuff: arg = %d.\n", arg));

    result = (idd_buffer_type *) make_node (sizeof (idd_buffer_type));
    result->length = sizeof (arg);
    result->type = idd_id;
    result->body = make_node (result->length);
    memcpy (result->body, &arg, result->length);

    DEBUG2 (fprintf (stderr,"idtobuff: result = %x.\n", result);
	fprintf (stderr, "idtobuff: result->length = %d.\n", result->length);
	fprintf (stderr, "idtobuff:  result->type = %s.\n",
	    idd_get_type_string (result->type));
	fprintf (stderr, "idtobuff: result->body (addr)= %x.\n",
	    result->body);
	fprintf (stderr, "idtobuff: result->body (content)= %x.\n",
	    *(result->body));
	fprintf (stderr, "idtobuff: exit.\n"));

    return ((caddr_t) result);
    }

/*------------------------------------------------------------*\
|  Routine: realtobuff
|
|  Description: 
|   This routine takes as input the address of a real and returns 
|   a buffer with the same binary representation
\*------------------------------------------------------------*/
static	caddr_t	realtobuff (arg)
double	*arg;
    {
    idd_buffer_type	*result;

    DEBUG2 (fprintf (stderr, "realtobuff: call.\n");
	fprintf (stderr, "realtobuff: arg = %d.\n", arg));

     result = (idd_buffer_type *) make_node (sizeof (idd_buffer_type));
     result->length = sizeof (double);
     result->type = idd_real;
     result->body = make_node (result->length);
     memcpy (result->body, arg, result->length);

    DEBUG2 (fprintf (stderr, "realtobuff: result = %x.\n", result);
	fprintf (stderr, "realtobuff: result->length = %d.\n",
	    result->length);
	fprintf (stderr, "realtobuff:  result->type = %s.\n",
	    idd_get_type_string (result->type));
	fprintf (stderr, "realtobuff: result->body = %x.\n", result->body);
	fprintf(stderr, "realtobuff: exit.\n"));

    return ((caddr_t) result);
    }

/*-------------------------------------------------------------------------*\
|  Routine: inttobool
|
|  Description:
|    Converts an integer to a boolean.  Valid integer values are 0
|    (converted to false) and 1 (converted to true)
*\-------------------------------------------------------------------------*/
static	caddr_t	inttobool(num)
int	num;
    {
    boolean	result;

    DEBUG2 (fprintf (stderr, "inttobool: call.\n");
	fprintf (stderr, "inttobool: num = %d\n", num));

    if (num == 1 | num == 0)
	{
	result = num;
	DEBUG2 (fprintf (stderr, "inttobool: result = %d.\n", result);
	    fprintf (stderr, "inttobool: exit.\n"));
	return ((caddr_t)result);
	}
    else
	{
	set_status (failed_conversion);
	DEBUG2 (fprintf (stderr, "inttobool: exit.failed conversion\n"));
	return (NULL);
	}
    }

/*---------------------------------------------------------------------------*\
|  Routine: realtobool
|
|  Description:
|    Converts a real to a boolean.  Valid  values are 0
|    (converted to false) and 1 (converted to true)
|    ??? is it ok that this comes in as a double and goes out as in integer?
*\--------------------------------------------------------------------------*/
static	caddr_t	realtobool (num)
caddr_t	num;
    {
    int	result;

    DEBUG2 (fprintf (stderr, "realtobool: call.\n");
	fprintf (stderr, "realtobool: num = %f\n", num));

    result = (int) itor (num);
    if ( result != 1 && result != 0)
	{
	result = NULL;
	set_status (failed_conversion);
	}
    DEBUG2 (fprintf (stderr, "realtobool: result = %d.\n", result);
	fprintf (stderr, "realtobool: exit.\n"));

    return ((caddr_t)result);
    }

/*---------------------------------------------------------------------------*\
|  Routine: strtobool
|
|  Description:
|    Converts a string to a boolean.  Valid  values are "false"
|    (converted to false) and "true" (converted to true)
*\--------------------------------------------------------------------------*/
static	caddr_t	strtobool (arg)
string	arg;
    {
    int	result;

    DEBUG2 (fprintf (stderr, "strtobool: call.\n");
	fprintf (stderr, "strtobool: arg = %s\n", arg));

    if (strcmp ("true",arg) == 0)
	result = 1;
    else if (strcmp ("false",arg) == 0)
	result = 0;
    else
	{
	result = NULL;
	set_status (failed_conversion);
	}

    DEBUG2 (fprintf (stderr, "strtobool: result = %d.\n", result);
	fprintf (stderr, "strtobool: exit.\n"));

    return ((caddr_t)result);
    }

/*--------------------------------------------------------------------------*\
|  Routine: convert_from_buffer
|
|  Description:
|    Converts a buffer to anything.  Buffers of buffer type can not be
|    converted to anything.  Everything else is handled by a recursive
|    call to funcall.
\*--------------------------------------------------------------------------*/
static	caddr_t	convert_from_buffer (to_type, arg)
idd_data_types	to_type;
idd_buffer_type	*arg;
    {
    caddr_t	result;
    long	dummy;

    DEBUG2 (fprintf (stderr, "convert_from_buffer: call.\n");
	fprintf (stderr, "convert_from_buffer: arg = %x.\n", arg);
	fprintf (stderr, "convert_from_buffer: arg->length = %d.\n",
	    arg->length);
	fprintf (stderr, "convert_from_buffer: arg->type = %s.\n", 
	    idd_get_type_string (arg->type));
	fprintf (stderr, "convert_from_buffer: arg->body = %x.\n",
	    arg->body));

/* If buffer is of buffer type set status to failed conversion. */

    if (arg->type == idd_buffer)
	{
	result = NULL;
	set_status (failed_conversion);
	}

/* If not buffer type reinvoke funcall with the actual type and value of the
   buffer. */

    else

/* If the argument is immediate than cast it into a long and then reinvoke
   funcall. */

	if (arg->type == idd_boolean || arg->type == idd_integer ||
	    arg->type == idd_id)
	    {
	    dummy = *((long *) (arg->body));
	    result = uc2_convert (to_type, arg->type, (caddr_t) dummy);
	    }

/* If the argument is not immediate cast it as a pointer. */

	else
	    result = uc2_convert (to_type, arg->type, (caddr_t) (arg->body));

    DEBUG2 (fprintf (stderr, "convert_from_buffer: result = %d.\n", result);
	fprintf (stderr, "convert_from_buffer: exit.\n"));

    return (result);
    }

/*--------------------------------------------------------------------------*\
|  Routine: booltoint
|
|  Description:
|    Converts a boolean to an integer.  This is easy
\*--------------------------------------------------------------------------*/
static	caddr_t	booltoint (arg)
int	arg;
    {
    int	result;

    DEBUG2 (fprintf (stderr, "booltoint: call.\n");
	fprintf (stderr, "booltoint: arg = %d.\n", arg));

    result = arg;

    DEBUG2 (fprintf (stderr, "booltoint: result = %d.\n", result);
	fprintf (stderr, "booltoint: exit.\n"));

    return ((caddr_t)result);
    }

/*--------------------------------------------------------------------------*\
|  Routine: booltoreal
|
|  Description:
|    Converts a boolean to a real.  This is easy
\*--------------------------------------------------------------------------*/
static	caddr_t	booltoreal (arg)
int	arg;
    {
    double 	*result;

    DEBUG2 (fprintf (stderr, "booltoreal: call.\n");
	fprintf (stderr, "booltoreal: arg = %d.\n", arg));

    result = (double *) itor (booltoint (arg));

    DEBUG2 (fprintf (stderr, "booltoreal: exit.\n"));

    return ((caddr_t) result);
    }

/*--------------------------------------------------------------------------*\
|  Routine: booltobuff
|
|  Description:
|    Converts a boolean to a buffer.
\*--------------------------------------------------------------------------*/
static	caddr_t	booltobuff (arg)
boolean	arg;
    {
    idd_buffer_type	*result;

    DEBUG2 (fprintf (stderr, "booltobuff: call.\n");
	fprintf (stderr, "booltobuff: arg = %d.\n", arg));

    result = (idd_buffer_type *) make_node (sizeof (idd_buffer_type));
    result->length = sizeof (arg);
    result->type = idd_boolean;
    result->body = make_node (result->length);
    memcpy (result->body, &arg, result->length);

    DEBUG2 (fprintf (stderr, "booltobuff: result = %x.\n", result);
	fprintf (stderr, "booltobuff: result->length = %d.\n",	
	    result->length);
	fprintf (stderr, "booltobuff:  result->type = %s.\n",
	    idd_get_type_string (result->type));
	fprintf (stderr, "booltobuff: result->body = %x.\n", result->body);
	fprintf (stderr, "booltobuff: exit.\n"));

    return ((caddr_t) result);
    }

/*--------------------------------------------------------------------------*\
|  Routine: uc2_get_undefined
| 
|  Description:  
|     Returns the "correct" undefined value
\*--------------------------------------------------------------------------*/
caddr_t	uc2_get_undefined (desired_type)
idd_data_types	desired_type;
    {
    caddr_t		ret_val;
    idd_buffer_type	*ret_buf;

    set_status (ok);

    switch (desired_type)
	{
	case idd_boolean:
	    ret_val = (caddr_t) UNDEFINED_BOOLEAN;
	    break;

	case idd_integer:
	    ret_val = (caddr_t) UNDEFINED_INTEGER;
	    break;

	case idd_real:
	    ret_val = make_node (sizeof (UNDEFINED_REAL));
	    *((double *) ret_val) = UNDEFINED_REAL;
	    break;

	case idd_string:
	    ret_val = u22_copy_string (UNDEFINED_STRING);
	    break;

	case idd_id:
	    ret_val = (caddr_t) UNDEFINED_ID;
	    break;

	case idd_buffer:
	    ret_buf = (idd_buffer_type *)make_node (sizeof (idd_buffer_type));
	    ret_buf-> length = UNDEFINED_BUFFER_LENGTH;
	    ret_val = (caddr_t) ret_buf;
	    break;

	case idd_undefined:
	case idd_record:
	    fprintf (stderr, "uc2_get_undefined: got an illegal type = %d\n",
		desired_type);
	    set_status (failed_conversion);
	    break;
	default:
	    fprintf (stderr, "uc2_get_undefined: got an unknown type = %d\n",
		desired_type);
	    set_status (failed_conversion);
	    break;
	}

    return (ret_val);
    }

/*--------------------------------------------------------------------------*\
|  Routine: uc2_undefined
| 
|  Description:  
|     Returns true if undefined.  This would be redundant with idd_undefined
|     except i didn't want to expose the bizarrness where integers are
|     stored directly in the pointer.  Therefore when that gets fixed,
|     this routine can be removed.
\*--------------------------------------------------------------------------*/
boolean uc2_undefined (type, data)
idd_data_types	type;
caddr_t		data;
    {
    boolean	value;

    set_status(ok);
    DEBUG2 (fprintf (stderr, "uc2_undefined: call.\n");
	fprintf(stderr, "uc2_undefined: type = %s.\n",
	    idd_get_type_string (type));
	fprintf (stderr, "uc2_undefined: data = %x.\n", data));
/* 
   Type determines which version of undefined to compare with.
*/
    switch (type)
	{
	case idd_boolean:
	    value = ((boolean) data == UNDEFINED_BOOLEAN);
	    break;
	case idd_integer:
	    value = ((int) data == UNDEFINED_INTEGER);
	    break;
	case idd_real:
	    value = (*((double *) data) == UNDEFINED_REAL);
	    break;
	case idd_string:
	    value = (!strcmp ((string) data, UNDEFINED_STRING));
	    break;
	case idd_record:
	    set_status (operation_undefined_types);
	    return (value);
	case idd_id:
	    value = ((iid_id_type) data == UNDEFINED_ID);
	    break;
	case idd_buffer:
	    value = ubo_is_undefined (*((idd_buffer_type *) data));
	    break;
	case idd_undefined:
	    value = true;
	    break;
        default:
	   fprintf (stderr, "uc2_undefined: unknown type of %d\n", type);
           value = false;
           break;
	}

    DEBUG2 (fprintf (stderr, "uc2_undefined: value = %d.\n", value);
	fprintf (stderr, "uc2_undefined: exiting.\n"));
    return (value);
    }

/*--------------------------------------------------------------------------*\
|  Routine: uc2_convert
| 
|  Description:  
|     call proper type conversion function
|???  current choice of routine is based on having 8 types.  Should be
|      changed to make more general.
\*--------------------------------------------------------------------------*/
caddr_t	uc2_convert (to_type, from_type, arg)
idd_data_types	to_type,
		from_type;
caddr_t		arg;
    {
    caddr_t	result;
    int		sum;

    DEBUG2 (fprintf (stderr, "uc2_convert: call.\n");
	fprintf (stderr, "uc2_convert: to_type = %s.\n",
	    idd_get_type_string (to_type));
	fprintf (stderr, "uc2_convert: from_type = %s.\n",
	    idd_get_type_string (from_type));
	fprintf (stderr, "uc2_convert: arg = %d.\n", arg));

/* If the operand type is the same as the conversion type return the existing
   value unmodified. */

    if (from_type == to_type)
	result = (caddr_t) arg;

/* If undefined, get new type of undefined. */

    else if (uc2_undefined (from_type, arg))
	result = uc2_get_undefined (to_type);

/* Perform conversion. */

    else
	{
	sum = from_to (from_type,to_type);
	DEBUG3 (fprintf (stderr, "uc2_convert: sum = %d\n", sum));

	switch (sum)
	    {
	    case from_to (idd_integer, idd_boolean):
		result = inttobool (arg);
		break;

	    case from_to (idd_boolean, idd_integer):
		result = booltoint (arg); 
		break;

	    case from_to (idd_real, idd_boolean):
		result = realtobool (arg); 
		break;

	    case from_to (idd_real, idd_integer):
		result = rtoi (arg); 
		break;

	    case from_to (idd_string, idd_boolean):
		result = strtobool (arg);
		break;

	    case from_to (idd_buffer, idd_boolean):
	    case from_to (idd_buffer, idd_integer):
	    case from_to (idd_buffer, idd_real):
	    case from_to (idd_buffer, idd_string):
	    case from_to (idd_buffer, idd_id):
		result = convert_from_buffer (to_type, arg);
		break;

	    case from_to (idd_string, idd_integer):
	    case from_to (idd_string, idd_id):
		result = astoi (arg);
		break;

	    case from_to (idd_boolean, idd_real):
		result = booltoreal (arg);
		break;

	    case from_to (idd_integer, idd_real):
		result = itor (arg);
		break;

	    case from_to (idd_string, idd_real):
		result = astor (arg);
		break;

	    case from_to (idd_boolean, idd_string):
		result = booltostr (arg);
		break;

	    case from_to (idd_id, idd_string):
	    case from_to (idd_integer, idd_string):
		result = itoa (arg);
		break;

	    case from_to (idd_real, idd_string):
		result = rtoa (arg);
		break;

	    case from_to (idd_undefined, idd_boolean):
	    case from_to (idd_undefined, idd_integer):
	    case from_to (idd_undefined, idd_real):
	    case from_to (idd_undefined, idd_string):
	    case from_to (idd_undefined, idd_record):
	    case from_to (idd_undefined, idd_id):
	    case from_to (idd_undefined, idd_buffer):
		result = UNDEF;
		break;

	    case from_to (idd_boolean, idd_buffer):
		result = booltobuff (arg);
		break;

	    case from_to (idd_integer, idd_buffer):
		result = inttobuff (arg);
		break;

	    case from_to (idd_real, idd_buffer):
		result = realtobuff (arg);
		break;

	    case from_to (idd_string, idd_buffer):
		result = strtobuff (arg);
		break;

	    case from_to (idd_id, idd_buffer):
		result = idtobuff (arg);
		break;

	    default:
		DEBUG1 (fprintf (stderr,
		    "uc2_convert: tried to convert from %s to %s\n",
		    idd_get_type_string (from_type),
		    idd_get_type_string (to_type)));
		set_status (failed_conversion);
		return (NULL);
	    }
	}

    DEBUG2 (fprintf (stderr, "uc2_convert: result = %d.\n", result);
	fprintf (stderr, "uc2_convert: exit.\n"));

    return (result);
    }
